Package: metaConvert
Type: Package
Title: An Automatic Suite for Estimation of Various Effect Size
        Measures
Version: 2.0.0
Authors@R: c(
        person(given = c("Corentin", "J."), family= "Gosling",  role = c("aut", "cre"),
                email = "corentin.gosling@parisnanterre.fr"),
        person(given = "Samuele",
               family= "Cortese", role = "aut",
               email = "samuele.cortese@gmail.com"),
        person(given = "Marco",
               family= "Solmi", role = "aut",
               email = "marco.solmi83@gmail.com"),
        person(given = "Belen",
               family= "Haza", role = "aut",
               email = "hazabelen@gmail.com"),
        person(given = "Eduard",
               family= "Vieta", role = "aut",
               email = "EVIETA@clinic.cat"),
        person(given = "Richard",
               family= "Delorme", role = "aut",
               email = "richard.delorme.rdb@gmail.com"),
        person(given = "Paolo",
               family= "Fusar-Poli",  role = "aut",
               email = "Paolo.fusar-poli@kcl.ac.uk"),
        person(given = "Joaquim",
               family= "Radua",  role = "aut",
               email = "radua@clinic.cat"))
Maintainer: Corentin J. Gosling <corentin.gosling@parisnanterre.fr>
Description: Automatically estimate 14 effect size measures from a well-formatted dataset, including Cohen's d, Hedges' g, mean difference, odds ratio, risk ratio, incidence rate ratio, risk difference, number needed to treat, Pearson correlation, Fisher's z, Cronbach's alpha, intraclass correlation coefficient, and single-group proportion. Provides a two-tier quality-flag diagnostic system for input validation and post-computation plausibility checks, missing-data guidance that tells users which columns would unlock additional estimators, post-hoc correction for attenuation due to measurement error, and standalone psychometric utilities (standard error of measurement, smallest detectable change, change-score reliability). Various other functions can help, for example, removing dependency between several effect sizes, or identifying differences between two datasets.
    This package is mainly designed to assist in conducting a systematic review with a meta-analysis but can be useful to any researcher interested in estimating an effect size.
Imports: compareDF, metafor, estimraw, rio
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0)
Suggests: testthat (>= 3.0.0), metaumbrella, TOSTER, esc, epiR,
        compute.es, meta, effectsize, MetaUtility, MASS, mvtnorm,
        psych, psychmeta, knitr, DT, rmarkdown
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-20 14:45:15 UTC; coren
Author: Corentin J. Gosling [aut, cre],
  Samuele Cortese [aut],
  Marco Solmi [aut],
  Belen Haza [aut],
  Eduard Vieta [aut],
  Richard Delorme [aut],
  Paolo Fusar-Poli [aut],
  Joaquim Radua [aut]
Repository: CRAN
Date/Publication: 2026-07-20 20:00:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,metaConvert)
S3method(print,metaConvert)
S3method(summary,metaConvert)
export(aggregate_df)
export(compare_df)
export(compute_sdc)
export(compute_sem)
export(convert_df)
export(data_extraction_sheet)
export(es_disattenuate)
export(es_from_2x2)
export(es_from_2x2_prop)
export(es_from_2x2_sum)
export(es_from_ancova_f)
export(es_from_ancova_f_pval)
export(es_from_ancova_md_ci)
export(es_from_ancova_md_pval)
export(es_from_ancova_md_sd)
export(es_from_ancova_md_se)
export(es_from_ancova_means_ci)
export(es_from_ancova_means_sd)
export(es_from_ancova_means_sd_pooled_adj)
export(es_from_ancova_means_sd_pooled_crude)
export(es_from_ancova_means_se)
export(es_from_ancova_t)
export(es_from_ancova_t_pval)
export(es_from_anova_f)
export(es_from_anova_pval)
export(es_from_beta_std)
export(es_from_beta_unstd)
export(es_from_cases_time)
export(es_from_chisq)
export(es_from_chisq_pval)
export(es_from_cohen_d)
export(es_from_cohen_d_adj)
export(es_from_cronbach_alpha)
export(es_from_etasq)
export(es_from_etasq_adj)
export(es_from_fisher_z)
export(es_from_hedges_g)
export(es_from_icc)
export(es_from_linreg_b_ci)
export(es_from_linreg_b_pval)
export(es_from_linreg_b_se)
export(es_from_linreg_t)
export(es_from_logreg_t)
export(es_from_md_ci)
export(es_from_md_pval)
export(es_from_md_sd)
export(es_from_md_se)
export(es_from_mean_change_ci)
export(es_from_mean_change_ci_single_group)
export(es_from_mean_change_pval)
export(es_from_mean_change_pval_single_group)
export(es_from_mean_change_sd)
export(es_from_mean_change_sd_single_group)
export(es_from_mean_change_se)
export(es_from_mean_change_se_single_group)
export(es_from_means_ci)
export(es_from_means_ci_pre_post)
export(es_from_means_ci_pre_post_single_group)
export(es_from_means_sd)
export(es_from_means_sd_pooled)
export(es_from_means_sd_pre_post)
export(es_from_means_sd_pre_post_single_group)
export(es_from_means_se)
export(es_from_means_se_pre_post)
export(es_from_means_se_pre_post_single_group)
export(es_from_med_min_max)
export(es_from_med_min_max_quarts)
export(es_from_med_quarts)
export(es_from_or)
export(es_from_or_ci)
export(es_from_or_pval)
export(es_from_or_se)
export(es_from_paired_f)
export(es_from_paired_f_pval)
export(es_from_paired_t)
export(es_from_paired_t_pval)
export(es_from_paired_t_single_group)
export(es_from_pearson_r)
export(es_from_phi)
export(es_from_plot_ancova_means)
export(es_from_plot_means)
export(es_from_prop_single_group)
export(es_from_prop_single_group_counts)
export(es_from_pt_bis_r)
export(es_from_pt_bis_r_pval)
export(es_from_rd_ci)
export(es_from_rd_pval)
export(es_from_rd_se)
export(es_from_rr_ci)
export(es_from_rr_pval)
export(es_from_rr_se)
export(es_from_spearman_rho)
export(es_from_student_t)
export(es_from_student_t_pval)
export(es_from_user_adj)
export(es_from_user_crude)
export(es_variab_from_means_ci)
export(es_variab_from_means_sd)
export(es_variab_from_means_se)
export(pool_arms)
export(reliability_change_score)
export(see_input_data)
export(summary.metaConvert)
importFrom(stats,aggregate.data.frame)
importFrom(stats,df)
importFrom(stats,dnorm)
importFrom(stats,na.omit)
importFrom(stats,optimize)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,sd)
importFrom(utils,packageVersion)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# metaConvert

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![CRAN
status](https://www.r-pkg.org/badges/version/metaConvert)](https://CRAN.R-project.org/package=metaConvert)
[![License: GPL
(\>=3)](https://img.shields.io/badge/license-GPL%20%28%3E%3D3%29-red.svg)](https://www.gnu.org/licenses/gpl-3.0.html)
<br> [![Monthly
downloads](https://cranlogs.r-pkg.org/badges/metaConvert)](https://cran.r-project.org/package=metaConvert)
[![Total
downloads](https://cranlogs.r-pkg.org/badges/grand-total/metaConvert)](https://cran.r-project.org/package=metaConvert)

## Important

If you are not familiar with R, you can access the core features of this
package on your web browser on its dedicated website
[(https://metaConvert.org/)](https://metaConvert.org/).

## Introduction

This package automatically estimates 14 effect size measures from a
well-formatted dataset. It is also the official implementation of the
metaDETECT framework, that aims to limit the errors in data extraction
and effect size computation for meta-analyses. Various other functions
can help, for example, removing dependency between several effect sizes,
or identifying differences between two datasets.

This package is mainly designed to assist in conducting a systematic
review with a meta-analysis but can be useful to any researcher
interested in estimating an effect size.

## Installation

You can install the released version of metaConvert from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("metaConvert")
```

## Citation

The metaConvert package was written by Corentin J Gosling, Samuele
Cortese, Marco Solmi, Belen Haza, Eduard Vieta, Richard Delorme, Paolo
Fusar-Poli and Joaquim Radua. It is licensed under the GNU General
Public License. <br>You can use metaConvert for free… but please cite
our work ;-) <br><br> <b>Citation:</b> <em>Gosling CJ, Cortese S, Solmi
M, et al. metaConvert: an automatic suite for estimation of 11 different
effect size measures and flexible conversion across them. Research
Synthesis Methods. Published online 2025:1-12.
<doi:10.1017/rsm.2025.11></em><br><br> To obtain a bibtex version of
this citation in R, type

``` r
citation(package = "metaConvert")
```

<!-- README.md is generated from README.Rmd. Please edit that file -->

# metaConvert

[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![CRAN
status](https://www.r-pkg.org/badges/version/metaConvert)](https://CRAN.R-project.org/package=metaConvert)
[![License: GPL
(\>=3)](https://img.shields.io/badge/license-GPL%20%28%3E%3D3%29-red.svg)](https://www.gnu.org/licenses/gpl-3.0.html)
<br> [![Monthly
downloads](https://cranlogs.r-pkg.org/badges/metaConvert)](https://cran.r-project.org/package=metaConvert)
[![Total
downloads](https://cranlogs.r-pkg.org/badges/grand-total/metaConvert)](https://cran.r-project.org/package=metaConvert)

## Important

If you are not familiar with R, you can access the core features of this
package on your web browser on its dedicated website
[(https://metaConvert.org/)](https://metaConvert.org/).

## Introduction

This package automatically estimates 11 effect size measures from a
well-formatted dataset. Various other functions can help, for example,
removing dependency between several effect sizes, or identifying
differences between two datasets.

This package is mainly designed to assist in conducting a systematic
review with a meta-analysis but can be useful to any researcher
interested in estimating an effect size.

## Installation

You can install the released version of metaConvert from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("metaConvert")
```

## Citation

The metaConvert package was written by Corentin J Gosling, Samuele
Cortese, Marco Solmi, Belen Haza, Eduard Vieta, Richard Delorme, Paolo
Fusar-Poli and Joaquim Radua. It is licensed under the GNU General
Public License. <br>You can use metaConvert for free… but please cite
our work ;-) <br><br> <b>Citation:</b> <em>Gosling CJ, Cortese S, Solmi
M, Haza B, Vieta E, Delorme R, Fusar-Poli P, & Radua J. (2024).
metaConvert: An Automatic Suite for Estimation of 11 Different Effect
Size Measures and Flexible Conversion Across Them. R package version
1.0.0. <https://metaConvert.org/></em><br><br> To obtain a bibtex
version of this citation in R, type

``` r
citation(package = "metaConvert")
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
library(metaConvert)
library(DT)

## ----eval = FALSE-------------------------------------------------------------
#  see_input_data(measure = "or")

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(see_input_data(measure = "or", type_of_measure = "natural", extension="data.frame"), options = list(  
    scrollX = TRUE,
    dom = c('t'),
    scrollY = "300px", 
    pageLength = 500,
    ordering = FALSE,
    rownames = FALSE,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center', 
                                     targets = "_all"))))

## ----eval = FALSE-------------------------------------------------------------
#  data_extraction_sheet(measure = "or")

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(data_extraction_sheet(measure = "or", extension="data.frame"), options = list(  
    scrollX = TRUE,
    dom = c('t'),
   ordering = FALSE,
    rownames = FALSE,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center', 
                                     targets = "_all"))))

## ----message=FALSE, fig.width= 11---------------------------------------------
compare_df(
    df_extractor_1 = df.compare1,
    df_extractor_2 = df.compare2,
    output = "html")

## ----message=FALSE, fig.width= 11---------------------------------------------
compare_df(
    df_extractor_1 = df.compare1,
    df_extractor_2 = df.compare2,
    ordering_columns = c("author", "year"),
    output = "html")

## ----message=FALSE------------------------------------------------------------
res = convert_df(x = df.short, measure = "g")

## ----eval= FALSE--------------------------------------------------------------
#  summary(res)

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(summary(res), options = list(  
    scrollX = TRUE,
    dom = c('t'),
    ordering = FALSE,
    rownames = FALSE,
    scrollY = "300px", 
    pageLength = 100,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center', 
                                     targets = "_all"))))

#' Convert a Student's t-test value to several effect size measures
#'
#' @param student_t Student's t-test value.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{student_t} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_student_t a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the Student's t-test value into a Cohen's d (D) and Hedges' g (G),
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate a Cohen's d** the formula used is (table 12.1 in Cooper):
#' \deqn{cohen\_d = student\_t * \sqrt{\frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp}}}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' **Important - Student's t, not Welch's t.** This formula is the exact inverse of the
#' pooled-variance (Student) t-test, so \code{student_t} must be the *equal-variance* t.
#' It should NOT be used with a Welch (unequal-variance) t-test, which is the default of
#' R's \code{t.test()}. A Welch t uses \eqn{\sqrt{s_1^2/n_1 + s_2^2/n_2}} rather than
#' \eqn{s_{pooled}\sqrt{1/n_1 + 1/n_2}}, so when group sizes and variances both differ,
#' plugging a Welch t into this formula yields a biased Cohen's d (the bias can exceed
#' 50% and may flip sign depending on which arm carries the larger variance; it vanishes
#' only when \eqn{n_1 = n_2}). A Welch t cannot be converted to Cohen's d from
#' \code{(t, n_exp, n_nexp)} alone: recovering the pooled SD requires the two arm SDs, in
#' which case \code{\link{es_from_means_sd}} should be used directly.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
#'  \tab https://metaconvert.org/html/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_student_t
#'
#' @md
#'
#' @examples
#' es_from_student_t(student_t = 2.1, n_exp = 20, n_nexp = 22)
es_from_student_t <- function(student_t, n_exp, n_nexp,
                              smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_student_t) {
  if (missing(reverse_student_t)) reverse_student_t <- rep(FALSE, length(student_t))
  reverse_student_t[is.na(reverse_student_t)] <- FALSE


  d <- student_t * sqrt(1 / n_exp + 1 / n_nexp)

  es <- .es_from_d(
    d = d, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_student_t
  )

  es$info_used <- "student_t"
  return(es)
}

#' Convert a Student's t-test p-value to several effect size measures
#'
#' @param student_t_pval p-value (two-tailed) from a Student's t-test. If your p-value is one-tailed, simply multiply it by two.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{student_t_pval} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_student_t_pval a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the Student's t-test p-value into a t-value,
#' and then relies on the calculations of the \code{\link{es_from_student_t}()} function.
#'
#' **To convert the p-value into a t-value,** the following formula is used (table 12.1 in Cooper):
#' \deqn{student\_t = qt(\frac{student\_t\_pval}{2}, df = n\_exp + n\_nexp - 2)}
#' Then, calculations of the \code{\link{es_from_student_t}()} are applied.
#'
#' Note that a two-sided p-value carries no direction, so the recovered t (and hence the
#' generated effect sizes) are always non-negative. Use \code{reverse_student_t_pval} to
#' encode the correct sign for effects that favour the non-experimental group. The same
#' equal-variance (Student, not Welch) assumption as \code{\link{es_from_student_t}} applies.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
#'  \tab https://metaconvert.org/html/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_student_t_pval
#'
#' @md
#'
#' @examples
#' es_from_student_t_pval(student_t_pval = 0.24, n_exp = 20, n_nexp = 22)
es_from_student_t_pval <- function(student_t_pval, n_exp, n_nexp,
                                   smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_student_t_pval) {
  if (missing(reverse_student_t_pval)) reverse_student_t_pval <- rep(FALSE, length(student_t_pval))
  reverse_student_t_pval[is.na(reverse_student_t_pval)] <- FALSE

  t <- qt(p = student_t_pval / 2, df = n_exp + n_nexp - 2, lower.tail = FALSE)

  es <- es_from_student_t(
    student_t = t, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_student_t = reverse_student_t_pval
  )

  es$info_used <- "student_t_pval"

  return(es)
}

#' Convert a one-way independent ANOVA F-value to several effect size measures
#'
#' @param anova_f ANOVA F-value (one-way, binary predictor).
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{anova_f} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_anova_f a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the F-value (one-way, binary predictor) into a t-value,
#' and then relies on the calculations of the \code{\link{es_from_student_t}()} function.
#'
#' **To convert the F-value into a t-value,** the following formula is used (table 12.1 in Cooper):
#' \deqn{student\_t = \sqrt{anova\_f}}
#' Then, calculations of the \code{\link{es_from_student_t}()} are applied.
#'
#' **Important - single numerator degree of freedom only.** The identity
#' \eqn{\sqrt{F} = |t|} holds only when the F-test has a single numerator degree of freedom,
#' i.e. a one-way ANOVA comparing exactly two groups (a binary predictor). Supplying an
#' omnibus F from a factor with three or more levels (numerator df > 1) produces a
#' meaningless effect size and is not detected by the function. In addition, \eqn{\sqrt{F}}
#' discards the sign of the effect, so the generated effect sizes are always non-negative;
#' use \code{reverse_anova_f} to encode direction.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
#'  \tab https://metaconvert.org/html/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_anova_f
#'
#' @md
#'
#' @examples
#' es_from_anova_f(anova_f = 2.01, n_exp = 20, n_nexp = 22)
es_from_anova_f <- function(anova_f, n_exp, n_nexp, smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_anova_f) {
  if (missing(reverse_anova_f)) reverse_anova_f <- rep(FALSE, length(anova_f))
  reverse_anova_f[is.na(reverse_anova_f)] <- FALSE

  t <- sqrt(anova_f)

  es <- es_from_student_t(
    student_t = t, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_student_t = reverse_anova_f
  )

  es$info_used <- "anova_f"

  return(es)
}

#' Convert a p-value from a one-way independent ANOVA to several effect size measures
#'
#' @param anova_f_pval p-value (two-tailed) from an ANOVA (binary predictor). If your p-value is one-tailed, simply multiply it by two.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{anova_f_pval} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_anova_f_pval a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the p-value from the F-value of an ANOVA (one-way, binary predictor) into a t-value,
#' and then relies on the calculations of the \code{\link{es_from_student_t}()} function.
#'
#' **To convert the p-value into a t-value,** the following formula is used (table 12.1 in Cooper):
#' \deqn{student\_t = qt(\frac{anova\_f\_pval}{2}, df = n\_exp + n\_nexp - 2)}
#' Then, calculations of the \code{\link{es_from_student_t}()} are applied.
#'
#' As for \code{\link{es_from_anova_f}}, this conversion is valid only for an F-test with a
#' single numerator degree of freedom (a two-group comparison): the p-value of a
#' multi-level (numerator df > 1) omnibus F is inverted here as if it were a two-sided
#' two-group t p-value, which is incorrect. The two-sided p-value also carries no
#' direction, so the generated effect sizes are always non-negative; use
#' \code{reverse_anova_f_pval} to encode the correct sign.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
#'  \tab https://metaconvert.org/html/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_anova_pval
#'
#' @md
#'
#' @examples
#' es_from_anova_pval(anova_f_pval = 0.0012, n_exp = 20, n_nexp = 22)
es_from_anova_pval <- function(anova_f_pval, n_exp, n_nexp, smd_to_cor = "viechtbauer",
                               smd_var = "borenstein", reverse_anova_f_pval) {
  if (missing(reverse_anova_f_pval)) reverse_anova_f_pval <- rep(FALSE, length(anova_f_pval))
  reverse_anova_f_pval[is.na(reverse_anova_f_pval)] <- FALSE


  es <- es_from_student_t_pval(
    student_t_pval = anova_f_pval, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_student_t_pval = reverse_anova_f_pval
  )

  es$info_used <- "anova_f_pval"

  return(es)
}

#' @noRd
.compact_measure_label <- function(measure, exp) {
  if (exp && measure == "logor") return("OR")
  if (exp && measure == "logrr") return("RR")
  if (exp && measure == "logirr") return("IRR")
  if (exp && measure == "loghr") return("HR")
  labels <- c(d = "d", g = "g", md = "MD", r = "r", z = "z",
              logor = "logOR", logrr = "logRR", logirr = "logIRR",
              loghr = "logHR",
              logvr = "logVR", logcvr = "logCVR",
              nnt = "NNT", rd = "RD", dw = "d_w", gw = "g_w", mdw = "MD_w",
              rp = "r_p", zp = "z_p",
              prop = "prop", alpha = "alpha", icc = "ICC")
  if (measure %in% names(labels)) labels[measure] else measure
}

.make_es_summary <- function(res, measure, exp, digits, suffix) {
  es_col <- paste0("es", suffix)
  ci_lo_col <- paste0("es_ci_lo", suffix)
  ci_up_col <- paste0("es_ci_up", suffix)
  out_col <- paste0("es_summary", suffix)
  label <- .compact_measure_label(measure, exp)

  res[[out_col]] <- ifelse(
    is.na(res[[es_col]]), NA_character_,
    paste0(label, " = ",
           round(as.numeric(as.character(res[[es_col]])), digits),
           " [",
           round(as.numeric(as.character(res[[ci_lo_col]])), digits),
           ", ",
           round(as.numeric(as.character(res[[ci_up_col]])), digits),
           "]")
  )
  res
}

.make_es_consistency <- function(res, digits, suffix) {
  es_col <- paste0("es", suffix)
  n_est_col <- paste0("n_estimations", suffix)
  min_info_col <- paste0("min_info", suffix)
  min_val_col <- paste0("min_es_value", suffix)
  max_info_col <- paste0("max_info", suffix)
  max_val_col <- paste0("max_es_value", suffix)
  overlap_col <- paste0("overlap_min_max", suffix)
  disp_col <- paste0("dispersion_es", suffix)
  out_col <- paste0("es_consistency", suffix)

  n_est <- suppressWarnings(as.numeric(as.character(res[[n_est_col]])))
  overlap_raw <- suppressWarnings(as.numeric(as.character(res[[overlap_col]])))
  disp_raw <- suppressWarnings(as.numeric(as.character(res[[disp_col]])))
  min_val <- suppressWarnings(as.numeric(as.character(res[[min_val_col]])))
  max_val <- suppressWarnings(as.numeric(as.character(res[[max_val_col]])))

  res[[out_col]] <- ifelse(
    is.na(res[[es_col]]), NA_character_,
    ifelse(!is.na(n_est) & n_est > 1,
      paste0("min: ", res[[min_info_col]], " = ", round(min_val, digits),
             ", max: ", res[[max_info_col]], " = ", round(max_val, digits),
             ", overlap: ", round(overlap_raw * 100), "%",
             ", SD: ", round(disp_raw, digits)),
      "")
  )
  res
}

# round ES/SE/CI columns (flags need full precision)
.round_numeric_cols <- function(res, digits, suffix) {
  cols_to_round <- paste0(
    c("es", "se", "es_ci_lo", "es_ci_up",
      "overlap_min_max", "diff_min_max", "dispersion_es",
      "min_es_value", "min_es_se", "min_es_ci_lo", "min_es_ci_up",
      "max_es_value", "max_es_se", "max_es_ci_lo", "max_es_ci_up"),
    suffix
  )
  for (col in cols_to_round) {
    if (!col %in% colnames(res)) next
    vals <- res[[col]]
    numeric_rows <- which(vals != "< 2 types of input data available")
    if (length(numeric_rows) > 0) {
      res[numeric_rows, col] <- round(
        as.numeric(as.character(vals[numeric_rows])), digits)
    }
  }
  res
}

#' Synthesize information of an object of class \dQuote{metaConvert} into a dataframe
#'
#' @param object an object of class \dQuote{metaConvert}
#' @param digits an integer value specifying the number of decimal places for the rounding of numeric values. Default is 3.
#' @param flags a logical value indicating whether quality/plausibility flags should be generated. Default is TRUE. The \code{es_flags} column includes both input validation flags (generated during \code{\link{convert_df}}) and post-computation quality flags.
#' @param flag_options a named list of thresholds overriding the defaults (and any options set in \code{\link{convert_df}}).
#' Available options:
#' \itemize{
#'   \item \code{smd_max} (default 3): |SMD| above this value is flagged
#'   \item \code{r_max} (default 0.95): same, for |r|
#'   \item \code{log_or_max} (default 5): same, for |logOR| and |logRR|
#'   \item \code{n_min} (default 10): flag sample sizes below this value
#'   \item \code{iqr_mult} (default 3): IQR multiplier for the cross-row outlier detection
#'   \item \code{dispersion_max_smd} / \code{dispersion_max_r} / \code{dispersion_max_logor} (defaults 0.5 / 0.15 / 1.0): maximum SD of the ES across estimation methods (SMD, correlation, logOR/logRR)
#'   \item \code{diff_max_smd} / \code{diff_max_r} / \code{diff_max_logor} (defaults 1.0 / 0.3 / 2.0): maximum min-max ES difference across estimation methods
#'   \item \code{overlap_min} (default 0.80): minimum CI overlap between the min/max estimates (0-1 scale)
#'   \item \code{enable_cross_row} (default TRUE): enable/disable the cross-row checks
#'   \item \code{direction_conflict_min} (default 2): number of significantly-positive and significantly-negative studies needed to raise the direction conflict flag (G1)
#' }
#' @param guidance a logical value indicating whether missing data guidance should be generated for rows where the effect size is NA. Default is TRUE. When enabled, a column \code{es_guidance} (or \code{es_guidance_crude}/\code{es_guidance_adjusted}) is appended, listing the closest estimation methods and which specific columns are missing.
#' @param include_raw a logical value indicating whether the raw input columns should be appended after the effect size columns in the returned dataframe. Default is TRUE.
#' @param ... other arguments that can be passed to the function
#'
#' @details
#' Summary method for objects of class \dQuote{metaConvert} produced by the \code{\link{convert_df}}
#' function. This function automatically:
#' 1. computes all effect sizes from all available input data
#' 1. selects, if requested, a  main effect size for each association/comparison using the information passed by
#' the user in the \code{es_selected} argument of the \code{convert_df} function
#' 1. identifies the smallest and largest effect size for each association/comparison
#' 1. estimates the absolute difference between the smallest and largest effect size for each
#' association/comparison
#' 1. estimates the percentage of overlap between the 95% confidence intervals of the smallest and
#' largest effect size for each association/comparison
#'
#' @return
#' This function returns a dataframe with many columns. We present below the information stored in each column of the returned dataframe
#'
#' **1. Raw user information.**
#' The first columns placed at the left of the returned dataset are simply information provided
#' by the users to facilitate the identification of each row.
#' If the following columns are missing in the original dataset, these columns will not appear in
#' the returned dataset.
#'
#' \tabular{ll}{
#'  \code{row_id} \tab Row number in the original dataset.\cr
#'  \tab \cr
#'  \code{study_id} \tab Identifier of the study.\cr
#'  \tab \cr
#'  \code{author} \tab Name of the author of the study.\cr
#'  \tab \cr
#'  \code{year} \tab Year of publication of the study.\cr
#'  \tab \cr
#'  \code{predictor} \tab Name of the predictor (intervention, risk factor, etc.).\cr
#'  \tab \cr
#'  \code{outcome} \tab Name of the outcome.\cr
#'  \tab \cr
#'  \code{info_expected} \tab Types of input data users expect to be used to estimate their effect size measure.\cr
#'  \tab \cr
#' }
#'
#' **2. Information on generated effect sizes.**
#' Then, the function returns information on calculations. For example, users can retrieve
#' the effect size measure estimated, the number and type(s) of input data allowing to estimate the
#' chosen effect size measure, and the method used to obtain a unique effect size if overlapping
#' input data were available.
#' These columns could have several suffix.
#' * If users requested to separate crude and adjusted estimates,
#' then the following columns will be presented with both a "_crude" suffix and a "_adjusted" suffix.
#' * If users did not request to separate the presentation of crude and adjusted estimates, the following columns
#' will have no suffix.
#'
#' For example, let's take column "all_info". It can be "all_info_crude" (all input data used to estimate any crude effect size),
#' "all_info_adjusted" (all input data leading to estimate any adjusted effect size),
#' or "all_info" (all input data leading to estimate any crude or adjusted effect sizes).
#'
#' To facilitate the presentation, we thus refer to these columns as \code{name_of_the_column*},
#' the \code{*} meaning that it could end by _crude, _adjusted or "".
#'
#' \tabular{ll}{
#'  \code{all_info*} \tab list of input data available in the dataset that was used to estimate any effect size measure.\cr
#'  \tab \cr
#'  \code{measure*} \tab effect size measure requested by the user.\cr
#'  \tab \cr
#'  \code{info_measure*} \tab input data available to estimate the requested effect size measure.\cr
#'  \tab \cr
#'  \code{n_estimations*} \tab number of input data available to estimate the requested effect size measure.\cr
#'  \tab \cr
#'  \code{es_selected*} \tab method chosen by users to estimate the main effect size when overlapping data are present.\cr
#'  \tab \cr
#'  \code{info_used*} \tab type of input data used to estimate the main effect size.\cr
#'  \tab \cr
#' }
#'
#' **3. Main effect size.**
#' The following columns contain the key information, namely, the main effect size + standard error + 95% CI.
#'
#' Again, the suffix of these columns can vary depending on the separation of effect sizes
#' estimated from crude and adjusted input data.
#'
#' \tabular{ll}{
#'  \code{es*} \tab main effect size value.\cr
#'  \tab \cr
#'  \code{se*} \tab standard error of the effect size.\cr
#'  \tab \cr
#'  \code{es_ci_lo*} \tab lower bound of the 95% CI around the effect size.\cr
#'  \tab \cr
#'  \code{es_ci_up*} \tab upper bound of the 95% CI around the effect size.\cr
#'  \tab \cr
#' }
#'
#' **4. Overlapping effect sizes**
#' These columns are useful ONLY if a given comparison (i.e., row) has multiple input data
#' enabling to compute the requested effect size measure.
#'
#' These columns identify the smallest/largest effect size per comparison,
#' and some indicators of consistency.
#'
#' Again, the suffix of these columns can vary depending on the separation of effect sizes
#' estimated from crude and adjusted input data.
#'
#' \tabular{ll}{
#'  \code{min_info*} \tab type of input data leading to the smallest effect size for the comparison.\cr
#'  \tab \cr
#'  \code{min_es_value*} \tab smallest effect size value for the comparison.\cr
#'  \tab \cr
#'  \code{min_es_se*} \tab standard error of the smallest effect size for the comparison.\cr
#'  \tab \cr
#'  \code{min_es_ci_lo*} \tab lower bound of the 95% CI of the smallest effect size for the comparison.\cr
#'  \tab \cr
#'  \code{min_es_ci_up*} \tab upper bound of the 95% CI of the smallest effect size for the comparison.\cr
#'  \tab \cr
#' }
#'
#' \tabular{ll}{
#'  \code{max_info*} \tab type of input data leading to the largest effect size for the comparison.\cr
#'  \tab \cr
#'  \code{max_es_value*} \tab largest effect size value for the comparison.\cr
#'  \tab \cr
#'  \code{max_es_se*} \tab standard error of the largest effect size for the comparison.\cr
#'  \tab \cr
#'  \code{max_es_ci_lo*} \tab lower bound of the 95% CI of the largest effect size for the comparison.\cr
#'  \tab \cr
#'  \code{max_es_ci_up*} \tab upper bound of the 95% CI of the largest effect size for the comparison.\cr
#'  \tab \cr
#' }
#'
#' \tabular{ll}{
#'  \code{diff_min_max*} \tab difference between the smallest and largest effect size for the comparison.\cr
#'  \tab \cr
#'  \code{overlap_min_max*} \tab % of overlap between the 95% CIs of the largest/smallest effect sizes for the comparison.\cr
#'  \tab \cr
#'  \code{dispersion_es*} \tab standard deviation of all effect sizes for the comparison.\cr
#'  \tab \cr
#' }
#'
#' **5. Quality/plausibility flags**
#' When \code{flags = TRUE} (the default), a flags* column is added, containing
#' the semicolon-separated list of issues detected for the row (empty when none).
#'
#' \tabular{ll}{
#'  \code{flags*} \tab quality/plausibility flags for the effect size in this row.\cr
#'  \tab \cr
#' }
#'
#' Flag categories:
#' \itemize{
#'   \item \strong{A (Numeric integrity)}: Inf/NaN values, negative SE, inverted CI, ES outside CI
#'   \item \strong{B (Bounds violations)}: correlation outside \eqn{[-1,1]}, non-positive OR/RR, NNT in (-1,1), proportion outside \eqn{[0,1]}, RD outside \eqn{[-1,1]}
#'   \item \strong{C (Plausibility)}: unusually large SMD, zero SE, high correlation, large logOR/logRR, small sample size
#'   \item \strong{D (Cross-row outliers)}: ES or SE is an outlier relative to other rows (IQR method)
#'   \item \strong{E (Internal consistency)}: high dispersion across estimation methods, low/zero CI overlap, large min-max difference
#' }
#'
#' **6. Missing data guidance**
#' When \code{guidance = TRUE} (the default), an es_guidance* column indicates,
#' for rows without an effect size, the closest estimation methods and which
#' input columns are missing (empty otherwise).
#'
#' \tabular{ll}{
#'  \code{es_guidance*} \tab guidance on which columns to add to obtain an effect size.\cr
#'  \tab \cr
#' }
#'
#' @seealso
#' \code{\link{metaConvert-package}} for the formatting of well-formatted datasets\cr
#' \code{\link{convert_df}} for estimating effect sizes from a dataset\cr
#'
#' @exportS3Method
#' @export summary.metaConvert
#'
#' @md
#' @examples
#' ### generate a summary of the results of an umbrella object
#' summary(
#'   convert_df(df.haza, measure = "g"),
#'   digits = 5)
summary.metaConvert <- function(object, digits = 3, flags = TRUE, flag_options = list(), guidance = TRUE, include_raw = TRUE, ...) {
  # object = convert_df(dat, verbose = FALSE,
  #                     or_to_rr = "dipietrantonj", measure="nnt")
  # digits = 3
  # object = convert_df(subset(df.haza, !is.na(n_cases_exp) & !is.na(n_cases_nexp) &
  #                              !is.na(n_controls_exp) & !is.na(n_controls_nexp)),
  #                     verbose = FALSE, hierarchy = "2x2", measure = "nnt")
  # rio::export(data.frame(do.call(rbind, lapply(object, function(x) x[1, "info_used"]))),
  #             "possible_inputs.xlsx", overwrite = TRUE)
  raw_data <- attr(object, "raw_data")
  hierarchy <- attr(object, "hierarchy")
  exp <- attr(object, "exp")
  measure <- attr(object, "measure")
  split_adjusted <- attr(object, "split_adjusted")
  es_selected <- attr(object, "es_selected")
  format <- attr(object, "format_adjusted")
  main_es <- attr(object, "main_es")

  # add missing columns to each dataset (used to easily detect information leading to an ES)
  list_df_es_enh <- lapply(object, .add_columns, y = c(
    "d", "d_se",
    "g", "g_se",
    "md", "md_se",
    "r", "r_se",
    "z", "z_se",
    "logor", "logor_se",
    "logrr", "logrr_se",
    "logirr", "logirr_se",
    "logcvr", "logcvr_se",
    "logvr", "logvr_se",
    "rp", "rp_se",
    "zp", "zp_se",
    "nnt", "nnt_se", "nnt_ci_lo", "nnt_ci_up",
    "rd", "rd_se", "rd_ci_lo", "rd_ci_up",
    "prop", "prop_se", "prop_ci_lo", "prop_ci_up",
    "alpha", "alpha_se", "alpha_ci_lo", "alpha_ci_up",
    "icc", "icc_se", "icc_ci_lo", "icc_ci_up"
  ))

  # extract the values for the correct effect measure
  df_es <- lapply(object, .extract_es, measure = measure, exp = exp)

  # name of all information that could lead to an ES
  list_order <- as.character(sapply(df_es, function(x) x$info_used[1]))
  # rio::export(list_order, "list_possible_inputs.xlsx")

  # hierarchy indicated by user
  if (es_selected == "auto") {
    ordering_full <- list_order
  } else {
    ordering_raw <- gsub(" ", "", unlist(strsplit(hierarchy, split = ">", fixed = TRUE)))
    ordering_full <- append(ordering_raw, list_order[!list_order %in% ordering_raw])
  }

  # append the user hierarchy with all information

  # warn users if wrong inputs have been indicated in the hierarchy
  if (hierarchy == "hierarchy" & length(ordering_full[!ordering_full %in% list_order] > 0)) {
    stop(paste0(
      "Watch out! You have indicated elements that do not exist in the hierarchy. Please discard and replace the following elements: ",
      paste(ordering_full[!ordering_full %in% list_order], collapse = " + ")
    ))
  }

  # final list of hierarchy properly organized
  ordering_tot <- ordering_full[ordering_full %in% list_order]

  adj_list <- ordering_tot[which(grepl("adj", ordering_tot, fixed = TRUE) |
    grepl("ancova", ordering_tot, fixed = TRUE))]

  idx_adj <- which(ordering_tot %in% adj_list)
  ordering_crude <- if (length(idx_adj) > 0) as.character(ordering_tot[-idx_adj]) else as.character(ordering_tot)
  ordering_adj <- as.character(ordering_tot[idx_adj])

  # -----------------------------------------------------------------------

  if (split_adjusted == TRUE & format == "wide" & main_es == TRUE) {
    res1 <- .generate_df(
      x = raw_data, list_df = df_es, ordering = ordering_crude, exp = exp,
      digits = digits, suffix = "_crude", measure = measure,
      es_selected = es_selected, list_df_es_enh = list_df_es_enh
    )
    # x = raw_data; list_df = df_es; ordering = ordering_crude; exp = exp;
    # digits = 3; suffix = "_crude"; measure = measure; main_es=TRUE
    res <- .generate_df(
      x = res1, list_df = df_es, ordering = ordering_adj, exp = exp,
      digits = digits, suffix = "_adjusted", measure = measure,
      es_selected = es_selected, list_df_es_enh = list_df_es_enh
    )
    # x = res1; list_df = df_es; ordering = ordering_adj; exp = exp;
    # digits = 3; suffix = "_adjusted"; measure = measure
  } else if (split_adjusted == TRUE & format == "long" & main_es == TRUE) {
    res1 <- .generate_df(
      x = raw_data, list_df = df_es, ordering = ordering_crude, exp = exp,
      digits = digits, suffix = "", measure = measure,
      es_selected = es_selected, list_df_es_enh = list_df_es_enh,
      main_es = TRUE
    )
    res1$adjusted_input = FALSE
    # x = raw_data; list_df = df_es; ordering = ordering_crude; exp = exp;
    # digits = 3; suffix = ""; measure = measure; es_selected = "hierarchy"; list_df_es_enh = list_df_es_enh

    res2 <- .generate_df(
      x = raw_data, list_df = df_es, ordering = ordering_adj, exp = exp,
      digits = digits, suffix = "", measure = measure,
      es_selected = es_selected, list_df_es_enh = list_df_es_enh,
      main_es = TRUE
    )
    res2$adjusted_input = TRUE

    res1_sub = subset(res1, !is.na(res1$measure))
    res2_sub = subset(res2, !is.na(res2$measure))
    res_transit <- rbind(res1_sub, res2_sub)
    res <- res_transit[order(res_transit$row_id), ]

  } else if (split_adjusted == FALSE & main_es == TRUE) {
    res <- .generate_df(
      x = raw_data, list_df = df_es, ordering = ordering_tot, exp = exp,
      digits = digits, suffix = "", measure = measure,
      es_selected = es_selected, list_df_es_enh = list_df_es_enh
    )

    res = subset(res, select = -c(adjusted_input))
  } else if (main_es == FALSE) {
    res1 <- .generate_df(
      x = raw_data, list_df = df_es, ordering = ordering_crude, exp = exp,
      digits = digits, suffix = "", measure = measure,
      es_selected = es_selected, list_df_es_enh = list_df_es_enh,
      main_es = FALSE
    )
    res1$adjusted_input = FALSE
    # x = raw_data; list_df = df_es; ordering = ordering_crude; exp = exp;
    # digits = 3; suffix = ""; measure = measure; es_selected = "hierarchy"; list_df_es_enh = list_df_es_enh

    res2 <- .generate_df(
      x = raw_data, list_df = df_es, ordering = ordering_adj, exp = exp,
      digits = digits, suffix = "", measure = measure,
      es_selected = es_selected, list_df_es_enh = list_df_es_enh,
      main_es = FALSE
    )
    res2$adjusted_input = TRUE

    res1_sub = subset(res1, !is.na(res1$measure))
    res2_sub = subset(res2, !is.na(res2$measure))
    res_transit <- rbind(res1_sub, res2_sub)
    res <- res_transit[order(res_transit$row_id), ]
    res$es_selected = "no selection"
  } else {
    stop("The combination of 'main_es', 'format_adjusted' and 'split_adjusted' is incorrect. Check documentation for more info.")
  }

  # compact summary columns
  if (split_adjusted == TRUE & format == "wide" & main_es == TRUE) {
    res <- .make_es_summary(res, measure, exp, digits, "_crude")
    res <- .make_es_consistency(res, digits, "_crude")
    res <- .make_es_summary(res, measure, exp, digits, "_adjusted")
    res <- .make_es_consistency(res, digits, "_adjusted")
  } else {
    res <- .make_es_summary(res, measure, exp, digits, "")
    res <- .make_es_consistency(res, digits, "")
  }

  # quality flags
  if (flags) {
    stored_opts <- attr(object, "flag_options")
    if (is.null(stored_opts)) stored_opts <- .default_flag_options()
    opts <- stored_opts
    opts[names(flag_options)] <- flag_options
    input_val <- attr(object, "input_validation")
    r_def <- attr(object, "r_defaulted")
    alpha_method <- attr(object, "alpha_to_es")
    if (is.null(alpha_method)) alpha_method <- "bonett"
    icc_method <- attr(object, "icc_to_es")
    if (is.null(icc_method)) icc_method <- "bonett"
    prop_method <- attr(object, "prop_to_es")
    if (is.null(prop_method)) prop_method <- "raw"
    pp_method <- attr(object, "pre_post_to_smd")
    if (is.null(pp_method)) pp_method <- "bonett"
    pool_sd_used <- attr(object, "pool_sd")
    if (is.null(pool_sd_used)) pool_sd_used <- FALSE
    smd_denom_used <- attr(object, "smd_denom_used")

    if (split_adjusted == TRUE & format == "wide" & main_es == TRUE) {
      res <- .flag_es_quality(res, measure, exp, "_crude", raw_data, opts, input_val,
                              alpha_to_es = alpha_method, icc_to_es = icc_method,
                              prop_to_es = prop_method, pre_post_to_smd = pp_method,
                              pool_sd = pool_sd_used, r_defaulted = r_def,
                              smd_denom = smd_denom_used)
      res <- .flag_es_quality(res, measure, exp, "_adjusted", raw_data, opts, input_val,
                              alpha_to_es = alpha_method, icc_to_es = icc_method,
                              prop_to_es = prop_method, pre_post_to_smd = pp_method,
                              pool_sd = pool_sd_used, r_defaulted = r_def,
                              smd_denom = smd_denom_used)
    } else {
      res <- .flag_es_quality(res, measure, exp, "", raw_data, opts, input_val,
                              alpha_to_es = alpha_method, icc_to_es = icc_method,
                              prop_to_es = prop_method, pre_post_to_smd = pp_method,
                              pool_sd = pool_sd_used, r_defaulted = r_def,
                              smd_denom = smd_denom_used)
    }
  }

  # missing data guidance
  if (guidance) {
    if (split_adjusted == TRUE & format == "wide" & main_es == TRUE) {
      res <- .add_es_guidance(res, "_crude", raw_data, measure, object)
      res <- .add_es_guidance(res, "_adjusted", raw_data, measure, object)
    } else {
      res <- .add_es_guidance(res, "", raw_data, measure, object)
    }
  }

  # rounding (flags above use full precision)
  if (split_adjusted == TRUE & format == "wide" & main_es == TRUE) {
    res <- .round_numeric_cols(res, digits, "_crude")
    res <- .round_numeric_cols(res, digits, "_adjusted")
  } else {
    res <- .round_numeric_cols(res, digits, "")
  }

  # reorder columns
  all_cols <- colnames(res)

  identity_names   <- c("row_id", "study_id", "author", "year", "outcome",
                        "predictor", "info_expected", "adjusted_input")
  provenance_names <- c("all_info", "info_measure", "es_guidance", "es_selected")
  quality_names    <- c("flags")
  summary_names    <- c("es_summary", "es_consistency", "n_estimations",
                        "dispersion_es")
  primary_names    <- c("es", "se", "es_ci_lo", "es_ci_up",
                        "info_used", "measure")
  minmax_names     <- c("overlap_min_max", "diff_min_max",
                        "min_info", "max_info",
                        "min_es_value", "min_es_se", "min_es_ci_lo", "min_es_ci_up",
                        "max_es_value", "max_es_se", "max_es_ci_lo", "max_es_ci_up")

  if (split_adjusted == TRUE & format == "wide" & main_es == TRUE) {
    suffixes <- c("_crude", "_adjusted")
  } else {
    suffixes <- ""
  }

  ordered <- character(0)
  ordered <- c(ordered, identity_names[identity_names %in% all_cols])
  for (sfx in suffixes) {
    for (block in list(provenance_names, quality_names, summary_names,
                       primary_names, minmax_names)) {
      cols <- paste0(block, sfx)
      ordered <- c(ordered, cols[cols %in% all_cols])
    }
  }
  remaining <- all_cols[!all_cols %in% ordered]
  if (isTRUE(include_raw)) {
    ordered <- c(ordered, remaining)
  }
  res <- res[, ordered, drop = FALSE]

  if (all(is.na(res$author))) res = subset(res, select = -c(author))
  if (all(is.na(res$year))) res = subset(res, select = -c(year))
  if (all(is.na(res$predictor))) res = subset(res, select = -c(predictor))
  if (all(is.na(res$outcome))) res = subset(res, select = -c(outcome))
  if (all(is.na(res$info_expected))) res = subset(res, select = -c(info_expected))

  main_cols <- which(colnames(res) %in% c(
    "es", "se", "es_ci_lo", "es_ci_up",
    "es_crude", "se_crude", "es_ci_lo_crude", "es_ci_up_crude",
    "es_adjusted", "se_adjusted", "es_ci_lo_adjusted", "es_ci_up_adjusted"
  ))

  res[, main_cols] <- lapply(res[, main_cols], function(x) as.numeric(as.character(x)))

  # diagnostic overview
  .measure_label <- function(m) {
    labels <- c(d = "Cohen's d", g = "Hedges' g", md = "Mean difference",
                r = "Pearson r", z = "Fisher's z", or = "Odds ratio",
                rr = "Risk ratio", irr = "Incidence rate ratio",
                logor = "log(OR)", logrr = "log(RR)", logirr = "log(IRR)",
                logvr = "log(VR)", logcvr = "log(CVR)",
                nnt = "NNT", rd = "Risk difference",
                dw = "Within-group d", gw = "Within-group g",
                mdw = "Within-group MD",
                rp = "Partial r", zp = "Fisher's z of partial r",
                prop = "Proportion", alpha = "Cronbach's alpha",
                icc = "ICC")
    if (m %in% names(labels)) labels[m] else m
  }

  .summary_for_suffix <- function(res, suffix) {
    es_col <- paste0("es", suffix)
    info_col <- paste0("info_used", suffix)
    flags_col <- paste0("flags", suffix)
    if (!es_col %in% colnames(res)) return(NULL)

    n_total <- nrow(res)
    es_vals <- res[[es_col]]
    n_estimated <- sum(!is.na(es_vals))
    pct <- round(100 * n_estimated / n_total)

    info_vals <- res[[info_col]]
    info_vals <- info_vals[!is.na(info_vals) & info_vals != ""]
    method_freq <- sort(table(info_vals), decreasing = TRUE)

    n_flags <- 0
    if (flags_col %in% colnames(res)) {
      flag_vals <- res[[flags_col]]
      n_flags <- sum(!is.na(flag_vals) & nchar(flag_vals) > 0)
    }

    missing_rows <- which(is.na(es_vals))

    list(n_total = n_total, n_estimated = n_estimated, pct = pct,
         method_freq = method_freq, n_flags = n_flags,
         missing_rows = missing_rows)
  }

  is_wide_split <- split_adjusted == TRUE & format == "wide" & main_es == TRUE

  if (is_wide_split) {
    info_crude <- .summary_for_suffix(res, "_crude")
    info_adj   <- .summary_for_suffix(res, "_adjusted")

    message("\n-- metaConvert summary --")
    message("Measure: ", .measure_label(measure), "  |  ", info_crude$n_total, " studies")

    # Crude
    message("\nCrude estimates: ", info_crude$n_estimated, "/", info_crude$n_total,
            " (", info_crude$pct, "%)")
    if (length(info_crude$method_freq) > 0) {
      top <- utils::head(info_crude$method_freq, 5)
      for (i in seq_along(top)) {
        message("  ", format(names(top)[i], width = 25), " ", top[i])
      }
    }
    if (info_crude$n_flags > 0) message(info_crude$n_flags, " quality flag(s) raised")
    if (length(info_crude$missing_rows) > 0 && length(info_crude$missing_rows) <= 10) {
      message("Missing ES in rows: ", paste(info_crude$missing_rows, collapse = ", "))
    } else if (length(info_crude$missing_rows) > 10) {
      message(length(info_crude$missing_rows), " rows with missing ES")
    }

    # Adjusted
    if (!is.null(info_adj)) {
      message("\nAdjusted estimates: ", info_adj$n_estimated, "/", info_adj$n_total,
              " (", info_adj$pct, "%)")
      if (length(info_adj$method_freq) > 0) {
        top <- utils::head(info_adj$method_freq, 5)
        for (i in seq_along(top)) {
          message("  ", format(names(top)[i], width = 25), " ", top[i])
        }
      }
      if (info_adj$n_flags > 0) message(info_adj$n_flags, " quality flag(s) raised")
    }
  } else {
    info <- .summary_for_suffix(res, "")
    if (!is.null(info)) {
      message("\n-- metaConvert summary --")
      message("Measure: ", .measure_label(measure), "  |  ", info$n_total, " studies")
      message("ES estimated: ", info$n_estimated, "/", info$n_total,
              " (", info$pct, "%)")
      if (length(info$method_freq) > 0) {
        message("\nMethods selected:")
        top <- utils::head(info$method_freq, 5)
        for (i in seq_along(top)) {
          message("  ", format(names(top)[i], width = 25), " ", top[i])
        }
      }
      if (info$n_flags > 0) message(info$n_flags, " quality flag(s) raised")
      if (length(info$missing_rows) > 0 && length(info$missing_rows) <= 10) {
        message("Missing ES in rows: ", paste(info$missing_rows, collapse = ", "))
      } else if (length(info$missing_rows) > 10) {
        message(length(info$missing_rows), " rows with missing ES")
      }
    }
  }

  return(res)
}


#' Print a summary of an object of class \dQuote{metaConvert}
#'
#' @param x an object of class \dQuote{metaConvert}
#' @param ... other arguments that can be passed to the function
#'
#' @details
#' Summary method for objects of class \dQuote{metaConvert}.
#'
#' @return
#' Implicitly calls the \code{\link{summary.metaConvert}} function.
#'
#' @export
#'
#' @md
#'
#' @seealso
#' \code{\link{summary.metaConvert}}
#'
#' @examples
#' ### print the results of an object of class metaConvert
#' convert_df(df.haza, measure = "g")
print.metaConvert <- function(x, ...) {
  y <- summary.metaConvert(x, digits = 3, ...)
  print(y)
  invisible(x)
}

#' Convert a \dQuote{metaConvert} object to a dataframe
#'
#' @param x an object of class \dQuote{metaConvert}
#' @param ... other arguments passed to \code{\link{summary.metaConvert}}
#'
#' @details
#' Convenience wrapper around \code{summary.metaConvert(x, ...)}.
#'
#' @return
#' A dataframe containing the effect size results, diagnostics, and all input data columns.
#'
#' @seealso
#' \code{\link{summary.metaConvert}}
#'
#' @exportS3Method
#'
#' @md
#'
#' @examples
#' ### get the full output as a dataframe
#' as.data.frame(convert_df(df.haza, measure = "g"))
as.data.frame.metaConvert <- function(x, ...) {
  summary.metaConvert(x, ...)
}

# # !!! for alll internal formulas, check whether nn_miss does not prevent the code to be ran
# # important to check the input of the method before calling these functions
#
# "Re: Specific question 4. I've thought of a new strategy. Instead of pivoting on the p-value, we can pivot on the "effective n". In other words, we calculate the n that would result in vz, and then we calculated vr using its formula with this n:
# effective_n = 1 / vz + 3
# vr = (1 - r^2) / (effective_n - 2)"
# 'Re: Specific question 1. That is strange, the formula of the variance of r is sqrt((1-r_raw_data^2) / (n-2)), see it e.g., in wikipedia (https://en.wikipedia.org/wiki/Pearson_correlation_coefficient#Standard_error). Indeed, you can check that this formula coincides with the one returned by "cor.test":'
#
# 'Re: Specific questions 2 and 3. I don't know, maybe we could use qnorm but trigger a warning?
# '


#
# One last question about this. In some instances, authors can obtain R without the sample size
# (e.g., when converting OR => R using the pearson/digby approaches).
# What should we do to estimate the 95% CI in these cases?
# We apply the qt() to obtain the 95% CI if users give the n_sample,
# but we derive the 95% from the Z (tanh(z_ci_lo) ; tanh(z_ci_up))
# if we have no information on sample? (or we always use tanh /
# we force users to indicate a sample size?)
#
# ## Specific question 3
# This question 2 led me to the exact same question for SMD. When converting OR + SE => SMD, users can obtain SMD + SE without having access to a sample size. In these cases, we derive the 95% CI using qnorm()? (otherwise, using qt, we present a SMD+SE but without 95% CI)
#

################### OR to RR ###################
.or_to_rr <- function(or, logor_se, or_ci_lo, or_ci_up,
                      n_cases, n_controls, n_exp, n_nexp, baseline_risk, or_to_rr) {
   if (or_to_rr == "grant") {
    logrr_grant <- suppressWarnings(log(or / (1 - baseline_risk + (baseline_risk * or))))
    logrr_ci_lo_grant_CI <- suppressWarnings(log(or_ci_lo / (1 - baseline_risk + (baseline_risk * or_ci_lo))))
    logrr_ci_up_grant_CI <- suppressWarnings(log(or_ci_up / (1 - baseline_risk + (baseline_risk * or_ci_up))))
    logrr_se_CI <- (logrr_ci_up_grant_CI - logrr_ci_lo_grant_CI) / (2 * qnorm(.975))
    res <- cbind(
      logrr = logrr_grant,
      logrr_se = logrr_se_CI,
      logrr_ci_lo = logrr_ci_lo_grant_CI,
      logrr_ci_up = logrr_ci_up_grant_CI
    )

    return(res)
  } else if (or_to_rr == "metaumbrella_cases") {
    contingency_meta_cases <- .estimate_n_from_or_and_n_cases(
      or = or, var = logor_se^2,
      n_cases = n_cases, n_controls = n_controls
    )

    calc_meta_cases <- es_from_2x2(
      n_cases_exp = contingency_meta_cases$n_cases_exp,
      n_controls_exp = contingency_meta_cases$n_controls_exp,
      n_cases_nexp = contingency_meta_cases$n_cases_nexp,
      n_controls_nexp = contingency_meta_cases$n_controls_nexp
    )

    res <- cbind(
      logrr = calc_meta_cases$logrr,
      logrr_se = calc_meta_cases$logrr_se,
      logrr_ci_lo = calc_meta_cases$logrr_ci_lo,
      logrr_ci_up = calc_meta_cases$logrr_ci_up
    )

    return(res)
  } else if (or_to_rr == "metaumbrella_exp") {
    contingency_meta_exp <- .estimate_n_from_or_and_n_exp(or = or, var = logor_se^2, n_exp = n_exp, n_nexp = n_nexp)
    calc_meta_exp <- es_from_2x2(
      n_cases_exp = contingency_meta_exp$n_cases_exp,
      n_controls_exp = contingency_meta_exp$n_controls_exp,
      n_cases_nexp = contingency_meta_exp$n_cases_nexp,
      n_controls_nexp = contingency_meta_exp$n_controls_nexp
    )

    res <- cbind(
      logrr = calc_meta_exp$logrr,
      logrr_se = calc_meta_exp$logrr_se,
      logrr_ci_lo = calc_meta_exp$logrr_ci_lo,
      logrr_ci_up = calc_meta_exp$logrr_ci_up
    )

    return(res)
  } else if (or_to_rr == "transpose") {
    res <- cbind(
      logrr = log(or),
      logrr_se = logor_se,
      logrr_ci_lo = log(or_ci_lo),
      logrr_ci_up = log(or_ci_up)
    )

    return(res)
  } else if (or_to_rr == "dipietrantonj") {
    n_dec <- max(
      nchar(gsub("^.+[.]", "", or)),
      nchar(gsub("^.+[.]", "", or_ci_lo)),
      nchar(gsub("^.+[.]", "", or_ci_up))
    )

    estim <- estimraw::estim_raw(
      es = or, lb = or_ci_lo, ub = or_ci_up,
      m1 = n_exp, m2 = n_nexp, dec = n_dec, measure = "or"
    )

    if (length(estim) != 4) {
      numb = 1
      if (!is.na(baseline_risk)) {
        numb = which.min(
          c(
            abs(estim[[1]]$c[1] / (estim[[1]]$c[1] + estim[[1]]$d[1]) -
                  baseline_risk),
            abs(estim[[2]]$c[1] / (estim[[2]]$c[1] + estim[[2]]$d[1]) -
                  baseline_risk)
            )
        )
      }
      calc_dipie <- es_from_2x2(
        n_cases_exp = estim[[numb]]$a[1],
        n_controls_exp = estim[[numb]]$b[1],
        n_cases_nexp = estim[[numb]]$c[1],
        n_controls_nexp = estim[[numb]]$d[1]
      )

    } else {
      calc_dipie <- es_from_2x2(
        n_cases_exp = estim$a[1],
        n_controls_exp = estim$b[1],
        n_cases_nexp = estim$c[1],
        n_controls_nexp = estim$d[1]
      )
    }
    res <- data.frame(
      logrr = as.numeric(calc_dipie$logrr),
      logrr_se = as.numeric(calc_dipie$logrr_se),
      logrr_ci_lo = as.numeric(calc_dipie$logrr_ci_lo),
      logrr_ci_up = as.numeric(calc_dipie$logrr_ci_up)
    )
    res$logrr = res$logrr
    res$logrr_se = res$logrr_se
    res$logrr_ci_lo = res$logrr_ci_lo
    res$logrr_ci_up = res$logrr_ci_up

    return(res)
  }
}
################### OR to 2x2 ##################
# internal function
.estimate_n_from_or_and_n_cases <- function(or, var, n_cases, n_controls) {
  res <- data.frame(n_cases_exp = NA, n_cases_nexp = NA, n_controls_exp = NA, n_controls_nexp = NA)

  if (!is.na(or) & !is.na(var) & !is.na(n_cases) & !is.na(n_controls)) {
    # Create all possibilites of n
    n_cases_nexp_sim1 <- 0:n_cases
    n_controls_nexp_sim1 <- round(n_controls * (1 - (n_cases - n_cases_nexp_sim1) / (n_cases + (or - 1) * n_cases_nexp_sim1)))
    n_cases_exp_sim1 <- n_cases - n_cases_nexp_sim1
    n_controls_exp_sim1 <- n_controls - n_controls_nexp_sim1
    # sim1: possiblities with strictly positive n
    idx_non_zero <- which(
      n_cases_nexp_sim1 > 0 &
        n_controls_nexp_sim1 > 0 &
        n_cases_exp_sim1 > 0 &
        n_controls_exp_sim1 > 0
    )
    n_cases_nexp_sim1 <- n_cases_nexp_sim1[idx_non_zero]
    n_controls_nexp_sim1 <- n_controls_nexp_sim1[idx_non_zero]
    n_cases_exp_sim1 <- n_cases_exp_sim1[idx_non_zero]
    n_controls_exp_sim1 <- n_controls_exp_sim1[idx_non_zero]

    # sim2: possiblities with positive n and at least one zero (Add 0.5 to the possiblities with any 0)
    n_cases_nexp_sim2 <- 0:n_cases
    n_controls_nexp_sim2 <- round((n_controls + 0.5) - (n_controls + 1) * (n_cases - n_cases_nexp_sim2 + 0.5) / ((n_cases_nexp_sim2 + 0.5) * or + n_cases - n_cases_nexp_sim2 + 0.5))
    n_cases_exp_sim2 <- n_cases - n_cases_nexp_sim2
    n_controls_exp_sim2 <- n_controls - n_controls_nexp_sim2
    # (n_cases_exp_sim2 + 0.5) / (n_cases_nexp_sim2 + 0.5) / (n_controls_exp_sim2 + 0.5) * (n_controls_nexp_sim2 + 0.5)
    # select the ones with some 0 but non-negative
    idx_some_zero <- which(
      (n_cases_nexp_sim2 == 0 | n_controls_nexp_sim2 == 0 | n_cases_exp_sim2 == 0 | n_controls_exp_sim2 == 0) &
        (n_cases_nexp_sim2 >= 0 & n_controls_nexp_sim2 >= 0 & n_cases_exp_sim2 >= 0 & n_controls_exp_sim2 >= 0)
    )
    n_cases_nexp_sim2 <- n_cases_nexp_sim2[idx_some_zero]
    n_controls_nexp_sim2 <- n_controls_nexp_sim2[idx_some_zero]
    n_cases_exp_sim2 <- n_cases_exp_sim2[idx_some_zero]
    n_controls_exp_sim2 <- n_controls_exp_sim2[idx_some_zero]

    # join both previous vectors
    n_cases_nexp_sim <- append(n_cases_nexp_sim1, n_cases_nexp_sim2)
    n_controls_nexp_sim <- append(n_controls_nexp_sim1, n_controls_nexp_sim2)
    n_cases_exp_sim <- append(n_cases_exp_sim1, n_cases_exp_sim2)
    n_controls_exp_sim <- append(n_controls_exp_sim1, n_controls_exp_sim2)

    some_zero <- n_cases_exp_sim == 0 | n_controls_exp_sim == 0 | n_cases_nexp_sim == 0 | n_controls_nexp_sim == 0

    var_sim <- ifelse(some_zero,
      1 / ((n_cases + 1) - (n_cases_nexp_sim + 0.5)) + 1 / ((n_controls + 1) - (n_controls_nexp_sim + 0.5)) + 1 / (n_cases_nexp_sim + 0.5) + 1 / (n_controls_nexp_sim + 0.5),
      1 / (n_cases - n_cases_nexp_sim) + 1 / (n_controls - n_controls_nexp_sim) + 1 / n_cases_nexp_sim + 1 / n_controls_nexp_sim
    )

    # var_sim2 = 1 / ((n_cases+1) - (n_cases_nexp_sim+0.5)) + 1 / ((n_controls+1) - (n_controls_nexp_sim+0.5)) + 1 / (n_cases_nexp_sim+0.5) + 1 / (n_controls_nexp_sim+0.5)

    best <- order((var_sim - var)^2)[1]

    res$n_cases_nexp <- n_cases_nexp_sim[best]
    res$n_controls_nexp <- n_controls_nexp_sim[best]
    res$n_cases_exp <- n_cases - res$n_cases_nexp
    res$n_controls_exp <- n_controls - res$n_controls_nexp
  }

  return(res)
}

#' Estimate the n, using the variance, the number of exposed and non-exposed subjects
#'
#' @param or OR
#' @param var variance
#' @param n_exp number of exposed participants
#' @param n_nexp number of non exposed participants
#'
#' @noRd
.estimate_n_from_or_and_n_exp <- function(or, var, n_exp, n_nexp) {
  res <- data.frame(n_cases_exp = NA, n_cases_nexp = NA, n_controls_exp = NA, n_controls_nexp = NA)

  if (!is.na(or) & !is.na(var) & !is.na(n_exp) & !is.na(n_nexp)) {
    # first: uncorrected values with 0
    n_controls_exp_sim1 <- 0:n_exp
    n_controls_nexp_sim1 <- round(n_nexp / (1 + (n_exp - n_controls_exp_sim1) / (or * n_controls_exp_sim1)))
    n_cases_exp_sim1 <- n_exp - n_controls_exp_sim1
    n_cases_nexp_sim1 <- n_nexp - n_controls_nexp_sim1
    # we take the ones without 0 and non-negative
    idx_non_zero <- which(n_cases_nexp_sim1 > 0 & n_controls_nexp_sim1 > 0 & n_cases_exp_sim1 > 0 & n_controls_exp_sim1 > 0) # Posem ">" i no "!=" per treure els negatius!
    n_cases_nexp_sim1 <- n_cases_nexp_sim1[idx_non_zero]
    n_controls_nexp_sim1 <- n_controls_nexp_sim1[idx_non_zero]
    n_cases_exp_sim1 <- n_cases_exp_sim1[idx_non_zero]
    n_controls_exp_sim1 <- n_controls_exp_sim1[idx_non_zero]

    # correcting by 0.5
    n_controls_exp_sim2 <- 0:n_exp
    n_controls_nexp_sim2 <- round((n_nexp + 0.5) - ((n_nexp + 1) * (n_exp - n_controls_exp_sim2 + 0.5)) / ((n_controls_exp_sim2 + 0.5) * or + n_exp - n_controls_exp_sim2 + 0.5))
    n_cases_exp_sim2 <- n_exp - n_controls_exp_sim2
    n_cases_nexp_sim2 <- n_nexp - n_controls_nexp_sim2

    # SELECT THE ONES THAT HAS SOME 0 BUT NO NEGATIVE ONES
    idx_some_zero <- which(
      (n_cases_nexp_sim2 == 0 | n_controls_nexp_sim2 == 0 | n_cases_exp_sim2 == 0 | n_controls_exp_sim2 == 0) &
        (n_cases_nexp_sim2 >= 0 & n_controls_nexp_sim2 >= 0 & n_cases_exp_sim2 >= 0 & n_controls_exp_sim2 >= 0)
    )
    n_cases_nexp_sim2 <- n_cases_nexp_sim2[idx_some_zero]
    n_controls_nexp_sim2 <- n_controls_nexp_sim2[idx_some_zero]
    n_cases_exp_sim2 <- n_cases_exp_sim2[idx_some_zero]
    n_controls_exp_sim2 <- n_controls_exp_sim2[idx_some_zero]

    n_controls_exp_sim <- append(n_controls_exp_sim1, n_controls_exp_sim2)
    n_controls_nexp_sim <- append(n_controls_nexp_sim1, n_controls_nexp_sim2)
    n_cases_exp_sim <- append(n_cases_exp_sim1, n_cases_exp_sim2)
    n_cases_nexp_sim <- append(n_cases_nexp_sim1, n_cases_nexp_sim2)


    some_zero <- n_cases_exp_sim == 0 | n_controls_exp_sim == 0 | n_cases_nexp_sim == 0 | n_controls_nexp_sim == 0
    var_sim <- ifelse(some_zero,
      1 / ((n_exp + 1) - (n_controls_exp_sim + 0.5)) + 1 / (n_controls_exp_sim + 0.5) + 1 / ((n_nexp + 1) - (n_controls_nexp_sim + 0.5)) + 1 / (n_controls_nexp_sim + 0.5),
      1 / (n_exp - n_controls_exp_sim) + 1 / n_controls_exp_sim + 1 / (n_nexp - n_controls_nexp_sim) + 1 / n_controls_nexp_sim
    )

    # var_sim = 1 / (n_exp - n_controls_exp_sim) + 1 / n_controls_exp_sim + 1 / (n_nexp - n_controls_nexp_sim) + 1 / n_controls_nexp_sim
    best <- order((var_sim - var)^2)[1]
    res$n_controls_exp <- n_controls_exp_sim[best]
    res$n_controls_nexp <- n_controls_nexp_sim[best]
    res$n_cases_exp <- n_exp - res$n_controls_exp
    res$n_cases_nexp <- n_nexp - res$n_controls_nexp
  }
  return(res)
}

################### SE of OR ###################
.se_from_or <- function(x) {
  or <- as.numeric(x[1])
  n_cases <- as.numeric(x[2])
  n_controls <- as.numeric(x[3])

  res <- data.frame(value = NA, var = NA, se = NA)

  if (!is.na(or) & !is.na(n_cases) & !is.na(n_controls)) {
    cases_exp <- 1:(n_cases - 1)
    cases_nexp <- n_cases - cases_exp
    controls_exp <- round(n_controls / (1 + cases_nexp * or / cases_exp))
    controls_exp[which(controls_exp < 1 | controls_exp > n_controls - 1)] <- NA
    controls_nexp <- n_controls - controls_exp
    v_or_mean <- mean(1 / cases_exp + 1 / cases_nexp + 1 / controls_exp + 1 / controls_nexp, na.rm = TRUE)

    res$value <- or
    res$var <- v_or_mean
    res$se <- sqrt(v_or_mean)
  }

  return(res)
}

################### RR to OR ###################
.rr_to_or <- function(rr, logrr_se, rr_ci_lo, rr_ci_up,
                      n_cases, n_controls, n_exp, n_nexp,
                      baseline_risk, rr_to_or) {
  if (rr_to_or == "grant") {
    logor_grant <- suppressWarnings(log(rr * (1 - baseline_risk) / (1 - rr * baseline_risk)))
    logor_ci_lo_grant <- suppressWarnings(log(rr_ci_lo * (1 - baseline_risk) / (1 - rr_ci_lo * baseline_risk)))
    logor_ci_up_grant <- suppressWarnings(log(rr_ci_up * (1 - baseline_risk) / (1 - rr_ci_up * baseline_risk)))
    logor_se <- (logor_ci_up_grant - logor_ci_lo_grant) / (2 * qnorm(.975))
    res <- cbind(
      logor = logor_grant,
      logor_se = logor_se,
      logor_ci_lo = logor_ci_lo_grant,
      logor_ci_up = logor_ci_up_grant
    )
    return(res)
  } else if (rr_to_or == "metaumbrella") {
    raw_res <- .metaumbrella_rr_se_to_or(rr = rr, logrr_se = logrr_se, n_cases = n_cases, n_controls = n_controls)
    res <- cbind(
      logor = log(raw_res$value),
      logor_se = raw_res$se,
      logor_ci_lo = log(raw_res$value) - qnorm(.975) * raw_res$se,
      logor_ci_up = log(raw_res$value) + qnorm(.975) * raw_res$se
    )
    return(res)
  } else if (rr_to_or == "transpose") {
    res <- cbind(
      logor = log(rr),
      logor_se = logrr_se,
      logor_ci_lo = log(rr_ci_lo),
      logor_ci_up = log(rr_ci_up)
    )
    return(res)
  } else if (rr_to_or == "dipietrantonj") {
    n_dec <- max(
      nchar(gsub("^.+[.]", "", rr)),
      nchar(gsub("^.+[.]", "", rr_ci_lo)),
      nchar(gsub("^.+[.]", "", rr_ci_up))
    )

    estim <- estimraw::estim_raw(
      es = rr, lb = rr_ci_lo, ub = rr_ci_up,
      m1 = n_exp, m2 = n_nexp, dec = n_dec, measure = "rr"
    )
    if (length(estim) != 4) {
      numb = 1
      if (!is.na(baseline_risk)) {
        numb = which.min(
          c(
            abs(estim[[1]]$c[1] / (estim[[1]]$c[1] + estim[[1]]$d[1]) - baseline_risk),
            abs(estim[[2]]$c[1] / (estim[[2]]$c[1] + estim[[2]]$d[1]) - baseline_risk)
          )
        )
      }
      calc_dipie <- es_from_2x2(
        n_cases_exp = estim[[numb]]$a[1],
        n_controls_exp = estim[[numb]]$b[1],
        n_cases_nexp = estim[[numb]]$c[1],
        n_controls_nexp = estim[[numb]]$d[1]
      )
    } else {
      calc_dipie <- es_from_2x2(
        n_cases_exp = estim$a[1],
        n_controls_exp = estim$b[1],
        n_cases_nexp = estim$c[1],
        n_controls_nexp = estim$d[1]
      )
    }
    res <- cbind(
      logor = calc_dipie$logor,
      logor_se = calc_dipie$logor_se,
      logor_ci_lo = calc_dipie$logor_ci_lo,
      logor_ci_up = calc_dipie$logor_ci_up
    )

    return(res)
  } else {
    stop(paste0("'", rr_to_or, "' not in tolerated values for the 'rr_to_or' argument. Possible inputs are: 'metaumbrella', 'transpose', 'grant_2x2', 'grant_CI'"))
  }
}

.metaumbrella_rr_se_to_or <- function(rr, logrr, logrr_se, n_cases, n_controls) {
  es <- data.frame(value = NA, se = NA)

  if (!is.na(rr) & !is.na(logrr_se) & !is.na(n_cases) & !is.na(n_controls)) {
    # uncorrected
    n_cases_nexp_sim1 <- 0:n_cases
    n_controls_nexp_sim1 <- round(n_cases_nexp_sim1 * ((rr * (n_cases + n_controls)) / (n_cases + (rr - 1) * n_cases_nexp_sim1) - 1))
    n_cases_exp_sim1 <- n_cases - n_cases_nexp_sim1
    n_controls_exp_sim1 <- n_controls - n_controls_nexp_sim1

    # we take only positives (no-zero)
    idx_non_zero <- which(n_cases_nexp_sim1 > 0 & n_controls_nexp_sim1 > 0 & n_cases_exp_sim1 > 0 & n_controls_exp_sim1 > 0) # Posem ">" i no "!=" per treure els negatius!
    n_cases_nexp_sim1 <- n_cases_nexp_sim1[idx_non_zero]
    n_controls_nexp_sim1 <- n_controls_nexp_sim1[idx_non_zero]
    n_cases_exp_sim1 <- n_cases_exp_sim1[idx_non_zero]
    n_controls_exp_sim1 <- n_controls_exp_sim1[idx_non_zero]

    # corregint 0.5
    n_cases_nexp_sim2 <- 0:n_cases
    n_controls_nexp_sim2 <- ((n_cases + n_controls - n_cases_nexp_sim2 + 1) - (n_cases + n_controls + 2) * (n_cases - n_cases_nexp_sim2 + 0.5) / ((n_cases_nexp_sim2 + 0.5) * rr + n_cases - n_cases_nexp_sim2 + 0.5))
    n_cases_exp_sim2 <- n_cases - n_cases_nexp_sim2
    n_controls_exp_sim2 <- n_controls - n_controls_nexp_sim2

    # we take the ones with some 0 but non negative
    idx_some_zero <- which(
      (n_cases_nexp_sim2 == 0 | n_controls_nexp_sim2 == 0 | n_cases_exp_sim2 == 0 | n_controls_exp_sim2 == 0) &
        (n_cases_nexp_sim2 >= 0 & n_controls_nexp_sim2 >= 0 & n_cases_exp_sim2 >= 0 & n_controls_exp_sim2 >= 0)
    )
    n_cases_nexp_sim2 <- n_cases_nexp_sim2[idx_some_zero]
    n_controls_nexp_sim2 <- n_controls_nexp_sim2[idx_some_zero]
    n_cases_exp_sim2 <- n_cases_exp_sim2[idx_some_zero]
    n_controls_exp_sim2 <- n_controls_exp_sim2[idx_some_zero]

    #
    n_controls_exp_sim <- append(n_controls_exp_sim1, n_controls_exp_sim2)
    n_controls_nexp_sim <- append(n_controls_nexp_sim1, n_controls_nexp_sim2)
    n_cases_nexp_sim <- append(n_cases_nexp_sim1, n_cases_nexp_sim2)
    n_cases_exp_sim <- append(n_cases_exp_sim1, n_cases_exp_sim2)

    some_zero <- n_cases_exp_sim == 0 | n_controls_exp_sim == 0 | n_cases_nexp_sim == 0 | n_controls_nexp_sim == 0
    # var(log RR) = 1/a - 1/n1 + 1/c - 1/n2: the two ARM-TOTAL terms are SUBTRACTED
    # (delta method for log of a binomial proportion; identical to es_from_2x2()'s
    # se_rr and the Cochrane Handbook). This is NOT the log-OR pattern 1/a+1/b+1/c+1/d
    # (all added). Matching the candidate table to logrr_se^2 with the arm-total terms
    # ADDED selected the wrong table along the RR-constrained family and biased the
    # reconstructed OR (round-trip recovery of a known OR failed). Mapping of the sweep
    # variables: a = n_cases - n_cases_nexp_sim, c = n_cases_nexp_sim,
    # n1 (exposed total) = n_cases + n_controls - (n_cases_nexp_sim + n_controls_nexp_sim),
    # n2 (non-exposed total) = n_cases_nexp_sim + n_controls_nexp_sim.
    var_sim <- ifelse(some_zero,
      1 / ((n_cases + 1) - (n_cases_nexp_sim + 0.5)) - 1 / ((n_cases + 1) + (n_controls + 1) - ((n_cases_nexp_sim + 0.5) + (n_controls_nexp_sim + 0.5))) +
        1 / (n_cases_nexp_sim + 0.5) - 1 / ((n_cases_nexp_sim + 0.5) + (n_controls_nexp_sim + 0.5)),
      1 / (n_cases - n_cases_nexp_sim) - 1 / (n_cases + n_controls - (n_cases_nexp_sim + n_controls_nexp_sim)) +
        1 / n_cases_nexp_sim - 1 / (n_cases_nexp_sim + n_controls_nexp_sim)
    )

    best <- order((var_sim - logrr_se^2)^2)[1]
    n_cases_nexp <- n_cases_nexp_sim[best]
    n_controls_nexp <- n_controls_nexp_sim[best]
    n_cases_exp <- n_cases - n_cases_nexp
    n_controls_exp <- n_controls - n_controls_nexp

    # es$n_cases_nexp = n_cases_nexp
    # es$n_controls_nexp = n_controls_nexp
    # es$n_cases_exp = n_cases_exp
    # es$n_controls_exp = n_controls_exp

    cont_table <- es_from_2x2(
      n_cases_exp = n_cases_exp, n_controls_exp = n_controls_exp,
      n_cases_nexp = n_cases_nexp, n_controls_nexp = n_controls_nexp
    )

    es$value <- exp(cont_table$logor)
    es$se <- cont_table$logor_se
  }
  return(es)
}



################# 2x2 to R/Z ###################
.contingency_to_cor <- function(n_cases_exp, n_controls_exp, n_cases_nexp, n_controls_nexp,
                               table_2x2_to_cor, reverse_2x2) {

  if (table_2x2_to_cor == "lipsey") {
    # TO DO
    # log_or <- log((n_cases_exp * n_controls_nexp) / (n_cases_nexp * n_controls_exp))
    # v.log_or <- 1 / n_cases_exp + 1 / n_cases_nexp + 1 / n_controls_exp + 1 / n_controls_nexp
    #
    # r <- (n_cases_exp * n_controls_nexp - n_controls_exp * n_cases_nexp) /
    #   sqrt((n_cases_exp + n_controls_exp) * (n_cases_nexp + n_controls_nexp) *
    #     (n_cases_exp + n_cases_nexp) * (n_controls_exp + n_cases_nexp))
    # r_lipsey <- ifelse(reverse_2x2, -r, r)
    # z_lipsey <- atanh(r_lipsey)
    # vz_lipsey <- v.log_or * (z_lipsey^2) / (log_or^2)
    #
    # z_lo_lipsey <- z_lipsey - qnorm(.975) * sqrt(vz_lipsey)
    # z_up_lipsey <- z_lipsey + qnorm(.975) * sqrt(vz_lipsey)
    # r_lo_lipsey <- tanh(z_lo_lipsey)
    # r_up_lipsey <- tanh(z_up_lipsey)
    #
    # effective_n = 1/vz_lipsey + 3
    # vr_lipsey = (1 - r_lipsey^2)^2 / (effective_n - 1)
    #
    # res <- cbind(
    #   r_lipsey, vr_lipsey, r_lo_lipsey, r_up_lipsey,
    #   z_lipsey, vz_lipsey, z_lo_lipsey, z_up_lipsey
    # )
    # return(res)
  } else if (table_2x2_to_cor == "tetrachoric") {
    res <- .tet_r(as.numeric(n_cases_exp),
                  as.numeric(n_controls_exp),
                  as.numeric(n_cases_nexp),
                  as.numeric(n_controls_nexp))
    res[res == "calculation failure"] <- NA

    # res[] <- lapply(res, function(x) as.numeric(as.character(x)))
    res[1] <- ifelse(reverse_2x2, -res[1], res[1])
    res[3] <- ifelse(reverse_2x2, -res[3], res[3])
    res[4] <- ifelse(reverse_2x2, -res[4], res[4])
    res[5] <- ifelse(reverse_2x2, -res[5], res[5])
    res[7] <- ifelse(reverse_2x2, -res[7], res[7])
    res[8] <- ifelse(reverse_2x2, -res[8], res[8])

    return(res)
  } else {
    stop(paste0("'", table_2x2_to_cor, "' not in tolerated values for the 'table_2x2_to_cor' argument. Possible inputs are: 'tetrachoric', 'cooper_delta', 'cooper_std' or 'lipsey'"))
  }
}

.tet_r <- function(n_cases_exp, n_controls_exp, n_cases_nexp, n_controls_nexp) {
  tryCatch(
    expr = {
      tet <- metafor::escalc(
        ai = n_cases_exp, bi = n_controls_exp,
        ci = n_cases_nexp, di = n_controls_nexp,
        measure = "RTET"
      )
      n_sample = n_cases_exp + n_controls_exp + n_cases_nexp + n_controls_nexp
      r <- tet$yi
      vr <- tet$vi
      z <- atanh(r)
      vz <- vr / ((1 - r^2)^2)

      z_lo <- z - qnorm(.975) * sqrt(vz)
      z_up <- z + qnorm(.975) * sqrt(vz)
      r_lo <- r - qnorm(.975) * sqrt(vr)
      r_up <- r + qnorm(.975) * sqrt(vr)

      dat <- cbind(r, vr, r_lo, r_up, z, vz, z_lo, z_up)
      return(dat)
    },
    error = function(e) {
      dat <- cbind(NA, NA, NA, NA, NA, NA, NA, NA)
      return(dat)
    },
    warning = function(w) {
      dat <- cbind(NA, NA, NA, NA, NA, NA, NA, NA)
      return(dat)
    },
    finally = {
    }
  )
}

################# OR to R/Z ###################
.or_to_cor <- function(or, logor_se,
                       n_cases,
                       n_exp,
                       small_margin_prop,
                       n_sample,
                       or_to_cor) {
  if (or_to_cor %in% c("bonett", "pearson")) {
    if (or_to_cor == "bonett") {
      c <- (1 - abs(n_exp/n_sample - n_cases/n_sample) / 5 - (1 / 2 - small_margin_prop)^2) / 2
    } else {
      c <- 1 / 2
    }

    r <- cos(pi / (1 + or^c))
    r_se <- logor_se * (pi * c * or^c) * sin(pi / (1 + or^c)) / (1 + or^c)^2

    or_ci_lo <- exp(log(or) - qnorm(.975) * logor_se)
    or_ci_up <- exp(log(or) + qnorm(.975) * logor_se)
    r_lo <- cos(pi / (1 + or_ci_lo^c))
    r_up <- cos(pi / (1 + or_ci_up^c))

    z <- atanh(r)
    z_se <- sqrt(r_se^2 / ((1 - r^2)^2)) # delta method
    z_lo <- atanh(r_lo)
    z_up <- atanh(r_up)

    res <- cbind(r, r_se, r_lo, r_up, z, z_se, z_lo, z_up)
    return(res)
  } else if (or_to_cor == "digby") {
    c <- 3 / 4

    r <- (or^c - 1) / (or^c + 1)
    r_se <- sqrt((c^2 / 4) * (1 - r^2)^2 * logor_se^2)

    z <- atanh(r)
    z_se <- sqrt(r_se^2 / ((1 - r^2)^2)) # delta method
    z_lo <- z - qnorm(.975) * sqrt(c^2 / 4 * logor_se^2)
    z_up <- z + qnorm(.975) * sqrt(c^2 / 4 * logor_se^2)

    r_lo <- tanh(z_lo)
    r_up <- tanh(z_up)

    res <- cbind(r, r_se, r_lo, r_up, z, z_se, z_lo, z_up)
    return(res)
  }
}

################# SMD to R/Z ###################
.smd_to_cor <- function(d, vd, n_exp, n_nexp, smd_to_cor, n_cov_ancova) {
  if (smd_to_cor == "viechtbauer") {
    df <- n_exp + n_nexp - 2 - n_cov_ancova
    h <- df / n_exp + df / n_nexp
    p <- n_exp / (n_exp + n_nexp)
    q <- n_nexp / (n_exp + n_nexp)
    r_pb <- d / sqrt(d^2 + h)

    f <- dnorm(qnorm(p, lower.tail = FALSE))
    r_viechtbauer <- sqrt(p * q) / f * r_pb
    r_trunc = ifelse(r_viechtbauer > 1, 1, ifelse(r_viechtbauer < -1, -1, r_viechtbauer))
    vr_viechtbauer <- 1 / (n_exp + n_nexp - 1) *
      (p * q / f^2 - (3 / 2 + (1 - p * qnorm(p, lower.tail = FALSE) / f) *
                              (1 + q * qnorm(p, lower.tail = FALSE) / f)) *
         r_trunc^2 + r_trunc^4)


    # ========= z ========= #
    fzp <- dnorm(qnorm(p))
    a_viechtbauer <- sqrt(fzp) / (p * (1 - p))^(1 / 4)
    z_viechtbauer <- (a_viechtbauer / 2) * log((1 + a_viechtbauer * r_trunc) /
                                               (1 - a_viechtbauer * r_trunc))
    vz_viechtbauer <- 1 / (n_exp + n_nexp - 1)
    # ========= 95% CI ===== #


    z_lo_viechtbauer <- z_viechtbauer - qnorm(.975) * sqrt(vz_viechtbauer)
    z_up_viechtbauer <- z_viechtbauer + qnorm(.975) * sqrt(vz_viechtbauer)
    r_lo_viechtbauer <- (1 / a_viechtbauer) * ((exp(2 * z_lo_viechtbauer / a_viechtbauer) - 1) / (exp(2 * z_lo_viechtbauer / a_viechtbauer) + 1))
    r_up_viechtbauer <- (1 / a_viechtbauer) * ((exp(2 * z_up_viechtbauer / a_viechtbauer) - 1) / (exp(2 * z_up_viechtbauer / a_viechtbauer) + 1))

    res <- cbind(
      r_viechtbauer, vr_viechtbauer, r_lo_viechtbauer, r_up_viechtbauer,
      z_viechtbauer, vz_viechtbauer, z_lo_viechtbauer, z_up_viechtbauer
    )

    return(res)
  } else if (smd_to_cor == "lipsey_cooper") {
    a <- ((n_exp + n_nexp)^2) / (n_exp * n_nexp)
    p <- n_exp / (n_exp + n_nexp)
    r_lipsey <- d / sqrt(d^2 + 1 / (p * (1 - p)))
    vr_lipsey <- a^2 * vd / ((d^2 + a)^3)
    z_lipsey <- atanh(r_lipsey)
    # delta method for z = atanh(r), r = d / sqrt(d^2 + a) with a = 1/(p*(1-p)):
    # dz/dd = 1/sqrt(d^2 + a), so Var(z) = vd / (d^2 + a). The additive denominator
    # term is the SQUARED point estimate d^2, NOT the sampling variance vd. This makes
    # vz_lipsey the EXACT Fisher transform of vr_lipsey above (vz = vr/(1-r^2)^2).
    # NB: esc::convert_d2r() returns vd/(vd + a) here, which is NOT Fisher-consistent
    # with its own r variance; metaConvert intentionally uses the consistent value, so
    # the lipsey_cooper z-SE differs from esc for large |d| (they agree as d -> 0).
    vz_lipsey <- vd / (d^2 + 1 / (p * (1 - p)))
    r_lo_lipsey <- r_lipsey - qt(.975, df = n_exp + n_nexp - 2) * sqrt(vr_lipsey)
    r_up_lipsey <- r_lipsey + qt(.975, df = n_exp + n_nexp - 2) * sqrt(vr_lipsey)
    z_lo_lipsey <- z_lipsey - qnorm(.975) * sqrt(vz_lipsey)
    z_up_lipsey <- z_lipsey + qnorm(.975) * sqrt(vz_lipsey)

    res <- cbind(
      r_lipsey, vr_lipsey, r_lo_lipsey, r_up_lipsey,
      z_lipsey, vz_lipsey, z_lo_lipsey, z_up_lipsey
    )

    return(res)
  }
}

################# PHI to R/Z ###################
.phi_to_cor <- function(phi, n_sample,
                        n_cases_exp, n_controls_exp, n_cases_nexp, n_controls_nexp,
                        phi_to_cor, reverse_phi) {
  if (phi_to_cor == "lipsey") {
    r <- ifelse(reverse_phi, -phi, phi)
    z <- atanh(r)
    vz <- z^2 / (r^2 * n_sample)
    z_lo <- z - qnorm(.975) * sqrt(vz)
    z_up <- z + qnorm(.975) * sqrt(vz)
    effective_n = 1 / vz + 3
    vr = (1 - r^2)^2 / (effective_n - 1)
    # t <- qt(pnorm(z / sqrt(vz)), n_sample - 2)
    # vr <- (r / t)^2
    # r_lo = r - qnorm(.975)*sqrt(vr)
    # r_up = r + qnorm(.975)*sqrt(vr)
    r_lo <- tanh(z_lo)
    r_up <- tanh(z_up)
    res <- cbind(r, vr, r_lo, r_up, z, vz, z_lo, z_up)
    return(res)
  } else if (phi_to_cor == "tetrachoric") {
    # if (!requireNamespace("mvtnorm", quietly = TRUE) & !requireNamespace("metafor", quietly = TRUE)) {
    #   stop("Please install the 'mvtnorm' and 'metafor' packages to compute tetrachoric correlation.")
    # } else if (!requireNamespace("metafor", quietly = TRUE)) {
    #   stop("Please install the 'metafor' package to compute tetrachoric correlation.")
    # } else if (!requireNamespace("mvtnorm", quietly = TRUE)) {
    #   stop("Please install the 'mvtnorm' package to compute tetrachoric correlation.")
    # }

    res <- .tet_r(
      as.numeric(n_cases_exp), as.numeric(n_controls_exp),
      as.numeric(n_cases_nexp), as.numeric(n_controls_nexp)
    )
    res[res == "calculation failure"] <- NA

    res[1] <- ifelse(reverse_phi, -res[1], res[1])
    res[3] <- ifelse(reverse_phi, -res[3], res[3])
    res[4] <- ifelse(reverse_phi, -res[4], res[4])
    res[5] <- ifelse(reverse_phi, -res[5], res[5])
    res[7] <- ifelse(reverse_phi, -res[7], res[7])
    res[8] <- ifelse(reverse_phi, -res[8], res[8])

    return(res)
  } else {
    stop(paste0("'", phi_to_cor, "' not in tolerated values for the 'phi_to_cor' argument. Possible inputs are: 'tetrachoric', or 'lipsey'"))
  }
}

################# CHI to R/Z ###################
.chi_to_cor <- function(chisq, n_sample,
                        n_cases_exp, n_controls_exp, n_cases_nexp, n_controls_nexp,
                        chisq_to_cor, reverse_chisq) {
  if (chisq_to_cor == "lipsey") {
    r <- sqrt(chisq / n_sample)
    r <- ifelse(reverse_chisq, -r, r)
    z <- atanh(r)
    vz <- z^2 / (chisq)
    z_lo <- z - qnorm(.975) * sqrt(vz)
    z_up <- z + qnorm(.975) * sqrt(vz)
    # t <- qt(pnorm(z / sqrt(vz)), n_sample - 2)
    # vr <- (r / t)^2
    effective_n = 1 / vz + 3
    vr = (1 - r^2)^2 / (effective_n - 1)
    r_lo <- tanh(z_lo)
    r_up <- tanh(z_up)
    # r_lo = r - qt(.975, n_sample - 2)*sqrt(vr)
    # r_up = r + qt(.975, n_sample - 2)*sqrt(vr)
    # r_lo <- tanh(z_lo)
    # r_up <- tanh(z_up)
    res <- cbind(r, vr, r_lo, r_up, z, vz, z_lo, z_up)
    return(res)
  } else if (chisq_to_cor == "tetrachoric") {
    # if (!requireNamespace("mvtnorm", quietly = TRUE) & !requireNamespace("metafor", quietly = TRUE)) {
    #   stop("Please install the 'mvtnorm' and 'metafor' packages to compute tetrachoric correlation.")
    # } else if (!requireNamespace("metafor", quietly = TRUE)) {
    #   stop("Please install the 'metafor' package to compute tetrachoric correlation.")
    # } else if (!requireNamespace("mvtnorm", quietly = TRUE)) {
    #   stop("Please install the 'mvtnorm' package to compute tetrachoric correlation.")
    # }

    res <- .tet_r(
      as.numeric(n_cases_exp), as.numeric(n_controls_exp),
      as.numeric(n_cases_nexp), as.numeric(n_controls_nexp)
    )
    res[res == "calculation failure"] <- NA

    res[1] <- ifelse(reverse_chisq, -res[1], res[1])
    res[3] <- ifelse(reverse_chisq, -res[3], res[3])
    res[4] <- ifelse(reverse_chisq, -res[4], res[4])
    res[5] <- ifelse(reverse_chisq, -res[5], res[5])
    res[7] <- ifelse(reverse_chisq, -res[7], res[7])
    res[8] <- ifelse(reverse_chisq, -res[8], res[8])

    return(res)
  } else {
    stop(paste0("'", chisq_to_cor, "' not in tolerated values for the 'chisq_to_cor' argument. Possible inputs are: 'tetrachoric', or 'lipsey'"))
  }
}


################# PRE POST helpers ##############
#' Guard a standardizing SD before it is used as a denominator
#'
#' Returns NA for any standardizer that is zero, negative or non-finite, so the
#' resulting SMD is NA rather than a silent Inf/NaN. Scoped deliberately to the
#' DENOMINATOR: the mean-change wrappers legitimately pass mean_pre_sd = 0 (the
#' pre slot is zeroed by construction), so guarding raw inputs would break them,
#' whereas a zero *standardizer* is always degenerate. Reachable when a study
#' reports SD = 0, or when r_pre_post = 1 with equal pre/post SDs makes the
#' change SD collapse to 0.
#'
#' @noRd
.guard_standardizer <- function(sd_value) {
  ifelse(is.finite(sd_value) & sd_value > 0, sd_value, NA_real_)
}

#' Guard a pre-post correlation
#'
#' |r| >= 1 is not a valid correlation and drives sd_change to 0 (or a negative
#' radicand) in every pre-post formula. Strictly at the bounds so that r = 0.99
#' still computes.
#'
#' @noRd
.guard_r_pre_post <- function(r) {
  ifelse(is.finite(r) & abs(r) < 1, r, NA_real_)
}

################# PRE POST to SMD ##############
.pre_post_to_smd <- function(mean_pre_exp, mean_pre_sd_exp,
                             mean_exp, mean_sd_exp,
                             mean_pre_nexp, mean_pre_sd_nexp,
                             mean_nexp, mean_sd_nexp,
                             n_exp, n_nexp,
                             r_pre_post_exp, r_pre_post_nexp,
                             pre_post_to_smd,
                             pool_sd = FALSE) {
  # pool_sd = TRUE: standardizing SD pooled across groups (Morris 2008, eq. 9/13)
  if (pool_sd) {
    return(.pooled_pre_post_to_smd(
      mean_pre_exp = mean_pre_exp, mean_pre_sd_exp = mean_pre_sd_exp,
      mean_exp = mean_exp, mean_sd_exp = mean_sd_exp,
      mean_pre_nexp = mean_pre_nexp, mean_pre_sd_nexp = mean_pre_sd_nexp,
      mean_nexp = mean_nexp, mean_sd_nexp = mean_sd_nexp,
      n_exp = n_exp, n_nexp = n_nexp,
      r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,
      pre_post_to_smd = pre_post_to_smd
    ))
  }

  res_exp <- .single_group_pre_post_to_smd(
    mean_pre = mean_pre_exp, mean_post = mean_exp,
    mean_pre_sd = mean_pre_sd_exp, mean_post_sd = mean_sd_exp,
    n = n_exp, r_pre_post = r_pre_post_exp,
    pre_post_to_smd = pre_post_to_smd
  )

  res_nexp <- .single_group_pre_post_to_smd(
    mean_pre = mean_pre_nexp, mean_post = mean_nexp,
    mean_pre_sd = mean_pre_sd_nexp, mean_post_sd = mean_sd_nexp,
    n = n_nexp, r_pre_post = r_pre_post_nexp,
    pre_post_to_smd = pre_post_to_smd
  )

  d_final <- res_exp[,"d"] - res_nexp[,"d"]
  g_final <- res_exp[,"g"] - res_nexp[,"g"]
  vd_final <- res_exp[,"var_d"] + res_nexp[,"var_d"]
  vg_final <- res_exp[,"var_g"] + res_nexp[,"var_g"]

  d_ci_lo <- d_final - sqrt(vd_final) * qt(.975, n_exp + n_nexp - 2)
  d_ci_up <- d_final + sqrt(vd_final) * qt(.975, n_exp + n_nexp - 2)
  g_ci_lo <- g_final - sqrt(vg_final) * qt(.975, n_exp + n_nexp - 2)
  g_ci_up <- g_final + sqrt(vg_final) * qt(.975, n_exp + n_nexp - 2)

  res <- cbind(
    d_final, vd_final, d_ci_lo, d_ci_up,
    g_final, vg_final, g_ci_lo, g_ci_up
  )

  return(res)
}

################# POOLED TWO-GROUP PRE POST to SMD ##############
#' Between-group pre/post SMD, standardizing SD pooled across arms
#'
#' Point estimates are Morris (2008): the numerator is the difference in mean change
#' between arms; the standardizer is pooled across arms (baseline SD for bonett /
#' d_ppc2 eq. 8-9; change SD for d_z/d_rm; average SD for d_av / d_ppc3 eq. 12-14).
#'
#' Each branch's variance follows metafor's LS *pattern* for the corresponding measure
#' -- an empirical leading term plus a g^2 term -- but note this is a pattern, not a
#' single closed-form rule: metafor puts n (or n1+n2), NOT the standardizer df, in the
#' g^2 denominator of its non-heteroscedastic LS forms, while evaluating J at the
#' standardizer df; the two df deliberately differ. So the g^2 term below is over 2N,
#' and J is J(nu) with nu the standardizer df. Var(d) = Var(g)/J^2 exactly, since
#' g = J*d with J a deterministic constant.
#'
#' The leading term is heteroscedasticity-robust: it is built from the EMPIRICAL pooled
#' change SD, not from Var(change) = 2*sigma^2*(1-r) (valid only when SD_pre = SD_post).
#' For bonett this reduces EXACTLY to Viechtbauer's published two-group pre/post
#' variance vi = 2(1-r)(1/nT + 1/nC) + g^2/(2N) when SD_pre = SD_post within each arm
#' (the df-weighted r_avg makes the reduction exact even when n1 != n2); under
#' heteroscedasticity it departs from that homoscedastic form, which understates the
#' variance (its coverage falls to ~0.86 at SD_pre/SD_post ~ 0.64). d_av uses the
#' two-group fourth-moment form Bonett (2008) eq. 19, metafor's SMCRPH carried across
#' two independent arms.
#'
#' Publication status of each pooled variance:
#'   - bonett: reduces to Viechtbauer's published vi (metafor-project Morris-2008 page)
#'             under homoscedasticity; the robust departure is metafor SMCRH / Bonett
#'             (2008) generalized across arms.
#'   - d_z:    exactly metafor::escalc(measure = "SMD", vtype = "LS") on the change
#'             scores (Hedges 1981) -- a published two-group variance.
#'   - d_rm:   d_rm = d_z * sqrt(2(1-r)) (Caldwell & Vigotsky 2020 eq. 13, a definition),
#'             so Var(d_rm) = 2(1-r)*Var(d_z) for known r.
#'   - d_av:   Bonett (2008) eq. 19 (two-group mixed design, all-four-SD standardizer).
#' The two pooled forms without a directly published two-group source (the robust
#' bonett departure, and d_rm's algebra) are Monte-Carlo calibrated in
#' tests/testthat/test-pooled-variance-calibration.R.
#'
#' CIs use qt(.975, m) with m = N - 2 the pooled standardizer df, for EVERY branch --
#' including d_av, whose Cousineau effective df nu = 2m/(1 + r2_avg) feeds ONLY the bias
#' correction J, not the interval (the single-group kernel follows the same split: its d_av
#' CI is on n - 1, not on its Cousineau df). qt is a small-sample-conservative choice (it
#' slightly over-covers at n ~ 10); metafor and Bonett (2008) use a normal (z) critical value.
#'
#' References:
#'   Morris (2008) ORM 11(2):364-386 -- d_ppc1/d_ppc2/d_ppc3 point estimates (eq. 6-14)
#'   Hedges (1981); Viechtbauer (2007) JEBS 32(1):39-60 -- two-sample SMD "LS"
#'   Bonett (2008) Psych Methods 13(2):99-109 eq. 10/19 -- heteroscedasticity-robust var
#'   Caldwell & Vigotsky (2020) PeerJ 8:e10314 eq. 13 -- d_rm = d_z * sqrt(2(1-r))
#'   Cousineau (2020) TQMP 16(4):418-421 eq. 2 -- the (1+r^2) effective df (d_av J only)
#'
#' @noRd
.pooled_pre_post_to_smd <- function(mean_pre_exp, mean_pre_sd_exp,
                                     mean_exp, mean_sd_exp,
                                     mean_pre_nexp, mean_pre_sd_nexp,
                                     mean_nexp, mean_sd_nexp,
                                     n_exp, n_nexp,
                                     r_pre_post_exp, r_pre_post_nexp,
                                     pre_post_to_smd) {
  if (pre_post_to_smd == "cooper") {
    pre_post_to_smd <- "morris_drm"
  }

  r_pre_post_exp <- .guard_r_pre_post(r_pre_post_exp)
  r_pre_post_nexp <- .guard_r_pre_post(r_pre_post_nexp)

  # A negative SD is invalid input (see .single_group_pre_post_to_smd): NA it out so a
  # negative raw SD cannot flip the sign of the sd_change cross term and survive.
  mean_pre_sd_exp  <- ifelse(is.finite(mean_pre_sd_exp)  & mean_pre_sd_exp  >= 0, mean_pre_sd_exp,  NA_real_)
  mean_sd_exp      <- ifelse(is.finite(mean_sd_exp)      & mean_sd_exp      >= 0, mean_sd_exp,      NA_real_)
  mean_pre_sd_nexp <- ifelse(is.finite(mean_pre_sd_nexp) & mean_pre_sd_nexp >= 0, mean_pre_sd_nexp, NA_real_)
  mean_sd_nexp     <- ifelse(is.finite(mean_sd_nexp)     & mean_sd_nexp     >= 0, mean_sd_nexp,     NA_real_)

  N <- n_exp + n_nexp
  m <- N - 2 # pooled degrees of freedom
  # Each arm needs >= 2 observations: the (n-1) pooling weights must be positive and
  # the between-arm factor N/(n1*n2) must be finite. m > 0 alone does not ensure this
  # (e.g. n_exp = 0, n_nexp = 30 gives m = 28 but a negative weight and T_change = Inf).
  valid_n <- is.finite(n_exp) & is.finite(n_nexp) & n_exp >= 2 & n_nexp >= 2
  m <- ifelse(is.finite(m) & m > 0 & valid_n, m, NA_real_)

  change_exp <- mean_exp - mean_pre_exp
  change_nexp <- mean_nexp - mean_pre_nexp
  mean_diff <- change_exp - change_nexp

  # df-weighted mean of the per-arm correlations. df-weighting (not n-weighting) is
  # what makes the robust bonett/dz variances reduce EXACTLY to Viechtbauer's published
  # vi = 2(1-r)(1/nT + 1/nC) + g^2/(2N) under homoscedasticity when the arms' SDs are
  # equal but n1 != n2 (an n-weighted mean leaves a residual there).
  r_avg <- ((n_exp - 1) * r_pre_post_exp + (n_nexp - 1) * r_pre_post_nexp) / m

  # Pooled change SD: the empirical scale of the numerator. Used by every branch,
  # so that no branch assumes SD_pre = SD_post.
  sd_change_exp <- sqrt(mean_pre_sd_exp^2 + mean_sd_exp^2 -
                        2 * r_pre_post_exp * mean_pre_sd_exp * mean_sd_exp)
  sd_change_nexp <- sqrt(mean_pre_sd_nexp^2 + mean_sd_nexp^2 -
                         2 * r_pre_post_nexp * mean_pre_sd_nexp * mean_sd_nexp)
  sd_change_pooled <- sqrt(((n_exp - 1) * sd_change_exp^2 +
                            (n_nexp - 1) * sd_change_nexp^2) / m)

  # Var(mean_diff) on the change scale, in units of sd_change_pooled^2
  T_change <- N / (n_exp * n_nexp)

  if (pre_post_to_smd == "bonett") {
    # Morris (2008) d_ppc2: numerator = difference in mean change, standardizer =
    # BASELINE SD pooled across arms (eq. 8-9). Variance is the SMCRH form: the
    # numerator's empirical variance expressed in standardizer units.
    sd_pooled <- .guard_standardizer(sqrt(((n_exp - 1) * mean_pre_sd_exp^2 +
                                           (n_nexp - 1) * mean_pre_sd_nexp^2) / m))
    nu <- m
    J <- .d_j(nu)
    d <- mean_diff / sd_pooled
    g <- d * J

    T1 <- (sd_change_pooled^2 / sd_pooled^2) * T_change
    var_g <- T1 + g^2 / (2 * N)
    # g = J*d with J a deterministic constant, so Var(d) = Var(g)/J^2 exactly.
    var_d <- var_g / J^2

  } else if (pre_post_to_smd == "morris_dz") {
    # Change-score metric. Once the change SD is pooled across arms this is exactly
    # an independent-groups Hedges g computed on the change scores, so its variance
    # is metafor::escalc(measure = "SMD", vtype = "LS") -- no r, no 2(1-r) term.
    sd_pooled <- .guard_standardizer(sd_change_pooled)
    nu <- m
    J <- .d_j(nu)
    d <- mean_diff / sd_pooled
    g <- d * J

    var_g <- T_change + g^2 / (2 * N)
    var_d <- var_g / J^2

  } else if (pre_post_to_smd == "morris_drm") {
    # Raw-score metric: d_rm = d_z * sqrt(2(1-r)) (Caldwell & Vigotsky 2020 eq. 13).
    # r is a known constant, so Var(d_rm) = 2(1-r) * Var(d_z): the 2(1-r) here is a
    # deterministic rescaling, NOT a homoscedasticity assumption.
    sd_pooled <- .guard_standardizer(sd_change_pooled)
    nu <- m
    J <- .d_j(nu)
    k <- sqrt(2 * (1 - r_avg))
    d <- (mean_diff / sd_pooled) * k
    g <- d * J

    T1 <- 2 * (1 - r_avg) * T_change
    var_g <- T1 + g^2 / (2 * N)
    var_d <- var_g / J^2

  } else if (pre_post_to_smd == "morris_dav") {
    # Morris (2008) d_ppc3: standardizer = quadratic mean of the pre and post SDs,
    # pooled across arms (eq. 12-13). Variance is the two-group heteroscedasticity-
    # robust form Bonett (2008) eq. 19 (the mixed-design "difference-in-differences"
    # standardized by all four cell SDs) == metafor SMCRPH carried across two
    # independent arms: a robust change-SD leading term plus a per-arm fourth-moment
    # g^2 term. This is NOT the homoscedastic SMCRP coefficient (1+r^2)/(4N), which is
    # only its SD_pre = SD_post special case.
    sd_av_exp <- sqrt((mean_pre_sd_exp^2 + mean_sd_exp^2) / 2)
    sd_av_nexp <- sqrt((mean_pre_sd_nexp^2 + mean_sd_nexp^2) / 2)
    sd_pooled <- .guard_standardizer(sqrt(((n_exp - 1) * sd_av_exp^2 +
                                           (n_nexp - 1) * sd_av_nexp^2) / m))
    # nu is used ONLY for the bias correction J. The (1+r^2) effective df is
    # homoscedastic (Cousineau 2020 eq. 2), so we feed it the df-weighted mean of r^2
    # (its two-arm generalization); the variance itself uses the fourth moments below.
    r2_avg <- ((n_exp - 1) * r_pre_post_exp^2 + (n_nexp - 1) * r_pre_post_nexp^2) / m
    nu <- 2 * m / (1 + r2_avg)
    J <- .d_j(nu)
    d <- mean_diff / sd_pooled
    g <- d * J

    T1 <- (sd_change_pooled^2 / sd_pooled^2) * T_change
    # Bonett (2008) eq. 19 fourth-moment g^2 coefficient (sum of per-arm terms over
    # 32*sd_pooled^4). Reduces to (1+r^2)/(4N)'s n-1 analogue when SD_pre = SD_post.
    fm_exp <- mean_pre_sd_exp^4 + mean_sd_exp^4 +
              2 * r_pre_post_exp^2 * mean_pre_sd_exp^2 * mean_sd_exp^2
    fm_nexp <- mean_pre_sd_nexp^4 + mean_sd_nexp^4 +
               2 * r_pre_post_nexp^2 * mean_pre_sd_nexp^2 * mean_sd_nexp^2
    g2_coef <- (fm_exp / (n_exp - 1) + fm_nexp / (n_nexp - 1)) / (32 * sd_pooled^4)
    var_g <- T1 + g^2 * g2_coef
    var_d <- var_g / J^2
  }

  # CI on the pooled standardizer df (N - 2 = m) for EVERY branch. dav's Cousineau
  # effective df nu = 2m/(1 + r2_avg) feeds ONLY the bias correction J (as in the
  # single-group kernel, whose dav CI is likewise on n - 1, not on its Cousineau df) --
  # the variance is built on m, so the interval is too. For bonett/dz/drm nu == m, so
  # this changes nothing there; it only homogenises the dav path.
  d_ci_lo <- d - sqrt(var_d) * qt(.975, m)
  d_ci_up <- d + sqrt(var_d) * qt(.975, m)
  g_ci_lo <- g - sqrt(var_g) * qt(.975, m)
  g_ci_up <- g + sqrt(var_g) * qt(.975, m)

  res <- cbind(
    d, var_d, d_ci_lo, d_ci_up,
    g, var_g, g_ci_lo, g_ci_up
  )

  return(res)
}

################# SINGLE GROUP PRE POST to SMD ##############
#' Calculate within-group standardized mean difference for a single group
#'
#' @param mean_pre mean at baseline (pre-test)
#' @param mean_post mean at follow-up (post-test)
#' @param mean_pre_sd standard deviation at baseline
#' @param mean_post_sd standard deviation at follow-up
#' @param n sample size
#' @param r_pre_post pre-post correlation
#' @param pre_post_to_smd method to use: "bonett" or "cooper"
#'
#' @return matrix with columns: d, var_d, d_ci_lo, d_ci_up, g, var_g, g_ci_lo, g_ci_up
#'
#' @noRd
.single_group_pre_post_to_smd <- function(mean_pre, mean_post,
                                           mean_pre_sd, mean_post_sd,
                                           n, r_pre_post,
                                           pre_post_to_smd) {
  # "cooper" alias kept for direct internal calls
  if (pre_post_to_smd == "cooper") {
    pre_post_to_smd <- "morris_drm"
  }

  r_pre_post <- .guard_r_pre_post(r_pre_post)

  # A negative SD is invalid input. NA it out so the ES is NA rather than a silently
  # corrupted one: a negative raw SD flips the sign of the -2*r*sd_pre*sd_post cross
  # term in sd_diff, yielding a wrong-but-finite standardizer that the .guard_standardizer
  # on the assembled denominator cannot catch. Zero is allowed: the mean-change wrappers
  # legitimately pass mean_pre_sd = 0 (the pre slot is zeroed by construction).
  mean_pre_sd  <- ifelse(is.finite(mean_pre_sd)  & mean_pre_sd  >= 0, mean_pre_sd,  NA_real_)
  mean_post_sd <- ifelse(is.finite(mean_post_sd) & mean_post_sd >= 0, mean_post_sd, NA_real_)

  # An arm needs n >= 2 for a variance/CI to exist. NA the standardizing SDs below 2 so
  # d, var and CI are ALL NA. morris_drm's var_d = 2*(1-r)/n + d^2/(2n) carries no J, so
  # the J(n-1) = NA path that silently nulls the other three branches leaves it finite at
  # n = 1 (and Inf at n = 0); nulling the SDs here closes that leak and matches the
  # pooled kernel's explicit n >= 2 guard.
  bad_n <- !(is.finite(n) & n >= 2)
  mean_pre_sd  <- ifelse(bad_n, NA_real_, mean_pre_sd)
  mean_post_sd <- ifelse(bad_n, NA_real_, mean_post_sd)

  if (pre_post_to_smd == "bonett") {
    # Bonett method: standardize by baseline SD
    # Matches metafor SMCRH (heteroscedastic-robust variance formula)
    J <- .d_j(n - 1)

    var_change <- mean_pre_sd^2 + mean_post_sd^2 - 2 * r_pre_post * mean_pre_sd * mean_post_sd

    sd_std <- .guard_standardizer(mean_pre_sd)
    d <- (mean_post - mean_pre) / sd_std
    g <- d * J

    # metafor SMCRH heteroscedastic variance formula (Bonett 2008)
    # var = sd_change^2 / (sd1i^2 * (n-1)) + g^2 / (2 * (n-1))
    var_g <- var_change / (sd_std^2 * (n - 1)) + g^2 / (2 * (n - 1))
    var_d <- var_g / (J^2)

    d_ci_lo <- d - sqrt(var_d) * qt(.975, n - 1)
    d_ci_up <- d + sqrt(var_d) * qt(.975, n - 1)
    g_ci_lo <- g - sqrt(var_g) * qt(.975, n - 1)
    g_ci_up <- g + sqrt(var_g) * qt(.975, n - 1)

    res <- cbind(
      d = d, var_d = var_d, d_ci_lo = d_ci_lo, d_ci_up = d_ci_up,
      g = g, var_g = var_g, g_ci_lo = g_ci_lo, g_ci_up = g_ci_up
    )

    return(res)
  } else if (pre_post_to_smd == "morris_drm") {
    # Morris d_rm (alias "cooper"): d_rm = d_z * sqrt(2(1-r)) (Caldwell & Vigotsky 2020),
    # i.e. the change SD rescaled onto the raw-score metric. This equals metafor's SMCR
    # (baseline/raw-SD standardizer) ONLY under homoscedasticity (SD_pre = SD_post); when
    # they differ the two diverge, so d_rm is defined by the Caldwell rescaling here, not
    # by SMCR. Its variance below is 2(1-r) * Var(d_z), the exact scaling of the SMCC form.
    # nb: the mean_change wrappers pass sd_change through mean_post_sd
    # (mean_pre = 0 and mean_pre_sd = 0, so sd_diff reduces to sd_change)
    J <- .d_j(n - 1)

    sd_diff <- .guard_standardizer(sqrt(mean_pre_sd^2 + mean_post_sd^2 -
                    (2 * r_pre_post * mean_pre_sd * mean_post_sd)))

    d <- (mean_post - mean_pre) / sd_diff * sqrt(2 * (1 - r_pre_post))
    g <- d * J

    # Viechtbauer corrected formula
    var_d <- 2 * (1 - r_pre_post) / n + d^2 / (2 * n)
    var_g <- J^2 * var_d

    d_ci_lo <- d - sqrt(var_d) * qt(.975, n - 1)
    d_ci_up <- d + sqrt(var_d) * qt(.975, n - 1)
    g_ci_lo <- g - sqrt(var_g) * qt(.975, n - 1)
    g_ci_up <- g + sqrt(var_g) * qt(.975, n - 1)

    res <- cbind(
      d = d, var_d = var_d, d_ci_lo = d_ci_lo, d_ci_up = d_ci_up,
      g = g, var_g = var_g, g_ci_lo = g_ci_lo, g_ci_up = g_ci_up
    )

    return(res)
  } else if (pre_post_to_smd == "morris_dz") {
    # Morris & DeShon d_z: standardize by change score SD
    # Matches metafor SMCC (change score standardization)
    # Most conservative when r is high
    J <- .d_j(n - 1)

    sd_diff <- .guard_standardizer(sqrt(mean_pre_sd^2 + mean_post_sd^2 -
                    2 * r_pre_post * mean_pre_sd * mean_post_sd))

    d <- (mean_post - mean_pre) / sd_diff
    g <- d * J

    # metafor SMCC variance formula (uses corrected g, not d)
    # var = 1/n + g^2/(2n)
    var_g <- 1 / n + g^2 / (2 * n)
    var_d <- var_g / (J^2)

    d_ci_lo <- d - sqrt(var_d) * qt(.975, n - 1)
    d_ci_up <- d + sqrt(var_d) * qt(.975, n - 1)
    g_ci_lo <- g - sqrt(var_g) * qt(.975, n - 1)
    g_ci_up <- g + sqrt(var_g) * qt(.975, n - 1)

    res <- cbind(
      d = d, var_d = var_d, d_ci_lo = d_ci_lo, d_ci_up = d_ci_up,
      g = g, var_g = var_g, g_ci_lo = g_ci_lo, g_ci_up = g_ci_up
    )

    return(res)
  } else if (pre_post_to_smd == "morris_dav") {
    # Morris (2008) d_av (d_ppc3): standardize by the quadratic mean of the pre and
    # post SDs. Its sampling variance is metafor SMCRPH == Bonett (2008) eq. 10: a
    # heteroscedasticity-robust change-SD leading term PLUS a fourth-moment g^2 term.
    # This is NOT the homoscedastic SMCRP form 2(1-r)/n + g^2(1+r^2)/(4n), which
    # assumes SD_pre = SD_post and understates var(g) when they differ (its CI
    # coverage falls to ~0.93 as SD_pre/SD_post moves away from 1). Morris (2008,
    # p.384) recommended AGAINST d_av precisely because its variance was unknown to
    # him; Bonett (2008) supplies it and this branch uses it.
    # nu = 2(n-1)/(1+r^2) is used ONLY for the Hedges bias correction J (Cousineau
    # 2020 eq. 2; metafor SMCRP/SMCRPH); the variance itself is on df = n-1.
    mi <- 2 * (n - 1) / (1 + r_pre_post^2)
    J <- .d_j(mi)

    sd_av <- .guard_standardizer(sqrt((mean_pre_sd^2 + mean_post_sd^2) / 2))

    d <- (mean_post - mean_pre) / sd_av
    g <- d * J

    # metafor SMCRPH "LS" == Bonett (2008) eq. 10 (verified: reproduces Bonett's
    # worked Example 2 -- n=60, var=0.0148 -- to the printed digit).
    sd_diff2 <- mean_pre_sd^2 + mean_post_sd^2 - 2 * r_pre_post * mean_pre_sd * mean_post_sd
    fm <- mean_pre_sd^4 + mean_post_sd^4 + 2 * r_pre_post^2 * mean_pre_sd^2 * mean_post_sd^2
    var_g <- sd_diff2 / (sd_av^2 * (n - 1)) + g^2 * fm / (8 * sd_av^4 * (n - 1))
    var_d <- var_g / (J^2)

    d_ci_lo <- d - sqrt(var_d) * qt(.975, n - 1)
    d_ci_up <- d + sqrt(var_d) * qt(.975, n - 1)
    g_ci_lo <- g - sqrt(var_g) * qt(.975, n - 1)
    g_ci_up <- g + sqrt(var_g) * qt(.975, n - 1)

    res <- cbind(
      d = d, var_d = var_d, d_ci_lo = d_ci_lo, d_ci_up = d_ci_up,
      g = g, var_g = var_g, g_ci_lo = g_ci_lo, g_ci_up = g_ci_up
    )

    return(res)
  }
}

################# R/Z to SMD ###################
.cor_to_smd <- function(r, r_se,
                        unit_increase_iv, sd_iv, unit_type,
                        n_sample, cor_to_smd) {
  if (cor_to_smd == "mathur") {
    increase <- ifelse(unit_type == "sd",
                       unit_increase_iv * sd_iv,
                       unit_increase_iv)

    d <- r * increase / (sd_iv * sqrt((1 - r^2)))
    d_se <- abs(d) * sqrt(1 / (r^2 * (n_sample - 3)) + 1 / (2 * (n_sample - 1)))
    res <- cbind(d, d_se)
    return(res)
  } else if (cor_to_smd == "viechtbauer") {
    res_g <- metafor::conv.delta(
      yi = r, vi = r_se^2, transf = metafor::transf.rtod, var.names = c("g", "g_var")
    )
    J <- .d_j(n_sample - 2)
    res_g$d <- res_g$g / J
    res_g$d_se <- sqrt(res_g$g_var / J^2)
    res <- cbind(res_g$d, res_g$d_se)

    return(res)
  } else if (cor_to_smd == "cooper") {
    d <- 2 * r / sqrt(1 - r^2)
    d_se <- sqrt(4 * r_se^2 / ((1 - r^2)^3))
    res <- cbind(d, d_se)

    return(res)
  }
}



.validate_pre_post_to_smd <- function(pre_post_to_smd, allowed_methods, context, func_name) {
  pre_post_to_smd_normalized <- ifelse(pre_post_to_smd == "cooper", "morris_drm", pre_post_to_smd)

  invalid <- !pre_post_to_smd_normalized %in% allowed_methods
  if (any(invalid)) {
    stop(paste0(
      "Invalid 'pre_post_to_smd' argument in ", func_name, "().\n\n",
      "Provided: '", paste(unique(pre_post_to_smd[invalid]), collapse = "', '"), "'\n",
      "Allowed methods: '", paste(allowed_methods, collapse = "', '"), "'\n\n",
      .get_method_restriction_rationale(context)
    ), call. = FALSE)
  }

  return(pre_post_to_smd_normalized)
}

.get_method_restriction_rationale <- function(context) {
  if (context == "mean_change") {
    paste0(
      "For mean change data, two standardization methods are available:\n\n",
      "  - 'morris_drm' (alias: 'cooper'): Raw score standardizer [DEFAULT]\n",
      "      d_rm = (mean_change / sd_change) * sqrt(2*(1-r))\n",
      "      REQUIRES pre-post correlation (r). Converts d_z to d_rm.\n\n",
      "  - 'morris_dz': Change score standardizer\n",
      "      d_z = mean_change / sd_change\n",
      "      INDEPENDENT of r. Direct standardization by change SD.\n\n",
      "Other methods not applicable:\n",
      "  - 'bonett': requires baseline SD (not available with mean change data)\n",
      "  - 'morris_dav': requires separate pre/post SDs (not available)\n\n",
      "See Morris & DeShon (2002) for guidance on choosing between d_rm and d_z."
    )
  } else if (context == "paired_t") {
    paste0(
      "For paired t-test data, two standardization methods are available:\n\n",
      "  - 'morris_drm' (alias: 'cooper'): Raw score standardizer [DEFAULT]\n",
      "      d_rm = t * sqrt(2*(1-r)/n)\n",
      "      REQUIRES pre-post correlation (r). Most common in meta-analysis.\n\n",
      "  - 'morris_dz': Change score standardizer\n",
      "      d_z = t / sqrt(n)\n",
      "      INDEPENDENT of r. Use when correlation is unknown or when\n",
      "      synthesizing with other d_z estimates.\n\n",
      "Note: d_rm and d_z are on different scales. See Morris & DeShon (2002)\n",
      "for guidance on choosing between them."
    )
  } else if (context == "pre_post_means") {
    paste0(
      "For pre-post means data, four standardization methods are available:\n\n",
      "  - 'bonett': baseline-SD standardizer (Morris 2008 d_ppc2). Sensitive to a\n",
      "      baseline SD that is restricted relative to the endpoint SD (e.g. by\n",
      "      eligibility cut-offs), which inflates the effect size.\n",
      "  - 'morris_drm' (alias: 'cooper'): change SD rescaled by sqrt(2(1-r)) onto\n",
      "      the raw-score metric. Depends on r_pre_post; assumes SD_pre = SD_post.\n",
      "  - 'morris_dz': change-SD standardizer. Independent of r, but NOT on the\n",
      "      same metric as an endpoint SMD -- do not pool the two (Cochrane 10.5.2).\n",
      "  - 'morris_dav': average-SD standardizer (Morris 2008 d_ppc3). Morris\n",
      "      recommends AGAINST it (2008, p.384): its sampling variance was unknown\n",
      "      and it is downward-biased when the post-treatment SD inflates.\n\n",
      "No single method is best in all cases; the choice changes the estimand.\n",
      "See Morris (2008) and Morris & DeShon (2002) for detailed comparisons."
    )
  } else {
    ""
  }
}


# tryCatch({
#   validate_positive(n_cases_exp, n_cases_nexp, n_controls_exp, n_controls_nexp,
#                     error_message = "The number of cases/controls in the exposed/non-exposed groups should be >0.")
# }, error = function(e) {
#   stop("Validation failed:", conditionMessage(e), "\n")
# })
#
#
# tryCatch({
#   validate_ci_symmetry(value, ci_lo, ci_up, func = "example_function",
#                        max_asymmetry_percent = 5)
# }, error = function(e) {
#   stop("Validation failed:", conditionMessage(e), "\n")
# })
#


# **A.** First, Cooper et al. (2019) - \code{table_2x2_to_cor = "cooper"} -
# proposes to convert the
# 2x2 table into a OR (formula above), to convert this OR into a SMD
# (see formula in \code{\link{es_from_or_se}()}), and to convert this
# SMD into a correlation coefficient (see formula in \code{\link{es_from_cohen_d}()},
# with the option \code{"smd_to_cor = 'lipsey_cooper'"}).
#
# **B.** Second, a correlation coefficient (more precisely - a phi coefficient)
# can be obtained from the contingency table using the formula given in
# Lipsey and Wilson (2001) - \code{table_2x2_to_cor = "lipsey"}.
# The formulas used to estimate the r and z are:
# \deqn{r = \frac{(n\_cases\_exp*n\_controls\_nexp - n\_controls\_exp*n\_cases\_nexp)}{\sqrt{(n\_exp) * (n\_nexp) * (n\_cases) * (n\_controls\_exp+n\_cases\_nexp)}}}
# \deqn{z = atanh(r)}
# \deqn{z\_se = logor\_se^2 * \frac{z^2}{\log(or)^2}}
# \deqn{z\_ci\_lo = z - qnorm(.975)*z\_se}
# \deqn{z\_ci\_up = z + qnorm(.975)*z\_se}
# \deqn{r\_ci\_lo = tanh(z\_ci\_lo)}
# \deqn{r\_ci\_up = tanh(z\_ci\_up)}
# \deqn{effective\_n = \frac{1}{z\_se^2 + 3}}
# \deqn{r\_se = \frac{(1 - r^2)^2}{effective\_n - 1}}

#' Aggregate a dataframe containing dependent effect sizes
#'
#' @param x a dataframe that should be aggregated (must contain effect size values and standard errors).
#' @param dependence The type of dependence in your dataframe (can be "outcomes", "times", or "subgroups"). See details.
#' @param agg_fact A character string identifying the column name that contains the clustering units (all rows with the same \code{agg_fact} value will be aggregated together).
#' @param es A character string identifying the column name containing the effect size values. Default is "es".
#' @param se A character string identifying the column name containing the standard errors of the effect size. Default is "se".
#' @param cor_unit The correlation between effect sizes coming from the same clustering unit. Used directly when \code{dependence = "outcomes"}. When \code{dependence = "times"} the within-cluster correlation is taken from a per-row \code{cor_unit} column if present (which must be constant within each cluster), and this argument is only used as a fallback when that column is absent; \code{dependence = "times"} additionally requires a \code{time_agg} column giving the time-point of each effect size. Ignored when \code{dependence = "subgroups"}.
#' @param col_mean a vector of character strings identifying the column names for which the dependent values are summarized by taking their mean.
#' @param col_weighted_mean a vector of character strings identifying the column names for which the dependent values are summarized by taking their weighted mean.
#' @param weights The weights that will be used to estimated the weighted means.
#' @param col_sum a vector of character strings identifying the column names for which the dependent values are summarized by taking their sum.
#' @param col_min a vector of character strings identifying the column names for which the dependent values are summarized by taking their minimum.
#' @param col_max a vector of character strings identifying the column names for which the dependent values are summarized by taking their maximum.
#' @param col_fact a vector of character strings identifying the column names that are factors (different values will be separated by a "/" character).
#' @param na.rm a logical vector indicating whether missing values should be ignored in the calculations for the \code{col_mean}, \code{col_weighted_mean}, \code{col_sum}, \code{col_min} and \code{col_max} arguments.
#'
#' @details
#' 1. In the \code{dependence} argument, you should indicate "outcomes" if the dependence within the same clustering unit (e.g., study) is due to the presence of multiple effect sizes produced from the same participants at the same time-point (e.g., multiple outcome measures)
#' 2. In the \code{dependence} argument, you should indicate "times" if the dependence within the same clustering unit (e.g., study) is due to the presence of multiple effect sizes produced from the same participants at the different time-points (e.g., an RCT with several follow-up waves). This option requires a \code{time_agg} column (the time-point of each effect size) and a \code{cor_unit} column giving the within-cluster correlation (constant within each cluster; falls back to the \code{cor_unit} argument if the column is absent).
#' 3. In the \code{dependence} argument, you should indicate "subgroups" if the dependence within the same clustering unit (e.g., study) is due to the presence of multiple effect sizes produced by independent subgroups (e.g., one effect size for boys, and one for girls).
#'
#' If you are working with ratio measures, make sure that the information on
#' the effect size estimates (i.e., the column passed to the es argument of the function)
#' is presented on the log scale.
#'
#' @return
#' The object returned by the \code{aggregate_df} contains, is a dataframe containing at the very least,
#' the aggregating factor, and the aggregated effect size values and standard errors. All columns indicated in the \code{col_*} arguments
#' will also be included in this dataframe.
#' \tabular{ll}{
#'  \code{row_index} \tab the row number in the original dataset.\cr
#'  \tab \cr
#'  \code{es} \tab the aggregated effect size value.\cr
#'  \tab \cr
#'  \code{se} \tab the standard error of the aggregated effect size.\cr
#'  \tab \cr
#'  \code{...} \tab any columns indicated in the \code{col_*} arguments.\cr
#'  \tab \cr
#' }
#'
#' @export aggregate_df
#'
#' @md
#'
#' @examples
#' res <- summary(convert_df(df.haza, measure = "d"))
#' aggregate_df(res, dependence = "outcomes", cor_unit = 0.8,
#'              agg_fact = "study_id", es = "es_crude", se = "se_crude",
#'              col_fact = c("outcome", "type_publication"))
#'
aggregate_df <- function(x, dependence = "outcomes", cor_unit = 0.8,
                         agg_fact, es = "es", se = "se",
                         col_mean = NA, col_weighted_mean = NA, weights = NA,
                         col_sum = NA, col_min = NA, col_max = NA,
                         col_fact = NA, na.rm = TRUE) {
  # x=x_mult_outcome; dependence = "outcomes"; cor_unit = 0.8;
  # agg_fact = "ID"; es = "value"; se = "se";
  # col_mean = NA; col_weighted_mean = NA; weights = NA;
  # col_sum = NA; col_min = NA; col_max = NA;
  # col_fact = "row_index"; na.rm = TRUE
  if (!dependence %in% c("outcomes", "times", "subgroups")) {
    stop(paste0("'", dependence, "' not in tolerated measures. Possible inputs are: 'outcomes', 'times', or 'subgroups'."))
  }

  if ((missing(es) & !"es" %in% colnames(x)) | (missing(se) & !"se" %in% colnames(x))) {
    stop(paste0("Please indicate the column names storing information on the effect size values and standard errors using the 'es' & 'se' arguments"))
  }
  if ((missing(es) & "es" %in% colnames(x))) {
    es <- "es"
  }
  if ((missing(se) & "se" %in% colnames(x))) {
    se <- "se"
  }

  if (length(es) > 1 | length(se) > 1) {
    stop(paste0("The '", es, "' and the '", se, "' arguments should contain only 1 element"))
  }
  if (!es %in% colnames(x) | !se %in% colnames(x)) {
    stop(paste0("The '", es, "' or the '", se, "' variable has not been found in your dataset"))
  }

  if (dependence == "outcomes") {
    res <- .agg.outcomes(
      x = x, agg_fact = agg_fact, es = es, se = se,
      cor_unit = cor_unit,
      col_mean = col_mean,
      col_weighted_mean = col_weighted_mean, weights = weights,
      col_sum = col_sum, col_min = col_min, col_max = col_max,
      col_fact = col_fact, na.rm = na.rm
    )
  } else if (dependence == "times") {
    res <- .agg.times(
      x = x, agg_fact = agg_fact, es = es, se = se,
      cor_unit = cor_unit,
      col_mean = col_mean,
      col_weighted_mean = col_weighted_mean, weights = weights,
      col_sum = col_sum, col_min = col_min, col_max = col_max,
      col_fact = col_fact, na.rm = na.rm
    )
  } else {
    res <- .agg.subgroups(
      x = x, agg_fact = agg_fact, es = es, se = se,
      col_mean = col_mean,
      col_weighted_mean = col_weighted_mean, weights = weights,
      col_sum = col_sum, col_min = col_min, col_max = col_max,
      col_fact = col_fact, na.rm = na.rm
    )
  }

  rownames(res) <- 1:nrow(res)
  return(res)
}

.agg.subgroups <- function(x, agg_fact, es, se,
                           col_mean = NA, col_weighted_mean = NA, weights = NA,
                           col_sum = NA, col_min = NA, col_max = NA,
                           col_fact = NA, na.rm = TRUE) {
  cols <- c(col_mean, col_weighted_mean, col_sum, col_min, col_max, col_fact)

  dup_col <- ifelse(length(na.omit(cols)) > 0, any(duplicated(na.omit(cols))), FALSE)

  if (dup_col) {
    stop("Each column of your dataset can be aggregated only once.")
  }

  if (any(!is.na(col_weighted_mean) & is.na(weights))) {
    stop("You should specify the 'weights' argument when requesting to calculate a weighted mean for some columns.")
  }

  df_mean <- df_w_mean <- df_sum <- df_min <- df_max <- df_fact <- NA

  dup_row <- duplicated(x[, agg_fact]) | duplicated(x[, agg_fact], fromLast = TRUE)

  x_tot <- cbind(x, es = x[, es], se = x[, se], row_index = 1:nrow(x))

  x_unique <- x_tot[!dup_row, ]

  x <- x_tot[dup_row, ]

  if (nrow(x) == length(unique(x[, agg_fact]))) {
    warning("the number of rows and unique 'agg_fact' values are equal")
    return(x_unique)
  }

  # Split on a character key: split() on a factor keeps an (empty) group for
  # every unused level, which would slip past the guards and inject phantom
  # all-NA rows into the output.
  x_split <- split(x, as.character(x[, agg_fact]))

  agg_bor <- do.call(rbind, lapply(x_split, .unique_es_subgroups, na.rm = na.rm))

  agg_bor$agg <- row.names(agg_bor)
  # agg_bor$agg <- unique(x[, agg_fact])

  if (any(!is.na(col_mean))) {
    df_mean <- aggregate.data.frame(x[, col_mean],
                                    by = list(agg = x[, agg_fact]),
                                    FUN = .mean_na, na.rm = na.rm)
    colnames(df_mean) <- c("agg", col_mean)
  }
  if (any(!is.na(col_weighted_mean))) {
    x_pond <- x[, col_weighted_mean, drop = FALSE] * x[, weights]

    sum_x_pond <- aggregate.data.frame(x_pond, by = list(agg = x[, agg_fact]),
                                       FUN = .sum_na, na.rm = na.rm)
    sum_N <- aggregate.data.frame(x[, weights], by = list(agg = x[, agg_fact]),
                                  FUN = .sum_na, na.rm = na.rm)

    # Carry the aggregate-produced (sorted) key alongside its own values -
    # unique(x[, agg_fact]) is in APPEARANCE order and would mis-pair labels
    # with values whenever the input is not pre-sorted. Indexing by name (not
    # $x) also supports multi-column col_weighted_mean.
    df_w_mean <- data.frame(agg = sum_x_pond$agg,
                            sum_x_pond[, col_weighted_mean, drop = FALSE] / sum_N$x,
                            check.names = FALSE)
    colnames(df_w_mean) <- c("agg", col_weighted_mean)
  }

  if (any(!is.na(col_sum))) {
    df_sum <- aggregate.data.frame(x[, col_sum], by = list(agg = x[, agg_fact]),
                                   FUN = .sum_na, na.rm = na.rm)
    colnames(df_sum) <- c("agg", col_sum)
  }

  if (any(!is.na(col_min))) {
    df_min <- aggregate.data.frame(x[, col_min], by = list(agg = x[, agg_fact]),
                                   FUN = .min_na, na.rm = na.rm)
    colnames(df_min) <- c("agg", col_min)
  }

  if (any(!is.na(col_max))) {
    df_max <- aggregate.data.frame(x[, col_max], by = list(agg = x[, agg_fact]),
                                   FUN = .max_na, na.rm = na.rm)
    colnames(df_max) <- c("agg", col_max)
  }

  if (any(!is.na(col_fact))) {
    df_fact <- aggregate.data.frame(x[, col_fact], by = list(agg = x[, agg_fact]),
                                    FUN = .agg.fact, na.rm = na.rm)
    colnames(df_fact) <- c("agg", col_fact)
  }

  # Merge only the frames that were actually built. The unused col_* placeholders
  # default to scalar NA; merging against them used to append an artifact column
  # named 'y' (later stripped with subset(-c(y))) - a landmine that collided with
  # and destroyed any real user column named 'y'.
  parts <- Filter(is.data.frame, list(agg_bor, df_mean, df_w_mean,
                                      df_sum, df_min, df_max, df_fact))
  df_add <- Reduce(function(a, b) merge(a, b), parts)

  first <- c(
    which(colnames(df_add) == "row_index"),
    which(colnames(df_add) == "agg")
  )

  df_add_clean <- cbind(df_add[, first], df_add[, -first])

  colnames(df_add_clean)[colnames(df_add_clean) == "agg"] <- agg_fact

  df_return <- rbind(x_unique[, colnames(df_add_clean)], df_add_clean)

  res <- df_return[order(df_return$row_index), ]
  rownames(res) <- 1:nrow(res)

  return(res)
}
.sum_na <- function(x, na.rm) {
  if (sum(is.na(x)) == length(x) & na.rm == TRUE) {
    NA
  } else {
    sum(x, na.rm = na.rm)
  }
}
.mean_na <- function(x, na.rm) {
  if (sum(is.na(x)) == length(x) & na.rm == TRUE) {
    NA
  } else {
    mean(x, na.rm = na.rm)
  }
}
.min_na <- function(x, na.rm) {
  if (sum(is.na(x)) == length(x) & na.rm == TRUE) {
    NA
  } else {
    min(x, na.rm = na.rm)
  }
}
.max_na <- function(x, na.rm) {
  if (sum(is.na(x)) == length(x) & na.rm == TRUE) {
    NA
  } else {
    max(x, na.rm = na.rm)
  }
}

.agg.outcomes <- function(x, agg_fact, es, se,
                          col_mean, col_weighted_mean, weights,
                          col_sum, col_min, col_max,
                          col_fact, cor_unit, na.rm) {
  cols <- c(col_mean, col_weighted_mean, col_sum, col_min, col_max, col_fact)

  dup_col <- ifelse(length(na.omit(cols)) > 0, any(duplicated(na.omit(cols))), FALSE)

  if (dup_col) {
    stop("Each column of your dataset can be aggregated only once.")
  }

  if (any(!is.na(col_weighted_mean) & is.na(weights))) {
    stop("You should specify the 'weights' argument when requesting to calculate a weighted mean for some columns.")
  }

  df_mean <- df_w_mean <- df_sum <- df_min <- df_max <- df_fact <- NA


  dup_row <- duplicated(x[, agg_fact]) | duplicated(x[, agg_fact], fromLast = TRUE)

  x_tot <- cbind(x,
    es = x[, es], se = x[, se],
    row_index = 1:nrow(x)
  )

  x_unique <- x_tot[!dup_row, ]

  x <- x_tot[dup_row, ]

  if (nrow(x) == length(unique(x[, agg_fact]))) {
    warning("the number of rows and unique 'agg_fact' values are equal")
    return(x_unique)
  }

  # Split on a character key: split() on a factor keeps an (empty) group for
  # every unused level, which would slip past the guards and inject phantom
  # all-NA rows into the output.
  x_split <- split(x, as.character(x[, agg_fact]))
  # measure = "SMD"
  agg_bor <- do.call(rbind, lapply(x_split, .unique_es_outcomes, cor_unit = cor_unit, na.rm = na.rm))

  agg_bor$agg <- row.names(agg_bor)

  if (any(!is.na(col_mean))) {
    df_mean <- aggregate.data.frame(x[, col_mean], by = list(agg = x[, agg_fact]), FUN = .mean_na, na.rm = na.rm)
    colnames(df_mean) <- c("agg", col_mean)
  }

  if (any(!is.na(col_weighted_mean))) {
    x_pond <- x[, col_weighted_mean, drop = FALSE] * x[, weights]
    sum_x_pond <- aggregate.data.frame(x_pond, by = list(agg = x[, agg_fact]), FUN = .sum_na, na.rm = na.rm)
    sum_N <- aggregate.data.frame(x[, weights], by = list(agg = x[, agg_fact]), FUN = .sum_na, na.rm = na.rm)
    df_w_mean <- data.frame(agg = sum_x_pond$agg,
                            sum_x_pond[, col_weighted_mean, drop = FALSE] / sum_N$x,
                            check.names = FALSE)
    colnames(df_w_mean) <- c("agg", col_weighted_mean)
  }

  if (any(!is.na(col_sum))) {
    df_sum <- aggregate.data.frame(x[, col_sum], by = list(agg = x[, agg_fact]),
                                   FUN = .sum_na, na.rm = na.rm)
    colnames(df_sum) <- c("agg", col_sum)
  }

  if (any(!is.na(col_min))) {
    df_min <- aggregate.data.frame(x[, col_min],
                                   by = list(agg = x[, agg_fact]),
                                   FUN = .min_na, na.rm = na.rm)
    colnames(df_min) <- c("agg", col_min)
  }

  if (any(!is.na(col_max))) {
    df_max <- aggregate.data.frame(x[, col_max],
                                   by = list(agg = x[, agg_fact]),
                                   FUN = .max_na, na.rm = na.rm)
    colnames(df_max) <- c("agg", col_max)
  }

  if (any(!is.na(col_fact))) {
    df_fact <- aggregate.data.frame(x[, col_fact],
                                    by = list(agg = x[, agg_fact]),
                                    FUN = .agg.fact, na.rm = na.rm)
    colnames(df_fact) <- c("agg", col_fact)
  }

  # Merge only the frames that were actually built (see .agg.subgroups for the
  # rationale - avoids the 'y' artifact-column collision).
  parts <- Filter(is.data.frame, list(agg_bor, df_mean, df_w_mean,
                                      df_sum, df_min, df_max, df_fact))
  df_add <- Reduce(function(a, b) merge(a, b), parts)

  first <- c(
    which(colnames(df_add) == "row_index"),
    which(colnames(df_add) == "agg")
  )

  df_add_clean <- cbind(df_add[, first], df_add[, -first])

  colnames(df_add_clean)[colnames(df_add_clean) == "agg"] <- agg_fact

  df_return <- rbind(x_unique[, colnames(df_add_clean)], df_add_clean)

  res <- df_return[order(df_return$row_index), ]
  return(res)
}
.agg.times <- function(x, agg_fact, es, se,
                          col_mean, col_weighted_mean, weights,
                          col_sum, col_min, col_max,
                          col_fact, cor_unit, na.rm) {
  cols <- c(col_mean, col_weighted_mean, col_sum, col_min, col_max, col_fact)

  dup_col <- ifelse(length(na.omit(cols)) > 0, any(duplicated(na.omit(cols))), FALSE)

  if (dup_col) {
    stop("Each column of your dataset can be aggregated only once.")
  }

  if (any(!is.na(col_weighted_mean) & is.na(weights))) {
    stop("You should specify the 'weights' argument when requesting to calculate a weighted mean for some columns.")
  }

  df_mean <- df_w_mean <- df_sum <- df_min <- df_max <- df_fact <- NA


  dup_row <- duplicated(x[, agg_fact]) | duplicated(x[, agg_fact], fromLast = TRUE)

  x_tot <- cbind(x,
                 es = x[, es], se = x[, se],
                 row_index = 1:nrow(x)
  )

  x_unique <- x_tot[!dup_row, ]

  x <- x_tot[dup_row, ]

  if (nrow(x) == length(unique(x[, agg_fact]))) {
    warning("the number of rows and unique 'agg_fact' values are equal")
    return(x_unique)
  }

  # dependence = "times" needs a per-row time index and a within-cluster
  # correlation. Require the time column, and fall back to the cor_unit ARGUMENT
  # when the data carries no cor_unit column (a cor_unit column takes precedence
  # when present).
  if (!"time_agg" %in% colnames(x)) {
    stop("dependence = 'times' requires a 'time_agg' column giving the time-point (wave) of each effect size.")
  }
  if (!"cor_unit" %in% colnames(x)) {
    x$cor_unit <- cor_unit
  }

  # Split on a character key: split() on a factor keeps an (empty) group for
  # every unused level, which would slip past the guards and inject phantom
  # all-NA rows into the output.
  x_split <- split(x, as.character(x[, agg_fact]))
  agg_bor <- do.call(rbind, lapply(x_split, .unique_es_times, na.rm = na.rm))

  agg_bor$agg <- row.names(agg_bor)

  if (any(!is.na(col_mean))) {
    df_mean <- aggregate.data.frame(x[, col_mean], by = list(agg = x[, agg_fact]), FUN = .mean_na, na.rm = na.rm)
    colnames(df_mean) <- c("agg", col_mean)
  }

  if (any(!is.na(col_weighted_mean))) {
    x_pond <- x[, col_weighted_mean, drop = FALSE] * x[, weights]
    sum_x_pond <- aggregate.data.frame(x_pond, by = list(agg = x[, agg_fact]), FUN = .sum_na, na.rm = na.rm)
    sum_N <- aggregate.data.frame(x[, weights], by = list(agg = x[, agg_fact]), FUN = .sum_na, na.rm = na.rm)
    df_w_mean <- data.frame(agg = sum_x_pond$agg,
                            sum_x_pond[, col_weighted_mean, drop = FALSE] / sum_N$x,
                            check.names = FALSE)
    colnames(df_w_mean) <- c("agg", col_weighted_mean)
  }

  if (any(!is.na(col_sum))) {
    df_sum <- aggregate.data.frame(x[, col_sum], by = list(agg = x[, agg_fact]),
                                   FUN = .sum_na, na.rm = na.rm)
    colnames(df_sum) <- c("agg", col_sum)
  }

  if (any(!is.na(col_min))) {
    df_min <- aggregate.data.frame(x[, col_min],
                                   by = list(agg = x[, agg_fact]),
                                   FUN = .min_na, na.rm = na.rm)
    colnames(df_min) <- c("agg", col_min)
  }

  if (any(!is.na(col_max))) {
    df_max <- aggregate.data.frame(x[, col_max],
                                   by = list(agg = x[, agg_fact]),
                                   FUN = .max_na, na.rm = na.rm)
    colnames(df_max) <- c("agg", col_max)
  }

  if (any(!is.na(col_fact))) {
    df_fact <- aggregate.data.frame(x[, col_fact],
                                    by = list(agg = x[, agg_fact]),
                                    FUN = .agg.fact, na.rm = na.rm)
    colnames(df_fact) <- c("agg", col_fact)
  }

  # Merge only the frames that were actually built (see .agg.subgroups for the
  # rationale - avoids the 'y' artifact-column collision).
  parts <- Filter(is.data.frame, list(agg_bor, df_mean, df_w_mean,
                                      df_sum, df_min, df_max, df_fact))
  df_add <- Reduce(function(a, b) merge(a, b), parts)

  first <- c(
    which(colnames(df_add) == "row_index"),
    which(colnames(df_add) == "agg")
  )

  df_add_clean <- cbind(df_add[, first], df_add[, -first])

  colnames(df_add_clean)[colnames(df_add_clean) == "agg"] <- agg_fact

  df_return <- rbind(x_unique[, colnames(df_add_clean)], df_add_clean)

  res <- df_return[order(df_return$row_index), ]
  return(res)
}

.unique_es_subgroups <- function(x, na.rm = TRUE) {
  row_index <- x$row_index[1]

  # Drop effect sizes with a missing es/se before combining, matching the
  # behaviour of metafor::aggregate.escalc() (which discards NA rows). Without
  # this, a single NA nullifies the whole cluster and can leave an incoherent
  # (es = NA, se = <valid>) pair.
  if (na.rm) {
    x <- x[!is.na(x$es) & !is.na(x$se), , drop = FALSE]
  }
  # A non-positive or non-finite SE cannot enter inverse-variance weighting: 1/se^2
  # becomes Inf/NaN, which turns the combined estimate into Inf/Inf = NaN paired with
  # se = 0 (an incoherent result). Drop such rows -- se = 0 is itself pathological and
  # is flagged upstream by the Tier-2 'SE is zero' check -- and warn so it is visible.
  bad_se <- !is.finite(x$se) | x$se <= 0
  if (any(bad_se)) {
    warning(sprintf(
      "aggregate_df: dropped %d effect size(s) with a non-positive/non-finite SE from a subgroup cluster before inverse-variance combination.",
      sum(bad_se)))
    x <- x[!bad_se, , drop = FALSE]
  }
  if (nrow(x) == 0) {
    return(data.frame(es = NA_real_, se = NA_real_, row_index = row_index))
  }

  weights <- 1 / (x$se^2)
  mean_es <- sum(weights * x$es) / sum(weights)
  se <- sqrt(1 / sum(weights))

  data.frame(es = mean_es, se = se, row_index = row_index)
}
.unique_es_outcomes <- function(x, cor_unit, na.rm = TRUE) {
  row_index <- x$row_index[1]

  if (na.rm) {
    x <- x[!is.na(x$es) & !is.na(x$se), , drop = FALSE]
  }
  if (nrow(x) == 0) {
    return(data.frame(es = NA_real_, se = NA_real_, row_index = row_index))
  }

  var_es <- x$se^2
  prod_se <- x$se %*% t(x$se)
  prod_se_r <- prod_se * cor_unit
  prod_se_r[lower.tri(prod_se_r)] <- 0
  diag(prod_se_r) <- 0

  mean_es <- mean(x$es)
  var <- (1 / length(x$es))^2 * (sum(var_es) + 2 * sum(prod_se_r))
  se <- sqrt(var)

  data.frame(es = mean_es, se = se, row_index = row_index)
}


.unique_es_times <- function(x, na.rm = TRUE) {
  row_index <- x[1, "row_index"]

  # NB: length(unique(.)) - a *count* of distinct values - not unique(.) > 1,
  # which tests the value and errors ("condition has length > 1") when a cluster
  # legitimately carries more than one cor_unit value.
  if (length(unique(x[, "cor_unit"])) > 1) {
    stop("cor_unit values should be constant within a cluster unit when 'dependence=times' is called")
  }
  cor_unit <- unique(x[, "cor_unit"])

  if (na.rm) {
    x <- x[!is.na(x[, "es"]) & !is.na(x[, "se"]), , drop = FALSE]
  }
  if (nrow(x) == 0) {
    return(data.frame(es = NA_real_, se = NA_real_, row_index = row_index))
  }

  time <- x[, "time_agg"]
  R <- outer(time, time, function(a, b) cor_unit^abs(a - b))

  # nrow= guards the length-1 case: diag(scalar) would build an identity matrix.
  S <- diag(sqrt(x[, "se"]^2), nrow = nrow(x))
  V <- S %*% R %*% S

  W <- try(solve(V), silent = TRUE)
  if (inherits(W, "try-error")) {
    stop("Cannot take inverse of the covariance matrix 'V'.")
  }

  sum_W <- sum(W)
  mean_es <- sum(W %*% x[, "es"]) / sum_W
  se <- sqrt(1 / sum_W)

  data.frame(es = mean_es, se = se, row_index = row_index)
}

.agg.fact <- function(x, na.rm = na.rm) {
  if (na.rm) {
    ifelse(length(unique(na.omit(x))) == 0,
      "N/A",
      ifelse(length(unique(na.omit(x))) == 1,
        unique(na.omit(x)),
        paste(na.omit(sort(unique(x))), collapse = " / ")
      )
    )
  } else {
    ifelse(length(unique(x)) == 0,
      "N/A",
      ifelse(length(unique(x)) == 1,
        unique(x),
        paste(sort(unique(x)), collapse = " / ")
      )
    )
  }
}

#' Convert an eta-squared value to various effect size measures
#'
#' @param etasq an eta-squared value (binary predictor, ANOVA model), defined as
#'   \eqn{SS_{effect} / SS_{total}}. For a one-way, two-group ANOVA this equals the
#'   partial eta-squared \eqn{SS_{effect} / (SS_{effect} + SS_{error})}, so the two
#'   coincide and either label is correct here (unlike the ANCOVA case handled by
#'   \code{\link{es_from_etasq_adj}()}, which requires the partial form specifically).
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation.
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_etasq a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function first computes a Cohen's d (D) and Hedges' g (G)
#' from the eta squared of a binary predictor (ANOVA model).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate a Cohen's d** the following formula is used (Cohen, 1988):
#' \deqn{d = 2 * \sqrt{\frac{etasq}{1 - etasq}}}
#'
#' Note that this closed form is the large-sample, **equal-groups** (\eqn{n\_exp = n\_nexp})
#' limit of the exact F-based conversion. For markedly unequal group sizes it is
#' approximate; supplying the ANOVA F (\code{\link{es_from_anova_f}()}), which
#' carries the per-group sample sizes, is more accurate.
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cohen, J. (1988). Statistical power analysis for the behavioral sciences. Routledge.
#'
#' @export es_from_etasq
#'
#' @md
#'
#' @examples
#' es_from_etasq(etasq = 0.28, n_exp = 20, n_nexp = 22)
es_from_etasq <- function(etasq, n_exp, n_nexp, smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_etasq) {
  if (missing(reverse_etasq)) reverse_etasq <- rep(FALSE, length(etasq))
  reverse_etasq[is.na(reverse_etasq)] <- FALSE


  d <- 2 * (sqrt(etasq / (1 - etasq)))

  es <- .es_from_d(
    d = d, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_etasq
  )

  es$info_used <- "etasq"

  return(es)
}

#' Convert an adjusted eta-squared value (i.e., from an ANCOVA) to various effect size measures
#'
#' @param etasq_adj an adjusted eta-squared value obtained from an ANCOVA model. This
#'   must be the **partial** eta-squared of the group effect,
#'   \eqn{\eta_p^2 = SS_{group} / (SS_{group} + SS_{error})} — the quantity that
#'   most software (SPSS, \code{car}, \code{effectsize}) prints for an ANCOVA term.
#'   Do **not** supply the classical eta-squared \eqn{SS_{group} / SS_{total}}: in an
#'   ANCOVA the total sum of squares also contains the covariate variance, so the
#'   classical value is smaller than the partial one and would be inverted into too
#'   small an F, biasing the effect size toward zero. (In a covariate-free one-way
#'   ANOVA the two definitions coincide; see \code{\link{es_from_etasq}()}.)
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param cov_outcome_r correlation between the outcome and covariate (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_etasq a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the adjusted (partial) eta-squared of a binary predictor
#' (ANCOVA model) into the ANCOVA F-statistic it implies, and then relies on the
#' calculations of the \code{\link{es_from_ancova_f}()} function. The returned
#' Cohen's d (D) and Hedges' g (G) are therefore expressed on the
#' **marginal (unadjusted) SD scale** — consistent with the other
#' \code{es_from_ancova_*} functions — not on the residual (covariate-adjusted)
#' SD scale on which a partial eta-squared is natively defined.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To convert the adjusted eta-squared into an ANCOVA F-statistic**,
#' the partial-eta-squared identity \eqn{\eta_p^2 = F / (F + df)} is inverted
#' (this is why a *partial* eta-squared, not a classical one, is required):
#' \deqn{df = n\_exp + n\_nexp - 2 - n\_cov\_ancova}
#' \deqn{ancova\_f = \frac{etasq\_adj * df}{1 - etasq\_adj}}
#'
#' **To estimate a Cohen's d** the formula used is (table 12.3 in Cooper):
#' \deqn{cohen\_d = \sqrt{ancova\_f * \frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp}} * \sqrt{1 - cov\_out\_cor^2}}
#'
#' Note that the back-transformation to the marginal SD scale requires
#' \code{cov_outcome_r}; when it is missing, the effect size estimates are
#' returned as NA (the marginal scale is not identified from the adjusted
#' eta-squared alone).
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 18. Adjusted: ANCOVA statistics, eta-squared'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @note
#' Two cautions apply. (1) \code{etasq_adj} must be the *partial* eta-squared (see
#' the parameter description); a classical eta-squared biases the effect size
#' toward zero. (2) The sampling variance is rebuilt from Cooper's eq. 12.26, which
#' assumes a covariate balanced across groups and treats \code{cov_outcome_r} as
#' known; in covariate-imbalanced designs the standard error is a lower bound
#' (anti-conservative), while the point estimate remains unbiased. See Lai and
#' Kelley (2012).
#'
#' @references
#' Cooper, H., Hedges, L. V., & Valentine, J. C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Lai, K., & Kelley, K. (2012). Accuracy in parameter estimation for ANCOVA and ANOVA contrasts: Sample size planning via narrow confidence intervals. British Journal of Mathematical and Statistical Psychology, 65(2), 350-370.
#'
#' @export es_from_etasq_adj
#'
#' @md
#'
#' @examples
#' es_from_etasq_adj(etasq_adj = 0.28, n_cov_ancova = 3, cov_outcome_r = 0.2, n_exp = 20, n_nexp = 22)
es_from_etasq_adj <- function(etasq_adj, n_exp, n_nexp, n_cov_ancova, cov_outcome_r,
                              smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_etasq) {
  if (missing(reverse_etasq)) reverse_etasq <- rep(FALSE, length(etasq_adj))
  reverse_etasq[is.na(reverse_etasq)] <- FALSE

  # A partial eta-squared from an ANCOVA is natively defined on the RESIDUAL
  # (covariate-adjusted) SD scale. Convert it back to the ANCOVA F-statistic it
  # implies (eta_p^2 = F / (F + df_err), so F = eta_p^2 * df_err / (1 - eta_p^2))
  # and delegate to es_from_ancova_f(), which back-transforms the point estimate
  # to the MARGINAL (unadjusted-SD) scale via sqrt(1 - cov_outcome_r^2)
  # (table 12.3 in Cooper). This keeps a single source of truth for the ANCOVA
  # family: the point estimate, the (1 - cov_outcome_r^2)-shrunk sampling
  # variance and the df = N - 2 - n_cov_ancova CIs are mutually consistent, and
  # etasq_adj agrees exactly with es_from_ancova_f fed the algebraically
  # equivalent statistic. When cov_outcome_r is NA the marginal scale is not
  # identified and the output degrades to NA (rather than silently emitting a
  # residual-scale value).
  df_err <- n_exp + n_nexp - 2 - n_cov_ancova
  f_implied <- etasq_adj * df_err / (1 - etasq_adj)

  es <- es_from_ancova_f(
    ancova_f = f_implied, cov_outcome_r = cov_outcome_r,
    n_cov_ancova = n_cov_ancova, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_f = reverse_etasq
  )

  es$info_used <- "etasq_adj"

  return(es)
}

#' default flag options
#' @noRd
.default_flag_options <- function() {
  list(
    smd_max = 3, # C1
    r_max = 0.95, # C4
    log_or_max = 5, # C5
    n_min = 10, # C6
    iqr_mult = 3, # D1/D2
    dispersion_max_smd = 0.5, # E1
    dispersion_max_r = 0.15, # E1
    dispersion_max_logor = 1.0, # E1
    diff_max_smd = 1.0, # E3
    diff_max_r = 0.3, # E3
    diff_max_logor = 2.0, # E3
    overlap_min = 0.85, # E2b
    enable_cross_row = TRUE, # D1/D2
    alpha_max = 0.99, # C7
    icc_max = 0.99, # C8
    group_ratio_max = 10, # C10
    direction_conflict_min = 2, # G1
    direction_conflict_pct = 10, # G1
    direction_conflict_lone_pct = 20, # G1, Inf to disable the lone tier
    direction_conflict_info = TRUE, # G1
    direction_disagreement_min = 0.1, # E5, unit-dependent for md/mdw
    enable_informational = FALSE, # V8
    ignore_sign_reversal = FALSE, # E1-E3
    outlier_min_deviation = NULL, # D1
    se_outlier_min_ratio = 3, # D2
    se_ratio_extreme = 10, # D2b
    sd_ratio_max = 10, # V12
    sd_ratio_bl_ep_min = 0.7, # V18
    baseline_imbalance_max = 0.30, # V21
    templated_min_match = 3, # V23
    paired_as_indep_tol = 0.07, # V26
    sd_outlier_ratio = 5, # V27/D3
    sd_outlier_smd = FALSE # V27/D3, off: smd pools mix instrument scales
  )
}


#' max absolute deviation from the median (used by E1)
#' for two values equals |a - b| / 2, so E1 == E3 at k = 2
#' @param v numeric vector of per-method effect-size estimates for one row
#' @return numeric scalar max(|v - median(v)|), or NA if < 2 finite values
#' @noRd
.dispersion_stat <- function(v) {
  v <- v[is.finite(v)]
  if (length(v) < 2) return(NA_real_)
  max(abs(v - stats::median(v)))
}

#' note for r-sensitive flags (A6, E2, E2b), which depend on a defaulted r_pre_post
#' @param is_def logical scalar (TRUE when this row's r_pre_post was defaulted)
#' @return character scalar (empty string when not r-sensitive)
#' @noRd
.r_sensitive_note <- function(is_def) {
  if (isTRUE(is_def)) {
    # no "; " here, the flag merge splits on it
    " (r-sensitive: depends on the default r_pre_post - re-check with the true pre-post correlation)"
  } else {
    ""
  }
}


#########

#' Count the number of decimal places in a numeric value
#'
#' Used by the CI asymmetry check to estimate rounding precision.
#' @param x numeric scalar
#' @return integer, number of decimal places (0 for integers, max 10)
#' @noRd
.count_decimals <- function(x) {
  if (is.na(x) || !is.finite(x)) return(2L)
  s <- format(x, scientific = FALSE, drop0trailing = TRUE)
  if (!grepl("\\.", s)) return(1L)  # integers assumed at least 1dp precision
  dp <- nchar(sub(".*\\.", "", s))
  min(dp, 10L)
}


#' Columns that must be non-negative in the input data
#' @return character vector of column names
#' @noRd
.positive_columns <- function() {
  c(
    # Sample sizes
    "n_exp", "n_nexp", "n_sample", "n_cases", "n_controls",
    "n_cases_exp", "n_cases_nexp", "n_controls_exp", "n_controls_nexp",
    # SDs
    "mean_sd_exp", "mean_sd_nexp", "mean_sd_pooled",
    "mean_pre_sd_exp", "mean_pre_sd_nexp",
    "mean_change_sd_exp", "mean_change_sd_nexp",
    "ancova_mean_sd_exp", "ancova_mean_sd_nexp", "ancova_mean_sd_pooled",
    "md_sd", "ancova_md_sd",
    # SEs
    "mean_se_exp", "mean_se_nexp", "mean_pre_se_exp", "mean_pre_se_nexp",
    "mean_change_se_exp", "mean_change_se_nexp",
    "ancova_mean_se_exp", "ancova_mean_se_nexp",
    "md_se", "ancova_md_se", "logor_se", "logrr_se", "logirr_se",
    "rd_se", "linreg_b_se",
    "user_se_crude", "user_se_adj",
    # Psychometric counts
    "n_items", "n_measurements",
    # Statistics & other
    "anova_f", "chisq", "etasq", "etasq_adj",
    "time_exp", "time_nexp", "baseline_rate"
    # p-values checked in .bounded_columns
  )
}

#' CI triplets to check for consistency in the input data
#'
#' Each triplet has val/lo/up column names and a scale indicator: "additive"
#' (symmetric CIs), "exp" (natural-scale OR/RR, checked on the log scale), or
#' "user" (detected at runtime from the original measure).
#'
#' @return list of lists, each with val/lo/up/scale
#' @noRd
.ci_triplets <- function() {
  list(
    # Means (additive scale)
    list(val = "mean_exp",         lo = "mean_ci_lo_exp",         up = "mean_ci_up_exp",         scale = "additive"),
    list(val = "mean_nexp",        lo = "mean_ci_lo_nexp",        up = "mean_ci_up_nexp",        scale = "additive"),
    list(val = "mean_pre_exp",     lo = "mean_pre_ci_lo_exp",     up = "mean_pre_ci_up_exp",     scale = "additive"),
    list(val = "mean_pre_nexp",    lo = "mean_pre_ci_lo_nexp",    up = "mean_pre_ci_up_nexp",    scale = "additive"),
    list(val = "mean_change_exp",  lo = "mean_change_ci_lo_exp",  up = "mean_change_ci_up_exp",  scale = "additive"),
    list(val = "mean_change_nexp", lo = "mean_change_ci_lo_nexp", up = "mean_change_ci_up_nexp", scale = "additive"),
    list(val = "ancova_mean_exp",  lo = "ancova_mean_ci_lo_exp",  up = "ancova_mean_ci_up_exp",  scale = "additive"),
    list(val = "ancova_mean_nexp", lo = "ancova_mean_ci_lo_nexp", up = "ancova_mean_ci_up_nexp", scale = "additive"),
    # Mean differences (additive scale)
    list(val = "md",               lo = "md_ci_lo",               up = "md_ci_up",               scale = "additive"),
    list(val = "ancova_md",        lo = "ancova_md_ci_lo",        up = "ancova_md_ci_up",        scale = "additive"),
    # User input (scale depends on user_es_original_measure - detected at runtime)
    list(val = "user_es_crude",    lo = "user_ci_lo_crude",       up = "user_ci_up_crude",       scale = "user"),
    list(val = "user_es_adj",      lo = "user_ci_lo_adj",         up = "user_ci_up_adj",         scale = "user"),
    # Log-scale ratios (additive on log scale)
    list(val = "logor",            lo = "logor_ci_lo",            up = "logor_ci_up",            scale = "additive"),
    list(val = "logrr",            lo = "logrr_ci_lo",            up = "logrr_ci_up",            scale = "additive"),
    list(val = "logirr",           lo = "logirr_ci_lo",           up = "logirr_ci_up",           scale = "additive"),
    # Risk difference (additive scale)
    list(val = "rd",        lo = "rd_ci_lo",        up = "rd_ci_up",        scale = "additive"),
    # Regression coefficient (additive scale)
    list(val = "linreg_b",  lo = "linreg_b_ci_lo",  up = "linreg_b_ci_up",  scale = "additive"),
    # Natural-scale ratios (exp scale - inherently asymmetric CIs)
    list(val = "or",               lo = "or_ci_lo",               up = "or_ci_up",               scale = "exp"),
    list(val = "rr",               lo = "rr_ci_lo",               up = "rr_ci_up",               scale = "exp"),
    # Plot-extracted means (additive scale)
    list(val = "plot_mean_exp",         lo = "plot_mean_ci_lo_exp",         up = "plot_mean_ci_up_exp",         scale = "additive"),
    list(val = "plot_mean_nexp",        lo = "plot_mean_ci_lo_nexp",        up = "plot_mean_ci_up_nexp",        scale = "additive"),
    list(val = "plot_ancova_mean_exp",  lo = "plot_ancova_mean_ci_lo_exp",  up = "plot_ancova_mean_ci_up_exp",  scale = "additive"),
    list(val = "plot_ancova_mean_nexp", lo = "plot_ancova_mean_ci_lo_nexp", up = "plot_ancova_mean_ci_up_nexp", scale = "additive")
  )
}

#' Validate input data before effect size computation
#'
#' Sets invalid values to NA and records per-row issues. When
#' \code{correct_inputs = FALSE}, invalid values are flagged but preserved.
#'
#' @param x data.frame, the input data after .check_data()
#' @param max_asymmetry numeric, percentage threshold for CI asymmetry detection (default 10)
#' @param verbose logical, if TRUE emit console messages for detected issues (default TRUE)
#' @param correct_inputs logical, if TRUE (default) invalid values are set to missing;
#'   if FALSE, values are flagged but preserved in the returned data.
#' @return list with $data (cleaned data.frame) and $issues (character vector, one per row)
#' @noRd
.validate_input_data <- function(x, max_asymmetry = 10, verbose = TRUE,
                                  enable_informational = FALSE,
                                  sd_ratio_max = 10,
                                  sd_ratio_bl_ep_min = 0.7,
                                  baseline_imbalance_max = 0.30,
                                  enable_cross_row = TRUE,
                                  templated_min_match = 3,
                                  measure = NULL,
                                  paired_as_indep_tol = 0.07,
                                  correct_inputs = TRUE) {
  n <- nrow(x)
  row_issues <- vector("list", n)
  for (i in seq_len(n)) row_issues[[i]] <- character(0)

  # Check non-negative columns
  for (col in .positive_columns()) {
    if (!col %in% colnames(x)) next
    bad <- which(x[[col]] < 0 & !is.na(x[[col]]))
    if (length(bad) > 0) {
      if (correct_inputs) {
        x[[col]][bad] <- NA_real_
        for (i in bad) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Negative input: '%s' set to missing", col))
        }
      } else {
        for (i in bad) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Negative input: '%s' = %g", col, x[[col]][i]))
        }
      }
    }
  }

  # V10: Zero SD/SE (impossible for real data, indicates data entry error)
  .sd_columns <- c("mean_sd_exp", "mean_sd_nexp", "mean_sd_pooled",
                    "mean_pre_sd_exp", "mean_pre_sd_nexp",
                    "mean_change_sd_exp", "mean_change_sd_nexp",
                    "ancova_mean_sd_exp", "ancova_mean_sd_nexp", "ancova_mean_sd_pooled",
                    "md_sd", "ancova_md_sd")
  .se_columns <- c("mean_se_exp", "mean_se_nexp",
                    "mean_change_se_exp", "mean_change_se_nexp",
                    "ancova_mean_se_exp", "ancova_mean_se_nexp",
                    "ancova_md_se")
  for (col in c(.sd_columns, .se_columns)) {
    if (!col %in% colnames(x)) next
    zeros <- which(x[[col]] == 0 & !is.na(x[[col]]))
    if (length(zeros) > 0) {
      type_label <- if (col %in% .sd_columns) "SD" else "SE"
      if (correct_inputs) {
        x[[col]][zeros] <- NA_real_
        for (i in zeros) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Zero %s: '%s' = 0 (the effect size or its precision cannot be validly computed)",
                    type_label, col))
        }
      } else {
        for (i in zeros) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Zero %s: '%s' = 0 (the effect size or its precision cannot be validly computed)",
                    type_label, col))
        }
      }
    }
  }

  # V11: Bounded columns - values outside theoretical range set to missing
  # Correlations must be in [-1, 1]; reliability/proportion must be in [0, 1]
  .bounded_columns <- list(
    # alpha can be negative, only > 1 impossible (V25 warns on negatives)
    list(col = "cronbach_alpha",  lo = -Inf, up = 1, label = "alpha"),
    # icc bounded below by -1/(k-1) >= -1
    list(col = "icc",             lo = -1, up = 1, label = "ICC"),
    list(col = "prop",            lo = 0, up = 1, label = "proportion"),
    list(col = "pearson_r",       lo = -1, up = 1, label = "correlation"),
    list(col = "spearman_r",      lo = -1, up = 1, label = "correlation"),
    # phi is a 2x2 correlation coefficient in [-1, 1]; an out-of-range value
    # otherwise reaches metafor::conv.2x2(), which raises a HARD ERROR that would
    # abort the whole convert_df() run (violating the one-bad-cell-cannot-abort rule)
    list(col = "phi",             lo = -1, up = 1, label = "phi (correlation)"),
    # p-values must be in [0, 1]
    list(col = "student_t_pval",        lo = 0, up = 1, label = "p-value"),
    list(col = "anova_f_pval",          lo = 0, up = 1, label = "p-value"),
    list(col = "ancova_t_pval",         lo = 0, up = 1, label = "p-value"),
    list(col = "ancova_f_pval",         lo = 0, up = 1, label = "p-value"),
    list(col = "chisq_pval",            lo = 0, up = 1, label = "p-value"),
    list(col = "paired_t_pval_exp",     lo = 0, up = 1, label = "p-value"),
    list(col = "paired_t_pval_nexp",    lo = 0, up = 1, label = "p-value"),
    list(col = "paired_f_pval_exp",     lo = 0, up = 1, label = "p-value"),
    list(col = "paired_f_pval_nexp",    lo = 0, up = 1, label = "p-value"),
    list(col = "mean_change_pval_exp",  lo = 0, up = 1, label = "p-value"),
    list(col = "mean_change_pval_nexp", lo = 0, up = 1, label = "p-value"),
    list(col = "rr_pval",               lo = 0, up = 1, label = "p-value"),
    list(col = "rd_pval",               lo = 0, up = 1, label = "p-value"),
    list(col = "linreg_b_pval",         lo = 0, up = 1, label = "p-value"),
    list(col = "or_pval",               lo = 0, up = 1, label = "p-value"),
    list(col = "md_pval",               lo = 0, up = 1, label = "p-value"),
    list(col = "ancova_md_pval",        lo = 0, up = 1, label = "p-value"),
    list(col = "pt_bis_r_pval",         lo = 0, up = 1, label = "p-value"),
    # Baseline risk must be a proportion (0-1), not a percentage
    list(col = "baseline_risk",          lo = 0, up = 1, label = "baseline risk (proportion)")
  )
  for (bc in .bounded_columns) {
    if (!bc$col %in% colnames(x)) next
    vals <- x[[bc$col]]
    bad <- which(!is.na(vals) & (vals < bc$lo | vals > bc$up))
    if (length(bad) > 0) {
      # Describe the valid range; one-sided when a bound is infinite
      range_txt <- if (is.infinite(bc$lo)) {
        sprintf("<= %g", bc$up)
      } else if (is.infinite(bc$up)) {
        sprintf(">= %g", bc$lo)
      } else {
        sprintf("[%g, %g]", bc$lo, bc$up)
      }
      if (correct_inputs) {
        x[[bc$col]][bad] <- NA_real_
        for (i in bad) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Out-of-range %s: '%s' = %g (valid: %s), set to missing",
                    bc$label, bc$col, vals[i], range_txt))
        }
      } else {
        for (i in bad) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Out-of-range %s: '%s' = %g (valid: %s)",
                    bc$label, bc$col, vals[i], range_txt))
        }
      }
    }
  }

  # V32: eta-squared must lie in [0, 1). A value >= 1 makes the Cohen's d
  # conversion diverge -- d = 2*sqrt(eta2 / (1 - eta2)) for a raw eta-squared,
  # and the implied ANCOVA F = eta2 * df / (1 - eta2) for an adjusted one -- so
  # such a row is set to missing (correct_inputs = TRUE) or flagged (FALSE).
  # Negatives are already caught by the non-negative-column check above. A
  # large-but-valid eta-squared that implies a huge SMD is surfaced downstream
  # by the post-computation Large-SMD check, so no separate plausibility tier is
  # added here.
  for (col in c("etasq", "etasq_adj")) {
    if (!col %in% colnames(x)) next
    vals <- x[[col]]
    bad <- which(!is.na(vals) & vals >= 1)
    if (length(bad) == 0) next
    if (correct_inputs) {
      x[[col]][bad] <- NA_real_
      for (i in bad) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INVALID] Out-of-range eta-squared: '%s' = %g (valid: [0, 1)), set to missing",
                  col, vals[i]))
      }
    } else {
      for (i in bad) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INVALID] Out-of-range eta-squared: '%s' = %g (valid: [0, 1))",
                  col, vals[i]))
      }
    }
  }

  # V25: negative alpha/icc kept, warn only
  for (rel in list(c("cronbach_alpha", "alpha"), c("icc", "ICC"))) {
    if (!rel[1] %in% colnames(x)) next
    neg <- which(!is.na(x[[rel[1]]]) & x[[rel[1]]] < 0)
    for (i in neg) {
      row_issues[[i]] <- c(row_issues[[i]],
        sprintf("[UNUSUAL] Negative %s: '%s' = %g. Mathematically possible but indicates serious measurement problems or an extraction error - verify against the primary report",
                rel[2], rel[1], x[[rel[1]]][i]))
    }
  }

  # V31: agreement-type ICC rows use a one-way variance approximation that
  # assumes negligible between-rater variance -- the SE is anti-conservative
  # when raters differ systematically (informational, always active, like
  # V18/V21). icc_type = NA or an absent column resolves to the package
  # default "agreement".
  if ("icc" %in% colnames(x)) {
    icc_type_eff <- if ("icc_type" %in% colnames(x)) {
      tt <- as.character(x[["icc_type"]])
      tt[is.na(tt)] <- "agreement"
      tt
    } else {
      rep("agreement", nrow(x))
    }
    agr <- which(!is.na(x[["icc"]]) & icc_type_eff == "agreement")
    for (i in agr) {
      row_issues[[i]] <- c(row_issues[[i]],
        "[INFO] ICC agreement-type SE assumes negligible between-rater variance and can be anti-conservative when raters differ systematically (see the es_from_icc documentation)")
    }
  }

  # Check CI consistency
  for (tri in .ci_triplets()) {
    if (!all(c(tri$val, tri$lo, tri$up) %in% colnames(x))) next
    val <- x[[tri$val]]; lo <- x[[tri$lo]]; up <- x[[tri$up]]

    # V2: Inverted CI (lo > up) -- rounding-aware: only flag when the inversion
    # exceeds the combined rounding error of the two bounds (mirrors V4/V28), so a
    # borderline pair like lo = 0.15, up = 0.14 reported at different precisions is
    # preserved rather than destroyed.
    both_lu <- which(!is.na(lo) & !is.na(up) & lo > up)
    inverted <- both_lu[vapply(both_lu, function(i) {
      pad <- 0.5 * 10^(-.count_decimals(lo[i])) + 0.5 * 10^(-.count_decimals(up[i]))
      lo[i] > up[i] + pad
    }, logical(1))]
    if (length(inverted) > 0) {
      if (correct_inputs) {
        x[[tri$val]][inverted] <- NA_real_
        x[[tri$lo]][inverted] <- NA_real_
        x[[tri$up]][inverted] <- NA_real_
        for (i in inverted) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Inverted CI for '%s' (lo > up) set to missing", tri$val))
        }
      } else {
        for (i in inverted) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Inverted CI for '%s': lo = %g > up = %g",
                    tri$val, lo[i], up[i]))
        }
      }
    }

    # V3: Value outside CI (re-read after V2 cleanup) -- rounding-aware. The point
    # value and the CI bounds may be reported to different decimal precisions, so a
    # value that merely rounds to a bound (e.g. md = 0.1 [1 dp] vs lo = 0.15 [2 dp],
    # both consistent with a true md in [0.145, 0.155)) must NOT be destroyed. Flag
    # (and, under correct_inputs, NA) only when the value lies outside by more than
    # the combined rounding error of the value and the offending bound.
    val <- x[[tri$val]]; lo <- x[[tri$lo]]; up <- x[[tri$up]]
    cand <- which(!is.na(val) & !is.na(lo) & !is.na(up) & lo <= up &
                    (val < lo | val > up))
    outside <- cand[vapply(cand, function(i) {
      re_val <- 0.5 * 10^(-.count_decimals(val[i]))
      pad_lo <- re_val + 0.5 * 10^(-.count_decimals(lo[i]))
      pad_up <- re_val + 0.5 * 10^(-.count_decimals(up[i]))
      (val[i] < lo[i] - pad_lo) || (val[i] > up[i] + pad_up)
    }, logical(1))]
    if (length(outside) > 0) {
      if (correct_inputs) {
        x[[tri$val]][outside] <- NA_real_
        x[[tri$lo]][outside] <- NA_real_
        x[[tri$up]][outside] <- NA_real_
        for (i in outside) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Value outside CI for '%s' set to missing", tri$val))
        }
      } else {
        for (i in outside) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Value outside CI for '%s': %g not in [%g, %g]",
                    tri$val, val[i], lo[i], up[i]))
        }
      }
    }
  }

  # V4: ci asymmetry, warn only (rounding); or/rr checked on log scale
  for (tri in .ci_triplets()) {
    if (!all(c(tri$val, tri$lo, tri$up) %in% colnames(x))) next
    val <- x[[tri$val]]; lo <- x[[tri$lo]]; up <- x[[tri$up]]

    has_all <- which(!is.na(val) & !is.na(lo) & !is.na(up) & lo <= up)
    if (length(has_all) == 0) next

    # For exp-scale CIs, convert to log scale before checking asymmetry
    if (identical(tri$scale, "exp")) {
      # Need all values > 0 for log transform
      has_all <- has_all[val[has_all] > 0 & lo[has_all] > 0 & up[has_all] > 0]
      if (length(has_all) == 0) next
      check_val <- log(val[has_all])
      check_lo <- log(lo[has_all])
      check_up <- log(up[has_all])
      log_transformed <- rep(TRUE, length(has_all))
    } else if (identical(tri$scale, "user")) {
      # User-entered data: detect scale from user_es_original_measure column.
      # Ratio types (OR, RR, HR, IRR) need log-transform; others are additive.
      measure_col <- if (grepl("crude$", tri$val)) {
        "user_es_original_measure_crude"
      } else {
        "user_es_original_measure_adj"
      }
      if (!measure_col %in% colnames(x)) next
      ratio_types <- c("or", "rr", "hr", "irr")
      is_ratio <- tolower(as.character(x[[measure_col]])) %in% ratio_types

      ratio_idx <- has_all[is_ratio[has_all]]
      additive_idx <- has_all[!is_ratio[has_all]]

      if (length(ratio_idx) > 0) {
        ratio_idx <- ratio_idx[val[ratio_idx] > 0 & lo[ratio_idx] > 0 & up[ratio_idx] > 0]
      }
      # Combine: ratio on log scale, additive on original scale
      check_val <- numeric(length(has_all))
      check_lo <- numeric(length(has_all))
      check_up <- numeric(length(has_all))
      # Map back to positions in has_all
      for (j in seq_along(has_all)) {
        idx <- has_all[j]
        if (idx %in% ratio_idx) {
          check_val[j] <- log(val[idx])
          check_lo[j] <- log(lo[idx])
          check_up[j] <- log(up[idx])
        } else {
          check_val[j] <- val[idx]
          check_lo[j] <- lo[idx]
          check_up[j] <- up[idx]
        }
      }
      # Remove rows that were ratio but had non-positive values (couldn't log-transform)
      keep <- rep(TRUE, length(has_all))
      for (j in seq_along(has_all)) {
        idx <- has_all[j]
        if (is_ratio[idx] && !(idx %in% ratio_idx)) keep[j] <- FALSE
      }
      has_all <- has_all[keep]
      check_val <- check_val[keep]
      check_lo <- check_lo[keep]
      check_up <- check_up[keep]
      if (length(has_all) == 0) next
      # Track which rows were log-transformed (for rounding computation)
      log_transformed <- has_all %in% ratio_idx
    } else {
      check_val <- val[has_all]
      check_lo <- lo[has_all]
      check_up <- up[has_all]
      log_transformed <- rep(FALSE, length(has_all))
    }

    lower_dist <- check_val - check_lo
    upper_dist <- check_up - check_val
    avg_dist <- (lower_dist + upper_dist) / 2

    # Only check where avg_dist > 0 (non-zero CI width)
    pos <- avg_dist > 0
    if (!any(pos)) next

    checkable <- has_all[pos]
    is_log <- log_transformed[pos]
    asym_pct <- (abs(upper_dist[pos] - lower_dist[pos]) / avg_dist[pos]) * 100

    # per-row threshold: max asymmetry explainable by rounding at the reported precision
    # max rounding asymmetry = sum of the three rounding errors / avg half-width
    rounding_threshold <- rep(max_asymmetry, sum(pos))
    orig_val <- val[checkable]
    orig_lo  <- lo[checkable]
    orig_up  <- up[checkable]
    for (j in seq_along(checkable)) {
      dp_val <- .count_decimals(orig_val[j])
      dp_lo  <- .count_decimals(orig_lo[j])
      dp_up  <- .count_decimals(orig_up[j])
      re_val <- 0.5 * 10^(-dp_val)
      re_lo  <- 0.5 * 10^(-dp_lo)
      re_up  <- 0.5 * 10^(-dp_up)
      if (identical(tri$scale, "exp") || is_log[j]) {
        # On log scale, rounding error ~ re / value (first-order)
        if (orig_val[j] > 0 && orig_lo[j] > 0 && orig_up[j] > 0) {
          re_val <- re_val / orig_val[j]
          re_lo  <- re_lo  / orig_lo[j]
          re_up  <- re_up  / orig_up[j]
        }
      }
      max_rounding_asym <- (re_val + re_lo + re_up) / avg_dist[pos][j] * 100
      rounding_threshold[j] <- max(max_asymmetry, max_rounding_asym * 2)
    }

    asymmetric <- checkable[asym_pct > rounding_threshold]
    if (length(asymmetric) > 0) {
      for (i in asymmetric) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INFO] Asymmetric CI for '%s' (>%g%% asymmetry)",
                  tri$val, max_asymmetry))
      }
    }
  }

  # V5: Logical consistency - cases cannot exceed total sample size
  # Each pair is (part, total): n_cases_exp <= n_exp, etc.
  part_total_pairs <- list(
    c("n_cases_exp",  "n_exp"),
    c("n_cases_nexp", "n_nexp"),
    c("n_controls_exp",  "n_exp"),
    c("n_controls_nexp", "n_nexp"),
    c("n_cases",   "n_sample"),
    c("n_controls", "n_sample")
  )
  for (pair in part_total_pairs) {
    part_col <- pair[1]; total_col <- pair[2]
    if (!part_col %in% colnames(x) || !total_col %in% colnames(x)) next
    bad <- which(!is.na(x[[part_col]]) & !is.na(x[[total_col]]) &
                 x[[part_col]] > x[[total_col]])
    if (length(bad) > 0) {
      if (correct_inputs) {
        x[[part_col]][bad] <- NA_real_
        x[[total_col]][bad] <- NA_real_
        for (i in bad) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Inconsistent input: '%s' > '%s', both set to missing",
                    part_col, total_col))
        }
      } else {
        for (i in bad) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] Inconsistent input: '%s' = %g > '%s' = %g",
                    part_col, x[[part_col]][i], total_col, x[[total_col]][i]))
        }
      }
    }
  }

  # V15: warn only, mismatch is often legitimate (multi-arm trial, analysis subset)
  if (all(c("n_exp", "n_nexp", "n_sample") %in% colnames(x))) {
    ne <- x[["n_exp"]]; nn <- x[["n_nexp"]]; ns <- x[["n_sample"]]
    mismatch <- which(!is.na(ne) & !is.na(nn) & !is.na(ns) &
                      (ne + nn) != ns)
    if (length(mismatch) > 0) {
      for (i in mismatch) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[UNUSUAL] Sample size mismatch: n_exp + n_nexp = %g != n_sample = %g. Often legitimate (multi-arm trial, or n_sample is an analysis subset); verify these refer to the same pool",
                  ne[i] + nn[i], ns[i]))
      }
    }
  }

  # V7: a + b must equal the margin; all three set to NA (cannot tell which is wrong)
  additive_triples <- list(
    list(a = "n_cases_exp",  b = "n_controls_exp",  total = "n_exp"),
    list(a = "n_cases_nexp", b = "n_controls_nexp", total = "n_nexp"),
    list(a = "n_cases",      b = "n_controls",      total = "n_sample"),
    # row margins
    list(a = "n_cases_exp",  b = "n_cases_nexp",    total = "n_cases"),
    list(a = "n_controls_exp", b = "n_controls_nexp", total = "n_controls")
  )
  for (triple in additive_triples) {
    if (!all(c(triple$a, triple$b, triple$total) %in% colnames(x))) next
    a <- x[[triple$a]]; b <- x[[triple$b]]; total <- x[[triple$total]]
    mismatch <- which(!is.na(a) & !is.na(b) & !is.na(total) &
                      (a + b) != total)
    if (length(mismatch) > 0) {
      if (correct_inputs) {
        x[[triple$a]][mismatch] <- NA_real_
        x[[triple$b]][mismatch] <- NA_real_
        x[[triple$total]][mismatch] <- NA_real_
        for (i in mismatch) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] 2x2 inconsistency: %s + %s != %s",
                    triple$a, triple$b, triple$total))
        }
      } else {
        for (i in mismatch) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INVALID] 2x2 inconsistency: %s + %s != %s (sum=%g, total=%g)",
                    triple$a, triple$b, triple$total,
                    a[i] + b[i], total[i]))
        }
      }
    }
  }

  # V8: Zero cell in 2x2 table (warn only - continuity correction will be applied)
  # Controlled by enable_informational (default FALSE: suppressed)
  cell_cols <- c("n_cases_exp", "n_cases_nexp", "n_controls_exp", "n_controls_nexp")
  present_cells <- cell_cols[cell_cols %in% colnames(x)]
  if (length(present_cells) == 4) {
    # V8b: Double-zero studies - both arms have zero events (degenerate OR/RR)
    cases_exp <- x[["n_cases_exp"]]; cases_nexp <- x[["n_cases_nexp"]]
    double_zero <- which(!is.na(cases_exp) & !is.na(cases_nexp) &
                         cases_exp == 0 & cases_nexp == 0)
    if (length(double_zero) > 0) {
      for (i in double_zero) {
        row_issues[[i]] <- c(row_issues[[i]],
          "[INFO] Double-zero study: both arms have zero events (n_cases_exp = 0, n_cases_nexp = 0). OR/RR are degenerate even with continuity correction")
      }
    }
    # Also check double-zero on the controls side
    if (all(c("n_controls_exp", "n_controls_nexp") %in% colnames(x))) {
      ctrl_exp <- x[["n_controls_exp"]]; ctrl_nexp <- x[["n_controls_nexp"]]
      double_zero_ctrl <- which(!is.na(ctrl_exp) & !is.na(ctrl_nexp) &
                                ctrl_exp == 0 & ctrl_nexp == 0)
      if (length(double_zero_ctrl) > 0) {
        for (i in double_zero_ctrl) {
          row_issues[[i]] <- c(row_issues[[i]],
            "[INFO] Double-zero study: both arms have zero non-events (n_controls_exp = 0, n_controls_nexp = 0). OR/RR are degenerate even with continuity correction")
        }
      }
    }

    # V8: Individual zero cells (informational)
    if (isTRUE(enable_informational)) {
      for (col in present_cells) {
        zero_rows <- which(!is.na(x[[col]]) & x[[col]] == 0)
        if (length(zero_rows) > 0) {
          for (i in zero_rows) {
            row_issues[[i]] <- c(row_issues[[i]],
              sprintf("[INFO] Zero cell in 2x2 table ('%s' = 0): OR/RR use +0.5 continuity correction",
                      col))
          }
        }
      }
    }
  }

  # V16: n = 1 per arm (SMD undefined - pooled SD has df = 0)
  for (ncol in c("n_exp", "n_nexp")) {
    if (!ncol %in% colnames(x)) next
    one_rows <- which(!is.na(x[[ncol]]) & x[[ncol]] == 1)
    if (length(one_rows) > 0) {
      for (i in one_rows) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INVALID] %s = 1: within-group variance is undefined (df = 0), SMD cannot be estimated",
                  ncol))
      }
    }
  }

  # V24: min <= q1 <= median <= q3 <= max, warn only
  # reverse_med flips sign at computation, raw cells stay ascending
  .quantile_groups <- list(
    list(suffix = "exp",
         cols  = c("min_exp",  "q1_exp",  "med_exp",  "q3_exp",  "max_exp")),
    list(suffix = "nexp",
         cols  = c("min_nexp", "q1_nexp", "med_nexp", "q3_nexp", "max_nexp"))
  )
  .quantile_labels <- c(min_exp = "min", q1_exp = "Q1", med_exp = "median",
                        q3_exp = "Q3", max_exp = "max",
                        min_nexp = "min", q1_nexp = "Q1", med_nexp = "median",
                        q3_nexp = "Q3", max_nexp = "max")
  for (qg in .quantile_groups) {
    present <- qg$cols[qg$cols %in% colnames(x)]
    if (length(present) < 2) next
    for (i in seq_len(n)) {
      vals <- vapply(present, function(cc) {
        v <- x[[cc]][i]; if (is.null(v)) NA_real_ else suppressWarnings(as.numeric(v))
      }, numeric(1))
      ok <- !is.na(vals)
      if (sum(ok) < 2) next
      pv <- vals[ok]; pn <- present[ok]   # canonical min->max order preserved
      bad <- which(diff(pv) < 0)
      if (length(bad) > 0) {
        bp <- bad[1]
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INVALID] Non-monotonic quantile summary for '%s' (need min<=Q1<=median<=Q3<=max): %s (%g) < %s (%g)",
                  qg$suffix,
                  .quantile_labels[[pn[bp + 1]]], pv[bp + 1],
                  .quantile_labels[[pn[bp]]],     pv[bp]))
      }
    }
  }

  # arms used by V28 and V29 (endpoint only, no min_pre/max_pre columns)
  .range_arms <- list(
    list(sfx = "exp",  mean = "mean_exp",  min = "min_exp",  max = "max_exp",
         sd = "mean_sd_exp",  n = "n_exp"),
    list(sfx = "nexp", mean = "mean_nexp", min = "min_nexp", max = "max_nexp",
         sd = "mean_sd_nexp", n = "n_nexp")
  )

  # V28: mean must lie in [min, max], warn only; tolerance for rounding of both values
  for (arm in .range_arms) {
    if (!all(c(arm$mean, arm$min, arm$max) %in% colnames(x))) next
    mv <- suppressWarnings(as.numeric(x[[arm$mean]]))
    lo <- suppressWarnings(as.numeric(x[[arm$min]]))
    hi <- suppressWarnings(as.numeric(x[[arm$max]]))
    chk <- which(!is.na(mv) & !is.na(lo) & !is.na(hi) & lo <= hi)
    for (i in chk) {
      re_mean <- 0.5 * 10^(-.count_decimals(mv[i]))
      pad_lo <- re_mean + 0.5 * 10^(-.count_decimals(lo[i]))
      pad_hi <- re_mean + 0.5 * 10^(-.count_decimals(hi[i]))
      if (mv[i] < lo[i] - pad_lo || mv[i] > hi[i] + pad_hi) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INVALID] Mean outside its reported range for '%s': mean = %g not in [min = %g, max = %g]",
                  arm$sfx, mv[i], lo[i], hi[i]))
      }
    }
  }

  # V29: (max - min)/SD should be near xi(n) = 2*qnorm((n - .375)/(n + .25))
  # (Wan 2014), warn only
  range_sd_lo_mult <- 0.5
  range_sd_hi_mult <- 2.5
  for (arm in .range_arms) {
    if (!all(c(arm$min, arm$max, arm$sd, arm$n) %in% colnames(x))) next
    lo  <- suppressWarnings(as.numeric(x[[arm$min]]))
    hi  <- suppressWarnings(as.numeric(x[[arm$max]]))
    sdv <- suppressWarnings(as.numeric(x[[arm$sd]]))
    nv  <- suppressWarnings(as.numeric(x[[arm$n]]))
    chk <- which(!is.na(lo) & !is.na(hi) & !is.na(sdv) & !is.na(nv) &
                 hi > lo & sdv > 0 & nv >= 2)
    for (i in chk) {
      xi <- 2 * stats::qnorm((nv[i] - 0.375) / (nv[i] + 0.25))
      if (!is.finite(xi) || xi <= 0) next
      r_obs <- (hi[i] - lo[i]) / sdv[i]
      if (r_obs < range_sd_lo_mult * xi || r_obs > range_sd_hi_mult * xi) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[UNUSUAL] Range/SD ratio implausible for '%s': (max - min)/SD = %.2f (expected ~%.1f for n = %d). Possible SD-as-SE, variance-as-SD, or range/IQR mix-up",
                  arm$sfx, r_obs, xi, as.integer(nv[i])))
      }
    }
  }

  # V26: within-subject measure but reported CI matches the independent-groups SE
  # implied critical value compared to z and t; no assumed r needed. warn only
  within_meas <- !is.null(measure) && tolower(measure) %in% c("mdw", "dw", "gw")
  if (within_meas) {
    z_crit <- stats::qnorm(0.975)
    for (sfx in c("crude", "adj")) {
      lo_col <- paste0("user_ci_lo_", sfx)
      up_col <- paste0("user_ci_up_", sfx)
      need26 <- c(lo_col, up_col, "mean_sd_exp", "mean_sd_nexp", "n_exp", "n_nexp")
      if (!all(need26 %in% colnames(x))) next
      for (i in seq_len(n)) {
        lo  <- suppressWarnings(as.numeric(x[[lo_col]][i]))
        up  <- suppressWarnings(as.numeric(x[[up_col]][i]))
        sde <- suppressWarnings(as.numeric(x[["mean_sd_exp"]][i]))
        sdn <- suppressWarnings(as.numeric(x[["mean_sd_nexp"]][i]))
        ne  <- suppressWarnings(as.numeric(x[["n_exp"]][i]))
        nn  <- suppressWarnings(as.numeric(x[["n_nexp"]][i]))
        if (any(is.na(c(lo, up, sde, sdn, ne, nn)))) next
        # within-subject signature: equal arm n; need usable SDs and a real CI
        if (ne != nn || ne < 2 || sde <= 0 || sdn <= 0 || up <= lo) next
        indep_se <- sqrt(sde^2 / ne + sdn^2 / nn)
        if (!is.finite(indep_se) || indep_se <= 0) next
        implied_crit <- ((up - lo) / 2) / indep_se
        t_crit <- stats::qt(0.975, ne + nn - 2)
        matches <- (abs(implied_crit - z_crit) / z_crit <= paired_as_indep_tol) ||
                   (abs(implied_crit - t_crit) / t_crit <= paired_as_indep_tol)
        if (isTRUE(matches)) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[UNUSUAL] Within-subject design (%s) but the reported CI matches an independent-groups SE (n_exp = n_nexp = %g; implied critical value %.2f). Paired pre/post data analysed as two independent groups overestimates the variance - verify the SE formula (a paired/within-subject SE should be used)",
                    tolower(measure), ne, implied_crit))
        }
      }
    }
  }

  # V9: possible change score entered as endpoint (small SD/|mean|)
  if (isTRUE(enable_informational)) {
    m_exp_col <- "mean_exp"; sd_exp_col <- "mean_sd_exp"
    m_nexp_col <- "mean_nexp"; sd_nexp_col <- "mean_sd_nexp"
    has_exp <- m_exp_col %in% colnames(x) && sd_exp_col %in% colnames(x)
    has_nexp <- m_nexp_col %in% colnames(x) && sd_nexp_col %in% colnames(x)

    for (i in seq_len(n)) {
      m_e <- if (has_exp) x[[m_exp_col]][i] else NA_real_
      s_e <- if (has_exp) x[[sd_exp_col]][i] else NA_real_
      m_n <- if (has_nexp) x[[m_nexp_col]][i] else NA_real_
      s_n <- if (has_nexp) x[[sd_nexp_col]][i] else NA_real_

      cv_e <- if (!is.na(m_e) && !is.na(s_e) && s_e > 0 && abs(m_e) > 0) s_e / abs(m_e) else NA_real_
      cv_n <- if (!is.na(m_n) && !is.na(s_n) && s_n > 0 && abs(m_n) > 0) s_n / abs(m_n) else NA_real_

      # negative mean, small cv
      if (!is.na(cv_e) && !is.na(m_e) && m_e < 0 && cv_e < 0.3) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INFO] Possible change score as endpoint: '%s' = %g with small SD = %g (SD/|mean| = %s)",
                  m_exp_col, m_e, s_e, round(cv_e, 2)))
      }
      if (!is.na(cv_n) && !is.na(m_n) && m_n < 0 && cv_n < 0.3) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INFO] Possible change score as endpoint: '%s' = %g with small SD = %g (SD/|mean| = %s)",
                  m_nexp_col, m_n, s_n, round(cv_n, 2)))
      }

      # positive means: both cvs must be very small
      if (!is.na(cv_e) && !is.na(cv_n) &&
          !is.na(m_e) && m_e > 0 && !is.na(m_n) && m_n > 0 &&
          cv_e < 0.1 && cv_n < 0.1) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INFO] Possible change scores as endpoint: both group means positive with very small SD/|mean| (exp: %s, nexp: %s)",
                  round(cv_e, 2), round(cv_n, 2)))
      }
    }
  }

  # V12: extreme sd ratio between arms (Kanukula 2024 #5)
  sd_pairs <- list(
    c("mean_sd_exp", "mean_sd_nexp"),
    c("mean_pre_sd_exp", "mean_pre_sd_nexp"),
    c("mean_change_sd_exp", "mean_change_sd_nexp"),
    c("ancova_mean_sd_exp", "ancova_mean_sd_nexp")
  )
  for (pair in sd_pairs) {
    if (!all(pair %in% colnames(x))) next
    sd1 <- x[[pair[1]]]; sd2 <- x[[pair[2]]]
    checkable <- which(!is.na(sd1) & !is.na(sd2) & sd1 > 0 & sd2 > 0)
    if (length(checkable) == 0) next
    ratio <- pmax(sd1[checkable], sd2[checkable]) / pmin(sd1[checkable], sd2[checkable])
    flagged <- checkable[ratio > sd_ratio_max]
    if (length(flagged) > 0) {
      for (i in flagged) {
        r <- max(sd1[i], sd2[i]) / min(sd1[i], sd2[i])
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[UNUSUAL] Extreme SD ratio between arms: %s = %g, %s = %g (ratio: %.1f, threshold: %g). Possible variance/SD confusion or extraction error",
                  pair[1], sd1[i], pair[2], sd2[i], r, sd_ratio_max))
      }
    }
  }

  # V14: identical sds across arms (Kanukula 2024 #22)
  if (isTRUE(enable_informational)) {
    for (pair in sd_pairs) {
      if (!all(pair %in% colnames(x))) next
      sd1 <- x[[pair[1]]]; sd2 <- x[[pair[2]]]
      identical_rows <- which(!is.na(sd1) & !is.na(sd2) & sd1 > 0 & sd2 > 0 &
                              sd1 == sd2)
      if (length(identical_rows) > 0) {
        for (i in identical_rows) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INFO] Identical SDs across arms: %s = %s = %g. Verify this is not the pooled or between-group SD",
                    pair[1], pair[2], sd1[i]))
        }
      }
    }
  }

  # V17: identical means across arms
  if (isTRUE(enable_informational)) {
    mean_pairs <- list(
      c("mean_exp", "mean_nexp"),
      c("mean_pre_exp", "mean_pre_nexp"),
      c("mean_change_exp", "mean_change_nexp"),
      c("ancova_mean_exp", "ancova_mean_nexp")
    )
    for (pair in mean_pairs) {
      if (!all(pair %in% colnames(x))) next
      m1 <- x[[pair[1]]]; m2 <- x[[pair[2]]]
      identical_rows <- which(!is.na(m1) & !is.na(m2) & m1 == m2)
      if (length(identical_rows) > 0) {
        for (i in identical_rows) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INFO] Identical means across arms: %s = %s = %g. Verify this is not a copy-paste error",
                    pair[1], pair[2], m1[i]))
        }
      }
    }
  }

  # V18: sd_baseline / sd_endpoint below threshold (df-weighted pooled sds)
  if (all(c("mean_pre_sd_exp", "mean_pre_sd_nexp",
            "mean_sd_exp",     "mean_sd_nexp",
            "n_exp", "n_nexp") %in% colnames(x))) {
    n_e <- x$n_exp; n_n <- x$n_nexp
    spre_e <- x$mean_pre_sd_exp; spre_n <- x$mean_pre_sd_nexp
    spost_e <- x$mean_sd_exp;    spost_n <- x$mean_sd_nexp
    checkable <- which(!is.na(spre_e) & !is.na(spre_n) &
                        !is.na(spost_e) & !is.na(spost_n) &
                        !is.na(n_e) & !is.na(n_n) &
                        spre_e > 0 & spre_n > 0 &
                        spost_e > 0 & spost_n > 0 &
                        n_e + n_n > 2)
    for (i in checkable) {
      sp_pre  <- sqrt(((n_e[i] - 1) * spre_e[i]^2 +
                       (n_n[i] - 1) * spre_n[i]^2) / (n_e[i] + n_n[i] - 2))
      sp_post <- sqrt(((n_e[i] - 1) * spost_e[i]^2 +
                       (n_n[i] - 1) * spost_n[i]^2) / (n_e[i] + n_n[i] - 2))
      q <- sp_pre / sp_post
      if (is.finite(q) && q < sd_ratio_bl_ep_min) {
        row_issues[[i]] <- c(row_issues[[i]],
          sprintf("[INFO] SD_baseline / SD_endpoint = %.2f (below %.2f). Pre-post SMD formulas (Bonett, d_rm, d_av) target a baseline-SD-standardised estimand structurally different from the ANCOVA-on-endpoint-SD target at this variance regime; choice of estimand non-trivial",
                  q, sd_ratio_bl_ep_min))
      }
    }
  }

  # V30: baseline statistic copied into the endpoint slot (Kanukula 2024 #9/#10).
  # Two timepoints from the same patients essentially never reproduce an
  # identical SD (or mean); an exact baseline<->endpoint tie signals a copy.
  # Complementary to V18, which tests the sd_baseline/sd_endpoint ratio and by
  # design ignores the ratio = 1.00 a copy produces. Warn only, data preserved,
  # always active (a high-specificity exact-equality check, like V24/V28).
  .v30_equal <- function(pre_col, post_col, positive_only) {
    if (!all(c(pre_col, post_col) %in% colnames(x))) return(rep(NA_real_, n))
    pre  <- suppressWarnings(as.numeric(x[[pre_col]]))
    post <- suppressWarnings(as.numeric(x[[post_col]]))
    ok <- !is.na(pre) & !is.na(post) & pre == post
    if (positive_only) ok <- ok & pre > 0
    ifelse(ok, post, NA_real_)
  }
  v30_sd_e <- .v30_equal("mean_pre_sd_exp",  "mean_sd_exp",  TRUE)
  v30_sd_n <- .v30_equal("mean_pre_sd_nexp", "mean_sd_nexp", TRUE)
  v30_mn_e <- .v30_equal("mean_pre_exp",     "mean_exp",     FALSE)
  v30_mn_n <- .v30_equal("mean_pre_nexp",    "mean_nexp",    FALSE)
  for (i in seq_len(n)) {
    sd_e <- v30_sd_e[i]; sd_n <- v30_sd_n[i]
    has_e <- !is.na(sd_e); has_n <- !is.na(sd_n)
    both_sd <- has_e && has_n
    # a lone-arm SD tie needs >= 2 decimals to be distinctive (integer SDs
    # collide by chance too often to flag on a single arm)
    lone_val <- if (has_e && !has_n) sd_e else if (has_n && !has_e) sd_n else NA_real_
    fire <- both_sd || (!is.na(lone_val) && .count_decimals(lone_val) >= 2)
    if (!isTRUE(fire)) next
    # NB: no "; " anywhere in this message - the flag merge in .flag_es_quality
    # splits on "; ", so an internal one would shatter and mis-route the flag
    parts <- character(0)
    if (has_e) parts <- c(parts, sprintf("'mean_sd_exp' = 'mean_pre_sd_exp' = %s",
                                          format(sd_e, scientific = FALSE)))
    if (has_n) parts <- c(parts, sprintf("'mean_sd_nexp' = 'mean_pre_sd_nexp' = %s",
                                          format(sd_n, scientific = FALSE)))
    mean_note <- if (!is.na(v30_mn_e[i]) && !is.na(v30_mn_n[i])) {
      " (both group means are identical across timepoints too - almost certainly a copied baseline block)"
    } else {
      ""
    }
    row_issues[[i]] <- c(row_issues[[i]], sprintf(
      "[UNUSUAL] Baseline SD copied into endpoint slot (%s). Two timepoints from the same patients virtually never reproduce an identical SD - verify the endpoint (post-treatment) SD was not overwritten with the baseline SD%s",
      paste(parts, collapse = " and "), mean_note))
  }

  # V21: standardised baseline imbalance above threshold
  if (all(c("mean_pre_exp", "mean_pre_nexp",
            "mean_pre_sd_exp", "mean_pre_sd_nexp",
            "n_exp", "n_nexp") %in% colnames(x))) {
    checkable <- which(!is.na(x$mean_pre_exp) & !is.na(x$mean_pre_nexp) &
                        !is.na(x$mean_pre_sd_exp) & !is.na(x$mean_pre_sd_nexp) &
                        !is.na(x$n_exp) & !is.na(x$n_nexp) &
                        x$mean_pre_sd_exp > 0 & x$mean_pre_sd_nexp > 0 &
                        x$n_exp + x$n_nexp > 2)
    for (i in checkable) {
      sp_pre <- sqrt(((x$n_exp[i] - 1) * x$mean_pre_sd_exp[i]^2 +
                      (x$n_nexp[i] - 1) * x$mean_pre_sd_nexp[i]^2) /
                     (x$n_exp[i] + x$n_nexp[i] - 2))
      if (is.finite(sp_pre) && sp_pre > 0) {
        imb <- abs(x$mean_pre_exp[i] - x$mean_pre_nexp[i]) / sp_pre
        if (is.finite(imb) && imb > baseline_imbalance_max) {
          row_issues[[i]] <- c(row_issues[[i]],
            sprintf("[INFO] |standardised baseline imbalance| = %.2f (above %.2f). Endpoint-only SMD formulas inherit ANCOVA omitted-variable bias of magnitude r * imb_std; ANCOVA-adjusted statistics recommended at this imbalance level",
                    imb, baseline_imbalance_max))
        }
      }
    }
  }

  # V22: ancova residual sd without cov_outcome_r; marginal back-transformation
  # 1/sqrt(1 - R^2) impossible
  ancova_resid_cols <- c("ancova_mean_sd_pooled",
                          "ancova_mean_sd_exp", "ancova_mean_sd_nexp",
                          "ancova_md_sd")
  any_ancova_sd <- rep(FALSE, n)
  for (col_name in ancova_resid_cols) {
    if (col_name %in% colnames(x)) {
      vals <- x[[col_name]]
      any_ancova_sd <- any_ancova_sd | (!is.na(vals) & vals > 0)
    }
  }
  cov_r_present <- if ("cov_outcome_r" %in% colnames(x)) {
    # back-transformation uses R^2, so negative r is fine; only |r| >= 1 invalid
    !is.na(x$cov_outcome_r) & abs(x$cov_outcome_r) < 1
  } else {
    rep(FALSE, n)
  }
  flagged_v22 <- which(any_ancova_sd & !cov_r_present)
  for (i in flagged_v22) {
    row_issues[[i]] <- c(row_issues[[i]],
      "[UNUSUAL] ANCOVA residual SD provided but cov_outcome_r missing. Residual SD is conditional on the covariate(s); without cov_outcome_r the marginal-scale back-transformation 1/sqrt(1 - R^2) cannot be applied. Provide cov_outcome_r to avoid silent SMD underestimation by ~5-15% (Cochrane Handbook section 6.5.2.5 does not flag this step)")
  }

  # V23: identical stat block across different study_id (templated data);
  # same study_id skipped, full block required
  if (isTRUE(enable_cross_row) && n >= 2) {
    stat_blocks <- list(
      baseline = c("mean_pre_exp", "mean_pre_sd_exp", "mean_pre_nexp", "mean_pre_sd_nexp"),
      endpoint = c("mean_exp", "mean_sd_exp", "mean_nexp", "mean_sd_nexp"),
      change   = c("mean_change_exp", "mean_change_sd_exp", "mean_change_nexp", "mean_change_sd_nexp")
    )
    sid <- if ("study_id" %in% colnames(x)) as.character(x[["study_id"]]) else rep(NA_character_, n)
    for (bl_name in names(stat_blocks)) {
      cols <- stat_blocks[[bl_name]]
      cols <- cols[cols %in% colnames(x)]
      if (length(cols) < 3) next
      mat <- suppressWarnings(matrix(as.numeric(as.matrix(x[, cols, drop = FALSE])),
                                     nrow = n, dimnames = list(NULL, cols)))
      complete <- which(rowSums(!is.finite(mat)) == 0)
      if (length(complete) < 2) next
      fp <- vapply(complete, function(i)
        paste(format(mat[i, ], digits = 15, scientific = FALSE), collapse = "|"),
        character(1))
      for (key in unique(fp[duplicated(fp)])) {
        grp  <- complete[fp == key]
        vals <- mat[grp[1], ]
        # require >= templated_min_match non-integer values
        if (sum(vals != round(vals)) < templated_min_match) next
        for (i in grp) {
          others <- setdiff(grp, i)
          others <- others[is.na(sid[i]) | is.na(sid[others]) | sid[others] != sid[i]]
          if (length(others) == 0) next
          row_issues[[i]] <- c(row_issues[[i]], sprintf(
            paste0("[INFO] Identical %s summary statistics shared with %s (%s). ",
                   "Independent studies rarely reproduce multi-decimal means/SDs exactly; ",
                   "verify these rows are not templated, cloned, or extracted from the same source"),
            bl_name, paste(.row_ref(others, sid), collapse = ", "),
            paste(sprintf("'%s'=%s", cols, format(vals, scientific = FALSE)), collapse = ", ")))
        }
      }
    }
  }

  issues <- vapply(row_issues, function(v) {
    if (length(v) == 0) return("")
    paste(v, collapse = "; ")
  }, character(1))

  if (verbose) {
    any_bad <- which(nchar(issues) > 0)
    if (length(any_bad) > 0) {
      msgs <- paste0("  Row ", any_bad, ": ", issues[any_bad])
      message("Input data validation:\n", paste(msgs, collapse = "\n"))
    }
  }

  list(data = x, issues = issues)
}


#########

#' Format a method suffix like " (from means_sd)"
#' @param method character scalar or NA/NULL
#' @return character scalar (empty string if method is unavailable)
#' @noRd
.method_suffix <- function(method) {
  if (is.null(method) || is.na(method) || !nzchar(method)) return("")
  paste0(" (from ", method, ")")
}

#' Format a min/max method suffix for cross-method flags
#' @param min_info character scalar or NA/NULL
#' @param max_info character scalar or NA/NULL
#' @return character scalar like " - min: means_sd, max: student_t"
#' @noRd
.minmax_suffix <- function(min_info, max_info) {
  parts <- character(0)
  if (!is.null(min_info) && !is.na(min_info) && nzchar(min_info))
    parts <- c(parts, paste0("min: ", min_info))
  if (!is.null(max_info) && !is.na(max_info) && nzchar(max_info))
    parts <- c(parts, paste0("max: ", max_info))
  if (length(parts) == 0) return("")
  paste0(" - ", paste(parts, collapse = ", "))
}

#' Format a cross-row reference as "study_id (row N)" or "row N"
#'
#' Used by cross-row flags (V23, Category H) so a referenced study can be
#' identified by its study_id without counting rows. Falls back to the bare
#' row number when no study_id is available for that row.
#'
#' @param idx integer vector of row indices to reference
#' @param study_id character vector of per-row study IDs (or NULL), indexed by idx
#' @return character vector of formatted references, same length as idx
#' @noRd
.row_ref <- function(idx, study_id = NULL) {
  vapply(idx, function(i) {
    sid_i <- if (!is.null(study_id) && !is.na(i) &&
                 i >= 1 && i <= length(study_id)) {
      as.character(study_id[i])
    } else {
      NA_character_
    }
    if (!is.na(sid_i) && nzchar(sid_i)) {
      sprintf("%s (row %d)", sid_i, as.integer(i))
    } else {
      sprintf("row %d", as.integer(i))
    }
  }, character(1))
}

#' TRUE when every multi-method row has min ~ -max (sign-reversal convention,
#' min ~ 1/max on the natural ratio scale)
#'
#' @param n_estimations numeric vector, number of estimation methods per row
#' @param min_es,max_es numeric vectors of min/max cross-method ES
#' @param on_exp_scale logical, TRUE when min/max are natural-scale ratios
#' @return logical scalar: TRUE if all multi-method rows are reversals
#' @noRd
.rows_all_reversed <- function(n_estimations, min_es, max_es, on_exp_scale = FALSE) {
  if (is.null(min_es) || is.null(max_es)) return(FALSE)
  multi <- which(!is.na(n_estimations) & n_estimations >= 2 &
                 !is.na(min_es) & !is.na(max_es) &
                 is.finite(min_es) & is.finite(max_es))
  if (length(multi) == 0) return(FALSE)
  rev <- vapply(multi, function(i) {
    lo <- min_es[i]; hi <- max_es[i]
    if (on_exp_scale) {
      if (lo <= 0 || hi <= 0) return(FALSE)
      lo <- log(lo); hi <- log(hi)
    }
    mag <- max(abs(lo), abs(hi))
    mag > 0 && abs(lo + hi) < 0.01 * mag
  }, logical(1))
  all(rev)
}


#########

#' Check for numeric integrity issues (Inf, NaN, negative SE, CI problems)
#' @param es numeric vector of effect sizes
#' @param se numeric vector of standard errors
#' @param ci_lo numeric vector of CI lower bounds
#' @param ci_up numeric vector of CI upper bounds
#' @param info_used character vector of method names (optional, for traceability)
#' @return list of character vectors (one per row)
#' @noRd
.flag_numeric_integrity <- function(es, se, ci_lo, ci_up, info_used = NULL,
                                     measure = NULL, exp = FALSE,
                                     n_total = NULL, n_exp = NULL, n_nexp = NULL,
                                     enable_informational = FALSE,
                                     r_defaulted = NULL,
                                     prop_to_es = "raw",
                                     smd_denom = NULL) {
  n <- length(es)
  flags <- vector("list", n)
  for (i in seq_len(n)) flags[[i]] <- character(0)

  for (i in seq_len(n)) {
    msuf <- if (!is.null(info_used)) .method_suffix(info_used[i]) else ""
    # r-sensitivity note for the SE-driven A6 CI-width check (see .r_sensitive_note)
    rsuf <- if (!is.null(r_defaulted) && length(r_defaulted) >= i) {
      .r_sensitive_note(r_defaulted[i])
    } else {
      ""
    }
    # A1: es non-finite
    if (is.nan(es[i]) || (!is.na(es[i]) && is.infinite(es[i]))) {
      flags[[i]] <- c(flags[[i]], paste0("[INVALID] ES is non-finite or undefined", msuf))
    }
    # A2: SE is Inf or NaN
    if (is.nan(se[i]) || (!is.na(se[i]) && is.infinite(se[i]))) {
      flags[[i]] <- c(flags[[i]], paste0("[INVALID] SE is non-finite or undefined", msuf))
    }
    # A3: SE is negative
    if (!is.na(se[i]) && is.finite(se[i]) && se[i] < 0) {
      flags[[i]] <- c(flags[[i]], paste0("[INVALID] Negative SE: ", round(se[i], 3), msuf))
    }
    # A4: CI lower > CI upper
    if (!is.na(ci_lo[i]) && !is.na(ci_up[i]) &&
        is.finite(ci_lo[i]) && is.finite(ci_up[i]) &&
        ci_lo[i] > ci_up[i]) {
      flags[[i]] <- c(flags[[i]], paste0("[INVALID] Inverted CI: lower > upper", msuf))
    }
    # A5: ES not within CI bounds
    if (!is.na(es[i]) && !is.na(ci_lo[i]) && !is.na(ci_up[i]) &&
        is.finite(es[i]) && is.finite(ci_lo[i]) && is.finite(ci_up[i]) &&
        ci_lo[i] <= ci_up[i]) {
      if (es[i] < ci_lo[i] || es[i] > ci_up[i]) {
        flags[[i]] <- c(flags[[i]], paste0("[INVALID] ES outside its CI bounds", msuf))
      }
    }

    # A6: ci width vs se
    # skipped for nnt (discontinuous CI) and exp=TRUE (se stays on the log scale)
    skip_a6 <- identical(measure, "nnt") || exp
    if (!skip_a6 &&
        !is.na(es[i]) && !is.na(se[i]) && !is.na(ci_lo[i]) && !is.na(ci_up[i]) &&
        is.finite(es[i]) && is.finite(se[i]) && is.finite(ci_lo[i]) && is.finite(ci_up[i]) &&
        se[i] > 0 && ci_lo[i] <= ci_up[i]) {
      ci_width <- ci_up[i] - ci_lo[i]
      # package CIs: qt for d/g/md/r and dw/gw/mdw, qnorm otherwise
      # user CIs: checked against z width, t width also accepted
      qt_measures <- c("d", "g", "dw", "gw", "md", "mdw", "r")
      n_i <- if (!is.null(n_total)) n_total[i] else NA_real_
      # paired measures: df = n_pairs - 1 (n_total double counts the subjects)
      within_measures <- c("dw", "gw", "mdw")
      n_exp_i <- if (!is.null(n_exp)) n_exp[i] else NA_real_
      n_nexp_i <- if (!is.null(n_nexp)) n_nexp[i] else NA_real_
      if (!is.null(measure) && measure %in% within_measures &&
          !is.na(n_exp_i) && is.finite(n_exp_i) && n_exp_i > 2) {
        t_df <- n_exp_i - 1
        n_eff <- n_exp_i
      } else if (!is.na(n_i) && n_i > 4) {
        t_df <- n_i - 2
        n_eff <- n_i
      } else {
        t_df <- NA_real_
        n_eff <- n_i
      }
      is_user_es <- !is.null(info_used) && !is.na(info_used[i]) &&
                    grepl("^user_(es|input)", info_used[i])
      # P7: Glass rows (smd_denom = control/control_robust) build their d/g CIs
      # on the control-arm df (SMD1 convention), not the pooled N - 2. The Glass df
      # applies ONLY when the SELECTED estimate actually came from the endpoint-means
      # family -- the only methods that honour smd_denom. When the hierarchy picked a
      # non-means method (cohen_d, etasq, t/F, ANCOVA, medians...), the CI is built on
      # the pooled N - 2 regardless of the smd_denom input column, so keying the Glass
      # df off the raw input alone would false-fire the A6 CI-width flag on valid rows.
      glass_row <- !is_user_es && !is.null(smd_denom) &&
        length(smd_denom) >= i && !is.na(smd_denom[i]) &&
        smd_denom[i] %in% c("control", "control_robust") &&
        !is.null(measure) && measure %in% c("d", "g") &&
        !is.null(info_used) && !is.na(info_used[i]) &&
        info_used[i] %in% c("means_sd", "means_se", "means_ci") &&
        !is.na(n_nexp_i) && is.finite(n_nexp_i) && n_nexp_i > 2
      if (glass_row) t_df <- n_nexp_i - 1
      if (is_user_es) {
        z_crit <- stats::qnorm(0.975)
      } else if (!is.null(measure) && measure %in% qt_measures &&
                 !is.na(t_df)) {
        z_crit <- stats::qt(0.975, t_df)
      } else {
        z_crit <- stats::qnorm(0.975)
      }
      expected_width <- 2 * z_crit * se[i]
      # ~10% at N=9, ~5% at N=36
      tol <- if (!is.na(n_eff) && n_eff > 0) max(0.05, 0.30 / sqrt(n_eff)) else 0.10
      # P8: raw-scale proportion CIs are deliberately clamped to [0, 1] by
      # es_from_prop_single_group(); when a bound sits at the clamp, compare
      # against the [0, 1]-clipped expected width instead of 2 * z * se.
      prop_clamped <- !is_user_es && identical(measure, "prop") &&
        identical(prop_to_es, "raw") &&
        (isTRUE(all.equal(ci_lo[i], 0)) || isTRUE(all.equal(ci_up[i], 1)))
      if (prop_clamped) {
        expected_width <- min(es[i] + z_crit * se[i], 1) -
          max(es[i] - z_crit * se[i], 0)
      }
      fires_a6 <- expected_width > 0 &&
        abs(ci_width - expected_width) / expected_width > tol
      # P6: package-computed md CIs legitimately use any df between the Welch
      # minimum (min(n1, n2) - 1; es_from_means_sd) and the pooled N - 2
      # (es_from_md_*, ANCOVA md) -- accept the whole band before flagging.
      if (fires_a6 && !is_user_es && identical(measure, "md") &&
          !is.na(n_exp_i) && !is.na(n_nexp_i) &&
          is.finite(n_exp_i) && is.finite(n_nexp_i) &&
          min(n_exp_i, n_nexp_i) > 2 && !is.na(t_df)) {
        w_pooled <- 2 * stats::qt(0.975, n_exp_i + n_nexp_i - 2) * se[i]
        w_welch_min <- 2 * stats::qt(0.975, min(n_exp_i, n_nexp_i) - 1) * se[i]
        fires_a6 <- ci_width < (1 - tol) * w_pooled ||
          ci_width > (1 + tol) * w_welch_min
        # report the closest band edge in the message
        expected_width <- if (ci_width < w_pooled) w_pooled else w_welch_min
      }
      if (fires_a6) {
        matches_t <- FALSE
        if (is_user_es && !is.null(measure) && measure %in% qt_measures &&
            !is.na(t_df)) {
          t_width <- 2 * stats::qt(0.975, t_df) * se[i]
          matches_t <- t_width > 0 &&
            abs(ci_width - t_width) / t_width <= tol
        }
        if (matches_t) {
          if (enable_informational) {
            flags[[i]] <- c(flags[[i]],
              sprintf("[INFO] CI width matches a t-based CI rather than Wald-z: z-expected %.3f, actual %.3f. Benign if the source built t CIs; if the source used Wald CIs, check the SE formula (e.g. paired vs independent)%s",
                      expected_width, ci_width, msuf))
          }
        } else {
          flags[[i]] <- c(flags[[i]],
            sprintf("[DISCORDANT] CI width inconsistent with SE: expected %.3f, actual %.3f%s%s",
                    expected_width, ci_width, msuf, rsuf))
        }
      }
    }
  }
  return(flags)
}


#########

#' Check for theoretically impossible values based on the effect size measure
#' @param es numeric vector of effect sizes
#' @param se numeric vector of standard errors
#' @param ci_lo numeric vector of CI lower bounds
#' @param ci_up numeric vector of CI upper bounds
#' @param measure character, the effect size measure
#' @param exp logical, whether OR/RR/IRR are on the exp (natural) scale
#' @param info_used character vector of method names (optional, for traceability)
#' @return list of character vectors (one per row)
#' @noRd
.flag_bounds_violations <- function(es, se, ci_lo, ci_up, measure, exp,
                                     info_used = NULL,
                                     baseline_risk = NULL,
                                     baseline_rate = NULL,
                                     alpha_to_es = "bonett",
                                     icc_to_es = "bonett",
                                     prop_to_es = "raw") {
  n <- length(es)
  flags <- vector("list", n)
  for (i in seq_len(n)) flags[[i]] <- character(0)

  for (i in seq_len(n)) {
    if (is.na(es[i])) next
    msuf <- if (!is.null(info_used)) .method_suffix(info_used[i]) else ""

    # B1: Correlation outside [-1, 1]
    if (measure == "r" && is.finite(es[i]) && abs(es[i]) > 1) {
      flags[[i]] <- c(flags[[i]],
        paste0("[INVALID] r outside [-1, 1]: r = ", round(es[i], 3), msuf))
    }

    # B2: OR/RR/IRR non-positive on natural scale
    if (exp && measure %in% c("logor", "logrr", "logirr", "loghr") &&
        is.finite(es[i]) && es[i] <= 0) {
      flags[[i]] <- c(flags[[i]],
        paste0("[INVALID] ", toupper(sub("log", "", measure)),
               " is non-positive: ", round(es[i], 3), msuf))
    }

    # B2b: CI bound non-positive on natural scale (Riley 2023)
    if (exp && measure %in% c("logor", "logrr", "logirr", "loghr")) {
      bound_problems <- character(0)
      if (!is.na(ci_lo[i]) && is.finite(ci_lo[i]) && ci_lo[i] <= 0) {
        bound_problems <- c(bound_problems,
                            paste0("CI lower = ", round(ci_lo[i], 3)))
      }
      if (!is.na(ci_up[i]) && is.finite(ci_up[i]) && ci_up[i] <= 0) {
        bound_problems <- c(bound_problems,
                            paste0("CI upper = ", round(ci_up[i], 3)))
      }
      if (length(bound_problems) > 0) {
        flags[[i]] <- c(flags[[i]],
          paste0("[INVALID] ", toupper(sub("log", "", measure)),
                 " CI bound non-positive: ",
                 paste(bound_problems, collapse = ", "), msuf))
      }
    }

    # B3: NNT magnitude < 1 (mathematically impossible: cannot treat fewer
    # than one person to prevent one event)
    if (measure == "nnt" && is.finite(es[i]) &&
        abs(es[i]) < 1 && es[i] != 0) {
      flags[[i]] <- c(flags[[i]],
        paste0("[INVALID] NNT magnitude < 1: NNT = ", round(es[i], 3), msuf))
    }

    # B4: Proportion outside [0, 1] (raw scale only - transformed scales are unbounded)
    if (measure == "prop" && prop_to_es == "raw" && is.finite(es[i]) &&
        (es[i] < 0 || es[i] > 1)) {
      flags[[i]] <- c(flags[[i]],
        paste0("[INVALID] Proportion outside [0, 1]: prop = ", round(es[i], 3), msuf))
    }

    # B5: NNT below minimum possible given baseline risk
    if (measure == "nnt" && is.finite(es[i]) && abs(es[i]) > 0) {
      br <- if (!is.null(baseline_risk) && length(baseline_risk) >= i) baseline_risk[i] else NA
      brate <- if (!is.null(baseline_rate) && length(baseline_rate) >= i) baseline_rate[i] else NA

      if (!is.na(br) && br > 0 && br < 1) {
        min_nnt <- 1 / br
        if (abs(es[i]) < min_nnt) {
          flags[[i]] <- c(flags[[i]],
            paste0("[INVALID] Minimum possible NNT = ", round(min_nnt, 1),
                   " (baseline_risk = ", round(br * 100, 1), "%)",
                   "; reported |NNT| = ", round(abs(es[i]), 3), msuf))
        }
      }
      if (!is.na(brate) && brate > 0) {
        min_nnt_pt <- 1 / brate
        if (abs(es[i]) < min_nnt_pt) {
          flags[[i]] <- c(flags[[i]],
            paste0("[INVALID] Minimum possible person-time NNT = ", round(min_nnt_pt, 1),
                   " (baseline_rate = ", round(brate, 4), ")",
                   "; reported |NNT| = ", round(abs(es[i]), 3), msuf))
        }
      }
    }

    # B6: RD outside [-1, 1]
    if (measure == "rd" && is.finite(es[i]) && abs(es[i]) > 1) {
      flags[[i]] <- c(flags[[i]],
        paste0("[INVALID] RD outside [-1, 1]: rd = ", round(es[i], 3), msuf))
    }

    # B6b: RD CI bound outside [-1, 1]
    if (measure == "rd") {
      bound_problems <- character(0)
      if (!is.na(ci_lo[i]) && is.finite(ci_lo[i]) && abs(ci_lo[i]) > 1) {
        bound_problems <- c(bound_problems,
                            paste0("CI lower = ", round(ci_lo[i], 3)))
      }
      if (!is.na(ci_up[i]) && is.finite(ci_up[i]) && abs(ci_up[i]) > 1) {
        bound_problems <- c(bound_problems,
                            paste0("CI upper = ", round(ci_up[i], 3)))
      }
      if (length(bound_problems) > 0) {
        flags[[i]] <- c(flags[[i]],
          paste0("[INVALID] RD CI bound outside [-1, 1]: ",
                 paste(bound_problems, collapse = ", "), msuf))
      }
    }

    # B7: alpha bounds
    # bonett scale: es > 0 means negative alpha, possible but suspect
    if (measure == "alpha" && is.finite(es[i])) {
      if (identical(alpha_to_es, "bonett")) {
        if (es[i] > 0) {
          flags[[i]] <- c(flags[[i]],
            paste0("[UNUSUAL] Bonett alpha ln(1-alpha) = ", round(es[i], 3),
                   " > 0 implies a negative Cronbach's alpha - possible but indicates serious measurement problems; verify extraction", msuf))
        }
      } else if (es[i] > 1) {
        flags[[i]] <- c(flags[[i]],
          paste0("[INVALID] Alpha exceeds 1: alpha = ", round(es[i], 3), msuf))
      }
      # B7b: raw-scale Wald CI escaping the upper bound (the point estimate is
      # valid, the symmetric interval is not; the bonett scale cannot overshoot)
      if (!identical(alpha_to_es, "bonett") &&
          !is.na(ci_up[i]) && is.finite(ci_up[i]) && ci_up[i] > 1 &&
          es[i] <= 1) {
        flags[[i]] <- c(flags[[i]],
          paste0("[UNUSUAL] Raw-scale alpha CI upper bound exceeds 1 (",
                 round(ci_up[i], 3),
                 ") - the symmetric Wald interval escapes the parameter space; consider alpha_to_es = 'bonett'", msuf))
      }
    }

    # B8: ICC bounds
    if (measure == "icc" && is.finite(es[i])) {
      if (identical(icc_to_es, "bonett")) {
        if (es[i] > 0) {
          flags[[i]] <- c(flags[[i]],
            paste0("[UNUSUAL] Bonett ICC ln(1-ICC) = ", round(es[i], 3),
                   " > 0 implies a negative ICC - possible but indicates serious measurement problems; verify extraction", msuf))
        }
      } else if (es[i] > 1) {
        flags[[i]] <- c(flags[[i]],
          paste0("[INVALID] ICC exceeds 1: icc = ", round(es[i], 3), msuf))
      }
      # B8b: raw-scale Wald CI escaping the parameter space [-1, 1]
      if (!identical(icc_to_es, "bonett") &&
          !is.na(es[i]) && es[i] <= 1 && es[i] >= -1) {
        icc_bound_problems <- character(0)
        if (!is.na(ci_up[i]) && is.finite(ci_up[i]) && ci_up[i] > 1) {
          icc_bound_problems <- c(icc_bound_problems,
                                  paste0("CI upper = ", round(ci_up[i], 3)))
        }
        if (!is.na(ci_lo[i]) && is.finite(ci_lo[i]) && ci_lo[i] < -1) {
          icc_bound_problems <- c(icc_bound_problems,
                                  paste0("CI lower = ", round(ci_lo[i], 3)))
        }
        if (length(icc_bound_problems) > 0) {
          flags[[i]] <- c(flags[[i]],
            paste0("[UNUSUAL] Raw-scale ICC CI bound outside [-1, 1]: ",
                   paste(icc_bound_problems, collapse = ", "),
                   " - the symmetric Wald interval escapes the parameter space; consider icc_to_es = 'bonett'", msuf))
        }
      }
    }
  }
  return(flags)
}


#########

#' Check for statistically unusual but not impossible values
#' @param es numeric vector of effect sizes
#' @param se numeric vector of standard errors
#' @param measure character, the effect size measure
#' @param opts list of flag options (thresholds)
#' @param n_sample numeric vector of sample sizes (can contain NA)
#' @param info_used character vector of method names (optional, for traceability)
#' @param n_exp numeric vector of experimental group sizes (optional, for C10)
#' @param n_nexp numeric vector of control group sizes (optional, for C10)
#' @return list of character vectors (one per row)
#' @noRd
.flag_plausibility <- function(es, se, measure, opts, n_sample,
                                info_used = NULL,
                                n_exp = NULL, n_nexp = NULL,
                                exp = FALSE,
                                prop_to_es = "raw") {
  n <- length(es)
  flags <- vector("list", n)
  for (i in seq_len(n)) flags[[i]] <- character(0)

  for (i in seq_len(n)) {
    msuf <- if (!is.null(info_used)) .method_suffix(info_used[i]) else ""

    # C1: SMD magnitude too large
    if (measure %in% c("d", "g", "dw", "gw") &&
        !is.na(es[i]) && is.finite(es[i]) &&
        abs(es[i]) > opts$smd_max) {
      flags[[i]] <- c(flags[[i]],
        paste0("[UNUSUAL] Large SMD: |", measure, "| = ",
               round(abs(es[i]), 3), " (threshold: ", opts$smd_max, ")", msuf))
    }

    # C2: SE equals zero
    if (!is.na(se[i]) && is.finite(se[i]) && se[i] == 0) {
      flags[[i]] <- c(flags[[i]], paste0("[INVALID] SE is zero", msuf))
    }

    # C4: Correlation magnitude very high
    if (measure == "r" &&
        !is.na(es[i]) && is.finite(es[i]) &&
        abs(es[i]) > opts$r_max && abs(es[i]) <= 1) {
      flags[[i]] <- c(flags[[i]],
        paste0("[UNUSUAL] High correlation: |r| = ",
               round(abs(es[i]), 3), " (threshold: ", opts$r_max, ")", msuf))
    }

    # C5: Log OR/RR/IRR/HR magnitude very large
    # threshold is on the log scale, transform back when exp = TRUE
    if (measure %in% c("logor", "logrr", "logirr", "loghr") &&
        !is.na(es[i]) && is.finite(es[i])) {
      log_val <- if (exp) {
        if (es[i] > 0) log(es[i]) else NA_real_  # es <= 0 is handled by B2
      } else {
        es[i]
      }
      if (!is.na(log_val) && abs(log_val) > opts$log_or_max) {
        flags[[i]] <- c(flags[[i]],
          paste0("[UNUSUAL] Large log ", toupper(sub("log", "", measure)),
                 ": |value| = ", round(abs(log_val), 3),
                 " (threshold: ", opts$log_or_max, ")", msuf))
      }
    }

    # C6: Very small sample size (informational - gated by enable_informational)
    if (isTRUE(opts$enable_informational) &&
        !is.na(n_sample[i]) && is.finite(n_sample[i]) &&
        n_sample[i] < opts$n_min) {
      flags[[i]] <- c(flags[[i]],
        paste0("[INFO] Small sample size: n = ", n_sample[i],
               " (threshold: ", opts$n_min, ")"))
    }

    # C7: Near-perfect Cronbach's alpha (raw scale only: ES in (0, 1])
    if (measure == "alpha" &&
        !is.na(es[i]) && is.finite(es[i]) &&
        es[i] > 0 && es[i] <= 1 && es[i] > opts$alpha_max) {
      flags[[i]] <- c(flags[[i]],
        paste0("[UNUSUAL] Near-perfect alpha: ", round(es[i], 3),
               " (threshold: ", opts$alpha_max, ")", msuf))
    }

    # C8: Near-perfect ICC (raw scale only: ES in (0, 1])
    if (measure == "icc" &&
        !is.na(es[i]) && is.finite(es[i]) &&
        es[i] > 0 && es[i] <= 1 && es[i] > opts$icc_max) {
      flags[[i]] <- c(flags[[i]],
        paste0("[UNUSUAL] Near-perfect ICC: ", round(es[i], 3),
               " (threshold: ", opts$icc_max, ")", msuf))
    }

    # C9: Extreme transformed proportion (logit or Freeman-Tukey scale)
    # On raw scale, B4 catches out-of-bounds. This flag targets transformed scales only.
    if (measure == "prop" && prop_to_es != "raw" &&
        !is.na(es[i]) && is.finite(es[i])) {
      is_extreme <- FALSE
      # Logit scale: |logit| > 6 corresponds to p < 0.0025 or p > 0.9975
      if (prop_to_es == "logit" && abs(es[i]) > 6) is_extreme <- TRUE
      # Freeman-Tukey scale: bounded to [0, pi/2 ~ 1.5708].
      # Values near 0 or near pi/2 indicate extreme proportions.
      # es < 0.05 ~ p < 0.002, es > 1.52 ~ p > 0.998
      if (prop_to_es == "freeman_tukey" &&
          (es[i] < 0.05 || es[i] > 1.52)) is_extreme <- TRUE
      if (is_extreme) {
        flags[[i]] <- c(flags[[i]],
          paste0("[UNUSUAL] Extreme transformed proportion: value = ",
                 round(es[i], 3), msuf))
      }
    }

    # C10: Extreme group size imbalance (two-group measures only)
    if (measure %in% c("d", "g", "dw", "gw", "logor", "logrr", "nnt", "rd") &&
        !is.null(n_exp) && !is.null(n_nexp) &&
        length(n_exp) >= i && length(n_nexp) >= i &&
        !is.na(n_exp[i]) && !is.na(n_nexp[i]) &&
        is.finite(n_exp[i]) && is.finite(n_nexp[i]) &&
        n_exp[i] > 0 && n_nexp[i] > 0) {
      grp_ratio <- max(n_exp[i], n_nexp[i]) / min(n_exp[i], n_nexp[i])
      if (grp_ratio > opts$group_ratio_max) {
        flags[[i]] <- c(flags[[i]],
          sprintf("[UNUSUAL] Extreme group size imbalance: %d vs %d (ratio: %.1f, threshold: %g)",
                  as.integer(n_exp[i]), as.integer(n_nexp[i]),
                  grp_ratio, opts$group_ratio_max))
      }
    }
  }
  return(flags)
}


#########

#' Detect cross-row outliers using IQR method
#' @param es numeric vector of effect sizes
#' @param se numeric vector of standard errors
#' @param opts list of flag options (thresholds)
#' @param info_used character vector of method names (optional, for traceability)
#' @param measure character, the effect size measure (optional, for value labels)
#' @return list of character vectors (one per row)
#' @noRd
.flag_cross_row_outliers <- function(es, se, opts, info_used = NULL,
                                      measure = NULL, n_total = NULL,
                                      n_exp = NULL, n_nexp = NULL,
                                      sd_exp = NULL, sd_nexp = NULL,
                                      suffix = "", exp = FALSE) {
  n <- length(es)
  flags <- vector("list", n)
  for (i in seq_len(n)) flags[[i]] <- character(0)

  if (!isTRUE(opts$enable_cross_row)) return(flags)

  # outlier decision on the log scale when exp = TRUE, messages keep the natural value
  log_measures <- c("logor", "logrr", "logirr", "loghr")
  on_exp_scale <- isTRUE(exp) && !is.null(measure) && measure %in% log_measures
  es_work <- es
  if (on_exp_scale) {
    es_work <- ifelse(!is.na(es) & is.finite(es) & es > 0, log(es), NA_real_)
  }

  # Label for the ES value (e.g. "d" or "ES"; natural-scale name when exp=TRUE)
  es_label <- if (!is.null(measure) && nzchar(measure)) measure else "ES"
  if (on_exp_scale) es_label <- sub("^log", "", es_label)

  # minimum deviation from median, avoids flagging homogeneous pools
  if (!is.null(opts$outlier_min_deviation)) {
    min_dev <- opts$outlier_min_deviation
  } else if (is.null(measure) || !nzchar(measure)) {
    min_dev <- 0
  } else {
    min_dev <- switch(measure,
      "d" = , "g" = 1.0, # 1 SD
      "logor" = , "logrr" = , "logirr" = , "loghr" = 1.0, # log scale
      "r" = 0.3,
      "z" = 0.5,
      "rd" = 0.3,
      0 # no floor for md, nnt, prop, alpha, icc
    )
  }

  # D1: ES outlier (decision on es_work - log scale for exp-displayed ratios)
  es_valid <- es_work[!is.na(es_work) & is.finite(es_work)]
  if (length(es_valid) >= 4) {
    es_med <- stats::median(es_valid)
    es_iqr <- stats::IQR(es_valid)
    if (es_iqr > 0) {
      for (i in seq_len(n)) {
        if (!is.na(es_work[i]) && is.finite(es_work[i]) &&
            abs(es_work[i] - es_med) > opts$iqr_mult * es_iqr &&
            abs(es_work[i] - es_med) > min_dev) {
          msuf <- if (!is.null(info_used)) .method_suffix(info_used[i]) else ""
          flags[[i]] <- c(flags[[i]],
            paste0("[UNUSUAL] ES outlier, IQR method: ", es_label, " = ",
                   round(es[i], 3), msuf))
        }
      }
    }
  }

  # D2: SE outlier. normalise SE before the IQR: d/g by expected SE,
  # logOR/logRR by 2x2 reconstruction, else sqrt(N)
  se_for_iqr <- se
  if (measure %in% c("d", "g") && !is.null(n_exp) && !is.null(n_nexp)) {
    has_both <- !is.na(n_exp) & !is.na(n_nexp) & is.finite(n_exp) & is.finite(n_nexp) &
                n_exp > 0 & n_nexp > 0 & !is.na(es) & is.finite(es) & !is.na(se) & is.finite(se)
    if (any(has_both)) {
      expected_se <- sqrt(1/n_exp[has_both] + 1/n_nexp[has_both] +
                          es[has_both]^2 / (2 * (n_exp[has_both] + n_nexp[has_both])))
      expected_se[expected_se == 0] <- NA_real_
      se_for_iqr[has_both] <- se[has_both] / expected_se
    }
    # Balanced-arms 2/sqrt(N) fallback for rows WITHOUT both arm sizes. This must run
    # whether or not ANY row had arm sizes: a pure pearson_r/fisher_z/spearman_r d or g
    # pool carries only n_sample (n_exp/n_nexp stay NA in raw_data), so nesting this inside
    # if(any(has_both)) left every row with its raw, N-dependent SE and falsely flagged a
    # legitimately small-N study as an SE outlier. Mirrors the branch-level no_data
    # fallback in the logOR/logRR branch below.
    not_norm <- !has_both & !is.na(se) & is.finite(se)
    if (any(not_norm)) {
      if (!is.null(n_total)) {
        approx <- not_norm & !is.na(n_total) & is.finite(n_total) & n_total > 0
        se_for_iqr[approx] <- se[approx] / (2 / sqrt(n_total[approx]))
        # only null out un-normalisable rows once SOME row has been normalised, to avoid
        # collapsing the whole pool to NA (matches the logOR branch's any()-guard)
        if (any(has_both) || any(approx)) se_for_iqr[not_norm & !approx] <- NA_real_
      } else if (any(has_both)) {
        se_for_iqr[not_norm] <- NA_real_
      }
    }
  } else if (measure %in% c("logor", "logrr") &&
             !is.null(n_exp) && !is.null(n_nexp) &&
             suffix != "_adjusted") {
    # 2x2 reconstruction accounts for event-rate variation across studies
    # skip adjusted estimates: multivariable SEs are larger than 2x2-implied SEs
    # 4/sqrt(N) ~ SE_logOR at a 50% event rate (puts fallback rows on the same ~1 scale)
    balanced_se_const <- 4
    has_data <- !is.na(n_exp) & !is.na(n_nexp) & is.finite(n_exp) & is.finite(n_nexp) &
                n_exp > 1 & n_nexp > 1 & !is.na(es) & is.finite(es) &
                !is.na(se) & is.finite(se) & se > 0
    if (any(has_data)) {
      for (j in which(has_data)) {
        # es is already the natural-scale OR when exp=TRUE
        or_j <- if (on_exp_scale) es[j] else exp(es[j])
        var_j <- se[j]^2
        n_exp_j <- as.integer(round(n_exp[j]))
        n_nexp_j <- as.integer(round(n_nexp[j]))
        # Skip if group sizes are too large (simulation is O(n_exp))
        if (n_exp_j > 5000 || n_nexp_j > 5000) {
          # Fallback: sqrt(N) normalization, scaled to the reconstruction (~1) pool
          se_for_iqr[j] <- se[j] * sqrt(n_exp_j + n_nexp_j) / balanced_se_const
          next
        }
        recon <- tryCatch(
          .estimate_n_from_or_and_n_exp(or_j, var_j, n_exp_j, n_nexp_j),
          error = function(e) NULL
        )
        if (!is.null(recon) && nrow(recon) > 0 &&
            all(is.finite(c(recon$n_cases_exp[1], recon$n_cases_nexp[1],
                            recon$n_controls_exp[1], recon$n_controls_nexp[1])))) {
          a <- recon$n_cases_exp[1]; b <- recon$n_cases_nexp[1]
          c_val <- recon$n_controls_exp[1]; d <- recon$n_controls_nexp[1]
          if (measure == "logor") {
            # logOR SE = sqrt(1/a + 1/b + 1/c + 1/d)
            cells <- c(a, b, c_val, d)
            if (any(cells == 0)) cells <- cells + 0.5  # continuity correction
            exp_se <- sqrt(sum(1 / cells))
          } else {
            # logRR SE = sqrt(1/a - 1/(a+c) + 1/b - 1/(b+d))
            if (a > 0 && b > 0 && (a + c_val) > 0 && (b + d) > 0) {
              exp_se <- sqrt(1/a - 1/(a + c_val) + 1/b - 1/(b + d))
            } else {
              exp_se <- NA_real_
            }
          }
          if (!is.na(exp_se) && exp_se > 0) {
            se_for_iqr[j] <- se[j] / exp_se
          } else {
            # Fallback (scaled to the reconstruction ~1 pool)
            se_for_iqr[j] <- se[j] * sqrt(n_exp_j + n_nexp_j) / balanced_se_const
          }
        } else {
          # Fallback (scaled to the reconstruction ~1 pool)
          se_for_iqr[j] <- se[j] * sqrt(n_exp_j + n_nexp_j) / balanced_se_const
        }
      }
    }
    # rows without arm counts: sqrt(N) fallback; no N at all -> excluded
    no_data <- !has_data & !is.na(se) & is.finite(se)
    if (any(no_data)) {
      if (!is.null(n_total)) {
        has_n <- no_data & !is.na(n_total) & is.finite(n_total) & n_total > 0
        se_for_iqr[has_n] <- se[has_n] * sqrt(n_total[has_n]) / balanced_se_const
        if (any(has_data) || any(has_n)) se_for_iqr[no_data & !has_n] <- NA_real_
      } else if (any(has_data)) {
        se_for_iqr[no_data] <- NA_real_
      }
    }
  } else if (!is.null(n_total)) {
    has_n <- !is.na(n_total) & is.finite(n_total) & n_total > 0
    if (any(has_n)) {
      se_for_iqr[has_n] <- se[has_n] * sqrt(n_total[has_n])
      se_for_iqr[!has_n] <- NA_real_
    }
  }
  se_iqr_valid <- se_for_iqr[!is.na(se_for_iqr) & is.finite(se_for_iqr)]
  se_ratio_min <- opts$se_outlier_min_ratio
  if (length(se_iqr_valid) >= 4) {
    se_med <- stats::median(se_iqr_valid)
    se_iqr <- stats::IQR(se_iqr_valid)
    if (se_iqr > 0 && se_med > 0) {
      for (i in seq_len(n)) {
        if (!is.na(se_for_iqr[i]) && is.finite(se_for_iqr[i]) &&
            abs(se_for_iqr[i] - se_med) > opts$iqr_mult * se_iqr) {
          # require at least se_outlier_min_ratio x from the median
          ratio <- se_for_iqr[i] / se_med
          if (ratio > 1 / se_ratio_min && ratio < se_ratio_min) next
          msuf <- if (!is.null(info_used)) .method_suffix(info_used[i]) else ""
          flags[[i]] <- c(flags[[i]],
            paste0("[UNUSUAL] SE outlier, IQR method: SE = ",
                   round(se[i], 3), msuf))
        }
      }
    }
  }

  # D2b: ratio-to-median fallback, catches scale mixing the IQR misses
  if (length(se_iqr_valid) >= 4 && se_med > 0) {
    se_ratio_extreme <- opts$se_ratio_extreme
    # decision on the normalised SE, fold-difference displayed on the raw SE scale
    se_raw_med <- stats::median(se[!is.na(se) & is.finite(se) & se > 0])
    for (i in seq_len(n)) {
      if (any(grepl("SE outlier", flags[[i]]))) next
      if (!is.na(se_for_iqr[i]) && is.finite(se_for_iqr[i])) {
        ratio <- se_for_iqr[i] / se_med
        if (ratio < 1 / se_ratio_extreme || ratio > se_ratio_extreme) {
          msuf <- if (!is.null(info_used)) .method_suffix(info_used[i]) else ""
          if (!is.na(se_raw_med) && se_raw_med > 0 && se[i] < se_raw_med) {
            fold <- se_raw_med / se[i]
            cmp <- if (is.finite(fold)) {
              paste0(round(fold, 1), "x tighter than the cohort median SE (",
                     round(se_raw_med, 3), ")")
            } else {
              paste0("far tighter than the cohort median SE (",
                     round(se_raw_med, 3), ")")
            }
          } else if (!is.na(se_raw_med) && se_raw_med > 0) {
            cmp <- paste0(round(se[i] / se_raw_med, 1),
                          "x the cohort median SE (", round(se_raw_med, 3), ")")
          } else {
            cmp <- "far from the cohort SE distribution"
          }
          flags[[i]] <- c(flags[[i]],
            paste0("[UNUSUAL] SE outlier, ratio method: SE = ",
                   round(se[i], 3), " (", cmp, ")", msuf))
        }
      }
    }
  }

  # D3 (V27): spread = pooled arm SD or se*sqrt(N); flags SE entered as SD
  # md/mdw by default, d/g/dw/gw only via sd_outlier_smd (mixed instrument scales)
  sd_default <- c("md", "mdw")
  sd_optin   <- c("d", "g", "dw", "gw")
  sd_apply <- !is.null(measure) &&
    (measure %in% sd_default ||
     (isTRUE(opts$sd_outlier_smd) && measure %in% sd_optin))
  if (sd_apply) {
    K_sd <- if (!is.null(opts$sd_outlier_ratio)) opts$sd_outlier_ratio else 5
    # effective N: n_total when available, else n_exp (within-subject: n_nexp NA)
    n_eff <- if (!is.null(n_total)) n_total else rep(NA_real_, n)
    if (!is.null(n_exp)) {
      miss <- (is.na(n_eff) | !is.finite(n_eff)) & !is.na(n_exp) & is.finite(n_exp)
      n_eff[miss] <- n_exp[miss]
    }
    pooled_sd <- if (!is.null(sd_exp) && !is.null(sd_nexp)) {
      sqrt((sd_exp^2 + sd_nexp^2) / 2)
    } else rep(NA_real_, n)
    implied_sd <- if (!is.null(n_eff)) se * sqrt(n_eff) else rep(NA_real_, n)
    spread <- ifelse(!is.na(pooled_sd) & is.finite(pooled_sd) & pooled_sd > 0,
                     pooled_sd, implied_sd)
    sp_ok <- !is.na(spread) & is.finite(spread) & spread > 0
    if (sum(sp_ok) >= 4) {
      sp_med <- stats::median(spread[sp_ok])
      if (sp_med > 0) {
        for (i in seq_len(n)) {
          if (any(grepl("SE outlier", flags[[i]]))) next  # D2 already named this row
          if (sp_ok[i] && spread[i] < sp_med / K_sd) {
            msuf <- if (!is.null(info_used)) .method_suffix(info_used[i]) else ""
            flags[[i]] <- c(flags[[i]],
              paste0("[UNUSUAL] SD outlier: spread = ", round(spread[i], 3),
                     " is ", round(sp_med / spread[i], 1),
                     "x below the pool median (", round(sp_med, 3),
                     "); an SD this small for the scale can mean a standard error was entered as a standard deviation",
                     msuf))
          }
        }
      }
    }
  }

  return(flags)
}


#########

#' Flag rows when studies in the pool have conflicting significant directions
#'
#' Often a group labeling error rather than genuine heterogeneity; all rows on
#' both sides are flagged. A side qualifies by the count floor
#' (>= direction_conflict_min rows and >= direction_conflict_pct % of the pool)
#' or by a lone significant row that is >= direction_conflict_lone_pct % of the
#' pool.
#'
#' @param es numeric vector of effect sizes (length n)
#' @param ci_lo numeric vector of CI lower bounds (length n)
#' @param ci_up numeric vector of CI upper bounds (length n)
#' @param opts flag options list (uses enable_cross_row, direction_conflict_min,
#'   direction_conflict_pct, and direction_conflict_lone_pct)
#' @param info_used character vector of method names for traceability (optional)
#' @param measure character, effect size measure type (optional)
#' @param exp logical, whether values are on exponentiated scale
#' @return list of character vectors (one per row)
#' @noRd
.flag_cross_row_direction_conflict <- function(es, ci_lo, ci_up, opts,
                                                info_used = NULL,
                                                measure = NULL,
                                                exp = FALSE) {
  n <- length(es)
  flags <- vector("list", n)
  for (i in seq_len(n)) flags[[i]] <- character(0)

  if (!isTRUE(opts$enable_cross_row)) return(flags)

  # signed measures only (md/mdw included)
  signed_measures <- c("d", "g", "dw", "gw", "md", "mdw", "r", "z",
                       "logor", "logrr", "logirr", "loghr")
  if (is.null(measure) || !measure %in% signed_measures) return(flags)

  # Null reference: 1 on exponentiated scale for log-scale measures, 0 otherwise
  log_measures <- c("logor", "logrr", "logirr", "loghr")
  null_val <- if (isTRUE(exp) && measure %in% log_measures) 1 else 0

  es_label <- if (!is.null(measure) && nzchar(measure)) measure else "ES"
  if (isTRUE(exp) && measure %in% log_measures) es_label <- sub("^log", "", es_label)

  sig_pos <- which(!is.na(ci_lo) & is.finite(ci_lo) & ci_lo > null_val)
  sig_neg <- which(!is.na(ci_up) & is.finite(ci_up) & ci_up < null_val)

  n_pos <- length(sig_pos)
  n_neg <- length(sig_neg)

  min_threshold <- if (!is.null(opts$direction_conflict_min)) {
    opts$direction_conflict_min
  } else {
    1
  }

  pct_threshold <- if (!is.null(opts$direction_conflict_pct)) {
    opts$direction_conflict_pct
  } else {
    10
  }

  lone_pct_threshold <- if (!is.null(opts$direction_conflict_lone_pct)) {
    opts$direction_conflict_lone_pct
  } else {
    20
  }

  n_valid <- sum(!is.na(es) & is.finite(es))
  if (n_valid == 0) return(flags)

  pct_pos <- 100 * n_pos / n_valid
  pct_neg <- 100 * n_neg / n_valid

  side_qualifies <- function(n_side, pct_side) {
    (n_side >= min_threshold && pct_side >= pct_threshold) ||
      (n_side >= 1 && pct_side >= lone_pct_threshold)
  }
  if (!side_qualifies(n_pos, pct_pos) || !side_qualifies(n_neg, pct_neg)) {
    return(flags)
  }

  # minority side flagged [UNUSUAL], majority side [INFO]; tie: all [UNUSUAL]
  emit_info <- !isFALSE(opts$direction_conflict_info)
  majority <- if (n_pos > n_neg) "pos" else if (n_neg > n_pos) "neg" else "tie"

  build_msg <- function(i, dir) {
    msuf     <- if (!is.null(info_used)) .method_suffix(info_used[i]) else ""
    dir_word <- if (dir == "pos") "positive" else "negative"
    n_opp    <- if (dir == "pos") n_neg else n_pos
    core <- paste0("this row is significantly ", dir_word, " (",
                   es_label, " = ", round(es[i], 3),
                   ", CI [", round(ci_lo[i], 3), ", ", round(ci_up[i], 3), "]", msuf, ")")
    is_minority <- (majority == "tie") || (majority != dir)
    if (is_minority) {
      paste0("[UNUSUAL] Direction outlier: ", core,
             ", opposite to the pool majority (", n_opp,
             " significant row(s) favour the other direction) - verify the arm/group labels and the extracted value against the primary (likely extraction error or a genuine outlier)")
    } else {
      paste0("[INFO] Direction conflict in pool: ", core,
             ", agreeing with the majority direction while ", n_opp,
             " significant row(s) point the opposite way (see the UNUSUAL outlier flag)")
    }
  }

  for (dir in c("pos", "neg")) {
    rows <- if (dir == "pos") sig_pos else sig_neg
    is_minority_side <- (majority == "tie") || (majority != dir)
    if (!is_minority_side && !emit_info) next  # majority side suppressed
    for (i in rows) flags[[i]] <- c(flags[[i]], build_msg(i, dir))
  }

  return(flags)
}


#########

#' Flag rows that share a non-NA study_id with another row
#'
#' Same trial entered more than once (full publication + subanalysis, abstract
#' + full paper). Informational: data untouched, the reviewer aggregates or
#' drops.
#'
#' @param study_id character vector (or NULL) of per-row study IDs
#' @param opts flag options list (uses enable_cross_row)
#' @param info_used character vector of method names for traceability (optional)
#' @return list of character vectors (one per row)
#' @noRd
.flag_cross_row_duplicates <- function(study_id, opts, info_used = NULL) {
  n <- length(study_id)
  flags <- vector("list", n)
  for (i in seq_len(n)) flags[[i]] <- character(0)

  if (!isTRUE(opts$enable_cross_row)) return(flags)
  if (is.null(study_id) || n == 0)    return(flags)

  sid   <- as.character(study_id)
  valid <- !is.na(sid) & nzchar(sid)
  if (sum(valid) < 2) return(flags)

  tab     <- table(sid[valid])
  dup_ids <- names(tab[tab >= 2])
  if (length(dup_ids) == 0) return(flags)

  # no method suffix, the duplicate is a row-level fact
  for (dup_id in dup_ids) {
    rows <- which(sid == dup_id & valid)
    for (i in rows) {
      others <- setdiff(rows, i)
      flags[[i]] <- c(flags[[i]],
        paste0("[INFO] Duplicate study_id '", dup_id,
               "': row shares study_id with ",
               paste(.row_ref(others, sid), collapse = ", "),
               " - verify these are independent observations; ",
               "use aggregate_df() if rows should be pooled, ",
               "or drop one row if they describe the same trial."))
    }
  }
  flags
}


#########

#' Check if SE is plausible given the total sample size
#'
#' For standardized metrics the SE has a known relationship with sample size,
#' which catches variance or SD entered as SE and model-mismatch errors
#' (per-unit-slope HR pooled with binary HRs). F1: SE below m / sqrt(N), with m
#' measure-specific. F2: SE above a measure-aware ceiling. F3 (loghr/logirr
#' only): SE below sqrt(1/n_exp + 1/n_nexp), the minimum of
#' sqrt(1/d_exp + 1/d_nexp) at d = n events; not applied to logor/logrr, whose
#' SE depends on event and non-event counts
#' (SE_logOR = sqrt(1/a + 1/b + 1/c + 1/d)) and approaches 0 as events near N,
#' so an N-only floor would false-alarm on routine 2x2 data. Skipped for
#' unstandardized measures (md, nnt, rd, alpha, icc, prop).
#'
#' @param es numeric vector of effect sizes
#' @param se numeric vector of standard errors
#' @param measure character, the effect size measure
#' @param n_total numeric vector of total sample sizes
#' @param info_used character vector of method names (optional, for traceability)
#' @param n_exp numeric vector of exposed-arm sample sizes (optional; required for F3)
#' @param n_nexp numeric vector of non-exposed-arm sample sizes (optional; required for F3)
#' @return list of character vectors (one per row)
#' @noRd
.flag_se_sample_size <- function(es, se, measure, n_total, info_used = NULL,
                                  n_exp = NULL, n_nexp = NULL) {
  n <- length(es)
  flags <- vector("list", n)
  for (i in seq_len(n)) flags[[i]] <- character(0)

  # Only apply to standardized metrics where SE scales predictably with N
  standardized <- c("d", "g", "dw", "gw", "r", "z", "logor", "logrr", "loghr", "logirr")
  if (!measure %in% standardized) return(flags)

  ratio_event_measures <- c("loghr", "logirr")

  for (i in seq_len(n)) {
    if (is.na(se[i]) || !is.finite(se[i]) || se[i] <= 0) next
    msuf <- if (!is.null(info_used)) .method_suffix(info_used[i]) else ""

    # F1 / F2 require a usable n_total
    if (!is.na(n_total[i]) && is.finite(n_total[i]) && n_total[i] >= 4) {
      sqrt_n <- sqrt(n_total[i])

      # F1: implausibly small SE (floors: 0.5/sqrt(N) d/g, 0.3/sqrt(N) r/z, 0.1 ratios)
      se_floor_mult <- switch(measure,
        "d" = , "g" = , "dw" = , "gw" = 0.5,
        "r" = , "z" = 0.3,
        0.1  # logOR, logRR, logHR, logIRR: conservative (event-rate dependent)
      )
      se_floor <- se_floor_mult / sqrt_n
      # r: true SE ~ (1 - r^2)/sqrt(n - 1), scale the floor by (1 - r^2)
      if (measure == "r" && !is.na(es[i]) && is.finite(es[i]) && abs(es[i]) <= 1) {
        se_floor <- se_floor * (1 - es[i]^2)
      }
      if (se[i] < se_floor) {
        flags[[i]] <- c(flags[[i]],
          sprintf("[UNUSUAL] Implausibly small SE for N=%d: SE=%.4f (floor ~%.4f)",
                  as.integer(n_total[i]), se[i], se_floor))
      }

      # F2: implausibly large SE
      # ceiling: tight for d/g and r/z, permissive for event-rate dependent ratios
      se_ceiling <- switch(measure,
        "d" = , "g" = , "dw" = , "gw" = max(1.0, 8.0 / sqrt_n),
        "r" = , "z" = max(0.6, 4.0 / sqrt_n),
        max(2.0, 8.0 / sqrt_n)  # logor/logrr/loghr/logirr: event-rate dependent
      )
      if (se[i] > se_ceiling) {
        flags[[i]] <- c(flags[[i]],
          sprintf("[UNUSUAL] Implausibly large SE for N=%d: SE=%.3f (ceiling ~%.2f)%s",
                  as.integer(n_total[i]), se[i], se_ceiling, msuf))
      }
    }

    # F3: theoretical Cox/Wald floor from per-arm sizes (ratio-of-event-rate
    # measures only; requires both n_exp and n_nexp). Skip if F1 already fired
    # on this row to avoid duplicate "small SE" messages.
    if (measure %in% ratio_event_measures &&
        !is.null(n_exp) && !is.null(n_nexp) &&
        !is.na(n_exp[i]) && !is.na(n_nexp[i]) &&
        is.finite(n_exp[i]) && is.finite(n_nexp[i]) &&
        n_exp[i] > 0 && n_nexp[i] > 0 &&
        !any(grepl("Implausibly small SE", flags[[i]]))) {
      se_floor_f3 <- sqrt(1 / n_exp[i] + 1 / n_nexp[i])
      if (se[i] < se_floor_f3) {
        flags[[i]] <- c(flags[[i]],
          sprintf("[UNUSUAL] SE below theoretical minimum given group sizes: SE=%.4f, floor=%.4f (n_exp=%d, n_nexp=%d) - likely per-unit-slope model, wrong N, or wrong SE%s",
                  se[i], se_floor_f3,
                  as.integer(n_exp[i]), as.integer(n_nexp[i]), msuf))
      }
    }
  }

  return(flags)
}


#########

#' Check internal consistency across estimation methods
#' @param n_estimations numeric vector
#' @param dispersion numeric vector (SD of ES across methods)
#' @param overlap numeric vector (CI overlap between min/max)
#' @param diff_mm numeric vector (diff between min and max ES)
#' @param opts list of flag options
#' @param measure character, the effect size measure
#' @param min_info character vector of method names for min ES (optional)
#' @param max_info character vector of method names for max ES (optional)
#' @return list of character vectors (one per row)
#' @noRd
.flag_internal_consistency <- function(n_estimations, dispersion, overlap,
                                        diff_mm, opts, measure,
                                        min_info = NULL, max_info = NULL,
                                        min_es = NULL, max_es = NULL,
                                        exp = FALSE, es = NULL,
                                        r_defaulted = NULL) {
  n <- length(n_estimations)
  flags <- vector("list", n)
  for (i in seq_len(n)) flags[[i]] <- character(0)

  diff_max <- if (measure %in% c("d", "g", "dw", "gw", "md", "mdw")) {
    opts$diff_max_smd
  } else if (measure == "r") {
    opts$diff_max_r
  } else if (measure %in% c("logor", "logrr", "logirr", "loghr")) {
    opts$diff_max_logor
  } else {
    opts$diff_max_smd  # fallback
  }

  dispersion_max <- if (measure %in% c("d", "g", "dw", "gw", "md", "mdw")) {
    opts$dispersion_max_smd
  } else if (measure == "r") {
    opts$dispersion_max_r
  } else if (measure %in% c("logor", "logrr", "logirr", "loghr")) {
    opts$dispersion_max_logor
  } else {
    opts$dispersion_max_smd  # fallback
  }

  # recompute diff on the log scale when exp = TRUE; dispersion approximated
  # via delta method (MaxAD(log X) ~ MaxAD(X) / X)
  log_measures <- c("logor", "logrr", "logirr", "loghr")
  on_exp_scale <- isTRUE(exp) && measure %in% log_measures

  # all rows min ~ -max: direction convention, skip E flags
  all_reversed <- isTRUE(opts$ignore_sign_reversal) &&
    .rows_all_reversed(n_estimations, min_es, max_es, on_exp_scale)

  for (i in seq_len(n)) {
    if (is.na(n_estimations[i]) || n_estimations[i] < 2) next
    if (all_reversed) next

    # min/max method suffix for cross-method flags
    mmsuf <- .minmax_suffix(
      if (!is.null(min_info)) min_info[i] else NULL,
      if (!is.null(max_info)) max_info[i] else NULL
    )
    # r-sensitivity note for the SE/CI-driven checks (E2/E2b only; E1/E3 are
    # r-invariant point-estimate checks under the default Bonett pre-post SMD)
    rsuf <- if (!is.null(r_defaulted) && length(r_defaulted) >= i) {
      .r_sensitive_note(r_defaulted[i])
    } else {
      ""
    }

    # Working values on the threshold's (log) scale
    disp_work <- dispersion[i]
    diff_work <- diff_mm[i]
    scale_note <- ""
    if (on_exp_scale) {
      scale_note <- " (log scale)"
      disp_work <- if (!is.na(dispersion[i]) && is.finite(dispersion[i]) &&
                       !is.null(es) && !is.na(es[i]) && is.finite(es[i]) &&
                       es[i] > 0) {
        dispersion[i] / es[i]
      } else {
        NA_real_
      }
      diff_work <- if (!is.null(min_es) && !is.null(max_es) &&
                       !is.na(min_es[i]) && !is.na(max_es[i]) &&
                       is.finite(min_es[i]) && is.finite(max_es[i]) &&
                       min_es[i] > 0 && max_es[i] > 0) {
        log(max_es[i]) - log(min_es[i])
      } else {
        NA_real_
      }
    }

    # E1: High dispersion across ES estimates (measure-specific threshold)
    if (!is.na(disp_work) && is.finite(disp_work) &&
        disp_work > dispersion_max) {
      flags[[i]] <- c(flags[[i]],
        paste0("[DISCORDANT] High dispersion across estimates: max|ES - median| = ",
               round(disp_work, 3), scale_note,
               " (threshold: ", dispersion_max, ")"))
    }

    # E2: Zero overlap between min/max CIs
    if (!is.na(overlap[i]) && is.finite(overlap[i]) && overlap[i] == 0) {
      flags[[i]] <- c(flags[[i]],
        paste0("[DISCORDANT] Zero CI overlap between min and max estimates", mmsuf, rsuf))
    }

    # E2b: Low overlap between min/max CIs
    if (!is.na(overlap[i]) && is.finite(overlap[i]) &&
        overlap[i] > 0 && overlap[i] < opts$overlap_min) {
      flags[[i]] <- c(flags[[i]],
        paste0("[DISCORDANT] Low CI overlap between min/max estimates: ",
               round(overlap[i] * 100, 1), "% (threshold: ",
               opts$overlap_min * 100, "%)", mmsuf, rsuf))
    }

    # E3: Large difference between min and max ES
    if (!is.na(diff_work) && is.finite(diff_work) &&
        diff_work > diff_max) {
      flags[[i]] <- c(flags[[i]],
        paste0("[DISCORDANT] Large min-max difference: ",
               round(diff_work, 3), scale_note,
               " (threshold: ", diff_max, ")", mmsuf))
    }
  }
  return(flags)
}


#########

#' Determine if a V-flag is relevant to a given suffix ("_crude" or "_adjusted").
#'
#' Extracts the column name from the flag string: adjusted columns (ancova_*,
#' *_adj, cov_outcome_*, n_cov_*) go to _adjusted only, sample sizes to both,
#' everything else to _crude only.
#'
#' @param flag_str single V-flag string, e.g. "Asymmetric CI for 'md' ..."
#' @param suffix "_crude" or "_adjusted"
#' @return logical: TRUE if the flag should appear in this scope
#' @noRd
.v_flag_matches_scope <- function(flag_str, suffix) {
  # Extract column name from the quoted part, e.g. 'md'
  m <- regmatches(flag_str, regexpr("'[^']+'", flag_str))
  if (length(m) == 0) return(TRUE)  # Can't determine -> show everywhere
  col <- gsub("'", "", m)

  # Adjusted-only columns
  if (grepl("ancova|_adj$|^cov_outcome|^n_cov", col)) {
    return(suffix == "_adjusted")
  }

  # Shared columns (sample sizes): appear in both
  shared <- c("n_exp", "n_nexp", "n_sample", "n_cases", "n_controls",
              "n_cases_exp", "n_cases_nexp", "n_controls_exp", "n_controls_nexp")
  if (col %in% shared) return(TRUE)

  # Everything else is crude-only
  return(suffix == "_crude")
}

#########

#' Apply all quality/plausibility flags to the summary dataframe
#' @param res data.frame, the assembled summary result
#' @param measure character, the effect size measure
#' @param exp logical, whether the measure is on exp() scale
#' @param suffix character, "" or "_crude" or "_adjusted"
#' @param raw_data data.frame, the original input data
#' @param opts list of flag options (thresholds)
#' @param input_validation character vector (one per raw_data row), or NULL.
#'   Contains semicolon-separated validation flags from .validate_input_data().
#' @return data.frame with added flags column
#' @noRd
.flag_es_quality <- function(res, measure, exp, suffix, raw_data, opts,
                             input_validation = NULL,
                             alpha_to_es = "bonett",
                             icc_to_es = "bonett",
                             prop_to_es = "raw",
                             pre_post_to_smd = "bonett",
                             pool_sd = FALSE,
                             r_defaulted = NULL,
                             smd_denom = NULL) {
  n <- nrow(res)
  flag_col <- paste0("flags", suffix)

  es <- suppressWarnings(as.numeric(as.character(res[[paste0("es", suffix)]])))
  se <- suppressWarnings(as.numeric(as.character(res[[paste0("se", suffix)]])))
  ci_lo <- suppressWarnings(as.numeric(as.character(res[[paste0("es_ci_lo", suffix)]])))
  ci_up <- suppressWarnings(as.numeric(as.character(res[[paste0("es_ci_up", suffix)]])))

  row_idx_map <- match(res$row_id, raw_data$row_id)
  # Per-row r_pre_post-default flag (aligned to raw_data rows, like input_validation)
  r_def_row <- if (!is.null(r_defaulted)) {
    as.logical(r_defaulted[row_idx_map])
  } else {
    rep(FALSE, n)
  }
  # per-row endpoint-SMD standardizer (Glass rows use control-df CIs -- A6)
  smd_denom_row <- if (!is.null(smd_denom)) {
    as.character(smd_denom[row_idx_map])
  } else {
    NULL
  }
  n_sample <- if ("n_sample" %in% colnames(raw_data)) {
    raw_data$n_sample[row_idx_map]
  } else {
    rep(NA_real_, n)
  }
  n_exp_vec <- if ("n_exp" %in% colnames(raw_data)) {
    raw_data$n_exp[row_idx_map]
  } else {
    rep(NA_real_, n)
  }
  n_nexp_vec <- if ("n_nexp" %in% colnames(raw_data)) {
    raw_data$n_nexp[row_idx_map]
  } else {
    rep(NA_real_, n)
  }
  # Total N: prefer n_sample, fall back to n_exp + n_nexp
  n_total <- ifelse(!is.na(n_sample), n_sample,
                    ifelse(!is.na(n_exp_vec) & !is.na(n_nexp_vec),
                           n_exp_vec + n_nexp_vec, NA_real_))

  n_est_col <- paste0("n_estimations", suffix)
  disp_col <- paste0("dispersion_es", suffix)
  overlap_col <- paste0("overlap_min_max", suffix)
  diff_col <- paste0("diff_min_max", suffix)

  n_estimations <- suppressWarnings(as.numeric(as.character(res[[n_est_col]])))
  dispersion <- suppressWarnings(as.numeric(as.character(res[[disp_col]])))
  overlap <- suppressWarnings(as.numeric(as.character(res[[overlap_col]])))
  diff_mm <- suppressWarnings(as.numeric(as.character(res[[diff_col]])))

  info_col <- paste0("info_used", suffix)
  min_info_col <- paste0("min_info", suffix)
  max_info_col <- paste0("max_info", suffix)
  info_used <- if (info_col %in% colnames(res)) as.character(res[[info_col]]) else NULL
  min_info_vec <- if (min_info_col %in% colnames(res)) as.character(res[[min_info_col]]) else NULL
  max_info_vec <- if (max_info_col %in% colnames(res)) as.character(res[[max_info_col]]) else NULL

  # baseline columns for the min NNT flag (B5)
  baseline_risk_vec <- if ("baseline_risk" %in% colnames(raw_data)) {
    raw_data$baseline_risk[match(res$row_id, raw_data$row_id)]
  } else {
    NULL
  }
  baseline_rate_vec <- if ("baseline_rate" %in% colnames(raw_data)) {
    raw_data$baseline_rate[match(res$row_id, raw_data$row_id)]
  } else {
    NULL
  }

  f_a <- .flag_numeric_integrity(es, se, ci_lo, ci_up, info_used,
                                  measure = measure, exp = exp,
                                  n_total = n_total, n_exp = n_exp_vec,
                                  n_nexp = n_nexp_vec,
                                  enable_informational = isTRUE(opts$enable_informational),
                                  r_defaulted = r_def_row,
                                  prop_to_es = prop_to_es,
                                  smd_denom = smd_denom_row)
  f_b <- .flag_bounds_violations(es, se, ci_lo, ci_up, measure, exp, info_used,
                                  baseline_risk = baseline_risk_vec,
                                  baseline_rate = baseline_rate_vec,
                                  alpha_to_es = alpha_to_es,
                                  icc_to_es = icc_to_es,
                                  prop_to_es = prop_to_es)
  f_c <- .flag_plausibility(es, se, measure, opts, n_sample, info_used,
                            n_exp = n_exp_vec, n_nexp = n_nexp_vec,
                            exp = exp, prop_to_es = prop_to_es)
  sd_exp_vec  <- if ("mean_sd_exp"  %in% colnames(raw_data)) {
    suppressWarnings(as.numeric(raw_data$mean_sd_exp[match(res$row_id, raw_data$row_id)]))
  } else NULL
  sd_nexp_vec <- if ("mean_sd_nexp" %in% colnames(raw_data)) {
    suppressWarnings(as.numeric(raw_data$mean_sd_nexp[match(res$row_id, raw_data$row_id)]))
  } else NULL
  f_d <- .flag_cross_row_outliers(es, se, opts, info_used, measure, n_total = n_total,
                                   n_exp = n_exp_vec, n_nexp = n_nexp_vec,
                                   sd_exp = sd_exp_vec, sd_nexp = sd_nexp_vec,
                                   suffix = suffix, exp = exp)
  f_g <- .flag_cross_row_direction_conflict(es, ci_lo, ci_up, opts, info_used,
                                             measure, exp)

  # H: Cross-row study duplication (reads study_id from raw_data;
  # always pass a length-n vector so f_dup[[i]] is safe downstream)
  study_id_vec <- if ("study_id" %in% colnames(raw_data)) {
    as.character(raw_data$study_id[match(res$row_id, raw_data$row_id)])
  } else {
    rep(NA_character_, n)
  }
  f_dup <- .flag_cross_row_duplicates(study_id_vec, opts, info_used)

  f_f <- .flag_se_sample_size(es, se, measure, n_total, info_used,
                              n_exp = n_exp_vec, n_nexp = n_nexp_vec)
  min_es_col <- paste0("min_es_value", suffix)
  max_es_col <- paste0("max_es_value", suffix)
  min_es_vec <- if (min_es_col %in% colnames(res)) {
    suppressWarnings(as.numeric(as.character(res[[min_es_col]])))
  } else NULL
  max_es_vec <- if (max_es_col %in% colnames(res)) {
    suppressWarnings(as.numeric(as.character(res[[max_es_col]])))
  } else NULL

  f_e <- .flag_internal_consistency(n_estimations, dispersion, overlap,
                                     diff_mm, opts, measure,
                                     min_info_vec, max_info_vec,
                                     min_es = min_es_vec, max_es = max_es_vec,
                                     exp = exp, es = es,
                                     r_defaulted = r_def_row)

  # E4: Cross-row NNT type mixing (risk-based vs rate-based)
  f_nnt_mix <- vector("list", n)
  for (i in seq_len(n)) f_nnt_mix[[i]] <- character(0)
  if (measure == "nnt" && !is.null(info_used)) {
    rate_methods <- "cases_time"
    valid_info <- info_used[!is.na(info_used) & nchar(info_used) > 0 & !is.na(es)]
    has_rate <- any(valid_info %in% rate_methods)
    has_risk <- any(!valid_info %in% rate_methods)
    if (has_rate && has_risk) {
      for (i in seq_len(n)) {
        if (is.na(info_used[i]) || is.na(es[i])) next
        is_rate <- info_used[i] %in% rate_methods
        this_type <- if (is_rate) "rate-based (person-time)" else "risk-based"
        other_type <- if (is_rate) "risk-based" else "rate-based (person-time)"
        msuf <- .method_suffix(info_used[i])
        f_nnt_mix[[i]] <- paste0(
          "[DISCORDANT] Mixed NNT types: this row uses ", this_type,
          " NNT", msuf, ", but other rows use ", other_type,
          " NNT - these have different units and should not be pooled together")
      }
    }
  }

  # E6: Cross-row SMD standardizer mixing (change-SD vs raw-score-SD metric)
  #
  # [INFO], not [DISCORDANT]: DISCORDANT is reserved for a single row whose
  # several input sources disagree with one another. This is a cross-row
  # property of the pool -- every row may be individually correct.
  #
  # An SMD standardized by the CHANGE SD (morris_dz on pre/post or mean-change
  # data) is not on the same scale as an SMD standardized by a raw-score SD:
  # under equal pre/post SDs, sd_change = sd_raw * sqrt(2(1-r)), so the two
  # differ by a factor 1/sqrt(2(1-r)) (they coincide only at r = 0.5). The
  # Cochrane Handbook (v6, section 10.5.2) advises against combining
  # change-score SMDs with post-intervention SMDs, "because the SDs used in the
  # standardization reflect different things". The package's own docs say the
  # same of d_rm vs d_z.
  #
  # Rows are classified by the standardizer their info_used + pre_post_to_smd
  # imply. Endpoint (means_sd etc.), baseline-SD (bonett) and raw-metric d_rm
  # rows all live on a raw-score SD, so they are pooled into one "raw-score-SD"
  # class and mixing them raises NOTHING -- d_rm is precisely the transformation
  # onto that metric. Only morris_dz rows sit on the change-SD metric.
  f_std_mix <- vector("list", n)
  for (i in seq_len(n)) f_std_mix[[i]] <- character(0)
  smd_measures <- c("d", "g", "dw", "gw")
  # Per-row effective standardizer method: a per-row pre_post_to_smd column (which the
  # pipeline applies per row) overrides the dataset-wide scalar. Reading only the scalar
  # would miss a mixed column -- the very configuration most likely to mix standardizers.
  eff_method <- rep(as.character(pre_post_to_smd), n)
  if (!is.null(raw_data) && "pre_post_to_smd" %in% colnames(raw_data)) {
    col_method <- as.character(raw_data$pre_post_to_smd[row_idx_map])
    eff_method <- ifelse(is.na(col_method) | col_method == "", eff_method, col_method)
  }
  if (isTRUE(opts$enable_cross_row) && measure %in% smd_measures &&
      !is.null(info_used) && any(eff_method == "morris_dz", na.rm = TRUE)) {
    # methods whose standardizer is the change SD when pre_post_to_smd is dz
    change_metric_methods <- c(
      "means_sd_pre_post", "means_se_pre_post", "means_ci_pre_post",
      "mean_change_sd", "mean_change_se", "mean_change_ci", "mean_change_pval",
      "paired_t", "paired_t_pval", "paired_f", "paired_f_pval",
      "means_sd_pre_post_single_group", "means_se_pre_post_single_group",
      "means_ci_pre_post_single_group",
      "mean_change_sd_single_group", "mean_change_se_single_group",
      "mean_change_ci_single_group", "mean_change_pval_single_group",
      "paired_t_single_group"
    )
    # A row is on the change-SD metric iff it used a change-eligible method AND its
    # effective standardizer is morris_dz (a means_sd_pre_post row on bonett/d_rm is
    # on the raw-score metric, not the change-SD metric).
    row_is_change_all <- (info_used %in% change_metric_methods) & (eff_method == "morris_dz")
    valid_idx <- which(!is.na(info_used) & nchar(info_used) > 0 & !is.na(es))
    is_change <- row_is_change_all[valid_idx]
    if (any(is_change) && any(!is_change)) {
      for (i in valid_idx) {
        row_is_change <- row_is_change_all[i]
        this_metric <- if (row_is_change) "the change-SD metric (d_z)" else "a raw-score-SD metric"
        other_metric <- if (row_is_change) "a raw-score-SD metric" else "the change-SD metric (d_z)"
        msuf <- .method_suffix(info_used[i])
        f_std_mix[[i]] <- paste0(
          "[INFO] Mixed SMD standardizers: this row is on ", this_metric, msuf,
          ", but other rows are on ", other_metric,
          " - these differ by a factor 1/sqrt(2(1-r)) and should not be pooled ",
          "(Cochrane Handbook 10.5.2). Use pre_post_to_smd = 'morris_drm' (or ",
          "'bonett') to put pre/post rows on the raw-score-SD metric of the ",
          "endpoint rows")
      }
    }
  }

  # E7: paired t/F rows coexisting with pooled-standardizer rows
  #
  # A paired t (or F) statistic identifies each arm's mean_change / sd_change
  # ratio but NOT the two arms' SD ratio, so the pooled standardizing SD is not
  # recoverable: these routes necessarily standardize each arm by its own SD and
  # subtract. When the user has opted into a pooled standardizer (pool_sd = TRUE)
  # for the rows that CAN be pooled, the two constructions differ whenever a
  # study's arm SDs differ. Informational: the paired-t rows are not wrong, they
  # are simply the best obtainable from the reported statistic. Silent under the
  # default (pool_sd = FALSE), where every row uses the per-arm construction.
  f_paired_t_mix <- vector("list", n)
  for (i in seq_len(n)) f_paired_t_mix[[i]] <- character(0)
  if (isTRUE(opts$enable_cross_row) && measure %in% smd_measures &&
      !is.null(info_used) && isTRUE(pool_sd)) {
    per_arm_only_methods <- c("paired_t", "paired_t_pval", "paired_f", "paired_f_pval")
    pooled_capable_methods <- c(
      "means_sd_pre_post", "means_se_pre_post", "means_ci_pre_post",
      "mean_change_sd", "mean_change_se", "mean_change_ci", "mean_change_pval"
    )
    valid_idx <- which(!is.na(info_used) & nchar(info_used) > 0 & !is.na(es))
    has_per_arm <- any(info_used[valid_idx] %in% per_arm_only_methods)
    has_pooled <- any(info_used[valid_idx] %in% pooled_capable_methods)
    # P25: under pool_sd = TRUE the per-arm fallback contradicts the user's
    # explicit pooling request even when NO pooled-capable row coexists (a pool
    # made only of paired t/F rows used to be silent); disclose it either way,
    # with a message that fits each situation.
    if (has_per_arm) {
      for (i in valid_idx) {
        if (!info_used[i] %in% per_arm_only_methods) next
        msuf <- .method_suffix(info_used[i])
        f_paired_t_mix[[i]] <- if (has_pooled) {
          paste0(
            "[INFO] Per-arm standardizer: a paired t/F statistic does not identify ",
            "the two arms' SD ratio", msuf, ", so this row standardizes each arm by ",
            "its own SD, while other rows in this pool use an SD pooled across arms ",
            "(you set pool_sd = TRUE). The two constructions coincide only when a ",
            "study's arm SDs are equal")
        } else {
          paste0(
            "[INFO] Per-arm standardizer: a paired t/F statistic does not identify ",
            "the two arms' SD ratio", msuf, ", so this row standardizes each arm by ",
            "its own SD although you set pool_sd = TRUE. The pooled construction ",
            "is not recoverable from a paired t/F statistic")
        }
      }
    }
  }

  # E5: min and max ES on opposite sides of the null (0, or 1 when exp = TRUE)
  # suppressed with E1-E3 when all rows show min ~ -max
  f_direction <- vector("list", n)
  for (i in seq_len(n)) f_direction[[i]] <- character(0)
  signed_measures <- c("d", "g", "dw", "gw", "md", "mdw", "r", "z",
                       "logor", "logrr", "logirr", "loghr")
  log_measures_e5 <- c("logor", "logrr", "logirr", "loghr")
  on_exp_scale_e5 <- isTRUE(exp) && measure %in% log_measures_e5
  null_ref <- if (on_exp_scale_e5) 1 else 0
  all_reversed <- isTRUE(opts$ignore_sign_reversal) &&
    .rows_all_reversed(n_estimations, min_es_vec, max_es_vec, on_exp_scale_e5)
  # near-null guard: both sides must sit >= direction_disagreement_min from the null
  dd_min <- if (!is.null(opts$direction_disagreement_min)) {
    opts$direction_disagreement_min
  } else {
    0.1
  }
  null_dist <- function(v) {
    if (on_exp_scale_e5) {
      if (v <= 0) Inf else abs(log(v))
    } else {
      abs(v)
    }
  }
  if (!all_reversed && measure %in% signed_measures) {
    min_es_col <- paste0("min_es_value", suffix)
    max_es_col <- paste0("max_es_value", suffix)
    if (min_es_col %in% colnames(res) && max_es_col %in% colnames(res)) {
      min_es <- suppressWarnings(as.numeric(as.character(res[[min_es_col]])))
      max_es <- suppressWarnings(as.numeric(as.character(res[[max_es_col]])))
      for (i in seq_len(n)) {
        if (!is.na(min_es[i]) && !is.na(max_es[i]) &&
            is.finite(min_es[i]) && is.finite(max_es[i]) &&
            min_es[i] < null_ref && max_es[i] > null_ref &&
            null_dist(min_es[i]) >= dd_min && null_dist(max_es[i]) >= dd_min) {
          mmsuf <- .minmax_suffix(
            if (!is.null(min_info_vec)) min_info_vec[i] else NULL,
            if (!is.null(max_info_vec)) max_info_vec[i] else NULL
          )
          f_direction[[i]] <- paste0(
            "[DISCORDANT] Direction disagreement: min ES = ", round(min_es[i], 3),
            ", max ES = ", round(max_es[i], 3),
            " - check group labeling", mmsuf)
        }
      }
    }
  }

  # Merge all flags per row (V-flags first, then A-F)
  res[[flag_col]] <- vapply(seq_len(n), function(i) {
    # Prepend input validation flags (V1-V4) if available
    v_flags <- character(0)
    if (!is.null(input_validation)) {
      row_idx <- match(res$row_id[i], raw_data$row_id)
      if (!is.na(row_idx) && nchar(input_validation[row_idx]) > 0) {
        v_flags <- strsplit(input_validation[row_idx], "; ")[[1]]
        # Filter V-flags by crude/adjusted scope when split
        if (suffix %in% c("_crude", "_adjusted")) {
          v_flags <- v_flags[vapply(v_flags, function(f) {
            .v_flag_matches_scope(f, suffix)
          }, logical(1))]
        }
      }
    }
    combined <- c(v_flags, f_a[[i]], f_b[[i]], f_c[[i]], f_d[[i]], f_e[[i]], f_f[[i]], f_g[[i]], f_dup[[i]], f_nnt_mix[[i]], f_std_mix[[i]], f_paired_t_mix[[i]], f_direction[[i]])
    if (length(combined) == 0) return("")
    paste(combined, collapse = "; ")
  }, character(1))

  return(res)
}

#' Convert a mean difference between two independent groups and standard deviation into several effect size measures
#'
#' @param md mean difference between two independent groups
#' @param md_sd standard deviation of the mean difference
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_md a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the mean difference and 95% CI into a Cohen's d (D) and Hedges' g (G).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **The formula used to obtain the Cohen's d** is:
#' \deqn{d = \frac{md}{md\_sd}}
#' Note that this formula is perfectly accurate only if the \code{md_sd} has been estimated by assuming that the variance of the two groups is equal.
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 10. Mean difference and dispersion (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_md_sd
#'
#' @md
#'
#' @examples
#' es_from_md_sd(md = 4, md_sd = 2, n_exp = 20, n_nexp = 22)
es_from_md_sd <- function(md, md_sd, n_exp, n_nexp, smd_to_cor = "viechtbauer",
                          smd_var = "borenstein", reverse_md) {
  if (missing(reverse_md)) reverse_md <- rep(FALSE, length(md))
  reverse_md[is.na(reverse_md)] <- FALSE
  if (length(reverse_md) == 1) reverse_md = c(rep(reverse_md, length(md)))
  if (length(reverse_md) != length(md)) stop("The length of the 'reverse_md' argument is incorrectly specified.")

  d <- md / md_sd

  # P4: the default d_se is left to .es_from_d, which computes the large-sample
  # variance under the requested smd_var convention (the "borenstein" branch is
  # the exact formula previously hard-coded here, so defaults are unchanged).
  es <- .es_from_d(
    d = d, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_md
  )

  es$info_used <- "md_sd"

  es$md <- ifelse(reverse_md, -md, md)
  es$md_se <- ifelse(!is.na(es$md), md_sd * sqrt(1 / n_exp + 1 / n_nexp), NA)
  es$md_ci_lo <- ifelse(!is.na(es$md), es$md - qt(.975, n_exp + n_nexp - 2) * es$md_se, NA)
  es$md_ci_up <- ifelse(!is.na(es$md), es$md + qt(.975, n_exp + n_nexp - 2) * es$md_se, NA)

  return(es)
}

#' Convert a mean difference between two independent groups and its standard error into several effect size measures
#'
#' @param md mean difference between two independent groups
#' @param md_se standard error of the mean difference
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer").
#' @param reverse_md a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function the standard error of a mean difference into a
#' standard deviation:
#' \deqn{inv\_n = \frac{1}{n\_exp} + \frac{1}{n\_nexp}}
#' \deqn{md\_sd = \frac{md\_se}{\sqrt{inv\_n}}}
#'
#' Calculations of the \code{\link{es_from_md_sd}} function are then used to estimate
#' the Cohen's d and other effect size measures.
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 10. Mean difference and dispersion (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_md_se
#'
#' @md
#'
#' @examples
#' es_from_md_se(md = 4, md_se = 2, n_exp = 20, n_nexp = 22)
es_from_md_se <- function(md, md_se, n_exp, n_nexp, smd_to_cor = "viechtbauer",
                          smd_var = "borenstein", reverse_md) {
  if (missing(reverse_md)) reverse_md <- rep(FALSE, length(md))
  reverse_md[is.na(reverse_md)] <- FALSE

  md_sd <- md_se / sqrt(1 / n_exp + 1 / n_nexp)

  es <- es_from_md_sd(
    md = md, md_sd = md_sd,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_md = reverse_md
  )

  es$info_used <- "md_se"

  return(es)
}

#' Convert a mean difference between two independent groups and 95% CI into several effect size measures
#'
#' @param md mean difference between two independent groups
#' @param md_ci_lo lower bound of the 95% CI of the mean difference
#' @param md_ci_up upper bound of the 95% CI of the mean difference
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer").
#' @param max_asymmetry A percentage indicating the tolerance before detecting asymmetry in the 95% CI bounds.
#' @param reverse_md a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts 95% CI of a mean difference into a standard error
#' (Cochrane Handbook section 6.5.2.3):
#' \deqn{md\_se = \frac{md\_ci\_up - md\_ci\_lo}{2 * qt(0.975, df = n\_exp + n\_nexp - 2)}}
#'
#' Calculations of the \code{\link{es_from_md_se}()} function are
#' then used to estimate the Cohen's d and other effect size measures.
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 10. Mean difference and dispersion (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_md_ci
#'
#' @md
#'
#' @examples
#' es_from_md_ci(md = 4, md_ci_lo = 2, md_ci_up = 6, n_exp = 20, n_nexp = 22)
es_from_md_ci <- function(md, md_ci_lo, md_ci_up, n_exp, n_nexp,
                          smd_to_cor = "viechtbauer", smd_var = "borenstein",
                          max_asymmetry = 10, reverse_md) {
  if (missing(reverse_md)) reverse_md <- rep(FALSE, length(md))
  reverse_md[is.na(reverse_md)] <- FALSE



  md_se <- (md_ci_up - md_ci_lo) / (2 * qt(0.975, n_exp + n_nexp - 2))

  es <- es_from_md_se(
    md = md, md_se = md_se,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_md = reverse_md
  )

  es$info_used <- "md_ci"

  return(es)
}

#' Convert a mean difference between two independent groups and its p-value into several effect size measures
#'
#' @param md mean difference between two independent groups
#' @param md_pval p-value of the mean difference
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer").
#' @param reverse_md a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the p-value of a mean difference
#' into a standard error (Cochrane Handbook section 6.5.2.3):
#' \deqn{t = qt(\frac{md\_pval}{2}, df = n\_exp + n\_nexp - 2)}
#' \deqn{md\_se = |\frac{md}{t}|}
#' Calculations of the \code{\link{es_from_md_se}} function are then used to estimate the Cohen's d and other effect size measures.
#'
#' @references
#' Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 10. Mean difference and dispersion (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_md_pval
#'
#' @md
#'
#' @examples
#' es_from_md_pval(md = 4, md_pval = 0.024, n_exp = 20, n_nexp = 22)
es_from_md_pval <- function(md, md_pval, n_exp, n_nexp, smd_to_cor = "viechtbauer",
                            smd_var = "borenstein", reverse_md) {
  if (missing(reverse_md)) reverse_md <- rep(FALSE, length(md))
  reverse_md[is.na(reverse_md)] <- FALSE

  t <- qt(p = md_pval / 2, df = n_exp + n_nexp - 2, lower.tail = FALSE)

  md_se <- abs(md / t)

  es <- es_from_md_se(
    md = md, md_se = md_se,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_md = reverse_md
  )

  es$info_used <- "md_pval"

  return(es)
}

.onAttach <- function(libname, pkgname) {
  installed_version <- utils::packageVersion("metaConvert")

  packageStartupMessage(
    paste0(
      "metaConvert v", installed_version, "\n",
      "-> Email us at cgosling@parisnanterre.fr if you notice any bug. <-\n",
      "If you use the package in a publication, please cite the associated reference:\n",
      "Gosling CJ, Cortese S, Solmi M, et al. metaConvert: an automatic suite for estimation of 11 different effect size measures and flexible conversion across them. Research Synthesis Methods. Published online 2025:1-12. doi:10.1017/rsm.2025.11."
    )
  )

  invisible()
}


# test_version_check <- function() {
#   installed_version <- utils::packageVersion("metaConvert")
#
#   cran_version <- package_version("999.0.0")
#
#   if (!is.null(cran_version) && installed_version < cran_version) {
#     packageStartupMessage(
#       sprintf(
#         "\nmetaConvert Update Available!\nCurrently using version %s, but version %s is available on CRAN.\n",
#         installed_version,
#         cran_version
#       ),
#       "To update, run: install.packages('metaConvert')\n",
#       "See NEWS for changes: https://cran.r-project.org/web/packages/metaConvert/news/news.html\n"
#     )
#   } else if (!is.null(cran_version) && installed_version == cran_version) {
#     packageStartupMessage(
#       paste0(
#         "metaConvert v", installed_version, ":\n",
#         "\nSince the companion paper of the package is under review, ",
#         "this version is still considered as a beta.\n",
#         "\n-> Email us at cgosling@parisnanterre.fr if you notice any bug. <-\n",
#         "\nTo help you start with this package, you can retrieve a tutorial online ",
#         "(https://metaconvert.org/tutorial.html)",
#         "\n\nYou can also quickly generate data extraction sheets using ",
#         "(https://metaconvert.org/input.html)\n"
#       )
#     )
#   }
# }
#
# test_version_check()

#' Convert a risk ratio value and standard error to various effect size measures
#'
#' @param rr risk ratio value
#' @param logrr log risk ratio value
#' @param logrr_se standard error of the log risk ratio
#' @param n_cases number of cases/events
#' @param n_controls number of controls/no-event
#' @param n_exp number of participants in the exposed group
#' @param n_nexp number of participants in the non-exposed group
#' @param baseline_risk proportion of cases in the non-exposed group (n_cases_nexp / n_nexp is used when missing)
#' @param reverse_rr a logical value indicating whether the direction of the generated effect sizes should be flipped.
#' @param rr_to_or formula used to convert the \code{rr} value into an odds ratio (see details).
#'
#' @details
#' This function converts the (log) risk ratio (RR) value and its standard error
#' to odds ratio (OR) and number needed to treat.
#'
#' **To estimate the odds ratio and its standard error**, various formulas can be used.
#'
#' **A.** First, the approach described in Grant (2014) can be used.
#' However, in the paper, only the formula to convert an RR value to a OR value
#' is described. To derive the variance, we used this formula to convert the bounds of the 95% CI, which
#' were then used to obtain the variance.
#'
#' This argument requires (rr + baseline_risk + rr_ci_lo + rr_ci_up) to generate a RR.
#' The following formulas are used (br = baseline_risk):
#' \deqn{or = \frac{rr * (1 - br)}{1 - rr * br}}
#' \deqn{or\_ci\_lo = \frac{rr\_ci\_lo * (1 - br)}{1 - rr\_ci\_lo * br}}
#' \deqn{or\_ci\_up = \frac{rr\_ci\_up * (1 - br)}{1 - rr\_ci\_up * br}}
#' \deqn{logor\_se = \frac{log(or\_ci\_up) - log(or\_ci\_lo)}{2 * qnorm(.975)}}
#'
#' **B.** Second, the formulas implemented in the metaumbrella package can be used (\code{or_to_rr = "metaumbrella_exp"}).
#' This argument requires (rr + logrr_se + n_exp + n_nexp) to generate a OR.
#' More precisely, we previously developed functions that simulate all combinations of the possible number of cases and controls
#' in the exposed and non-exposed groups compatible with the actual value of the RR.
#' Then, the functions select the contingency table whose standard error coincides best with the standard error reported.
#' The RR value and its standard are obtained from this estimated contingency table.
#'
#' **C.** Third, it is possible to transpose the RR to a OR (\code{rr_to_or = "transpose"}).
#' This argument requires (rr + logrr_se) to generate a OR.
#' It is known that OR and RR are similar when the baseline risk is small.
#' Therefore, users can request to simply transpose the RR value & standard error into a OR value & standard error.
#' \deqn{or = rr}
#' \deqn{logor\_se = logrr\_se}
#'
#' **D.** Fourth, it is possible to recreate the 2x2 table using the dipietrantonj's formulas (\code{rr_to_or = "dipietrantonj"}).
#' This argument requires (rr + logrr_ci_lo + logrr_ci_lo) to generate a OR. Information on this approach can be retrieved in
#' Di Pietrantonj (2006).
#'
#' **To estimate the NNT**, the formulas used are :
#' \deqn{nnt = \frac{1}{br * (1 - rr)}}
#'
#' @references
#' Di Pietrantonj C. (2006). Four-fold table cell frequencies imputation in meta analysis. Statistics in medicine, 25(13), 2299-2322. https://doi.org/10.1002/sim.2287
#'
#' Gosling, C. J., Solanes, A., Fusar-Poli, P., & Radua, J. (2023). metaumbrella: the first comprehensive suite to perform data analysis in umbrella reviews with stratification of the evidence. BMJ mental health, 26(1), e300534. https://doi.org/10.1136/bmjment-2022-300534
#'
#' Grant R. L. (2014). Converting an odds ratio to a range of plausible relative risks for better communication of research findings. BMJ (Clinical research ed.), 348, f7450. https://doi.org/10.1136/bmj.f7450
#'
#' Veroniki, A. A., Pavlides, M., Patsopoulos, N. A., & Salanti, G. (2013). Reconstructing 2x2 contingency tables from odds ratios using the Di Pietrantonj method: difficulties, constraints and impact in meta-analysis results. Research synthesis methods, 4(1), 78-94. https://doi.org/10.1002/jrsm.1061
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab RR\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + NNT + RD\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 3. Risk Ratio'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_rr_se
#'
#' @md
#'
#' @examples
#' es_from_rr_se(rr = 2.12, logrr_se = 0.242, n_exp = 120, n_nexp = 44)
es_from_rr_se <- function(rr, logrr, logrr_se, baseline_risk,
                          n_exp, n_nexp, n_cases, n_controls,
                          rr_to_or = "metaumbrella",
                          reverse_rr) {

  if (missing(rr)) rr <- rep(NA_real_, length(logrr))
  if (missing(logrr)) logrr <- rep(NA_real_, length(rr))
  if (missing(logrr_se)) logrr_se <- rep(NA_real_, length(rr))
  if (missing(baseline_risk)) {
    baseline_risk <- rep(NA_real_, length(rr))
  }
  if (missing(n_exp)) {
    n_exp <- rep(NA_real_, length(rr))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA_real_, length(rr))
  }
  if (missing(n_cases)) {
    n_cases <- rep(NA_real_, length(rr))
  }
  if (missing(n_controls)) {
    n_controls <- rep(NA_real_, length(rr))
  }
  if (missing(reverse_rr)) reverse_rr <- rep(FALSE, length(rr))
  reverse_rr[is.na(reverse_rr)] <- FALSE
  if (length(reverse_rr) == 1) reverse_rr = c(rep(reverse_rr, length(rr)))
  if (length(reverse_rr) != length(rr)) stop("The length of the 'reverse_rr' argument is incorrectly specified.")

  if (!all(rr_to_or %in% c("metaumbrella", "transpose",
                           "grant", "dipietrantonj"))) {
    stop(paste0("'",
                unique(rr_to_or[!rr_to_or %in% c("metaumbrella", "transpose", "grant", "dipietrantonj")]),
                "' not in tolerated values for the 'rr_to_or' argument.
                Possible inputs are: 'metaumbrella', 'transpose', 'grant', 'dipietrantonj'"))
  }


  rr <- ifelse(is.na(rr) & !is.na(logrr), exp(logrr), rr)
  # rr <- ifelse(reverse_rr, 1 / rr, rr)

  # RR -------
  es <- data.frame(
    logrr = ifelse(reverse_rr, -log(rr), log(rr)),
    logrr_se = logrr_se,
    logrr_ci_lo = ifelse(reverse_rr, -log(rr) - qnorm(.975) * logrr_se,
                                      log(rr) - qnorm(.975) * logrr_se),
    logrr_ci_up = ifelse(reverse_rr, -log(rr) + qnorm(.975) * logrr_se,
                                      log(rr) + qnorm(.975) * logrr_se)
  )
  rr_ci_lo <- exp(log(rr) - qnorm(.975) * logrr_se)
  rr_ci_up <- exp(log(rr) + qnorm(.975) * logrr_se)

  # OR -------
  es$logor <- es$logor_se <- es$logor_ci_lo <- es$logor_ci_up <- NA

  dat_or <- data.frame(
    rr = rr, logrr_se = logrr_se, rr_ci_lo = rr_ci_lo, rr_ci_up = rr_ci_up,
    n_cases = n_cases, n_controls = n_controls, n_exp = n_exp, n_nexp = n_nexp,
    baseline_risk = baseline_risk, rr_to_or = rr_to_or
  )

  nn_miss <- with(dat_or, which(
      (rr_to_or == "grant" & !is.na(rr) & !is.na(baseline_risk) & !is.na(rr_ci_lo) & !is.na(rr_ci_up)) |
      (rr_to_or == "metaumbrella" & !is.na(rr) & !is.na(logrr_se) & !is.na(n_cases) & !is.na(n_controls)) |
      (rr_to_or == "transpose" & !is.na(rr) & !is.na(logrr_se)) |
      (rr_to_or == "dipietrantonj" & !is.na(rr) & !is.na(rr_ci_lo) & !is.na(rr_ci_up) & !is.na(n_exp) & !is.na(n_nexp))
  ))


  if (length(nn_miss) != 0) {
    res_or <- t(mapply(.rr_to_or,
      rr = dat_or$rr[nn_miss],
      logrr_se = dat_or$logrr_se[nn_miss],
      rr_ci_lo = dat_or$rr_ci_lo[nn_miss],
      rr_ci_up = dat_or$rr_ci_up[nn_miss],
      n_cases = dat_or$n_cases[nn_miss],
      n_controls = dat_or$n_controls[nn_miss],
      n_exp = dat_or$n_exp[nn_miss],
      n_nexp = dat_or$n_nexp[nn_miss],
      baseline_risk = dat_or$baseline_risk[nn_miss],
      rr_to_or = dat_or$rr_to_or[nn_miss]
    ))

    es$logor[nn_miss] <- ifelse(reverse_rr[nn_miss], -res_or[, 1], res_or[, 1])
    es$logor_se[nn_miss] <- res_or[, 2]
    # On reverse the point estimate is negated, so the (symmetric, log-scale) CI must be
    # NEGATED AND SWAPPED: new_lo = -old_up, new_up = -old_lo. Swapping alone left a
    # wrong-signed, inverted interval (lo > up) that did not bracket the reversed estimate.
    es$logor_ci_lo[nn_miss] <- ifelse(reverse_rr[nn_miss], -res_or[, 4], res_or[, 3])
    es$logor_ci_up[nn_miss] <- ifelse(reverse_rr[nn_miss], -res_or[, 3], res_or[, 4])
  }

  # Risk difference from RR + baseline_risk
  rd <- baseline_risk * (1 - rr)

  es$rd <- ifelse(reverse_rr, -rd, rd)
  # delta method
  rd_se <- baseline_risk * rr * logrr_se
  es$rd_se <- rd_se
  es$rd_ci_lo <- es$rd - qnorm(.975) * rd_se
  es$rd_ci_up <- es$rd + qnorm(.975) * rd_se

  es$nnt <- ifelse(rd == 0, NA, 1 / rd)
  es$nnt <- ifelse(reverse_rr, -es$nnt, es$nnt)
  es$nnt_se <- ifelse(rd == 0, NA, rd_se / rd^2)
  rd_ci_lo_raw <- rd - qnorm(.975) * rd_se
  rd_ci_up_raw <- rd + qnorm(.975) * rd_se
  crosses_zero <- (rd_ci_lo_raw < 0 & rd_ci_up_raw > 0) | rd == 0
  es$nnt_ci_lo <- ifelse(crosses_zero, NA,
                          ifelse(reverse_rr, -1 / rd_ci_lo_raw, 1 / rd_ci_up_raw))
  es$nnt_ci_up <- ifelse(crosses_zero, NA,
                          ifelse(reverse_rr, -1 / rd_ci_up_raw, 1 / rd_ci_lo_raw))

  es$info_used <- "rr_se"
  return(es)
}


#' Convert a risk ratio value and 95% confidence interval to various effect size measures
#'
#' @param rr risk ratio value
#' @param logrr log risk ratio value
#' @param rr_ci_lo lower bound of the 95% CI around the risk ratio value
#' @param rr_ci_up upper bound of the 95% CI around the risk ratio value
#' @param logrr_ci_lo lower bound of the 95% CI around the log risk ratio value
#' @param logrr_ci_up upper bound of the 95% CI around the log risk ratio value
#' @param n_cases number of cases/events
#' @param n_controls number of controls/no-event
#' @param n_exp number of participants in the exposed group (only required for the \code{rr_to_or = "grant_CI"}, \code{rr_to_or = "grant_2x2"} arguments).
#' @param n_nexp number of participants in the non-exposed group (only required for the \code{rr_to_or = "grant_CI"}, \code{rr_to_or = "grant_2x2"} arguments).
#' @param baseline_risk proportion of cases in the non-exposed group (only required for the \code{rr_to_or = "grant_CI"} and \code{rr_to_or = "grant_2x2"} arguments).
#' @param reverse_rr a logical value indicating whether the direction of the generated effect sizes should be flipped.
#' @param max_asymmetry A percentage indicating the tolerance before detecting asymmetry in the 95% CI bounds.
#' @param rr_to_or formula used to convert the \code{rr} value into an odds ratio (see details).
#'
#' @details
#' This function uses the 95% CI of the (log) risk ratio to obtain the standard error (Section 6.5.2.2 in the Cochrane Handbook).
#' \deqn{logrr\_se = \frac{\log{rr\_ci\_up} - \log{rr\_ci\_lo}}{2 * qnorm(.975)}}
#'
#' Then, calculations of the \code{\link{es_from_rr_se}()} are applied.
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab RR\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + NNT\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 3. Risk Ratio'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_rr_ci
#'
#' @md
#'
#' @examples
#' es_from_rr_ci(
#'   rr = 1, rr_ci_lo = 0.5, rr_ci_up = 2,
#'   n_cases = 42, n_controls = 38, baseline_risk = 0.08
#' )
es_from_rr_ci <- function(rr, rr_ci_lo, rr_ci_up, logrr, logrr_ci_lo, logrr_ci_up, baseline_risk,
                          n_exp, n_nexp, n_cases, n_controls, rr_to_or = "metaumbrella",
                          max_asymmetry = 10, reverse_rr) {
  if (missing(rr)) {
    rr <- rep(NA_real_, length(logrr))
  }
  if (missing(logrr)) {
    logrr <- rep(NA_real_, length(rr))
  }
  if (missing(baseline_risk)) {
    baseline_risk <- rep(NA_real_, length(rr))
  }
  if (missing(n_exp)) {
    n_exp <- rep(NA_real_, length(rr))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA_real_, length(rr))
  }
  if (missing(n_cases)) {
    n_cases <- rep(NA_real_, length(rr))
  }
  if (missing(n_controls)) {
    n_controls <- rep(NA_real_, length(rr))
  }
  if (missing(rr_ci_lo)) {
    rr_ci_lo <- rep(NA_real_, length(rr))
  }
  if (missing(rr_ci_up)) {
    rr_ci_up <- rep(NA_real_, length(rr))
  }
  if (missing(logrr_ci_lo)) {
    logrr_ci_lo <- rep(NA_real_, length(rr))
  }
  if (missing(logrr_ci_up)) {
    logrr_ci_up <- rep(NA_real_, length(rr))
  }
  if (missing(reverse_rr)) {
    reverse_rr <- rep(FALSE, length(rr))
  }
  reverse_rr[is.na(reverse_rr)] <- FALSE

  rr <- ifelse(is.na(rr) & !is.na(logrr), exp(logrr), rr)
  logrr_ci_lo <- ifelse(is.na(logrr_ci_lo) & !is.na(rr_ci_lo), log(rr_ci_lo), logrr_ci_lo)
  logrr_ci_up <- ifelse(is.na(logrr_ci_up) & !is.na(rr_ci_up), log(rr_ci_up), logrr_ci_up)

  logrr_se <- (logrr_ci_up - logrr_ci_lo) / (2 * qnorm(.975))
  es <- es_from_rr_se(
    rr = rr, logrr_se = logrr_se,
    baseline_risk = baseline_risk, n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls,
    rr_to_or = rr_to_or, reverse_rr = reverse_rr
  )

  es$info_used <- "rr_ci"

  return(es)
}

#' Convert a risk ratio value and its p-value to various effect size measures
#'
#' @param rr risk ratio value
#' @param logrr log risk ratio value
#' @param rr_pval p-value of the risk ratio
#' @param n_cases number of cases/events
#' @param n_controls number of controls/no-event
#' @param n_exp number of participants in the exposed group (only required for the \code{rr_to_or = "grant_CI"}, \code{rr_to_or = "grant_2x2"} arguments).
#' @param n_nexp number of participants in the non-exposed group (only required for the \code{rr_to_or = "grant_CI"}, \code{rr_to_or = "grant_2x2"} arguments).
#' @param baseline_risk proportion of cases in the non-exposed group (only required for the \code{rr_to_or = "grant_CI"} and \code{rr_to_or = "grant_2x2"} arguments).
#' @param reverse_rr_pval a logical value indicating whether the direction of the generated effect sizes should be flipped.
#' @param rr_to_or formula used to convert the \code{rr} value into an odds ratio (see details).
#'
#' @details
#' This function uses the p-value of the (log) risk ratio to obtain the standard error (Section 6.3.2 in the Cochrane Handbook).
#' \deqn{logrr\_z = qnorm(rr_pval/2, lower.tail=FALSE)}
#' \deqn{logrr\_se = |\frac{\log(rr)}{logrr\_z}|}
#'
#' Then, calculations of \code{\link{es_from_rr_se}} are applied.
#'
#' @references
#' Higgins, J. P., Thomas, J., Chandler, J., Cumpston, M., Li, T., Page, M. J., & Welch, V. A. (Eds.). (2019). Cochrane handbook for systematic reviews of interventions. John Wiley & Sons.
#'
#' @export es_from_rr_pval
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab RR\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + NNT\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 3. Risk Ratio'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_rr <- es_from_rr_pval(
#'   rr = 3.51, rr_pval = 0.001,
#'   n_cases = 12, n_controls = 68
#' )
es_from_rr_pval <- function(rr, logrr, rr_pval, baseline_risk,
                            n_exp, n_nexp, n_cases, n_controls,
                            rr_to_or = "metaumbrella",
                            reverse_rr_pval) {
  if (missing(rr)) rr <- rep(NA_real_, length(logrr))
  if (missing(logrr)) logrr <- rep(NA_real_, length(rr))
  if (missing(baseline_risk)) {
    baseline_risk <- rep(NA_real_, length(rr))
  }
  if (missing(n_exp)) {
    n_exp <- rep(NA_real_, length(rr))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA_real_, length(rr))
  }
  if (missing(n_cases)) {
    n_cases <- rep(NA_real_, length(rr))
  }
  if (missing(n_controls)) {
    n_controls <- rep(NA_real_, length(rr))
  }
  if (missing(reverse_rr_pval)) reverse_rr_pval <- rep(FALSE, length(rr))
  reverse_rr_pval[is.na(reverse_rr_pval)] <- FALSE

  rr <- ifelse(is.na(rr) & !is.na(logrr), exp(logrr), rr)
  # Mirror es_from_or_pval: take the positive critical value and the absolute
  # magnitude, so rr == 1 (log(rr) = 0) gives logrr_se = 0 (finite) rather than the
  # 0/0 = NaN produced by the earlier sign(log(rr)) denominator. The direction of the
  # effect is carried by rr itself, so the SE only needs its magnitude.
  z_rr <- qnorm(rr_pval / 2, lower.tail = FALSE)
  logrr_se <- abs(log(rr) / z_rr)

  es <- es_from_rr_se(
    rr = rr, logrr_se = logrr_se, n_cases = n_cases,
    baseline_risk = baseline_risk, n_exp = n_exp, n_nexp = n_nexp,
    n_controls = n_controls,
    rr_to_or = rr_to_or, reverse_rr = reverse_rr_pval
  )

  es$info_used <- "rr_pval"

  return(es)
}

#' Convert an adjusted mean difference and adjusted standard deviation between two independent groups obtained from an ANCOVA model into several effect size measures
#'
#' @param ancova_md adjusted mean difference between two independent groups
#' @param ancova_md_sd covariate-adjusted pooled within-group standard deviation
#' @param cov_outcome_r correlation between the outcome and covariate (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_md a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function first computes an "adjusted" Cohen's d (D), Hedges' g (G)
#' from the adjusted mean difference (MD).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate the SE of the adjusted MD**:
#' \deqn{md\_se = ancova\_md\_sd * \sqrt{\frac{1}{n\_exp} + \frac{1}{n\_nexp}}}
#' \deqn{md\_lo = md - md\_se * qt(.975, n\_exp + n\_nexp-2-n\_cov\_ancova)}
#' \deqn{md\_up = md + md\_se * qt(.975, n\_exp + n\_nexp-2-n\_cov\_ancova)}
#'
#' **To recover the unadjusted pooled SD** (for Cohen's d, table 12.3 in Cooper):
#' \deqn{md\_sd = \frac{ancova\_md\_sd}{\sqrt{1 - cor\_outcome\_r^2}}}
#'
#' **To estimate the Cohen's d** (table 12.3 in Cooper):
#' \deqn{d = \frac{ancova\_md}{md\_sd}}
#'
#' **To estimate other effect size measures**,
#' Calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 20. Adjusted: Mean difference and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @note
#' The standardized effect size variance assumes the covariate is balanced across
#' groups (Cooper eq. 12.26) and treats \code{cov_outcome_r} as known; it omits the
#' covariate-imbalance ("leverage") term of the exact ANCOVA variance, which cannot
#' be recovered from summary statistics. In observational or otherwise
#' covariate-imbalanced designs the standard error is therefore a lower bound
#' (anti-conservative), while the point estimate remains unbiased. See Lai and
#' Kelley (2012).
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Lai, K., & Kelley, K. (2012). Accuracy in parameter estimation for ANCOVA and ANOVA contrasts: Sample size planning via narrow confidence intervals. British Journal of Mathematical and Statistical Psychology, 65(2), 350-370.
#'
#' @export es_from_ancova_md_sd
#'
#' @md
#'
#' @examples
#' es_from_ancova_md_sd(
#'   ancova_md = 4, ancova_md_sd = 2,
#'   cov_outcome_r = 0.5, n_cov_ancova = 5,
#'   n_exp = 20, n_nexp = 22
#' )
es_from_ancova_md_sd <- function(ancova_md, ancova_md_sd,
                                 cov_outcome_r, n_cov_ancova,
                                 n_exp, n_nexp,
                                 smd_to_cor = "viechtbauer", smd_var = "borenstein",
                                 reverse_ancova_md) {
  if (missing(reverse_ancova_md)) reverse_ancova_md <- rep(FALSE, length(ancova_md))
  reverse_ancova_md[is.na(reverse_ancova_md)] <- FALSE
  if (length(reverse_ancova_md) == 1) reverse_ancova_md = c(rep(reverse_ancova_md, length(ancova_md)))
  if (length(reverse_ancova_md) != length(ancova_md)) stop("The length of the 'reverse_ancova_md' argument is incorrectly specified.")

  md_sd <- ancova_md_sd / sqrt(1 - cov_outcome_r^2)

  d <- ancova_md / md_sd

  es <- .es_from_d(
    d = d, cov_outcome_r = cov_outcome_r,
    adjusted = TRUE,
    n_cov_ancova = n_cov_ancova, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_ancova_md
  )

  es$info_used <- "ancova_md_sd"

  es$md <- ifelse(reverse_ancova_md, -ancova_md, ancova_md)
  es$md_se <- ancova_md_sd * sqrt(1 / n_exp + 1 / n_nexp)
  es$md_ci_lo <- es$md - qt(.975, n_exp + n_nexp - 2 - n_cov_ancova) * es$md_se
  es$md_ci_up <- es$md + qt(.975, n_exp + n_nexp - 2 - n_cov_ancova) * es$md_se

  return(es)
}


#' Convert an adjusted mean difference and standard error between two independent groups obtained from an ANCOVA model into several effect size measures
#'
#' @param ancova_md adjusted mean difference between two independent groups
#' @param ancova_md_se covariate-adjusted standard error of the mean difference
#' @param cov_outcome_r correlation between the outcome and covariate (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_md a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the mean difference (MD) standard error into a standard deviation,
#' and then relies on the calculations of the \code{\link{es_from_ancova_md_sd}} function.
#'
#' **To convert the standard error into a standard deviation**, the following formula is used.
#' \deqn{ancova\_md\_sd = \frac{ancova\_md\_se}{\sqrt{1 / n_exp + 1 / n_nexp}}}
#' Calculations of the \code{\link{es_from_ancova_md_sd}()} are then applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 20. Adjusted: Mean difference and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_ancova_md_se
#'
#' @md
#'
#' @examples
#' es_from_ancova_md_se(
#'   ancova_md = 4, ancova_md_se = 2,
#'   cov_outcome_r = 0.5, n_cov_ancova = 5,
#'   n_exp = 20, n_nexp = 22
#' )
es_from_ancova_md_se <- function(ancova_md, ancova_md_se,
                                 cov_outcome_r, n_cov_ancova,
                                 n_exp, n_nexp,
                                 smd_to_cor = "viechtbauer", smd_var = "borenstein",
                                 reverse_ancova_md) {
  if (missing(reverse_ancova_md)) reverse_ancova_md <- rep(FALSE, length(ancova_md))
  reverse_ancova_md[is.na(reverse_ancova_md)] <- FALSE
  if (length(reverse_ancova_md) == 1) reverse_ancova_md = c(rep(reverse_ancova_md, length(ancova_md)))
  if (length(reverse_ancova_md) != length(ancova_md)) stop("The length of the 'reverse_ancova_md' argument is incorrectly specified.")

  ancova_md_sd <- ancova_md_se / sqrt(1/n_exp + 1/n_nexp)

  es <- es_from_ancova_md_sd(ancova_md = ancova_md,
                             ancova_md_sd = ancova_md_sd,
                             cov_outcome_r = cov_outcome_r,
                             n_cov_ancova = n_cov_ancova,
                             n_exp = n_exp, n_nexp = n_nexp,
                             smd_to_cor = smd_to_cor, smd_var = smd_var,
                             reverse_ancova_md = reverse_ancova_md)

  es$info_used <- "ancova_md_se"

  return(es)
}


#' Convert an adjusted mean difference and adjusted standard deviation between two independent groups obtained from an ANCOVA model into several effect size measures
#'
#' @param ancova_md adjusted mean difference between two independent groups
#' @param ancova_md_ci_lo lower bound of the covariate-adjusted 95% CI of the mean difference
#' @param ancova_md_ci_up upper bound of the covariate-adjusted 95% CI of the mean difference
#' @param cov_outcome_r correlation between the outcome and covariate (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_md a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the mean difference (MD) 95% CI into a standard error,
#' and then relies on the calculations of the \code{\link{es_from_ancova_md_se}} function.
#'
#' **To convert the 95% CI into a standard error,** the following formula is used (table 12.3 in Cooper):
#' \deqn{md\_se = \frac{ancova\_md\_ci\_up - ancova\_md\_ci\_lo}{(2 * qt(0.975, n\_exp + n\_nexp - 2 - n\_cov\_ancova))}}
#' Calculations of the \code{\link{es_from_ancova_md_se}()} are then applied.
#'
#' @export es_from_ancova_md_ci
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 20. Adjusted: Mean difference and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_ancova_md_ci(
#'   ancova_md = 4, ancova_md_ci_lo = 2,
#'   ancova_md_ci_up = 6,
#'   cov_outcome_r = 0.5, n_cov_ancova = 5,
#'   n_exp = 20, n_nexp = 22
#' )
es_from_ancova_md_ci <- function(ancova_md, ancova_md_ci_lo, ancova_md_ci_up,
                                 cov_outcome_r, n_cov_ancova,
                                 n_exp, n_nexp,
                                 smd_to_cor = "viechtbauer", smd_var = "borenstein",
                                 reverse_ancova_md) {
  if (missing(reverse_ancova_md)) reverse_ancova_md <- rep(FALSE, length(ancova_md))
  reverse_ancova_md[is.na(reverse_ancova_md)] <- FALSE
  if (length(reverse_ancova_md) == 1) reverse_ancova_md = c(rep(reverse_ancova_md, length(ancova_md)))
  if (length(reverse_ancova_md) != length(ancova_md)) stop("The length of the 'reverse_ancova_md' argument is incorrectly specified.")

  ancova_md_se <- (ancova_md_ci_up - ancova_md_ci_lo) / (2 * qt(0.975, n_exp + n_nexp - 2 - n_cov_ancova))

  es <- es_from_ancova_md_se(ancova_md = ancova_md,
                             ancova_md_se = ancova_md_se,
                             cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
                             n_exp = n_exp, n_nexp = n_nexp,
                             smd_to_cor = smd_to_cor, smd_var = smd_var,
                             reverse_ancova_md = reverse_ancova_md)

  es$info_used <- "ancova_md_ci"

  return(es)
}

#' Convert an adjusted mean difference and adjusted standard deviation between two independent groups obtained from an ANCOVA model into several effect size measures
#'
#' @param ancova_md adjusted mean difference between two independent groups
#' @param ancova_md_pval p-value (two-tailed) of the adjusted mean difference
#' @param cov_outcome_r correlation between the outcome and covariate (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_md a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the mean difference (MD) p-value into a standard error,
#' and then relies on the calculations of the \code{\link{es_from_ancova_md_se}()} function.
#'
#' **To convert the p-value into a standard error,** the following formula is used (table 12.3 in Cooper):
#' \deqn{t = qt(p = \frac{ancova\_md\_pval}{2}, df = n\_exp + n\_nexp - 2 - n\_cov\_ancova)}
#' \deqn{ancova\_md\_se = | \frac{ancova\_md}{t} |}
#' Calculations of the \code{\link{es_from_ancova_md_se}()} are then applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 20. Adjusted: Mean difference and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_ancova_md_pval
#'
#' @md
#'
#' @examples
#' es_from_ancova_md_pval(
#'   ancova_md = 4, ancova_md_pval = 0.05,
#'   cov_outcome_r = 0.5, n_cov_ancova = 5,
#'   n_exp = 20, n_nexp = 22
#' )
es_from_ancova_md_pval <- function(ancova_md, ancova_md_pval,
                                   cov_outcome_r, n_cov_ancova,
                                   n_exp, n_nexp,
                                   smd_to_cor = "viechtbauer", smd_var = "borenstein",
                                   reverse_ancova_md) {
  if (missing(reverse_ancova_md)) reverse_ancova_md <- rep(FALSE, length(ancova_md))
  reverse_ancova_md[is.na(reverse_ancova_md)] <- FALSE
  if (length(reverse_ancova_md) == 1) reverse_ancova_md = c(rep(reverse_ancova_md, length(ancova_md)))
  if (length(reverse_ancova_md) != length(ancova_md)) stop("The length of the 'reverse_ancova_md' argument is incorrectly specified.")

  t <- qt(p = ancova_md_pval / 2,
          df = n_exp + n_nexp - 2 - n_cov_ancova,
          lower.tail = FALSE)

  ancova_md_se <- abs(ancova_md / t)

  es <- es_from_ancova_md_se(ancova_md = ancova_md,
                             ancova_md_se = ancova_md_se,
                             cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
                             n_exp = n_exp, n_nexp = n_nexp,
                             smd_to_cor = smd_to_cor, smd_var = smd_var,
                             reverse_ancova_md = reverse_ancova_md)

  es$info_used <- "ancova_md_pval"

  return(es)
}

################### es guidance ###################

#' mapping of methods to their required input columns
#' @return named list of lists with $specific and $shared
#' @noRd
.method_required_columns <- function() {
  list(
    # smd
    cohen_d = list(
      specific = c("cohen_d"),
      shared   = c("n_exp", "n_nexp")
    ),
    hedges_g = list(
      specific = c("hedges_g"),
      shared   = c("n_exp", "n_nexp")
    ),
    cohen_d_adj = list(
      specific = c("cohen_d_adj"),
      shared   = c("n_exp", "n_nexp", "n_cov_ancova", "cov_outcome_r")
    ),

    # or
    or = list(
      specific = list(c("or", "logor")),
      shared   = c("n_exp", "n_nexp")
    ),
    or_se = list(
      specific = list(c("or", "logor"), "logor_se"),
      shared   = character(0)
    ),
    or_ci = list(
      specific = list(c("or", "logor"), c("or_ci_lo", "logor_ci_lo"), c("or_ci_up", "logor_ci_up")),
      shared   = character(0)
    ),
    or_pval = list(
      specific = list(c("or", "logor"), "or_pval"),
      shared   = c("n_exp", "n_nexp")
    ),
    logreg_t = list(
      specific = list(c("or", "logor", "rr", "logrr"), "logreg_t"),
      shared   = c("n_exp", "n_nexp")
    ),

    # correlation
    pearson_r = list(
      specific = c("pearson_r"),
      shared   = c("n_sample")
    ),
    fisher_z = list(
      specific = c("fisher_z"),
      shared   = c("n_sample")
    ),

    # means
    means_sd = list(
      specific = c("mean_exp", "mean_sd_exp", "mean_nexp", "mean_sd_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    means_se = list(
      specific = c("mean_exp", "mean_se_exp", "mean_nexp", "mean_se_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    means_ci = list(
      specific = c("mean_exp", "mean_ci_lo_exp", "mean_ci_up_exp",
                    "mean_nexp", "mean_ci_lo_nexp", "mean_ci_up_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    means_sd_pooled = list(
      specific = c("mean_exp", "mean_nexp", "mean_sd_pooled"),
      shared   = c("n_exp", "n_nexp")
    ),

    # means from plot
    means_plot = list(
      specific = c("plot_mean_exp", "plot_mean_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),

    # pre-post means
    means_sd_pre_post = list(
      specific = c("mean_pre_exp", "mean_exp", "mean_pre_sd_exp", "mean_sd_exp",
                    "mean_pre_nexp", "mean_nexp", "mean_pre_sd_nexp", "mean_sd_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    means_se_pre_post = list(
      specific = c("mean_pre_exp", "mean_exp", "mean_pre_se_exp", "mean_se_exp",
                    "mean_pre_nexp", "mean_nexp", "mean_pre_se_nexp", "mean_se_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    means_ci_pre_post = list(
      specific = c("mean_pre_exp", "mean_exp",
                    "mean_pre_ci_lo_exp", "mean_pre_ci_up_exp",
                    "mean_ci_lo_exp", "mean_ci_up_exp",
                    "mean_pre_nexp", "mean_nexp",
                    "mean_pre_ci_lo_nexp", "mean_pre_ci_up_nexp",
                    "mean_ci_lo_nexp", "mean_ci_up_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),

    # mean change
    mean_change_sd = list(
      specific = c("mean_change_exp", "mean_change_sd_exp",
                    "mean_change_nexp", "mean_change_sd_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    mean_change_se = list(
      specific = c("mean_change_exp", "mean_change_se_exp",
                    "mean_change_nexp", "mean_change_se_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    mean_change_ci = list(
      specific = c("mean_change_exp",
                    "mean_change_ci_lo_exp", "mean_change_ci_up_exp",
                    "mean_change_nexp",
                    "mean_change_ci_lo_nexp", "mean_change_ci_up_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    mean_change_pval = list(
      specific = c("mean_change_exp", "mean_change_pval_exp",
                    "mean_change_nexp", "mean_change_pval_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),

    # paired statistics
    paired_t = list(
      specific = c("paired_t_exp", "paired_t_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    paired_t_pval = list(
      specific = c("paired_t_pval_exp", "paired_t_pval_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    paired_f = list(
      specific = c("paired_f_exp", "paired_f_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    paired_f_pval = list(
      specific = c("paired_f_pval_exp", "paired_f_pval_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),

    # anova / student t
    student_t = list(
      specific = c("student_t"),
      shared   = c("n_exp", "n_nexp")
    ),
    student_t_pval = list(
      specific = c("student_t_pval"),
      shared   = c("n_exp", "n_nexp")
    ),
    anova_f = list(
      specific = c("anova_f"),
      shared   = c("n_exp", "n_nexp")
    ),
    anova_f_pval = list(
      specific = c("anova_f_pval"),
      shared   = c("n_exp", "n_nexp")
    ),
    etasq = list(
      specific = c("etasq"),
      shared   = c("n_exp", "n_nexp")
    ),
    etasq_adj = list(
      specific = c("etasq_adj"),
      shared   = c("n_exp", "n_nexp", "n_cov_ancova", "cov_outcome_r")
    ),
    pt_bis_r = list(
      specific = c("pt_bis_r"),
      shared   = c("n_exp", "n_nexp")
    ),
    pt_bis_r_pval = list(
      specific = c("pt_bis_r_pval"),
      shared   = c("n_exp", "n_nexp")
    ),

    # mean difference
    md_sd = list(
      specific = c("md", "md_sd"),
      shared   = c("n_exp", "n_nexp")
    ),
    md_se = list(
      specific = c("md", "md_se"),
      shared   = c("n_exp", "n_nexp")
    ),
    md_ci = list(
      specific = c("md", "md_ci_lo", "md_ci_up"),
      shared   = c("n_exp", "n_nexp")
    ),
    md_pval = list(
      specific = c("md", "md_pval"),
      shared   = c("n_exp", "n_nexp")
    ),

    # medians
    med_quarts = list(
      specific = c("q1_exp", "med_exp", "q3_exp",
                    "q1_nexp", "med_nexp", "q3_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    med_min_max = list(
      specific = c("min_exp", "med_exp", "max_exp",
                    "min_nexp", "med_nexp", "max_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    med_min_max_quarts = list(
      specific = c("min_exp", "q1_exp", "med_exp", "q3_exp", "max_exp",
                    "min_nexp", "q1_nexp", "med_nexp", "q3_nexp", "max_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),

    # regression
    beta_std = list(
      specific = c("beta_std", "sd_dv"),
      shared   = c("n_exp", "n_nexp")
    ),
    beta_unstd = list(
      specific = c("beta_unstd", "sd_dv"),
      shared   = c("n_exp", "n_nexp")
    ),
    linreg_t = list(
      specific = c("linreg_t"),
      shared   = c("n_sample", "n_covariates")
    ),
    linreg_b_se = list(
      specific = c("linreg_b", "linreg_b_se"),
      shared   = c("n_sample", "n_covariates")
    ),
    linreg_b_ci = list(
      specific = c("linreg_b", "linreg_b_ci_lo", "linreg_b_ci_up"),
      shared   = c("n_sample", "n_covariates")
    ),
    linreg_b_pval = list(
      specific = c("linreg_b", "linreg_b_pval"),
      shared   = c("n_sample", "n_covariates")
    ),

    # 2x2 table
    "2x2" = list(
      specific = c("n_cases_exp", "n_cases_nexp",
                    "n_controls_exp", "n_controls_nexp"),
      shared   = character(0)
    ),
    "2x2_sum" = list(
      specific = c("n_cases_exp", "n_cases_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    "2x2_prop" = list(
      specific = c("prop_cases_exp", "prop_cases_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),

    # chi-square / phi
    chisq = list(
      specific = c("chisq"),
      shared   = c("n_sample", "n_cases", "n_exp")
    ),
    chisq_pval = list(
      specific = c("chisq_pval"),
      shared   = c("n_sample", "n_cases", "n_exp")
    ),
    phi = list(
      specific = c("phi"),
      shared   = c("n_sample", "n_cases", "n_exp")
    ),

    # risk ratio
    rr_se = list(
      specific = list(c("rr", "logrr"), "logrr_se"),
      shared   = character(0)
    ),
    rr_ci = list(
      specific = list(c("rr", "logrr"), c("rr_ci_lo", "logrr_ci_lo"), c("rr_ci_up", "logrr_ci_up")),
      shared   = character(0)
    ),
    rr_pval = list(
      specific = list(c("rr", "logrr"), "rr_pval"),
      shared   = c("n_exp", "n_nexp")
    ),

    # risk difference
    rd_se = list(
      specific = c("rd", "rd_se"),
      shared   = character(0)
    ),
    rd_ci = list(
      specific = c("rd", "rd_ci_lo", "rd_ci_up"),
      shared   = character(0)
    ),
    rd_pval = list(
      specific = c("rd", "rd_pval"),
      shared   = character(0)
    ),

    # ancova means
    ancova_means_sd = list(
      specific = c("ancova_mean_exp", "ancova_mean_nexp",
                    "ancova_mean_sd_exp", "ancova_mean_sd_nexp"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_means_se = list(
      specific = c("ancova_mean_exp", "ancova_mean_nexp",
                    "ancova_mean_se_exp", "ancova_mean_se_nexp"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_means_ci = list(
      specific = c("ancova_mean_exp", "ancova_mean_nexp",
                    "ancova_mean_ci_lo_exp", "ancova_mean_ci_up_exp",
                    "ancova_mean_ci_lo_nexp", "ancova_mean_ci_up_nexp"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_means_sd_pooled = list(
      specific = c("ancova_mean_exp", "ancova_mean_nexp", "mean_sd_pooled"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_means_sd_pooled_adj = list(
      specific = c("ancova_mean_exp", "ancova_mean_nexp", "ancova_mean_sd_pooled"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_means_plot = list(
      specific = c("plot_ancova_mean_exp", "plot_ancova_mean_nexp"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),

    # ancova statistics
    ancova_t = list(
      specific = c("ancova_t"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_f = list(
      specific = c("ancova_f"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_t_pval = list(
      specific = c("ancova_t_pval"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_f_pval = list(
      specific = c("ancova_f_pval"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),

    # ancova mean difference
    ancova_md_sd = list(
      specific = c("ancova_md", "ancova_md_sd"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_md_se = list(
      specific = c("ancova_md", "ancova_md_se"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_md_ci = list(
      specific = c("ancova_md", "ancova_md_ci_lo", "ancova_md_ci_up"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),
    ancova_md_pval = list(
      specific = c("ancova_md", "ancova_md_pval"),
      shared   = c("n_exp", "n_nexp", "cov_outcome_r", "n_cov_ancova")
    ),

    # user input
    user_input_crude = list(
      specific = c("user_es_crude", "user_se_crude"),
      shared   = c("user_es_original_measure_crude")
    ),
    user_input_adj = list(
      specific = c("user_es_adj", "user_se_adj"),
      shared   = c("user_es_original_measure_adj")
    ),

    # irr
    cases_time = list(
      specific = c("n_cases_exp", "n_cases_nexp", "time_exp", "time_nexp"),
      shared   = character(0)
    ),

    # variability ratios
    variability_means_sd = list(
      specific = c("mean_exp", "mean_nexp", "mean_sd_exp", "mean_sd_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    variability_means_se = list(
      specific = c("mean_exp", "mean_nexp", "mean_se_exp", "mean_se_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),
    variability_means_ci = list(
      specific = c("mean_exp", "mean_nexp",
                    "mean_ci_lo_exp", "mean_ci_up_exp",
                    "mean_ci_lo_nexp", "mean_ci_up_nexp"),
      shared   = c("n_exp", "n_nexp")
    ),

    # single group pre-post means
    means_sd_pre_post_single_group = list(
      specific = c("mean_pre_exp", "mean_exp",
                    "mean_pre_sd_exp", "mean_sd_exp"),
      shared   = c("n_exp")
    ),
    means_se_pre_post_single_group = list(
      specific = c("mean_pre_exp", "mean_exp",
                    "mean_pre_se_exp", "mean_se_exp"),
      shared   = c("n_exp")
    ),
    means_ci_pre_post_single_group = list(
      specific = c("mean_pre_exp", "mean_exp",
                    "mean_pre_ci_lo_exp", "mean_pre_ci_up_exp",
                    "mean_ci_lo_exp", "mean_ci_up_exp"),
      shared   = c("n_exp")
    ),

    # single group mean change
    mean_change_sd_single_group = list(
      specific = c("mean_change_exp", "mean_change_sd_exp"),
      shared   = c("n_exp")
    ),
    mean_change_se_single_group = list(
      specific = c("mean_change_exp", "mean_change_se_exp"),
      shared   = c("n_exp")
    ),
    mean_change_ci_single_group = list(
      specific = c("mean_change_exp",
                    "mean_change_ci_lo_exp", "mean_change_ci_up_exp"),
      shared   = c("n_exp")
    ),
    mean_change_pval_single_group = list(
      specific = c("mean_change_exp", "mean_change_pval_exp"),
      shared   = c("n_exp")
    ),

    # single group paired t
    paired_t_single_group = list(
      specific = c("paired_t_exp"),
      shared   = c("n_exp")
    ),

    # single group proportions
    prop_single_group = list(
      specific = c("prop"),
      shared   = c("n_sample")
    ),
    prop_single_group_counts = list(
      specific = c("n_cases"),
      shared   = c("n_sample")
    ),

    # psychometric
    cronbach_alpha = list(
      specific = c("cronbach_alpha"),
      shared   = c("n_sample", "n_items")
    ),
    icc = list(
      specific = c("icc"),
      shared   = c("n_sample", "n_measurements")
    ),
    spearman_r = list(
      specific = c("spearman_r"),
      shared   = c("n_sample")
    )
  )
}


#' human-readable method descriptions
#' @return named character vector
#' @noRd
.method_descriptions <- function() {
  c(
    cohen_d                = "Cohen's d (direct input)",
    hedges_g               = "Hedges' g (direct input)",
    cohen_d_adj            = "Adjusted Cohen's d (direct input)",

    or                     = "Odds ratio (direct input)",
    or_se                  = "Odds ratio + standard error",
    or_ci                  = "Odds ratio + confidence interval",
    or_pval                = "Odds ratio + p-value",
    logreg_t               = "Logistic regression t-value",

    pearson_r              = "Pearson correlation (direct input)",
    fisher_z               = "Fisher's z (direct input)",

    means_sd               = "Means + SDs of two groups",
    means_se               = "Means + SEs of two groups",
    means_ci               = "Means + CIs of two groups",
    means_sd_pooled        = "Means + pooled SD",
    means_plot             = "Means from plot",

    means_sd_pre_post      = "Pre/post means + SDs of two groups",
    means_se_pre_post      = "Pre/post means + SEs of two groups",
    means_ci_pre_post      = "Pre/post means + CIs of two groups",

    mean_change_sd         = "Mean change + SDs of two groups",
    mean_change_se         = "Mean change + SEs of two groups",
    mean_change_ci         = "Mean change + CIs of two groups",
    mean_change_pval       = "Mean change + p-values of two groups",

    paired_t               = "Paired t-values of two groups",
    paired_t_pval          = "Paired t p-values of two groups",
    paired_f               = "Paired F-values of two groups",
    paired_f_pval          = "Paired F p-values of two groups",

    student_t              = "Student's t-value",
    student_t_pval         = "Student's t p-value",
    anova_f                = "ANOVA F-value",
    anova_f_pval           = "ANOVA F p-value",
    etasq                  = "Eta-squared",
    etasq_adj              = "Adjusted eta-squared",
    pt_bis_r               = "Point-biserial correlation",
    pt_bis_r_pval          = "Point-biserial correlation p-value",

    md_sd                  = "Mean difference + SD",
    md_se                  = "Mean difference + SE",
    md_ci                  = "Mean difference + CI",
    md_pval                = "Mean difference + p-value",

    med_quarts             = "Medians + quartiles",
    med_min_max            = "Medians + min/max",
    med_min_max_quarts     = "Medians + quartiles + min/max",

    beta_std               = "Standardized regression coefficient",
    beta_unstd             = "Unstandardized regression coefficient",
    linreg_t               = "Linear regression t-value",
    linreg_b_se            = "Regression coefficient + standard error",
    linreg_b_ci            = "Regression coefficient + confidence interval",
    linreg_b_pval          = "Regression coefficient + p-value",

    "2x2"                  = "2x2 contingency table (full)",
    "2x2_sum"              = "2x2 table (cases + group totals)",
    "2x2_prop"             = "2x2 table (proportions)",

    chisq                  = "Chi-squared value",
    chisq_pval             = "Chi-squared p-value",
    phi                    = "Phi coefficient",

    rr_se                  = "Risk ratio + standard error",
    rr_ci                  = "Risk ratio + confidence interval",
    rr_pval                = "Risk ratio + p-value",

    rd_se                  = "Risk difference + standard error",
    rd_ci                  = "Risk difference + confidence interval",
    rd_pval                = "Risk difference + p-value",

    ancova_means_sd        = "ANCOVA adjusted means + SDs",
    ancova_means_se        = "ANCOVA adjusted means + SEs",
    ancova_means_ci        = "ANCOVA adjusted means + CIs",
    ancova_means_sd_pooled = "ANCOVA adjusted means + pooled SD",
    ancova_means_sd_pooled_adj = "ANCOVA adjusted means + adjusted pooled SD",
    ancova_means_plot      = "ANCOVA adjusted means from plot",

    ancova_t               = "ANCOVA t-value",
    ancova_f               = "ANCOVA F-value",
    ancova_t_pval          = "ANCOVA t p-value",
    ancova_f_pval          = "ANCOVA F p-value",

    ancova_md_sd           = "ANCOVA mean difference + SD",
    ancova_md_se           = "ANCOVA mean difference + SE",
    ancova_md_ci           = "ANCOVA mean difference + CI",
    ancova_md_pval         = "ANCOVA mean difference + p-value",

    user_input_crude       = "User-provided ES (crude) - set user_es_original_measure for conversion",
    user_input_adj         = "User-provided ES (adjusted) - set user_es_original_measure for conversion",

    cases_time             = "Cases + person-time (IRR)",

    variability_means_sd   = "Variability ratio from means + SDs",
    variability_means_se   = "Variability ratio from means + SEs",
    variability_means_ci   = "Variability ratio from means + CIs",

    means_sd_pre_post_single_group  = "Pre/post means + SDs (single group)",
    means_se_pre_post_single_group  = "Pre/post means + SEs (single group)",
    means_ci_pre_post_single_group  = "Pre/post means + CIs (single group)",

    mean_change_sd_single_group     = "Mean change + SD (single group)",
    mean_change_se_single_group     = "Mean change + SE (single group)",
    mean_change_ci_single_group     = "Mean change + CI (single group)",
    mean_change_pval_single_group   = "Mean change + p-value (single group)",

    paired_t_single_group           = "Paired t-value (single group)",

    prop_single_group               = "Proportion (direct input)",
    prop_single_group_counts        = "Cases + total (counts)",

    cronbach_alpha                  = "Cronbach's alpha (direct input)",
    icc                             = "ICC (direct input)",
    spearman_r                      = "Spearman correlation (converted to Pearson)"
  )
}


#' methods applicable to a given measure (crude/adjusted)
#' @param measure the effect size measure
#' @param suffix "" or "_crude" or "_adjusted"
#' @return character vector
#' @noRd
.get_applicable_methods <- function(measure, suffix) {

  smd_post <- c(
    "cohen_d", "hedges_g",
    "means_sd", "means_se", "means_ci", "means_sd_pooled", "means_plot",
    "student_t", "student_t_pval", "anova_f", "anova_f_pval",
    "etasq", "pt_bis_r", "pt_bis_r_pval",
    "md_sd", "md_se", "md_ci", "md_pval",
    "med_quarts", "med_min_max", "med_min_max_quarts",
    "beta_std", "beta_unstd"
  )

  smd_paired <- c(
    "means_sd_pre_post", "means_se_pre_post", "means_ci_pre_post",
    "mean_change_sd", "mean_change_se", "mean_change_ci", "mean_change_pval",
    "paired_t", "paired_t_pval", "paired_f", "paired_f_pval"
  )

  smd_adj <- c(
    "cohen_d_adj", "etasq_adj",
    "ancova_means_sd", "ancova_means_se", "ancova_means_ci",
    "ancova_means_sd_pooled", "ancova_means_sd_pooled_adj", "ancova_means_plot",
    "ancova_t", "ancova_f", "ancova_t_pval", "ancova_f_pval",
    "ancova_md_sd", "ancova_md_se", "ancova_md_ci", "ancova_md_pval"
  )

  or_cat  <- c("or", "or_se", "or_ci", "or_pval", "logreg_t")
  cor_cat <- c("pearson_r", "fisher_z", "spearman_r")
  cont_cat <- c("2x2", "2x2_sum", "2x2_prop")
  phi_cat <- c("chisq", "chisq_pval", "phi")
  rr_cat  <- c("rr_se", "rr_ci", "rr_pval")
  rd_cat  <- c("rd_se", "rd_ci", "rd_pval")
  irr_cat <- c("cases_time")
  var_cat <- c("variability_means_sd", "variability_means_se", "variability_means_ci")

  within_group <- c(
    "means_sd_pre_post_single_group", "means_se_pre_post_single_group",
    "means_ci_pre_post_single_group",
    "mean_change_sd_single_group", "mean_change_se_single_group",
    "mean_change_ci_single_group", "mean_change_pval_single_group",
    "paired_t_single_group"
  )

  prop_sg     <- c("prop_single_group", "prop_single_group_counts")
  partial_cor <- c("linreg_t", "linreg_b_se", "linreg_b_ci", "linreg_b_pval")

  if (measure %in% c("dw", "gw", "mdw")) {
    return(c(within_group, "user_input_crude"))
  } else if (measure == "prop") {
    return(c(prop_sg, "user_input_crude"))
  } else if (measure == "alpha") {
    return(c("cronbach_alpha", "user_input_crude"))
  } else if (measure == "icc") {
    return(c("icc", "user_input_crude"))
  } else if (measure %in% c("hr", "loghr")) {
    # HR needs per-subject time-to-event data the wide extraction sheet cannot hold,
    # so convert_df() runs ONLY the user-input methods (hierarchy c(USER_crude,
    # USER_adjusted)). Without this case HR fell through to the switch default and
    # guidance offered ~55 means/SD/variability methods that can never yield a hazard
    # ratio. Respect the crude/adjusted split so es_guidance_crude/_adjusted are right.
    if (suffix == "_crude") return("user_input_crude")
    if (suffix == "_adjusted") return("user_input_adj")
    return(c("user_input_crude", "user_input_adj"))
  } else if (measure %in% c("rp", "zp")) {
    return(c(partial_cor, "user_input_crude"))
  }

  crude_cats <- switch(
    measure,
    "d" =, "g" =, "md" =
      c(smd_post, smd_paired, or_cat, cont_cat, cor_cat, phi_cat),
    "logor" =, "or" =
      c(or_cat, cont_cat, rr_cat, rd_cat, phi_cat, cor_cat, smd_post),
    "logrr" =, "rr" =
      c(rr_cat, cont_cat, or_cat, rd_cat, phi_cat),
    "logirr" =, "irr" =
      irr_cat,
    "nnt" =
      c(rd_cat, cont_cat, or_cat, rr_cat, irr_cat, phi_cat),
    "rd" =
      c(rd_cat, cont_cat, or_cat, rr_cat, irr_cat, phi_cat),
    "r" =, "z" =
      c(cor_cat, cont_cat, or_cat, phi_cat, smd_post, smd_paired),
    "logvr" =, "logcvr" =
      c(var_cat, smd_post, smd_paired),
    c(smd_post, smd_paired, or_cat, cont_cat, cor_cat,
      phi_cat, rr_cat, irr_cat, var_cat)
  )

  adj_cats <- switch(
    measure,
    "d" =, "g" =, "md" = smd_adj,
    # ancova methods not used for other measures
    character(0)
  )

  crude_all <- unique(c(crude_cats, "user_input_crude"))
  adj_all   <- unique(c(adj_cats, "user_input_adj"))

  if (suffix == "_crude") {
    return(crude_all)
  } else if (suffix == "_adjusted") {
    return(adj_all)
  } else {
    return(unique(c(crude_all, adj_all)))
  }
}


#' true if at least one column of the group is non-NA
#' @param spec_entry column name(s)
#' @param row_data one row of data
#' @return logical
#' @noRd
.has_any_value <- function(spec_entry, row_data) {
  for (col in spec_entry) {
    if (col %in% names(row_data) && !is.na(row_data[[col]])) return(TRUE)
  }
  return(FALSE)
}


#' near-miss methods for one row
#' @param row_data one row of raw_data
#' @param methods_to_check method names
#' @param method_cols output of .method_required_columns()
#' @param descriptions output of .method_descriptions()
#' @param max_suggestions max near-misses shown
#' @return character
#' @noRd
.diagnose_missing_data <- function(row_data, methods_to_check, method_cols,
                                    descriptions = .method_descriptions(),
                                    max_suggestions = 3) {
  near_misses <- list()

  for (method in methods_to_check) {
    if (!method %in% names(method_cols)) next

    mc <- method_cols[[method]]
    spec <- mc$specific
    shared <- mc$shared

    all_specific <- character(0)
    has_any_specific <- FALSE

    if (is.list(spec)) {
      for (entry in spec) {
        if (.has_any_value(entry, row_data)) {
          has_any_specific <- TRUE
        }
        all_specific <- c(all_specific, entry)
      }
    } else {
      all_specific <- spec
      for (col in spec) {
        if (col %in% names(row_data) && !is.na(row_data[[col]])) {
          has_any_specific <- TRUE
          break
        }
      }
    }

    if (!has_any_specific) next

    all_required <- unique(c(all_specific, shared))
    missing_cols <- character(0)

    if (is.list(spec)) {
      for (entry in spec) {
        if (length(entry) > 1) {
          # missing only if none of the alternatives present
          if (!.has_any_value(entry, row_data)) {
            missing_cols <- c(missing_cols, entry[1])
          }
        } else {
          if (!.has_any_value(entry, row_data)) {
            missing_cols <- c(missing_cols, entry)
          }
        }
      }
    } else {
      for (col in spec) {
        if (!(col %in% names(row_data)) || is.na(row_data[[col]])) {
          missing_cols <- c(missing_cols, col)
        }
      }
    }

    for (col in shared) {
      if (!(col %in% names(row_data)) || is.na(row_data[[col]])) {
        missing_cols <- c(missing_cols, col)
      }
    }

    missing_specific <- character(0)
    missing_shared   <- character(0)
    for (col in missing_cols) {
      if (col %in% shared) {
        missing_shared <- c(missing_shared, col)
      } else {
        missing_specific <- c(missing_specific, col)
      }
    }

    if (length(missing_cols) > 0) {
      near_misses[[length(near_misses) + 1]] <- list(
        method  = method,
        missing = missing_cols,
        missing_specific = missing_specific,
        missing_shared   = missing_shared,
        only_shared = length(missing_specific) == 0,
        n_missing = length(missing_cols)
      )
    }
  }

  if (length(near_misses) == 0) {
    return("No partial input data found. See data_extraction_sheet() for required columns.")
  }

  # methods missing only shared cols first
  only_shared <- vapply(near_misses, function(x) x$only_shared, logical(1))
  n_miss <- vapply(near_misses, function(x) x$n_missing, integer(1))
  near_misses <- near_misses[order(!only_shared, n_miss)]

  near_misses <- near_misses[seq_len(min(length(near_misses), max_suggestions))]

  msgs <- vapply(near_misses, function(nm) {
    label <- if (nm$method %in% names(descriptions)) {
      descriptions[[nm$method]]
    } else {
      nm$method
    }
    if (nm$only_shared) {
      paste0(label, " [complete input]: only add '",
             paste(nm$missing_shared, collapse = "' + '"), "'")
    } else {
      paste0(label, ": add '", paste(nm$missing, collapse = "' + '"), "'")
    }
  }, character(1))

  paste(msgs, collapse = "; ")
}


#' add the es_guidance column to the summary result
#' @param res the assembled summary result
#' @param suffix "" or "_crude" or "_adjusted"
#' @param raw_data original input data
#' @param measure the effect size measure
#' @param object the metaConvert object
#' @return data.frame
#' @noRd
.add_es_guidance <- function(res, suffix, raw_data, measure, object) {
  n <- nrow(res)
  guidance_col <- paste0("es_guidance", suffix)
  es_col <- paste0("es", suffix)

  method_cols  <- .method_required_columns()
  descriptions <- .method_descriptions()

  methods_to_check <- .get_applicable_methods(measure, suffix)

  res[[guidance_col]] <- rep("", n)

  # only rows with NA es
  for (i in seq_len(n)) {
    es_val <- suppressWarnings(as.numeric(as.character(res[[es_col]][i])))
    if (is.na(es_val)) {
      row_id <- res$row_id[i]
      raw_row_idx <- which(raw_data$row_id == row_id)

      if (length(raw_row_idx) == 1) {
        row_data <- as.list(raw_data[raw_row_idx, ])
        res[[guidance_col]][i] <- .diagnose_missing_data(
          row_data, methods_to_check, method_cols, descriptions
        )
      }
    }
  }

  return(res)
}

#' Disattenuate (correct for unreliability) a correlation coefficient
#'
#' @param r observed correlation coefficient (values outside \eqn{[-1, 1]}
#'   trigger a warning; the arithmetic is still applied so the inputs can be
#'   inspected)
#' @param r_se standard error of the observed correlation. Optional: when omitted,
#'   it is derived from \code{n_sample} (see Details).
#' @param reliability_x reliability of measure X (e.g., target PROM)
#' @param reliability_y reliability of measure Y (e.g., comparator instrument)
#' @param n_sample sample size. Used only to derive \code{r_se} when \code{r_se}
#'   is not supplied; otherwise unused.
#'
#' @details
#' Corrects an observed correlation for attenuation due to measurement error
#' in both measures, using the classical disattenuation formula
#' (Hunter & Schmidt, 2004; Spearman, 1904):
#'
#' \deqn{r_c = \frac{r_{obs}}{\sqrt{rel_x \times rel_y}}}
#'
#' where \eqn{rel_x} and \eqn{rel_y} are reliability coefficients for the
#' two measures (e.g., Cronbach's alpha, test-retest ICC).
#'
#' The standard error of the corrected correlation is approximated as:
#' \deqn{r_c\_se = \frac{r\_se}{\sqrt{rel_x \times rel_y}}}
#'
#' This approximation treats the reliabilities as known constants, so the SE
#' and CI are a lower bound when the reliabilities are themselves estimated
#' (Hunter & Schmidt, 2004, Ch. 3).
#'
#' When \code{r_se} is not supplied, it is first obtained from the sample size as
#' the large-sample Pearson correlation SE \eqn{\sqrt{(1 - r^2)^2 / (n - 1)}}
#' (Cooper et al., 2019).
#'
#' The function also provides Fisher's z transformation of the corrected
#' correlation for use in meta-analysis, with
#' \eqn{SE(z_c) = SE(r_c) / (1 - r_c^2)} (delta method). The corrected-r
#' confidence interval is obtained by back-transforming the Fisher-z interval
#' (\eqn{\tanh}), so it always lies within (-1, 1).
#'
#' When the corrected correlation is extreme (\eqn{|r_c| > 0.999}, including
#' the mathematically impossible \eqn{|r_c| \ge 1} that inconsistent inputs
#' produce), no meaningful Fisher's z or CI exists: the corrected-r CI and all
#' Fisher's z outputs (\code{z_corrected}, \code{z_corrected_se} and its CI
#' bounds) are set to \code{NA} and a warning is emitted. The corrected point
#' estimate and its first-order standard error are always returned as
#' computed, so the offending inputs can be inspected.
#'
#' This function is typically applied to the results of
#' \code{summary(convert_df(..., measure = "r"))}:
#'
#' \preformatted{
#' res <- summary(convert_df(my_data, measure = "r"))
#' corrected <- es_disattenuate(
#'   r = res$es, r_se = res$se,
#'   reliability_x = my_data$rel_target,
#'   reliability_y = my_data$rel_comparator,
#'   n_sample = my_data$n_sample
#' )
#' }
#'
#' @export es_disattenuate
#'
#' @references
#' Hunter, J. E., & Schmidt, F. L. (2004). Methods of Meta-Analysis: Correcting
#' Error and Bias in Research Findings (2nd ed.). Sage Publications.
#'
#' Spearman, C. (1904). The proof and measurement of association between two things.
#' The American Journal of Psychology, 15(1), 72-101.
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of
#' research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @md
#'
#' @return
#' A data.frame with the corrected correlation and its Fisher's z
#' transformation (with their standard errors and 95% CIs), and the
#' attenuation factor.
#'
#' @examples
#' es_disattenuate(r = 0.50, r_se = 0.05,
#'                 reliability_x = 0.85, reliability_y = 0.80,
#'                 n_sample = 100)
#'
#' # Only correct for one measure's unreliability (set other to 1)
#' es_disattenuate(r = 0.50, r_se = 0.05,
#'                 reliability_x = 0.85, reliability_y = 1.0,
#'                 n_sample = 100)
es_disattenuate <- function(r, r_se, reliability_x, reliability_y, n_sample) {

  if (missing(n_sample)) n_sample <- rep(NA_real_, length(r))
  if (missing(r_se)) r_se <- rep(NA_real_, length(r))
  # Recycle scalar inputs to the length of r: the masked indexing below
  # (n_sample[need_se]) NA-pads when the vector is shorter than the mask,
  # which used to silently yield NA SEs for every row after the first.
  if (length(r_se) == 1) r_se <- rep(r_se, length(r))
  if (length(n_sample) == 1) n_sample <- rep(n_sample, length(r))
  if (length(reliability_x) == 1) reliability_x <- rep(reliability_x, length(r))
  if (length(reliability_y) == 1) reliability_y <- rep(reliability_y, length(r))

  # When r_se is not supplied, derive it from the sample size using the
  # large-sample Pearson correlation variance (Cooper et al., 2019):
  # SE(r) = sqrt((1 - r^2)^2 / (n - 1)).
  need_se <- is.na(r_se) & !is.na(n_sample) & !is.na(r)
  r_se[need_se] <- sqrt((1 - r[need_se]^2)^2 / (n_sample[need_se] - 1))

  if (any(!is.na(r) & abs(r) > 1, na.rm = TRUE)) {
    warning("Observed r outside [-1, 1] detected; the disattenuated r is invalid")
  }
  if (any(!is.na(reliability_x) & (reliability_x <= 0 | reliability_x > 1), na.rm = TRUE)) {
    warning("reliability_x outside (0, 1] detected; corrected r may be invalid or Inf")
  }
  if (any(!is.na(reliability_y) & (reliability_y <= 0 | reliability_y > 1), na.rm = TRUE)) {
    warning("reliability_y outside (0, 1] detected; corrected r may be invalid or Inf")
  }

  A <- sqrt(reliability_x * reliability_y)

  r_corrected <- r / A

  # first-order approx
  r_corrected_se <- r_se / A

  # Single constant for the extreme trigger AND the atanh bound: a corrected r
  # beyond it has no meaningful Fisher's z, so every derived-scale output is
  # suppressed below (the bounded value never surfaces in the output).
  extreme_threshold <- 0.999
  extreme <- which(!is.na(r_corrected) & abs(r_corrected) > extreme_threshold)
  if (length(extreme) > 0) {
    warning(sprintf(
      "Corrected r exceeds %s in %d row(s); the CI and Fisher's z outputs are set to NA as unreliable. Check the reliability inputs (and the observed r) - this often reflects a reliability value that is too low for the observed correlation.",
      extreme_threshold, length(extreme)
    ), call. = FALSE)
  }
  # bound corrected r to avoid Inf in atanh (extreme rows are NA'd below)
  r_bounded <- pmin(pmax(r_corrected, -extreme_threshold), extreme_threshold)
  z_corrected <- atanh(r_bounded)
  # dz/dr = 1/(1-r^2)
  z_corrected_se <- r_corrected_se / (1 - r_bounded^2)
  z_corrected_ci_lo <- z_corrected - qnorm(0.975) * z_corrected_se
  z_corrected_ci_up <- z_corrected + qnorm(0.975) * z_corrected_se

  # CI from back-transformed z interval
  r_corrected_ci_lo <- tanh(z_corrected_ci_lo)
  r_corrected_ci_up <- tanh(z_corrected_ci_up)
  # For extreme rows, NA every derived-scale output (r CI AND the z columns):
  # a bounded atanh would be a pure artifact (identical whatever the input),
  # and r_c >= 1 has no valid Fisher's z or SE. Only the point estimate and
  # its first-order SE are kept.
  if (length(extreme) > 0) {
    r_corrected_ci_lo[extreme] <- NA_real_
    r_corrected_ci_up[extreme] <- NA_real_
    z_corrected[extreme] <- NA_real_
    z_corrected_se[extreme] <- NA_real_
    z_corrected_ci_lo[extreme] <- NA_real_
    z_corrected_ci_up[extreme] <- NA_real_
  }

  result <- data.frame(
    r_corrected = r_corrected,
    r_corrected_se = r_corrected_se,
    r_corrected_ci_lo = r_corrected_ci_lo,
    r_corrected_ci_up = r_corrected_ci_up,
    z_corrected = z_corrected,
    z_corrected_se = z_corrected_se,
    z_corrected_ci_lo = z_corrected_ci_lo,
    z_corrected_ci_up = z_corrected_ci_up,
    attenuation_factor = A
  )

  return(result)
}

.normalize_smd_var <- function(smd_var) {
  # Map the user-facing names for the SMD sampling-variance convention onto the
  # internal canonical labels "LS" and "LS2" (metafor's vtype names). The
  # user-facing names follow the package convention of author names (as for
  # smd_to_cor = "viechtbauer", pre_post_to_smd = "bonett", etc.):
  #   "borenstein" (default, current metaConvert behaviour) = Borenstein et al.
  #     (2009, Introduction to Meta-Analysis, ch. 4): v_d = 1/n1 + 1/n2 + d^2/(2N)
  #     then v_g = cm^2 * v_d  (bit-exact with metafor vtype = "LS2").
  #   "hedges_olkin" (opt-in) = Hedges & Olkin (1985) / Viechtbauer (2007),
  #     metafor's own default: v_g = 1/n1 + 1/n2 + g^2/(2N), v_d = v_g / cm^2. This
  #     unifies the endpoint/between-group family with the pre/post family.
  # The metafor codes "LS2"/"LS" are kept as accepted synonyms.
  lut <- c(
    "borenstein"  = "LS2",
    "hedges_olkin" = "LS", "hedges-olkin" = "LS", "viechtbauer" = "LS"
  )
  key <- tolower(as.character(smd_var))
  # NA (e.g. a blank cell in a per-row smd_var column on a direct call) falls
  # back to the package default, mirroring .normalize_smd_denom's NA -> "pooled".
  key[is.na(smd_var)] <- "borenstein"
  out <- unname(lut[key])
  if (any(is.na(out))) {
    stop(paste0(
      "'", paste(unique(smd_var[is.na(out)]), collapse = "', '"),
      "' not in tolerated values for the 'smd_var' argument. Possible inputs are: ",
      "'borenstein' (the default) or 'hedges_olkin' (alias 'viechtbauer')."
    ), call. = FALSE)
  }
  out
}

.normalize_smd_denom <- function(smd_denom) {
  # Map the user-facing names for the SMD standardiser onto the internal canonical
  # labels "pooled" / "control" / "control_robust". Glass's delta (the control-SD
  # standardiser) follows the package's author/name convention as "glass"; the
  # descriptive "control" / "control_robust" spellings are kept as accepted synonyms.
  # Returns NA for unrecognised values so the caller can raise a targeted error.
  lut <- c(
    "pooled" = "pooled",
    "glass" = "control", "control" = "control",
    "glass_robust" = "control_robust", "control_robust" = "control_robust"
  )
  unname(lut[tolower(as.character(smd_denom))])
}

.d_j <- function(x) {
  # Hedges' small-sample correction J(df) is only defined for df > 1.
  # When df <= 1, the correction (and hence Hedges' g) is returned as NA.
  bad <- which(!is.na(x) & x <= 1)
  if (length(bad) > 0) {
    warning(sprintf(
      "Hedges' J correction is undefined for df <= 1; %d row(s) had df <= 1 and will produce NA for Hedges' g. This usually reflects extremely small samples (n_exp + n_nexp - 2 <= 1).",
      length(bad)
    ), call. = FALSE)
  }
  j <- ifelse(x <= 1, NA, 1) * exp(lgamma(x / 2) - 0.5 * log(x / 2) - lgamma((x - 1) / 2))
  return(j)
}

.glass_from_means <- function(mean_exp, mean_sd_exp, mean_nexp, mean_sd_nexp,
                              n_exp, n_nexp, smd_to_cor = "viechtbauer",
                              smd_var = "borenstein", reverse, robust = FALSE) {
  # Glass's delta: the endpoint mean difference standardised by the CONTROL
  # (non-experimental) endpoint SD instead of the pooled SD. Bit-exact with
  # metafor measure = "SMD1" (homoscedastic variance, with the vtype matched to
  # smd_var: "borenstein" = LS2, "hedges_olkin" = LS) and "SMD1H" (robust,
  # heteroscedasticity-consistent variance). glass_robust/SMD1H has a single
  # variance form, so smd_var is ignored for it. The Hedges small-sample
  # correction and every variance term use the control-group degrees of freedom
  # (n_nexp - 1), NOT the pooled df (this is what distinguishes SMD1 from SMD).
  if (missing(reverse)) reverse <- rep(FALSE, length(mean_exp))
  reverse[is.na(reverse)] <- FALSE
  smd_var <- .normalize_smd_var(smd_var)
  if (length(smd_var) == 1) smd_var <- rep(smd_var, length(mean_exp))

  di   <- (mean_exp - mean_nexp) / mean_sd_nexp           # uncorrected Glass delta
  df_g <- n_nexp - 1
  cm_g <- .d_j(df_g)                                      # cm(n_nexp - 1)
  g    <- cm_g * di

  # g-scale sampling variance
  vg_hom <- ifelse(smd_var == "LS",
    1 / n_exp + 1 / n_nexp + g^2 / (2 * n_nexp),                          # SMD1, vtype "LS"
    cm_g^2 * (1 / n_exp + 1 / n_nexp + di^2 / (2 * n_nexp))               # SMD1, vtype "LS2"
  )
  vg_rob <- (mean_sd_exp^2 / mean_sd_nexp^2) / (n_exp - 1) +
    1 / (n_nexp - 1) + g^2 / (2 * (n_nexp - 1))                           # SMD1H
  vg   <- if (robust) vg_rob else vg_hom
  g_se <- sqrt(vg)
  d_se <- g_se / cm_g                                     # d-scale SE (g_se = cm * d_se)

  # r / z / logOR / d scaffolding from the uncorrected Glass delta + its SE
  es <- .es_from_d(
    d = di, d_se = d_se, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, reverse = reverse
  )

  # override g with the control-df correction (.es_from_d used the pooled df)
  g_signed <- ifelse(reverse, -g, g)
  es$g      <- g_signed
  es$g_se   <- g_se
  es$g_ci_lo <- g_signed - qt(.975, df_g) * g_se
  es$g_ci_up <- g_signed + qt(.975, df_g) * g_se
  # keep the d CI on the same (control) df as g for internal consistency
  es$d_ci_lo <- es$d - qt(.975, df_g) * es$d_se
  es$d_ci_up <- es$d + qt(.975, df_g) * es$d_se
  es
}

.es_from_d <- function(d, d_se, n_exp, n_nexp, n_sample, smd_to_cor = "viechtbauer",
                       adjusted, n_cov_ancova, cov_outcome_r, reverse,
                       smd_var = "borenstein") {
  if (missing(d_se)) d_se <- rep(NA_real_, length(d))
  if (missing(n_exp)) n_exp <- rep(NA_real_, length(d))
  if (missing(n_nexp)) n_nexp <- rep(NA_real_, length(d))
  if (missing(n_sample)) n_sample <- rep(NA_real_, length(d))
  if (missing(adjusted)) adjusted <- rep(FALSE, length(d))
  if (missing(n_cov_ancova)) n_cov_ancova <- rep(0, length(d))
  if (missing(cov_outcome_r)) cov_outcome_r <- rep(0.5, length(d))
  if (missing(reverse)) reverse <- rep(FALSE, length(d))
  reverse[is.na(reverse)] <- FALSE
  if (length(reverse) == 1) reverse = c(rep(reverse, length(d)))
  if (length(reverse) != length(d)) stop("The length of the 'reverse' argument is incorrectly specified.")
  if (length(adjusted) == 1) adjusted = c(rep(adjusted, length(d)))
  if (length(adjusted) != length(d)) stop("The length of the 'adjusted' argument is incorrectly specified.")


  if (!all(smd_to_cor %in% c("viechtbauer", "lipsey_cooper"))) {
    stop(paste0(
      "'",
      unique(smd_to_cor[!smd_to_cor %in% c("viechtbauer", "lipsey_cooper")]),
      "' not in tolerated values for the 'smd_to_cor' argument.",
      " Possible inputs are: 'viechtbauer', 'lipsey_cooper'"
    ))
  }

  smd_var <- .normalize_smd_var(smd_var)
  if (length(smd_var) == 1) smd_var <- rep(smd_var, length(d))
  if (length(smd_var) != length(d)) stop("The length of the 'smd_var' argument is incorrectly specified.")
  use_LS <- smd_var == "LS"

  # ========= FLIP THE EFFECT SIZE ========== #
  d <- ifelse(reverse, -d, d)
  # ========= homogeneize sample sizes ========== #
  n_sample <- ifelse(!is.na(n_sample), n_sample, n_exp + n_nexp)
  n_exp <- ifelse(!is.na(n_exp), n_exp, n_sample / 2)
  n_nexp <- ifelse(!is.na(n_nexp), n_nexp, n_sample / 2)

  df <- ifelse(adjusted,
    n_exp + n_nexp - 2 - n_cov_ancova,
    n_exp + n_nexp - 2
  )

  # ========= d_se ========= #
  # Large-sample default variance (only used for rows where d_se was not supplied).
  # The leading term is 1/n1 + 1/n2 (adjusted for the covariate multiple correlation
  # in the ANCOVA case). g_se = d_se * J holds in BOTH conventions (see below), so the
  # only thing smd_var changes is the *default* d_se:
  #   LS2 (default): v_d = leading + d^2/(2N)                  -> v_g = cm^2 * v_d
  #   LS  (opt-in) : v_g = leading + g^2/(2N), v_d = v_g/cm^2  (g = d * J, metafor default)
  leading <- ifelse(adjusted,
    (n_exp + n_nexp) / (n_exp * n_nexp) * (1 - cov_outcome_r^2),
    (n_exp + n_nexp) / (n_exp * n_nexp)
  )
  if (any(use_LS)) {
    J_var <- suppressWarnings(.d_j(df))
    d_se_ls <- sqrt((leading + (d * J_var)^2 / (2 * (n_exp + n_nexp))) / J_var^2)
  } else {
    d_se_ls <- rep(NA_real_, length(d))
  }
  d_se <- ifelse(
    !is.na(d_se),
    d_se,
    ifelse(use_LS,
      d_se_ls,
      sqrt(leading + d^2 / (2 * (n_exp + n_nexp)))
    )
  )

  d_ci_lo <- d - d_se * qt(.975, df)
  d_ci_up <- d + d_se * qt(.975, df)

  # ========= OR ========= #
  logor <- d * pi / sqrt(3)
  logor_se <- sqrt(d_se^2 * pi^2 / 3)
  logor_ci_lo <- logor - logor_se * qnorm(.975)
  logor_ci_up <- logor + logor_se * qnorm(.975)

  #######################################################
  # track non missing information for speedy conversion #
  #######################################################
  nn_miss <- which(!is.na(d) & !is.na(d_se) & !is.na(n_exp) & !is.na(n_nexp))

  g <- g_se <- g_ci_lo <- g_ci_up <-
    r <- r_se <- r_ci_lo <- r_ci_up <-
    z <- z_se <- z_ci_lo <- z_ci_up <- rep(NA, length(d))

  # ========= g ========= #
  J <- .d_j(df[nn_miss])
  g[nn_miss] <- d[nn_miss] * J
  g_se[nn_miss] <- sqrt(d_se[nn_miss]^2 * (J^2))
  g_ci_lo[nn_miss] <- g[nn_miss] - g_se[nn_miss] * qt(.975, df[nn_miss])
  g_ci_up[nn_miss] <- g[nn_miss] + g_se[nn_miss] * qt(.975, df[nn_miss])


  # ========= r/Z ========= #
  dat_r <- data.frame(
    d = d, vd = d_se^2, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, n_cov_ancova = n_cov_ancova
  )

  if (length(nn_miss) != 0) {
    cor <- t(mapply(.smd_to_cor,
      d = dat_r$d[nn_miss],
      vd = dat_r$vd[nn_miss],
      n_exp = dat_r$n_exp[nn_miss],
      n_nexp = dat_r$n_nexp[nn_miss],
      smd_to_cor = dat_r$smd_to_cor[nn_miss],
      n_cov_ancova = dat_r$n_cov_ancova[nn_miss]
    ))

    r[nn_miss] <- cor[, 1]
    r_se[nn_miss] <- sqrt(cor[, 2])
    r_ci_lo[nn_miss] <- cor[, 3]
    r_ci_up[nn_miss] <- cor[, 4]
    z[nn_miss] <- cor[, 5]
    z_se[nn_miss] <- sqrt(cor[, 6])
    z_ci_lo[nn_miss] <- cor[, 7]
    z_ci_up[nn_miss] <- cor[, 8]
  }

  res <- data.frame(
    d, d_se, d_ci_lo, d_ci_up,
    g, g_se, g_ci_lo, g_ci_up,
    r, r_se, r_ci_lo, r_ci_up,
    z, z_se, z_ci_lo, z_ci_up,
    logor, logor_se, logor_ci_lo, logor_ci_up
  )

  return(res)
}

#' Convert a t-statistic obtained from an ANCOVA model into several effect size measures.
#'
#' @param ancova_t a t-statistic from an ANCOVA (binary predictor)
#' @param cov_outcome_r correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_t a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function first computes an "adjusted" Cohen's d (D), and
#' Hedges' g (G) from the t-value of an ANCOVA (binary predictor).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate a Cohen's d** the formula used is (table 12.3 in Cooper):
#' \deqn{cohen\_d =  ancova\_t* \sqrt{\frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp}}\sqrt{1 - cov\_out\_cor^2}}
#'
#' **To estimate other effect size measures**,
#' Calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 18. Adjusted: ANCOVA statistics, eta-squared'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @note
#' The Cohen's d point estimate is recovered from the reported ANCOVA t (which
#' already embeds the covariate adjustment), but its sampling variance is rebuilt
#' from Cooper's eq. 12.26, which assumes a covariate balanced across groups and
#' treats \code{cov_outcome_r} as known. In observational or otherwise
#' covariate-imbalanced designs the standard error is a lower bound
#' (anti-conservative), while the point estimate remains unbiased. See Lai and
#' Kelley (2012).
#'
#' @references
#' Cooper, H., Hedges, L. V., & Valentine, J. C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Lai, K., & Kelley, K. (2012). Accuracy in parameter estimation for ANCOVA and ANOVA contrasts: Sample size planning via narrow confidence intervals. British Journal of Mathematical and Statistical Psychology, 65(2), 350-370.
#'
#' @export es_from_ancova_t
#'
#' @md
#'
#' @examples
#' es_from_ancova_t(ancova_t = 2, cov_outcome_r = 0.2, n_cov_ancova = 3, n_exp = 20, n_nexp = 20)
es_from_ancova_t <- function(ancova_t, cov_outcome_r, n_cov_ancova, n_exp, n_nexp,
                             smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_ancova_t) {
  if (missing(reverse_ancova_t)) reverse_ancova_t <- rep(FALSE, length(ancova_t))
  reverse_ancova_t[is.na(reverse_ancova_t)] <- FALSE

  d <- ancova_t * sqrt(1/n_exp + 1/n_nexp) * sqrt(1 - cov_outcome_r^2)

  es <- .es_from_d(
    d = d, adjusted = TRUE, cov_outcome_r = cov_outcome_r,
    n_cov_ancova = n_cov_ancova, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_ancova_t
  )

  es$info_used <- "ancova_t"

  return(es)
}

#' Convert a F-statistic obtained from an ANCOVA model into several effect size measures.
#'
#' @param ancova_f a F-statistic from an ANCOVA (binary predictor)
#' @param cov_outcome_r correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_f a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function first computes an "adjusted" Cohen's d (D), and
#' Hedges' g (G) from the F-value of an ANCOVA (binary predictor).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate a Cohen's d** the formula used is (table 12.3 in Cooper):
#' \deqn{cohen\_d =  \sqrt{ancova\_f * \frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp}} * \sqrt{1 - cov\_out\_cor^2}}
#'
#' **To estimate other effect size measures**,
#' Calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 18. Adjusted: ANCOVA statistics, eta-squared'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L. V., & Valentine, J. C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_ancova_f
#'
#' @md
#'
#' @examples
#' es_from_ancova_f(ancova_f = 4, cov_outcome_r = 0.2, n_cov_ancova = 3, n_exp = 20, n_nexp = 20)
es_from_ancova_f <- function(ancova_f, cov_outcome_r, n_cov_ancova, n_exp, n_nexp,
                             smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_ancova_f) {
  if (missing(reverse_ancova_f)) reverse_ancova_f <- rep(FALSE, length(ancova_f))
  reverse_ancova_f[is.na(reverse_ancova_f)] <- FALSE

  t <- sqrt(ancova_f)

  es <- es_from_ancova_t(
    ancova_t = t, cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_t = reverse_ancova_f
  )

  es$info_used <- "ancova_f"

  return(es)
}

#' Convert a two-tailed p-value of an ANCOVA t-test into several effect size measures.
#'
#' @param ancova_t_pval a two-tailed p-value of a t-test in an ANCOVA (binary predictor)
#' @param cov_outcome_r correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_t_pval a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the p-value of an ANCOVA (binary predictor) into a t value,
#' and then relies on the calculations of the \code{\link{es_from_ancova_t}()} function.
#'
#' **To convert the p-value into a t-value,** the following formula is used (table 12.3 in Cooper):
#' \deqn{df = n\_exp + n\_nexp - 2 - n\_cov\_ancova}
#' \deqn{t = | qt(ancova\_t\_pval/2, df = df) |}
#' Then, calculations of the \code{\link{es_from_ancova_t}()} are applied.
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 18. Adjusted: ANCOVA statistics, eta-squared'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_ancova_t_pval
#'
#' @md
#'
#' @examples
#' es_from_ancova_t_pval(
#'   ancova_t_pval = 0.05, cov_outcome_r = 0.2,
#'   n_cov_ancova = 3, n_exp = 20, n_nexp = 20
#' )
es_from_ancova_t_pval <- function(ancova_t_pval, cov_outcome_r, n_cov_ancova, n_exp, n_nexp,
                                  smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_ancova_t_pval) {
  if (missing(reverse_ancova_t_pval)) reverse_ancova_t_pval <- rep(FALSE, length(ancova_t_pval))
  reverse_ancova_t_pval[is.na(reverse_ancova_t_pval)] <- FALSE

  t_inv <- abs(qt(
    p = ancova_t_pval / 2,
    df = n_exp + n_nexp - 2 - n_cov_ancova,
    lower.tail = FALSE
  ))

  es <- es_from_ancova_t(
    ancova_t = t_inv, cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_t = reverse_ancova_t_pval
  )

  es$info_used <- "ancova_t_pval"

  return(es)
}

#' Convert a two-tailed p-value of an ANCOVA t-test into several effect size measures.
#'
#' @param ancova_f_pval a two-tailed p-value of an F-test in an ANCOVA (binary predictor)
#' @param cov_outcome_r correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_f_pval a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the p-value of an ANCOVA (binary predictor) into a t value,
#' and then relies on the calculations of the \code{\link{es_from_ancova_t}()} function.
#'
#' **To convert the p-value into a t-value,** the following formula is used (table 12.3 in Cooper):
#' \deqn{df = n\_exp + n\_nexp - 2 - n\_cov\_ancova}
#' \deqn{t = | qt(ancova\_f\_pval/2, df = df) |}
#' Then, calculations of the \code{\link{es_from_ancova_t}()} are applied.
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 18. Adjusted: ANCOVA statistics, eta-squared'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_ancova_f_pval
#'
#' @md
#'
#' @examples
#' es_from_ancova_f_pval(
#'   ancova_f_pval = 0.05, cov_outcome_r = 0.2,
#'   n_cov_ancova = 3, n_exp = 20, n_nexp = 20
#' )
es_from_ancova_f_pval <- function(ancova_f_pval, cov_outcome_r, n_cov_ancova, n_exp, n_nexp,
                                  smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_ancova_f_pval) {
  if (missing(reverse_ancova_f_pval)) reverse_ancova_f_pval <- rep(FALSE, length(ancova_f_pval))
  reverse_ancova_f_pval[is.na(reverse_ancova_f_pval)] <- FALSE

  t_inv <- abs(qt(
    p = ancova_f_pval / 2,
    df = n_exp + n_nexp - 2 - n_cov_ancova,
    lower.tail = FALSE
  ))

  es <- es_from_ancova_t(
    ancova_t = t_inv, cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_t = reverse_ancova_f_pval
  )

  es$info_used <- "ancova_f_pval"

  return(es)
}

#' Convert pre-post means of two independent groups into various effect size measures
#'
#' @param mean_pre_exp mean of the experimental/exposed group at baseline
#' @param mean_exp mean of the experimental/exposed group at follow up
#' @param mean_pre_sd_exp standard deviation of the experimental/exposed group at baseline
#' @param mean_sd_exp standard deviation of the experimental/exposed group at follow up
#' @param mean_pre_nexp mean of the non-experimental/non-exposed group at baseline
#' @param mean_nexp mean of the non-experimental/non-exposed group at follow up
#' @param mean_pre_sd_nexp standard deviation of the non-experimental/non-exposed group at baseline
#' @param mean_sd_nexp standard deviation of the non-experimental/non-exposed group at follow up
#' @param n_exp number of the experimental/exposed group
#' @param n_nexp number of the non-experimental/non-exposed group
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group
#' @param pre_post_to_smd formula used to convert the pre and post means/SD into a SMD (see details).
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pool_sd a logical value indicating whether the standardizing SD should be pooled across the two
#'   groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
#'   equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
#'   which to prefer, so this is a deliberate choice and not a technical detail.
#'   \itemize{
#'     \item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
#'       within-group values are subtracted, their variances adding because the arms are independent. This
#'       is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
#'       SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
#'       broadly applicable of the two.
#'     \item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
#'       is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
#'       assumes the two arms' true standardizing SDs are equal.
#'   }
#' @param reverse_means_pre_post a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts pre-post means of two independent groups into a Cohen's d (D) and Hedges' g (G).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **Four approaches can be used to compute the Cohen's d.**
#'
#' In these approaches, the standard deviation of the difference within each group first needs to be obtained:
#' \deqn{adj\_exp = 2*r\_pre\_post\_exp*mean\_pre\_sd\_exp*mean\_sd\_exp}
#' \deqn{sd\_change\_exp = \sqrt{mean\_pre\_sd\_exp^2 + mean\_sd\_exp^2 - adj\_exp}}
#' \deqn{adj\_nexp = 2*r\_pre\_post\_nexp*mean\_pre\_sd\_nexp*mean\_sd\_nexp}
#' \deqn{sd\_change\_nexp = \sqrt{mean\_pre\_sd\_nexp^2 + mean\_sd\_nexp^2 - adj\_nexp}}
#'
#' 1. In the approach described by Bonett (\code{pre_post_to_smd = "bonett"}), one Cohen's d per group is obtained by
#' standardizing the pre-post mean difference by the standard deviation at baseline (Bonett, 2008):
#' \deqn{cohen\_d\_exp = \frac{mean\_exp - mean\_pre\_exp}{mean\_pre\_sd\_exp}}
#' \deqn{cohen\_d\_nexp = \frac{mean\_nexp - mean\_pre\_nexp}{mean\_pre\_sd\_nexp}}
#'
#' 2. In the approach described by Cooper (\code{pre_post_to_smd = "cooper"} or \code{"morris_drm"}), the following
#' formulas are used (Cooper et al., 2019; Morris & DeShon, 2002):
#' \deqn{cohen\_d\_exp = \frac{mean\_exp - mean\_pre\_exp}{sd\_change\_exp} * \sqrt{2 * (1 - r\_pre\_post\_exp)}}
#' \deqn{cohen\_d\_nexp = \frac{mean\_nexp - mean\_pre\_nexp}{sd\_change\_nexp} * \sqrt{2 * (1 - r\_pre\_post\_nexp)}}
#'
#' 3. In the approach described by Morris & DeShon (\code{pre_post_to_smd = "morris_dz"}), the pre-post mean difference
#' is standardized by the standard deviation of the change (Morris & DeShon, 2002):
#' \deqn{cohen\_d\_exp = \frac{mean\_exp - mean\_pre\_exp}{sd\_change\_exp}}
#' \deqn{cohen\_d\_nexp = \frac{mean\_nexp - mean\_pre\_nexp}{sd\_change\_nexp}}
#'
#' 4. In the approach described by Morris (\code{pre_post_to_smd = "morris_dav"}), the pre-post mean
#' difference is standardized by the average of the baseline and follow up SDs (Morris, 2008). Note that
#' Morris (2008, p.384) recommended \emph{against} this standardizer because its sampling variance was
#' unknown to him; the variance used here is the heteroscedasticity-robust form of Bonett (2008, eq. 10/19):
#' \deqn{cohen\_d\_exp = \frac{mean\_exp - mean\_pre\_exp}{\sqrt{(mean\_pre\_sd\_exp^2 + mean\_sd\_exp^2)/2}}}
#' \deqn{cohen\_d\_nexp = \frac{mean\_nexp - mean\_pre\_nexp}{\sqrt{(mean\_pre\_sd\_nexp^2 + mean\_sd\_nexp^2)/2}}}
#'
#' Last, the Cohen's d reflecting the within-group change from baseline to follow-up are combined into one Cohen's d:
#' \deqn{cohen\_d = d\_exp - d\_nexp}
#' \deqn{cohen\_d\_se = \sqrt{cohen\_d\_se\_exp^2 + cohen\_d\_se\_nexp^2}}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.
#'
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386. https://doi.org/10.1177/1094428106291059
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @export es_from_means_sd_pre_post
#'
#' @md
#'
#' @examples
#' es_from_means_sd_pre_post(
#'   n_exp = 36, n_nexp = 35,
#'   mean_pre_exp = 98, mean_exp = 102,
#'   mean_pre_sd_exp = 16, mean_sd_exp = 17,
#'   mean_pre_nexp = 96, mean_nexp = 102,
#'   mean_pre_sd_nexp = 14, mean_sd_nexp = 15,
#'   r_pre_post_exp = 0.8, r_pre_post_nexp = 0.8
#' )
es_from_means_sd_pre_post <- function(mean_pre_exp, mean_exp, mean_pre_sd_exp, mean_sd_exp,
                                      mean_pre_nexp, mean_nexp, mean_pre_sd_nexp, mean_sd_nexp,
                                      n_exp, n_nexp, r_pre_post_exp, r_pre_post_nexp,
                                      smd_to_cor = "viechtbauer",
                                      pre_post_to_smd = "bonett",
                                      pool_sd = FALSE,
                                      reverse_means_pre_post) {
  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("bonett", "morris_drm", "morris_dz", "morris_dav"),
    context = "pre_post_means",
    func_name = "es_from_means_sd_pre_post"
  )

  if (missing(reverse_means_pre_post)) reverse_means_pre_post <- rep(FALSE, length(mean_pre_exp))
  reverse_means_pre_post[is.na(reverse_means_pre_post)] <- FALSE
  if (length(reverse_means_pre_post) == 1) reverse_means_pre_post = c(rep(reverse_means_pre_post, length(mean_pre_exp)))
  if (length(reverse_means_pre_post) != length(mean_pre_exp)) stop("The length of the 'reverse_means_pre_post' argument is incorrectly specified.")

  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(mean_pre_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_pre_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  g <- g_se <- g_ci_lo <- g_ci_up <-
    d <- d_se <- d_ci_lo <- d_ci_up <- rep(NA, length(mean_pre_exp))

  nn_miss <- which(!is.na(mean_pre_exp) & !is.na(mean_pre_sd_exp) &
    !is.na(mean_pre_nexp) & !is.na(mean_pre_sd_nexp) &
    !is.na(mean_exp) & !is.na(mean_sd_exp) &
    !is.na(mean_nexp) & !is.na(mean_sd_nexp) &
    !is.na(r_pre_post_exp) & !is.na(r_pre_post_nexp) &
    !is.na(n_exp) & !is.na(n_nexp))

  dat_smd_pre_post <- data.frame(
    mean_pre_exp = mean_pre_exp, mean_pre_sd_exp = mean_pre_sd_exp,
    mean_exp = mean_exp, mean_sd_exp = mean_sd_exp,
    mean_pre_nexp = mean_pre_nexp, mean_pre_sd_nexp = mean_pre_sd_nexp,
    mean_nexp = mean_nexp, mean_sd_nexp = mean_sd_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,
    pre_post_to_smd = pre_post_to_smd,
    pool_sd = pool_sd
  )
  if (length(nn_miss) != 0) {
    smd_pp <- t(mapply(.pre_post_to_smd,
      mean_pre_exp = dat_smd_pre_post$mean_pre_exp[nn_miss],
      mean_pre_sd_exp = dat_smd_pre_post$mean_pre_sd_exp[nn_miss],
      mean_exp = dat_smd_pre_post$mean_exp[nn_miss],
      mean_sd_exp = dat_smd_pre_post$mean_sd_exp[nn_miss],
      mean_pre_nexp = dat_smd_pre_post$mean_pre_nexp[nn_miss],
      mean_pre_sd_nexp = dat_smd_pre_post$mean_pre_sd_nexp[nn_miss],
      mean_nexp = dat_smd_pre_post$mean_nexp[nn_miss],
      mean_sd_nexp = dat_smd_pre_post$mean_sd_nexp[nn_miss],
      n_exp = dat_smd_pre_post$n_exp[nn_miss],
      n_nexp = dat_smd_pre_post$n_nexp[nn_miss],
      r_pre_post_exp = dat_smd_pre_post$r_pre_post_exp[nn_miss],
      r_pre_post_nexp = dat_smd_pre_post$r_pre_post_nexp[nn_miss],
      pre_post_to_smd = dat_smd_pre_post$pre_post_to_smd[nn_miss],
      pool_sd = dat_smd_pre_post$pool_sd[nn_miss]
    ))

    d[nn_miss] <- smd_pp[, 1]
    d_se[nn_miss] <- sqrt(smd_pp[, 2])
    d_ci_lo[nn_miss] <- smd_pp[, 3]
    d_ci_up[nn_miss] <- smd_pp[, 4]
    g[nn_miss] <- smd_pp[, 5]
    g_se[nn_miss] <- sqrt(smd_pp[, 6])
    g_ci_lo[nn_miss] <- smd_pp[, 7]
    g_ci_up[nn_miss] <- smd_pp[, 8]
  }

  es <- .es_from_d(
    d = d, d_se = d_se, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, reverse = reverse_means_pre_post
  )

  # d/g CIs from .pre_post_to_smd
  es$d <- ifelse(reverse_means_pre_post, -d, d)
  es$d_se <- d_se
  es$d_ci_lo <- ifelse(reverse_means_pre_post, -d_ci_up, d_ci_lo)
  es$d_ci_up <- ifelse(reverse_means_pre_post, -d_ci_lo, d_ci_up)

  es$g <- ifelse(reverse_means_pre_post, -g, g)
  es$g_se <- g_se
  es$g_ci_lo <- ifelse(reverse_means_pre_post, -g_ci_up, g_ci_lo)
  es$g_ci_up <- ifelse(reverse_means_pre_post, -g_ci_lo, g_ci_up)

  md_exp <- mean_exp - mean_pre_exp
  md_var_exp <- (mean_pre_sd_exp^2 + mean_sd_exp^2 -
    2 * r_pre_post_exp * mean_pre_sd_exp * mean_sd_exp) / n_exp

  md_nexp <- mean_nexp - mean_pre_nexp
  md_var_nexp <- (mean_pre_sd_nexp^2 + mean_sd_nexp^2 -
    2 * r_pre_post_nexp * mean_pre_sd_nexp * mean_sd_nexp) / n_nexp

  es$md <- ifelse(reverse_means_pre_post, md_nexp - md_exp, md_exp - md_nexp)
  es$md_se <- sqrt(md_var_exp + md_var_nexp)
  es$md_ci_lo <- es$md - es$md_se * qt(.975, n_exp + n_nexp - 2)
  es$md_ci_up <- es$md + es$md_se * qt(.975, n_exp + n_nexp - 2)

  es$info_used <- "means_sd_pre_post"

  return(es)
}

#' Convert pre-post means of two independent groups into various effect size measures
#'
#' @param mean_pre_exp mean of the experimental/exposed group at baseline
#' @param mean_pre_se_exp standard error of the experimental/exposed group at baseline
#' @param mean_exp mean of the experimental/exposed group at follow up
#' @param mean_se_exp standard error of the experimental/exposed group at follow up
#' @param mean_pre_nexp mean of the non-experimental/non-exposed group at baseline
#' @param mean_pre_se_nexp standard error of the non-experimental/non-exposed group at baseline
#' @param mean_nexp mean of the non-experimental/non-exposed group at follow up
#' @param mean_se_nexp standard error of the non-experimental/non-exposed group at follow up
#' @param n_exp number of the experimental/exposed group
#' @param n_nexp number of the non-experimental/non-exposed group
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group
#' @param pre_post_to_smd formula used to convert the pre and post means/SD into a SMD (see details).
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pool_sd a logical value indicating whether the standardizing SD should be pooled across the two
#'   groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
#'   equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
#'   which to prefer, so this is a deliberate choice and not a technical detail.
#'   \itemize{
#'     \item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
#'       within-group values are subtracted, their variances adding because the arms are independent. This
#'       is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
#'       SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
#'       broadly applicable of the two.
#'     \item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
#'       is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
#'       assumes the two arms' true standardizing SDs are equal.
#'   }
#' @param reverse_means_pre_post a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the pre/post standard errors of two independent groups into standard deviations (Section 6.5.2.2 in the Cochrane Handbook).
#' \deqn{mean\_pre\_sd\_exp = mean\_pre\_se\_exp * \sqrt{n\_exp}}
#' \deqn{mean\_pre\_sd\_nexp = mean\_pre\_se\_nexp * \sqrt{n\_nexp}}
#' \deqn{mean\_sd\_exp = mean\_se\_exp * \sqrt{n\_exp}}
#' \deqn{mean\_sd\_nexp = mean\_se\_nexp * \sqrt{n\_nexp}}
#'
#' Then, calculations of the \code{\link{es_from_means_sd_pre_post}()} are applied.
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_means_se_pre_post
#'
#' @md
#'
#' @examples
#' es_from_means_sd_pre_post(
#'   n_exp = 36, n_nexp = 35,
#'   mean_pre_exp = 98, mean_exp = 102,
#'   mean_pre_sd_exp = 16, mean_sd_exp = 17,
#'   mean_pre_nexp = 96, mean_nexp = 102,
#'   mean_pre_sd_nexp = 14, mean_sd_nexp = 15,
#'   r_pre_post_exp = 0.8, r_pre_post_nexp = 0.8
#' )
es_from_means_se_pre_post <- function(mean_pre_exp, mean_exp, mean_pre_se_exp, mean_se_exp,
                                      mean_pre_nexp, mean_nexp, mean_pre_se_nexp, mean_se_nexp,
                                      n_exp, n_nexp, r_pre_post_exp, r_pre_post_nexp,
                                      smd_to_cor = "viechtbauer",
                                      pre_post_to_smd = "bonett",
                                      pool_sd = FALSE,
                                      reverse_means_pre_post) {
  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("bonett", "morris_drm", "morris_dz", "morris_dav"),
    context = "pre_post_means",
    func_name = "es_from_means_se_pre_post"
  )


  if (missing(reverse_means_pre_post)) reverse_means_pre_post <- rep(FALSE, length(mean_pre_exp))
  reverse_means_pre_post[is.na(reverse_means_pre_post)] <- FALSE
  if (length(reverse_means_pre_post) == 1) reverse_means_pre_post <- c(rep(reverse_means_pre_post, length(mean_pre_exp)))
  if (length(reverse_means_pre_post) != length(mean_pre_exp)) stop("The length of the 'reverse_means_pre_post' argument is incorrectly specified.")
  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(mean_pre_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_pre_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  sd_pre_exp <- mean_pre_se_exp * sqrt(n_exp)
  sd_exp <- mean_se_exp * sqrt(n_exp)
  sd_pre_nexp <- mean_pre_se_nexp * sqrt(n_nexp)
  sd_nexp <- mean_se_nexp * sqrt(n_nexp)

  es <- es_from_means_sd_pre_post(
    mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
    mean_pre_sd_exp = sd_pre_exp, mean_sd_exp = sd_exp,
    mean_pre_nexp = mean_pre_nexp, mean_nexp = mean_nexp,
    mean_pre_sd_nexp = sd_pre_nexp, mean_sd_nexp = sd_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    pool_sd = pool_sd,
    reverse_means_pre_post = reverse_means_pre_post
  )

  es$info_used <- "means_se_pre_post"

  return(es)
}

#' Convert pre-post means of two independent groups into various effect size measures
#'
#' @param mean_pre_exp mean of the experimental/exposed group at baseline
#' @param mean_exp mean of the experimental/exposed group at follow up
#' @param mean_pre_nexp mean of the non-experimental/non-exposed group at baseline
#' @param mean_nexp mean of the non-experimental/non-exposed group at follow up
#' @param mean_pre_ci_lo_exp lower bound of the 95% CI of the mean of the experimental/exposed group at baseline
#' @param mean_pre_ci_up_exp upper bound of the 95% CI of the mean of the experimental/exposed group at baseline
#' @param mean_ci_lo_exp lower bound of the 95% CI of the mean of the experimental/exposed group at follow up
#' @param mean_ci_up_exp upper bound of the 95% CI of the mean of the experimental/exposed group at follow up
#' @param mean_pre_ci_lo_nexp lower bound of the 95% CI of the mean of the non-experimental/non-exposed group at baseline
#' @param mean_pre_ci_up_nexp upper bound of the 95% CI of the mean of the non-experimental/non-exposed group at baseline
#' @param mean_ci_lo_nexp lower bound of the 95% CI of the mean of the non-experimental/non-exposed group at follow up
#' @param mean_ci_up_nexp upper bound of the 95% CI of the mean of the non-experimental/non-exposed group at follow up
#' @param n_exp number of the experimental/exposed group
#' @param n_nexp number of the non-experimental/non-exposed group
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group
#' @param pre_post_to_smd formula used to convert the pre and post means/SD into a SMD (see details).
#' @param max_asymmetry A percentage indicating the tolerance before detecting asymmetry in the 95% CI bounds.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pool_sd a logical value indicating whether the standardizing SD should be pooled across the two
#'   groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
#'   equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
#'   which to prefer, so this is a deliberate choice and not a technical detail.
#'   \itemize{
#'     \item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
#'       within-group values are subtracted, their variances adding because the arms are independent. This
#'       is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
#'       SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
#'       broadly applicable of the two.
#'     \item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
#'       is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
#'       assumes the two arms' true standardizing SDs are equal.
#'   }
#' @param reverse_means_pre_post a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the bounds of the 95% CI of the pre/post means of two independent groups into standard errors (Section 6.3.1 in the Cochrane Handbook).
#' \deqn{mean\_pre\_se\_exp = \frac{mean\_pre\_ci\_up\_exp - mean\_pre\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
#' \deqn{mean\_pre\_se\_nexp = \frac{mean\_pre\_ci\_up\_nexp - mean\_pre\_ci\_lo\_nexp}{2 * qt{(0.975, df = n\_nexp - 1)}}}
#' \deqn{mean\_se\_exp = \frac{mean\_ci\_up\_exp - mean\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
#' \deqn{mean\_se\_nexp = \frac{mean\_ci\_up\_nexp - mean\_ci\_lo\_nexp}{2 * qt{(0.975, df = n\_nexp - 1)}}}
#'
#' Then, calculations of the \code{\link{es_from_means_se_pre_post}} are applied.
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @export es_from_means_ci_pre_post
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_means_ci_pre_post(
#'   n_exp = 36, n_nexp = 35,
#'   mean_pre_exp = 98,
#'   mean_pre_ci_lo_exp = 88,
#'   mean_pre_ci_up_exp = 108,
#'   mean_exp = 102,
#'   mean_ci_lo_exp = 92,
#'   mean_ci_up_exp = 112,
#'   mean_pre_nexp = 96,
#'   mean_pre_ci_lo_nexp = 86,
#'   mean_pre_ci_up_nexp = 106,
#'   mean_nexp = 102,
#'   mean_ci_lo_nexp = 92,
#'   mean_ci_up_nexp = 112,
#'   r_pre_post_exp = 0.8, r_pre_post_nexp = 0.8
#' )
es_from_means_ci_pre_post <- function(mean_pre_exp, mean_exp,
                                      mean_pre_ci_lo_exp, mean_pre_ci_up_exp,
                                      mean_ci_lo_exp, mean_ci_up_exp,
                                      mean_pre_nexp, mean_nexp,
                                      mean_pre_ci_lo_nexp, mean_pre_ci_up_nexp,
                                      mean_ci_lo_nexp, mean_ci_up_nexp,
                                      n_exp, n_nexp, r_pre_post_exp, r_pre_post_nexp,
                                      smd_to_cor = "viechtbauer",
                                      pre_post_to_smd = "bonett", max_asymmetry = 10,
                                      pool_sd = FALSE,
                                      reverse_means_pre_post) {

  if (missing(reverse_means_pre_post)) reverse_means_pre_post <- rep(FALSE, length(mean_pre_exp))
  reverse_means_pre_post[is.na(reverse_means_pre_post)] <- FALSE
  if (length(reverse_means_pre_post) == 1) reverse_means_pre_post <- c(rep(reverse_means_pre_post, length(mean_pre_exp)))
  if (length(reverse_means_pre_post) != length(mean_pre_exp)) stop("The length of the 'reverse_means_pre_post' argument is incorrectly specified.")
  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(mean_pre_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_pre_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  .check_ci_asymmetry <- function(val, lo, up, label) {
    half_up <- abs(up - val)
    half_lo <- abs(val - lo)
    denom <- pmax(half_up, half_lo)
    asym_pct <- ifelse(denom == 0, 0, abs(half_up - half_lo) / denom * 100)
    bad <- which(!is.na(asym_pct) & asym_pct > max_asymmetry)
    if (length(bad) > 0) {
      warning(sprintf("Asymmetric CI detected for '%s' in row(s) %s (asymmetry > %s%%)",
                      label, paste(bad, collapse = ", "), max_asymmetry), call. = FALSE)
    }
  }
  .check_ci_asymmetry(mean_pre_exp, mean_pre_ci_lo_exp, mean_pre_ci_up_exp, "mean_pre_exp")
  .check_ci_asymmetry(mean_exp, mean_ci_lo_exp, mean_ci_up_exp, "mean_exp")
  .check_ci_asymmetry(mean_pre_nexp, mean_pre_ci_lo_nexp, mean_pre_ci_up_nexp, "mean_pre_nexp")
  .check_ci_asymmetry(mean_nexp, mean_ci_lo_nexp, mean_ci_up_nexp, "mean_nexp")

  df_exp <- n_exp - 1
  df_nexp <- n_nexp - 1

  se_pre_exp <- (mean_pre_ci_up_exp - mean_pre_ci_lo_exp) / (2 * qt(0.975, df_exp))
  se_exp <- (mean_ci_up_exp - mean_ci_lo_exp) / (2 * qt(0.975, df_exp))
  se_pre_nexp <- (mean_pre_ci_up_nexp - mean_pre_ci_lo_nexp) / (2 * qt(0.975, df_nexp))
  se_nexp <- (mean_ci_up_nexp - mean_ci_lo_nexp) / (2 * qt(0.975, df_nexp))

  es <- es_from_means_se_pre_post(
    mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
    mean_pre_se_exp = se_pre_exp, mean_se_exp = se_exp,
    mean_pre_nexp = mean_pre_nexp, mean_nexp = mean_nexp,
    mean_pre_se_nexp = se_pre_nexp, mean_se_nexp = se_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    pool_sd = pool_sd,
    reverse_means_pre_post = reverse_means_pre_post
  )

  es$info_used <- "means_ci_pre_post"

  return(es)
}

#' Convert a point-biserial correlation coefficient into several effect size measures
#'
#' @param pt_bis_r value of a point-biserial correlation coefficient
#' @param n_exp total number of participants in the exposed group
#' @param n_nexp total number of participants in the non exposed group
#' @param smd_to_cor formula used to convert the \code{pt_bis_r} value into a coefficient correlation.
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_pt_bis_r a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function uses a point-biserial correlation coefficient to estimate a
#' Cohen's d (D) and Hedges' g (G).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **The formula used to obtain the Cohen's d are (Viechtbauer, 2021)**:
#' \deqn{m = n\_exp + n\_nexp - 2}
#' \deqn{h = \frac{m}{n\_exp} + \frac{m}{n\_nexp}}
#' \deqn{d = \frac{pt\_bis\_r * \sqrt{h}}{\sqrt{1 - pt\_bis\_r^2}}}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Viechtbauer (2021). Accessed at: https://stats.stackexchange.com/questions/526789/convert-correlation-r-to-cohens-d-unequal-groups-of-known-size
#'
#' @export es_from_pt_bis_r
#'
#' @md
#'
#' @examples
#' es_from_pt_bis_r(pt_bis_r = 0.2, n_exp = 121, n_nexp = 121)
es_from_pt_bis_r <- function(pt_bis_r, n_exp, n_nexp, smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_pt_bis_r) {
  if (missing(reverse_pt_bis_r)) reverse_pt_bis_r <- rep(FALSE, length(pt_bis_r))
  reverse_pt_bis_r[is.na(reverse_pt_bis_r)] <- FALSE
  if (length(reverse_pt_bis_r) == 1) reverse_pt_bis_r = c(rep(reverse_pt_bis_r, length(pt_bis_r)))
  if (length(reverse_pt_bis_r) != length(pt_bis_r)) stop("The length of the 'reverse_pt_bis_r' argument is incorrectly specified.")

  df <- n_exp + n_nexp - 2
  h <- df / n_exp + df / n_nexp
  p <- n_exp / (n_exp + n_nexp)
  q <- n_nexp / (n_exp + n_nexp)

  d <- pt_bis_r * sqrt(h) / sqrt(1 - pt_bis_r^2)
  d <- ifelse(reverse_pt_bis_r, -d, d)
  es <- .es_from_d(d = d, n_exp = n_exp, n_nexp = n_nexp,
                   smd_to_cor = smd_to_cor, smd_var = smd_var)

  es$info_used <- "pt_bis_r"

  return(es)
}

#' Convert a p-value of a point-biserial correlation coefficient into several effect size measures
#'
#' @param pt_bis_r_pval p-value of a point-biserial correlation coefficient
#' @param n_exp total number of participants in the exposed group
#' @param n_nexp total number of participants in the non exposed group
#' @param smd_to_cor formula used to convert the \code{pt_bis_r_pval} value into a coefficient correlation.
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_pt_bis_r_pval a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the p-value of a point biserial correlation into a Student's t-value.
#'
#' **The formula used to obtain this Student's t-value is**:
#' \deqn{t = pt(\frac{pt\_bis\_r\_pval}{2}, df = n\_exp + n\_nexp - 2)}
#'
#' Calculations of the \code{\link{es_from_student_t}} function are then applied.
#'
#' Because a two-sided p-value carries no direction, the recovered t (and hence the
#' generated effect sizes) are always non-negative; use \code{reverse_pt_bis_r_pval} to
#' encode the correct sign.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
#'
#' @export es_from_pt_bis_r_pval
#'
#' @md
#'
#' @examples
#' es_from_pt_bis_r_pval(pt_bis_r_pval = 0.2, n_exp = 121, n_nexp = 121)
es_from_pt_bis_r_pval <- function(pt_bis_r_pval, n_exp, n_nexp,
                                  smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_pt_bis_r_pval) {
  if (missing(reverse_pt_bis_r_pval)) reverse_pt_bis_r_pval <- rep(FALSE, length(n_exp))
  reverse_pt_bis_r_pval[is.na(reverse_pt_bis_r_pval)] <- FALSE

  t <- qt(p = pt_bis_r_pval / 2, df = n_exp + n_nexp - 2, lower.tail = FALSE)

  es <- es_from_student_t(
    student_t = t, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_student_t = reverse_pt_bis_r_pval
  )

  es$info_used <- "pt_bis_r_pval"

  return(es)
}

#' Convert a Spearman's rank correlation coefficient to several effect size measures
#'
#' @param spearman_r a Spearman's rank correlation coefficient value
#' @param n_sample the total number of participants
#' @param sd_iv the standard deviation of the independent variable
#' @param unit_increase_iv a value of the independent variable that will be used to estimate the Cohen's d (see details).
#' @param unit_type the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "value"
#' @param n_exp number of the experimental/exposed group
#' @param n_nexp number of the non-experimental/non-exposed group
#' @param cor_to_smd formula used to convert the derived Pearson's r value into a SMD.
#' @param reverse_spearman_r a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function first converts a Spearman's rank correlation coefficient to an approximate Pearson's
#' correlation coefficient using the formula proposed by Rupinski & Dunlap (1996):
#' \deqn{r_p = 2 \sin(\pi / 6 \times r_s)}
#'
#' The standard error of the resulting Pearson's r is derived via the delta method.
#' The sampling variance of the Spearman rho under bivariate normality carries an
#' extra \eqn{(1 + r_s^2 / 2)} factor relative to the Pearson variance
#' (Bonett & Wright, 2000); omitting it would understate the SE, increasingly so
#' for strong correlations:
#' \deqn{r_p\_se = \sqrt{\left(\frac{\pi}{3} \cos\left(\frac{\pi}{6} r_s\right)\right)^2 \times \left(1 + \frac{r_s^2}{2}\right)\frac{(1 - r_s^2)^2}{n - 1}}}
#'
#' The converted Pearson's r is then further converted to a Fisher's z, Cohen's d, Hedges' g, and
#' odds ratio using the same formulas as \code{\link{es_from_pearson_r}()}. This
#' delta-method SE is propagated to the Fisher's z and (for the
#' \code{cor_to_smd = "viechtbauer"} and \code{cor_to_smd = "cooper"} paths,
#' whose SMD standard errors are proportional to the input r SE) to the Cohen's d,
#' Hedges' g and odds ratio, so all measures share the corrected r variance.
#' The \code{cor_to_smd = "mathur"} SMD variance contains no r SE term, so its
#' d/g/OR standard errors are left as computed by \code{\link{es_from_pearson_r}()}.
#'
#' @export es_from_spearman_rho
#'
#' @references
#' Rupinski, M. T., & Dunlap, W. P. (1996). Approximating Pearson product-moment correlations from
#' Kendall's tau and Spearman's rho. Educational and Psychological Measurement, 56(3), 419-429.
#'
#' Bonett, D. G., & Wright, T. A. (2000). Sample size requirements for estimating Pearson, Kendall
#' and Spearman correlations. Psychometrika, 65(1), 23-28.
#'
#' @md
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab R + Z\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + OR\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 4. Pearson's r or Fisher's z'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @examples
#' es_from_spearman_rho(
#'   spearman_r = .55, n_sample = 100
#' )
es_from_spearman_rho <- function(spearman_r, n_sample,
                                 n_exp, n_nexp, cor_to_smd = "viechtbauer",
                                 sd_iv, unit_increase_iv, unit_type = "raw_scale",
                                 reverse_spearman_r) {
  if (missing(reverse_spearman_r)) {
    reverse_spearman_r <- rep(FALSE, length(spearman_r))
  }
  reverse_spearman_r[is.na(reverse_spearman_r)] <- FALSE
  if (length(reverse_spearman_r) == 1) reverse_spearman_r = c(rep(reverse_spearman_r, length(spearman_r)))
  if (length(reverse_spearman_r) != length(spearman_r)) stop("The length of the 'reverse_spearman_r' argument is incorrectly specified.")

  if (missing(n_exp)) {
    n_exp <- rep(NA, length(spearman_r))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA, length(spearman_r))
  }
  if (missing(n_sample)) {
    n_sample <- rep(NA, length(spearman_r))
  }
  if (missing(sd_iv)) {
    sd_iv <- rep(NA, length(spearman_r))
  }
  if (missing(unit_increase_iv)) {
    unit_increase_iv <- rep(NA, length(spearman_r))
  }
  if (missing(unit_type)) {
    unit_type <- rep(NA, length(spearman_r))
  }

  # Mirror the sample-size fallback es_from_pearson_r applies internally, so a
  # direct call supplying only n_exp/n_nexp gets the same delta-method r/z SEs
  # (and corrected d/g/OR SEs) as the equivalent n_sample call, instead of NA
  # r_se/z_se next to uncorrected SMD SEs.
  n_sample <- ifelse(is.na(n_sample), n_exp + n_nexp, n_sample)

  # Rupinski & Dunlap (1996)
  r <- 2 * sin(pi / 6 * spearman_r)

  es <- es_from_pearson_r(
    pearson_r = r, sd_iv = sd_iv, n_sample = n_sample,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    unit_increase_iv = unit_increase_iv, unit_type = unit_type,
    reverse_pearson_r = reverse_spearman_r
  )

  # SE of the converted Pearson r_p via the delta method on the Spearman -> Pearson
  # transform r_p = 2*sin(pi/6*r_s):  d r_p / d r_s = (pi/3)*cos(pi/6*r_s).
  # The sampling variance of the Spearman rho under bivariate normality carries an
  # extra (1 + r_s^2/2) factor relative to the Pearson variance (Bonett & Wright,
  # 2000); using the plain Pearson form would understate the SE (increasingly so
  # for strong correlations).
  r_applied <- ifelse(reverse_spearman_r, -spearman_r, spearman_r)
  deriv <- (pi / 3) * cos(pi / 6 * r_applied)
  var_spearman <- (1 + r_applied^2 / 2) * (1 - r_applied^2)^2 / (n_sample - 1)
  r_se_delta <- sqrt(deriv^2 * var_spearman)

  # Reuse the (correct) converted Pearson r and z that es_from_pearson_r computed.
  r_p_applied <- es$r
  z_applied <- es$z

  # Propagate the corrected r_p SE into the converted SMD/OR measures. For the
  # (default) viechtbauer path, d_se is proportional to the input r_se
  # (d_se = |d(rtod)/dr_p| * r_se), and the cooper path is likewise exactly
  # linear in r_se (d_se = sqrt(4 * r_se^2 / (1 - r^2)^3)), so scaling d_se by
  # r_se_delta / r_se_pearson yields delta-consistent d/g/OR SEs on both paths;
  # g, OR and all CIs are then rebuilt by .es_from_d exactly as elsewhere in the
  # package. Only the mathur SMD variance genuinely does not route through r_se
  # (d_se = |d| * sqrt(1/(r^2 (n-3)) + 1/(2 (n-1))) has no r_se term), so those
  # rows are left unchanged (scale = 1).
  r_se_pearson <- es$r_se
  scale <- ifelse(cor_to_smd %in% c("viechtbauer", "cooper") &
                    !is.na(r_se_pearson) &
                    r_se_pearson > 0 & is.finite(r_se_delta),
                  r_se_delta / r_se_pearson, 1)
  es <- .es_from_d(d = es$d, d_se = es$d_se * scale,
                   n_exp = n_exp, n_nexp = n_nexp, n_sample = n_sample)

  # Overwrite the round-trip r/z from .es_from_d with the direct Spearman->Pearson
  # values and their delta-method SEs.
  es$r <- r_p_applied
  es$r_se <- r_se_delta
  es$r_ci_lo <- r_p_applied - qt(.975, n_sample - 2) * r_se_delta
  es$r_ci_up <- r_p_applied + qt(.975, n_sample - 2) * r_se_delta

  # dz/dr = 1/(1-r^2)
  es$z <- z_applied
  r_bounded <- pmin(pmax(r_p_applied, -0.9999), 0.9999)
  es$z_se <- r_se_delta / (1 - r_bounded^2)
  es$z_ci_lo <- es$z - qnorm(.975) * es$z_se
  es$z_ci_up <- es$z + qnorm(.975) * es$z_se

  es$info_used <- "spearman_r"
  return(es)
}

#' Convert means and standard deviations of two independent groups into several effect size measures
#'
#' @param mean_exp mean of participants in the experimental/exposed group.
#' @param mean_nexp mean of participants in the non-experimental/non-exposed group.
#' @param mean_sd_exp standard deviation of participants in the experimental/exposed group.
#' @param mean_sd_nexp standard deviation of participants in the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the generated \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param smd_denom standardizer for the standardized mean difference. "pooled" (default) uses the pooled endpoint SD (Cohen's d / Hedges' g); "glass" (alias "control") uses the control (non-experimental) endpoint SD (Glass's delta); "glass_robust" (alias "control_robust") is Glass's delta with a heteroscedasticity-consistent sampling variance.
#' @param reverse_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function first computes a Cohen's d (D), Hedges' g (G) and mean difference (MD)
#' from the means and standard deviations of two independent groups.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate a mean difference**  (formulas 12.1-12.6 in Cooper):
#' \deqn{md = mean\_exp - mean\_nexp}
#' \deqn{md\_se = \sqrt{\frac{mean\_sd\_exp^2}{n\_exp} + \frac{mean\_sd\_nexp^2}{n\_nexp}}}
#' The confidence interval uses the unequal-variance (Welch-Satterthwaite) degrees of freedom
#' that match this standard error (as in \code{stats::t.test(var.equal = FALSE)}):
#' \deqn{df = \frac{(s_e^2/n\_exp + s_n^2/n\_nexp)^2}{(s_e^2/n\_exp)^2/(n\_exp - 1) + (s_n^2/n\_nexp)^2/(n\_nexp - 1)}}
#' \deqn{md\_ci\_lo = md - md\_se * qt(.975, df)}
#' \deqn{md\_ci\_up = md + md\_se * qt(.975, df)}
#'
#' **To estimate a Cohen's d** the following formulas are used (formulas 12.10-12.18 in Cooper):
#' \deqn{mean\_sd\_pooled = \sqrt{\frac{(n\_exp - 1) * sd\_exp^2 + (n\_nexp - 1) * sd\_nexp^2}{n\_exp+n\_nexp-2}}}
#' \deqn{cohen\_d =  \frac{mean\_exp - mean\_nexp}{mean\_sd\_pooled}}
#' \deqn{cohen\_d\_se = \sqrt{\frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp} + \frac{cohen\_d^2}{2(n\_exp+n\_nexp)}}}
#' \deqn{cohen\_d\_ci\_lo = cohen\_d - cohen\_d\_se * qt(.975, df = n\_exp + n\_nexp - 2)}
#' \deqn{cohen\_d\_ci\_up = cohen\_d + cohen\_d\_se * qt(.975, df = n\_exp + n\_nexp - 2)}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 9. Means and dispersion (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_means_sd
#'
#' @md
#'
#' @examples
#' es_from_means_sd(
#'   n_exp = 55, n_nexp = 55,
#'   mean_exp = 2.3, mean_sd_exp = 1.2,
#'   mean_nexp = 1.9, mean_sd_nexp = 0.9
#' )
es_from_means_sd <- function(mean_exp, mean_sd_exp, mean_nexp, mean_sd_nexp, n_exp, n_nexp,
                             smd_to_cor = "viechtbauer", smd_var = "borenstein",
                             smd_denom = "pooled", reverse_means) {
  n_row <- length(mean_exp)
  if (missing(reverse_means)) reverse_means <- rep(FALSE, n_row)
  reverse_means[is.na(reverse_means)] <- FALSE
  if (length(reverse_means) == 1) reverse_means = c(rep(reverse_means, n_row))
  if (length(reverse_means) != n_row) stop("The length of the 'reverse_means' argument is incorrectly specified.")

  # smd_denom may be a single value or specified per row (like smd_var / smd_to_cor).
  # A blank/NA cell defaults to "pooled". Each row is standardised by its OWN choice, so a
  # mixed column can no longer silently collapse to the first row's value (previously the
  # match.arg(...[1]) took the first element and applied it to every row).
  smd_denom_in <- as.character(smd_denom)
  if (length(smd_denom_in) == 1) smd_denom_in <- rep(smd_denom_in, n_row)
  if (length(smd_denom_in) != n_row) stop("The length of the 'smd_denom' argument is incorrectly specified.")
  smd_denom_in[is.na(smd_denom_in)] <- "pooled"
  smd_denom <- .normalize_smd_denom(smd_denom_in) # "glass" -> "control", etc.
  if (any(is.na(smd_denom))) {
    stop(paste0(
      "'", paste(unique(smd_denom_in[is.na(smd_denom)]), collapse = "', '"),
      "' not in tolerated values for the 'smd_denom' argument. Possible inputs are: ",
      "'pooled' (the default), 'glass' (alias 'control') or 'glass_robust' (alias 'control_robust')."
    ), call. = FALSE)
  }

  # recycle the other per-row method arguments so they can be subset alongside the rows
  if (length(smd_var) == 1) smd_var <- rep(smd_var, n_row)
  if (length(smd_to_cor) == 1) smd_to_cor <- rep(smd_to_cor, n_row)

  es <- NULL
  for (dn in c("pooled", "control", "control_robust")) {
    sel <- which(smd_denom == dn)
    if (length(sel) == 0) next
    if (dn == "pooled") {
      pooled_sd <- sqrt(((n_exp[sel] - 1) * mean_sd_exp[sel]^2 +
        (n_nexp[sel] - 1) * mean_sd_nexp[sel]^2) / (n_exp[sel] + n_nexp[sel] - 2))
      d <- (mean_exp[sel] - mean_nexp[sel]) / pooled_sd
      es_sel <- .es_from_d(
        d = d, n_exp = n_exp[sel], n_nexp = n_nexp[sel],
        smd_to_cor = smd_to_cor[sel], smd_var = smd_var[sel], reverse = reverse_means[sel]
      )
    } else {
      # Glass's delta: standardise by the control (non-experimental) endpoint SD.
      es_sel <- .glass_from_means(
        mean_exp = mean_exp[sel], mean_sd_exp = mean_sd_exp[sel],
        mean_nexp = mean_nexp[sel], mean_sd_nexp = mean_sd_nexp[sel],
        n_exp = n_exp[sel], n_nexp = n_nexp[sel],
        smd_to_cor = smd_to_cor[sel], smd_var = smd_var[sel],
        reverse = reverse_means[sel], robust = (dn == "control_robust")
      )
    }
    es_sel$.row_order <- sel
    es <- if (is.null(es)) es_sel else rbind(es, es_sel)
  }
  es <- es[order(es$.row_order), , drop = FALSE]
  es$.row_order <- NULL
  rownames(es) <- NULL

  es$info_used <- "means_sd"

  es$md <- ifelse(reverse_means, mean_nexp - mean_exp, mean_exp - mean_nexp)
  es$md <- ifelse(!is.na(es$md) & !is.na(mean_sd_exp) & !is.na(mean_sd_nexp) &
    !is.na(n_exp) & !is.na(n_nexp),
  es$md, NA
  )
  # Unpooled (Welch) SE for the mean difference; pair it with the Welch-Satterthwaite df
  # (not the pooled n1 + n2 - 2) so the CI is coherent with the unequal-variance SE. This
  # matches stats::t.test(var.equal = FALSE). Falls back to the pooled df when the
  # Welch df is undefined (e.g. both arm SDs zero, or an arm n < 2).
  se_e2 <- mean_sd_exp^2 / n_exp
  se_n2 <- mean_sd_nexp^2 / n_nexp
  md_df <- (se_e2 + se_n2)^2 / (se_e2^2 / (n_exp - 1) + se_n2^2 / (n_nexp - 1))
  md_df <- ifelse(is.finite(md_df) & md_df > 0, md_df, n_exp + n_nexp - 2)
  es$md_se <- ifelse(!is.na(es$md), sqrt(se_e2 + se_n2), NA)
  es$md_ci_lo <- ifelse(!is.na(es$md), es$md - qt(.975, md_df) * es$md_se, NA)
  es$md_ci_up <- ifelse(!is.na(es$md), es$md + qt(.975, md_df) * es$md_se, NA)

  return(es)
}

#' Convert means and standard errors of two independent groups several effect size measures
#'
#' @param mean_exp mean of participants in the experimental/exposed group.
#' @param mean_nexp mean of participants in the non-experimental/non-exposed group.
#' @param mean_se_exp standard error of participants in the experimental/exposed group.
#' @param mean_se_nexp standard error of participants in the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param smd_denom standardizer for the standardized mean difference. "pooled" (default) uses the pooled endpoint SD (Cohen's d / Hedges' g); "glass" (alias "control") uses the control (non-experimental) endpoint SD (Glass's delta); "glass_robust" (alias "control_robust") is Glass's delta with a heteroscedasticity-consistent sampling variance.
#' @param reverse_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the standard errors of two independent groups into standard deviations,
#' and then relies on the calculations of the \code{\link{es_from_means_sd}()} function.
#'
#' **To convert the standard errors into standard deviations**, the following formula is used.
#' \deqn{mean\_sd\_exp = mean\_se\_exp * \sqrt{n\_exp}}
#' \deqn{mean\_sd\_nexp = mean\_se\_nexp * \sqrt{n\_nexp}}
#' Then, calculations of the \code{\link{es_from_means_sd}()} are applied.
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 9. Means and dispersion (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @export es_from_means_se
#'
#' @md
#'
#' @examples
#' es_from_means_se(
#'   mean_exp = 42, mean_se_exp = 11,
#'   mean_nexp = 42, mean_se_nexp = 15,
#'   n_exp = 43, n_nexp = 34
#' )
es_from_means_se <- function(mean_exp, mean_se_exp, mean_nexp, mean_se_nexp, n_exp, n_nexp,
                             smd_to_cor = "viechtbauer", smd_var = "borenstein",
                             smd_denom = "pooled", reverse_means) {
  if (missing(reverse_means)) reverse_means <- rep(FALSE, length(mean_exp))
  reverse_means[is.na(reverse_means)] <- FALSE

  sd_exp <- mean_se_exp * sqrt(n_exp)
  sd_nexp <- mean_se_nexp * sqrt(n_nexp)

  es <- es_from_means_sd(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_exp = mean_exp, mean_sd_exp = sd_exp,
    mean_nexp = mean_nexp, mean_sd_nexp = sd_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, smd_denom = smd_denom,
    reverse_means = reverse_means
  )

  es$info_used <- "means_se"

  return(es)
}

#' Convert means of two groups and the pooled standard deviation into several effect size measures
#'
#' @param mean_exp mean of participants in the experimental/exposed group.
#' @param mean_nexp mean of participants in the non-experimental/non-exposed group.
#' @param mean_sd_pooled pooled standard deviation across both groups.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function first computes a Cohen's d (D), Hedges' g (G) and mean difference (MD)
#' from the means of two independent groups and the pooled standard deviation across the groups.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate a mean difference**  (formulas 12.1-12.6 in Cooper):
#' \deqn{md = mean\_exp - mean\_nexp}
#' \deqn{md\_se = \sqrt{\frac{n_exp+n_nexp}{n_exp*n_nexp} * mean_sd_pooled^2}}
#' \deqn{md\_ci\_lo = md - md\_se * qt(.975, df = n\_exp + n\_nexp - 2)}
#' \deqn{md\_ci\_up = md + md\_se * qt(.975, df = n\_exp + n\_nexp - 2)}
#'
#' **To estimate a Cohen's d** the following formulas are used (formulas 12.10-12.18 in Cooper):
#' \deqn{cohen\_d =  \frac{mean\_exp - mean\_nexp}{means\_sd\_pooled}}
#' \deqn{cohen\_d\_se = \sqrt{\frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp} + \frac{cohen\_d^2}{2(n\_exp+n\_nexp)}}}
#' \deqn{cohen\_d\_ci\_lo = cohen\_d - cohen\_d\_se * qt(.975, df = n\_exp + n\_nexp - 2)}
#' \deqn{cohen\_d\_ci\_up = cohen\_d + cohen\_d\_se * qt(.975, df = n\_exp + n\_nexp - 2)}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 9. Means and dispersion (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_means_sd_pooled
#'
#' @md
#'
#' @examples
#' es_from_means_sd_pooled(
#'   n_exp = 55, n_nexp = 55,
#'   mean_exp = 2.3, mean_nexp = 1.9,
#'   mean_sd_pooled = 0.9
#' )
es_from_means_sd_pooled <- function(mean_exp, mean_nexp, mean_sd_pooled, n_exp, n_nexp,
                                    smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_means) {
  if (missing(reverse_means)) reverse_means <- rep(FALSE, length(mean_exp))
  reverse_means[is.na(reverse_means)] <- FALSE
  if (length(reverse_means) == 1) reverse_means = c(rep(reverse_means, length(mean_exp)))
  if (length(reverse_means) != length(mean_exp)) stop("The length of the 'reverse_means' argument is incorrectly specified.")

  d <- (mean_exp - mean_nexp) / mean_sd_pooled

  es <- .es_from_d(
    d = d, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_means
  )

  es$info_used <- "means_sd_pooled"

  es$md <- ifelse(reverse_means, mean_nexp - mean_exp, mean_exp - mean_nexp)
  es$md <- ifelse(!is.na(es$md) & !is.na(mean_sd_pooled) &
    !is.na(n_exp) & !is.na(n_nexp),
  es$md, NA
  )
  es$md_se <- ifelse(!is.na(es$md), sqrt((n_exp + n_nexp) / (n_exp * n_nexp) * mean_sd_pooled^2), NA)
  es$md_ci_lo <- ifelse(!is.na(es$md), es$md - qt(.975, n_exp + n_nexp - 2) * es$md_se, NA)
  es$md_ci_up <- ifelse(!is.na(es$md), es$md + qt(.975, n_exp + n_nexp - 2) * es$md_se, NA)

  return(es)
}

#' Convert means and 95% CI of two independent groups several effect size measures
#'
#' @param mean_exp mean of participants in the experimental/exposed group.
#' @param mean_nexp mean of participants in the non-experimental/non-exposed group.
#' @param mean_ci_lo_exp lower bound of the 95% CI of the mean of the experimental/exposed group
#' @param mean_ci_up_exp upper bound of the 95% CI of the mean of the experimental/exposed group
#' @param mean_ci_lo_nexp lower bound of the 95% CI of the mean of the non-experimental/non-exposed group.
#' @param mean_ci_up_nexp upper bound of the 95% CI of the mean of the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param smd_denom standardizer for the standardized mean difference. "pooled" (default) uses the pooled endpoint SD (Cohen's d / Hedges' g); "glass" (alias "control") uses the control (non-experimental) endpoint SD (Glass's delta); "glass_robust" (alias "control_robust") is Glass's delta with a heteroscedasticity-consistent sampling variance.
#' @param max_asymmetry A percentage indicating the tolerance before detecting asymmetry in the 95% CI bounds.
#' @param reverse_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the 95% CI of two independent groups into a standard error,
#' and then relies on the calculations of the \code{\link{es_from_means_se}()} function.
#'
#' **To convert the 95% CIs into standard errors,** the following formula is used (table 12.3 in Cooper):
#' \deqn{mean\_se\_exp = \frac{mean\_ci\_up\_exp - mean\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
#' \deqn{mean\_se\_nexp = \frac{mean\_ci\_up\_nexp - mean\_ci\_lo\_nexp}{2 * qt{(0.975, df = n\_nexp - 1)}}}
#' Calculations of the \code{\link{es_from_means_se}()} are then applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 9. Means and dispersion (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @export es_from_means_ci
#'
#' @md
#'
#' @examples
#' es_from_means_ci(
#'   n_exp = 55, n_nexp = 55,
#'   mean_exp = 25, mean_ci_lo_exp = 15, mean_ci_up_exp = 35,
#'   mean_nexp = 18, mean_ci_lo_nexp = 12, mean_ci_up_nexp = 24
#' )
es_from_means_ci <- function(mean_exp, mean_ci_lo_exp, mean_ci_up_exp,
                             mean_nexp, mean_ci_lo_nexp, mean_ci_up_nexp,
                             n_exp, n_nexp, smd_to_cor = "viechtbauer",
                             smd_var = "borenstein", smd_denom = "pooled",
                             max_asymmetry = 10,
                             reverse_means) {
  if (missing(reverse_means)) reverse_means <- rep(FALSE, length(mean_exp))
  reverse_means[is.na(reverse_means)] <- FALSE

  df_exp <- n_exp - 1
  df_nexp <- n_nexp - 1

  se_exp <- (mean_ci_up_exp - mean_ci_lo_exp) / (2 * qt(0.975, df_exp))
  se_nexp <- (mean_ci_up_nexp - mean_ci_lo_nexp) / (2 * qt(0.975, df_nexp))

  es <- es_from_means_se(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_exp = mean_exp,
    mean_se_exp = se_exp,
    mean_nexp = mean_nexp, mean_se_nexp = se_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, smd_denom = smd_denom,
    reverse_means = reverse_means
  )

  es$info_used <- "means_ci"

  return(es)
}

#' Convert a risk difference value and its standard error into several effect size measures
#'
#' @param rd risk difference value (control risk minus treatment risk)
#' @param rd_se standard error of the risk difference
#' @param n_exp number of participants in the exposed/treatment group
#' @param n_nexp number of participants in the non-exposed/control group
#' @param n_cases number of cases/events across both groups
#' @param n_controls number of controls/no-event across both groups
#' @param n_sample total number of participants in the sample
#' @param baseline_risk proportion of cases in the non-exposed/control group.
#'   Required for converting RD to OR and RR.
#' @param reverse_rd a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts a risk difference (RD) and its standard error into an odds ratio (OR),
#' risk ratio (RR), and number needed to treat (NNT).
#'
#' A risk difference is a ratio-family (binary-outcome) effect size and is **not** converted
#' to a standardized mean difference (D/G) or a correlation (R/Z). Such a conversion would
#' require reconstructing an odds ratio from an assumed baseline risk, which is not identified
#' by the risk difference alone; and because the baseline risk would be treated as a fixed
#' known constant, the resulting standardized SEs would be anti-conservative. In metaConvert
#' the standardized families are reached only through the odds ratio or a raw 2x2 table (the
#' Cox transform \eqn{d = \log(or)\sqrt{3}/\pi}), neither of which needs an assumed baseline
#' risk. To obtain a D/G/R/Z from a risk difference, first convert it to an OR (supplying the
#' baseline risk) and then use \code{\link{es_from_or_se}}.
#'
#' **NNT is always computed from RD:**
#' \deqn{nnt = \frac{1}{rd}}
#' \deqn{nnt\_se = \frac{rd\_se}{rd^2}}
#'
#' **When \code{baseline_risk} is available, the following conversions are performed.**
#'
#' **To estimate the odds ratio:**
#' Let \eqn{pt = baseline\_risk - rd} (treatment group risk). Then:
#' \deqn{or = \frac{pt \times (1 - baseline\_risk)}{baseline\_risk \times (1 - pt)}}
#' \deqn{logor\_se = \frac{rd\_se}{pt \times (1 - pt)}}
#' where the SE is derived via the delta method from \eqn{\frac{d(\log OR)}{d(RD)} = \frac{-1}{pt \times (1 - pt)}}.
#'
#' **To estimate the risk ratio:**
#' \deqn{rr = 1 - \frac{rd}{baseline\_risk}}
#' \deqn{logrr\_se = \frac{rd\_se}{|baseline\_risk - rd|}}
#' where the SE is derived via the delta method from \eqn{\frac{d(\log RR)}{d(RD)} = \frac{-1}{baseline\_risk - rd}}.
#'
#' Note that the conversions to OR and RR assume the baseline risk is a fixed constant.
#'
#' @references
#' Deeks, J.J. (2002). Issues in the selection of a summary statistic for meta-analysis of clinical trials with binary outcomes. Statistics in Medicine, 21(11), 1575-1600.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab RD\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + RR + NNT\cr
#'  \tab \cr
#'  \code{required input data} \tab rd + rd_se\cr
#'  \tab \cr
#' }
#'
#' @export es_from_rd_se
#'
#' @md
#'
#' @examples
#' es_from_rd_se(rd = 0.15, rd_se = 0.05,
#'               baseline_risk = 0.30, n_exp = 100, n_nexp = 100)
es_from_rd_se <- function(rd, rd_se,
                          baseline_risk,
                          n_exp, n_nexp, n_cases, n_controls, n_sample,
                          reverse_rd) {

  if (missing(rd)) rd <- rep(NA_real_, length(rd_se))
  if (missing(rd_se)) rd_se <- rep(NA_real_, length(rd))
  if (missing(baseline_risk)) baseline_risk <- rep(NA_real_, length(rd))
  if (missing(n_exp)) n_exp <- rep(NA_real_, length(rd))
  if (missing(n_nexp)) n_nexp <- rep(NA_real_, length(rd))
  if (missing(n_cases)) n_cases <- rep(NA_real_, length(rd))
  if (missing(n_controls)) n_controls <- rep(NA_real_, length(rd))
  if (missing(n_sample)) n_sample <- rep(NA_real_, length(rd))
  if (missing(reverse_rd)) reverse_rd <- rep(FALSE, length(rd))
  reverse_rd[is.na(reverse_rd)] <- FALSE
  if (length(reverse_rd) == 1) reverse_rd <- rep(reverse_rd, length(rd))
  if (length(reverse_rd) != length(rd)) {
    stop("The length of the 'reverse_rd' argument is incorrectly specified.")
  }

  # OR -------
  # A risk difference converts to the ratio family (OR + RR + NNT) only. It is
  # deliberately NOT converted to a standardized mean difference or correlation:
  # RD -> SMD/r/z is not identified without an assumed baseline_risk, and treating
  # baseline_risk as a fixed known constant yields anti-conservative SEs. The
  # standardized families are reached in metaConvert only through the odds ratio
  # or a raw 2x2 table (the Cox transform); see es_from_or_se() / es_from_2x2().
  treatment_risk <- baseline_risk - rd
  valid_pt <- !is.na(treatment_risk) & treatment_risk > 0 & treatment_risk < 1 &
              !is.na(baseline_risk) & baseline_risk > 0 & baseline_risk < 1

  or <- ifelse(valid_pt,
    (treatment_risk * (1 - baseline_risk)) / (baseline_risk * (1 - treatment_risk)),
    NA_real_
  )
  logOR <- suppressWarnings(log(or))
  # delta method
  logor_se <- ifelse(valid_pt, rd_se / abs(treatment_risk * (1 - treatment_risk)), NA_real_)

  es <- data.frame(
    logor = ifelse(reverse_rd, -logOR, logOR),
    logor_se = logor_se
  )
  es$logor_ci_lo <- es$logor - qnorm(.975) * logor_se
  es$logor_ci_up <- es$logor + qnorm(.975) * logor_se

  # RR -------
  valid_pt_rr <- !is.na(treatment_risk) & treatment_risk >= 0 & treatment_risk <= 1 &
                 !is.na(baseline_risk) & baseline_risk > 0 & baseline_risk <= 1
  rr <- ifelse(valid_pt_rr,
    treatment_risk / baseline_risk,
    NA_real_
  )
  rr <- ifelse(!is.na(rr) & rr > 0, rr, NA_real_)
  logRR <- suppressWarnings(log(rr))
  logrr_se <- ifelse(!is.na(rr) & !is.na(treatment_risk) & treatment_risk > 0,
    rd_se / abs(treatment_risk),
    NA_real_
  )

  es$logrr <- ifelse(reverse_rd, -logRR, logRR)
  es$logrr_se <- logrr_se
  es$logrr_ci_lo <- es$logrr - qnorm(.975) * logrr_se
  es$logrr_ci_up <- es$logrr + qnorm(.975) * logrr_se

  # RD -------
  es$rd <- ifelse(reverse_rd, -rd, rd)
  es$rd_se <- rd_se
  es$rd_ci_lo <- es$rd - qnorm(.975) * rd_se
  es$rd_ci_up <- es$rd + qnorm(.975) * rd_se

  # NNT -------
  es$nnt <- ifelse(rd == 0, NA, 1 / rd)
  es$nnt <- ifelse(reverse_rd, -es$nnt, es$nnt)
  es$nnt_se <- ifelse(rd == 0, NA, rd_se / rd^2)
  rd_ci_lo_raw <- rd - qnorm(.975) * rd_se
  rd_ci_up_raw <- rd + qnorm(.975) * rd_se
  crosses_zero <- (rd_ci_lo_raw < 0 & rd_ci_up_raw > 0) | rd == 0
  es$nnt_ci_lo <- ifelse(crosses_zero, NA,
                          ifelse(reverse_rd, -1 / rd_ci_lo_raw, 1 / rd_ci_up_raw))
  es$nnt_ci_up <- ifelse(crosses_zero, NA,
                          ifelse(reverse_rd, -1 / rd_ci_up_raw, 1 / rd_ci_lo_raw))

  es$info_used <- "rd_se"
  return(es)
}


#' Convert a risk difference value and its 95% confidence interval to several effect size measures
#'
#' @param rd risk difference value
#' @param rd_ci_lo lower bound of the 95% CI around the risk difference
#' @param rd_ci_up upper bound of the 95% CI around the risk difference
#' @param n_exp number of participants in the exposed/treatment group
#' @param n_nexp number of participants in the non-exposed/control group
#' @param n_cases number of cases/events across both groups
#' @param n_controls number of controls/no-event across both groups
#' @param n_sample total number of participants in the sample
#' @param baseline_risk proportion of cases in the non-exposed/control group
#' @param reverse_rd a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function computes the standard error of the risk difference from its 95% CI
#' (Section 6.5.2.2 in the Cochrane Handbook):
#' \deqn{rd\_se = \frac{rd\_ci\_up - rd\_ci\_lo}{2 \times qnorm(.975)}}
#'
#' Then, calculations of \code{\link{es_from_rd_se}()} are applied.
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab RD\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + RR + NNT\cr
#'  \tab \cr
#'  \code{required input data} \tab rd + rd_ci_lo + rd_ci_up\cr
#'  \tab \cr
#' }
#'
#' @export es_from_rd_ci
#'
#' @md
#'
#' @examples
#' es_from_rd_ci(
#'   rd = 0.15, rd_ci_lo = 0.05, rd_ci_up = 0.25,
#'   baseline_risk = 0.30, n_exp = 100, n_nexp = 100
#' )
es_from_rd_ci <- function(rd, rd_ci_lo, rd_ci_up,
                          baseline_risk,
                          n_exp, n_nexp, n_cases, n_controls, n_sample,
                          reverse_rd) {

  if (missing(rd)) rd <- rep(NA_real_, length(rd_ci_lo))
  if (missing(rd_ci_lo)) rd_ci_lo <- rep(NA_real_, length(rd))
  if (missing(rd_ci_up)) rd_ci_up <- rep(NA_real_, length(rd))
  if (missing(baseline_risk)) baseline_risk <- rep(NA_real_, length(rd))
  if (missing(n_exp)) n_exp <- rep(NA_real_, length(rd))
  if (missing(n_nexp)) n_nexp <- rep(NA_real_, length(rd))
  if (missing(n_cases)) n_cases <- rep(NA_real_, length(rd))
  if (missing(n_controls)) n_controls <- rep(NA_real_, length(rd))
  if (missing(n_sample)) n_sample <- rep(NA_real_, length(rd))
  if (missing(reverse_rd)) reverse_rd <- rep(FALSE, length(rd))
  reverse_rd[is.na(reverse_rd)] <- FALSE

  rd_se <- (rd_ci_up - rd_ci_lo) / (2 * qnorm(.975))

  es <- es_from_rd_se(
    rd = rd, rd_se = rd_se,
    baseline_risk = baseline_risk,
    n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls,
    n_sample = n_sample,
    reverse_rd = reverse_rd
  )

  es$info_used <- "rd_ci"
  return(es)
}


#' Convert a risk difference value and its p-value to several effect size measures
#'
#' @param rd risk difference value
#' @param rd_pval p-value of the risk difference
#' @param n_exp number of participants in the exposed/treatment group
#' @param n_nexp number of participants in the non-exposed/control group
#' @param n_cases number of cases/events across both groups
#' @param n_controls number of controls/no-event across both groups
#' @param n_sample total number of participants in the sample
#' @param baseline_risk proportion of cases in the non-exposed/control group
#' @param reverse_rd_pval a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function uses the p-value of the risk difference to obtain the standard error
#' (Section 6.3.2 in the Cochrane Handbook):
#' \deqn{z = qnorm(rd\_pval / 2, lower.tail = FALSE)}
#' \deqn{rd\_se = |\frac{rd}{z}|}
#'
#' Then, calculations of \code{\link{es_from_rd_se}()} are applied.
#'
#' @references
#' Higgins, J. P., Thomas, J., Chandler, J., Cumpston, M., Li, T., Page, M. J., & Welch, V. A. (Eds.). (2019). Cochrane handbook for systematic reviews of interventions. John Wiley & Sons.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab RD\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + RR + NNT\cr
#'  \tab \cr
#'  \code{required input data} \tab rd + rd_pval\cr
#'  \tab \cr
#' }
#'
#' @export es_from_rd_pval
#'
#' @md
#'
#' @examples
#' es_from_rd_pval(
#'   rd = 0.15, rd_pval = 0.01,
#'   baseline_risk = 0.30, n_exp = 100, n_nexp = 100
#' )
es_from_rd_pval <- function(rd, rd_pval,
                            baseline_risk,
                            n_exp, n_nexp, n_cases, n_controls, n_sample,
                            reverse_rd_pval) {

  if (missing(rd)) rd <- rep(NA_real_, length(rd_pval))
  if (missing(rd_pval)) rd_pval <- rep(NA_real_, length(rd))
  if (missing(baseline_risk)) baseline_risk <- rep(NA_real_, length(rd))
  if (missing(n_exp)) n_exp <- rep(NA_real_, length(rd))
  if (missing(n_nexp)) n_nexp <- rep(NA_real_, length(rd))
  if (missing(n_cases)) n_cases <- rep(NA_real_, length(rd))
  if (missing(n_controls)) n_controls <- rep(NA_real_, length(rd))
  if (missing(n_sample)) n_sample <- rep(NA_real_, length(rd))
  if (missing(reverse_rd_pval)) reverse_rd_pval <- rep(FALSE, length(rd))
  reverse_rd_pval[is.na(reverse_rd_pval)] <- FALSE

  z_rd <- qnorm(rd_pval / 2, lower.tail = FALSE)
  rd_se <- abs(rd / z_rd)

  es <- es_from_rd_se(
    rd = rd, rd_se = rd_se,
    baseline_risk = baseline_risk,
    n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls,
    n_sample = n_sample,
    reverse_rd = reverse_rd_pval
  )

  es$info_used <- "rd_pval"
  return(es)
}

#' Pool or split arms in multi-arm trials
#'
#' @param x a data.frame in metaConvert wide format (as used by \code{\link{convert_df}}).
#' @param study_id character string: column name identifying which rows belong
#'   to the same multi-arm trial.
#' @param pool_side character string: column name whose values indicate which
#'   side to pool (\code{"exp"} or \code{"nexp"}) or \code{NA} for single-comparison rows
#'   that should be left unchanged.
#' @param method either \code{"pool"} (Cochrane-recommended pooling of summary
#'   statistics) or \code{"split"} (divide the shared arm's sample size).
#'   See details.
#' @param verbose logical: whether to print warnings about columns set to NA
#'   and non-identical shared-side values. Default is \code{TRUE}.
#'
#' @details
#' Multi-arm trials create dependency when multiple experimental arms share a
#' common control group (or vice versa). This function resolves the dependency
#' before effect size computation with \code{\link{convert_df}}.
#'
#' With \code{method = "pool"} (recommended), the arms to be pooled are combined
#' into a single composite arm using Cochrane Handbook formulas (Table 23.3.a):
#' \itemize{
#'   \item Sample sizes and event counts: summed
#'   \item Means: weighted average by sample size
#'   \item SDs: Cochrane pooling formula
#'   \item SEs: recomputed from pooled SD and pooled n
#'   \item CIs, medians, paired statistics: set to NA (cannot be pooled)
#' }
#' Returns one row per multi-arm group.
#'
#' With \code{method = "split"}, the shared arm's sample size (and event counts)
#' is divided equally across comparisons. Each comparison retains its own arm
#' data unchanged, so the output has the same number of rows as the input but
#' with adjusted sample sizes on the shared side. Simpler but slightly
#' conservative.
#'
#' @return A data.frame suitable for \code{\link{convert_df}}. The
#'   \code{pool_side} column is removed from the output.
#'
#' @references
#' Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA
#' (editors). Cochrane Handbook for Systematic Reviews of Interventions.
#' Chapter 23, Table 23.3.a.
#'
#' @export pool_arms
#'
#' @md
#'
#' @examples
#' # Two experimental arms vs one shared control
#' dat <- data.frame(
#'   study_id = c("Study1", "Study1", "Study2"),
#'   pool_side = c("exp", "exp", NA),
#'   n_exp = c(30, 25, 40),
#'   n_nexp = c(28, 28, 35),
#'   mean_exp = c(12.5, 14.2, 10.0),
#'   mean_sd_exp = c(3.1, 2.8, 4.0),
#'   mean_nexp = c(10.1, 10.1, 9.5),
#'   mean_sd_nexp = c(3.0, 3.0, 3.8)
#' )
#'
#' # Pool experimental arms into one composite arm
#' pooled <- pool_arms(dat, study_id = "study_id",
#'                     pool_side = "pool_side", method = "pool")
#'
#' # Or split the shared control's sample size
#' split_dat <- pool_arms(dat, study_id = "study_id",
#'                        pool_side = "pool_side", method = "split")
#'
pool_arms <- function(x,
                      study_id,
                      pool_side,
                      method = c("pool", "split"),
                      verbose = TRUE) {

  method <- match.arg(method)

  if (!is.data.frame(x)) {
    stop("'x' must be a data.frame.")
  }
  if (!study_id %in% colnames(x)) {
    stop(paste0("Column '", study_id, "' not found in 'x'."))
  }
  if (!pool_side %in% colnames(x)) {
    stop(paste0("Column '", pool_side, "' not found in 'x'."))
  }

  ps_vals <- x[[pool_side]]
  valid_vals <- ps_vals %in% c("exp", "nexp") | is.na(ps_vals)
  if (!all(valid_vals)) {
    bad <- unique(ps_vals[!valid_vals])
    stop(paste0("Column '", pool_side, "' contains invalid values: ",
                paste(bad, collapse = ", "),
                ". Allowed values are 'exp', 'nexp', or NA."))
  }

  # Check: within each study_id group, all non-NA pool_side values must be the
  # same. Split on a character key so a factor study_id does not spawn empty
  # groups for unused levels.
  groups <- split(ps_vals, as.character(x[[study_id]]))
  for (grp_name in names(groups)) {
    v <- groups[[grp_name]]
    vals <- na.omit(v)
    if (length(unique(vals)) > 1) {
      stop(paste0("Study '", grp_name,
                  "' has mixed pool_side values ('exp' and 'nexp'). ",
                  "All rows of a multi-arm trial must pool the same side."))
    }
    # A study with some tagged and some NA rows is ambiguous: the NA rows are
    # routed to pass-through and end up as a separate output row sharing this
    # study_id, silently escaping the pooling/splitting.
    if (verbose && any(is.na(v)) && any(!is.na(v))) {
      warning(paste0("Study '", grp_name, "' has both tagged (pool_side = '",
                     paste(unique(vals), collapse = "/"),
                     "') and untagged (NA) rows. The untagged row(s) are left ",
                     "unchanged and NOT pooled/split with the rest of the ",
                     "study, producing a separate output row with the same ",
                     "study_id. Tag all arms of a multi-arm comparison, or give ",
                     "a genuinely independent row a distinct study_id."))
    }
  }

  # Collision-safe ordering key (avoid clobbering a user column named .orig_row).
  order_key <- ".orig_row"
  while (order_key %in% colnames(x)) order_key <- paste0(order_key, "_")
  x[[order_key]] <- seq_len(nrow(x))

  is_passthrough <- is.na(x[[pool_side]])
  x_pass <- x[is_passthrough, , drop = FALSE]
  x_multi <- x[!is_passthrough, , drop = FALSE]

  if (nrow(x_multi) == 0) {
    res <- x_pass
    res[[pool_side]] <- NULL
    res[[order_key]] <- NULL
    return(res)
  }

  # Split on a character key so a factor study_id does not create empty groups.
  x_split <- split(x_multi, as.character(x_multi[[study_id]]))

  if (method == "pool") {
    processed <- lapply(x_split, function(grp) {
      side <- na.omit(unique(grp[[pool_side]]))[1]
      if (nrow(grp) == 1) {
        if (verbose) {
          warning(paste0("Study '", grp[[study_id]][1],
                         "' has only 1 row with pool_side = '", side,
                         "'. Passing through unchanged."))
        }
        return(grp)
      }
      .pool_one_group(grp, side = side, pool_side_col = pool_side,
                       study_id_col = study_id, verbose = verbose)
    })
  } else {
    processed <- lapply(x_split, function(grp) {
      side <- na.omit(unique(grp[[pool_side]]))[1]
      if (nrow(grp) == 1) {
        if (verbose) {
          warning(paste0("Study '", grp[[study_id]][1],
                         "' has only 1 row with pool_side = '", side,
                         "'. Passing through unchanged."))
        }
        return(grp)
      }
      .split_one_group(grp, side = side, pool_side_col = pool_side,
                        study_id_col = study_id, verbose = verbose)
    })
  }

  x_processed <- do.call(rbind, processed)

  all_cols <- union(colnames(x_pass), colnames(x_processed))
  for (col in setdiff(all_cols, colnames(x_pass))) {
    x_pass[[col]] <- NA
  }
  for (col in setdiff(all_cols, colnames(x_processed))) {
    x_processed[[col]] <- NA
  }

  res <- rbind(x_pass[, all_cols, drop = FALSE],
               x_processed[, all_cols, drop = FALSE])

  res <- res[order(res[[order_key]]), , drop = FALSE]

  res[[pool_side]] <- NULL
  res[[order_key]] <- NULL
  rownames(res) <- seq_len(nrow(res))

  return(res)
}

#########

# Column families to pool for a given side ("exp" or "nexp")
.pool_cols <- function(side) {
  s <- side  # "exp" or "nexp"

  list(
    n = paste0("n_", s),

    means = list(
      list(mean = paste0("mean_", s),        n = paste0("n_", s)),
      list(mean = paste0("mean_pre_", s),     n = paste0("n_", s)),
      list(mean = paste0("mean_change_", s),  n = paste0("n_", s)),
      list(mean = paste0("plot_mean_", s),    n = paste0("n_", s)),
      list(mean = paste0("ancova_mean_", s),  n = paste0("n_", s)),
      list(mean = paste0("plot_ancova_mean_", s), n = paste0("n_", s))
    ),

    sds = list(
      list(sd = paste0("mean_sd_", s),
           mean = paste0("mean_", s),
           n = paste0("n_", s)),
      list(sd = paste0("mean_pre_sd_", s),
           mean = paste0("mean_pre_", s),
           n = paste0("n_", s)),
      list(sd = paste0("mean_change_sd_", s),
           mean = paste0("mean_change_", s),
           n = paste0("n_", s))
    ),

    ses = list(
      list(se = paste0("mean_se_", s),
           sd = paste0("mean_sd_", s),
           n = paste0("n_", s)),
      list(se = paste0("mean_pre_se_", s),
           sd = paste0("mean_pre_sd_", s),
           n = paste0("n_", s)),
      list(se = paste0("mean_change_se_", s),
           sd = paste0("mean_change_sd_", s),
           n = paste0("n_", s)),
      list(se = paste0("ancova_mean_se_", s),
           sd = paste0("ancova_mean_sd_", s),    # note: may not exist directly
           n = paste0("n_", s))
    ),

    cis = c(
      paste0("mean_ci_lo_", s), paste0("mean_ci_up_", s),
      paste0("mean_pre_ci_lo_", s), paste0("mean_pre_ci_up_", s),
      paste0("mean_change_ci_lo_", s), paste0("mean_change_ci_up_", s),
      paste0("ancova_mean_ci_lo_", s), paste0("ancova_mean_ci_up_", s),
      paste0("plot_mean_ci_lo_", s), paste0("plot_mean_ci_up_", s),
      paste0("plot_ancova_mean_ci_lo_", s), paste0("plot_ancova_mean_ci_up_", s)
    ),

    counts = c(
      paste0("n_cases_", s),
      paste0("n_controls_", s)
    ),

    proportions = list(
      list(prop = paste0("prop_cases_", s),
           cases = paste0("n_cases_", s),
           n = paste0("n_", s))
    ),

    time = paste0("time_", s),

    na_cols = c(
      paste0("med_", s), paste0("q1_", s), paste0("q3_", s),
      paste0("min_", s), paste0("max_", s),
      paste0("paired_t_", s), paste0("paired_t_pval_", s),
      paste0("paired_f_", s), paste0("paired_f_pval_", s),
      paste0("r_pre_post_", s),
      paste0("mean_change_pval_", s)
    ),

    # bars: SD derived from bounds, pooled, then reconstructed around the mean
    plot_sds = list(
      list(lo = paste0("plot_mean_sd_lo_", s),
           up = paste0("plot_mean_sd_up_", s),
           mean = paste0("plot_mean_", s),
           n = paste0("n_", s)),
      list(lo = paste0("plot_ancova_mean_sd_lo_", s),
           up = paste0("plot_ancova_mean_sd_up_", s),
           mean = paste0("plot_ancova_mean_", s),
           n = paste0("n_", s))
    ),

    plot_ses = list(
      list(lo = paste0("plot_mean_se_lo_", s),
           up = paste0("plot_mean_se_up_", s),
           mean = paste0("plot_mean_", s),
           sd_lo = paste0("plot_mean_sd_lo_", s),
           sd_up = paste0("plot_mean_sd_up_", s),
           n = paste0("n_", s)),
      list(lo = paste0("plot_ancova_mean_se_lo_", s),
           up = paste0("plot_ancova_mean_se_up_", s),
           mean = paste0("plot_ancova_mean_", s),
           sd_lo = paste0("plot_ancova_mean_sd_lo_", s),
           sd_up = paste0("plot_ancova_mean_sd_up_", s),
           n = paste0("n_", s))
    ),

    # ancova_mean_sd_pooled is shared across arms, handled apart
    ancova_sds = list(
      list(sd = paste0("ancova_mean_sd_", s),
           mean = paste0("ancova_mean_", s),
           n = paste0("n_", s))
    )
  )
}

# Cochrane SD pooling for two arms
.pool_sd_cochrane <- function(n1, sd1, m1, n2, sd2, m2) {
  if (any(is.na(c(n1, sd1, m1, n2, sd2, m2)))) return(NA_real_)
  sqrt(((n1 - 1) * sd1^2 + (n2 - 1) * sd2^2 +
          n1 * n2 / (n1 + n2) * (m1^2 + m2^2 - 2 * m1 * m2)) /
         (n1 + n2 - 1))
}

# Iterative Cochrane SD pooling for k arms
.pool_sd_cochrane_multi <- function(ns, sds, means) {
  k <- length(ns)
  if (k == 0) return(NA_real_)
  if (k == 1) return(sds[1])

  n_cum <- ns[1]
  sd_cum <- sds[1]
  m_cum <- means[1]

  for (i in 2:k) {
    sd_new <- .pool_sd_cochrane(n_cum, sd_cum, m_cum, ns[i], sds[i], means[i])
    m_new <- (n_cum * m_cum + ns[i] * means[i]) / (n_cum + ns[i])
    n_cum <- n_cum + ns[i]
    sd_cum <- sd_new
    m_cum <- m_new
  }

  return(sd_cum)
}

# Get column value from group, handling missing columns
.get_col <- function(grp, col) {
  if (col %in% colnames(grp)) grp[[col]] else rep(NA_real_, nrow(grp))
}

# Pool one multi-arm group (method = "pool")
.pool_one_group <- function(grp, side, pool_side_col, study_id_col, verbose = TRUE) {

  other_side <- ifelse(side == "exp", "nexp", "exp")
  cols_info <- .pool_cols(side)

  result <- grp[1, , drop = FALSE]
  na_warned <- character(0)

  n_col <- cols_info$n
  if (n_col %in% colnames(grp)) {
    ns <- grp[[n_col]]
    result[[n_col]] <- sum(ns, na.rm = TRUE)
    if (all(is.na(ns))) result[[n_col]] <- NA_real_
  }

  n_pooled <- if (n_col %in% colnames(result)) result[[n_col]] else NA_real_

  for (m_info in cols_info$means) {
    mc <- m_info$mean
    nc <- m_info$n
    if (!mc %in% colnames(grp)) next
    vals <- grp[[mc]]
    weights <- .get_col(grp, nc)
    if (all(is.na(vals))) {
      result[[mc]] <- NA_real_
    } else if (all(is.na(weights))) {
      result[[mc]] <- mean(vals, na.rm = TRUE)
      if (verbose) warning(paste0("No sample sizes for weighting '", mc,
                                   "'. Using unweighted mean."))
    } else {
      ok <- !is.na(vals) & !is.na(weights)
      result[[mc]] <- sum(vals[ok] * weights[ok]) / sum(weights[ok])
    }
  }

  for (sd_info in c(cols_info$sds, cols_info$ancova_sds)) {
    sdc <- sd_info$sd
    mc <- sd_info$mean
    nc <- sd_info$n
    if (!sdc %in% colnames(grp)) next
    sd_vals <- grp[[sdc]]
    m_vals <- .get_col(grp, mc)
    n_vals <- .get_col(grp, nc)
    ok <- !is.na(sd_vals) & !is.na(m_vals) & !is.na(n_vals)
    if (sum(ok) < 2) {
      result[[sdc]] <- if (sum(ok) == 1) sd_vals[ok] else NA_real_
    } else {
      result[[sdc]] <- .pool_sd_cochrane_multi(n_vals[ok], sd_vals[ok], m_vals[ok])
    }
  }

  for (se_info in cols_info$ses) {
    sec <- se_info$se
    sdc <- se_info$sd
    if (!sec %in% colnames(grp)) next
    sd_pooled <- if (sdc %in% colnames(result)) result[[sdc]] else NA_real_
    if (!is.na(sd_pooled) && !is.na(n_pooled) && n_pooled > 0) {
      result[[sec]] <- sd_pooled / sqrt(n_pooled)
    } else {
      result[[sec]] <- NA_real_
    }
  }

  for (ci_col in cols_info$cis) {
    if (ci_col %in% colnames(grp)) {
      if (verbose && any(!is.na(grp[[ci_col]])) && !ci_col %in% na_warned) {
        na_warned <- c(na_warned, ci_col)
      }
      result[[ci_col]] <- NA_real_
    }
  }

  for (cnt_col in cols_info$counts) {
    if (cnt_col %in% colnames(grp)) {
      vals <- grp[[cnt_col]]
      result[[cnt_col]] <- if (all(is.na(vals))) NA_real_ else sum(vals, na.rm = TRUE)
    }
  }

  for (p_info in cols_info$proportions) {
    pc <- p_info$prop
    cc <- p_info$cases
    nc <- p_info$n
    if (!pc %in% colnames(grp)) next
    cases_pooled <- if (cc %in% colnames(result)) result[[cc]] else NA_real_
    if (!is.na(cases_pooled) && !is.na(n_pooled) && n_pooled > 0) {
      result[[pc]] <- cases_pooled / n_pooled
    } else {
      result[[pc]] <- NA_real_
    }
  }

  tc <- cols_info$time
  if (tc %in% colnames(grp)) {
    vals <- grp[[tc]]
    result[[tc]] <- if (all(is.na(vals))) NA_real_ else sum(vals, na.rm = TRUE)
  }

  # plot SD bars: derive SD, pool, reconstruct bounds
  for (ps_info in cols_info$plot_sds) {
    lo_col <- ps_info$lo
    up_col <- ps_info$up
    m_col <- ps_info$mean
    n_c <- ps_info$n
    if (!lo_col %in% colnames(grp) && !up_col %in% colnames(grp)) next

    # Derive SDs from plot bars: SD = up - mean (or mean - lo)
    m_vals <- .get_col(grp, m_col)
    up_vals <- .get_col(grp, up_col)
    lo_vals <- .get_col(grp, lo_col)
    n_vals <- .get_col(grp, n_c)

    derived_sds <- ifelse(!is.na(up_vals) & !is.na(m_vals),
                          up_vals - m_vals,
                          ifelse(!is.na(m_vals) & !is.na(lo_vals),
                                 m_vals - lo_vals, NA_real_))

    ok <- !is.na(derived_sds) & !is.na(m_vals) & !is.na(n_vals)
    if (sum(ok) >= 2) {
      sd_pooled <- .pool_sd_cochrane_multi(n_vals[ok], derived_sds[ok], m_vals[ok])
      m_pooled <- result[[m_col]]
      if (lo_col %in% colnames(grp)) result[[lo_col]] <- m_pooled - sd_pooled
      if (up_col %in% colnames(grp)) result[[up_col]] <- m_pooled + sd_pooled
    } else {
      if (lo_col %in% colnames(grp)) result[[lo_col]] <- NA_real_
      if (up_col %in% colnames(grp)) result[[up_col]] <- NA_real_
    }
  }

  # plot SE bars: prefer the pooled SD reconstructed from the plot_sd bars;
  # otherwise recover per-arm SDs from the SE bars themselves (SD = SE * sqrt(n)),
  # Cochrane-pool them, and recompute - so a study reporting only SE error bars
  # (a very common case) keeps a usable pooled interval instead of being dropped.
  for (pse_info in cols_info$plot_ses) {
    lo_col <- pse_info$lo
    up_col <- pse_info$up
    m_col <- pse_info$mean
    sd_up_col <- pse_info$sd_up
    n_c <- pse_info$n
    if (!lo_col %in% colnames(grp) && !up_col %in% colnames(grp)) next

    m_pooled <- if (m_col %in% colnames(result)) result[[m_col]] else NA_real_

    # (a) pooled SD already reconstructed from the plot_sd bars this round
    sd_up_val <- if (sd_up_col %in% colnames(result)) result[[sd_up_col]] else NA_real_
    sd_pooled <- if (!is.na(sd_up_val) && !is.na(m_pooled)) sd_up_val - m_pooled else NA_real_

    # (b) otherwise derive per-arm SDs from the SE bars and Cochrane-pool them
    if (is.na(sd_pooled)) {
      m_vals  <- .get_col(grp, m_col)
      up_vals <- .get_col(grp, up_col)
      lo_vals <- .get_col(grp, lo_col)
      n_vals  <- .get_col(grp, n_c)
      se_bars <- ifelse(!is.na(up_vals) & !is.na(m_vals),
                        up_vals - m_vals,
                        ifelse(!is.na(m_vals) & !is.na(lo_vals),
                               m_vals - lo_vals, NA_real_))
      derived_sds <- se_bars * sqrt(n_vals)
      ok <- !is.na(derived_sds) & !is.na(m_vals) & !is.na(n_vals)
      if (sum(ok) >= 2) {
        sd_pooled <- .pool_sd_cochrane_multi(n_vals[ok], derived_sds[ok], m_vals[ok])
      } else if (sum(ok) == 1) {
        sd_pooled <- derived_sds[ok]
      }
    }

    if (!is.na(sd_pooled) && !is.na(n_pooled) && n_pooled > 0) {
      se_pooled <- sd_pooled / sqrt(n_pooled)
      if (lo_col %in% colnames(grp)) result[[lo_col]] <- m_pooled - se_pooled
      if (up_col %in% colnames(grp)) result[[up_col]] <- m_pooled + se_pooled
    } else {
      if (lo_col %in% colnames(grp)) result[[lo_col]] <- NA_real_
      if (up_col %in% colnames(grp)) result[[up_col]] <- NA_real_
    }
  }

  for (na_col in cols_info$na_cols) {
    if (na_col %in% colnames(grp)) {
      if (verbose && any(!is.na(grp[[na_col]])) && !na_col %in% na_warned) {
        na_warned <- c(na_warned, na_col)
      }
      result[[na_col]] <- NA_real_
    }
  }

  if (verbose && length(na_warned) > 0) {
    warning(paste0("Study '", grp[[colnames(grp)[1]]][1],
                   "': the following columns cannot be pooled and were set to NA: ",
                   paste(na_warned, collapse = ", ")))
  }

  # shared side taken from first row; warn if values differ across arms
  other_cols <- grep(paste0("_", other_side, "$"), colnames(grp), value = TRUE)
  for (oc in other_cols) {
    vals <- grp[[oc]]
    if (length(unique(na.omit(vals))) > 1 && verbose) {
      warning(paste0("Shared-side column '", oc,
                     "' differs across arms of study '",
                     grp[[study_id_col]][1],
                     "'. Using first row's value."))
    }
  }

  # n_sample follows the pooled n; pooled-SD columns are no longer valid
  n_other <- if (paste0("n_", other_side) %in% colnames(result)) {
    result[[paste0("n_", other_side)]]
  } else {
    NA_real_
  }
  if ("n_sample" %in% colnames(result)) {
    if (!is.na(n_pooled) && !is.na(n_other)) {
      result[["n_sample"]] <- n_pooled + n_other
    } else {
      # Cannot recompute a consistent total; do not keep the stale first-row
      # value (mirrors the split method, which sets NA in the same situation).
      result[["n_sample"]] <- NA_real_
    }
  }
  if ("mean_sd_pooled" %in% colnames(result)) {
    result[["mean_sd_pooled"]] <- NA_real_
  }
  if ("ancova_mean_sd_pooled" %in% colnames(result)) {
    result[["ancova_mean_sd_pooled"]] <- NA_real_
  }

  return(result)
}

# Split one multi-arm group (method = "split")
.split_one_group <- function(grp, side, pool_side_col, study_id_col, verbose = TRUE) {

  other_side <- ifelse(side == "exp", "nexp", "exp")
  k <- nrow(grp)

  # The split method consumes only the first row's shared-side n / counts; if the
  # duplicated shared-arm rows disagree, the extras are silently discarded. Warn,
  # mirroring the pool method (and the @param verbose contract).
  if (verbose) {
    other_cols <- grep(paste0("_", other_side, "$"), colnames(grp), value = TRUE)
    for (oc in other_cols) {
      vals <- grp[[oc]]
      if (length(unique(na.omit(vals))) > 1) {
        warning(paste0("Shared-side column '", oc,
                       "' differs across arms of study '",
                       grp[[study_id_col]][1],
                       "'. Using first row's value."))
      }
    }
  }

  # Divide the shared (other) side's sample size by k
  n_col <- paste0("n_", other_side)
  if (n_col %in% colnames(grp) && !all(is.na(grp[[n_col]]))) {
    n_shared <- grp[[n_col]][1]
    if (!is.na(n_shared)) {
      n_each <- floor(n_shared / k)
      remainder <- n_shared - n_each * k
      grp[[n_col]] <- n_each
      # Distribute remainder to first rows
      if (remainder > 0) {
        grp[[n_col]][seq_len(remainder)] <- n_each + 1
      }
    }
  }

  # Divide shared side 2x2 counts by k
  for (cnt_col in c(paste0("n_cases_", other_side),
                     paste0("n_controls_", other_side))) {
    if (cnt_col %in% colnames(grp) && !all(is.na(grp[[cnt_col]]))) {
      cnt_shared <- grp[[cnt_col]][1]
      if (!is.na(cnt_shared)) {
        cnt_each <- floor(cnt_shared / k)
        remainder <- cnt_shared - cnt_each * k
        grp[[cnt_col]] <- cnt_each
        if (remainder > 0) {
          grp[[cnt_col]][seq_len(remainder)] <- cnt_each + 1
        }
      }
    }
  }

  if ("n_sample" %in% colnames(grp)) {
    n_exp_vals <- if ("n_exp" %in% colnames(grp)) grp[["n_exp"]] else NA_real_
    n_nexp_vals <- if ("n_nexp" %in% colnames(grp)) grp[["n_nexp"]] else NA_real_
    grp[["n_sample"]] <- ifelse(!is.na(n_exp_vals) & !is.na(n_nexp_vals),
                                 n_exp_vals + n_nexp_vals, NA_real_)
  }

  return(grp)
}

#' Convert an intraclass correlation coefficient (ICC) into an effect size measure
#'
#' @param icc intraclass correlation coefficient value
#' @param n_sample total sample size (number of subjects)
#' @param n_measurements number of measurements or raters
#' @param icc_type ICC type: \code{"agreement"} for ICC(2,1) (two-way random, absolute agreement)
#'   or \code{"consistency"} for ICC(3,1) (two-way mixed, consistency). Default is \code{"agreement"}.
#' @param icc_to_es method used to compute the effect size from ICC.
#'   Must be either \code{"bonett"} or \code{"raw"}.
#'
#' @details
#' This function computes an effect size from an ICC.
#'
#' 1. When \code{icc_to_es = "bonett"} (default), the Bonett (2002) transformation
#' is applied:
#' \deqn{T(ICC) = \ln(1 - ICC)}
#' with the one-way random-model / two-way-consistency leading-order sampling
#' variance (Bonett, 2002; Donner & Eliasziw, 1987):
#' \deqn{T\_se = \sqrt{\frac{2 (1 + (k-1) ICC)^2}{k (k - 1)(n - 1)}}}
#'
#' 2. When \code{icc_to_es = "raw"}, the raw ICC is used and its standard error
#' is obtained by the delta method (\eqn{(1 - ICC)} times the transformed-scale SE).
#'
#' **Scope of the SE formula.** For the two-way consistency ICC(3,1)
#' (\code{icc_type = "consistency"}) this SE is exact at leading order: deriving
#' it from \eqn{F_0 = MSR/MSE} (with degrees of freedom \eqn{n-1} and
#' \eqn{(n-1)(k-1)}) reduces to the same expression, and it still depends on the
#' ICC value (it is \emph{not} \eqn{\rho}-free). For the two-way
#' absolute-agreement ICC(2,1) (\code{icc_type = "agreement"}, the default) the
#' same formula is only a **one-way approximation that assumes negligible
#' between-rater variance**: when raters differ systematically
#' (\eqn{\sigma^2_{rater} > 0}), the ICC(2,1) estimator depends on the
#' between-rater mean square, which has only \eqn{k - 1} degrees of freedom, so
#' its true sampling variance does not shrink at the \eqn{1/n} rate this formula
#' assumes and the reported SE/CI can be markedly anti-conservative (simulation:
#' 95\% CI coverage around 0.74-0.76 with moderate rater variance, degrading as
#' \eqn{n} grows). The exact ICC(2,1) variance requires the rater-variance
#' component, which summary data do not report; a per-row informational flag
#' (V31) marks agreement-type rows for this reason. If the raters are known to
#' be exchangeable (negligible rater variance), the approximation is accurate.
#'
#' **Scale note.** Under the default \code{icc_to_es = "bonett"} the returned
#' \code{icc_se} column is on the \eqn{\ln(1 - ICC)} scale. If you feed it to
#' \code{\link{compute_sem}} (whose \code{icc_se} argument expects the RAW-scale
#' SE), convert it first: \code{raw_se = icc_se * (1 - icc)} - or call
#' \code{es_from_icc} with \code{icc_to_es = "raw"}.
#'
#' @export es_from_icc
#'
#' @references
#' Bonett, D. G. (2002). Sample size requirements for estimating intraclass correlations
#' with desired precision. Statistics in Medicine, 21(9), 1331-1335.
#'
#' Shrout, P. E., & Fleiss, J. L. (1979). Intraclass correlations: uses in assessing rater
#' reliability. Psychological Bulletin, 86(2), 420-428.
#'
#' @md
#'
#' @return
#' This function estimates the standard error of the ICC.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab icc\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab N/A\cr
#'  \tab \cr
#'  \code{required input data} \tab icc + n_sample + n_measurements\cr
#'  \tab \cr
#' }
#'
#' @examples
#' es_from_icc(
#'   icc = 0.80, n_sample = 50, n_measurements = 2, icc_type = "agreement"
#' )
es_from_icc <- function(icc, n_sample, n_measurements, icc_type = "agreement",
                        icc_to_es = "bonett") {

  if (missing(n_sample)) n_sample <- rep(NA, length(icc))
  if (missing(n_measurements)) n_measurements <- rep(NA, length(icc))
  if (missing(icc_type)) icc_type <- rep("agreement", length(icc))
  icc_type[is.na(icc_type)] <- "agreement"
  if (length(icc_type) == 1) icc_type <- rep(icc_type, length(icc))

  if (!all(icc_type %in% c("agreement", "consistency"))) {
    stop(paste0("'", unique(icc_type[!icc_type %in% c("agreement", "consistency")]),
                "' not in tolerated values for the 'icc_type' argument. ",
                "Possible inputs are: 'agreement', 'consistency'"))
  }

  if (!icc_to_es %in% c("bonett", "raw")) {
    stop(paste0("'", icc_to_es, "' not in tolerated values for the 'icc_to_es' argument. ",
                "Possible inputs are: 'bonett', 'raw'"))
  }

  # P16: per-element guards -- |icc| > 1 is impossible (V11 bounds), icc = 1
  # has no Bonett transform (log(0)) and a degenerate raw SE, n <= 1 makes the
  # SE denominator (n - 1) non-positive, and k < 2 leaves the ICC undefined.
  # Direct calls degrade to NA instead of emitting Inf/NaN.
  invalid <- (!is.na(icc) & (icc > 1 | icc < -1)) |
    (!is.na(icc) & icc == 1) |
    (!is.na(n_sample) & n_sample <= 1) |
    (!is.na(n_measurements) & n_measurements < 2)

  nn_miss <- which(!is.na(icc) & !is.na(n_sample) & !is.na(n_measurements) &
                     !invalid)

  n <- length(icc)
  icc_es <- rep(NA_real_, n)
  icc_es_se <- rep(NA_real_, n)

  if (length(nn_miss) != 0) {
    rho <- icc[nn_miss]
    ns <- n_sample[nn_miss]
    k <- n_measurements[nn_miss]

    # Bonett (2002) variance-stabilised SE of ln(1 - ICC) for a single-measure ICC.
    # For the two-way consistency ICC(3,1), deriving Var(ln(1 - ICC)) directly from
    # F0 = MSR / MSE (df = n - 1 and (n - 1)(k - 1)) gives
    # [(1 + (k - 1) * rho) / k]^2 * 2k / ((k - 1)(n - 1)), which reduces to the
    # expression below (confirmed by simulation for the consistency case) -- it is
    # NOT independent of rho. For the two-way absolute-agreement ICC(2,1) the same
    # expression is only a one-way approximation that assumes NEGLIGIBLE
    # between-rater variance; with sigma^2_rater > 0 the ICC(2,1) estimator depends
    # on MSC (k - 1 df only) and this SE is anti-conservative (see the roxygen
    # details and the V31 informational flag). The exact ICC(2,1) variance needs
    # the rater-variance component, which summary data do not carry.
    bonett_transformed_se <- sqrt(
      2 * (1 + (k - 1) * rho)^2 / (k * (k - 1) * (ns - 1))
    )

    if (icc_to_es == "bonett") {
      icc_es[nn_miss] <- log(1 - rho)
      icc_es_se[nn_miss] <- bonett_transformed_se
    } else {
      icc_es[nn_miss] <- rho
      icc_es_se[nn_miss] <- (1 - rho) * bonett_transformed_se
    }
  }

  icc_ci_lo <- icc_es - qnorm(0.975) * icc_es_se
  icc_ci_up <- icc_es + qnorm(0.975) * icc_es_se

  result <- data.frame(
    icc = icc_es,
    icc_se = icc_es_se,
    icc_ci_lo = icc_ci_lo,
    icc_ci_up = icc_ci_up,
    n_sample = n_sample,
    n_measurements = n_measurements,
    icc_type = icc_type,
    info_used = "icc"
  )

  return(result)
}

#' Compute change-score reliability
#'
#' @param reliability single-occasion reliability coefficient (e.g., Cronbach's alpha or ICC)
#' @param r_pre_post pre-post correlation of observed scores
#'
#' @details
#' Computes the reliability of a change (difference) score from a single-occasion
#' reliability coefficient and the pre-post correlation of observed scores,
#' assuming equal variances AND equal reliabilities at pre and post
#' (Lord, 1963; Cronbach & Furby, 1970):
#'
#' \deqn{rel_{change} = \frac{rel - r_{12}}{1 - r_{12}}}
#'
#' This is the equal-variance/equal-reliability special case of the general
#' Lord (1963) formula; when the pre and post variances (or reliabilities)
#' differ materially, the general formula should be used instead.
#'
#' This is useful for disattenuating change-score correlations, where the
#' reliability of the change score is needed rather than the single-occasion reliability.
#'
#' A negative result (which occurs when \code{r_pre_post} exceeds
#' \code{reliability}) is population-impossible under classical test theory
#' and signals inconsistent inputs; it is returned as computed but triggers a
#' warning, since a negative value fed to \code{\link{es_disattenuate}} as a
#' reliability would produce \code{NaN} (square root of a negative number).
#'
#' @export reliability_change_score
#'
#' @references
#' Lord, F. M. (1963). Elementary models for measuring change.
#' In C. W. Harris (Ed.), Problems in measuring change. University of Wisconsin Press.
#'
#' Cronbach, L. J., & Furby, L. (1970). How we should measure "change" - or should we?
#' Psychological Bulletin, 74(1), 68-80.
#'
#' @md
#'
#' @return
#' A data.frame containing the change-score reliability (\code{rel_change}).
#'
#' @examples
#' reliability_change_score(reliability = 0.85, r_pre_post = 0.60)
reliability_change_score <- function(reliability, r_pre_post) {

  if (any(!is.na(r_pre_post) & r_pre_post >= 1, na.rm = TRUE)) {
    warning("r_pre_post >= 1 will produce Inf/NaN (division by zero in Lord formula)")
  }
  if (any(!is.na(reliability) & (reliability < 0 | reliability > 1), na.rm = TRUE)) {
    warning("reliability outside [0, 1] detected; results may be invalid")
  }
  if (any(!is.na(r_pre_post) & (r_pre_post < -1 | r_pre_post > 1), na.rm = TRUE)) {
    warning("r_pre_post outside [-1, 1] detected; results may be invalid")
  }

  rel_change <- (reliability - r_pre_post) / (1 - r_pre_post)

  # r_pre_post > reliability is population-impossible under classical test
  # theory (it would require a true-score correlation > 1); the negative
  # output is preserved but flagged, since it will produce NaN downstream if
  # used as a reliability in es_disattenuate().
  if (any(!is.na(rel_change) & rel_change < 0, na.rm = TRUE)) {
    warning("Negative change-score reliability produced (r_pre_post > reliability): inputs are inconsistent under classical test theory, and a negative value used as a reliability in es_disattenuate() will produce NaN")
  }

  result <- data.frame(
    rel_change = rel_change
  )

  return(result)
}


#' Compute Standard Error of Measurement (SEM) from reliability and SD
#'
#' @param sd standard deviation of the PROM/test scores
#' @param icc intraclass correlation coefficient (test-retest reliability)
#' @param n_sample sample size (used for the SEM sampling-variance estimation).
#'   Required in BOTH variance regimes: it feeds \eqn{Var(SD)} in the
#'   external-\code{icc_se} delta method and the degrees of freedom of the
#'   same-sample chi-square variance, so when it is omitted the SEM standard
#'   error and CI are \code{NA}.
#' @param icc_se standard error of the ICC (optional), on the RAW ICC scale.
#'   Note that \code{\link{es_from_icc}} under its default
#'   \code{icc_to_es = "bonett"} returns a column named \code{icc_se} on the
#'   \eqn{\ln(1 - ICC)} scale: that value must be converted first
#'   (\code{raw_se = bonett_se * (1 - icc)}) before being passed here (or use
#'   \code{icc_to_es = "raw"}, which returns the raw-scale SE directly).
#'   When supplied, the ICC is treated as an independent external estimate;
#'   when omitted, the ICC and SD are assumed to come from the same sample
#'   (see Details).
#' @param n_measurements number of measurement occasions or raters (k) used to
#'   estimate the ICC; default 2 (test-retest); must be >= 2. Only used when
#'   \code{icc_se} is not supplied.
#'
#' @details
#' Computes the standard error of measurement (SEM) from a reliability
#' coefficient and the standard deviation of scores:
#'
#' \deqn{SEM = SD \times \sqrt{1 - ICC}}
#'
#' The sampling variance of SEM is computed in one of two ways:
#'
#' 1. **When \code{icc_se} is supplied**, the ICC is treated as an external
#' estimate, independent of the within-sample SD, and the bivariate delta method
#' (with zero covariance) is used:
#' \deqn{Var(SEM) = \frac{SD^2}{4(1 - ICC)} \times Var(ICC) + (1 - ICC) \times Var(SD)}
#' with \eqn{Var(ICC) = icc\_se^2} and \eqn{Var(SD) \approx SD^2 / (2(n-1))}.
#'
#' 2. **When \code{icc_se} is not supplied**, the ICC and SD are assumed to come
#' from the same reliability sample, where they are strongly positively
#' correlated. Because \eqn{SEM = SD\sqrt{1 - ICC}} equals the within-subject
#' residual root-mean-square \eqn{\sqrt{MSE}}, and \eqn{MSE / \sigma_e^2} follows
#' a scaled chi-square with \eqn{(n-1)(k-1)} degrees of freedom, the exact
#' sampling variance is
#' \deqn{Var(SEM) = \frac{SEM^2}{2(n-1)(k-1)}}
#' where \eqn{k} is the number of measurement occasions/raters
#' (\code{n_measurements}, default 2 for test-retest). Treating the same-sample SD
#' and ICC as independent (the delta method of case 1) would overestimate this
#' variance by roughly 2-6x, so the exact form is used instead.
#'
#' The 95% CI is likewise regime-specific. In the same-sample regime (case 2)
#' the degrees of freedom \eqn{df = (n - 1)(k - 1)} are known, so the exact
#' chi-square interval
#' \deqn{[SEM \sqrt{df / \chi^2_{0.975, df}},\; SEM \sqrt{df / \chi^2_{0.025, df}}]}
#' is returned; the symmetric Wald interval undercovers at small df (89.7%
#' instead of 95% at n = 10, k = 2). When \code{icc_se} is supplied (case 1)
#' the df behind the external estimate is unknown, so the Wald interval
#' \eqn{SEM \pm 1.96 \times SE}, truncated at 0, is kept.
#'
#' Degenerate inputs: \code{n_measurements} = 1 makes the same-sample df zero,
#' so the SE and CI are returned as \code{NA} with a warning; an ICC > 1 is
#' impossible and yields \code{NA} for the SEM, its SE and CI (with a warning).
#'
#' @export compute_sem
#'
#' @references
#' Weir, J. P. (2005). Quantifying test-retest reliability using the intraclass correlation
#' coefficient and the SEM. Journal of Strength and Conditioning Research, 19(1), 231-240.
#'
#' @md
#'
#' @return
#' A data.frame with the SEM, its standard error and 95% CI.
#'
#' @examples
#' compute_sem(sd = 10, icc = 0.85, n_sample = 100)
compute_sem <- function(sd, icc, n_sample, icc_se, n_measurements = 2) {

  if (missing(icc_se)) icc_se <- rep(NA_real_, length(sd))
  if (missing(n_sample)) n_sample <- rep(NA_real_, length(sd))
  if (missing(n_measurements)) n_measurements <- rep(2, length(sd))
  n_measurements[is.na(n_measurements)] <- 2
  if (length(n_measurements) == 1) n_measurements <- rep(n_measurements, length(sd))

  if (any(!is.na(sd) & sd < 0, na.rm = TRUE)) {
    warning("Negative SD detected; SEM will be invalid")
  }
  if (any(!is.na(icc) & (icc < 0 | icc > 1), na.rm = TRUE)) {
    warning("ICC outside [0, 1] detected; SEM is invalid (set to NA when ICC > 1)")
  }
  # icc_se must be the RAW-scale SE of the ICC. A raw-scale SE larger than
  # (1 - icc) is implausible and is the signature of es_from_icc()'s default
  # Bonett output (the SE of ln(1 - ICC)) being passed through unchanged.
  # Warn only - the value is used exactly as supplied.
  scale_suspect <- !is.na(icc_se) & !is.na(icc) & icc < 1 & icc_se > (1 - icc)
  if (any(scale_suspect, na.rm = TRUE)) {
    warning("icc_se looks like a transformed-scale (Bonett, ln(1 - ICC)) standard error (icc_se > 1 - icc); compute_sem() needs the RAW-scale ICC standard error. If it comes from es_from_icc() with the default icc_to_es = 'bonett', convert it first: raw_se = bonett_se * (1 - icc). The value is used as supplied.")
  }

  # ICC > 1 is impossible: sqrt(1 - icc) would be NaN, and no variance is
  # defensible - return NA (not NaN paired with a zero SE).
  sem <- sd * sqrt(pmax(1 - icc, 0))
  sem[!is.na(icc) & icc > 1] <- NA_real_
  k <- n_measurements

  # Same-sample df = (n - 1)(k - 1); k = 1 (or n = 1) makes it zero, so the
  # chi-square variance/CI below would divide by zero - invalidate the df and
  # warn where the same-sample regime actually needs it.
  df_sem <- (n_sample - 1) * (k - 1)
  df_sem[!is.na(df_sem) & df_sem < 1] <- NA_real_
  if (any(is.na(icc_se) & !is.na(k) & k < 2, na.rm = TRUE)) {
    warning("n_measurements must be >= 2 to estimate the same-sample SEM sampling variance; SE and CI set to NA for the affected row(s)")
  }

  # Sampling variance of SEM, in two regimes:
  #
  # (A) icc_se SUPPLIED -- the ICC is an external estimate, independent of the
  #     within-sample SD, so the bivariate delta method with Cov(SD, ICC) = 0
  #     applies:
  #       Var(SEM) = (SD^2 / (4(1 - ICC))) * Var(ICC) + (1 - ICC) * Var(SD)
  #     with Var(ICC) = icc_se^2 and Var(SD) = SD^2 / (2(n - 1)).
  #
  # (B) icc_se NOT supplied -- ICC and SD come from the SAME reliability sample and
  #     are strongly positively correlated; treating them as independent (case A)
  #     overestimates Var(SEM) by ~2-6x. Since SEM = SD * sqrt(1 - ICC) = sqrt(MSE)
  #     (the within-subject residual RMS), and MSE / sigma_e^2 ~ chi-square / df
  #     with df = (n - 1)(k - 1), the exact sampling variance is
  #       Var(SEM) = SEM^2 / (2 (n - 1)(k - 1)),   k = n_measurements (default 2).
  #     Verified by simulation to within ~1%.
  var_sd <- sd^2 / (2 * (n_sample - 1))
  var_sem_delta <- (sd^2 / (4 * (1 - icc))) * icc_se^2 + (1 - icc) * var_sd
  var_sem_exact <- sem^2 / (2 * df_sem)

  # icc > 1: no defensible variance (NA, not the old maximally-confident 0);
  # icc == 1 exactly: the coherent degenerate case sem = 0, se = 0.
  var_sem <- ifelse(!is.na(icc) & icc > 1, NA_real_,
                    ifelse(!is.na(icc) & icc == 1, 0,
                           ifelse(!is.na(icc_se), var_sem_delta, var_sem_exact)))
  sem_se <- sqrt(var_sem)

  # CI, regime-specific:
  # - same-sample (icc_se missing): df is known, use the exact chi-square
  #   interval for sigma_e (the Wald interval undercovers at small df,
  #   89.7% at n = 10, k = 2);
  # - external icc_se: df unknown, keep the truncated Wald interval.
  same_sample <- is.na(icc_se)
  sem_ci_lo <- ifelse(same_sample,
                      sem * sqrt(df_sem / stats::qchisq(0.975, df_sem)),
                      pmax(0, sem - qnorm(0.975) * sem_se))
  sem_ci_up <- ifelse(same_sample,
                      sem * sqrt(df_sem / stats::qchisq(0.025, df_sem)),
                      sem + qnorm(0.975) * sem_se)

  result <- data.frame(
    sem = sem,
    sem_se = sem_se,
    sem_ci_lo = sem_ci_lo,
    sem_ci_up = sem_ci_up
  )

  return(result)
}


#' Compute Smallest Detectable Change (SDC) from SEM
#'
#' @param sem standard error of measurement
#' @param sem_se standard error of the SEM (optional; for propagating uncertainty)
#'
#' @details
#' Computes the smallest detectable change (SDC) from the standard error
#' of measurement:
#'
#' \deqn{SDC = 1.96 \times \sqrt{2} \times SEM}
#'
#' The SDC represents the smallest change in scores that can be detected
#' beyond measurement error with 95% confidence. If \code{sem_se} is provided,
#' the uncertainty in SDC is propagated:
#'
#' \deqn{SDC\_se = 1.96 \times \sqrt{2} \times SEM\_se}
#'
#' @export compute_sdc
#'
#' @references
#' de Vet, H. C. W., Terwee, C. B., Mokkink, L. B., & Knol, D. L. (2011).
#' Measurement in Medicine: A Practical Guide. Cambridge University Press.
#'
#' @md
#'
#' @return
#' A data.frame with the SDC, its standard error and 95% CI.
#'
#' @examples
#' sem_res <- compute_sem(sd = 10, icc = 0.85, n_sample = 100)
#' compute_sdc(sem = sem_res$sem, sem_se = sem_res$sem_se)
compute_sdc <- function(sem, sem_se) {

  if (missing(sem_se)) sem_se <- rep(NA, length(sem))

  multiplier <- qnorm(0.975) * sqrt(2)

  sdc <- multiplier * sem
  sdc_se <- multiplier * sem_se

  sdc_ci_lo <- pmax(0, sdc - qnorm(0.975) * sdc_se)
  sdc_ci_up <- sdc + qnorm(0.975) * sdc_se

  result <- data.frame(
    sdc = sdc,
    sdc_se = sdc_se,
    sdc_ci_lo = sdc_ci_lo,
    sdc_ci_up = sdc_ci_up
  )

  return(result)
}

#' Convert a standardized regression coefficient and the standard deviation of the dependent variable
#' into several effect size measures
#'
#' @param beta_std a standardized regression coefficient value (binary predictor, no other covariables in the model)
#' @param sd_dv standard deviation of the dependent variable
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_beta_std a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts a standardized linear regression coefficient
#' (coming from a model with only one binary predictor), into an
#' unstandardized linear regression coefficient.
#'
#' \deqn{sd\_dummy = \sqrt{\frac{n_exp - (n_exp^2 / (n_exp + n_nexp))}{(n_exp + n_nexp - 1)}}}
#' \deqn{unstd\_beta = beta\_std * \frac{sd\_dv}{sd\_dummy}}
#'
#' Calculations of the \code{\link{es_from_beta_unstd}} functions are then used.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 13. (Un-)Standardized regression coefficient'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
#'
#' @export es_from_beta_std
#'
#' @md
#'
#' @examples
#' es_from_beta_std(beta_std = 0.35, sd_dv = 0.98, n_exp = 20, n_nexp = 22)
es_from_beta_std <- function(beta_std, sd_dv, n_exp, n_nexp,
                             smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_beta_std) {
  if (missing(reverse_beta_std)) reverse_beta_std <- rep(FALSE, length(beta_std))
  reverse_beta_std[is.na(reverse_beta_std)] <- FALSE
  if (length(reverse_beta_std) == 1) reverse_beta_std = c(rep(reverse_beta_std, length(beta_std)))
  if (length(reverse_beta_std) != length(beta_std)) stop("The length of the 'reverse_beta_std' argument is incorrectly specified.")

  beta_std <- ifelse(reverse_beta_std, -beta_std, beta_std)


  sd_dummy <- sqrt((n_exp - (n_exp^2 / (n_exp + n_nexp))) / (n_exp + n_nexp - 1))

  unstd_beta <- beta_std * (sd_dv / sd_dummy)

  es <- es_from_beta_unstd(
    beta_unstd = unstd_beta, sd_dv = sd_dv,
    n_exp = n_exp, n_nexp = n_nexp, smd_to_cor = smd_to_cor, smd_var = smd_var
  )

  es$info_used <- "beta_std"

  return(es)
}

#' Convert an unstandardized regression coefficient and the standard deviation of the dependent variable
#' into several effect size measures
#'
#' @param beta_unstd an unstandardized regression coefficient value (binary predictor, no other covariables in the model)
#' @param sd_dv standard deviation of the dependent variable
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_beta_unstd a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function estimates a Cohen's d (D) and Hedges' g (G) from an unstandardized linear regression coefficient (coming from a model with only one binary predictor),
#' and the standard deviation of the dependent variable.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **The formula used to obtain the Cohen's d is**:
#' \deqn{N = n\_exp + n\_nexp}
#' \deqn{sd\_pooled = \sqrt{\frac{sd\_dv^2 * (N - 1) - unstd\_beta^2 * \frac{n\_exp * n\_nexp}{N}}{N - 2}}}
#' \deqn{cohen\_d = \frac{unstd\_beta}{sd\_pooled}}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 13. (Un-)Standardized regression coefficient'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
#'
#' @export es_from_beta_unstd
#'
#' @md
#'
#' @examples
#' es_from_beta_unstd(beta_unstd = 0.7, sd_dv = 0.98, n_exp = 20, n_nexp = 22)
es_from_beta_unstd <- function(beta_unstd, sd_dv, n_exp, n_nexp,
                               smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_beta_unstd) {
  if (missing(reverse_beta_unstd)) reverse_beta_unstd <- rep(FALSE, length(beta_unstd))
  reverse_beta_unstd[is.na(reverse_beta_unstd)] <- FALSE


  # Within-group variance from the ANOVA SS decomposition
  # SS_total = SS_between + SS_within, with SS_between = beta_unstd^2 * n_exp*n_nexp/N.
  # This is >= 0 for any genuine binary-predictor OLS; it goes negative only when the
  # reported sd_dv is too small to be consistent with beta_unstd (a mathematically
  # impossible input, e.g. |standardized beta| > 1, or the residual SD entered as sd_dv).
  # Such rows are set to NA rather than masked with abs(), which would fabricate a
  # plausible-looking effect size.
  within_var <- ((sd_dv^2 * (n_exp + n_nexp - 1)) -
                 (beta_unstd^2 * ((n_exp * n_nexp) / (n_exp + n_nexp)))) /
                (n_exp + n_nexp - 2)

  n_impossible <- sum(within_var < 0, na.rm = TRUE)
  if (n_impossible > 0) {
    warning(sprintf(
      "es_from_beta_unstd: %d row(s) had a reported sd_dv too small to be consistent with beta_unstd (implied within-group variance < 0). These inputs are mathematically impossible (e.g. |standardized beta| > 1); the effect sizes are set to NA for those rows.",
      n_impossible
    ), call. = FALSE)
  }

  within_var <- ifelse(within_var < 0, NA_real_, within_var)
  sd_pooled <- sqrt(within_var) # NA where the inputs were impossible

  d <- beta_unstd / sd_pooled

  es <- .es_from_d(
    d = d, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_beta_unstd
  )

  es$info_used <- "beta_unstd"

  return(es)
}


#' Convert a t-statistic from a linear regression model to several effect size measures
#'
#' @param linreg_t a t-statistic from a linear regression model
#' @param n_sample the total number of participants
#' @param n_covariates the number of covariates in the model (excluding the predictor of interest).
#' @param sd_iv the standard deviation of the independent variable (optional, see details)
#' @param unit_increase_iv a value of the independent variable that will be used to estimate the Cohen's d (optional, see details).
#' @param unit_type the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "raw_scale"
#' @param n_exp number of the experimental/exposed group (optional)
#' @param n_nexp number of the non-experimental/non-exposed group (optional)
#' @param cor_to_smd formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.
#' @param reverse_linreg_t a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts a t-statistic from a linear regression model into a
#' partial correlation coefficient (Rp) and its Fisher's z transformation (Zp).
#'
#' The partial correlation is obtained as (Aloe & Thompson, 2013, Eq. 2; Gustafson, 1961):
#' \deqn{r_p = \frac{t}{\sqrt{t^2 + df}}}
#' where \eqn{df = n\_sample - n\_covariates - 2} is the residual degrees of freedom
#' of the regression model (i.e. \eqn{n} minus the intercept, the focal predictor,
#' and the \code{n_covariates} covariates).
#'
#' Its sampling variance is estimated as recommended by van Aert & Goos (2023, Eq. 5):
#' \deqn{var(r_p) = \frac{(1 - r_p^2)^2}{df}}
#'
#' The Fisher's z transformation and its variance are:
#' \deqn{z_p = atanh(r_p)}
#' \deqn{var(z_p) = \frac{1}{n - n\_covariates - 3}}
#'
#' Cohen's d, Hedges' g and odds ratio are then converted from the partial correlation
#' using the calculations of the \code{\link{es_from_pearson_r}} function.
#' A partial correlation controls for covariates and thus targets a different estimand
#' than a two-group comparison or a bivariate correlation (Aloe & Thompson, 2013);
#' the converted d/g/OR values should not be pooled with such effect sizes.
#' For meta-analyses of regression results, use \code{measure = "rp"} or
#' \code{measure = "zp"} in \code{\link{convert_df}}.
#'
#' @export es_from_linreg_t
#'
#' @references
#' Aloe, A. M., & Thompson, C. G. (2013). The synthesis of partial effect sizes.
#' \emph{Journal of the Society for Social Work and Research}, 4(4), 390--405.
#'
#' Gustafson, R. L. (1961). Partial correlations in regression computations.
#' \emph{Journal of the American Statistical Association}, 56, 363--367.
#'
#' Mathur, M. B., & VanderWeele, T. J. (2020). A simple, interpretable conversion from
#' Pearson's correlation to Cohen's d for continuous exposures.
#' \emph{Epidemiology}, 31(2), e16--e18.
#'
#' van Aert, R. C. M., & Goos, C. (2023). A critical reflection on computing the sampling
#' variance of the partial correlation coefficient. \emph{Research Synthesis Methods},
#' 14(3), 520--525.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab Rp (partial correlation) + Zp (Fisher's z of partial r)\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + OR (approximate, see details)\cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_linreg_t(linreg_t = 2.5, n_sample = 100, n_covariates = 2)
#'
#' # Reproduce Aloe & Thompson (2013) Table 1, Cole et al. (2004)
#' es_from_linreg_t(linreg_t = 6.19, n_sample = 232, n_covariates = 6)
es_from_linreg_t <- function(linreg_t, n_sample, n_covariates,
                             sd_iv, unit_increase_iv, unit_type = "raw_scale",
                             n_exp, n_nexp, cor_to_smd = "viechtbauer",
                             reverse_linreg_t) {

  if (missing(reverse_linreg_t)) reverse_linreg_t <- rep(FALSE, length(linreg_t))
  reverse_linreg_t[is.na(reverse_linreg_t)] <- FALSE
  if (length(reverse_linreg_t) == 1) reverse_linreg_t <- rep(reverse_linreg_t, length(linreg_t))
  if (length(reverse_linreg_t) != length(linreg_t)) stop("The length of the 'reverse_linreg_t' argument is incorrectly specified.")

  if (missing(n_exp)) n_exp <- rep(NA, length(linreg_t))
  if (missing(n_nexp)) n_nexp <- rep(NA, length(linreg_t))
  if (missing(sd_iv)) sd_iv <- rep(NA, length(linreg_t))
  if (missing(unit_increase_iv)) unit_increase_iv <- rep(NA, length(linreg_t))
  if (missing(unit_type)) unit_type <- rep(NA, length(linreg_t))

  if (!all(cor_to_smd %in% c("cooper", "mathur", "viechtbauer"))) {
    stop(paste0("'",
                unique(cor_to_smd[!cor_to_smd %in% c("cooper", "mathur", "viechtbauer")]),
                "' not in tolerated values for the 'cor_to_smd' argument.
                Possible inputs are: 'cooper', 'mathur', 'viechtbauer'"))
  }

  df <- n_sample - n_covariates - 2

  # A non-positive residual df makes the partial correlation undefined for THAT row.
  # convert_df() calls this function on the whole column, so a hard stop() would abort
  # the entire run because of one bad cell (violating the package's "one bad cell cannot
  # abort a convert_df() run" convention). NA the offending rows and warn instead, so the
  # valid rows still compute -- mirroring the sibling es_from_beta_unstd().
  n_bad <- sum(df <= 0, na.rm = TRUE)
  if (n_bad > 0) {
    warning(sprintf(
      "es_from_linreg_t: %d row(s) had a non-positive residual df (n_sample - n_covariates - 2 <= 0); their effect sizes were set to NA.",
      n_bad), call. = FALSE)
    df[!is.na(df) & df <= 0] <- NA_real_
  }

  rp <- linreg_t / sqrt(linreg_t^2 + df)
  rp <- ifelse(reverse_linreg_t, -rp, rp)

  rp_se <- sqrt((1 - rp^2)^2 / df)
  rp_ci_lo <- rp - qt(.975, df) * rp_se
  rp_ci_up <- rp + qt(.975, df) * rp_se

  zp <- atanh(rp)
  # target Fisher-z variance of a partial correlation controlling for
  # n_covariates variables is 1 / (n - n_covariates - 3); with the residual
  # df = n - n_covariates - 2 this is 1 / (df - 1).
  zp_se <- sqrt(1 / (df - 1))
  zp_ci_lo <- zp - qnorm(.975) * zp_se
  zp_ci_up <- zp + qnorm(.975) * zp_se

  n_sample_calc <- ifelse(is.na(n_sample), n_exp + n_nexp, n_sample)
  n_exp <- ifelse(is.na(n_exp), n_sample_calc / 2, n_exp)
  n_nexp <- ifelse(is.na(n_nexp), n_sample_calc / 2, n_nexp)

  dat_cor <- data.frame(
    r = rp, r_se = rp_se,
    sd_iv = sd_iv, n_sample = n_sample_calc,
    unit_increase_iv = unit_increase_iv,
    unit_type = unit_type,
    cor_to_smd = cor_to_smd
  )

  nn_miss <- with(dat_cor, which(
    (cor_to_smd == "mathur" & !is.na(r) & !is.na(sd_iv) &
      !is.na(n_sample) & !is.na(unit_increase_iv) & !is.na(unit_type)) |
      (cor_to_smd == "cooper" & !is.na(r) & !is.na(r_se)) |
      (cor_to_smd == "viechtbauer" & !is.na(r) & !is.na(r_se) & !is.na(n_sample))
  ))

  es <- data.frame(
    d = rep(NA, nrow(dat_cor)),
    d_se = rep(NA, nrow(dat_cor))
  )

  if (length(nn_miss) != 0) {
    res_d <- t(mapply(.cor_to_smd,
      r = dat_cor$r[nn_miss],
      r_se = dat_cor$r_se[nn_miss],
      n_sample = dat_cor$n_sample[nn_miss],
      sd_iv = dat_cor$sd_iv[nn_miss],
      unit_increase_iv = dat_cor$unit_increase_iv[nn_miss],
      unit_type = dat_cor$unit_type[nn_miss],
      cor_to_smd = dat_cor$cor_to_smd[nn_miss]
    ))

    es$d[nn_miss] <- unlist(res_d[, 1])
    es$d_se[nn_miss] <- unlist(res_d[, 2])
  }

  es <- .es_from_d(d = es$d, d_se = es$d_se, n_exp = n_exp, n_nexp = n_nexp, n_sample = n_sample_calc)

  es$rp <- rp
  es$rp_se <- rp_se
  es$rp_ci_lo <- rp_ci_lo
  es$rp_ci_up <- rp_ci_up

  es$zp <- zp
  es$zp_se <- zp_se
  es$zp_ci_lo <- zp_ci_lo
  es$zp_ci_up <- zp_ci_up

  es$info_used <- "linreg_t"

  return(es)
}


#' Convert an unstandardized regression coefficient and its standard error
#' into several effect size measures
#'
#' @param linreg_b unstandardized regression coefficient from a linear regression model
#' @param linreg_b_se standard error of the regression coefficient
#' @param n_sample the total number of participants
#' @param n_covariates the number of covariates in the model (excluding the predictor of interest).
#' @param sd_iv the standard deviation of the independent variable (optional, see details)
#' @param unit_increase_iv a value of the independent variable that will be used to estimate the Cohen's d (optional, see details).
#' @param unit_type the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "raw_scale"
#' @param n_exp number of the experimental/exposed group (optional)
#' @param n_nexp number of the non-experimental/non-exposed group (optional)
#' @param cor_to_smd formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.
#' @param reverse_linreg_b a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function derives a t-statistic from a regression coefficient and its standard error,
#' then converts it into a partial correlation and other effect size measures.
#'
#' The Wald t-statistic is computed as:
#' \deqn{t = \frac{b}{SE(b)}}
#'
#' Once the t-statistic is obtained, all subsequent conversions follow the same formulas as
#' in \code{\link{es_from_linreg_t}}.
#'
#' For binary predictors without covariates, use \code{\link{es_from_beta_unstd}} instead,
#' which converts directly to Cohen's d.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab Rp (partial correlation) + Zp (Fisher's z of partial r)\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + OR (approximate, see \code{\link{es_from_linreg_t}})\cr
#' }
#'
#' @references
#' Aloe, A. M., & Thompson, C. G. (2013). The synthesis of partial effect sizes.
#' \emph{Journal of the Society for Social Work and Research}, 4(4), 390--405.
#'
#' van Aert, R. C. M., & Goos, C. (2023). A critical reflection on computing the sampling
#' variance of the partial correlation coefficient. \emph{Research Synthesis Methods},
#' 14(3), 520--525.
#'
#' @export es_from_linreg_b_se
#'
#' @md
#'
#' @examples
#' es_from_linreg_b_se(linreg_b = 1.5, linreg_b_se = 0.6, n_sample = 100, n_covariates = 2)
es_from_linreg_b_se <- function(linreg_b, linreg_b_se, n_sample, n_covariates,
                                sd_iv, unit_increase_iv, unit_type = "raw_scale",
                                n_exp, n_nexp, cor_to_smd = "viechtbauer",
                                reverse_linreg_b) {

  if (missing(reverse_linreg_b)) reverse_linreg_b <- rep(FALSE, length(linreg_b))
  reverse_linreg_b[is.na(reverse_linreg_b)] <- FALSE
  if (length(reverse_linreg_b) == 1) reverse_linreg_b <- rep(reverse_linreg_b, length(linreg_b))
  if (length(reverse_linreg_b) != length(linreg_b)) stop("The length of the 'reverse_linreg_b' argument is incorrectly specified.")

  if (missing(n_exp)) n_exp <- rep(NA, length(linreg_b))
  if (missing(n_nexp)) n_nexp <- rep(NA, length(linreg_b))
  if (missing(sd_iv)) sd_iv <- rep(NA, length(linreg_b))
  if (missing(unit_increase_iv)) unit_increase_iv <- rep(NA, length(linreg_b))
  if (missing(unit_type)) unit_type <- rep(NA, length(linreg_b))

  linreg_t <- linreg_b / linreg_b_se

  es <- es_from_linreg_t(
    linreg_t = linreg_t, n_sample = n_sample, n_covariates = n_covariates,
    sd_iv = sd_iv, unit_increase_iv = unit_increase_iv, unit_type = unit_type,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_linreg_t = reverse_linreg_b
  )

  es$info_used <- "linreg_b_se"

  return(es)
}


#' Convert an unstandardized regression coefficient and its confidence interval
#' into several effect size measures
#'
#' @param linreg_b unstandardized regression coefficient from a linear regression model
#' @param linreg_b_ci_lo lower bound of the 95% confidence interval of the regression coefficient
#' @param linreg_b_ci_up upper bound of the 95% confidence interval of the regression coefficient
#' @param n_sample the total number of participants
#' @param n_covariates the number of covariates in the model (excluding the predictor of interest).
#' @param sd_iv the standard deviation of the independent variable (optional, see details)
#' @param unit_increase_iv a value of the independent variable that will be used to estimate the Cohen's d (optional, see details).
#' @param unit_type the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "raw_scale"
#' @param n_exp number of the experimental/exposed group (optional)
#' @param n_nexp number of the non-experimental/non-exposed group (optional)
#' @param cor_to_smd formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.
#' @param reverse_linreg_b a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function derives the standard error from the 95% confidence interval using a
#' t-distribution with \eqn{df = n\_sample - n\_covariates - 2} degrees of freedom:
#' \deqn{SE(b) = \frac{ci\_up - ci\_lo}{2 \times qt(.975, df)}}
#'
#' Then, calculations of the \code{\link{es_from_linreg_b_se}} function are applied.
#' For binary predictors without covariates, use \code{\link{es_from_beta_unstd}} instead.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab Rp (partial correlation) + Zp (Fisher's z of partial r)\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + OR (approximate, see \code{\link{es_from_linreg_t}})\cr
#' }
#'
#' @references
#' Aloe, A. M., & Thompson, C. G. (2013). The synthesis of partial effect sizes.
#' \emph{Journal of the Society for Social Work and Research}, 4(4), 390--405.
#'
#' @export es_from_linreg_b_ci
#'
#' @md
#'
#' @examples
#' es_from_linreg_b_ci(
#'   linreg_b = 1.5, linreg_b_ci_lo = 0.3, linreg_b_ci_up = 2.7,
#'   n_sample = 100, n_covariates = 2
#' )
es_from_linreg_b_ci <- function(linreg_b, linreg_b_ci_lo, linreg_b_ci_up,
                                n_sample, n_covariates,
                                sd_iv, unit_increase_iv, unit_type = "raw_scale",
                                n_exp, n_nexp, cor_to_smd = "viechtbauer",
                                reverse_linreg_b) {

  if (missing(reverse_linreg_b)) reverse_linreg_b <- rep(FALSE, length(linreg_b))
  reverse_linreg_b[is.na(reverse_linreg_b)] <- FALSE
  if (length(reverse_linreg_b) == 1) reverse_linreg_b <- rep(reverse_linreg_b, length(linreg_b))
  if (length(reverse_linreg_b) != length(linreg_b)) stop("The length of the 'reverse_linreg_b' argument is incorrectly specified.")

  if (missing(n_exp)) n_exp <- rep(NA, length(linreg_b))
  if (missing(n_nexp)) n_nexp <- rep(NA, length(linreg_b))
  if (missing(sd_iv)) sd_iv <- rep(NA, length(linreg_b))
  if (missing(unit_increase_iv)) unit_increase_iv <- rep(NA, length(linreg_b))
  if (missing(unit_type)) unit_type <- rep(NA, length(linreg_b))

  df <- n_sample - n_covariates - 2
  linreg_b_se <- (linreg_b_ci_up - linreg_b_ci_lo) / (2 * qt(.975, df))

  es <- es_from_linreg_b_se(
    linreg_b = linreg_b, linreg_b_se = linreg_b_se,
    n_sample = n_sample, n_covariates = n_covariates,
    sd_iv = sd_iv, unit_increase_iv = unit_increase_iv, unit_type = unit_type,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_linreg_b = reverse_linreg_b
  )

  es$info_used <- "linreg_b_ci"

  return(es)
}


#' Convert an unstandardized regression coefficient and its p-value
#' into several effect size measures
#'
#' @param linreg_b unstandardized regression coefficient from a linear regression model
#' @param linreg_b_pval two-sided p-value of the regression coefficient
#' @param n_sample the total number of participants
#' @param n_covariates the number of covariates in the model (excluding the predictor of interest).
#' @param sd_iv the standard deviation of the independent variable (optional, see details)
#' @param unit_increase_iv a value of the independent variable that will be used to estimate the Cohen's d (optional, see details).
#' @param unit_type the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "raw_scale"
#' @param n_exp number of the experimental/exposed group (optional)
#' @param n_nexp number of the non-experimental/non-exposed group (optional)
#' @param cor_to_smd formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.
#' @param reverse_linreg_b_pval a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function recovers the t-statistic from the two-sided p-value:
#' \deqn{t = qt(1 - pval/2, df) \times sign(b)}
#' where \eqn{df = n\_sample - n\_covariates - 2}.
#'
#' The sign of the regression coefficient gives the direction, which the two-tailed
#' p-value does not carry.
#' Then, calculations of the \code{\link{es_from_linreg_t}} function are applied.
#' For binary predictors without covariates, use \code{\link{es_from_beta_unstd}} instead.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab Rp (partial correlation) + Zp (Fisher's z of partial r)\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + OR (approximate, see \code{\link{es_from_linreg_t}})\cr
#' }
#'
#' @references
#' Aloe, A. M., & Thompson, C. G. (2013). The synthesis of partial effect sizes.
#' \emph{Journal of the Society for Social Work and Research}, 4(4), 390--405.
#'
#' @export es_from_linreg_b_pval
#'
#' @md
#'
#' @examples
#' es_from_linreg_b_pval(
#'   linreg_b = 1.5, linreg_b_pval = 0.01,
#'   n_sample = 100, n_covariates = 2
#' )
es_from_linreg_b_pval <- function(linreg_b, linreg_b_pval,
                                  n_sample, n_covariates,
                                  sd_iv, unit_increase_iv, unit_type = "raw_scale",
                                  n_exp, n_nexp, cor_to_smd = "viechtbauer",
                                  reverse_linreg_b_pval) {

  if (missing(reverse_linreg_b_pval)) reverse_linreg_b_pval <- rep(FALSE, length(linreg_b))
  reverse_linreg_b_pval[is.na(reverse_linreg_b_pval)] <- FALSE
  if (length(reverse_linreg_b_pval) == 1) reverse_linreg_b_pval <- rep(reverse_linreg_b_pval, length(linreg_b))
  if (length(reverse_linreg_b_pval) != length(linreg_b)) stop("The length of the 'reverse_linreg_b_pval' argument is incorrectly specified.")

  if (missing(n_exp)) n_exp <- rep(NA, length(linreg_b))
  if (missing(n_nexp)) n_nexp <- rep(NA, length(linreg_b))
  if (missing(sd_iv)) sd_iv <- rep(NA, length(linreg_b))
  if (missing(unit_increase_iv)) unit_increase_iv <- rep(NA, length(linreg_b))
  if (missing(unit_type)) unit_type <- rep(NA, length(linreg_b))

  df <- n_sample - n_covariates - 2
  linreg_t <- qt(1 - linreg_b_pval / 2, df) * sign(linreg_b)

  es <- es_from_linreg_t(
    linreg_t = linreg_t, n_sample = n_sample, n_covariates = n_covariates,
    sd_iv = sd_iv, unit_increase_iv = unit_increase_iv, unit_type = unit_type,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_linreg_t = reverse_linreg_b_pval
  )

  es$info_used <- "linreg_b_pval"

  return(es)
}

#' Automatically compute effect sizes from a well formatted dataset
#'
#' @param x a well formatted dataset
#' @param measure the effect size measure that will be estimated from the information stored in the dataset. See details.
#' @param main_es a logical variable indicating whether a main effect size should be selected when overlapping data are present. See details.
#' @param split_adjusted a logical value indicating whether crude and adjusted effect sizes should be presented separately. See details.
#' @param format_adjusted presentation format of the adjusted effect sizes. See details.
#' @param hierarchy a character string indicating the hierarchy in the information to be prioritized for the effect size calculations. See details.
#' @param selection_auto a character string giving details on the best "auto" hierarchy to use (only useful when \code{hierarchy="auto"} and \code{measure= "d", "g" or "md"}). See details.
#' @param es_selected the method used to select the main effect size when several information allows to estimate an effect size for the same association/comparison. Must be either "minimum" (the smallest effect size will be selected), "maximum" (the largest effect size will be selected) or "hierarchy" (the effect size computed from the information specified highest in the hierarchy will be selected). See details.
#' @param table_2x2_to_cor formula used to obtain a correlation coefficient from the contingency table. For now only 'tetrachoric' is available.
#' @param rr_to_or formula used to convert the \code{rr} value into an odds ratio.
#' @param or_to_rr formula used to convert the \code{or} value into a risk ratio.
#' @param or_to_cor formula used to convert the \code{or} value into a correlation coefficient.
#' @param pre_post_to_smd formula used to obtain a SMD from pre/post means and SD of two independent groups.
#' @param pool_sd a logical value indicating whether the standardizing SD should be pooled across the two
#'   groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
#'   equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
#'   which to prefer, so this is a deliberate choice and not a technical detail.
#'   \itemize{
#'     \item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
#'       within-group values are subtracted, their variances adding because the arms are independent. This
#'       is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
#'       SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
#'       broadly applicable of the two.
#'     \item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
#'       is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
#'       assumes the two arms' true standardizing SDs are equal.
#'   }
#' @param r_pre_post pre-post correlation across the two groups (use this argument only if the precise correlation in each group is unknown). Note that when this correlation is defaulted rather than reported, the pre/post standard errors and confidence intervals are unreliable: at a true correlation of 0.3 with the default 0.8, the reported pre/post variance is about 68% too small (95% CI coverage ~0.75). Supply \code{r_pre_post_exp}/\code{r_pre_post_nexp} when available, or run a sensitivity analysis over plausible values.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation.
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param smd_denom standardizer for the standardized mean difference. "pooled" (default) uses the pooled endpoint SD (Cohen's d / Hedges' g); "glass" (alias "control") uses the control (non-experimental) endpoint SD (Glass's delta); "glass_robust" (alias "control_robust") is Glass's delta with a heteroscedasticity-consistent sampling variance. Only the endpoint means family (es_from_means_sd/se/ci) honours this argument: rows whose effect size comes from any other method (t/F, cohen_d/hedges_g, eta-squared, point-biserial r, medians/ranges, plots, ANCOVA, raw mean differences) always use the pooled-SD standardizer, and a message lists the scoping when a non-pooled value is requested ("glass_robust" additionally has a single variance form, so smd_var is ignored for it).
#' @param cor_to_smd formula used to convert a correlation coefficient value into a SMD.
#' @param prop_to_es method used to compute the effect size from the proportion. Must be either "raw", "logit" or "freeman_tukey" (see \code{\link{es_from_prop_single_group}}).
#' @param alpha_to_es method used to compute the effect size from Cronbach's alpha. Must be either "bonett" or "raw" (see \code{\link{es_from_cronbach_alpha}}).
#' @param icc_to_es method used to compute the effect size from an ICC. Must be either "bonett" or "raw" (see \code{\link{es_from_icc}}).
#' @param yates_chisq a logical value indicating whether the Chi square has been performed using Yates' correction for continuity. Can also be given as a column of the dataset when studies differ (rows left NA use this argument).
#' @param unit_type the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "value" (see \code{\link{es_from_pearson_r}}).
#' @param max_asymmetry A percentage indicating the tolerance before detecting asymmetry in the 95% CI bounds. Asymmetric CIs are only flagged, not modified (see \code{\link{summary.metaConvert}}).
#' @param verbose a logical variable indicating whether text outputs and messages should be generated. We recommend turning this option to FALSE only after having carefully read all the generated messages.
#' @param correct_inputs a logical value indicating whether invalid input values (negative SDs, inverted CI bounds, etc.) should be set to NA before the calculations (default TRUE). If FALSE, invalid values are only flagged in the 'es_flags' column and kept in the data.
#' @param flag_options a named list of thresholds used by the quality flags (see \code{\link{summary.metaConvert}} for the list of options and defaults).
#'
#' @details
#' This function automatically computes or converts between 14 effect sizes
#' measures from any relevant type of input data stored in the
#' dataset you pass to this function.
#'
#' ## Input validation
#' Before the calculations, the input data are checked (negative sample
#' sizes/SD/SE/p-values, inverted CI bounds, values outside their CI,
#' asymmetric CI). By default invalid values are set to NA; issues appear
#' in the 'es_flags' column of summary().
#'
#' ## Effect size measures
#' Possible effect size measures are:
#' 1. Cohen's d ("d")
#' 2. Hedges' g ("g")
#' 3. mean difference ("md")
#' 4. (log) odds ratio ("or" and "logor")
#' 5. (log) risk ratio ("rr" and "logrr")
#' 6. (log) incidence rate ratio ("irr" and "logirr")
#' 7. correlation coefficient ("r")
#' 8. transformed r-to-z correlation coefficient ("z")
#' 9. partial correlation coefficient ("rp")
#' 10. Fisher's z of partial correlation ("zp")
#' 11. log variability ratio ("logvr")
#' 12. log coefficient of variation ("logcvr")
#' 13. number needed to treat ("nnt")
#' 14. risk difference ("rd")
#'
#' ## Computation of a main effect size
#' If you enter multiple types of input data
#' (e.g., means/sd of two groups and a student t-test value)
#' for the same comparison i.e., for the same row of the dataset,
#' the \code{convert_df()} function can have two behaviours.
#' If you set:
#' - \code{main_es = FALSE} the function will estimate all possible effect sizes from all
#' types of input data (which implies that if a comparison has **several types of input data**,
#' it will result in **multiple rows** in the dataframe returned by the function)
#' - \code{main_es = TRUE} the function will select one effect size per comparison
#' (which implies that if a comparison has **several types of input data**,
#' it will result in a **unique row** in the dataframe returned by the function)
#'
#' ## Selection of input data for the computation of the main effect size
#' If you choose to estimate one main effect size (i.e., by setting \code{main_es = TRUE}),
#' you have several options to select this main effect size.
#' If you set:
#' - \code{es_selected = "auto"}: the main effect size will be **automatically** selected, by prioritizing
#' specific types of input data over other (see next section "Hierarchy").
#' - \code{es_selected = "hierarchy"}: the main effect size will be selected, by prioritizing
#' specific types of input data over other (see next section "Hierarchy").
#' - \code{es_selected = "minimum"}: the main effect size will be selected, by selecting
#' the lowest effect size available.
#' - \code{es_selected = "maximum"}: the main effect size will be selected, by selecting
#' the highest effect size available.
#'
#' ## Hierarchy
#' More than 70 different combinations of input data can be used to estimate an effect size.
#' You can retrieve the effect size measures estimated by each combination of input data
#' in the \code{\link{see_input_data}()} function and online \code{https://metaconvert.org/input.html}.
#'
#' You have two options to use a hierarchy in the types of input data.
#' * an automatic way (\code{es_selected = "auto"})
#' * an manual way (\code{es_selected = "hierarchy"})
#'
#' ### Automatic
#' If you select an automatic hierarchy, here are the types of input data that will be prioritized.
#'
#' #### Crude SMD or MD (\code{measure=c("d", "g", "md")} and \code{selection_auto="crude"})
#' 1. User's input effect size value
#' 2. SMD value
#' 3. Means at post-test
#' 4. ANOVA/Student's t-test/point biserial correlation statistics
#' 5. Linear regression estimates
#' 6. Mean difference values
#' 7. Quartiles/median/maximum values
#' 8. Post-test means extracted from a plot
#' 9. Pre-test+post-test means or mean change
#' 10. Paired ANOVA/t-test statistics
#' 11. Odds ratio value
#' 12. Contingency table
#' 13. Correlation coefficients
#' 14. Phi/chi-square value
#'
#' #### Paired SMD or MD (\code{measure=c("d", "g", "md")} and \code{selection_auto="paired"})
#' 1. User's input effect size value
#' 2. Paired SMD value
#' 3. Pre-test+post-test means or mean change
#' 4. Paired ANOVA/t-test statistics
#' 5. Means at post-test
#' 6. ANOVA/Student's t-test/point biserial correlation
#' 7. Linear regression estimates
#' 8. Mean difference values
#' 9. Quartiles/median/maximum values
#' 10. Odds ratio value
#' 11. Contingency table
#' 12. Correlation coefficients
#' 13. Phi/chi-square value
#'
#' #### Adjusted SMD or MD (\code{measure=c("d", "g", "md")} and \code{selection_auto="adjusted"})
#' 1. User's input adjusted effect size value
#' 2. Adjusted SMD value
#' 3. Estimated marginal means from ANCOVA
#' 4. F- or t-test value from ANCOVA
#' 5. Adjusted mean difference from ANCOVA
#' 6. Estimated marginal means from ANCOVA extracted from a plot
#'
#' #### Odds Ratio (\code{measure=c("or")})
#' 1.	User's input effect size value
#' 2.	Odds ratio value
#' 3.	Contingency table
#' 4.	Risk ratio values
#' 5.	Phi/chi-square value
#' 6.	Correlation coefficients
#' 7.	(Then hierarchy as for "d" or "g" option crude)
#'
#' #### Risk Ratio (\code{measure=c("rr")})
#' 1.	User's input effect size value
#' 2.	Risk ratio values
#' 3.	Contingency table
#' 4.	Odds ratio values
#' 5.	Phi/chi-square value
#'
#' #### Incidence rate ratio (\code{measure=c("irr")})
#' 1.	User's input effect size value
#' 2.	Number of cases and time of disease free observation time
#'
#' #### Correlation (\code{measure=c("r", "z")})
#' 1. User's input effect size value
#' 2. Correlation coefficients
#' 3. Contingency table
#' 4. Odds ratio value
#' 5. Phi/chi-square value
#' 6. SMD value
#' 7. Means at post-test
#' 8. ANOVA/Student's t-test/point biserial correlation
#' 9. Linear regression estimates
#' 10. Mean difference values
#' 11 Quartiles/median/maximum values
#' 12. Post-test means extracted from a plot
#' 13. Pre-test+post-test means or mean change
#' 14. Paired ANOVA/t-test
#'
#' #### Variability ratios (\code{measure=c("vr", "cvr")})
#' 1.	User's input effect size value
#' 2. means/variability indices at post-test
#' 3. means/variability indices at post-test extracted from a plot
#'
#' #### Number needed to treat (\code{measure=c("nnt")})
#' 1.	User's input effect size value
#' 2.	Contingency table
#' 3.	Odds ratio values
#' 4.	Risk ratio values
#' 5.	Incidence rate ratio (person-time NNT, requires baseline_rate)
#' 6.	Phi/chi-square value
#'
#' NNT values should not be pooled directly (the NNT confidence interval is
#' disjoint when the risk difference crosses zero). You should pool RD/OR/RR
#' values and convert the pooled estimate to NNT (Deeks, 2002; Cochrane
#' Handbook, Chapter 15).
#'
#' #### Risk difference (\code{measure=c("rd")})
#' 1.	User's input effect size value
#' 2.	Contingency table
#' 3.	Odds ratio values
#' 4.	Risk ratio values
#' 5.	Incidence rate ratio (requires baseline_rate)
#' 6.	Phi/chi-square value
#'
#' ### Manual
#'
#' If you select a manual hierarchy, you can specify the order in which you want to
#' use each type of input data.
#' You can prioritize some types of input data by placing them at the begining of the
#' hierarchy argument, and you must separate all input data with a ">" separator.
#' For example, if you set:
#' - \code{hierarchy = "means_sd > means_se > student_t"}, the convert_df function will prioritize
#' the means + SD, then the means + SE, then the Student's t-test to estimate the main effect
#' size.
#' - \code{hierarchy = "2x2 > or_se > phi"}, the convert_df function will prioritize
#' the contingency table, then the odds ratio value + SE, then the phi coefficient to estimate
#' the main effect size.
#'
#' Importantly, if none of the types of input data indicated in the \code{hierarchy} argument
#' can be used to estimate the target effect size measure,
#' the \code{convert_df()} function will automatically try to use other types of input
#' data to estimate an effect size.
#'
#' ## Adjusted effect sizes
#' Some datasets will be composed of crude (i.e., non-adjusted) types of input data
#' (such as standard means + SD, Student's t-test, etc.) and adjusted types of input data
#' (such as means + SE from an ANCOVA model, a t-test from an ANCOVA, etc.).
#'
#' In these situations, you can decide to:
#' - treat crude and adjusted input data the same way \code{split_adjusted = FALSE}
#' - split calculations for crude and adjusted types of input data \code{split_adjusted = TRUE}
#'
#' If you want to split the calculations, you can decide to present the final dataset:
#' - in a long format (i.e., crude and adjusted effect sizes presented in separate rows \code{format_adjusted = "long"})
#' - in a wide format (i.e., crude and adjusted effect sizes presented in separate columns \code{format_adjusted = "wide"})
#'
#' @return
#' The \code{convert_df()} function returns a list of
#' more than 70 dataframes
#' (one for each function automatically applied to the dataset).
#' These dataframes systematically contain the columns described in
#' \code{\link{metaConvert-package}}.
#' The list of dataframes can be easily converted to a single,
#' calculations-ready dataframe
#' using the summary function (see \code{\link{summary.metaConvert}}).
#'
#' @md
#'
#' @export convert_df
#'
#' @examples
#' res <- convert_df(df.haza,
#'   measure = "g",
#'   split_adjusted = TRUE,
#'   es_selected = "minimum",
#'   format_adjusted = "long"
#' )
#' summary(res)
convert_df <- function(x, measure = c("d", "g", "md", "logor", "logrr", "logirr",
                                      "nnt", "rd", "r", "z", "rp", "zp", "logvr", "logcvr"),
                       main_es = TRUE,
                       es_selected = c("auto", "hierarchy", "minimum", "maximum"),
                       selection_auto = c("crude", "paired", "adjusted"),
                       split_adjusted = TRUE,
                       format_adjusted = c("wide", "long"),
                       verbose = TRUE,
                       max_asymmetry = 50,
                       hierarchy = "means_sd > means_se > means_ci",
                       table_2x2_to_cor = "tetrachoric",
                       rr_to_or = "metaumbrella",
                       or_to_rr = "metaumbrella_cases",
                       or_to_cor = "bonett",
                       smd_to_cor = "viechtbauer",
                       smd_var = "borenstein",
                       smd_denom = "pooled",
                       pre_post_to_smd = "bonett",
                       r_pre_post = 0.8,
                       cor_to_smd = "viechtbauer",
                       unit_type = "raw_scale",
                       yates_chisq = FALSE,
                       pool_sd = FALSE,
                       prop_to_es = "raw",
                       alpha_to_es = "bonett",
                       icc_to_es = "bonett",
                       correct_inputs = TRUE,
                       flag_options = list()) {
  # @param table_2x2_to_cor formula used to obtain a correlation coefficient from the contingency table.
  # table_2x2_to_cor = "tetrachoric",
  # x = dat
  # measure = "nnt";
  # main_es = TRUE;
  # es_selected = "hierarchy";
  # split_adjusted = TRUE;
  # format_adjusted = "wide";
  # verbose = TRUE;
  # hierarchy = "means_sd > means_se > means_ci";
  # rr_to_or = "metaumbrella";
  # or_to_rr = "dipietrantonj";
  # or_to_cor = "bonett";
  # table_2x2_to_cor = "lipsey";
  # smd_to_cor = "viechtbauer";
  # chisq_to_cor = "tetrachoric";
  # phi_to_cor = "tetrachoric";
  # pre_post_to_smd = "bonett";
  # cor_to_smd = "viechtbauer";
  # unit_type = "raw_scale";
  # yates_chisq = FALSE
  # max_asymmetry = 5
  # selection_auto = c("crude", "paired", "adjusted")
  measure = measure[1]
  es_selected = es_selected[1]
  selection_auto = selection_auto[1]
  format_adjusted = format_adjusted[1]

  x <- .check_data(x, split_adjusted = split_adjusted,
                   main_es = main_es, format = format_adjusted)
  # rows with defaulted r_pre_post
  .r_defaulted_exp <- is.na(x[, "r_pre_post_exp"])
  .r_defaulted_nexp <- is.na(x[, "r_pre_post_nexp"])
  x[.r_defaulted_exp, "r_pre_post_exp"] <- r_pre_post
  x[.r_defaulted_nexp, "r_pre_post_nexp"] <- r_pre_post

  # An unreported r that is silently imputed is a substantive assumption. How it
  # bites depends on the standardizer: for change-SD methods (morris_dz, cooper/
  # morris_drm) the assumed r scales the POINT estimate; for baseline-SD (bonett) and
  # average-SD (morris_dav) the point estimate is r-free and only the SE/CI move.
  # Warn once, and only for rows that actually feed an r-consuming (pre/post, mean-
  # change, paired) route -- datasets with no such data never reach the formulas.
  if (verbose) {
    .r_consuming_cols <- c(
      "mean_pre_exp", "mean_pre_nexp", "mean_pre_sd_exp", "mean_pre_sd_nexp",
      "mean_change_exp", "mean_change_nexp",
      "mean_change_sd_exp", "mean_change_sd_nexp",
      "mean_change_se_exp", "mean_change_se_nexp",
      "mean_change_pval_exp", "mean_change_pval_nexp",
      "paired_t_exp", "paired_t_nexp", "paired_f_exp", "paired_f_nexp",
      "paired_t_pval_exp", "paired_t_pval_nexp",
      "paired_f_pval_exp", "paired_f_pval_nexp"
    )
    .r_consuming_cols <- intersect(.r_consuming_cols, colnames(x))
    if (length(.r_consuming_cols) > 0) {
      .rows_with_pp <- rowSums(!is.na(x[, .r_consuming_cols, drop = FALSE])) > 0
      .n_imputed <- sum((.r_defaulted_exp | .r_defaulted_nexp) & .rows_with_pp)
      if (.n_imputed > 0) {
        # Change-SD standardizers make the point estimate itself depend on r; the
        # mean-change and paired-t/F routes are always standardized by the change SD
        # (coerced to cooper), so their point estimates depend on r regardless of the
        # requested method.
        .r_scales_point <- pre_post_to_smd %in% c("morris_dz", "morris_drm", "cooper")
        .impact <- if (.r_scales_point) {
          paste0("Under the '", pre_post_to_smd, "' standardizer the assumed\n",
                 "  correlation scales the POINT estimate, not only the SE.")
        } else {
          paste0("Under the '", pre_post_to_smd, "' standardizer the point estimate is\n",
                 "  r-free and only the SE/CI depend on the assumed correlation; note that\n",
                 "  mean-change and paired-t/F rows are standardized by the change SD, whose\n",
                 "  POINT estimate does depend on r.")
        }
        message(
          "Note: r_pre_post was not reported for ", .n_imputed, " row(s) with pre/post, ",
          "mean-change or paired data;\n  the default r_pre_post = ", r_pre_post,
          " was assumed. ", .impact,
          "\n  Supply r_pre_post_exp/r_pre_post_nexp when available, or run a ",
          "sensitivity analysis\n  over plausible values."
        )
      }
    }
  }

  r_pre_post = rep(r_pre_post, nrow(x))
  for (i in c("rr_to_or",
              "or_to_rr",
              "or_to_cor",
              # "table_2x2_to_cor",
              "smd_to_cor",
              "smd_var",
              "smd_denom",
              "pre_post_to_smd",
              "cor_to_smd",
              "unit_type")) {
    if (i %in% colnames(x)) {
      if (length(sapply(mget(i), function(x) x)) > 1) {
        stop("The length of the argument '", i, "' should be 1, or should be specified in a column of the dataset.")
      }
      x[, i][is.na(x[, i])] <- sapply(mget(i), function(x) x)
    } else {
      if (length(sapply(mget(i), function(x) x)) > 1) {
        stop("The length of the argument '", i, "' should be 1, or should be specified in a column of the dataset.")
      }
      x[, i] <- sapply(mget(i), function(x) x)
    }
  }

  # P10: smd_denom is honoured only by the endpoint means family; rows reaching
  # the SMD via any other route keep the pooled-SD standardizer. Say so once,
  # or a "Glass" analysis silently mixes estimands across methods.
  .denom_used <- .normalize_smd_denom(x[, "smd_denom"])
  if (any(!is.na(.denom_used) & .denom_used != "pooled")) {
    message(
      "Note: smd_denom is honoured by the endpoint means family only ",
      "(es_from_means_sd, es_from_means_se, es_from_means_ci).\n  Rows whose effect size ",
      "comes from any other method (t/F statistics, cohen_d/hedges_g, eta-squared,\n  ",
      "point-biserial r, medians/ranges/quartiles, plots, ANCOVA, raw mean differences) ",
      "use the pooled-SD\n  standardizer. Check 'info_used' in summary() to see which ",
      "standardizer each row received."
    )
  }

  # input validation
  enable_info <- if (!is.null(flag_options$enable_informational)) {
    flag_options$enable_informational
  } else {
    .default_flag_options()$enable_informational
  }
  sd_ratio_max_val <- if (!is.null(flag_options$sd_ratio_max)) {
    flag_options$sd_ratio_max
  } else {
    .default_flag_options()$sd_ratio_max
  }
  sd_ratio_bl_ep_min_val <- if (!is.null(flag_options$sd_ratio_bl_ep_min)) {
    flag_options$sd_ratio_bl_ep_min
  } else {
    .default_flag_options()$sd_ratio_bl_ep_min
  }
  baseline_imb_max_val <- if (!is.null(flag_options$baseline_imbalance_max)) {
    flag_options$baseline_imbalance_max
  } else {
    .default_flag_options()$baseline_imbalance_max
  }
  enable_cross_row_val <- if (!is.null(flag_options$enable_cross_row)) {
    flag_options$enable_cross_row
  } else {
    .default_flag_options()$enable_cross_row
  }
  templated_min_match_val <- if (!is.null(flag_options$templated_min_match)) {
    flag_options$templated_min_match
  } else {
    .default_flag_options()$templated_min_match
  }
  paired_as_indep_tol_val <- if (!is.null(flag_options$paired_as_indep_tol)) {
    flag_options$paired_as_indep_tol
  } else {
    .default_flag_options()$paired_as_indep_tol
  }
  validation <- .validate_input_data(x, max_asymmetry = max_asymmetry, verbose = verbose,
                                      enable_informational = enable_info,
                                      sd_ratio_max = sd_ratio_max_val,
                                      sd_ratio_bl_ep_min = sd_ratio_bl_ep_min_val,
                                      baseline_imbalance_max = baseline_imb_max_val,
                                      enable_cross_row = enable_cross_row_val,
                                      templated_min_match = templated_min_match_val,
                                      measure = measure,
                                      paired_as_indep_tol = paired_as_indep_tol_val,
                                      correct_inputs = correct_inputs)
  x <- validation$data

  # V6: Flag rows where r_pre_post used the default value AND pre-post data is present
  .pre_post_cols <- c(
    "mean_pre_exp", "mean_post_exp", "mean_pre_nexp", "mean_post_nexp",
    "mean_change_exp", "mean_change_nexp",
    "mean_pre_single_group", "mean_post_single_group",
    "mean_change_single_group", "paired_t", "paired_f"
  )
  .pre_post_cols <- intersect(.pre_post_cols, colnames(x))
  .has_pre_post <- if (length(.pre_post_cols) > 0) {
    rowSums(!is.na(x[, .pre_post_cols, drop = FALSE])) > 0
  } else {
    rep(FALSE, nrow(x))
  }
  .r_defaulted <- (.r_defaulted_exp | .r_defaulted_nexp) & .has_pre_post
  for (i in which(.r_defaulted)) {
    msg <- sprintf("[INFO] Default r_pre_post = %s used (not provided by user)", r_pre_post[1])
    if (nzchar(validation$issues[i])) {
      validation$issues[i] <- paste(validation$issues[i], msg, sep = "; ")
    } else {
      validation$issues[i] <- msg
    }
  }

  if (verbose) message("Calculations in progress, it may take up to 30 sec...")
  measure = tolower(measure)
  if (!measure %in% c("d", "g", "md", "dw", "gw", "mdw", "r", "z", "rp", "zp", "or", "rr", "irr", "hr", "logor", "logrr", "logirr", "loghr", "logvr", "logcvr", "nnt", "rd", "prop", "alpha", "icc")) {
    stop(paste0("'", measure, "' not in tolerated measures. Possible inputs are: 'md', 'd', 'g', 'dw', 'gw', 'mdw', 'or', 'rr', 'irr', 'hr', 'logor', 'logrr', 'logirr', 'loghr', 'r', 'z', 'rp', 'zp', 'logvr', 'logcvr', 'nnt', 'rd', 'prop', 'alpha', 'icc'"))
  } else if (!split_adjusted %in% c(TRUE, FALSE)) {
    stop(paste0("'", split_adjusted, "' not in tolerated values for the 'split_adjusted' argument. Should be a logical value (TRUE/FALSE)"))
  } else if (!format_adjusted %in% c("wide", "long")) {
    stop(paste0("'", format_adjusted, "' not in tolerated values for the 'format_adjusted' argument. Should be either 'long' or 'wide'."))
  } else if (!es_selected %in% c("auto", "minimum", "maximum", "hierarchy")) {
    stop(paste0("'", es_selected, "' not in tolerated values for the 'es_selected' argument. Possible inputs are: 'minimum', 'maximum', 'hierarchy'"))
  } else if (!main_es %in% c(TRUE, FALSE)) {
    stop(paste0("'", main_es, "' not in tolerated values for the 'main_es' argument. Should be a logical value (TRUE/FALSE)"))
  }

  if (measure %in% c("or", "rr", "irr", "hr")) {
    exp <- TRUE
    if (measure == "or") {
      measure <- "logor"
    } else if (measure == "rr") {
      measure <- "logrr"
    } else if (measure == "irr") {
      measure <- "logirr"
    } else if (measure == "hr") {
      measure <- "loghr"
    }
  } else {
    exp <- FALSE
  }

  # SMD ----------------------------------------------
  es_cohen_d <- with(x, es_from_cohen_d(
    cohen_d = cohen_d, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_d = reverse_d)
  )

  es_cohen_d_adj <- with(x, es_from_cohen_d_adj(
    cohen_d_adj = cohen_d_adj, n_cov_ancova = n_cov_ancova, cov_outcome_r = cov_outcome_r,
    n_exp = n_exp, n_nexp = n_nexp, smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_d = reverse_d
  ))
  es_hedges_g <- with(x, es_from_hedges_g(
    hedges_g = hedges_g, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_g = reverse_g
  ))


  # OR ----------------------------------------------
  es_odds_ratio <- with(x, es_from_or(
    or = or, logor = logor, baseline_risk = baseline_risk, n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls, n_sample = n_sample, small_margin_prop = small_margin_prop,
    or_to_rr = or_to_rr, or_to_cor = or_to_cor, reverse_or = reverse_or
  ))
  es_odds_ratio_se <- with(x, es_from_or_se(
    or = or, logor = logor, logor_se = logor_se, small_margin_prop = small_margin_prop,
    baseline_risk = baseline_risk,
    n_exp = n_exp, n_nexp = n_nexp, n_sample = n_sample,
    n_cases = n_cases, n_controls = n_controls,
    or_to_rr = or_to_rr, or_to_cor = or_to_cor,
    reverse_or = reverse_or
  ))
  es_odds_ratio_ci <- with(x, es_from_or_ci(
    or = or, or_ci_lo, or_ci_up, logor = logor, small_margin_prop = small_margin_prop,
    baseline_risk = baseline_risk, n_exp = n_exp, n_nexp = n_nexp,
    logor_ci_lo = logor_ci_lo, logor_ci_up = logor_ci_up,
    n_cases = n_cases, n_controls = n_controls, n_sample = n_sample,
    or_to_rr = or_to_rr, or_to_cor = or_to_cor, reverse_or = reverse_or,
    max_asymmetry = max_asymmetry
  ))
  es_odds_ratio_pval <- with(x, es_from_or_pval(
    or = or, logor = logor, or_pval = or_pval, small_margin_prop = small_margin_prop,
    baseline_risk = baseline_risk, n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls, n_sample = n_sample,
    or_to_rr = or_to_rr, or_to_cor = or_to_cor, reverse_or_pval = reverse_or_pval
  ))
  es_logreg_t <- with(x, es_from_logreg_t(
    or = or, logor = logor, rr = rr, logrr = logrr,
    logreg_t = logreg_t, small_margin_prop = small_margin_prop,
    baseline_risk = baseline_risk, n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls, n_sample = n_sample,
    or_to_rr = or_to_rr, or_to_cor = or_to_cor,
    rr_to_or = rr_to_or,
    reverse_logreg_t = reverse_logreg_t
  ))
  # R ----------------------------------------------
  es_pearson_r <- with(x, es_from_pearson_r(
    pearson_r = pearson_r, n_sample = n_sample,
    unit_type = unit_type, sd_iv = sd_iv,
    unit_increase_iv = unit_increase_iv,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_pearson_r = reverse_pearson_r
  ))
  es_fisher_z <- with(x, es_from_fisher_z(
    fisher_z = fisher_z, n_sample = n_sample,
    unit_type = unit_type, sd_iv = sd_iv,
    unit_increase_iv = unit_increase_iv,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_fisher_z = reverse_fisher_z
  ))
  es_spearman_r <- with(x, es_from_spearman_rho(
    spearman_r = spearman_r, n_sample = n_sample,
    sd_iv = sd_iv, unit_increase_iv = unit_increase_iv,
    unit_type = unit_type,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_spearman_r = reverse_spearman_r
  ))

  # MEANS ----------------------------------------------
  es_means_sd_raw <- with(x, es_from_means_sd(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_exp = mean_exp, mean_sd_exp = mean_sd_exp,
    mean_nexp = mean_nexp, mean_sd_nexp = mean_sd_nexp,
    smd_var = smd_var, smd_denom = smd_denom, smd_to_cor = smd_to_cor, reverse_means = reverse_means
  )
  )
  es_means_se_raw <- with(
    x,
    es_from_means_se(
      n_exp = n_exp, n_nexp = n_nexp,
      mean_exp = mean_exp, mean_se_exp = mean_se_exp,
      mean_nexp = mean_nexp, mean_se_nexp = mean_se_nexp,
      smd_var = smd_var, smd_denom = smd_denom, smd_to_cor = smd_to_cor, reverse_means = reverse_means
    )
  )
  es_means_ci_raw <- with(
    x,
    es_from_means_ci(
      n_exp = n_exp, n_nexp = n_nexp,
      mean_exp = mean_exp, mean_ci_lo_exp = mean_ci_lo_exp, mean_ci_up_exp = mean_ci_up_exp,
      mean_nexp = mean_nexp, mean_ci_lo_nexp = mean_ci_lo_nexp, mean_ci_up_nexp = mean_ci_up_nexp,
      smd_var = smd_var, smd_denom = smd_denom, smd_to_cor = smd_to_cor, reverse_means = reverse_means,
      max_asymmetry = max_asymmetry
    )
  )
  es_means_sd_pooled <- with(
    x,
    es_from_means_sd_pooled(
      n_exp = n_exp, n_nexp = n_nexp, mean_exp = mean_exp, mean_nexp = mean_nexp,
      mean_sd_pooled = mean_sd_pooled, smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_means = reverse_means
    )
  )
  # plot
  es_plot_means_raw <- with(x, es_from_plot_means(
    n_exp = n_exp, n_nexp = n_nexp,
    plot_mean_exp = plot_mean_exp, plot_mean_nexp = plot_mean_nexp,
    plot_mean_sd_lo_exp = plot_mean_sd_lo_exp, plot_mean_sd_lo_nexp = plot_mean_sd_lo_nexp,
    plot_mean_sd_up_exp = plot_mean_sd_up_exp, plot_mean_sd_up_nexp = plot_mean_sd_up_nexp,
    plot_mean_se_lo_exp = plot_mean_se_lo_exp, plot_mean_se_lo_nexp = plot_mean_se_lo_nexp,
    plot_mean_se_up_exp = plot_mean_se_up_exp, plot_mean_se_up_nexp = plot_mean_se_up_nexp,
    plot_mean_ci_lo_exp = plot_mean_ci_lo_exp, plot_mean_ci_lo_nexp = plot_mean_ci_lo_nexp,
    plot_mean_ci_up_exp = plot_mean_ci_up_exp, plot_mean_ci_up_nexp = plot_mean_ci_up_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor,
    reverse_plot_means = reverse_plot_means
  ))
  # PRE POST MEANS ----------------------------------------------
  es_means_sd_pre_post <- with(
    x,
    es_from_means_sd_pre_post(
      n_exp = n_exp, n_nexp = n_nexp,
      mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
      mean_pre_sd_exp = mean_pre_sd_exp, mean_sd_exp = mean_sd_exp,
      mean_pre_nexp = mean_pre_nexp, mean_nexp = mean_nexp,
      mean_pre_sd_nexp = mean_pre_sd_nexp, mean_sd_nexp = mean_sd_nexp,
      r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

      smd_to_cor = smd_to_cor, reverse_means_pre_post = reverse_means_pre_post,
      pre_post_to_smd = pre_post_to_smd,
      pool_sd = pool_sd
    )
  )
  es_means_se_pre_post <- with(
    x,
    es_from_means_se_pre_post(
      n_exp = n_exp, n_nexp = n_nexp,
      mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
      mean_pre_se_exp = mean_pre_se_exp, mean_se_exp = mean_se_exp,
      mean_pre_nexp = mean_pre_nexp, mean_nexp = mean_nexp,
      mean_pre_se_nexp = mean_pre_se_nexp, mean_se_nexp = mean_se_nexp,
      r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

      smd_to_cor = smd_to_cor, reverse_means_pre_post = reverse_means_pre_post,
      pre_post_to_smd = pre_post_to_smd,
      pool_sd = pool_sd
    )
  )
  es_means_ci_pre_post <- with(
    x,
    es_from_means_ci_pre_post(
      n_exp = n_exp, n_nexp = n_nexp,
      mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
      mean_pre_ci_lo_exp = mean_pre_ci_lo_exp, mean_pre_ci_up_exp = mean_pre_ci_up_exp,
      mean_ci_lo_exp = mean_ci_lo_exp, mean_ci_up_exp = mean_ci_up_exp,
      mean_pre_nexp = mean_pre_nexp, mean_nexp = mean_nexp,
      mean_pre_ci_lo_nexp = mean_pre_ci_lo_nexp, mean_pre_ci_up_nexp = mean_pre_ci_up_nexp,
      mean_ci_lo_nexp = mean_ci_lo_nexp, mean_ci_up_nexp = mean_ci_up_nexp,
      r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

      smd_to_cor = smd_to_cor, reverse_means_pre_post = reverse_means_pre_post,
      pre_post_to_smd = pre_post_to_smd,
      pool_sd = pool_sd,
      max_asymmetry = max_asymmetry
    )
  )

  # bonett/morris_dav need separate pre/post SDs, use cooper for mean change and paired t/f
  pre_post_to_smd_restricted <- if (pre_post_to_smd %in% c("bonett", "morris_dav")) {
    "cooper"
  } else {
    pre_post_to_smd
  }

  # The coercion above silently changes the ESTIMAND for the affected rows: they
  # are standardized by the change SD (cooper/d_rm) while means_sd_pre_post rows
  # keep the requested baseline-SD (bonett) or average-SD (morris_dav)
  # standardizer. Tell the user whenever data that actually routes through a
  # coerced method is present.
  if (verbose && !identical(pre_post_to_smd_restricted, pre_post_to_smd)) {
    .coerced_cols <- c(
      "mean_change_exp", "mean_change_nexp", "mean_change_sd_exp",
      "mean_change_sd_nexp", "mean_change_se_exp", "mean_change_se_nexp",
      "mean_change_pval_exp", "mean_change_pval_nexp",
      "paired_t_exp", "paired_t_nexp", "paired_f_exp", "paired_f_nexp",
      "paired_t_pval_exp", "paired_t_pval_nexp",
      "paired_f_pval_exp", "paired_f_pval_nexp"
    )
    .coerced_cols <- intersect(.coerced_cols, colnames(x))
    .has_coerced_data <- length(.coerced_cols) > 0 &&
      any(!is.na(x[, .coerced_cols, drop = FALSE]))
    if (.has_coerced_data) {
      message(
        "Note: pre_post_to_smd = '", pre_post_to_smd, "' requires separate pre/post SDs, ",
        "which mean-change and paired t/F data do not carry.\n",
        "  Those rows use 'cooper' (morris_drm: the change SD, rescaled by sqrt(2(1 - r)) ",
        "onto the\n  raw-score metric) instead. Both standardizers are on the raw-score ",
        "scale, so the rows\n  remain combinable, but 'morris_drm' additionally assumes ",
        "equal pre/post SDs and depends\n  on r_pre_post. Supply pre/post SDs to use '",
        pre_post_to_smd, "' throughout."
      )
    }
  }

  es_means_change_sd <- with(x, es_from_mean_change_sd(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_change_exp = mean_change_exp, mean_change_sd_exp = mean_change_sd_exp,
    mean_change_nexp = mean_change_nexp, mean_change_sd_nexp = mean_change_sd_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

    smd_to_cor = smd_to_cor, reverse_mean_change = reverse_mean_change,
    pre_post_to_smd = pre_post_to_smd_restricted,
    pool_sd = pool_sd
  ))
  es_mean_change_se <- with(x, es_from_mean_change_se(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_change_exp = mean_change_exp, mean_change_se_exp = mean_change_se_exp,
    mean_change_nexp = mean_change_nexp, mean_change_se_nexp = mean_change_se_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

    smd_to_cor = smd_to_cor, reverse_mean_change = reverse_mean_change,
    pre_post_to_smd = pre_post_to_smd_restricted,
    pool_sd = pool_sd
  ))
  es_mean_change_ci <- with(x, es_from_mean_change_ci(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_change_exp = mean_change_exp,
    mean_change_ci_lo_exp = mean_change_ci_lo_exp, mean_change_ci_up_exp = mean_change_ci_up_exp,
    mean_change_nexp = mean_change_nexp,
    mean_change_ci_lo_nexp = mean_change_ci_lo_nexp, mean_change_ci_up_nexp = mean_change_ci_up_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

    smd_to_cor = smd_to_cor, reverse_mean_change = reverse_mean_change,
    pre_post_to_smd = pre_post_to_smd_restricted,
    pool_sd = pool_sd,
    max_asymmetry = max_asymmetry
  ))
  es_mean_change_pval <- with(x, es_from_mean_change_pval(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_change_exp = mean_change_exp, mean_change_pval_exp = mean_change_pval_exp,
    mean_change_nexp = mean_change_nexp, mean_change_pval_nexp = mean_change_pval_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

    smd_to_cor = smd_to_cor, reverse_mean_change = reverse_mean_change,
    pre_post_to_smd = pre_post_to_smd_restricted,
    pool_sd = pool_sd
  ))

  # SINGLE-GROUP WITHIN-GROUP EFFECTS (experimental group only) --------------
  es_means_pp_sg <- with(x, es_from_means_sd_pre_post_single_group(
    mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
    mean_pre_sd_exp = mean_pre_sd_exp, mean_sd_exp = mean_sd_exp,
    n_exp = n_exp, r_pre_post_exp = r_pre_post_exp,
    pre_post_to_smd = pre_post_to_smd, smd_to_cor = smd_to_cor,
    reverse_means_pre_post = reverse_means_pre_post
  ))

  es_means_se_pp_sg <- with(x, es_from_means_se_pre_post_single_group(
    mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
    mean_pre_se_exp = mean_pre_se_exp, mean_se_exp = mean_se_exp,
    n_exp = n_exp, r_pre_post_exp = r_pre_post_exp,
    pre_post_to_smd = pre_post_to_smd, smd_to_cor = smd_to_cor,
    reverse_means_pre_post = reverse_means_pre_post
  ))

  es_means_ci_pp_sg <- with(x, es_from_means_ci_pre_post_single_group(
    mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
    mean_pre_ci_lo_exp = mean_pre_ci_lo_exp, mean_pre_ci_up_exp = mean_pre_ci_up_exp,
    mean_ci_lo_exp = mean_ci_lo_exp, mean_ci_up_exp = mean_ci_up_exp,
    n_exp = n_exp, r_pre_post_exp = r_pre_post_exp,
    pre_post_to_smd = pre_post_to_smd, smd_to_cor = smd_to_cor,
    max_asymmetry = max_asymmetry,
    reverse_means_pre_post = reverse_means_pre_post
  ))

  es_mean_change_sg <- with(x, es_from_mean_change_sd_single_group(
    mean_change_exp = mean_change_exp, mean_change_sd_exp = mean_change_sd_exp,
    n_exp = n_exp, r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd_restricted,
    reverse_mean_change = reverse_mean_change
  ))

  es_mean_change_se_sg <- with(x, es_from_mean_change_se_single_group(
    mean_change_exp = mean_change_exp, mean_change_se_exp = mean_change_se_exp,
    n_exp = n_exp, r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd_restricted,
    reverse_mean_change = reverse_mean_change
  ))

  es_mean_change_ci_sg <- with(x, es_from_mean_change_ci_single_group(
    mean_change_exp = mean_change_exp,
    mean_change_ci_lo_exp = mean_change_ci_lo_exp, mean_change_ci_up_exp = mean_change_ci_up_exp,
    n_exp = n_exp, r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor, max_asymmetry = max_asymmetry,
    pre_post_to_smd = pre_post_to_smd_restricted,
    reverse_mean_change = reverse_mean_change
  ))

  es_mean_change_pval_sg <- with(x, es_from_mean_change_pval_single_group(
    mean_change_exp = mean_change_exp, mean_change_pval_exp = mean_change_pval_exp,
    n_exp = n_exp, r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd_restricted,
    reverse_mean_change = reverse_mean_change
  ))

  es_paired_t_sg <- with(x, es_from_paired_t_single_group(
    paired_t_exp = paired_t_exp,
    n_exp = n_exp, r_pre_post_exp = r_pre_post_exp,
    pre_post_to_smd = pre_post_to_smd_restricted, smd_to_cor = smd_to_cor,
    reverse_paired_t = reverse_paired_t
  ))

  # SINGLE-GROUP PROPORTIONS --------------
  es_prop_sg <- with(x, es_from_prop_single_group(
    prop = prop,
    n_sample = n_sample,
    prop_to_es = prop_to_es,
    reverse_prop = reverse_prop
  ))

  es_prop_counts_sg <- with(x, es_from_prop_single_group_counts(
    n_cases = n_cases,
    n_sample = n_sample,
    prop_to_es = prop_to_es,
    reverse_prop = reverse_prop
  ))

  # CRONBACH'S ALPHA -----------------------------------------
  es_alpha_sg <- with(x, es_from_cronbach_alpha(
    cronbach_alpha = cronbach_alpha, n_sample = n_sample,
    n_items = n_items, alpha_to_es = alpha_to_es
  ))

  # ICC -------------------------------------------------------
  es_icc_sg <- with(x, es_from_icc(
    icc = icc, n_sample = n_sample,
    n_measurements = n_measurements, icc_type = icc_type,
    icc_to_es = icc_to_es
  ))

  es_paired_t <- with(x, es_from_paired_t(
    paired_t_exp = paired_t_exp, paired_t_nexp = paired_t_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

    smd_to_cor = smd_to_cor, reverse_paired_t = reverse_paired_t,
    pre_post_to_smd = pre_post_to_smd_restricted
  ))

  es_paired_t_pval <- with(x, es_from_paired_t_pval(
    paired_t_pval_exp = paired_t_pval_exp,
    paired_t_pval_nexp = paired_t_pval_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

    smd_to_cor = smd_to_cor, reverse_paired_t_pval = reverse_paired_t_pval,
    pre_post_to_smd = pre_post_to_smd_restricted
  ))

  es_paired_f <- with(x, es_from_paired_f(paired_f_exp, paired_f_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

    smd_to_cor = smd_to_cor, reverse_paired_f = reverse_paired_f,
    pre_post_to_smd = pre_post_to_smd_restricted
  ))

  es_paired_f_pval <- with(x, es_from_paired_f_pval(
    paired_f_pval_exp = paired_f_pval_exp,
    paired_f_pval_nexp = paired_f_pval_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,

    smd_to_cor = smd_to_cor, reverse_paired_f_pval = reverse_paired_f_pval,
    pre_post_to_smd = pre_post_to_smd_restricted
  ))
  # ANOVA, Student t-test  ----------------------------------------------
  es_t_student <- with(x, es_from_student_t(
    student_t = student_t, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_student_t = reverse_student_t
  ))
  es_t_student_pval <- with(x, es_from_student_t_pval(
    student_t_pval = student_t_pval, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_student_t_pval = reverse_student_t_pval
  ))
  es_anova_f <- with(x, es_from_anova_f(
    anova_f = anova_f, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_anova_f = reverse_anova_f
  ))
  es_anova_f_pval <- with(x, es_from_anova_pval(
    anova_f_pval = anova_f_pval, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_anova_f_pval = reverse_anova_f_pval
  ))
  es_etasq <- with(x, es_from_etasq(
    etasq = etasq, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_etasq = reverse_etasq
  ))
  es_etasq_adj <- with(x, es_from_etasq_adj(
    etasq_adj = etasq_adj, n_exp = n_exp, n_nexp = n_nexp, n_cov_ancova = n_cov_ancova,
    cov_outcome_r = cov_outcome_r, smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_etasq = reverse_etasq
  ))

  # MD   ----------------------------------------------
  es_md_sd <- with(x, es_from_md_sd(
    md = md, md_sd = md_sd, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_md = reverse_md
  ))
  es_md_se <- with(x, es_from_md_se(
    md = md, md_se = md_se, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_md = reverse_md
  ))
  es_md_ci <- with(x, es_from_md_ci(
    md = md, md_ci_lo = md_ci_lo, md_ci_up = md_ci_up, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_md = reverse_md,
    max_asymmetry = max_asymmetry
  ))
  es_md_pval <- with(x, es_from_md_pval(
    md = md, md_pval = md_pval, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_md = reverse_md
  ))

  # RANGE/QUARTILES  ----------------------------------------------
  es_med_quarts <- with(x, es_from_med_quarts(
    q1_exp = q1_exp, med_exp = med_exp, q3_exp = q3_exp, n_exp = n_exp,
    q1_nexp = q1_nexp, med_nexp = med_nexp, q3_nexp = q3_nexp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_med = reverse_med
  ))
  es_med_min_max <- with(x, es_from_med_min_max(
    min_exp = min_exp, med_exp = med_exp, max_exp = max_exp, n_exp = n_exp,
    min_nexp = min_nexp, med_nexp = med_nexp, max_nexp = max_nexp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_med = reverse_med
  ))
  es_med_min_max_quarts <- with(x, es_from_med_min_max_quarts(
    min_exp = min_exp, q1_exp = q1_exp, med_exp = med_exp, q3_exp = q3_exp, max_exp = max_exp, n_exp = n_exp,
    min_nexp = min_nexp, q1_nexp = q1_nexp, med_nexp = med_nexp, q3_nexp = q3_nexp, max_nexp = max_nexp,
    n_nexp = n_nexp, smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_med = reverse_med
  ))

  # ANCOVA MEANS ------------------------------------------------------------------
  es_ancova_means_sd <- with(x, es_from_ancova_means_sd(
    ancova_mean_exp = ancova_mean_exp, ancova_mean_nexp = ancova_mean_nexp,
    ancova_mean_sd_exp = ancova_mean_sd_exp, ancova_mean_sd_nexp = ancova_mean_sd_nexp,
    cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_ancova_means = reverse_ancova_means
  ))
  es_ancova_means_se <- with(x, es_from_ancova_means_se(
    ancova_mean_exp = ancova_mean_exp, ancova_mean_nexp = ancova_mean_nexp,
    ancova_mean_se_exp = ancova_mean_se_exp, ancova_mean_se_nexp = ancova_mean_se_nexp,
    cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_ancova_means = reverse_ancova_means
  ))
  es_ancova_means_ci <- with(x, es_from_ancova_means_ci(
    ancova_mean_exp = ancova_mean_exp, ancova_mean_nexp = ancova_mean_nexp,
    ancova_mean_ci_lo_exp = ancova_mean_ci_lo_exp, ancova_mean_ci_up_exp = ancova_mean_ci_up_exp,
    ancova_mean_ci_lo_nexp = ancova_mean_ci_lo_nexp, ancova_mean_ci_up_nexp = ancova_mean_ci_up_nexp,
    cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_ancova_means = reverse_ancova_means
  ))
  es_ancova_means_sd_pooled_adj <- with(x, es_from_ancova_means_sd_pooled_adj(
    ancova_mean_exp = ancova_mean_exp, ancova_mean_nexp = ancova_mean_nexp,
    ancova_mean_sd_pooled = ancova_mean_sd_pooled,
    n_exp = n_exp, n_nexp = n_nexp, cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_ancova_means = reverse_ancova_means
  ))
  es_ancova_means_sd_pooled <- with(x, es_from_ancova_means_sd_pooled_crude(
    ancova_mean_exp = ancova_mean_exp, ancova_mean_nexp = ancova_mean_nexp,
    mean_sd_pooled = mean_sd_pooled, cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    n_exp = n_exp, n_nexp = n_nexp, smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_ancova_means = reverse_ancova_means
  ))
  # plot
  es_plot_ancova_means <- with(x, es_from_plot_ancova_means(
    n_exp = n_exp, n_nexp = n_nexp,
    plot_ancova_mean_exp = plot_ancova_mean_exp, plot_ancova_mean_nexp = plot_ancova_mean_nexp,
    plot_ancova_mean_sd_lo_exp = plot_ancova_mean_sd_lo_exp, plot_ancova_mean_sd_lo_nexp = plot_ancova_mean_sd_lo_nexp,
    plot_ancova_mean_sd_up_exp = plot_ancova_mean_sd_up_exp, plot_ancova_mean_sd_up_nexp = plot_ancova_mean_sd_up_nexp,
    plot_ancova_mean_se_lo_exp = plot_ancova_mean_se_lo_exp, plot_ancova_mean_se_lo_nexp = plot_ancova_mean_se_lo_nexp,
    plot_ancova_mean_se_up_exp = plot_ancova_mean_se_up_exp, plot_ancova_mean_se_up_nexp = plot_ancova_mean_se_up_nexp,
    plot_ancova_mean_ci_lo_exp = plot_ancova_mean_ci_lo_exp, plot_ancova_mean_ci_lo_nexp = plot_ancova_mean_ci_lo_nexp,
    plot_ancova_mean_ci_up_exp = plot_ancova_mean_ci_up_exp, plot_ancova_mean_ci_up_nexp = plot_ancova_mean_ci_up_nexp,
    cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_plot_ancova_means = reverse_plot_ancova_means
  ))
  # ANCOVA MD ------------------------------------------------------------------
  es_ancova_md_sd <- with(x, es_from_ancova_md_sd(ancova_md = ancova_md, ancova_md_sd = ancova_md_sd,
         cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
         n_exp = n_exp, n_nexp = n_nexp,
         smd_var = smd_var, smd_to_cor = smd_to_cor,
         reverse_ancova_md = reverse_ancova_md))
  es_ancova_md_se <- with(x, es_from_ancova_md_se(ancova_md = ancova_md, ancova_md_se = ancova_md_se,
         cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
         n_exp = n_exp, n_nexp = n_nexp,
         smd_var = smd_var, smd_to_cor = smd_to_cor,
         reverse_ancova_md = reverse_ancova_md))
  es_ancova_md_ci <- with(x, es_from_ancova_md_ci(ancova_md = ancova_md,
         ancova_md_ci_lo = ancova_md_ci_lo, ancova_md_ci_up = ancova_md_ci_up,
         cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
         n_exp = n_exp, n_nexp = n_nexp,
         smd_var = smd_var, smd_to_cor = smd_to_cor,
         reverse_ancova_md = reverse_ancova_md))
  es_ancova_md_pval <- with(x, es_from_ancova_md_pval(
        ancova_md = ancova_md,
        ancova_md_pval = ancova_md_pval,
        cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
        n_exp = n_exp, n_nexp = n_nexp,
        smd_var = smd_var, smd_to_cor = smd_to_cor,
        reverse_ancova_md = reverse_ancova_md))
  # ANCOVA F, T, P ------------------------------------------------------------------
  es_ancova_t <- with(x, es_from_ancova_t(
    ancova_t = ancova_t, cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    n_exp = n_exp, n_nexp = n_nexp, smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_ancova_t = reverse_ancova_t
  ))
  es_ancova_f <- with(x, es_from_ancova_f(
    ancova_f = ancova_f, cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    n_exp = n_exp, n_nexp = n_nexp, smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_ancova_f = reverse_ancova_f
  ))
  es_ancova_t_pval <- with(x, es_from_ancova_t_pval(
    ancova_t_pval = ancova_t_pval, cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    n_exp = n_exp, n_nexp = n_nexp, smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_ancova_t_pval = reverse_ancova_t_pval
  ))
  es_ancova_f_pval <- with(x, es_from_ancova_f_pval(
    ancova_f_pval = ancova_f_pval, cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    n_exp = n_exp, n_nexp = n_nexp, smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_ancova_f_pval = reverse_ancova_f_pval
  ))

  # CHI-SQ + PHI --------------------------------------------------------
  # yates_chisq column overrides the argument
  yates_chisq_vec <- x$yates_chisq
  yates_chisq_vec[is.na(yates_chisq_vec)] <- as.logical(yates_chisq)[1]

  es_chisq <- with(x, es_from_chisq(
    chisq = chisq, n_sample = n_sample,
    n_cases = n_cases, n_exp = n_exp,
    yates_chisq = yates_chisq_vec,
    reverse_chisq = reverse_chisq
  ))

  es_chisq_pval <- with(x, es_from_chisq_pval(
    chisq_pval = chisq_pval,
    n_cases = n_cases, n_exp = n_exp,
    yates_chisq = yates_chisq_vec,
    n_sample = n_sample, reverse_chisq_pval = reverse_chisq_pval
  ))

  es_phi <- with(x, es_from_phi(
    phi = phi, n_sample = n_sample,
    n_cases = n_cases, n_exp = n_exp,
    reverse_phi = reverse_phi
  ))

  # COR-PB   --------------------------------------------------------
  es_r_point_bis <- with(x, es_from_pt_bis_r(
    pt_bis_r = pt_bis_r, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor,
    reverse_pt_bis_r = reverse_pt_bis_r
  ))

  es_r_point_bis_pval <- with(x, es_from_pt_bis_r_pval(
    pt_bis_r_pval = pt_bis_r_pval,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor,
    reverse_pt_bis_r_pval = reverse_pt_bis_r_pval
  ))

  # 2x2, PROP --------------------------------------------------------
  es_2x2 <- with(x, es_from_2x2(
    n_cases_exp = n_cases_exp, n_cases_nexp = n_cases_nexp,
    n_controls_exp = n_controls_exp, n_controls_nexp = n_controls_nexp,
    # table_2x2_to_cor = table_2x2_to_cor,
    reverse_2x2 = reverse_2x2
  ))
  es_2x2_sum <- with(x, es_from_2x2_sum(
    n_cases_exp = n_cases_exp, n_cases_nexp = n_cases_nexp, n_exp = n_exp, n_nexp = n_nexp,
    # table_2x2_to_cor = table_2x2_to_cor,
    reverse_2x2 = reverse_2x2
  ))
  es_prop <- with(x, es_from_2x2_prop(
    prop_cases_exp = prop_cases_exp, prop_cases_nexp = prop_cases_nexp, n_exp = n_exp, n_nexp = n_nexp,
    # table_2x2_to_cor = table_2x2_to_cor,
    reverse_prop = reverse_prop
  ))

  # RR --------------------------------------------------------
  es_rr_se <- with(x, es_from_rr_se(
    rr = rr, logrr = logrr, logrr_se = logrr_se,
    baseline_risk = baseline_risk, n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls,
    rr_to_or = rr_to_or, reverse_rr = reverse_rr
  ))
  es_rr_ci <- with(x, es_from_rr_ci(
    rr = rr, logrr = logrr,
    baseline_risk = baseline_risk, n_exp = n_exp, n_nexp = n_nexp,
    rr_ci_lo = rr_ci_lo, logrr_ci_lo = logrr_ci_lo, rr_ci_up = rr_ci_up, logrr_ci_up = logrr_ci_up, n_cases = n_cases, n_controls = n_controls,
    rr_to_or = rr_to_or, reverse_rr = reverse_rr,
    max_asymmetry = max_asymmetry
  ))
  es_rr_pval <- with(x, es_from_rr_pval(
    rr = rr, logrr = logrr, rr_pval = rr_pval,
    baseline_risk = baseline_risk, n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls,
    rr_to_or = rr_to_or, reverse_rr = reverse_rr_pval
  ))

  # RD (risk difference) --------------------------------------------------------
  es_rd_se <- with(x, es_from_rd_se(
    rd = rd, rd_se = rd_se,
    baseline_risk = baseline_risk,
    n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls, n_sample = n_sample,
    reverse_rd = reverse_rd
  ))
  es_rd_ci <- with(x, es_from_rd_ci(
    rd = rd,
    rd_ci_lo = rd_ci_lo, rd_ci_up = rd_ci_up,
    baseline_risk = baseline_risk,
    n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls, n_sample = n_sample,
    reverse_rd = reverse_rd
  ))
  es_rd_pval <- with(x, es_from_rd_pval(
    rd = rd, rd_pval = rd_pval,
    baseline_risk = baseline_risk,
    n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls, n_sample = n_sample,
    reverse_rd_pval = reverse_rd_pval
  ))

  # regression
  es_std_beta <- with(x, es_from_beta_std(
    beta_std = beta_std, sd_dv = sd_dv, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_beta_std = reverse_beta_std
  ))
  es_unstd_beta <- with(x, es_from_beta_unstd(
    beta_unstd = beta_unstd, sd_dv = sd_dv, n_exp = n_exp, n_nexp = n_nexp,
    smd_var = smd_var, smd_to_cor = smd_to_cor, reverse_beta_unstd = reverse_beta_unstd
  ))

  # regression t-statistic (partial correlation)
  es_linreg_t <- with(x, es_from_linreg_t(
    linreg_t = linreg_t, n_sample = n_sample, n_covariates = n_covariates,
    sd_iv = sd_iv, unit_increase_iv = unit_increase_iv, unit_type = unit_type,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_linreg_t = reverse_linreg_t
  ))

  # regression coefficient (partial correlation)
  es_linreg_b_se <- with(x, es_from_linreg_b_se(
    linreg_b = linreg_b, linreg_b_se = linreg_b_se,
    n_sample = n_sample, n_covariates = n_covariates,
    sd_iv = sd_iv, unit_increase_iv = unit_increase_iv, unit_type = unit_type,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_linreg_b = reverse_linreg_b
  ))
  es_linreg_b_ci <- with(x, es_from_linreg_b_ci(
    linreg_b = linreg_b, linreg_b_ci_lo = linreg_b_ci_lo, linreg_b_ci_up = linreg_b_ci_up,
    n_sample = n_sample, n_covariates = n_covariates,
    sd_iv = sd_iv, unit_increase_iv = unit_increase_iv, unit_type = unit_type,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_linreg_b = reverse_linreg_b
  ))
  es_linreg_b_pval <- with(x, es_from_linreg_b_pval(
    linreg_b = linreg_b, linreg_b_pval = linreg_b_pval,
    n_sample = n_sample, n_covariates = n_covariates,
    sd_iv = sd_iv, unit_increase_iv = unit_increase_iv, unit_type = unit_type,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    reverse_linreg_b_pval = reverse_linreg_b_pval
  ))

  # user input
  es_user_crude <- es_from_user_crude(
    user_es_original_measure_crude = x$user_es_original_measure_crude,
    user_es_crude = x$user_es_crude,
    user_se_crude = x$user_se_crude,
    user_ci_lo_crude = x$user_ci_lo_crude,
    user_ci_up_crude = x$user_ci_up_crude,
    user_es_target_measure_crude = measure,
    n_exp = x$n_exp, n_nexp = x$n_nexp, n_sample = x$n_sample,
    n_cases = x$n_cases, n_controls = x$n_controls,
    baseline_risk = x$baseline_risk,
    small_margin_prop = x$small_margin_prop,
    or_to_rr = or_to_rr, or_to_cor = or_to_cor,
    smd_to_cor = smd_to_cor, cor_to_smd = cor_to_smd,
    rr_to_or = rr_to_or
  )
  es_user_adj <- es_from_user_adj(
    user_es_original_measure_adj = x$user_es_original_measure_adj,
    user_es_adj = x$user_es_adj,
    user_se_adj = x$user_se_adj,
    user_ci_lo_adj = x$user_ci_lo_adj,
    user_ci_up_adj = x$user_ci_up_adj,
    user_es_target_measure_adj = measure,
    n_exp = x$n_exp, n_nexp = x$n_nexp, n_sample = x$n_sample,
    n_cases = x$n_cases, n_controls = x$n_controls,
    baseline_risk = x$baseline_risk,
    small_margin_prop = x$small_margin_prop,
    or_to_rr = or_to_rr, or_to_cor = or_to_cor,
    smd_to_cor = smd_to_cor, cor_to_smd = cor_to_smd,
    rr_to_or = rr_to_or
  )
  # survival
  es_cases_time <- with(x, es_from_cases_time(
    n_cases_exp = n_cases_exp, n_cases_nexp = n_cases_nexp,
    time_exp = time_exp, time_nexp = time_nexp,
    baseline_rate = baseline_rate,
    reverse_irr = reverse_irr
  ))
  # variability
  var_means_sd <- with(x, es_variab_from_means_sd(
    mean_exp = mean_exp, mean_nexp = mean_nexp,
    mean_sd_exp = mean_sd_exp, mean_sd_nexp = mean_sd_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    reverse_means_variability = reverse_means_variability
  ))
  var_means_se <- with(x, es_variab_from_means_se(
    mean_exp = mean_exp, mean_nexp = mean_nexp,
    mean_se_exp = mean_se_exp, mean_se_nexp = mean_se_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    reverse_means_variability = reverse_means_variability
  ))
  var_means_ci <- with(x, es_variab_from_means_ci(
    mean_exp = mean_exp, mean_nexp = mean_nexp,
    mean_ci_lo_exp = mean_ci_lo_exp, mean_ci_up_exp = mean_ci_up_exp,
    mean_ci_lo_nexp = mean_ci_lo_nexp, mean_ci_up_nexp = mean_ci_up_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    reverse_means_variability = reverse_means_variability
  ))
  # ========================================= STEP 2. SYNTHESIS OF CALCULATIONS =========================================== #

  smd_list_L1 = list(es_cohen_d = es_cohen_d,
                es_hedges_g = es_hedges_g)
  # Reported precision (a logOR SE, or a reported CI) outranks es_odds_ratio, whose SE
  # is *simulated* from the cell marginals (es_from_or) and runs ~1.5x wide. When a study
  # reports an OR together with a CI/SE AND has n_cases/n_controls, the reported interval
  # is the more reliable source and must not be discarded for the marginal reconstruction.
  # This also aligns the OR order with the RR list (es_rr_se, es_rr_ci first).
  or_list_L2 = list(es_odds_ratio_se = es_odds_ratio_se,
                 es_odds_ratio_ci = es_odds_ratio_ci,
                 es_odds_ratio = es_odds_ratio,
                 es_odds_ratio_pval = es_odds_ratio_pval,
                 es_logreg_t = es_logreg_t)
  rr_list_L3 = list(es_rr_se = es_rr_se,
                     es_rr_ci = es_rr_ci,
                     es_rr_pval = es_rr_pval)
  rd_list_L30 = list(es_rd_se = es_rd_se,
                     es_rd_ci = es_rd_ci,
                     es_rd_pval = es_rd_pval)
  cor_list_L4 = list(es_pearson_r = es_pearson_r,
                  es_fisher_z = es_fisher_z,
                  es_spearman_r = es_spearman_r)
  irr_list_L5 = list(es_cases_time = es_cases_time)

  var_list_L6 = list(var_means_sd = var_means_sd,
                     var_means_se = var_means_se,
                     var_means_ci = var_means_ci)

  contingency_list_L7 = list(es_2x2 = es_2x2,
                         es_2x2_sum = es_2x2_sum,
                         es_prop = es_prop)

  phi_chisq_list_L8 = list(es_chisq = es_chisq,
                           es_phi = es_phi,
                           es_chisq_pval = es_chisq_pval)

  means_post_list_L9 = list(es_means_sd_raw = es_means_sd_raw,
                         es_means_sd_pooled = es_means_sd_pooled,
                         es_means_se_raw = es_means_se_raw,
                         es_means_ci_raw = es_means_ci_raw)
  md_post_list_L10 = list(es_md_sd = es_md_sd,
                      es_md_se = es_md_se,
                      es_md_ci = es_md_ci,
                      es_md_pval = es_md_pval)
  anova_list_L11 = list(es_t_student = es_t_student,
                    es_anova_f = es_anova_f,
                    es_r_point_bis = es_r_point_bis,
                    es_t_student_pval = es_t_student_pval,
                    es_anova_f_pval = es_anova_f_pval,
                    es_r_point_bis_pval = es_r_point_bis_pval,
                    es_etasq = es_etasq)
  medians_list_L12 = list(es_med_quarts = es_med_quarts,
                      es_med_min_max = es_med_min_max,
                      es_med_min_max_quarts = es_med_min_max_quarts)
  regression_list_L13 = list(es_std_beta = es_std_beta,
                           es_unstd_beta = es_unstd_beta)
  partial_cor_list_L27 = list(es_linreg_t = es_linreg_t,
                               es_linreg_b_se = es_linreg_b_se,
                               es_linreg_b_ci = es_linreg_b_ci,
                               es_linreg_b_pval = es_linreg_b_pval)

  md_paired_list_L14 = list(es_means_change_sd = es_means_change_sd,
                   es_mean_change_se = es_mean_change_se,
                   es_mean_change_ci = es_mean_change_ci,
                   es_mean_change_pval = es_mean_change_pval)
  means_paired_list_L15 = list(es_means_sd_pre_post = es_means_sd_pre_post,
                           es_means_se_pre_post = es_means_se_pre_post,
                           es_means_ci_pre_post = es_means_ci_pre_post)
  anova_paired_list_L16 = list(es_paired_t = es_paired_t,
                           es_paired_t_pval = es_paired_t_pval,
                           es_paired_f = es_paired_f,
                           es_paired_f_pval = es_paired_f_pval)

  es_cohen_d_adj_L17 = list(es_cohen_d_adj = es_cohen_d_adj)
  ancova_adjusted_list_L18 = list(es_ancova_t = es_ancova_t,
                                  es_ancova_f = es_ancova_f,
                                  es_ancova_t_pval = es_ancova_t_pval,
                                  es_ancova_f_pval = es_ancova_f_pval,
                                  es_etasq_adj = es_etasq_adj)
  means_adjusted_list_L19 = list(es_ancova_means_sd = es_ancova_means_sd,
                             es_ancova_means_sd_pooled = es_ancova_means_sd_pooled,
                             es_ancova_means_se = es_ancova_means_se,
                             es_ancova_means_ci = es_ancova_means_ci,
                             es_ancova_means_sd_pooled_adj = es_ancova_means_sd_pooled_adj)

  md_adjusted_list_L20 = list(es_ancova_md_sd = es_ancova_md_sd,
                              es_ancova_md_se = es_ancova_md_se,
                              es_ancova_md_ci = es_ancova_md_ci,
                              es_ancova_md_pval = es_ancova_md_pval)


  plot_raw_L21 = list(es_plot_means_raw)
  plot_adjusted_L22 = list(es_plot_ancova_means)
  user_raw_L23 = list(es_user_crude)
  es_user_adj_L24 = list(es_user_adj)

  within_group_list_L25 = list(es_means_pp_sg = es_means_pp_sg,
                               es_means_se_pp_sg = es_means_se_pp_sg,
                               es_means_ci_pp_sg = es_means_ci_pp_sg,
                               es_mean_change_sg = es_mean_change_sg,
                               es_mean_change_se_sg = es_mean_change_se_sg,
                               es_mean_change_ci_sg = es_mean_change_ci_sg,
                               es_mean_change_pval_sg = es_mean_change_pval_sg,
                               es_paired_t_sg = es_paired_t_sg)

  prop_list_L26 = list(es_prop_sg = es_prop_sg,
                       es_prop_counts_sg = es_prop_counts_sg)

  alpha_list_L28 = list(es_alpha_sg = es_alpha_sg)

  icc_list_L29 = list(es_icc_sg = es_icc_sg)

  USER_crude = user_raw_L23
  USER_adjusted = es_user_adj_L24

  SMD_post = c(smd_list_L1, means_post_list_L9,
               anova_list_L11, regression_list_L13, md_post_list_L10,
               medians_list_L12, plot_raw_L21)

  SMD_paired = c(means_paired_list_L15, anova_paired_list_L16, md_paired_list_L14)

  SMD_adjusted = c(es_cohen_d_adj_L17, means_adjusted_list_L19, ancova_adjusted_list_L18,
                   md_adjusted_list_L20,
                   plot_adjusted_L22)

  OR = c(or_list_L2)

  CONT = contingency_list_L7

  RR = rr_list_L3

  RD_stand = rd_list_L30

  PHI = phi_chisq_list_L8

  COR = cor_list_L4

  IRR = irr_list_L5

  VAR = var_list_L6


  if (es_selected == "auto") {
    if (measure %in% c("d", "g", "md")) {
      # OR and the raw 2x2 table (CONT) are the sole gateway into the SMD
      # family via the Cox transform d = log(OR) * sqrt(3)/pi. RD_stand is
      # NOT used here: RD -> SMD is not identified without an assumed
      # baseline_risk and its SE is anti-conservative (baseline_risk treated
      # as a fixed constant). RD keeps its legitimate ratio conversions below.

      if (selection_auto == "crude") {
        res = c(USER_crude, SMD_post, SMD_paired, OR,
                CONT, COR, PHI, SMD_adjusted,
                USER_adjusted,
                #not used
                RR, RD_stand, IRR, VAR)

      } else if (selection_auto == "adjusted") {
        res = c(USER_adjusted, SMD_adjusted,
                USER_crude, SMD_post,
                SMD_paired, OR, CONT, COR,
                PHI,
                #not used
                RR, RD_stand, IRR, VAR)
      } else if (selection_auto == "paired") {
        res = c(USER_crude, SMD_paired, SMD_post, SMD_adjusted,
                USER_adjusted, OR, CONT, COR,
                PHI,
                #not used
                RR, RD_stand, IRR, VAR)
      } else {
        stop("selection_auto should be one of 'crude', 'adjusted' or 'paired'")
      }

    } else if (measure %in% c("logor", "or")) {
      res = c(USER_crude, OR, CONT, RR, RD_stand, PHI, COR, SMD_post, USER_adjusted,
              #not used
              SMD_paired, SMD_adjusted, IRR, VAR)

    } else if (measure %in% c("logrr", "rr")) {
      res = c(USER_crude, RR, CONT, OR, RD_stand, PHI, USER_adjusted,
              #not used
              SMD_post, SMD_paired,
              COR, SMD_adjusted,
              IRR, VAR)

    } else if (measure %in% c("logirr", "irr")) {
      res = c(USER_crude, IRR, USER_adjusted,
              #not used
              SMD_post, SMD_paired, OR,
              CONT, COR, PHI, SMD_adjusted,
              RR, RD_stand, VAR)

    } else if (measure %in% c("loghr", "hr")) {
      # HR: user input only
      res = c(USER_crude, USER_adjusted)

    } else if (measure %in% c("nnt")) {
      res = c(USER_crude, RD_stand, CONT, OR, RR, IRR, PHI, USER_adjusted,
              #not used
              SMD_post, SMD_paired, COR, SMD_adjusted,
              VAR)

    } else if (measure %in% c("rd")) {
      res = c(USER_crude, RD_stand, CONT, OR, RR, IRR, PHI, USER_adjusted,
              #not used
              SMD_post, SMD_paired, COR, SMD_adjusted,
              VAR)

    } else if (measure %in% c("r", "z")) {
      # As for the SMD family, OR and the raw 2x2 (CONT) are the sole gateway
      # into the correlation family (r/z are derived downstream of d_se).
      # RD_stand stays #not used for the same identification/SE reason.
      res = c(USER_crude, COR, CONT, OR, PHI, SMD_post,
              SMD_paired, USER_adjusted,
              #not used
              SMD_adjusted,
              RR, RD_stand, IRR, VAR)

    } else if (measure %in% c("logvr", "logcvr")) {
      res = c(USER_crude, VAR, SMD_post, SMD_paired, USER_adjusted,
              #not used
              OR, CONT, RD_stand, COR, PHI, SMD_adjusted,
              RR, IRR)

    } else if (measure %in% c("dw", "gw", "mdw")) {
      res = c(USER_crude, within_group_list_L25)

    } else if (measure == "prop") {
      res = c(USER_crude, prop_list_L26)

    } else if (measure == "alpha") {
      res = c(USER_crude, alpha_list_L28)

    } else if (measure == "icc") {
      res = c(USER_crude, icc_list_L29)

    } else if (measure %in% c("rp", "zp")) {
      res = c(USER_crude, partial_cor_list_L27)

    }
  } else {
    if (measure %in% c("dw", "gw", "mdw")) {
      res = c(USER_crude, within_group_list_L25)
    } else if (measure == "prop") {
      res = c(USER_crude, prop_list_L26)
    } else if (measure == "alpha") {
      res = c(USER_crude, alpha_list_L28)
    } else if (measure == "icc") {
      res = c(USER_crude, icc_list_L29)
    } else if (measure %in% c("rp", "zp")) {
      res = c(USER_crude, partial_cor_list_L27)
    } else if (measure %in% c("loghr", "hr")) {
      res = c(USER_crude, USER_adjusted)
    } else if (measure %in% c("logor", "or")) {
      # Non-auto modes mirror the auto-mode ordering for every measure, so the
      # fallback selection (e.g. hierarchy mode with a default hierarchy) agrees
      # with es_selected = "auto" -- here, preferring the directly reported OR
      # over a reconstructed 2x2.
      res = c(USER_crude, OR, CONT, RR, RD_stand, PHI, COR, SMD_post, USER_adjusted,
              #not used
              SMD_paired, SMD_adjusted, IRR, VAR)
    } else if (measure %in% c("logrr", "rr")) {
      # Prefer the directly reported RR over a reconstructed 2x2/OR (mirrors auto).
      res = c(USER_crude, RR, CONT, OR, RD_stand, PHI, USER_adjusted,
              #not used
              SMD_post, SMD_paired,
              COR, SMD_adjusted,
              IRR, VAR)
    } else if (measure %in% c("logirr", "irr")) {
      res = c(USER_crude, IRR, USER_adjusted,
              #not used
              SMD_post, SMD_paired, OR,
              CONT, COR, PHI, SMD_adjusted,
              RR, RD_stand, VAR)
    } else if (measure %in% c("nnt", "rd")) {
      # Ratio-family targets: RD_stand is a legitimate source (RD -> OR/RR/NNT).
      res = c(USER_crude, RD_stand, CONT, OR, RR, IRR, PHI, USER_adjusted,
              #not used
              SMD_post, SMD_paired, COR, SMD_adjusted,
              VAR)
    } else if (measure %in% c("r", "z")) {
      # OR and the raw 2x2 are the sole gateway into the correlation family;
      # RD_stand stays #not used (RD -> r/z not identified without an assumed
      # baseline_risk; anti-conservative SE). Mirrors auto.
      res = c(USER_crude, COR, CONT, OR, PHI, SMD_post,
              SMD_paired, USER_adjusted,
              #not used
              SMD_adjusted,
              RR, RD_stand, IRR, VAR)
    } else if (measure %in% c("logvr", "logcvr")) {
      res = c(USER_crude, VAR, SMD_post, SMD_paired, USER_adjusted,
              #not used
              OR, CONT, RD_stand, COR, PHI, SMD_adjusted,
              RR, IRR)
    } else {
      # d/g/md targets. selection_auto (crude/adjusted/paired) does not apply
      # outside es_selected = "auto", so the crude ordering is used. OR and the
      # raw 2x2 are the sole gateway into the SMD family; RD_stand is demoted
      # (RD -> SMD is not identified without an assumed baseline_risk and yields
      # anti-conservative SEs).
      res = c(USER_crude, SMD_post, SMD_paired, OR,
              CONT, COR, PHI, SMD_adjusted,
              USER_adjusted,
              #not used
              RR, RD_stand, IRR, VAR)
    }
  }

  df_sq_list = list(
    es_odds_ratio_pval,  es_rr_pval, es_chisq, es_chisq_pval,
    es_md_pval, es_anova_f,  es_anova_f_pval, es_r_point_bis_pval,
    es_etasq, es_mean_change_pval, es_paired_t_pval, es_paired_f,
    es_paired_f_pval, es_ancova_f, es_ancova_f_pval, es_etasq_adj, es_ancova_md_pval
  )
  warning_reverse = FALSE
  for (i in seq_along(df_sq_list)) {
    df <- df_sq_list[[i]]
    ci_lo_cols <- grep("ci_lo", names(df), value = TRUE)

    if (length(ci_lo_cols) > 0) {
      for (col in ci_lo_cols) {
        if (any(!is.na(df[[col]]))) {
          warning_reverse = TRUE
          break
        }
      }
    }
  }

  if (warning_reverse & verbose) {
    warning("When you enter input data that cannot be negative (F-test, eta-squared, p-value, or chi-square values), do not forget to properly set up the direction of the generated effect size using corresponding reverse_* argument!")
  }
  expected_length <- if (measure %in% c("dw", "gw", "mdw")) {
    9
  } else if (measure == "prop") {
    3
  } else if (measure %in% c("alpha", "icc", "loghr", "hr")) {
    2
  } else if (measure %in% c("rp", "zp")) {
    5
  } else {
    76
  }
  if (length(res) != expected_length) {
    stop ("failure to estimate all effect sizes (not expected, you trigerred a bug, please contact cgosling@parisnanterre.fr). Expected ", expected_length, ", got ", length(res))
  }
  class(res) <- "metaConvert"
  attr(res, "raw_data") <- x
  attr(res, "exp") <- exp
  attr(res, "measure") <- measure
  attr(res, "split_adjusted") <- split_adjusted
  attr(res, "es_selected") <- es_selected
  attr(res, "format_adjusted") <- format_adjusted
  attr(res, "hierarchy") <- hierarchy
  attr(res, "main_es") <- main_es
  flag_opts <- .default_flag_options()
  flag_opts[names(flag_options)] <- flag_options
  attr(res, "flag_options") <- flag_opts
  attr(res, "input_validation") <- validation$issues
  # rows where r_pre_post was defaulted (used by the summary flags)
  attr(res, "r_defaulted") <- .r_defaulted
  attr(res, "alpha_to_es") <- alpha_to_es
  attr(res, "icc_to_es") <- icc_to_es
  attr(res, "prop_to_es") <- prop_to_es
  # standardizer used by the pre/post routes (drives the E6/E7 mixing flags)
  attr(res, "pre_post_to_smd") <- pre_post_to_smd
  attr(res, "pool_sd") <- pool_sd
  # per-row endpoint-SMD standardizer ("pooled"/"control"/"control_robust"),
  # used by the summary A6 CI-width check (Glass CIs are on qt(.975, n_nexp-1))
  attr(res, "smd_denom_used") <- .normalize_smd_denom(x[, "smd_denom"])
  return(res)
}
# x_save2 = x; list_df = df_es; ordering = ordering_crude; digits = digits;
# suffix = "_crude"; measure = measure
#
# x = dat; exp = TRUE; es_selected = "hierarchy"
# split_adjusted = TRUE; measure = "d_to_or, smd_to
# hierarchy = "means_sd"#hierarchy#"user_input_crude"
# digits = 3
# x$user_es_measure_crude = "ROR"
# x$user_es_measure_adj = "MOR"
# x$split_adjustedusted = x$se_adjusted = 2
# load_all(); View(convert_df(df.haza));


# x = dat
# measure = "d"
# main_es = TRUE
# split_adjusted = TRUE
# format_adjusted = "wide"
# verbose = TRUE
# es_selected = "hierarchy"
# phi_to_cor = "tetrachoric";
# chisq_to_cor = "tetrachoric";
# rr_to_or = "metaumbrella"
# or_to_rr = "metaumbrella_cases"
# or_to_cor = "bonett"
# cor_to_smd = "cooper"
# table_2x2_to_cor = "lipsey"
# yates_chisq = FALSE
# smd_to_cor = "viechtbauer"
# pre_post_to_smd = "morris"
# unit_type = "raw_scale"
# hierarchy = "rr_se"
# start_time <- Sys.time()
# end_time <- Sys.time()
# end_time - start_time

## ANC MEANS v T (raw data) ---------------------------------

test_that("ancova means and ancova t converge", {
  x = rep(c("A", "B"), each=25)
  c = rnorm(50)
  y = rnorm(50) + c
  r = cor.test(~c+y)$estimate
  res = lm(y~0+x+c)
  dat=data.frame(study=1)
  dat$ancova_mean_exp = summary(res)$coefficients[1,1]
  dat$ancova_mean_se_exp = summary(res)$coefficients[1,2]
  dat$ancova_mean_nexp = summary(res)$coefficients[2,1]
  dat$ancova_mean_se_nexp = summary(res)$coefficients[2,2]
  dat$n_exp=25; dat$n_nexp=25
  dat$cov_outcome_r=r
  dat$n_cov_ancova = 1

  res2 = lm(y~x+c)
  dat$ancova_t = summary(res2)$coefficients[2,3]

  es.mcv_d_t <- summary(convert_df(dat,
                                   verbose = FALSE,
                                   es_selected = "hierarchy",
                                   hierarchy = "ancova_t",
                                   measure = "d",
                                   pre_post_to_smd = "cooper"
  ), digits = 11)
  es.mcv_d <- summary(convert_df(dat,
                                 verbose = FALSE,
                                 es_selected = "hierarchy", hierarchy = "ancova_means_se",
                                 measure = "d",
                                 pre_post_to_smd = "cooper"
  ), digits = 11)
  ## d
  expect_equal(unique(es.mcv_d$info_used_adjusted), "ancova_means_se")
  expect_equal(unique(es.mcv_d_t$info_used_adjusted), "ancova_t")
  expect_equal(abs(es.mcv_d$es_adjusted), abs(es.mcv_d_t$es_adjusted),
               tolerance = 1e-1)
  expect_equal(es.mcv_d$se_adjusted, es.mcv_d_t$se_adjusted, tolerance = 1e-1)
})

## ANC MD v T (raw data) ---------------------------------

test_that("ancova MD+SE and ancova t converge", {
  x = rep(c("A", "B"), each=25)
  c = rnorm(50)
  y = rnorm(50) + c
  r = cor.test(~c+y)$estimate
  res = lm(y~x+c)

  dat=data.frame(study=1)
  dat$ancova_md = summary(res)$coefficients[2,1]
  dat$ancova_md_se = summary(res)$coefficients[2,2]
  dat$n_exp=25; dat$n_nexp=25
  dat$cov_outcome_r=r
  dat$n_cov_ancova = 1
  dat$ancova_t = summary(res)$coefficients[2,3]

  es.mcv_d_t <- summary(convert_df(dat,
                                   verbose = FALSE,
                                   es_selected = "hierarchy", hierarchy = "ancova_t",
                                   measure = "d",
                                   pre_post_to_smd = "cooper"
  ), digits = 11)
  es.mcv_d <- summary(convert_df(dat,
                                 verbose = FALSE,
                                 es_selected = "hierarchy", hierarchy = "ancova_md_se",
                                 measure = "d",
                                 pre_post_to_smd = "cooper"
  ), digits = 11)
  ## d
  expect_equal(unique(es.mcv_d$info_used_adjusted), "ancova_md_se")
  expect_equal(unique(es.mcv_d_t$info_used_adjusted), "ancova_t")
  expect_equal(abs(es.mcv_d$es_adjusted), abs(es.mcv_d_t$es_adjusted),
               tolerance = 1e-10)
  expect_equal(es.mcv_d$se_adjusted, es.mcv_d_t$se_adjusted, tolerance = 1e-10)
})

#' Convert a Pearson's correlation coefficient to several effect size measures
#'
#' @param pearson_r a Pearson's correlation coefficient value
#' @param n_sample the total number of participants
#' @param sd_iv the standard deviation of the independent variable
#' @param unit_increase_iv a value of the independent variable that will be used to estimate the Cohen's d (see details).
#' @param unit_type the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "value"
#' @param n_exp number of the experimental/exposed group
#' @param n_nexp number of the non-experimental/non-exposed group
#' @param cor_to_smd formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.
#' @param reverse_pearson_r a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function estimates the variance of a Pearson's correlation coefficient, and computes the Fisher's r-to-z
#' transformation.
#' Cohen's d (D), Hedges' g (G) are converted from the Pearson's r, and odds ratio (OR)
#' are converted from the Cohen's d.
#'
#' 1. **The formula used to estimate the standard error of the Pearson's correlation coefficient** and 95% CI
#' are (Formula 12.27 in Cooper):
#' \deqn{R\_se = \sqrt{\frac{(1 - pearson\_r^2)^2}{n\_sample - 1}}}
#' \deqn{R\_lo = pearson\_r - qt(.975, n\_sample - 2) * R\_se}
#' \deqn{R\_up = pearson\_r + qt(.975, n\_sample - 2) * R\_se}
#'
#' 2. **The formula used to estimate the Fisher's z** are (Formula 12.28 & 12.29 in Cooper):
#' \deqn{Z = atanh(r)}
#' \deqn{Z\_se = \sqrt{\frac{1}{n\_sample - 3}}}
#' \deqn{Z\_ci\_lo = Z - qnorm(.975) * Z\_se}
#' \deqn{Z\_ci\_up = Z + qnorm(.975) * Z\_se}
#'
#' 3. Several approaches can be used to convert a correlation coefficient to a SMD.
#'
#' **A.** Mathur proposes to use this formula (Formula 1.2 in Mathur, \code{cor_to_smd = "mathur"}):
#' \deqn{increase = ifelse(unit_type == "sd", unit\_increase\_iv * sd\_iv, unit\_increase\_iv)}
#' \deqn{d = \frac{r * increase}{sd_iv * \sqrt{1 - r^2}}}
#' \deqn{d\_se = abs(d) * \sqrt{\frac{1}{r^2 * (n\_sample - 3)} + \frac{1}{2*(n\_sample - 1))}}}
#' The resulting Cohen's d is the average increase in the dependent variable associated with an increase of x units in the independent variable (with x = \code{unit_increase_iv}).
#'
#' **B.** Viechtbauer proposes to use the delta method to derive a Cohen's d from a correlation coefficient (Viechtbauer, 2023, \code{cor_to_smd = "viechtbauer"})
#'
#' **C.** Cooper converts a point-biserial correlation to Cohen's d with the fixed
#' per-2-SD transformation (Formula 12.38 & 12.39 in Cooper, \code{cor_to_smd = "cooper"}).
#' Unlike the \code{"mathur"} option, this branch does NOT use \code{unit_increase_iv},
#' \code{unit_type} or \code{sd_iv}; it always returns the standardized mean difference
#' between two groups lying one predictor standard deviation apart:
#' \deqn{d = \frac{2 * r}{\sqrt{1 - r^2}}}
#' \deqn{d\_se = \sqrt{\frac{4 * R\_se^2}{(1 - r^2)^3}}}
#' where \eqn{R\_se} is the standard error of the correlation. It therefore coincides
#' with the \code{cor_to_smd = "mathur"} result only in the special case
#' \code{unit_type = "sd"} and \code{unit_increase_iv = 2}.
#'
#' To know how the Cohen's d value is converted to other effect measures (G/OR), see details of the \code{\link{es_from_cohen_d}} function.
#'
#' @export es_from_pearson_r
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Mathur, M. B., & VanderWeele, T. J. (2020). A Simple, Interpretable Conversion from Pearson's Correlation to Cohen's for d Continuous Exposures. Epidemiology (Cambridge, Mass.), 31(2), e16-e18. https://doi.org/10.1097/EDE.0000000000001105
#'
#' Viechtbauer W (2010). "Conducting meta-analyses in R with the metafor package." Journal of Statistical Software, 36(3), 1-48. doi:10.18637/jss.v036.i03.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab R + Z\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + OR\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 4. Pearson's r or Fisher's z'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_pearson_r(
#'   pearson_r = .51, sd_iv = 0.24, n_sample = 214,
#'   unit_increase_iv = 1, unit_type = "sd"
#' )
es_from_pearson_r <- function(pearson_r, sd_iv, n_sample,
                              n_exp, n_nexp, cor_to_smd = "viechtbauer",
                              unit_increase_iv, unit_type = "raw_scale", reverse_pearson_r) {
  if (missing(reverse_pearson_r)) {
    reverse_pearson_r <- rep(FALSE, length(pearson_r))
  }
  reverse_pearson_r[is.na(reverse_pearson_r)] <- FALSE
  if (length(reverse_pearson_r) == 1) reverse_pearson_r = c(rep(reverse_pearson_r, length(pearson_r)))
  if (length(reverse_pearson_r) != length(pearson_r)) stop("The length of the 'reverse_pearson_r' argument is incorrectly specified.")

  if (missing(n_exp)) {
    n_exp <- rep(NA, length(pearson_r))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA, length(pearson_r))
  }
  if (missing(n_sample)) {
    n_sample <- rep(NA, length(pearson_r))
  }
  if (missing(sd_iv)) {
    sd_iv <- rep(NA, length(pearson_r))
  }
  if (missing(unit_increase_iv)) {
    unit_increase_iv <- rep(NA, length(pearson_r))
  }
  if (missing(unit_type)) {
    unit_type <- rep(NA, length(pearson_r))
  }

  if (!all(cor_to_smd %in% c("cooper", "mathur", "viechtbauer"))) {
    stop(paste0("'",
                unique(cor_to_smd[!cor_to_smd %in% c("cooper", "mathur", "viechtbauer")]),
                "' not in tolerated values for the 'cor_to_smd' argument.
                Possible inputs are: 'cooper', 'mathur', 'viechtbauer'"))
  }

  # pearson_r = dat$pearson_r; sd_iv = unit_increase_iv = rep(NA, length(pearson_r))#dat$sd_iv;
  # n_sample = dat$n_sample;
  # n_exp = dat$n_exp; n_nexp = dat$n_nexp; cor_to_smd = "viechtbauer";
  # unit_type = "raw_scale"; reverse_pearson_r = rep(FALSE, length(pearson_r))

  n_sample <- ifelse(is.na(n_sample), n_exp + n_nexp, n_sample)
  n_exp <- ifelse(is.na(n_exp), n_sample / 2, n_exp)
  n_nexp <- ifelse(is.na(n_nexp), n_sample / 2, n_nexp)

  r <- ifelse(reverse_pearson_r, -pearson_r, pearson_r)
  r_se <- sqrt((1 - r^2)^2 / (n_sample - 1))

  dat_cor <- data.frame(
    r = r, r_se = r_se,
    sd_iv = sd_iv, n_sample = n_sample,
    unit_increase_iv = unit_increase_iv,
    unit_type = unit_type,
    cor_to_smd = cor_to_smd
  )

  nn_miss <- with(dat_cor, which(
    (cor_to_smd == "mathur" & !is.na(r) & !is.na(sd_iv) &
      !is.na(n_sample) & !is.na(unit_increase_iv) & !is.na(unit_type)) |
      (cor_to_smd == "cooper" & !is.na(r) & !is.na(r_se)) |
      (cor_to_smd == "viechtbauer" & !is.na(r) & !is.na(r_se) & !is.na(n_sample))
  ))

  es <- data.frame(
    d = rep(NA, nrow(dat_cor)),
    d_se = rep(NA, nrow(dat_cor))
  )

  if (length(nn_miss) != 0) {
    res_d <- t(mapply(.cor_to_smd,
      r = dat_cor$r[nn_miss],
      r_se = dat_cor$r_se[nn_miss],
      n_sample = dat_cor$n_sample[nn_miss],
      sd_iv = dat_cor$sd_iv[nn_miss],
      unit_increase_iv = dat_cor$unit_increase_iv[nn_miss],
      unit_type = dat_cor$unit_type[nn_miss],
      cor_to_smd = dat_cor$cor_to_smd[nn_miss]
    ))

    es$d[nn_miss] <- unlist(res_d[, 1])
    es$d_se[nn_miss] <- unlist(res_d[, 2])
  }

  es <- .es_from_d(d = es$d, d_se = es$d_se, n_exp = n_exp, n_nexp = n_nexp, n_sample = n_sample)

  es$r <- r
  es$r_se <- r_se
  es$r_ci_lo <- r - qt(.975, n_sample - 2) * r_se
  es$r_ci_up <- r + qt(.975, n_sample - 2) * r_se

  es$z <- atanh(r)
  es$z_se <- sqrt(1 / (n_sample - 3))
  es$z_ci_lo <- es$z - qnorm(.975) * es$z_se
  es$z_ci_up <- es$z + qnorm(.975) * es$z_se

  es$info_used <- "pearson_r"
  return(es)
}

#' Convert a Fisher's z (r-to-z transformation) to several effect size measures
#'
#' @param fisher_z a Fisher's r-to-z transformed correlation coefficient
#' @param n_sample the total number of participants
#' @param sd_iv the standard deviation of the independent variable
#' @param unit_increase_iv a value of the independent variable that will be used to estimate the Cohen's d (see details).
#' @param unit_type the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "value"
#' @param reverse_fisher_z a logical value indicating whether the direction of the generated effect sizes should be flipped.
#' @param n_exp number of the experimental/exposed group
#' @param n_nexp number of the non-experimental/non-exposed group
#' @param cor_to_smd formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.
#'
#' @details
#' This function converts estimates the standard error of the Fisher's z and performs the z-to-r Fisher's transformation.
#'
#' Last, it converts this r value into a Cohen's d and OR (see details in \code{\link{es_from_pearson_r}()}).
#'
#'
#' @export es_from_fisher_z
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Mathur, M. B., & VanderWeele, T. J. (2020). A Simple, Interpretable Conversion from Pearson's Correlation to Cohen's for d Continuous Exposures. Epidemiology (Cambridge, Mass.), 31(2), e16-e18. https://doi.org/10.1097/EDE.0000000000001105
#'
#' Viechtbauer W (2010). "Conducting meta-analyses in R with the metafor package." Journal of Statistical Software, 36(3), 1-48. doi:10.18637/jss.v036.i03.
#'
#' @md
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab R + Z\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + OR\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 4. Pearson's r or Fisher's z'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @examples
#' es_from_fisher_z(
#'   fisher_z = .21, n_sample = 44,
#' )
es_from_fisher_z <- function(fisher_z, n_sample, unit_type = "raw_scale",
                             n_exp, n_nexp, cor_to_smd = "viechtbauer",
                             sd_iv, unit_increase_iv, reverse_fisher_z) {
  if (missing(reverse_fisher_z)) {
    reverse_fisher_z <- rep(FALSE, length(fisher_z))
  }
  reverse_fisher_z[is.na(reverse_fisher_z)] <- FALSE
  if (missing(n_exp)) {
    n_exp <- rep(NA, length(fisher_z))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA, length(fisher_z))
  }
  if (missing(n_sample)) {
    n_sample <- rep(NA, length(fisher_z))
  }
  if (missing(sd_iv)) {
    sd_iv <- rep(NA, length(fisher_z))
  }
  if (missing(unit_increase_iv)) {
    unit_increase_iv <- rep(NA, length(fisher_z))
  }
  if (missing(unit_type)) {
    unit_type <- rep(NA, length(fisher_z))
  }
  r <- tanh(fisher_z)

  es <- es_from_pearson_r(
    pearson_r = r, sd_iv = sd_iv, n_sample = n_sample,
    n_exp = n_exp, n_nexp = n_nexp, cor_to_smd = cor_to_smd,
    unit_increase_iv = unit_increase_iv, unit_type = unit_type,
    reverse_pearson_r = reverse_fisher_z
  )

  es$info_used <- "fisher_z"
  return(es)
}

# known es types
.known_es_types <- function() {
  c("d", "g", "md", "or", "logor", "rr", "logrr", "r", "z", "rd",
    "irr", "logirr", "nnt", "dw", "gw", "mdw", "hr", "loghr")
}

# output slots of the entered measure (dw mirrors d, etc.)
.user_slot_prefixes <- function(orig_type) {
  switch(orig_type,
    "d"   = c("d", "dw"),
    "dw"  = c("dw", "d"),
    "g"   = c("g", "gw"),
    "gw"  = c("gw", "g"),
    "md"  = c("md", "mdw"),
    "mdw" = c("mdw", "md"),
    "or"  = , "logor"  = "logor",
    "rr"  = , "logrr"  = "logrr",
    "irr" = , "logirr" = "logirr",
    "hr"  = , "loghr"  = "loghr",
    "r"   = "r",
    "z"   = "z",
    "rd"  = "rd",
    "nnt" = "nnt",
    NULL
  )
}

# CI to restore on the entered measure after dispatch (converters rebuild
# t-based CIs; the entered slot must stay wald-z coherent with the recovered SE)
.user_preserved_ci <- function(orig_measure_input, user_ci_lo, user_ci_up,
                                es_val, se_val) {
  orig <- tolower(as.character(orig_measure_input))
  user_gave_ci <- !is.na(user_ci_lo) & !is.na(user_ci_up)
  natural_ratio <- orig %in% c("or", "rr", "irr", "hr")
  # ratio originals without a user ci: converters already build log-scale CIs
  additive <- orig %in% c("d", "g", "md", "dw", "gw", "mdw", "r", "z", "rd")

  log_lo <- suppressWarnings(log(user_ci_lo))
  log_up <- suppressWarnings(log(user_ci_up))
  log_lo[!is.finite(log_lo)] <- NA_real_
  log_up[!is.finite(log_up)] <- NA_real_

  z_lo <- ifelse(additive & !is.na(es_val) & !is.na(se_val),
                 es_val - qnorm(.975) * se_val, NA_real_)
  z_up <- ifelse(additive & !is.na(es_val) & !is.na(se_val),
                 es_val + qnorm(.975) * se_val, NA_real_)

  lo <- ifelse(user_gave_ci, ifelse(natural_ratio, log_lo, user_ci_lo), z_lo)
  up <- ifelse(user_gave_ci, ifelse(natural_ratio, log_up, user_ci_up), z_up)
  list(lo = lo, up = up, user_gave_ci = user_gave_ci)
}

.dispatch_user_conversion <- function(original_type, es_val, se_val,
                                       n_exp, n_nexp, n_sample,
                                       n_cases, n_controls,
                                       baseline_risk, small_margin_prop,
                                       or_to_rr, or_to_cor, smd_to_cor,
                                       cor_to_smd, rr_to_or) {
  len <- length(es_val)
  no_reverse <- rep(FALSE, len)

  switch(original_type,
    "d" = {
      res_d <- .es_from_d(d = es_val, d_se = se_val,
                 n_exp = n_exp, n_nexp = n_nexp, n_sample = n_sample,
                 smd_to_cor = rep(smd_to_cor, len), reverse = no_reverse)
      # dw = d for user input
      res_d$dw       <- res_d$d
      res_d$dw_se    <- res_d$d_se
      res_d$dw_ci_lo <- res_d$d_ci_lo
      res_d$dw_ci_up <- res_d$d_ci_up
      res_d
    },
    "g" = {
      # g -> d, keeping the user se
      df <- n_exp + n_nexp - 2
      J <- .d_j(df)
      # no sample sizes: J = 1
      J <- ifelse(is.na(J), 1, J)
      d <- es_val / J
      d_se <- se_val / J
      res_g <- .es_from_d(d = d, d_se = d_se,
                 n_exp = n_exp, n_nexp = n_nexp, n_sample = n_sample,
                 smd_to_cor = rep(smd_to_cor, len), reverse = no_reverse)
      # keep user g/se when conversion returned NA
      res_g$g <- ifelse(is.na(res_g$g) & !is.na(es_val), es_val, res_g$g)
      res_g$g_se <- ifelse(is.na(res_g$g_se) & !is.na(se_val), se_val, res_g$g_se)
      res_g$g_ci_lo <- ifelse(is.na(res_g$g_ci_lo) & !is.na(es_val) & !is.na(se_val),
                               es_val - qnorm(.975) * se_val, res_g$g_ci_lo)
      res_g$g_ci_up <- ifelse(is.na(res_g$g_ci_up) & !is.na(es_val) & !is.na(se_val),
                               es_val + qnorm(.975) * se_val, res_g$g_ci_up)
      res_g$gw       <- res_g$g
      res_g$gw_se    <- res_g$g_se
      res_g$gw_ci_lo <- res_g$g_ci_lo
      res_g$gw_ci_up <- res_g$g_ci_up
      res_g
    },
    "logor" = {
      es_from_or_se(or = exp(es_val), logor_se = se_val,
                    baseline_risk = baseline_risk,
                    small_margin_prop = small_margin_prop,
                    n_exp = n_exp, n_nexp = n_nexp,
                    n_cases = n_cases, n_controls = n_controls,
                    n_sample = n_sample,
                    or_to_rr = or_to_rr, or_to_cor = or_to_cor,
                    reverse_or = no_reverse)
    },
    "or" = {
      # se on the log scale
      es_from_or_se(or = es_val, logor_se = se_val,
                    baseline_risk = baseline_risk,
                    small_margin_prop = small_margin_prop,
                    n_exp = n_exp, n_nexp = n_nexp,
                    n_cases = n_cases, n_controls = n_controls,
                    n_sample = n_sample,
                    or_to_rr = or_to_rr, or_to_cor = or_to_cor,
                    reverse_or = no_reverse)
    },
    "logrr" = {
      # A risk ratio is a ratio-family measure: convert to OR / NNT / RD only,
      # NOT to an SMD or correlation. Reaching a standardized family would route
      # through the OR and an assumed baseline risk (not identified from the RR
      # alone, anti-conservative SE); this matches the risk-ratio pipeline
      # measure (es_from_rr_se) and the risk-difference user path. To obtain a
      # D/G/R/Z from a risk ratio, enter it as an odds ratio instead.
      es_from_rr_se(rr = exp(es_val), logrr_se = se_val,
                    baseline_risk = baseline_risk,
                    n_exp = n_exp, n_nexp = n_nexp,
                    n_cases = n_cases, n_controls = n_controls,
                    rr_to_or = rr_to_or,
                    reverse_rr = no_reverse)
    },
    "rr" = {
      # See the logrr case: a risk ratio converts to OR / NNT / RD only.
      es_from_rr_se(rr = es_val, logrr_se = se_val,
                    baseline_risk = baseline_risk,
                    n_exp = n_exp, n_nexp = n_nexp,
                    n_cases = n_cases, n_controls = n_controls,
                    rr_to_or = rr_to_or,
                    reverse_rr = no_reverse)
    },
    "r" = {
      r <- es_val
      r_se_user <- se_val
      n_s <- ifelse(!is.na(n_sample), n_sample, n_exp + n_nexp)
      n_e <- ifelse(!is.na(n_exp), n_exp, n_s / 2)
      n_ne <- ifelse(!is.na(n_nexp), n_nexp, n_s / 2)

      nn <- which(!is.na(r) & !is.na(r_se_user) & !is.na(n_s))
      d_val <- d_se_val <- rep(NA_real_, len)
      if (length(nn) > 0) {
        d_res <- t(mapply(.cor_to_smd,
          r = r[nn], r_se = r_se_user[nn], n_sample = n_s[nn],
          sd_iv = NA, unit_increase_iv = NA, unit_type = NA,
          cor_to_smd = cor_to_smd))
        d_val[nn] <- unlist(d_res[, 1])
        d_se_val[nn] <- unlist(d_res[, 2])
      }

      es <- .es_from_d(d = d_val, d_se = d_se_val,
                       n_exp = n_e, n_nexp = n_ne, n_sample = n_s,
                       reverse = no_reverse)

      es$r <- r
      es$r_se <- r_se_user
      es$r_ci_lo <- r - qt(.975, n_s - 2) * r_se_user
      es$r_ci_up <- r + qt(.975, n_s - 2) * r_se_user
      z_val <- atanh(r)
      z_se_val <- r_se_user / (1 - r^2)
      es$z <- z_val
      es$z_se <- z_se_val
      es$z_ci_lo <- z_val - qnorm(.975) * z_se_val
      es$z_ci_up <- z_val + qnorm(.975) * z_se_val
      es
    },
    "z" = {
      z_user <- es_val
      z_se_user <- se_val
      r <- tanh(z_user)
      r_se_user <- z_se_user * (1 - r^2)  # inverse delta method
      n_s <- ifelse(!is.na(n_sample), n_sample, n_exp + n_nexp)
      n_e <- ifelse(!is.na(n_exp), n_exp, n_s / 2)
      n_ne <- ifelse(!is.na(n_nexp), n_nexp, n_s / 2)

      nn <- which(!is.na(r) & !is.na(r_se_user) & !is.na(n_s))
      d_val <- d_se_val <- rep(NA_real_, len)
      if (length(nn) > 0) {
        d_res <- t(mapply(.cor_to_smd,
          r = r[nn], r_se = r_se_user[nn], n_sample = n_s[nn],
          sd_iv = NA, unit_increase_iv = NA, unit_type = NA,
          cor_to_smd = cor_to_smd))
        d_val[nn] <- unlist(d_res[, 1])
        d_se_val[nn] <- unlist(d_res[, 2])
      }

      es <- .es_from_d(d = d_val, d_se = d_se_val,
                       n_exp = n_e, n_nexp = n_ne, n_sample = n_s,
                       reverse = no_reverse)

      es$r <- r
      es$r_se <- r_se_user
      es$r_ci_lo <- r - qt(.975, n_s - 2) * r_se_user
      es$r_ci_up <- r + qt(.975, n_s - 2) * r_se_user
      es$z <- z_user
      es$z_se <- z_se_user
      es$z_ci_lo <- z_user - qnorm(.975) * z_se_user
      es$z_ci_up <- z_user + qnorm(.975) * z_se_user
      es
    },
    "rd" = {
      es_from_rd_se(rd = es_val, rd_se = se_val,
                    baseline_risk = baseline_risk,
                    n_exp = n_exp, n_nexp = n_nexp,
                    n_cases = n_cases, n_controls = n_controls,
                    n_sample = n_sample,
                    reverse_rd = no_reverse)
    },
    "md" = {
      can_convert <- !is.na(n_exp) & !is.na(n_nexp) & n_exp > 0 & n_nexp > 0
      if (any(can_convert)) {
        res_md <- es_from_md_se(md = es_val, md_se = se_val,
                      n_exp = n_exp, n_nexp = n_nexp,
                      smd_to_cor = smd_to_cor, reverse_md = no_reverse)
        no_convert <- which(!can_convert)
        if (length(no_convert) > 0) {
          res_md$md[no_convert] <- es_val[no_convert]
          res_md$md_se[no_convert] <- se_val[no_convert]
          res_md$md_ci_lo[no_convert] <- es_val[no_convert] - qnorm(.975) * se_val[no_convert]
          res_md$md_ci_up[no_convert] <- es_val[no_convert] + qnorm(.975) * se_val[no_convert]
        }
      } else {
        res_md <- data.frame(
          md = es_val,
          md_se = se_val,
          md_ci_lo = es_val - qnorm(.975) * se_val,
          md_ci_up = es_val + qnorm(.975) * se_val
        )
      }
      res_md$mdw       <- res_md$md
      res_md$mdw_se    <- res_md$md_se
      res_md$mdw_ci_lo <- res_md$md_ci_lo
      res_md$mdw_ci_up <- res_md$md_ci_up
      res_md
    },
    "logirr" = {
      data.frame(
        logirr = es_val,
        logirr_se = se_val,
        logirr_ci_lo = es_val - qnorm(.975) * se_val,
        logirr_ci_up = es_val + qnorm(.975) * se_val
      )
    },
    "loghr" = {
      data.frame(
        loghr = es_val,
        loghr_se = se_val,
        loghr_ci_lo = es_val - qnorm(.975) * se_val,
        loghr_ci_up = es_val + qnorm(.975) * se_val
      )
    },
    "nnt" = {
      # nnt -> rd
      rd <- ifelse(es_val == 0, NA, 1 / es_val)
      rd_se <- ifelse(es_val == 0 | se_val == 0, NA, se_val / es_val^2)
      res <- es_from_rd_se(rd = rd, rd_se = rd_se,
                           baseline_risk = baseline_risk,
                           n_exp = n_exp, n_nexp = n_nexp,
                           n_cases = n_cases, n_controls = n_controls,
                           n_sample = n_sample,
                           reverse_rd = no_reverse)
      # keep the RD-inverted CI (Altman 1998)
      res$nnt <- es_val
      res$nnt_se <- se_val
      res
    },
    "mdw" = {
      data.frame(
        mdw       = es_val,
        mdw_se    = se_val,
        mdw_ci_lo = es_val - qnorm(.975) * se_val,
        mdw_ci_up = es_val + qnorm(.975) * se_val,
        md        = es_val,
        md_se     = se_val,
        md_ci_lo  = es_val - qnorm(.975) * se_val,
        md_ci_up  = es_val + qnorm(.975) * se_val
      )
    },
    "dw" = {
      data.frame(
        dw       = es_val,
        dw_se    = se_val,
        dw_ci_lo = es_val - qnorm(.975) * se_val,
        dw_ci_up = es_val + qnorm(.975) * se_val,
        d        = es_val,
        d_se     = se_val,
        d_ci_lo  = es_val - qnorm(.975) * se_val,
        d_ci_up  = es_val + qnorm(.975) * se_val
      )
    },
    "gw" = {
      data.frame(
        gw       = es_val,
        gw_se    = se_val,
        gw_ci_lo = es_val - qnorm(.975) * se_val,
        gw_ci_up = es_val + qnorm(.975) * se_val,
        g        = es_val,
        g_se     = se_val,
        g_ci_lo  = es_val - qnorm(.975) * se_val,
        g_ci_up  = es_val + qnorm(.975) * se_val
      )
    },
    NULL
  )
}


#' Directly input a crude effect size value + variance, with optional conversion
#'
#' @param user_es_original_measure_crude the type of effect size entered (see details).
#' @param user_es_crude effect size value
#' @param user_se_crude standard error of the effect size (on the log scale for the or/rr/irr/hr measures)
#' @param user_ci_lo_crude lower bound of the 95% CI around the effect size value
#' @param user_ci_up_crude upper bound of the 95% CI around the effect size value
#' @param user_es_target_measure_crude the effect size measure of the output (automatically set to the \code{measure} argument when called by the \code{\link{convert_df}} function)
#' @param n_exp number of participants in the exposed group
#' @param n_nexp number of participants in the non-exposed group
#' @param n_sample total number of participants in the sample
#' @param n_cases number of cases/events across exposed/non-exposed groups
#' @param n_controls number of controls/no-event across exposed/non-exposed groups
#' @param baseline_risk proportion of cases in the non-exposed group
#' @param small_margin_prop smallest margin proportion of cases/events in the underlying 2x2 table
#' @param or_to_rr formula used to convert an odds ratio value into a risk ratio (see \code{\link{es_from_or_se}}).
#' @param or_to_cor formula used to convert an odds ratio value into a correlation coefficient (see \code{\link{es_from_or_se}}).
#' @param smd_to_cor formula used to convert a SMD value into a coefficient correlation (see \code{\link{es_from_cohen_d}}).
#' @param cor_to_smd formula used to convert a correlation coefficient value into a SMD (see \code{\link{es_from_pearson_r}}).
#' @param rr_to_or formula used to convert a risk ratio value into an odds ratio (see \code{\link{es_from_rr_se}}).
#' @param measure deprecated alias for \code{user_es_target_measure_crude}, kept for backward compatibility with metaConvert <= 1.0.3.
#' @param user_es_measure_crude deprecated alias for \code{user_es_original_measure_crude}, kept for backward compatibility with metaConvert <= 1.0.3.
#'
#' @details
#' This function is a generic function allowing to include any crude effect size measure value + variance.
#' Importantly, when the \code{user_es_original_measure_crude} is one of the known measures
#' (d, g, md, mdw, dw, gw, or, logor, rr, logrr, irr, logirr, hr, r, z, rd, nnt),
#' conversions towards the other effect size measures are performed.
#' Otherwise, no conversion is performed (the effect size value + variance you enter is
#' the value + variance exported by this function) and a warning is issued.
#' The sample sizes and baseline risk are used only to perform the conversions.
#'
#' For the or/rr/irr/hr measures, the standard error you enter must be on the log scale.
#' If you indicate the 95% CI bounds instead of the standard error, the log-transformation
#' is applied automatically.
#'
#' @md
#'
#' @export es_from_user_crude
#'
#' @return
#' This function allows to directly input any of the available effect size measures
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab Any of the available measures\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab Depends on the entered measure (see details)\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 23. User's input (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @examples
#' dat = data.frame(user_es_original_measure_crude = "or", user_es_crude = 2.5,
#'                  user_ci_lo_crude = 1.2, user_ci_up_crude = 5.2,
#'                  n_exp = 120, n_nexp = 44)
#' summary(convert_df(dat, measure = "g"))
es_from_user_crude <- function(user_es_original_measure_crude,
                                user_es_crude, user_se_crude,
                                user_ci_lo_crude, user_ci_up_crude,
                                user_es_target_measure_crude = "g",
                                n_exp, n_nexp, n_sample,
                                n_cases, n_controls, baseline_risk,
                                small_margin_prop,
                                or_to_rr = "metaumbrella_cases",
                                or_to_cor = "pearson",
                                smd_to_cor = "viechtbauer",
                                cor_to_smd = "viechtbauer",
                                rr_to_or = "metaumbrella",
                                measure, user_es_measure_crude) {

  # Backward compatibility with the pre-2.0 argument names (metaConvert <= 1.0.3,
  # still used by the published metaumbrella): 'user_es_measure_crude' was the
  # entered (original) measure and 'measure' was the requested (target) measure.
  if (!missing(user_es_measure_crude) && missing(user_es_original_measure_crude))
    user_es_original_measure_crude <- user_es_measure_crude
  # The alias must not silently override an explicitly supplied new-style
  # argument, and its use should be visible to the caller.
  if (!missing(measure) && missing(user_es_target_measure_crude)) {
    warning("The 'measure' argument of es_from_user_crude() is deprecated; use 'user_es_target_measure_crude'.",
            call. = FALSE)
    user_es_target_measure_crude <- measure
  }

  len <- length(user_es_original_measure_crude)

  if (missing(user_es_crude)) user_es_crude <- rep(NA_real_, len)
  if (missing(user_se_crude)) user_se_crude <- rep(NA_real_, len)
  if (missing(user_ci_lo_crude)) user_ci_lo_crude <- rep(NA_real_, len)
  if (missing(user_ci_up_crude)) user_ci_up_crude <- rep(NA_real_, len)
  if (missing(n_exp)) n_exp <- rep(NA_real_, len)
  if (missing(n_nexp)) n_nexp <- rep(NA_real_, len)
  if (missing(n_sample)) n_sample <- rep(NA_real_, len)
  if (missing(n_cases)) n_cases <- rep(NA_real_, len)
  if (missing(n_controls)) n_controls <- rep(NA_real_, len)
  if (missing(baseline_risk)) baseline_risk <- rep(NA_real_, len)
  if (missing(small_margin_prop)) small_margin_prop <- rep(NA_real_, len)

  # es/se from ci bounds
  es_val <- ifelse(is.na(user_es_crude) & !is.na(user_es_original_measure_crude) &
                    !is.na(user_ci_lo_crude) & !is.na(user_ci_up_crude),
    (user_ci_up_crude + user_ci_lo_crude) / 2,
    user_es_crude
  )

  se_val <- ifelse(is.na(user_se_crude) & !is.na(user_es_original_measure_crude) &
                    !is.na(user_ci_lo_crude) & !is.na(user_ci_up_crude),
    (user_ci_up_crude - user_ci_lo_crude) / (2 * qnorm(.975)),
    user_se_crude
  )

  ci_lo <- ifelse(is.na(user_ci_lo_crude) & !is.na(user_es_original_measure_crude) &
                   !is.na(user_es_crude) & !is.na(user_se_crude),
    user_es_crude - user_se_crude * qnorm(.975),
    user_ci_lo_crude
  )

  ci_up <- ifelse(is.na(user_ci_up_crude) & !is.na(user_es_original_measure_crude) &
                   !is.na(user_es_crude) & !is.na(user_se_crude),
    user_es_crude + user_se_crude * qnorm(.975),
    user_ci_up_crude
  )

  target <- unique(user_es_target_measure_crude)
  if (length(target) != 1) target <- target[1]

  orig_type <- tolower(user_es_original_measure_crude)
  known_types <- .known_es_types()

  # or/rr/irr/hr: se from ci on the log scale
  ratio_from_ci <- which(orig_type %in% c("or", "rr", "irr", "hr") &
                          is.na(user_se_crude) &
                          !is.na(user_ci_lo_crude) & !is.na(user_ci_up_crude) &
                          user_ci_lo_crude > 0 & user_ci_up_crude > 0)
  if (length(ratio_from_ci) > 0) {
    log_lo <- log(user_ci_lo_crude[ratio_from_ci])
    log_up <- log(user_ci_up_crude[ratio_from_ci])
    se_val[ratio_from_ci] <- (log_up - log_lo) / (2 * qnorm(.975))
    es_val[ratio_from_ci] <- ifelse(
      is.na(user_es_crude[ratio_from_ci]),
      (log_lo + log_up) / 2,
      log(user_es_crude[ratio_from_ci])
    )
    ci_lo[ratio_from_ci] <- log_lo
    ci_up[ratio_from_ci] <- log_up
    ot <- orig_type[ratio_from_ci]
    orig_type[ratio_from_ci] <- ifelse(ot == "or",  "logor",
                                ifelse(ot == "rr",  "logrr",
                                ifelse(ot == "irr", "logirr",
                                ifelse(ot == "hr",  "loghr", ot))))
  }

  # natural irr/hr -> log scale
  irr_with_se <- which(orig_type == "irr" & !is.na(se_val))
  if (length(irr_with_se) > 0) {
    es_val[irr_with_se] <- log(es_val[irr_with_se])
    orig_type[irr_with_se] <- "logirr"
  }

  hr_with_se <- which(orig_type == "hr" & !is.na(se_val))
  if (length(hr_with_se) > 0) {
    es_val[hr_with_se] <- log(es_val[hr_with_se])
    orig_type[hr_with_se] <- "loghr"
  }

  ratio_se_check <- which(orig_type %in% c("or", "rr") &
                           !is.na(user_se_crude) & !is.na(user_es_crude) &
                           user_es_crude > 0 & user_se_crude > user_es_crude)
  if (length(ratio_se_check) > 0) {
    warning(paste0(
      "For rows with user_es_original_measure_crude = 'or'/'rr', ",
      "user_se_crude must be on the LOG scale (SE of logOR/logRR), ",
      "not the natural scale. Detected SE > ES in ",
      length(ratio_se_check), " row(s), which suggests the SE may be ",
      "on the natural scale. If so, provide CI bounds instead and the ",
      "log-scale SE will be derived automatically."
    ))
  }

  has_known_type <- !is.na(orig_type) & orig_type %in% known_types
  has_unknown_type <- !is.na(orig_type) & !(orig_type %in% known_types)

  if (any(has_unknown_type)) {
    unknown_vals <- unique(orig_type[has_unknown_type])
    warning(paste0(
      "Unknown user_es_original_measure_crude values: ",
      paste(unknown_vals, collapse = ", "),
      ". No conversion applied for these rows. Known types: ",
      paste(known_types, collapse = ", "), "."
    ))
  }

  res <- data.frame(info_used = rep("user_input_crude", len))

  res[[target]] <- rep(NA_real_, len)
  res[[paste0(target, "_se")]] <- rep(NA_real_, len)
  res[[paste0(target, "_ci_lo")]] <- rep(NA_real_, len)
  res[[paste0(target, "_ci_up")]] <- rep(NA_real_, len)

  passthrough_idx <- which(!has_known_type & !is.na(es_val))
  if (length(passthrough_idx) > 0) {
    res[[target]][passthrough_idx] <- es_val[passthrough_idx]
    res[[paste0(target, "_se")]][passthrough_idx] <- se_val[passthrough_idx]
    res[[paste0(target, "_ci_lo")]][passthrough_idx] <- ci_lo[passthrough_idx]
    res[[paste0(target, "_ci_up")]][passthrough_idx] <- ci_up[passthrough_idx]
  }

  for (es_type in unique(orig_type[has_known_type])) {
    idx <- which(has_known_type & orig_type == es_type)
    if (length(idx) == 0) next

    converted <- .dispatch_user_conversion(
      original_type = es_type,
      es_val = es_val[idx], se_val = se_val[idx],
      n_exp = n_exp[idx], n_nexp = n_nexp[idx],
      n_sample = n_sample[idx],
      n_cases = n_cases[idx], n_controls = n_controls[idx],
      baseline_risk = baseline_risk[idx],
      small_margin_prop = small_margin_prop[idx],
      or_to_rr = or_to_rr, or_to_cor = or_to_cor,
      smd_to_cor = smd_to_cor, cor_to_smd = cor_to_smd,
      rr_to_or = rr_to_or
    )

    if (!is.null(converted)) {
      for (col in names(converted)) {
        if (col == "info_used") next
        if (!col %in% names(res)) res[[col]] <- rep(NA_real_, len)
        res[[col]][idx] <- converted[[col]]
      }
    }
  }

  # restore user ci on the entered measure
  pres <- .user_preserved_ci(user_es_original_measure_crude,
                              user_ci_lo_crude, user_ci_up_crude,
                              es_val, se_val)
  for (es_type in unique(orig_type[has_known_type])) {
    prefixes <- .user_slot_prefixes(es_type)
    if (is.null(prefixes)) next
    idx <- which(has_known_type & orig_type == es_type &
                 !is.na(pres$lo) & !is.na(pres$up))
    if (es_type == "nnt") idx <- idx[pres$user_gave_ci[idx]]
    if (length(idx) == 0) next
    for (prefix in prefixes) {
      lo_col <- paste0(prefix, "_ci_lo")
      up_col <- paste0(prefix, "_ci_up")
      if (!lo_col %in% names(res) || !up_col %in% names(res)) next
      res[[lo_col]][idx] <- pres$lo[idx]
      res[[up_col]][idx] <- pres$up[idx]
    }
  }

  attr(res, "measure") <- user_es_original_measure_crude
  return(res)
}


#' Directly input an adjusted effect size value + variance, with optional conversion
#'
#' @param user_es_original_measure_adj the type of effect size entered (see details).
#' @param user_es_adj adjusted effect size value
#' @param user_se_adj adjusted standard error of the effect size (on the log scale for the or/rr/irr/hr measures)
#' @param user_ci_lo_adj adjusted lower bound of the 95% CI around the effect size value
#' @param user_ci_up_adj adjusted upper bound of the 95% CI around the effect size value
#' @param user_es_target_measure_adj the effect size measure of the output (automatically set to the \code{measure} argument when called by the \code{\link{convert_df}} function)
#' @param n_exp number of participants in the exposed group
#' @param n_nexp number of participants in the non-exposed group
#' @param n_sample total number of participants in the sample
#' @param n_cases number of cases/events across exposed/non-exposed groups
#' @param n_controls number of controls/no-event across exposed/non-exposed groups
#' @param baseline_risk proportion of cases in the non-exposed group
#' @param small_margin_prop smallest margin proportion of cases/events in the underlying 2x2 table
#' @param or_to_rr formula used to convert an odds ratio value into a risk ratio (see \code{\link{es_from_or_se}}).
#' @param or_to_cor formula used to convert an odds ratio value into a correlation coefficient (see \code{\link{es_from_or_se}}).
#' @param smd_to_cor formula used to convert a SMD value into a coefficient correlation (see \code{\link{es_from_cohen_d}}).
#' @param cor_to_smd formula used to convert a correlation coefficient value into a SMD (see \code{\link{es_from_pearson_r}}).
#' @param rr_to_or formula used to convert a risk ratio value into an odds ratio (see \code{\link{es_from_rr_se}}).
#' @param measure deprecated alias for \code{user_es_target_measure_adj}, kept for backward compatibility with metaConvert <= 1.0.3.
#' @param user_es_measure_adj deprecated alias for \code{user_es_original_measure_adj}, kept for backward compatibility with metaConvert <= 1.0.3.
#'
#' @details
#' This function is a generic function allowing to include any adjusted effect size measure value + variance.
#' Importantly, when the \code{user_es_original_measure_adj} is one of the known measures
#' (d, g, md, mdw, dw, gw, or, logor, rr, logrr, irr, logirr, hr, r, z, rd, nnt),
#' conversions towards the other effect size measures are performed.
#' Otherwise, no conversion is performed (the effect size value + variance you enter is
#' the value + variance exported by this function) and a warning is issued.
#' The sample sizes and baseline risk are used only to perform the conversions.
#'
#' For the or/rr/irr/hr measures, the standard error you enter must be on the log scale.
#' If you indicate the 95% CI bounds instead of the standard error, the log-transformation
#' is applied automatically.
#'
#' @md
#'
#' @export es_from_user_adj
#'
#' @return
#' This function allows to directly input any of the available effect size measures
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab Any of the available measures\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab Depends on the entered measure (see details)\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 24. User's input (adjusted)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @examples
#' dat = data.frame(user_es_original_measure_adj = "or", user_es_adj = 2.5,
#'                  user_ci_lo_adj = 1.2, user_ci_up_adj = 5.2,
#'                  n_exp = 120, n_nexp = 44)
#' summary(convert_df(dat, measure = "g"))
es_from_user_adj <- function(user_es_original_measure_adj,
                              user_es_adj, user_se_adj,
                              user_ci_lo_adj, user_ci_up_adj,
                              user_es_target_measure_adj = "g",
                              n_exp, n_nexp, n_sample,
                              n_cases, n_controls, baseline_risk,
                              small_margin_prop,
                              or_to_rr = "metaumbrella_cases",
                              or_to_cor = "pearson",
                              smd_to_cor = "viechtbauer",
                              cor_to_smd = "viechtbauer",
                              rr_to_or = "metaumbrella",
                              measure, user_es_measure_adj) {

  # Backward compatibility with the pre-2.0 argument names (metaConvert <= 1.0.3):
  # 'user_es_measure_adj' was the entered (original) measure and 'measure' was the
  # requested (target) measure.
  if (!missing(user_es_measure_adj) && missing(user_es_original_measure_adj))
    user_es_original_measure_adj <- user_es_measure_adj
  # The alias must not silently override an explicitly supplied new-style
  # argument, and its use should be visible to the caller.
  if (!missing(measure) && missing(user_es_target_measure_adj)) {
    warning("The 'measure' argument of es_from_user_adj() is deprecated; use 'user_es_target_measure_adj'.",
            call. = FALSE)
    user_es_target_measure_adj <- measure
  }

  len <- length(user_es_original_measure_adj)

  if (missing(user_es_adj)) user_es_adj <- rep(NA_real_, len)
  if (missing(user_se_adj)) user_se_adj <- rep(NA_real_, len)
  if (missing(user_ci_lo_adj)) user_ci_lo_adj <- rep(NA_real_, len)
  if (missing(user_ci_up_adj)) user_ci_up_adj <- rep(NA_real_, len)
  if (missing(n_exp)) n_exp <- rep(NA_real_, len)
  if (missing(n_nexp)) n_nexp <- rep(NA_real_, len)
  if (missing(n_sample)) n_sample <- rep(NA_real_, len)
  if (missing(n_cases)) n_cases <- rep(NA_real_, len)
  if (missing(n_controls)) n_controls <- rep(NA_real_, len)
  if (missing(baseline_risk)) baseline_risk <- rep(NA_real_, len)
  if (missing(small_margin_prop)) small_margin_prop <- rep(NA_real_, len)

  es_val <- ifelse(is.na(user_es_adj) & !is.na(user_es_original_measure_adj) &
                    !is.na(user_ci_lo_adj) & !is.na(user_ci_up_adj),
    (user_ci_up_adj + user_ci_lo_adj) / 2,
    user_es_adj
  )

  se_val <- ifelse(is.na(user_se_adj) & !is.na(user_es_original_measure_adj) &
                    !is.na(user_ci_lo_adj) & !is.na(user_ci_up_adj),
    (user_ci_up_adj - user_ci_lo_adj) / (2 * qnorm(.975)),
    user_se_adj
  )

  ci_lo <- ifelse(is.na(user_ci_lo_adj) & !is.na(user_es_original_measure_adj) &
                   !is.na(user_es_adj) & !is.na(user_se_adj),
    user_es_adj - user_se_adj * qnorm(.975),
    user_ci_lo_adj
  )

  ci_up <- ifelse(is.na(user_ci_up_adj) & !is.na(user_es_original_measure_adj) &
                   !is.na(user_es_adj) & !is.na(user_se_adj),
    user_es_adj + user_se_adj * qnorm(.975),
    user_ci_up_adj
  )

  target <- unique(user_es_target_measure_adj)
  if (length(target) != 1) target <- target[1]

  orig_type <- tolower(user_es_original_measure_adj)
  known_types <- .known_es_types()

  ratio_from_ci <- which(orig_type %in% c("or", "rr", "irr", "hr") &
                          is.na(user_se_adj) &
                          !is.na(user_ci_lo_adj) & !is.na(user_ci_up_adj) &
                          user_ci_lo_adj > 0 & user_ci_up_adj > 0)
  if (length(ratio_from_ci) > 0) {
    log_lo <- log(user_ci_lo_adj[ratio_from_ci])
    log_up <- log(user_ci_up_adj[ratio_from_ci])
    se_val[ratio_from_ci] <- (log_up - log_lo) / (2 * qnorm(.975))
    es_val[ratio_from_ci] <- ifelse(
      is.na(user_es_adj[ratio_from_ci]),
      (log_lo + log_up) / 2,
      log(user_es_adj[ratio_from_ci])
    )
    ci_lo[ratio_from_ci] <- log_lo
    ci_up[ratio_from_ci] <- log_up
    ot <- orig_type[ratio_from_ci]
    orig_type[ratio_from_ci] <- ifelse(ot == "or",  "logor",
                                ifelse(ot == "rr",  "logrr",
                                ifelse(ot == "irr", "logirr",
                                ifelse(ot == "hr",  "loghr", ot))))
  }

  irr_with_se <- which(orig_type == "irr" & !is.na(se_val))
  if (length(irr_with_se) > 0) {
    es_val[irr_with_se] <- log(es_val[irr_with_se])
    orig_type[irr_with_se] <- "logirr"
  }

  hr_with_se <- which(orig_type == "hr" & !is.na(se_val))
  if (length(hr_with_se) > 0) {
    es_val[hr_with_se] <- log(es_val[hr_with_se])
    orig_type[hr_with_se] <- "loghr"
  }

  ratio_se_check <- which(orig_type %in% c("or", "rr") &
                           !is.na(user_se_adj) & !is.na(user_es_adj) &
                           user_es_adj > 0 & user_se_adj > user_es_adj)
  if (length(ratio_se_check) > 0) {
    warning(paste0(
      "For rows with user_es_original_measure_adj = 'or'/'rr', ",
      "user_se_adj must be on the LOG scale (SE of logOR/logRR), ",
      "not the natural scale. Detected SE > ES in ",
      length(ratio_se_check), " row(s), which suggests the SE may be ",
      "on the natural scale. If so, provide CI bounds instead and the ",
      "log-scale SE will be derived automatically."
    ))
  }

  has_known_type <- !is.na(orig_type) & orig_type %in% known_types
  has_unknown_type <- !is.na(orig_type) & !(orig_type %in% known_types)

  if (any(has_unknown_type)) {
    unknown_vals <- unique(orig_type[has_unknown_type])
    warning(paste0(
      "Unknown user_es_original_measure_adj values: ",
      paste(unknown_vals, collapse = ", "),
      ". No conversion applied for these rows. Known types: ",
      paste(known_types, collapse = ", "), "."
    ))
  }

  res <- data.frame(info_used = rep("user_input_adj", len))

  res[[target]] <- rep(NA_real_, len)
  res[[paste0(target, "_se")]] <- rep(NA_real_, len)
  res[[paste0(target, "_ci_lo")]] <- rep(NA_real_, len)
  res[[paste0(target, "_ci_up")]] <- rep(NA_real_, len)

  passthrough_idx <- which(!has_known_type & !is.na(es_val))
  if (length(passthrough_idx) > 0) {
    res[[target]][passthrough_idx] <- es_val[passthrough_idx]
    res[[paste0(target, "_se")]][passthrough_idx] <- se_val[passthrough_idx]
    res[[paste0(target, "_ci_lo")]][passthrough_idx] <- ci_lo[passthrough_idx]
    res[[paste0(target, "_ci_up")]][passthrough_idx] <- ci_up[passthrough_idx]
  }

  for (es_type in unique(orig_type[has_known_type])) {
    idx <- which(has_known_type & orig_type == es_type)
    if (length(idx) == 0) next

    converted <- .dispatch_user_conversion(
      original_type = es_type,
      es_val = es_val[idx], se_val = se_val[idx],
      n_exp = n_exp[idx], n_nexp = n_nexp[idx],
      n_sample = n_sample[idx],
      n_cases = n_cases[idx], n_controls = n_controls[idx],
      baseline_risk = baseline_risk[idx],
      small_margin_prop = small_margin_prop[idx],
      or_to_rr = or_to_rr, or_to_cor = or_to_cor,
      smd_to_cor = smd_to_cor, cor_to_smd = cor_to_smd,
      rr_to_or = rr_to_or
    )

    if (!is.null(converted)) {
      for (col in names(converted)) {
        if (col == "info_used") next
        if (!col %in% names(res)) res[[col]] <- rep(NA_real_, len)
        res[[col]][idx] <- converted[[col]]
      }
    }
  }

  # restore user ci on the entered measure
  pres <- .user_preserved_ci(user_es_original_measure_adj,
                              user_ci_lo_adj, user_ci_up_adj,
                              es_val, se_val)
  for (es_type in unique(orig_type[has_known_type])) {
    prefixes <- .user_slot_prefixes(es_type)
    if (is.null(prefixes)) next
    idx <- which(has_known_type & orig_type == es_type &
                 !is.na(pres$lo) & !is.na(pres$up))
    if (es_type == "nnt") idx <- idx[pres$user_gave_ci[idx]]
    if (length(idx) == 0) next
    for (prefix in prefixes) {
      lo_col <- paste0(prefix, "_ci_lo")
      up_col <- paste0(prefix, "_ci_up")
      if (!lo_col %in% names(res) || !up_col %in% names(res)) next
      res[[lo_col]][idx] <- pres$lo[idx]
      res[[up_col]][idx] <- pres$up[idx]
    }
  }

  attr(res, "measure") <- user_es_original_measure_adj
  return(res)
}

#' Convert pre-post means of a single group into standard effect size measures
#'
#' @param mean_pre_exp mean of the group at baseline
#' @param mean_exp mean of the group at follow up
#' @param mean_pre_sd_exp standard deviation of the group at baseline
#' @param mean_sd_exp standard deviation of the group at follow up
#' @param n_exp number of participants in the group
#' @param r_pre_post_exp pre-post correlation within the group
#' @param pre_post_to_smd formula used to convert the pre and post means/SD into a SMD (see details).
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param reverse_means_pre_post a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts pre-post means of a single group into within-group Cohen's d (dw), Hedges' g (gw), and mean difference (mdw).
#' These within-group effect sizes quantify the standardized change from baseline to follow-up within one group.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from Cohen's d.
#'
#' Four formulas can be used to convert the pre/post means into a SMD (\code{pre_post_to_smd} argument):
#'
#' 1. \code{"bonett"}: baseline SD standardizer (Bonett, 2008; equivalent to the SMCRH measure in metafor)
#' \deqn{d\_w = \frac{mean\_post\_exp - mean\_pre\_exp}{mean\_pre\_sd\_exp}}
#' \deqn{var(g\_w) = \frac{sd\_change^2}{sd\_pre^2(n-1)} + \frac{g\_w^2}{2(n-1)}}
#' where \deqn{sd\_change = \sqrt{sd\_pre^2 + sd\_post^2 - 2r\,sd\_pre\,sd\_post}}
#'
#' 2. \code{"cooper"} (alias: "morris_drm"): raw score standardizer (Cooper 2019, Morris & DeShon 2002)
#' \deqn{d\_rm = \frac{mean\_post\_exp - mean\_pre\_exp}{sd\_change} * \sqrt{2 * (1 - r\_pre\_post\_exp)}}
#' \deqn{var(d\_rm) = \frac{2 * (1 - r\_pre\_post\_exp)}{n\_exp} + \frac{d\_rm^2}{2 * n\_exp}}
#'
#' 3. \code{"morris_dz"}: change score standardizer (Morris & DeShon, 2002; equivalent to the SMCC measure in metafor)
#' \deqn{d\_z = \frac{mean\_post\_exp - mean\_pre\_exp}{sd\_change}}
#' \deqn{var(g\_z) = \frac{1}{n} + \frac{g\_z^2}{2n}}
#'
#' 4. \code{"morris_dav"}: average SD standardizer (Bonett, 2008; equivalent to the SMCRP measure in metafor)
#' \deqn{d\_av = \frac{mean\_post\_exp - mean\_pre\_exp}{\sqrt{(mean\_pre\_sd\_exp^2 + mean\_post\_sd\_exp^2)/2}}}
#' \deqn{g\_av = d\_av \times J(mi), \quad mi = \frac{2(n\_exp - 1)}{1 + r\_pre\_post\_exp^2}}
#' \deqn{var(g\_av) = J^2 \left[\frac{2(1 - r)}{n} + \frac{d^2(1 + r^2)}{4n}\right]}
#'
#' The within-group Hedges' g is obtained by applying a bias correction factor J to d:
#' \deqn{g\_w = d\_w * J(n\_exp-1)}
#'
#' The within-group mean difference is simply:
#' \deqn{md\_w = mean\_post\_exp - mean\_pre\_exp}
#' \deqn{var(md\_w) = \frac{sd\_change^2}{n\_exp}}
#'
#' To estimate other effect size measures,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied, treating the single group as a matched design.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MDw + Dw + Gw\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.
#'
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386. https://doi.org/10.1177/1094428106291059
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @export es_from_means_sd_pre_post_single_group
#'
#' @md
#'
#' @examples
#' es_from_means_sd_pre_post_single_group(
#'   n_exp = 36,
#'   mean_pre_exp = 98, mean_exp = 102,
#'   mean_pre_sd_exp = 16, mean_sd_exp = 17,
#'   r_pre_post_exp = 0.8
#' )
es_from_means_sd_pre_post_single_group <- function(mean_pre_exp, mean_exp,
                                                    mean_pre_sd_exp, mean_sd_exp,
                                                    n_exp, r_pre_post_exp = 0.8,
                                                    pre_post_to_smd = "bonett",
                                                    smd_to_cor = "viechtbauer",
                                                    reverse_means_pre_post) {
  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("bonett", "morris_drm", "morris_dz", "morris_dav"),
    context = "pre_post_means",
    func_name = "es_from_means_sd_pre_post_single_group"
  )

  if (missing(reverse_means_pre_post)) reverse_means_pre_post <- rep(FALSE, length(mean_pre_exp))
  reverse_means_pre_post[is.na(reverse_means_pre_post)] <- FALSE
  if (length(reverse_means_pre_post) == 1) reverse_means_pre_post <- c(rep(reverse_means_pre_post, length(mean_pre_exp)))
  if (length(reverse_means_pre_post) != length(mean_pre_exp)) stop("The length of the 'reverse_means_pre_post' argument is incorrectly specified.")

  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_pre_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  g <- g_se <- g_var <- g_ci_lo <- g_ci_up <-
    d <- d_se <- d_var <- d_ci_lo <- d_ci_up <-
    mdw <- mdw_se <- mdw_ci_lo <- mdw_ci_up <- rep(NA, length(mean_pre_exp))

  nn_miss <- which(!is.na(mean_pre_exp) & !is.na(mean_pre_sd_exp) &
    !is.na(mean_exp) & !is.na(mean_sd_exp) &
    !is.na(r_pre_post_exp) & !is.na(n_exp))

  dat_smd_pre_post <- data.frame(
    mean_pre = mean_pre_exp, mean_pre_sd = mean_pre_sd_exp,
    mean_post = mean_exp, mean_post_sd = mean_sd_exp,
    n = n_exp,
    r_pre_post = r_pre_post_exp,
    pre_post_to_smd = pre_post_to_smd
  )

  if (length(nn_miss) != 0) {
    smd_pp <- t(mapply(.single_group_pre_post_to_smd,
      mean_pre = dat_smd_pre_post$mean_pre[nn_miss],
      mean_post = dat_smd_pre_post$mean_post[nn_miss],
      mean_pre_sd = dat_smd_pre_post$mean_pre_sd[nn_miss],
      mean_post_sd = dat_smd_pre_post$mean_post_sd[nn_miss],
      n = dat_smd_pre_post$n[nn_miss],
      r_pre_post = dat_smd_pre_post$r_pre_post[nn_miss],
      pre_post_to_smd = dat_smd_pre_post$pre_post_to_smd[nn_miss]
    ))

    d[nn_miss] <- smd_pp[, 1]
    d_var[nn_miss] <- smd_pp[, 2]
    d_se[nn_miss] <- sqrt(smd_pp[, 2])
    d_ci_lo[nn_miss] <- smd_pp[, 3]
    d_ci_up[nn_miss] <- smd_pp[, 4]
    g[nn_miss] <- smd_pp[, 5]
    g_var[nn_miss] <- smd_pp[, 6]
    g_se[nn_miss] <- sqrt(smd_pp[, 6])
    g_ci_lo[nn_miss] <- smd_pp[, 7]
    g_ci_up[nn_miss] <- smd_pp[, 8]

    mdw[nn_miss] <- mean_exp[nn_miss] - mean_pre_exp[nn_miss]
    mdw_var <- (mean_pre_sd_exp[nn_miss]^2 + mean_sd_exp[nn_miss]^2 -
                2 * r_pre_post_exp[nn_miss] * mean_pre_sd_exp[nn_miss] * mean_sd_exp[nn_miss]) / n_exp[nn_miss]
    mdw_se[nn_miss] <- sqrt(mdw_var)
    mdw_ci_lo[nn_miss] <- mdw[nn_miss] - qt(0.975, n_exp[nn_miss] - 1) * mdw_se[nn_miss]
    mdw_ci_up[nn_miss] <- mdw[nn_miss] + qt(0.975, n_exp[nn_miss] - 1) * mdw_se[nn_miss]
  }

  es <- .es_from_d(
    d = d, d_se = d_se, n_exp = n_exp, n_nexp = n_exp,
    smd_to_cor = smd_to_cor, reverse = reverse_means_pre_post
  )

  # replace d/g from .es_from_d by the pre/post values
  es$d <- ifelse(reverse_means_pre_post, -d, d)
  es$d_se <- d_se
  es$d_ci_lo <- ifelse(reverse_means_pre_post, -d_ci_up, d_ci_lo)
  es$d_ci_up <- ifelse(reverse_means_pre_post, -d_ci_lo, d_ci_up)

  es$g <- ifelse(reverse_means_pre_post, -g, g)
  es$g_se <- g_se
  es$g_ci_lo <- ifelse(reverse_means_pre_post, -g_ci_up, g_ci_lo)
  es$g_ci_up <- ifelse(reverse_means_pre_post, -g_ci_lo, g_ci_up)

  es$mdw <- ifelse(reverse_means_pre_post, -mdw, mdw)
  es$mdw_se <- mdw_se
  es$mdw_ci_lo <- ifelse(reverse_means_pre_post, -mdw_ci_up, mdw_ci_lo)
  es$mdw_ci_up <- ifelse(reverse_means_pre_post, -mdw_ci_lo, mdw_ci_up)

  es$info_used <- "means_sd_pre_post_single_group"

  return(es)
}

#' Convert pre-post means and standard errors of a single group into standard effect size measures
#'
#' @param mean_pre_exp mean of the group at baseline
#' @param mean_exp mean of the group at follow up
#' @param mean_pre_se_exp standard error of the group at baseline
#' @param mean_se_exp standard error of the group at follow up
#' @param n_exp number of participants in the group
#' @param r_pre_post_exp pre-post correlation within the group
#' @param pre_post_to_smd formula used to convert the pre and post means/SD into a SMD (see details).
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param reverse_means_pre_post a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the pre/post standard errors of a single group into standard deviations (Section 6.5.2.2 in the Cochrane Handbook).
#' \deqn{mean\_pre\_sd\_exp = mean\_pre\_se\_exp * \sqrt{n\_exp}}
#' \deqn{mean\_post\_sd\_exp = mean\_post\_se\_exp * \sqrt{n\_exp}}
#'
#' Then, calculations of the \code{\link{es_from_means_sd_pre_post_single_group}()} are applied.
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.
#'
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386.
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MDw + Dw + Gw\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_means_se_pre_post_single_group
#'
#' @md
#'
#' @examples
#' es_from_means_se_pre_post_single_group(
#'   n_exp = 36,
#'   mean_pre_exp = 98, mean_exp = 102,
#'   mean_pre_se_exp = 2.67, mean_se_exp = 2.83,
#'   r_pre_post_exp = 0.8
#' )
es_from_means_se_pre_post_single_group <- function(mean_pre_exp, mean_exp,
                                                    mean_pre_se_exp, mean_se_exp,
                                                    n_exp, r_pre_post_exp = 0.8,
                                                    pre_post_to_smd = "bonett",
                                                    smd_to_cor = "viechtbauer",
                                                    reverse_means_pre_post) {
  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("bonett", "morris_drm", "morris_dz", "morris_dav"),
    context = "pre_post_means",
    func_name = "es_from_means_se_pre_post_single_group"
  )

  if (missing(reverse_means_pre_post)) reverse_means_pre_post <- rep(FALSE, length(mean_pre_exp))
  reverse_means_pre_post[is.na(reverse_means_pre_post)] <- FALSE
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_pre_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  sd_pre <- mean_pre_se_exp * sqrt(n_exp)
  sd_post <- mean_se_exp * sqrt(n_exp)

  es <- es_from_means_sd_pre_post_single_group(
    mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
    mean_pre_sd_exp = sd_pre, mean_sd_exp = sd_post,
    n_exp = n_exp,
    r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    reverse_means_pre_post = reverse_means_pre_post
  )

  es$info_used <- "means_se_pre_post_single_group"

  return(es)
}

#' Convert pre-post means and 95% CI of a single group into standard effect size measures
#'
#' @param mean_pre_exp mean of the group at baseline
#' @param mean_exp mean of the group at follow up
#' @param mean_pre_ci_lo_exp lower bound of the 95% CI of the mean at baseline
#' @param mean_pre_ci_up_exp upper bound of the 95% CI of the mean at baseline
#' @param mean_ci_lo_exp lower bound of the 95% CI of the mean at follow up
#' @param mean_ci_up_exp upper bound of the 95% CI of the mean at follow up
#' @param n_exp number of participants in the group
#' @param r_pre_post_exp pre-post correlation within the group
#' @param pre_post_to_smd formula used to convert the pre and post means/SD into a SMD (see details).
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param max_asymmetry percentage indicating the tolerance before detecting asymmetry in the 95% CI bounds.
#' @param reverse_means_pre_post a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the bounds of the 95% CI of the pre/post means of a single group into standard errors (Section 6.3.1 in the Cochrane Handbook).
#' \deqn{mean\_pre\_se\_exp = \frac{mean\_pre\_ci\_up\_exp - mean\_pre\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
#' \deqn{mean\_post\_se\_exp = \frac{mean\_post\_ci\_up\_exp - mean\_post\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
#'
#' Then, calculations of the \code{\link{es_from_means_se_pre_post_single_group}()} are applied.
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022.
#'
#' Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.
#'
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386.
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_means_ci_pre_post_single_group
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MDw + Dw + Gw\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_means_ci_pre_post_single_group(
#'   n_exp = 36,
#'   mean_pre_exp = 98,
#'   mean_pre_ci_lo_exp = 88, mean_pre_ci_up_exp = 108,
#'   mean_exp = 102,
#'   mean_ci_lo_exp = 92, mean_ci_up_exp = 112,
#'   r_pre_post_exp = 0.8
#' )
es_from_means_ci_pre_post_single_group <- function(mean_pre_exp, mean_exp,
                                                    mean_pre_ci_lo_exp, mean_pre_ci_up_exp,
                                                    mean_ci_lo_exp, mean_ci_up_exp,
                                                    n_exp, r_pre_post_exp = 0.8,
                                                    pre_post_to_smd = "bonett",
                                                    smd_to_cor = "viechtbauer",
                                                    max_asymmetry = 10,
                                                    reverse_means_pre_post) {

  if (missing(reverse_means_pre_post)) reverse_means_pre_post <- rep(FALSE, length(mean_pre_exp))
  reverse_means_pre_post[is.na(reverse_means_pre_post)] <- FALSE
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_pre_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  .check_ci_asymmetry <- function(val, lo, up, label) {
    half_up <- abs(up - val)
    half_lo <- abs(val - lo)
    denom <- pmax(half_up, half_lo)
    asym_pct <- ifelse(denom == 0, 0, abs(half_up - half_lo) / denom * 100)
    bad <- which(!is.na(asym_pct) & asym_pct > max_asymmetry)
    if (length(bad) > 0) {
      warning(sprintf("Asymmetric CI detected for '%s' in row(s) %s (asymmetry > %s%%)",
                      label, paste(bad, collapse = ", "), max_asymmetry), call. = FALSE)
    }
  }
  .check_ci_asymmetry(mean_pre_exp, mean_pre_ci_lo_exp, mean_pre_ci_up_exp, "mean_pre_exp")
  .check_ci_asymmetry(mean_exp, mean_ci_lo_exp, mean_ci_up_exp, "mean_exp")

  df <- n_exp - 1

  se_pre <- (mean_pre_ci_up_exp - mean_pre_ci_lo_exp) / (2 * qt(0.975, df))
  se_post <- (mean_ci_up_exp - mean_ci_lo_exp) / (2 * qt(0.975, df))

  es <- es_from_means_se_pre_post_single_group(
    mean_pre_exp = mean_pre_exp, mean_exp = mean_exp,
    mean_pre_se_exp = se_pre, mean_se_exp = se_post,
    n_exp = n_exp,
    r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    reverse_means_pre_post = reverse_means_pre_post
  )

  es$info_used <- "means_ci_pre_post_single_group"

  return(es)
}

#' Convert mean change and standard deviation of a single group into standard effect size measures
#'
#' @param mean_change_exp mean change of participants in the group.
#' @param mean_change_sd_exp standard deviation of the mean change for participants in the group.
#' @param n_exp number of participants in the group.
#' @param r_pre_post_exp pre-post correlation within the group
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the mean change into a SMD (\code{"cooper"} by default; \code{"morris_dz"} also accepted, see details).
#' @param reverse_mean_change a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function computes within-group Cohen's d (dw), Hedges' g (gw), and mean difference (mdw)
#' from the mean change (MC) and standard deviation of a single group.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from Cohen's d.
#'
#' The effect size (d_rm) depends linearly on the pre-post correlation:
#' \deqn{d\_rm = (mean\_change\_exp / sd\_change) \times \sqrt{2 \times (1 - r\_pre\_post\_exp)}}
#' If \code{r_pre_post_exp} is not provided, a value of 0.8 is assumed. A misspecified correlation
#' biases the point estimate, not only its precision; when the correlation is not reported,
#' a sensitivity analysis using several plausible values (e.g., 0.3, 0.5, 0.7) is advisable.
#'
#' This function simply internally calls the \code{\link{es_from_means_sd_pre_post_single_group}} function but setting:
#' \deqn{mean\_pre\_exp = 0}
#' \deqn{mean\_pre\_sd\_exp = 0}
#' \deqn{mean\_exp = mean\_change\_exp}
#' \deqn{mean\_sd\_exp = mean\_change\_sd\_exp}
#'
#' To know more about the calculations, see \code{\link{es_from_means_sd_pre_post_single_group}} function.
#'
#' @references
#' Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.
#'
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386.
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MDw + Dw + Gw\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_mean_change_sd_single_group
#'
#' @md
#'
#' @examples
#' es_from_mean_change_sd_single_group(
#'   n_exp = 36,
#'   mean_change_exp = 8.4, mean_change_sd_exp = 9.13,
#'   r_pre_post_exp = 0.2
#' )
es_from_mean_change_sd_single_group <- function(mean_change_exp, mean_change_sd_exp,
                                                 n_exp, r_pre_post_exp = 0.8,
                                                 smd_to_cor = "viechtbauer",
                                                 pre_post_to_smd = "cooper",
                                                 reverse_mean_change) {
  if (missing(reverse_mean_change)) reverse_mean_change <- rep(FALSE, length(mean_change_exp))
  reverse_mean_change[is.na(reverse_mean_change)] <- FALSE
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_change_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "mean_change",
    func_name = "es_from_mean_change_sd_single_group"
  )

  es <- es_from_means_sd_pre_post_single_group(
    mean_pre_exp = rep(0, length(mean_change_exp)),
    mean_exp = mean_change_exp,
    mean_pre_sd_exp = rep(0, length(mean_change_exp)),
    mean_sd_exp = mean_change_sd_exp,
    n_exp = n_exp,
    r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    reverse_means_pre_post = reverse_mean_change
  )

  es$info_used <- "mean_change_sd_single_group"

  return(es)
}

#' Convert mean change and standard error of a single group into standard effect size measures
#'
#' @param mean_change_exp mean change of participants in the group.
#' @param mean_change_se_exp standard error of the mean change for participants in the group.
#' @param n_exp number of participants in the group.
#' @param r_pre_post_exp pre-post correlation within the group
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the mean change into a SMD (\code{"cooper"} by default; \code{"morris_dz"} also accepted, see details).
#' @param reverse_mean_change a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the mean change and standard error of a single group
#' into within-group Cohen's d, Hedges' g, and mean difference. The effect sizes are then converted to other measures.
#'
#' The effect size (d_rm) depends linearly on the pre-post correlation (0.8 assumed when
#' \code{r_pre_post_exp} is missing); see \code{\link{es_from_mean_change_sd_single_group}}.
#'
#' This function simply internally calls the \code{\link{es_from_means_se_pre_post_single_group}} function but setting:
#' \deqn{mean\_pre\_exp = 0}
#' \deqn{mean\_pre\_se\_exp = 0}
#' \deqn{mean\_exp = mean\_change\_exp}
#' \deqn{mean\_se\_exp = mean\_change\_se\_exp}
#'
#' To know more about the calculations, see \code{\link{es_from_means_se_pre_post_single_group}} function.
#'
#' @references
#' Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.
#'
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386.
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MDw + Dw + Gw\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_mean_change_se_single_group
#'
#' @md
#'
#' @examples
#' es_from_mean_change_se_single_group(
#'   n_exp = 36,
#'   mean_change_exp = 8.4, mean_change_se_exp = 1.52,
#'   r_pre_post_exp = 0.2
#' )
es_from_mean_change_se_single_group <- function(mean_change_exp, mean_change_se_exp,
                                                 n_exp, r_pre_post_exp = 0.8,
                                                 smd_to_cor = "viechtbauer",
                                                 pre_post_to_smd = "cooper",
                                                 reverse_mean_change) {
  if (missing(reverse_mean_change)) reverse_mean_change <- rep(FALSE, length(mean_change_exp))
  reverse_mean_change[is.na(reverse_mean_change)] <- FALSE
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_change_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "mean_change",
    func_name = "es_from_mean_change_se_single_group"
  )

  es <- es_from_means_se_pre_post_single_group(
    mean_pre_exp = rep(0, length(mean_change_exp)),
    mean_exp = mean_change_exp,
    mean_pre_se_exp = rep(0, length(mean_change_exp)),
    mean_se_exp = mean_change_se_exp,
    n_exp = n_exp,
    r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    reverse_means_pre_post = reverse_mean_change
  )

  es$info_used <- "mean_change_se_single_group"

  return(es)
}

#' Convert mean change and 95% CI of a single group into standard effect size measures
#'
#' @param mean_change_exp mean change of participants in the group.
#' @param mean_change_ci_lo_exp lower bound of the 95% CI around the mean change.
#' @param mean_change_ci_up_exp upper bound of the 95% CI around the mean change.
#' @param n_exp number of participants in the group.
#' @param r_pre_post_exp pre-post correlation within the group
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the mean change into a SMD (\code{"cooper"} by default; \code{"morris_dz"} also accepted, see details).
#' @param max_asymmetry percentage indicating the tolerance before detecting asymmetry in the 95% CI bounds.
#' @param reverse_mean_change a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the mean change and 95% CI of a single group
#' into within-group Cohen's d, Hedges' g, and mean difference. The effect sizes are then converted to other measures.
#'
#' The effect size (d_rm) depends linearly on the pre-post correlation (0.8 assumed when
#' \code{r_pre_post_exp} is missing); see \code{\link{es_from_mean_change_sd_single_group}}.
#'
#' This function simply internally calls the \code{\link{es_from_means_ci_pre_post_single_group}} function but setting:
#' \deqn{mean\_pre\_exp = 0}
#' \deqn{mean\_pre\_ci\_lo\_exp = 0}
#' \deqn{mean\_pre\_ci\_up\_exp = 0}
#' \deqn{mean\_exp = mean\_change\_exp}
#' \deqn{mean\_ci\_lo\_exp = mean\_change\_ci\_lo\_exp}
#' \deqn{mean\_ci\_up\_exp = mean\_change\_ci\_up\_exp}
#'
#' To know more about the calculations, see \code{\link{es_from_means_sd_pre_post_single_group}} function.
#'
#' @references
#' Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.
#'
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386.
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_mean_change_ci_single_group
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MDw + Dw + Gw\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_mean_change_ci_single_group(
#'   n_exp = 36,
#'   mean_change_exp = 8.4,
#'   mean_change_ci_lo_exp = 6.4, mean_change_ci_up_exp = 10.4,
#'   r_pre_post_exp = 0.2
#' )
es_from_mean_change_ci_single_group <- function(mean_change_exp,
                                                 mean_change_ci_lo_exp, mean_change_ci_up_exp,
                                                 n_exp, r_pre_post_exp = 0.8,
                                                 max_asymmetry = 10,
                                                 smd_to_cor = "viechtbauer",
                                                 pre_post_to_smd = "cooper",
                                                 reverse_mean_change) {
  if (missing(reverse_mean_change)) reverse_mean_change <- rep(FALSE, length(mean_change_exp))
  reverse_mean_change[is.na(reverse_mean_change)] <- FALSE
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_change_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "mean_change",
    func_name = "es_from_mean_change_ci_single_group"
  )

  es <- es_from_means_ci_pre_post_single_group(
    mean_pre_exp = rep(0, length(mean_change_exp)),
    mean_pre_ci_lo_exp = rep(0, length(mean_change_exp)),
    mean_pre_ci_up_exp = rep(0, length(mean_change_exp)),
    mean_exp = mean_change_exp,
    mean_ci_lo_exp = mean_change_ci_lo_exp,
    mean_ci_up_exp = mean_change_ci_up_exp,
    n_exp = n_exp,
    r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    max_asymmetry = max_asymmetry,
    reverse_means_pre_post = reverse_mean_change
  )

  es$info_used <- "mean_change_ci_single_group"

  return(es)
}

#' Convert mean change and p-value of a single group into standard effect size measures
#'
#' @param mean_change_exp mean change of participants in the group.
#' @param mean_change_pval_exp p-value of the mean change for participants in the group.
#' @param n_exp number of participants in the group.
#' @param r_pre_post_exp pre-post correlation within the group
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the mean change into a SMD (\code{"cooper"} by default; \code{"morris_dz"} also accepted, see details).
#' @param reverse_mean_change a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the mean change and associated p-value of a single group
#' into within-group Cohen's d, Hedges' g, and mean difference. The effect sizes are then converted to other measures.
#'
#' The effect size (d_rm) depends linearly on the pre-post correlation (0.8 assumed when
#' \code{r_pre_post_exp} is missing); see \code{\link{es_from_mean_change_sd_single_group}}.
#'
#' To start, this function estimates the mean change standard error from the p-value:
#' \deqn{t <- qt(p = mean\_change\_pval\_exp / 2, df = n\_exp - 1, lower.tail = FALSE)}
#' \deqn{mean\_change\_se\_exp <- |\frac{mean\_change\_exp}{t}|}
#'
#' Then, this function simply internally calls the \code{\link{es_from_means_se_pre_post_single_group}} function but setting:
#' \deqn{mean\_pre\_exp = 0}
#' \deqn{mean\_pre\_se\_exp = 0}
#' \deqn{mean\_exp = mean\_change\_exp}
#' \deqn{mean\_se\_exp = mean\_change\_se\_exp}
#'
#' To know more about other calculations, see \code{\link{es_from_means_sd_pre_post_single_group}} function.
#'
#' @references
#' Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.
#'
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386.
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_mean_change_pval_single_group
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MDw + Dw + Gw\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_mean_change_pval_single_group(
#'   n_exp = 36,
#'   mean_change_exp = 8.4, mean_change_pval_exp = 0.13,
#'   r_pre_post_exp = 0.8
#' )
es_from_mean_change_pval_single_group <- function(mean_change_exp, mean_change_pval_exp,
                                                   n_exp, r_pre_post_exp = 0.8,
                                                   smd_to_cor = "viechtbauer",
                                                   pre_post_to_smd = "cooper",
                                                   reverse_mean_change) {
  if (missing(reverse_mean_change)) reverse_mean_change <- rep(FALSE, length(mean_change_exp))
  reverse_mean_change[is.na(reverse_mean_change)] <- FALSE
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_change_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "mean_change",
    func_name = "es_from_mean_change_pval_single_group"
  )

  t_stat <- qt(p = mean_change_pval_exp / 2, df = n_exp - 1, lower.tail = FALSE)
  mean_change_se <- abs(mean_change_exp / t_stat)

  es <- es_from_means_se_pre_post_single_group(
    mean_pre_exp = rep(0, length(mean_change_exp)),
    mean_exp = mean_change_exp,
    mean_pre_se_exp = rep(0, length(mean_change_exp)),
    mean_se_exp = mean_change_se,
    n_exp = n_exp,
    r_pre_post_exp = r_pre_post_exp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    reverse_means_pre_post = reverse_mean_change
  )

  es$info_used <- "mean_change_pval_single_group"

  return(es)
}

#' Convert paired t-test statistic from a single group into standard effect size measures
#'
#' @param paired_t_exp paired t-test statistic (pre-post comparison) for the group.
#' @param n_exp number of participants in the group.
#' @param r_pre_post_exp pre-post correlation within the group
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the paired t statistic into a SMD (\code{"cooper"} by default; \code{"morris_dz"} also accepted, see details).
#' @param reverse_paired_t a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts a paired t-test statistic from a single group
#' into within-group Cohen's d, Hedges' g, and mean difference using the raw score standardizer (morris_drm/cooper).
#'
#' The paired t-test statistic is related to the mean difference and standard error of difference:
#' \deqn{t\_paired = \frac{mean\_diff}{SE\_diff}}
#'
#' where:
#' \deqn{SE\_diff = \frac{SD\_diff}{\sqrt{n}}}
#' \deqn{SD\_diff = \sqrt{SD\_pre^2 + SD\_post^2 - 2 \times r \times SD\_pre \times SD\_post}}
#'
#' This function calculates the raw score standardized mean difference (d_rm, Morris & DeShon 2002):
#' \deqn{d\_rm = t\_paired \times \sqrt{\frac{2 \times (1 - r)}{n}}}
#'
#' The variance is calculated as:
#' \deqn{var(d\_rm) = \frac{2 \times (1 - r)}{n} + \frac{d\_rm^2}{2 \times n}}
#'
#' The result depends on the pre-post correlation (0.8 assumed when \code{r_pre_post_exp}
#' is missing); see \code{\link{es_from_mean_change_sd_single_group}}.
#'
#' The within-group Hedges' g is obtained by applying a bias correction factor J to d.
#'
#' To estimate other effect size measures, calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @references
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125.
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MDw + Dw + Gw\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 15. Paired: pre-post paired t-test'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_paired_t_single_group
#'
#' @md
#'
#' @examples
#' es_from_paired_t_single_group(
#'   paired_t_exp = 2.5,
#'   n_exp = 20,
#'   r_pre_post_exp = 0.7
#' )
es_from_paired_t_single_group <- function(paired_t_exp, n_exp, r_pre_post_exp = 0.8,
                                          smd_to_cor = "viechtbauer",
                                          pre_post_to_smd = "cooper",
                                          reverse_paired_t) {
  if (missing(reverse_paired_t)) reverse_paired_t <- rep(FALSE, length(paired_t_exp))
  reverse_paired_t[is.na(reverse_paired_t)] <- FALSE
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(paired_t_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "paired_t",
    func_name = "es_from_paired_t_single_group"
  )

  r_pre_post_exp <- .guard_r_pre_post(r_pre_post_exp)

  J <- .d_j(n_exp - 1)

  # per-row method selection (a mixed vector previously fell wholesale into drm).
  # Recycled to the input length so a scalar method still yields one value per row.
  dz <- rep(pre_post_to_smd == "morris_dz", length.out = length(paired_t_exp))

  # d_z. metafor SMCC convention (variance built from the corrected g), matching
  # .single_group_pre_post_to_smd so the paired-t and mean-change routes return
  # identical SEs on equivalent inputs
  d_dz <- paired_t_exp / sqrt(n_exp)
  d_var_dz <- (1 / n_exp + (J * d_dz)^2 / (2 * n_exp)) / J^2

  # d_rm
  d_drm <- paired_t_exp * sqrt(2 * (1 - r_pre_post_exp) / n_exp)
  d_var_drm <- 2 * (1 - r_pre_post_exp) / n_exp + d_drm^2 / (2 * n_exp)

  d <- ifelse(dz, d_dz, d_drm)
  d_var <- ifelse(dz, d_var_dz, d_var_drm)

  # A single paired observation (n < 2) has no estimable within-subject variance. The
  # morris_drm branch carries no J(n-1) factor, so it stays finite at n = 1 (and Inf at
  # n = 0), leaking a spurious d/g/logOR/r (and an impossible |r| > 1) downstream. NA
  # such rows so this route matches the mean-change route (which NAs via its n >= 2
  # kernel guard); the qt(.975, n-1) CI is already NaN there.
  bad_n <- !(is.finite(n_exp) & n_exp >= 2)
  d[bad_n] <- NA_real_; d_var[bad_n] <- NA_real_

  d_se <- sqrt(d_var)

  d_ci_lo <- d - qt(0.975, n_exp - 1) * d_se
  d_ci_up <- d + qt(0.975, n_exp - 1) * d_se

  g <- d * J
  g_se <- d_se * J
  g_ci_lo <- d_ci_lo * J
  g_ci_up <- d_ci_up * J

  # md not recoverable from the t statistic alone
  md <- rep(NA_real_, length(paired_t_exp))
  md_se <- rep(NA_real_, length(paired_t_exp))
  md_ci_lo <- rep(NA_real_, length(paired_t_exp))
  md_ci_up <- rep(NA_real_, length(paired_t_exp))

  es <- .es_from_d(
    d = d, d_se = d_se, n_exp = n_exp, n_nexp = n_exp,
    smd_to_cor = smd_to_cor, reverse = reverse_paired_t
  )

  es$d <- ifelse(reverse_paired_t, -d, d)
  es$d_se <- d_se
  es$d_ci_lo <- ifelse(reverse_paired_t, -d_ci_up, d_ci_lo)
  es$d_ci_up <- ifelse(reverse_paired_t, -d_ci_lo, d_ci_up)

  es$g <- ifelse(reverse_paired_t, -g, g)
  es$g_se <- g_se
  es$g_ci_lo <- ifelse(reverse_paired_t, -g_ci_up, g_ci_lo)
  es$g_ci_up <- ifelse(reverse_paired_t, -g_ci_lo, g_ci_up)

  es$md <- ifelse(reverse_paired_t, -md, md)
  es$md_se <- md_se
  es$md_ci_lo <- ifelse(reverse_paired_t, -md_ci_up, md_ci_lo)
  es$md_ci_up <- ifelse(reverse_paired_t, -md_ci_lo, md_ci_up)

  es$info_used <- "paired_t_single_group"

  return(es)
}

#' Convert Cronbach's alpha into an effect size measure
#'
#' @param cronbach_alpha Cronbach's alpha reliability coefficient
#' @param n_sample the total number of participants that completed the scale
#' @param n_items number of items in the scale
#' @param alpha_to_es method used to compute the effect size from Cronbach's alpha.
#'   Must be either \code{"bonett"} or \code{"raw"}.
#'
#' @details
#' This function computes an effect size from a Cronbach's alpha.
#'
#' 1. When \code{alpha_to_es = "bonett"} (default), the Bonett (2002) transformation
#' is applied:
#' \deqn{T(\alpha) = \ln(1 - \alpha)}
#' \deqn{T\_se = \sqrt{\frac{2k}{(k - 1)(n - 2)}}}
#'
#' 2. When \code{alpha_to_es = "raw"}, the raw alpha is used. Its standard error is
#' the delta-method back-transform of the Bonett (2002) transformed variance
#' (equivalently the van Zyl, Neudecker & Nel, 2000, asymptotic variance; this is
#' the form implemented by \code{metafor}'s \code{measure = "ARAW"}):
#' \deqn{\alpha\_se = (1 - \alpha) \sqrt{\frac{2k}{(k - 1)(n - 2)}}}
#' Note that Feldt et al.'s (1987) classical asymptotic variance instead uses an
#' \eqn{(n - 1)} denominator; the \eqn{(n - 2)} form above follows Bonett (2002)
#' for consistency with the Bonett transformation used in method 1.
#'
#' The Bonett transformation stabilizes the variance and is recommended for meta-analysis.
#'
#' @export es_from_cronbach_alpha
#'
#' @references
#' Bonett, D. G. (2002). Sample size requirements for testing and estimating coefficient alpha.
#' Journal of Educational and Behavioral Statistics, 27(4), 335-340.
#'
#' Feldt, L. S., Woodruff, D. J., & Salih, F. A. (1987). Statistical inference for coefficient alpha.
#' Applied Psychological Measurement, 11(1), 93-103.
#'
#' van Zyl, J. M., Neudecker, H., & Nel, D. G. (2000). On the distribution of the maximum likelihood
#' estimator of Cronbach's alpha. Psychometrika, 65(3), 271-280.
#'
#' @md
#'
#' @return
#' This function estimates the standard error of the Cronbach's alpha.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab alpha\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab N/A\cr
#'  \tab \cr
#'  \code{required input data} \tab cronbach_alpha + n_sample + n_items\cr
#'  \tab \cr
#' }
#'
#' @examples
#' es_from_cronbach_alpha(
#'   cronbach_alpha = 0.85, n_sample = 200, n_items = 10
#' )
es_from_cronbach_alpha <- function(cronbach_alpha, n_sample, n_items,
                                   alpha_to_es = "bonett") {

  if (missing(n_sample)) n_sample <- rep(NA, length(cronbach_alpha))
  if (missing(n_items)) n_items <- rep(NA, length(cronbach_alpha))

  if (!alpha_to_es %in% c("bonett", "raw")) {
    stop(paste0("'", alpha_to_es, "' not in tolerated values for the 'alpha_to_es' argument. ",
                "Possible inputs are: 'bonett', 'raw'"))
  }

  # P16: per-element guards -- alpha > 1 is impossible (V11 upper bound),
  # n <= 2 makes the SE denominator (n - 2) non-positive, and a single item
  # (k < 2) leaves alpha undefined. alpha = 1 exactly is a valid boundary on
  # the raw scale (es = 1, se = 0) but has no Bonett transform (log(0)).
  # Direct calls degrade to NA instead of emitting Inf/NaN (the pipeline
  # already NAs the impossible values via Tier-1 validation).
  invalid <- (!is.na(cronbach_alpha) & cronbach_alpha > 1) |
    (!is.na(cronbach_alpha) & cronbach_alpha == 1 & alpha_to_es == "bonett") |
    (!is.na(n_sample) & n_sample <= 2) |
    (!is.na(n_items) & n_items < 2)

  nn_miss <- which(!is.na(cronbach_alpha) & !is.na(n_sample) & !is.na(n_items) &
                     !invalid)

  n <- length(cronbach_alpha)
  alpha_es <- rep(NA_real_, n)
  alpha_es_se <- rep(NA_real_, n)

  if (length(nn_miss) != 0) {
    a <- cronbach_alpha[nn_miss]
    ns <- n_sample[nn_miss]
    k <- n_items[nn_miss]

    if (alpha_to_es == "bonett") {
      alpha_es[nn_miss] <- log(1 - a)
      alpha_es_se[nn_miss] <- sqrt(2 * k / ((k - 1) * (ns - 2)))
    } else {
      alpha_es[nn_miss] <- a
      alpha_es_se[nn_miss] <- (1 - a) * sqrt(2 * k / ((k - 1) * (ns - 2)))
    }
  }

  alpha_ci_lo <- alpha_es - qnorm(0.975) * alpha_es_se
  alpha_ci_up <- alpha_es + qnorm(0.975) * alpha_es_se

  result <- data.frame(
    alpha = alpha_es,
    alpha_se = alpha_es_se,
    alpha_ci_lo = alpha_ci_lo,
    alpha_ci_up = alpha_ci_up,
    n_sample = n_sample,
    n_items = n_items,
    info_used = "cronbach_alpha"
  )

  return(result)
}

#' Convert the number of cases and the person-time of disease-free observation in two independent groups into an incidence rate ratio (IRR)
#'
#' @param n_cases_exp number of cases in the exposed group
#' @param n_cases_nexp number of cases in the non-exposed group
#' @param time_exp person-time of disease-free observation in the exposed group
#' @param time_nexp person-time of disease-free observation in the non-exposed group
#' @param baseline_rate incidence rate of events (per person-time) in the non-exposed group (n_cases_nexp / time_nexp is used when missing)
#' @param reverse_irr a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function estimates the incidence rate ratio from the number of cases and
#' the person-time of disease-free observation in two independent groups.
#'
#' **The formula used to obtain the IRR and its standard error** are (Cochrane Handbook (section 6.7.1):
#' \deqn{logirr = log(\frac{n\_cases\_exp / time\_exp}{n\_cases\_nexp / time\_nexp)}}
#' \deqn{logirr\_se = \sqrt{\frac{1}{n\_cases\_exp} + \frac{1}{n\_cases\_nexp}}}
#'
#' **To estimate a person-time NNT** (Mayne et al., 2006), the following formulas are used:
#' \deqn{ird = baseline\_rate \times (1 - irr)}
#' \deqn{nnt = \frac{1}{ird}}
#' where \code{ird} is the incidence rate difference and \code{baseline_rate} is the
#' incidence rate in the control group.
#'
#' **To estimate the standard error of the IRD**, two formulas are used.
#' When \code{baseline_rate} is missing:
#' \deqn{ird\_se = \sqrt{\frac{n\_cases\_exp}{time\_exp^2} + \frac{n\_cases\_nexp}{time\_nexp^2}}}
#' When \code{baseline_rate} is entered by users, the delta method is used:
#' \deqn{ird\_se = baseline\_rate \times IRR \times logirr\_se}
#'
#' @return
#' This function estimates IRR and, when baseline rate information is available, NNT.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab IRR + NNT\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab N/A\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 5. Incidence Ratio Ratio'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_cases_time
#'
#' @references
#' Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' Mayne, T. J., Whalen, E., & Rost, K. (2006). Annualized was found better than absolute risk reduction in the calculation of number needed to treat in chronic conditions. Journal of clinical epidemiology, 59(3), 217-223.
#'
#' @md
#'
#' @examples
#' es_from_cases_time(
#'   n_cases_exp = 241, n_cases_nexp = 554,
#'   time_exp = 12.764, time_nexp = 19.743
#' )
es_from_cases_time <- function(n_cases_exp, n_cases_nexp, time_exp, time_nexp,
                                baseline_rate, reverse_irr) {
  if (missing(reverse_irr)) reverse_irr <- rep(FALSE, length(n_cases_exp))
  reverse_irr[is.na(reverse_irr)] <- FALSE
  if (length(reverse_irr) == 1) reverse_irr = c(rep(reverse_irr, length(n_cases_exp)))
  if (length(reverse_irr) != length(n_cases_exp)) stop("The length of the 'reverse_irr' argument is incorrectly specified.")
  if (missing(baseline_rate)) baseline_rate <- rep(NA_real_, length(n_cases_exp))

  # IRR (log scale)
  logirr_raw <- log((n_cases_exp / time_exp) / (n_cases_nexp / time_nexp))
  logirr_se <- sqrt(1 / n_cases_exp + 1 / n_cases_nexp)

  es <- data.frame(
    logirr = ifelse(reverse_irr, -logirr_raw, logirr_raw),
    logirr_se = logirr_se
  )

  es$logirr_ci_lo <- es$logirr - es$logirr_se * qnorm(.975)
  es$logirr_ci_up <- es$logirr + es$logirr_se * qnorm(.975)

  # Person-time NNT (Mayne et al. 2006)
  provided_baseline <- !is.na(baseline_rate)
  baseline_rate <- ifelse(is.na(baseline_rate), n_cases_nexp / time_nexp, baseline_rate)

  irr <- exp(logirr_raw)

  ird <- baseline_rate * (1 - irr)

  es$rd <- ifelse(reverse_irr, -ird, ird)
  # delta method if baseline_rate given, poisson variance otherwise
  ird_se <- ifelse(
    provided_baseline,
    baseline_rate * irr * logirr_se,
    sqrt(n_cases_exp / time_exp^2 + n_cases_nexp / time_nexp^2)
  )
  es$rd_se <- ird_se
  es$rd_ci_lo <- es$rd - qnorm(.975) * ird_se
  es$rd_ci_up <- es$rd + qnorm(.975) * ird_se

  es$nnt <- ifelse(ird == 0, NA, 1 / ird)
  es$nnt <- ifelse(reverse_irr, -es$nnt, es$nnt)
  es$nnt_se <- ifelse(ird == 0, NA, ird_se / ird^2)
  ird_ci_lo <- ird - qnorm(.975) * ird_se
  ird_ci_up <- ird + qnorm(.975) * ird_se
  crosses_zero <- (ird_ci_lo < 0 & ird_ci_up > 0) | ird == 0
  es$nnt_ci_lo <- ifelse(crosses_zero, NA,
                          ifelse(reverse_irr, -1 / ird_ci_lo, 1 / ird_ci_up))
  es$nnt_ci_up <- ifelse(crosses_zero, NA,
                          ifelse(reverse_irr, -1 / ird_ci_up, 1 / ird_ci_lo))

  es$info_used <- "cases_time"
  return(es)
}

#' Meta-analytic dataset inspired from Haza and colleagues (2024)
#'
#' Dataset of a meta-analysis exploring the specificity of social functioning of children with ADHD (compared to healthy controls) in case-control studies.
#' This dataset contains:
#' 1. several information coming from the same participants (due to the completion of multiple outcomes).
#' 1. several information coming from the same study (due to the presence of multiple subgroups).
#' 1. overlapping information for the same comparison
#' 1. several information types from which a standardized mean difference can be estimated/converted
#'
#' @source Haza B, Gosling CJ, Conty L & Pinabiaux C (2024). Social Functioning in Children and Adolescents with ADHD: A Meta-analysis. Journal of Child Psychology and Psychiatry and Allied Disciplines.
"df.haza"

#' Short version of the df.haza dataset
#'
#' This dataset is a shoter version of the \code{\link{df.haza}} dataset.
#'
#' @source Haza B, Gosling CJ, Conty L & Pinabiaux C (2024). Social Functioning in Children and Adolescents with ADHD: A Meta-analysis. Journal of Child Psychology and Psychiatry and Allied Disciplines.
"df.short"


#' Fictitious dataset 1
#'
#' First fictitious dataset aiming to understand how the \code{\link{compare_df}} function works.
#' Slightly different from \code{\link{df.compare1}}
#'
"df.compare1"

#' Fictitious dataset 2
#'
#' First fictitious dataset aiming to understand how the \code{\link{compare_df}} function works.
#' Slightly different from \code{\link{df.compare2}}
#'
"df.compare2"

#' Simulated dataset for a COSMIN-based systematic review of PROMs
#'
#' Simulated dataset of a systematic review evaluating measurement properties
#' of a fictional patient-reported outcome measure (the Mental Health Wellbeing
#' Scale, MHWS) following the COSMIN framework. Designed to demonstrate the
#' psychometric functions of the metaConvert package.
#'
#' The dataset contains studies reporting:
#' \itemize{
#'   \item Internal consistency (Cronbach's alpha)
#'   \item Test-retest reliability (ICC)
#'   \item Criterion and construct validity (Pearson and Spearman correlations)
#'   \item Responsiveness (change-score correlations)
#'   \item Floor/ceiling effects (proportions)
#' }
#'
#' Additional columns support standalone psychometric utility functions
#' (SEM, SDC, disattenuation, change-score reliability).
#'
"df.psychom"

#' Converts the means and bounds of an error bar (generally extracted from a plot) into four effect measures (SMD, MD, OR, COR)
#'
#' @param plot_mean_exp mean of participants in the experimental/exposed group (extracted from a plot).
#' @param plot_mean_nexp mean of participants in the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_mean_sd_lo_exp lower bound of an error bar depicting -1 SD from the mean of the experimental/exposed group (extracted from a plot).
#' @param plot_mean_sd_lo_nexp lower bound of an error bar depicting -1 SD from the mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_mean_sd_up_exp upper bound of an error bar depicting +1 SD from the mean of the experimental/exposed group (extracted from a plot).
#' @param plot_mean_sd_up_nexp upper bound of an error bar depicting +1 SD from the mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_mean_se_lo_exp lower bound of an error bar depicting -1 SE from the mean of the experimental/exposed group (extracted from a plot).
#' @param plot_mean_se_lo_nexp lower bound of an error bar depicting -1 SE from the mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_mean_se_up_exp upper bound of an error bar depicting +1 SE from the mean of the experimental/exposed group (extracted from a plot).
#' @param plot_mean_se_up_nexp upper bound of an error bar depicting +1 SE from the mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_mean_ci_lo_exp lower bound of an error bar depicting the 95% CI of the mean of the experimental/exposed group (extracted from a plot).
#' @param plot_mean_ci_lo_nexp lower bound of an error bar depicting the 95% CI of the mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_mean_ci_up_exp upper bound of an error bar depicting the 95% CI of the mean of the experimental/exposed group (extracted from a plot).
#' @param plot_mean_ci_up_nexp upper bound of an error bar depicting the 95% CI of the mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_plot_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function uses the bounds of an error bar of a mean obtained from a plot into a standard deviation.
#' Then, a mean difference (MD), Cohen's d (D), and Hedges' g (G) are estimated.
#' Odds ratio (OR), risk ratio (RR) and correlation coefficients (R/Z) are converted from the Cohen's d value.
#'
#' **To convert the bound of an error bar into a standard deviation,**
#' this function always prioritizes information from the \code{plot_mean_sd_*} arguments,
#' then those from the \code{plot_mean_se_*} arguments,
#' then those from the \code{plot_mean_ci_*} arguments.
#'
#' 1. If the bounds of the standard deviations are provided, the following formulas are used:
#' \deqn{mean\_sd\_lo\_exp = plot\_mean\_exp - plot\_mean\_sd\_lo\_exp}
#' \deqn{mean\_sd\_up\_exp = plot\_mean\_sd\_up\_exp - plot\_mean\_exp}
#' \deqn{mean\_sd\_exp = \frac{mean\_sd\_lo\_exp + mean\_sd\_up\_exp}{2}}
#'
#' \deqn{mean\_sd\_lo\_nexp = plot\_mean\_nexp - plot\_mean\_sd\_lo\_nexp}
#' \deqn{mean\_sd\_up\_nexp = plot\_mean\_sd\_up\_nexp - plot\_mean\_nexp}
#' \deqn{mean\_sd\_nexp = \frac{mean\_sd\_lo\_nexp + mean\_sd\_up\_nexp}{2}}
#'
#' Note that if only one bound (e.g., the upper bound) is provided, it will be the
#' only information used to estimate the standard deviation value.
#'
#' Then, calculations of the \code{\link{es_from_means_sd}} are used.
#'
#' 2. If the bounds of the standard errors are provided, the following formulas are used:
#' \deqn{mean\_se\_lo\_exp = plot\_mean\_exp - plot\_mean\_se\_lo\_exp}
#' \deqn{mean\_se\_up\_exp = plot\_mean\_se\_up\_exp - plot\_mean\_exp}
#' \deqn{mean\_se\_exp = \frac{mean\_se\_lo\_exp + mean\_se\_up\_exp}{2}}
#'
#' \deqn{mean\_se\_lo\_nexp = plot\_mean\_nexp - plot\_mean\_se\_lo\_nexp}
#' \deqn{mean\_se\_up\_nexp = plot\_mean\_se\_up\_nexp - plot\_mean\_nexp}
#' \deqn{mean\_se\_nexp = \frac{mean\_se\_lo\_nexp + mean\_se\_up\_nexp}{2}}
#'
#' Note that if only one bound (e.g., the upper bound) is provided, it will be the
#' only information used to estimate the standard error value.
#'
#' Then, calculations of the \code{\link{es_from_means_se}()} are used.
#'
#' 3. If the bounds of the 95% confidence intervals are provided, the calculations
#' of the \code{\link{es_from_means_ci}} are used.
#'
#' @export es_from_plot_means
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 21. From plot: means and dispersion (crude)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_plot_means(
#'   n_exp = 35, n_nexp = 35,
#'   plot_mean_exp = 89, plot_mean_nexp = 104,
#'   plot_mean_sd_lo_exp = 69, plot_mean_sd_lo_nexp = 83,
#'   plot_mean_sd_up_exp = 109, plot_mean_sd_up_nexp = 125
#' )
es_from_plot_means <- function(n_exp, n_nexp,
                               plot_mean_exp, plot_mean_nexp,
                               plot_mean_sd_lo_exp, plot_mean_sd_lo_nexp,
                               plot_mean_sd_up_exp, plot_mean_sd_up_nexp,
                               plot_mean_se_lo_exp, plot_mean_se_lo_nexp,
                               plot_mean_se_up_exp, plot_mean_se_up_nexp,
                               plot_mean_ci_lo_exp, plot_mean_ci_lo_nexp,
                               plot_mean_ci_up_exp, plot_mean_ci_up_nexp,
                               smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_plot_means) {
  if (missing(plot_mean_sd_lo_exp)) plot_mean_sd_lo_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_sd_lo_nexp)) plot_mean_sd_lo_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_sd_up_exp)) plot_mean_sd_up_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_sd_up_nexp)) plot_mean_sd_up_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_se_lo_exp)) plot_mean_se_lo_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_se_lo_nexp)) plot_mean_se_lo_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_se_up_exp)) plot_mean_se_up_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_se_up_nexp)) plot_mean_se_up_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_ci_lo_exp)) plot_mean_ci_lo_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_ci_lo_nexp)) plot_mean_ci_lo_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_ci_up_exp)) plot_mean_ci_up_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_mean_ci_up_nexp)) plot_mean_ci_up_nexp <- rep(NA_real_, length(n_exp))

  if (missing(reverse_plot_means)) reverse_plot_means <- rep(FALSE, length(n_exp))
  reverse_plot_means[is.na(reverse_plot_means)] <- FALSE

  ## SD ------
  #### exp
  mean_sd_exp_lo <- plot_mean_exp - plot_mean_sd_lo_exp
  mean_sd_exp_up <- plot_mean_sd_up_exp - plot_mean_exp
  mean_sd_exp_transit <- apply(cbind(mean_sd_exp_lo, mean_sd_exp_up), 1, mean, na.rm = TRUE)
  mean_sd_exp <- ifelse(is.na(mean_sd_exp_lo) &
                          is.na(mean_sd_exp_up), NA_real_,
                        mean_sd_exp_transit)
  #### nexp
  mean_sd_nexp_lo <- plot_mean_nexp - plot_mean_sd_lo_nexp
  mean_sd_nexp_up <- plot_mean_sd_up_nexp - plot_mean_nexp
  mean_sd_nexp_transit <- apply(cbind(mean_sd_nexp_lo, mean_sd_nexp_up), 1, mean, na.rm = TRUE)
  mean_sd_nexp <- ifelse(is.na(mean_sd_nexp_lo) &
                           is.na(mean_sd_nexp_up), NA_real_,
                         mean_sd_nexp_transit)

  ## SE ------
  #### exp

  mean_se_exp_lo <- plot_mean_exp - plot_mean_se_lo_exp
  mean_se_exp_up <- plot_mean_se_up_exp - plot_mean_exp
  mean_se_exp_transit <- apply(cbind(mean_se_exp_lo, mean_se_exp_up), 1, mean, na.rm = TRUE)
  mean_se_exp <- ifelse(is.na(mean_se_exp_lo) & is.na(mean_se_exp_up),
                        NA_real_, mean_se_exp_transit)
  #### nexp
  mean_se_nexp_lo <- plot_mean_nexp - plot_mean_se_lo_nexp
  mean_se_nexp_up <- plot_mean_se_up_nexp - plot_mean_nexp
  mean_se_nexp_transit <- apply(cbind(mean_se_nexp_lo, mean_se_nexp_up), 1, mean, na.rm = TRUE)
  mean_se_nexp <- ifelse(is.na(mean_se_nexp_lo) & is.na(mean_se_nexp_up), NA_real_,
                         mean_se_nexp_transit)

  es <- es_from_means_sd(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_exp = plot_mean_exp, mean_sd_exp = mean_sd_exp,
    mean_nexp = plot_mean_nexp, mean_sd_nexp = mean_sd_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_means = reverse_plot_means
  )

  es_1 <- es_from_means_se(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_exp = plot_mean_exp, mean_se_exp = mean_se_exp,
    mean_nexp = plot_mean_nexp, mean_se_nexp = mean_se_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_means = reverse_plot_means
  )

  es_2 <- es_from_means_ci(
    n_exp = n_exp, n_nexp = n_nexp,
    mean_exp = plot_mean_exp, mean_nexp = plot_mean_nexp,
    mean_ci_lo_exp = plot_mean_ci_lo_exp, mean_ci_lo_nexp = plot_mean_ci_lo_nexp,
    mean_ci_up_exp = plot_mean_ci_up_exp, mean_ci_up_nexp = plot_mean_ci_up_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_means = reverse_plot_means
  )

  row_miss <- which(is.na(es$d) & is.na(es$d_se))
  if (length(row_miss) > 0) {
    es[row_miss, ] <- es_1[row_miss, ]
    row_miss_2 <- which(is.na(es$d) & is.na(es$d_se))
    if (length(row_miss_2) > 0) {
      es[row_miss_2, ] <- es_2[row_miss_2, ]
    }
  }

  es$info_used <- "means_plot"

  return(es)
}

#' Converts the means and bounds of an error bar (generally extracted from a plot) into four effect measures (SMD, MD, OR, COR)
#'
#' @param plot_ancova_mean_exp ancova_mean of participants in the experimental/exposed group (extracted from a plot).
#' @param plot_ancova_mean_nexp ancova_mean of participants in the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_ancova_mean_sd_lo_exp lower bound of an error bar depicting -1 SD from the ancova_mean of the experimental/exposed group (extracted from a plot).
#' @param plot_ancova_mean_sd_lo_nexp lower bound of an error bar depicting -1 SD from the ancova_mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_ancova_mean_sd_up_exp upper bound of an error bar depicting +1 SD from the ancova_mean of the experimental/exposed group (extracted from a plot).
#' @param plot_ancova_mean_sd_up_nexp upper bound of an error bar depicting +1 SD from the ancova_mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_ancova_mean_se_lo_exp lower bound of an error bar depicting -1 SE from the ancova_mean of the experimental/exposed group (extracted from a plot).
#' @param plot_ancova_mean_se_lo_nexp lower bound of an error bar depicting -1 SE from the ancova_mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_ancova_mean_se_up_exp upper bound of an error bar depicting +1 SE from the ancova_mean of the experimental/exposed group (extracted from a plot).
#' @param plot_ancova_mean_se_up_nexp upper bound of an error bar depicting +1 SE from the ancova_mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_ancova_mean_ci_lo_exp lower bound of an error bar depicting the 95% CI of the ancova_mean of the experimental/exposed group (extracted from a plot).
#' @param plot_ancova_mean_ci_lo_nexp lower bound of an error bar depicting the 95% CI of the ancova_mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param plot_ancova_mean_ci_up_exp upper bound of an error bar depicting the 95% CI of the ancova_mean of the experimental/exposed group (extracted from a plot).
#' @param plot_ancova_mean_ci_up_nexp upper bound of an error bar depicting the 95% CI of the ancova_mean of the non-experimental/non-exposed group (extracted from a plot).
#' @param cov_outcome_r correlation between the outcome and covariate (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_plot_ancova_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function uses the bounds of an error bar of a mean obtained from a plot into a standard deviation.
#' Then, a mean difference (MD), Cohen's d (D), and Hedges' g (G) are estimated.
#' Odds ratio (OR), risk ratio (RR) and correlation coefficients (R/Z) are converted from the Cohen's d value.
#'
#' **To convert the bound of an error bar into a standard deviation,**
#' this function always prioritizes information from the \code{plot_ancova_mean_sd_*} arguments,
#' then those from the \code{plot_ancova_mean_se_*} arguments,
#' then those from the \code{plot_ancova_mean_ci_*} arguments.
#'
#' 1. If the bounds of the standard deviations are provided, the following formulas are used:
#' \deqn{ancova\_mean\_sd\_lo\_exp = plot\_ancova\_mean\_exp - plot\_ancova\_mean\_sd\_lo\_exp}
#' \deqn{ancova\_mean\_sd\_up\_exp = plot\_ancova\_mean\_sd\_up\_exp - plot\_ancova\_mean\_exp}
#' \deqn{ancova\_mean\_sd\_exp = \frac{ancova\_mean\_sd\_lo\_exp + ancova\_mean\_sd\_up\_exp}{2}}
#'
#' \deqn{mean\_sd\_lo\_nexp = plot\_ancova\_mean\_nexp - plot\_ancova\_mean\_sd\_lo\_nexp}
#' \deqn{mean\_sd\_up\_nexp = plot\_ancova\_mean\_sd\_up\_nexp - plot\_ancova\_mean\_nexp}
#' \deqn{mean\_sd\_nexp = \frac{mean\_sd\_lo\_nexp + mean\_sd\_up\_nexp}{2}}
#'
#' Then, calculations of the \code{\link{es_from_ancova_means_sd}} are used.
#'
#' 2. If the bounds of the standard errors are provided, the following formulas are used:
#' \deqn{ancova\_mean\_se\_lo\_exp = plot\_ancova\_mean\_exp - plot\_ancova\_mean\_se\_lo\_exp}
#' \deqn{ancova\_mean\_se\_up\_exp = plot\_ancova\_mean\_se\_up\_exp - plot\_ancova\_mean\_exp}
#' \deqn{ancova\_mean\_se\_exp = \frac{ancova\_mean\_se\_lo\_exp + ancova\_mean\_se\_up\_exp}{2}}
#'
#' \deqn{mean\_se\_lo\_nexp = plot\_ancova\_mean\_nexp - plot\_ancova\_mean\_se\_lo\_nexp}
#' \deqn{mean\_se\_up\_nexp = plot\_ancova\_mean\_se\_up\_nexp - plot\_ancova\_mean\_nexp}
#' \deqn{mean\_se\_nexp = \frac{mean\_se\_lo\_nexp + mean\_se\_up\_nexp}{2}}
#'
#' Then, calculations of the \code{\link{es_from_ancova_means_se}} are used.
#'
#' 3. If the bounds of the 95% confidence intervals are provided, the calculations
#' of the \code{\link{es_from_ancova_means_ci}()} are used.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 22. From plot: adjusted means and dispersion (adjusted)'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_plot_ancova_means
#'
#' @md
#'
#' @examples
#' es_from_plot_ancova_means(
#'   n_exp = 35, n_nexp = 35,
#'   cov_outcome_r = 0.2, n_cov_ancova = 4,
#'   plot_ancova_mean_exp = 89, plot_ancova_mean_nexp = 104,
#'   plot_ancova_mean_sd_lo_exp = 69, plot_ancova_mean_sd_lo_nexp = 83,
#'   plot_ancova_mean_sd_up_exp = 109, plot_ancova_mean_sd_up_nexp = 125
#' )
es_from_plot_ancova_means <- function(n_exp, n_nexp,
                                      plot_ancova_mean_exp, plot_ancova_mean_nexp,
                                      plot_ancova_mean_sd_lo_exp, plot_ancova_mean_sd_lo_nexp,
                                      plot_ancova_mean_sd_up_exp, plot_ancova_mean_sd_up_nexp,
                                      plot_ancova_mean_se_lo_exp, plot_ancova_mean_se_lo_nexp,
                                      plot_ancova_mean_se_up_exp, plot_ancova_mean_se_up_nexp,
                                      plot_ancova_mean_ci_lo_exp, plot_ancova_mean_ci_lo_nexp,
                                      plot_ancova_mean_ci_up_exp, plot_ancova_mean_ci_up_nexp,
                                      cov_outcome_r, n_cov_ancova,
                                      smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_plot_ancova_means) {
  if (missing(plot_ancova_mean_sd_lo_exp)) plot_ancova_mean_sd_lo_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_sd_lo_nexp)) plot_ancova_mean_sd_lo_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_sd_up_exp)) plot_ancova_mean_sd_up_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_sd_up_nexp)) plot_ancova_mean_sd_up_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_se_lo_exp)) plot_ancova_mean_se_lo_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_se_lo_nexp)) plot_ancova_mean_se_lo_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_se_up_exp)) plot_ancova_mean_se_up_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_se_up_nexp)) plot_ancova_mean_se_up_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_ci_lo_exp)) plot_ancova_mean_ci_lo_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_ci_lo_nexp)) plot_ancova_mean_ci_lo_nexp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_ci_up_exp)) plot_ancova_mean_ci_up_exp <- rep(NA_real_, length(n_exp))
  if (missing(plot_ancova_mean_ci_up_nexp)) plot_ancova_mean_ci_up_nexp <- rep(NA_real_, length(n_exp))

  if (missing(reverse_plot_ancova_means)) reverse_plot_ancova_means <- rep(FALSE, length(n_exp))
  reverse_plot_ancova_means[is.na(reverse_plot_ancova_means)] <- FALSE

  ## SD ------
  #### exp
  ancova_mean_sd_exp_lo <- plot_ancova_mean_exp - plot_ancova_mean_sd_lo_exp
  ancova_mean_sd_exp_up <- plot_ancova_mean_sd_up_exp - plot_ancova_mean_exp
  ancova_mean_sd_exp_transit <- apply(cbind(ancova_mean_sd_exp_lo, ancova_mean_sd_exp_up), 1, mean, na.rm = TRUE)
  ancova_mean_sd_exp <- ifelse(is.na(ancova_mean_sd_exp_lo) & is.na(ancova_mean_sd_exp_up), NA_real_, ancova_mean_sd_exp_transit)
  #### nexp
  ancova_mean_sd_nexp_lo <- plot_ancova_mean_nexp - plot_ancova_mean_sd_lo_nexp
  ancova_mean_sd_nexp_up <- plot_ancova_mean_sd_up_nexp - plot_ancova_mean_nexp
  ancova_mean_sd_nexp_transit <- apply(cbind(ancova_mean_sd_nexp_lo, ancova_mean_sd_nexp_up), 1, mean, na.rm = TRUE)
  ancova_mean_sd_nexp <- ifelse(is.na(ancova_mean_sd_nexp_lo) & is.na(ancova_mean_sd_nexp_up), NA_real_, ancova_mean_sd_nexp_transit)

  ## SE ------
  #### exp
  ancova_mean_se_exp_lo <- plot_ancova_mean_exp - plot_ancova_mean_se_lo_exp
  ancova_mean_se_exp_up <- plot_ancova_mean_se_up_exp - plot_ancova_mean_exp
  ancova_mean_se_exp_transit <- apply(cbind(ancova_mean_se_exp_lo, ancova_mean_se_exp_up), 1, mean, na.rm = TRUE)
  ancova_mean_se_exp <- ifelse(is.na(ancova_mean_se_exp_lo) & is.na(ancova_mean_se_exp_up), NA_real_, ancova_mean_se_exp_transit)
  # ancova_mean_sd_conv_exp = ancova_mean_se_exp * sqrt(n_exp)
  # ancova_mean_sd_conv_exp = ancova_mean_se_exp * sqrt(n_exp - 1)
  #### nexp
  ancova_mean_se_nexp_lo <- plot_ancova_mean_nexp - plot_ancova_mean_se_lo_nexp
  ancova_mean_se_nexp_up <- plot_ancova_mean_se_up_nexp - plot_ancova_mean_nexp
  ancova_mean_se_nexp_transit <- apply(cbind(ancova_mean_se_nexp_lo, ancova_mean_se_nexp_up), 1, mean, na.rm = TRUE)
  ancova_mean_se_nexp <- ifelse(is.na(ancova_mean_se_nexp_lo) & is.na(ancova_mean_se_nexp_up), NA_real_, ancova_mean_se_nexp_transit)
  # ancova_mean_sd_conv_nexp = ancova_mean_se_nexp * sqrt(n_nexp - 1)
  # ancova_mean_sd_conv_nexp = ancova_mean_se_nexp * sqrt(n_nexp)

  es <- es_from_ancova_means_sd(
    n_exp = n_exp, n_nexp = n_nexp,
    ancova_mean_exp = plot_ancova_mean_exp,
    ancova_mean_sd_exp = ancova_mean_sd_exp,
    ancova_mean_nexp = plot_ancova_mean_nexp,
    ancova_mean_sd_nexp = ancova_mean_sd_nexp,
    cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_means = reverse_plot_ancova_means
  )
  es_1 <- es_from_ancova_means_se(
    n_exp = n_exp, n_nexp = n_nexp,
    ancova_mean_exp = plot_ancova_mean_exp,
    ancova_mean_se_exp = ancova_mean_se_exp,
    ancova_mean_nexp = plot_ancova_mean_nexp,
    ancova_mean_se_nexp = ancova_mean_se_nexp,
    cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_means = reverse_plot_ancova_means
  )
  es_2 <- es_from_ancova_means_ci(
    n_exp = n_exp, n_nexp = n_nexp,
    ancova_mean_exp = plot_ancova_mean_exp,
    ancova_mean_ci_lo_exp = plot_ancova_mean_ci_lo_exp,
    ancova_mean_ci_up_exp = plot_ancova_mean_ci_up_exp,
    ancova_mean_nexp = plot_ancova_mean_nexp,
    ancova_mean_ci_lo_nexp = plot_ancova_mean_ci_lo_nexp,
    ancova_mean_ci_up_nexp = plot_ancova_mean_ci_up_nexp,
    cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_means = reverse_plot_ancova_means
  )
  # ## CI ------
  # #### exp
  # ancova_mean_se_exp_conv = (plot_ancova_mean_ci_up_exp - plot_ancova_mean_ci_lo_exp) / (2 * qt(0.975, n_exp - 1))
  # ancova_mean_sd_conv_exp_ci = ancova_mean_se_exp_conv * sqrt(n_exp)
  # # ancova_mean_sd_conv_exp_ci = ancova_mean_se_exp_conv * sqrt(n_exp - 1)
  # #### nexp
  # ancova_mean_se_nexp_conv = (plot_ancova_mean_ci_up_nexp - plot_ancova_mean_ci_lo_nexp) / (2 * qt(0.975, n_nexp - 1))
  # # ancova_mean_sd_conv_nexp_ci = ancova_mean_se_nexp_conv * sqrt(n_nexp - 1)
  # ancova_mean_sd_conv_nexp_ci = ancova_mean_se_nexp_conv * sqrt(n_nexp)
  #
  #
  # es_1 = es_from_ancova_means_sd(n_exp = n_exp, n_nexp = n_nexp,
  #                             ancova_mean_exp = plot_ancova_mean_exp, ancova_mean_sd_exp = ancova_mean_sd_conv_exp,
  #                             ancova_mean_nexp = plot_ancova_mean_nexp, ancova_mean_sd_nexp = ancova_mean_sd_conv_nexp,
  #                             cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
  #                             smd_to_cor = smd_to_cor, reverse_ancova_means = reverse_plot_ancova_means)
  #
  # es_2 = es_from_ancova_means_sd(n_exp = n_exp, n_nexp = n_nexp,
  #                             ancova_mean_exp = plot_ancova_mean_exp, ancova_mean_sd_exp = ancova_mean_sd_conv_exp_ci,
  #                             ancova_mean_nexp = plot_ancova_mean_nexp, ancova_mean_sd_nexp = ancova_mean_sd_conv_nexp_ci,
  #                             cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
  #                             smd_to_cor = smd_to_cor, reverse_ancova_means = reverse_plot_ancova_means)

  row_miss <- which(is.na(es$d) & is.na(es$d_se))
  if (length(row_miss) > 0) {
    es[row_miss, ] <- es_1[row_miss, ]
    row_miss_2 <- which(is.na(es$d) & is.na(es$d_se))
    if (length(row_miss_2) > 0) {
      es[(row_miss_2), ] <- es_2[(row_miss_2), ]
    }
  }
  es$info_used <- "ancova_means_plot"

  return(es)
}

# m1 = rnorm(40, 50, 10)
# m2 = rnorm(40, 40, 10)
# sd = c(rnorm(15, 10, 2), rep(NA, 25))
# se = c(rep(NA, 15), rnorm(15, 8, 2), rep(NA, 10))
# ci = c(rep(NA, 30), abs(rnorm(10, 4, 1)))
# n_exp = runif(40, 10, 50)
# n_nexp = runif(40, 10, 50)
# reverse_means = rep(NA, 40)
# dat = data.frame(
#   n_exp = runif(40, 10, 50),
#   n_nexp = runif(40, 10, 50),
#   plot_mean_exp = m1,
#   plot_mean_nexp = m2,
#   plot_mean_sd_lo_exp = c(m1 - sd),
#   plot_mean_sd_lo_nexp = c(m2 - sd),
#   plot_mean_sd_up_exp = c(m1 + sd),
#   plot_mean_sd_up_nexp = c(m2 + sd),
#   plot_mean_se_lo_exp = c( m1 - se),
#   plot_mean_se_lo_nexp = c(m2 - se),
#   plot_mean_se_up_exp = c(m1 + se),
#   plot_mean_se_up_nexp = c(m2 + se),
#   plot_mean_ci_lo_exp = c(m1 - ci),
#   plot_mean_ci_lo_nexp = m2 - ci,
#   plot_mean_ci_up_exp = m1 + ci,
#   plot_mean_ci_up_nexp = m2 + ci)
#
# m1 = rnorm(40, 50, 10)
# m2 = rnorm(40, 40, 10)
# sd = c(rnorm(15, 10, 2), rep(NA, 25))
# se = c(rep(NA, 15), rnorm(15, 8, 2), rep(NA, 10))
# ci = c(rep(NA, 30), abs(rnorm(10, 4, 1)))
#
#   n_exp = runif(40, 10, 50)
#   n_nexp = runif(40, 10, 50)
#   plot_mean_exp = m1
#   plot_mean_nexp = m2
#   plot_mean_sd_lo_exp = c(m1 - sd)
#   plot_mean_sd_lo_nexp = c(m2 - sd)
#   plot_mean_sd_up_exp = c(m1 + sd)
#   plot_mean_sd_up_nexp = c(m2 + sd)
#   plot_mean_se_lo_exp = c( m1 - se)
#   plot_mean_se_lo_nexp = c(m2 - se)
#   plot_mean_se_up_exp = c(m1 + se)
#   plot_mean_se_up_nexp = c(m2 + se)
#   plot_mean_ci_lo_exp = c(m1 - ci)
#   plot_mean_ci_lo_nexp = m2 - ci
#   plot_mean_ci_up_exp = m1 + ci
#   plot_mean_ci_up_nexp = m2 + ci
#
#   res = es_from_plot_means(n_exp = dat$n_exp, n_nexp=dat$n_nexp,
#                      plot_mean_exp=dat$plot_mean_exp,
#                      plot_mean_nexp = dat$plot_mean_nexp,
#                      plot_mean_sd_lo_exp=dat$plot_mean_sd_lo_exp, plot_mean_sd_lo_nexp=dat$plot_mean_sd_lo_nexp,
#                      plot_mean_sd_up_exp=dat$plot_mean_sd_up_exp, plot_mean_sd_up_nexp=dat$plot_mean_sd_up_nexp,
#                      plot_mean_se_lo_exp=dat$plot_mean_se_lo_exp, plot_mean_se_lo_nexp=dat$plot_mean_se_lo_nexp,
#                      plot_mean_se_up_exp=dat$plot_mean_se_up_exp, plot_mean_se_up_nexp=dat$plot_mean_se_up_nexp,
#                      plot_mean_ci_lo_exp=dat$plot_mean_ci_lo_exp, plot_mean_ci_lo_nexp=dat$plot_mean_ci_lo_nexp,
#                      plot_mean_ci_up_exp=dat$plot_mean_ci_up_exp, plot_mean_ci_up_nexp=dat$plot_mean_ci_up_nexp,
#                      reverse_plot_means)

#' Convert an odds ratio value and its standard error into several effect size measures
#'
#' @param or odds ratio value
#' @param logor log odds ratio value
#' @param logor_se the standard error of the log odds ratio
#' @param n_cases number of cases/events across exposed/non-exposed groups
#' @param n_controls number of controls/no-event across exposed/non-exposed groups
#' @param n_exp number of participants in the exposed group
#' @param n_nexp number of participants in the non-exposed group
#' @param n_sample total number of participants in the sample
#' @param baseline_risk proportion of cases in the non-exposed group
#' @param small_margin_prop smallest margin proportion of cases/events in the underlying 2x2 table
#' @param reverse_or a logical value indicating whether the direction of the generated effect sizes should be flipped.
#' @param or_to_rr formula used to convert the \code{or} value into a risk ratio (see details).
#' @param or_to_cor formula used to convert the \code{or} value into a correlation coefficient (see details).
#'
#' @details
#' This function converts the log odds ratio into a Risk ratio (RR), Cohen's d (D), Hedges' g (G)
#' and correlation coefficients (R/Z).
#'
#' **To estimate the Cohen's d value and its standard error**
#' The following formulas are used (Cooper et al., 2019):
#' \deqn{d = \log(or) * \frac{\sqrt{3}}{\pi}}
#' \deqn{d\_se = \sqrt{\frac{logor\_se^2 * 3}{\pi^2}}}
#'
#' **To estimate the risk ratio and its standard error, various formulas can be used.**
#'
#' **A.** First, the approach described in Grant (2014) can be used.
#' However, in the paper, only the formula to convert an OR value to a RR value
#' is described.
#' To derive the variance, we used this formula to convert the bounds of the 95% CI, which
#' were then used to obtain the variance.
#'
#' This argument requires (or + baseline_risk + or_ci_lo + or_ci_up) to generate a RR.
#' The following formulas are used (br = baseline_risk):
#' \deqn{rr = \frac{or}{1 - br + br*or}}
#' \deqn{rr\_ci\_lo = \frac{or\_ci\_lo}{1 - br + br*or\_ci\_lo}}
#' \deqn{rr\_ci\_up = \frac{or\_ci\_up}{1 - br + br*or\_ci\_up}}
#' \deqn{logrr\_se = \frac{log(rr\_ci\_up) - log(rr\_ci\_lo)}{2 * qnorm(.975)}}
#'
#'
#' **B.** Second, the formulas implemented in the metaumbrella package can be used
#' (\code{or_to_rr = "metaumbrella_cases"} or \code{or_to_rr = "metaumbrella_exp"}).
#' This argument requires (or + logor_se + n_cases + n_controls) or (or + logor_se + n_exp + n_nexp)
#' to generate a RR.
#' More precisely, when the OR value and its standard error, plus either
#' (i) the number of cases and controls or
#' (ii) the number of participants in the exposed and non-exposed groups,
#' are available, we previously developed functions that simulate all combinations of the possible
#' number of cases and controls
#' in the exposed and non-exposed groups compatible with the actual value of the OR.
#' Then, the functions select the contingency table whose standard error coincides best with
#' the standard error reported.
#' The RR value and its standard are obtained from this estimated contingency table.
#'
#' **C.** Third, it is possible to transpose the RR to a OR (\code{or_to_rr = "transpose"}).
#' This argument requires (or + logor_se) to generate a OR.
#' It is known that OR and RR are similar when the baseline risk is small.
#' Therefore, users can request to simply transpose the OR value & standard error into a RR value & standard error.
#' \deqn{rr = or}
#' \deqn{logrr\_se = logor\_se}
#'
#' **D.** Fourth, it is possible to recreate the 2x2 table using the dipietrantonj's formulas (\code{or_to_rr = "dipietrantonj"}).
#' This argument requires (or + logor_ci_lo + logor_ci_lo) to generate a RR. Information on this approach can be retrieved in
#' Di Pietrantonj (2006).
#'
#' **To estimate the NNT, the formulas used are :**
#' \deqn{treatment\_risk = \frac{or \times br}{1 - br + or \times br}}
#' \deqn{rd = br - treatment\_risk}
#' \deqn{nnt = \frac{1}{rd} = \frac{1 - br \times (1 - or)}{br \times (1 - br)}}
#'
#' **To estimate a correlation coefficient, various formulas can be used.**
#'
#' **A.** First, the approach described in Pearson (1900) can be used (\code{or_to_cor = "pearson"}).
#' This argument requires (or + logor_se) to generate a R/Z.
#' It converts the OR value and its standard error to a tetrachoric correlation.
#' Note that the formula assumes that each cell of the 2x2 used to estimate the OR has been added 1/2 before estimating the OR value and its standard error.
#' If it is not the case, formulas can produce slightly less accurate results.
#'
#' \deqn{c = \frac{1}{2}}
#' \deqn{r = \cos{\frac{\pi}{1+or^c}}}
#' \deqn{r\_se = logor\_se * (\pi * c * or^c) * \frac{\sin(\pi / (1+or^c))}{(1+or^c)^2}}
#' \deqn{or\_ci\_lo = exp(log(or) - qnorm(.975)*logor\_se)}
#' \deqn{or\_ci\_up = exp(log(or) + qnorm(.975)*logor\_se)}
#' \deqn{r\_ci\_lo = cos(\frac{\pi}{1 + or\_ci\_lo^c})}
#' \deqn{r\_ci\_up = cos(\frac{\pi}{1 + or\_ci\_up^c})}
#' \deqn{z = atanh(r)}
#' \deqn{z\_se = \sqrt{\frac{r\_se^2}{(1 - r^2)^2}}}
#' \deqn{z\_ci\_lo = atanh(r\_lo)}
#' \deqn{z\_ci\_up = atanh(r\_up)}
#'
#' **B.** Second, the approach described in Digby (1983) can be used (\code{or_to_cor = "digby"}).
#' This argument requires (or + logor_se) to generate a R/Z.
#' It converts the OR value and its standard error to a tetrachoric correlation.
#' Note that the formula assumes that each cell of the 2x2 used to estimate the OR has been added 1/2 before estimating the OR value and its standard error.
#' If it is not the case, formulas can produce slightly less accurate results.
#'
#' \deqn{c = \frac{3}{4}}
#' \deqn{r = \frac{or^c - 1}{or^c + 1}}
#' \deqn{r\_se = \sqrt{\frac{c^2}{4} * (1 - r^2)^2 * logor\_se^2}}
#' \deqn{z = atanh(r)}
#' \deqn{z\_se = \sqrt{\frac{r\_se^2}{(1 - r^2)^2}}}
#' \deqn{z\_ci\_lo = z - qnorm(.975)*\sqrt{\frac{c^2}{4} * logor\_se}}
#' \deqn{z\_ci\_up = z + qnorm(.975)*\sqrt{\frac{c^2}{4} * logor\_se}}
#' \deqn{r\_ci\_lo = tanh(z\_lo)}
#' \deqn{r\_ci\_up = tanh(z\_up)}
#'
#' **C.** Third, the approach described in Bonett (2005) can be used (\code{or_to_cor = "bonett"}).
#' This argument requires (or + logor_se + n_cases + n_exp + small_margin_prop) to generate a R/Z.
#' Note that the formula assumes that each cell of the 2x2 used to estimate the OR has been added 1/2 before estimating the OR value and its standard error.
#' If it is not the case, formulas can produce slightly less accurate results.
#'
#' \deqn{c = \frac{\frac{1 - |n\_exp - n\_cases|}{5} - (0.5 - small\_margin\_prop)^2}{2}}
#' \deqn{r = \cos{\frac{\pi}{1+or^c}}}
#' \deqn{r\_se = logor\_se * (\pi * c * or^c) * \frac{\sin(\frac{\pi}{1+or^c})}{(1+or^c)^2}}
#' \deqn{or\_ci\_lo = exp(log(or) - qnorm(.975)*logor\_se)}
#' \deqn{or\_ci\_up = exp(log(or) + qnorm(.975)*logor\_se)}
#' \deqn{r\_ci\_lo = cos(\frac{\pi}{1 + or\_ci\_lo^c})}
#' \deqn{r\_ci\_up = cos(\frac{\pi}{1 + or\_ci\_up^c})}
#' \deqn{z = atanh(r)}
#' \deqn{z\_se = \sqrt{\frac{r\_se^2}{(1 - r^2)^2}} }
#' \deqn{z\_ci\_lo = atanh(r\_lo)}
#' \deqn{z\_ci\_up = atanh(r\_up)}
#'
#' **D.** Last, the approach described in Cooper et al. (2019) can be used (\code{or_to_cor = "lipsey_cooper"}).
#' This argument requires (or + logor_se + n_exp + n_nexp) to generate a R/Z.
#' As shown above, the function starts to estimate a SMD from the OR.
#' Then, as described in \code{\link{es_from_cohen_d}}, it converts this Cohen's d value into a correlation
#' coefficient using the \code{"lipsey_cooper"} formulas.
#'
#'
#' @export es_from_or_se
#'
#' @md
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab RR + NNT + RD\cr
#'  \code{} \tab D + G + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 2. Odds Ratio'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Bonett, Douglas G. and Robert M. Price. (2005). Inferential Methods for the Tetrachoric Correlation Coefficient. Journal of Educational and Behavioral Statistics 30:213-25.
#'
#' Bonett, D. G., & Price, R. M. (2007). Statistical inference for generalized Yule coefficients in 2* 2 contingency tables. Sociological methods & research, 35(3), 429-446.
#'
#' Cooper, H., Hedges, L. V., & Valentine, J. C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Di Pietrantonj C. (2006). Four-fold table cell frequencies imputation in meta analysis. Statistics in medicine, 25(13), 2299-2322. https://doi.org/10.1002/sim.2287
#'
#' Digby, Peter G. N. (1983). Approximating the Tetrachoric Correlation Coefficient. Biometrics 39:753-7.
#'
#' Gosling, C. J., Solanes, A., Fusar-Poli, P., & Radua, J. (2023). metaumbrella: the first comprehensive suite to perform data analysis in umbrella reviews with stratification of the evidence. BMJ mental health, 26(1), e300534. https://doi.org/10.1136/bmjment-2022-300534
#'
#' Grant R. L. (2014). Converting an odds ratio to a range of plausible relative risks for better communication of research findings. BMJ (Clinical research ed.), 348, f7450. https://doi.org/10.1136/bmj.f7450
#'
#' Pearson, K. (1900). Mathematical Contributions to the Theory of Evolution. VII: On the Correlation of Characters Not Quantitatively Measurable. Philosophical Transactions of the Royal Statistical Society of London, Series A 19:1-47
#'
#' Veroniki, A. A., Pavlides, M., Patsopoulos, N. A., & Salanti, G. (2013). Reconstructing 2x2 contingency tables from odds ratios using the Di Pietrantonj method: difficulties, constraints and impact in meta-analysis results. Research synthesis methods, 4(1), 78-94. https://doi.org/10.1002/jrsm.1061
#'
#' @examples
#' es_from_or_se(or = 2.12, logor_se = 0.242, n_exp = 120, n_nexp = 44)
es_from_or_se <- function(or, logor, logor_se, baseline_risk,
                          small_margin_prop,
                          n_exp, n_nexp, n_cases, n_controls, n_sample,
                          or_to_rr = "metaumbrella_cases",
                          or_to_cor = "pearson", reverse_or) {

  if (!all(or_to_cor %in% c("pearson", "digby", "bonett", "lipsey_cooper"))) {
    stop(paste0("'",
                unique(or_to_cor[!or_to_cor %in% c("pearson", "digby", "bonett", "lipsey_cooper")]),
                "' not in tolerated values for the 'or_to_cor' argument.
                Possible inputs are: 'pearson', 'digby', 'bonett', 'lipsey_cooper'"))

  } else if (!all(or_to_rr %in% c("metaumbrella_cases", "metaumbrella_exp", "transpose", "grant", "dipietrantonj"))) {
    stop(paste0("'",
                unique(or_to_rr[!or_to_rr %in% c("metaumbrella_cases", "metaumbrella_exp", "transpose", "grant", "dipietrantonj")]),
                "' not in tolerated values for the 'or_to_rr' argument.
                Possible inputs are: 'metaumbrella_cases', 'metaumbrella_exp', 'transpose', 'grant', 'dipietrantonj'"))
  }


  # or = dat$or; logor = dat$logor; logor_se = dat$logor_se;
  # baseline_risk = dat$baseline_risk;
  # small_margin_prop = dat$small_margin_prop;
  # n_exp = dat$n_exp; n_nexp = dat$n_nexp; n_cases = dat$n_cases; n_controls = dat$n_controls;
  # or_to_rr = "metaumbrella_cases";
  # or_to_cor = "cooper_delta";
  # reverse_or <- rep(FALSE, length(or))
  # or_to_rr = "dipietrantonj"
  if (missing(or)) {
    or <- rep(NA_real_, length(logor))
  }
  if (missing(logor)) {
    logor <- rep(NA_real_, length(or))
  }
  if (missing(baseline_risk)) {
    baseline_risk <- rep(NA_real_, length(or))
  }
  if (missing(small_margin_prop)) {
    small_margin_prop <- rep(NA_real_, length(or))
  }
  if (missing(n_exp)) {
    n_exp <- rep(NA_real_, length(or))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA_real_, length(or))
  }
  if (missing(n_cases)) {
    n_cases <- rep(NA_real_, length(or))
  }
  if (missing(n_controls)) {
    n_controls <- rep(NA_real_, length(or))
  }
  if (missing(reverse_or)) {
    reverse_or <- rep(FALSE, length(or))
  }
  if (missing(n_sample)) {
    n_sample <- rep(NA_real_, length(or))
  }
  reverse_or[is.na(reverse_or)] <- FALSE
  if (length(reverse_or) == 1) reverse_or = c(rep(reverse_or, length(or)))
  if (length(reverse_or) != length(or)) stop("The length of the 'reverse_or' argument is incorrectly specified.")

  or <- ifelse(is.na(or) & !is.na(logor), exp(logor), or)

  logOR <- suppressWarnings(log(or))

  d <- logOR * sqrt(3) / pi
  d_se <- sqrt(logor_se^2 * 3 / (pi^2))

  n_sample <- ifelse(is.na(n_sample),
    ifelse(!is.na(n_cases) & !is.na(n_controls),
           n_cases + n_controls,
           n_exp + n_nexp),
    n_sample
  )

  es <- .es_from_d(
    d = d, d_se = d_se, n_exp = n_exp, n_nexp = n_nexp,
    n_sample = n_sample, reverse = reverse_or,
    smd_to_cor = rep("lipsey_cooper", length(d))
  )

  row_miss = which(is.na(d_se))
  es$d[row_miss] <- es$d_se[row_miss] <-
    es$d_ci_lo[row_miss] <- es$d_ci_up[row_miss] <-
    es$g[row_miss] <- es$g_se[row_miss] <-
    es$g_ci_lo[row_miss] <- es$g_ci_up[row_miss] <- NA
  es$r[row_miss] <- es$r_se[row_miss] <-
    es$r_ci_lo[row_miss] <- es$r_ci_up[row_miss] <-
    es$z[row_miss] <- es$z_se[row_miss] <-
    es$z_ci_lo[row_miss] <- es$z_ci_up[row_miss] <- NA


  # OR -------
  es$logor <- ifelse(reverse_or, -logOR, logOR)
  es$logor_se <- logor_se
  es$logor_ci_lo <- es$logor - es$logor_se * qnorm(.975)
  es$logor_ci_up <- es$logor + es$logor_se * qnorm(.975)
  or_ci_lo <- exp(log(or) - logor_se * qnorm(.975))
  or_ci_up <- exp(log(or) + logor_se * qnorm(.975))

  # RR -------
  es$logrr <- es$logrr_se <- es$logrr_ci_lo <- es$logrr_ci_up <- NA

  dat_rr <- data.frame(
    or = or, logor_se = logor_se, or_ci_lo = or_ci_lo, or_ci_up = or_ci_up,
    n_cases = n_cases, n_controls = n_controls, n_exp = n_exp, n_nexp = n_nexp,
    baseline_risk = baseline_risk, or_to_rr = or_to_rr
  )

  # print(paste0(or, ", ", or_ci_lo, ", ", or_ci_up, ", ",n_exp, ", ",n_nexp))
  nn_miss <- with(dat_rr, which(
      (or_to_rr == "grant" & !is.na(or) & !is.na(baseline_risk)) |
      (or_to_rr == "metaumbrella_cases" & !is.na(or) & !is.na(logor_se) &
         !is.na(n_cases) & !is.na(n_controls)) |
      (or_to_rr == "metaumbrella_exp" & !is.na(or) & !is.na(logor_se) &
         !is.na(n_exp) & !is.na(n_nexp)) |
      (or_to_rr == "transpose" & !is.na(or) & !is.na(logor_se)) |
      (or_to_rr == "dipietrantonj" & !is.na(or) & !is.na(or_ci_lo) & !is.na(or_ci_up) &
         !is.na(n_exp) & !is.na(n_nexp))
  ))


  if (length(nn_miss) != 0) {
    res_rr <- data.frame(t(mapply(.or_to_rr,
      or = dat_rr$or[nn_miss],
      logor_se = dat_rr$logor_se[nn_miss],
      or_ci_lo = dat_rr$or_ci_lo[nn_miss],
      or_ci_up = dat_rr$or_ci_up[nn_miss],
      n_cases = dat_rr$n_cases[nn_miss],
      n_controls = dat_rr$n_controls[nn_miss],
      n_exp = dat_rr$n_exp[nn_miss],
      n_nexp = dat_rr$n_nexp[nn_miss],
      baseline_risk = dat_rr$baseline_risk[nn_miss],
      or_to_rr = dat_rr$or_to_rr[nn_miss]
    )))
    res_rr$logrr = unlist(res_rr$logrr)
    res_rr$logrr_se = unlist(res_rr$logrr_se)
    res_rr$logrr_ci_lo = unlist(res_rr$logrr_ci_lo)
    res_rr$logrr_ci_up = unlist(res_rr$logrr_ci_up)
    es$logrr[nn_miss] <- ifelse(reverse_or[nn_miss], -res_rr[, 1], res_rr[, 1])
    es$logrr_se[nn_miss] <- res_rr[, 2]
    # On reverse: negate AND swap the CI bounds (new_lo = -old_up, new_up = -old_lo);
    # swapping alone left a wrong-signed, inverted interval that did not bracket -logrr.
    es$logrr_ci_lo[nn_miss] <- ifelse(reverse_or[nn_miss], -res_rr[, 4], res_rr[, 3])
    es$logrr_ci_up[nn_miss] <- ifelse(reverse_or[nn_miss], -res_rr[, 3], res_rr[, 4])
  }

  # COR -------
  dat_cor <- data.frame(
    or = or, logor_se = logor_se,
    n_cases = n_cases, n_exp = n_exp,
    n_sample = n_sample,
    small_margin_prop = small_margin_prop,
    or_to_cor = or_to_cor
  )

  nn_miss <- with(dat_cor, which(
    (or_to_cor == "bonett" & !is.na(or) & !is.na(logor_se) &
       !is.na(small_margin_prop) & !is.na(n_sample) &
      !is.na(n_exp) & !is.na(n_cases)) |
      (or_to_cor == "pearson" & !is.na(or) & !is.na(logor_se)) |
      (or_to_cor == "digby" & !is.na(or) & !is.na(logor_se))
  ))

  if (length(nn_miss) != 0) {
    no_cooper = which(or_to_cor != "lipsey_cooper")
    es$r[no_cooper] <- es$r_se[no_cooper] <- es$r_ci_lo[no_cooper] <- es$r_ci_up[no_cooper] <-
      es$z[no_cooper] <- es$z_se[no_cooper] <- es$z_ci_lo[no_cooper] <- es$z_ci_up[no_cooper] <- NA

    res_cor <- t(mapply(.or_to_cor,
      or = dat_cor$or[nn_miss],
      logor_se = dat_cor$logor_se[nn_miss],
      n_cases = dat_cor$n_cases[nn_miss],
      n_exp = dat_cor$n_exp[nn_miss],
      n_sample = dat_cor$n_sample[nn_miss],
      small_margin_prop = dat_cor$small_margin_prop[nn_miss],
      or_to_cor = dat_cor$or_to_cor[nn_miss]
    ))

    # On reverse: negate AND swap each CI (new_lo = -old_up, new_up = -old_lo). For r,
    # tanh is odd so negate-and-swap of the r bounds is correct even for the asymmetric,
    # z-back-transformed r interval. Swapping alone left inverted, wrong-signed bounds.
    es$r[nn_miss] <- ifelse(reverse_or[nn_miss], -res_cor[, 1], res_cor[, 1])
    es$r_se[nn_miss] <- res_cor[, 2]
    es$r_ci_lo[nn_miss] <- ifelse(reverse_or[nn_miss], -res_cor[, 4], res_cor[, 3])
    es$r_ci_up[nn_miss] <- ifelse(reverse_or[nn_miss], -res_cor[, 3], res_cor[, 4])
    es$z[nn_miss] <- ifelse(reverse_or[nn_miss], -res_cor[, 5], res_cor[, 5])
    es$z_se[nn_miss] <- res_cor[, 6]
    es$z_ci_lo[nn_miss] <- ifelse(reverse_or[nn_miss], -res_cor[, 8], res_cor[, 7])
    es$z_ci_up[nn_miss] <- ifelse(reverse_or[nn_miss], -res_cor[, 7], res_cor[, 8])
  }

  # Risk difference from OR + baseline_risk (Grant 2014)
  treatment_risk <- (or * baseline_risk) / (1 - baseline_risk + or * baseline_risk)
  rd <- baseline_risk - treatment_risk

  es$rd <- ifelse(reverse_or, -rd, rd)
  # delta method
  drd_dor <- baseline_risk * (1 - baseline_risk) / (1 - baseline_risk + or * baseline_risk)^2
  rd_se <- abs(drd_dor) * or * logor_se
  es$rd_se <- rd_se
  es$rd_ci_lo <- es$rd - qnorm(.975) * rd_se
  es$rd_ci_up <- es$rd + qnorm(.975) * rd_se

  es$nnt <- ifelse(rd == 0, NA, 1 / rd)
  es$nnt <- ifelse(reverse_or, -es$nnt, es$nnt)
  es$nnt_se <- ifelse(rd == 0, NA, rd_se / rd^2)
  rd_ci_lo_raw <- rd - qnorm(.975) * rd_se
  rd_ci_up_raw <- rd + qnorm(.975) * rd_se
  crosses_zero <- (rd_ci_lo_raw < 0 & rd_ci_up_raw > 0) | rd == 0
  es$nnt_ci_lo <- ifelse(crosses_zero, NA,
                          ifelse(reverse_or, -1 / rd_ci_lo_raw, 1 / rd_ci_up_raw))
  es$nnt_ci_up <- ifelse(crosses_zero, NA,
                          ifelse(reverse_or, -1 / rd_ci_up_raw, 1 / rd_ci_lo_raw))

  es$info_used <- "or_se"
  return(es)
}

#' Convert an odds ratio value to several effect size measures
#'
#' @param or odds ratio value
#' @param logor log odds ratio value
#' @param n_cases number of cases/events
#' @param n_controls number of controls/no-event
#' @param n_exp number of participants in the exposed group
#' @param n_nexp number of participants in the non-exposed group
#' @param n_sample total number of participants in the sample
#' @param baseline_risk proportion of cases in the non-exposed group (n_cases_nexp / n_nexp is used when missing)
#' @param small_margin_prop smallest margin proportion of the underlying 2x2 table
#' @param reverse_or a logical value indicating whether the direction of the generated effect sizes should be flipped.
#' @param or_to_rr formula used to convert the \code{or} value into a risk ratio (see details).
#' @param or_to_cor formula used to convert the \code{or} value into a correlation coefficient (see details).
#'
#' @details
#' This function computes the standard error of the log odds ratio.
#' Risk ratio (RR), Cohen's d (D), Hedges' g (G) and correlation coefficients (R/Z),
#' are converted from the odds ratio value.
#'
#' **Estimation of the standard error of the log OR.**
#' This function generates the standard error of an odds ratio (OR) based on the OR value and the number of cases and controls.
#' More precisely, this function simulates all combinations of the possible number of cases and controls in the exposed and non-exposed groups
#' compatible with the reported OR value and with the overall number of cases and controls. Then, our function assumes that the variance of the OR
#' is equal to the mean of the standard error of all possible situations. This estimation thus necessarily comes with some imprecision and should not
#' be used before having requested the value (or raw data) to authors of the original report.
#'
#' **Conversion of other effect size measures.**
#' Calculations of \code{\link{es_from_or_se}()} are then applied to
#' estimate the other effect size measures
#'
#' @references
#' Gosling, C. J., Solanes, A., Fusar-Poli, P., & Radua, J. (2023). metaumbrella: the first comprehensive suite to perform data analysis in umbrella reviews with stratification of the evidence. BMJ mental health, 26(1), e300534. https://doi.org/10.1136/bmjment-2022-300534
#'
#' @export es_from_or
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab N/A\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + RR + NNT\cr
#'  \code{} \tab D + G + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 2. Odds Ratio'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_or_guess <- es_from_or(or = 0.5, n_cases = 210, n_controls = 220)
#' es_or <- es_from_or_se(or = 0.5, logor_se = 0.4, n_cases = 210, n_controls = 220)
#' round(es_or_guess$logor_se, 0.10) == round(es_or$logor_se, 0.10)
es_from_or <- function(or, logor, n_cases, n_controls, n_sample,
                       small_margin_prop, baseline_risk,
                       n_exp, n_nexp,
                       or_to_cor = "bonett", or_to_rr = "metaumbrella_cases",
                       reverse_or) {
  if (missing(or)) {
    or <- rep(NA_real_, length(logor))
  }
  if (missing(logor)) {
    logor <- rep(NA_real_, length(or))
  }
  if (missing(baseline_risk)) {
    baseline_risk <- rep(NA_real_, length(or))
  }
  if (missing(small_margin_prop)) {
    small_margin_prop <- rep(NA_real_, length(or))
  }
  if (missing(n_exp)) {
    n_exp <- rep(NA_real_, length(or))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA_real_, length(or))
  }
  if (missing(n_cases)) {
    n_cases <- rep(NA_real_, length(or))
  }
  if (missing(n_controls)) {
    n_controls <- rep(NA_real_, length(or))
  }
  if (missing(reverse_or)) {
    reverse_or <- rep(FALSE, length(or))
  }
  if (missing(n_sample)) {
    n_sample <- rep(NA_real_, length(or))
  }
  reverse_or[is.na(reverse_or)] <- FALSE

  or <- ifelse(is.na(or) & !is.na(logor), exp(logor), or)

  dat_or_se <- data.frame(or, n_cases, n_controls)
  log_or_se <- do.call(rbind, apply(dat_or_se, 1, .se_from_or))

  es <- es_from_or_se(
    or = or, logor_se = log_or_se$se,
    n_cases = n_cases, n_controls = n_controls,
    small_margin_prop = small_margin_prop,
    baseline_risk = baseline_risk, n_sample=n_sample,
    n_exp = n_exp, n_nexp = n_nexp,
    or_to_cor = or_to_cor, or_to_rr = or_to_rr,
    reverse_or = reverse_or
  )

  es$info_used <- "or"
  return(es)
}

#' Convert an odds ratio value and its 95% confidence interval to several effect size measures
#'
#' @param or odds ratio value
#' @param logor log odds ratio value
#' @param or_ci_lo lower bound of the 95% CI around the odds ratio value
#' @param or_ci_up upper bound of the 95% CI around the odds ratio value
#' @param logor_ci_lo lower bound of the 95% CI around the log odds ratio value
#' @param logor_ci_up upper bound of the 95% CI around the log odds ratio value
#' @param n_cases number of cases/events
#' @param n_controls number of controls/no-event
#' @param n_exp number of participants in the exposed group (only required for the \code{or_to_rr = "grant"}, and \code{or_to_rr = "metaumbrella_exp"} arguments)
#' @param n_nexp number of participants in the non-exposed group (only required for the \code{or_to_rr = "grant"}, and \code{or_to_rr = "metaumbrella_exp"} arguments)
#' @param n_sample total number of participants in the sample
#' @param baseline_risk proportion of cases in the non-exposed group (only required for the \code{or_to_rr = "grant"} argument).
#' @param small_margin_prop smallest margin proportion of the underlying 2x2 table
#' @param reverse_or a logical value indicating whether the direction of the generated effect sizes should be flipped.
#' @param or_to_cor formula used to convert the \code{or} value into a correlation coefficient (see details).
#' @param or_to_rr formula used to convert the \code{or} value into a risk ratio (see details).
#' @param max_asymmetry A percentage indicating the tolerance before detecting asymmetry in the 95% CI bounds.
#'
#' @details
#' This function computes the standard error of the (log) odds ratio
#' into a standard error (Section 6.5.2.2 in the Cochrane Handbook).
#' \deqn{logor\_se = \frac{\log{or\_ci\_up} - \log{or\_ci\_lo}}{2 * qnorm(.975)}}
#'
#' Then, calculations of \code{\link{es_from_or_se}} are applied.
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @export es_from_or_ci
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab RR + NNT\cr
#'  \code{} \tab D + G + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 2. Odds Ratio'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_or <- es_from_or_ci(
#'   or = 1, or_ci_lo = 0.5, or_ci_up = 2,
#'   n_cases = 42, n_controls = 38, baseline_risk = 0.08,
#'   or_to_rr = "grant"
#' )
es_from_or_ci <- function(or, or_ci_lo, or_ci_up, logor, logor_ci_lo, logor_ci_up,
                          baseline_risk, small_margin_prop, n_exp, n_nexp,
                          n_cases, n_controls, n_sample, max_asymmetry = 10,
                          or_to_cor = "bonett", or_to_rr = "metaumbrella_cases",
                          reverse_or) {
  if (missing(or)) {
    or <- rep(NA_real_, length(logor))
  }
  if (missing(logor)) {
    logor <- rep(NA_real_, length(or))
  }
  if (missing(baseline_risk)) {
    baseline_risk <- rep(NA_real_, length(or))
  }
  if (missing(small_margin_prop)) {
    small_margin_prop <- rep(NA_real_, length(or))
  }
  if (missing(n_exp)) {
    n_exp <- rep(NA_real_, length(or))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA_real_, length(or))
  }
  if (missing(n_cases)) {
    n_cases <- rep(NA_real_, length(or))
  }
  if (missing(n_controls)) {
    n_controls <- rep(NA_real_, length(or))
  }
  if (missing(or_ci_lo)) {
    or_ci_lo <- rep(NA_real_, length(or))
  }
  if (missing(logor_ci_lo)) {
    logor_ci_lo <- rep(NA_real_, length(or))
  }
  if (missing(or_ci_up)) {
    or_ci_up <- rep(NA_real_, length(or))
  }
  if (missing(logor_ci_up)) {
    logor_ci_up <- rep(NA_real_, length(or))
  }
  if (missing(reverse_or)) {
    reverse_or <- rep(FALSE, length(or))
  }
  if (missing(n_sample)) {
    n_sample <- rep(NA_real_, length(or))
  }
  reverse_or[is.na(reverse_or)] <- FALSE

  or <- ifelse(is.na(or) & !is.na(logor), exp(logor), or)


  logor_ci_lo <- ifelse(is.na(logor_ci_lo) & !is.na(or_ci_lo), log(or_ci_lo), logor_ci_lo)
  logor_ci_up <- ifelse(is.na(logor_ci_up) & !is.na(or_ci_up), log(or_ci_up), logor_ci_up)
  logor_se <- (logor_ci_up - logor_ci_lo) / (2 * qnorm(.975))

  es <- es_from_or_se(
    or = or, logor_se = logor_se,
    n_cases = n_cases, n_controls = n_controls,
    baseline_risk = baseline_risk, small_margin_prop = small_margin_prop,
    n_exp = n_exp, n_nexp = n_nexp, n_sample = n_sample,
    or_to_cor = or_to_cor, or_to_rr = or_to_rr,
    reverse_or = reverse_or
  )

  es$info_used <- "or_ci"

  return(es)
}

#' Convert an odds ratio value and its standard error to several effect size measures
#'
#' @param or odds ratio value
#' @param logor log odds ratio value
#' @param or_pval p-value of the (log) odds ratio
#' @param n_cases number of cases/events
#' @param n_controls number of controls/no-event
#' @param n_exp number of participants in the exposed group (only required for the \code{or_to_rr = "grant"}, and \code{or_to_rr = "metaumbrella_exp"} arguments)
#' @param n_nexp number of participants in the non-exposed group (only required for the \code{or_to_rr = "grant"}, and \code{or_to_rr = "metaumbrella_exp"} arguments)
#' @param n_sample total number of participants in the sample
#' @param baseline_risk proportion of cases in the non-exposed group (only required for the \code{or_to_rr = "grant"} argument).
#' @param small_margin_prop smallest margin proportion of the underlying 2x2 table
#' @param reverse_or_pval a logical value indicating whether the direction of the generated effect sizes should be flipped.
#' @param or_to_cor formula used to convert the \code{or} value into a correlation coefficient (see details).
#' @param or_to_rr formula used to convert the \code{or} value into a risk ratio (see details).
#'
#' @details
#' This function computes the standard error of the (log) odds ratio into
#' from a p-value (Section 6.3.2 in the Cochrane Handbook).
#' \deqn{logor\_z = qnorm(or_pval/2, lower.tail=FALSE)}
#' \deqn{logor\_se = |\frac{\log(or)}{logor\_z}|}
#'
#' Then, calculations of \code{\link{es_from_or_se}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab RR + NNT\cr
#'  \code{} \tab D + G + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 2. Odds Ratio'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_or_pval
#'
#' @references
#' Higgins, J. P., Thomas, J., Chandler, J., Cumpston, M., Li, T., Page, M. J., & Welch, V. A. (Eds.). (2019). Cochrane handbook for systematic reviews of interventions. John Wiley & Sons.
#'
#' @md
#'
#' @examples
#' es_or <- es_from_or_pval(
#'   or = 3.51, or_pval = 0.001,
#'   n_cases = 12, n_controls = 68
#' )
es_from_or_pval <- function(or, logor, or_pval, baseline_risk, small_margin_prop,
                            n_exp, n_nexp, n_cases, n_controls, n_sample,
                            or_to_rr = "metaumbrella_cases",
                            or_to_cor = "bonett", reverse_or_pval) {
  if (missing(or)) {
    or <- rep(NA_real_, length(logor))
  }
  if (missing(logor)) {
    logor <- rep(NA_real_, length(or))
  }
  if (missing(baseline_risk)) {
    baseline_risk <- rep(NA_real_, length(or))
  }
  if (missing(small_margin_prop)) {
    small_margin_prop <- rep(NA_real_, length(or))
  }
  if (missing(n_exp)) {
    n_exp <- rep(NA_real_, length(or))
  }
  if (missing(n_nexp)) {
    n_nexp <- rep(NA_real_, length(or))
  }
  if (missing(n_cases)) {
    n_cases <- rep(NA_real_, length(or))
  }
  if (missing(n_controls)) {
    n_controls <- rep(NA_real_, length(or))
  }
  if (missing(reverse_or_pval)) {
    reverse_or_pval <- rep(FALSE, length(or))
  }
  if (missing(n_sample)) {
    n_sample <- rep(NA_real_, length(or))
  }
  reverse_or_pval[is.na(reverse_or_pval)] <- FALSE

  or <- ifelse(is.na(or) & !is.na(logor), exp(logor), or)
  logOR <- suppressWarnings(log(or))

  z_or <- qnorm(or_pval / 2, lower.tail = FALSE)
  logor_se <- abs(logOR / z_or)

  es <- es_from_or_se(
    or = or, logor_se = logor_se,
    baseline_risk = baseline_risk, small_margin_prop = small_margin_prop,
    n_exp = n_exp, n_nexp = n_nexp, n_sample = n_sample,
    n_cases = n_cases, n_controls = n_controls,
    or_to_cor = or_to_cor, or_to_rr = or_to_rr,
    reverse_or = reverse_or_pval
  )

  es$info_used <- "or_pval"

  return(es)
}

#' Convert an odds ratio or risk ratio and a Wald t-statistic from a regression model into several effect size measures
#'
#' @param or odds ratio value (from logistic regression)
#' @param logor log odds ratio value
#' @param rr risk ratio value (from log-binomial or modified Poisson regression)
#' @param logrr log risk ratio value
#' @param logreg_t a t-statistic (Wald statistic) from a logistic, log-binomial, or modified Poisson regression model
#' @param n_cases number of cases/events across exposed/non-exposed groups
#' @param n_controls number of controls/no-event across exposed/non-exposed groups
#' @param n_exp number of participants in the exposed group
#' @param n_nexp number of participants in the non-exposed group
#' @param n_sample total number of participants in the sample
#' @param baseline_risk proportion of cases in the non-exposed group
#' @param small_margin_prop smallest margin proportion of cases/events in the underlying 2x2 table
#' @param reverse_logreg_t a logical value indicating whether the direction of the generated effect sizes should be flipped.
#' @param or_to_rr formula used to convert the \code{or} value into a risk ratio (see details).
#' @param or_to_cor formula used to convert the \code{or} value into a correlation coefficient (see details).
#' @param rr_to_or formula used to convert the \code{rr} value into an odds ratio (see details).
#'
#' @details
#' This function derives the standard error of the log odds ratio (or log risk ratio) from a
#' Wald t-statistic reported in a regression model.
#'
#' **To estimate the standard error of the log OR (or log RR)**, the formulas used are:
#' \deqn{t = \frac{\beta}{SE(\beta)}}
#' \deqn{SE(\beta) = \frac{|\beta|}{|t|}}
#' where \eqn{\beta} is \eqn{\log(OR)} or \eqn{\log(RR)} depending on the model.
#'
#' Then, if an OR (or logOR) is entered, calculations of \code{\link{es_from_or_se}()} are applied.
#' If a RR (or logRR) is entered, calculations of \code{\link{es_from_rr_se}()} are applied.
#'
#' Note that the standardized-mean-difference and correlation conversions (D, G, R, Z) are
#' produced for **OR inputs only**. RR inputs are treated as a ratio measure and yield
#' RR + OR + NNT + RD: RR is not converted to a standardized mean difference or correlation,
#' because that would require going through the OR and the baseline risk (see
#' \code{\link{es_from_rr_se}}). To obtain a SMD or correlation from an RR, convert it to an
#' OR first (supplying the baseline risk) and then use the OR path.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR + RR \cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR inputs: D + G + R + Z (+ RR + NNT + RD); RR inputs: OR + NNT + RD \cr
#' }
#'
#' @references
#' Sanchez-Meca, J., Marin-Martinez, F., & Chacon-Moscoso, S. (2003). Effect-size indices for
#' dichotomized outcomes in meta-analysis. \emph{Psychological Methods}, 8(4), 448--467.
#'
#' @md
#'
#' @export es_from_logreg_t
#'
#' @examples
#' es_or <- es_from_logreg_t(
#'   or = 2.12, logreg_t = 3.21,
#'   n_cases = 50, n_controls = 150
#' )
#'
#' es_rr <- es_from_logreg_t(
#'   rr = 1.5, logreg_t = 2.8,
#'   n_exp = 100, n_nexp = 100
#' )
es_from_logreg_t <- function(or, logor, rr, logrr, logreg_t,
                         baseline_risk, small_margin_prop,
                         n_exp, n_nexp, n_cases, n_controls, n_sample,
                         or_to_rr = "metaumbrella_cases",
                         or_to_cor = "bonett",
                         rr_to_or = "metaumbrella",
                         reverse_logreg_t) {

  len <- if (!missing(or)) length(or) else if (!missing(logor)) length(logor) else if (!missing(rr)) length(rr) else if (!missing(logrr)) length(logrr) else length(logreg_t)

  if (missing(or)) or <- rep(NA_real_, len)
  if (missing(logor)) logor <- rep(NA_real_, len)
  if (missing(rr)) rr <- rep(NA_real_, len)
  if (missing(logrr)) logrr <- rep(NA_real_, len)
  if (missing(baseline_risk)) baseline_risk <- rep(NA_real_, len)
  if (missing(small_margin_prop)) small_margin_prop <- rep(NA_real_, len)
  if (missing(n_exp)) n_exp <- rep(NA_real_, len)
  if (missing(n_nexp)) n_nexp <- rep(NA_real_, len)
  if (missing(n_cases)) n_cases <- rep(NA_real_, len)
  if (missing(n_controls)) n_controls <- rep(NA_real_, len)
  if (missing(n_sample)) n_sample <- rep(NA_real_, len)
  if (missing(logreg_t)) logreg_t <- rep(NA_real_, len)
  if (missing(reverse_logreg_t)) reverse_logreg_t <- rep(FALSE, len)
  reverse_logreg_t[is.na(reverse_logreg_t)] <- FALSE

  has_or <- !is.na(or) | !is.na(logor)
  has_rr <- !is.na(rr) | !is.na(logrr)

  # OR -------
  or <- ifelse(is.na(or) & !is.na(logor), exp(logor), or)
  logOR <- suppressWarnings(log(or))
  logor_se <- abs(logOR / logreg_t)

  es_or <- es_from_or_se(
    or = or, logor_se = logor_se,
    baseline_risk = baseline_risk, small_margin_prop = small_margin_prop,
    n_exp = n_exp, n_nexp = n_nexp, n_sample = n_sample,
    n_cases = n_cases, n_controls = n_controls,
    or_to_cor = or_to_cor, or_to_rr = or_to_rr,
    reverse_or = reverse_logreg_t
  )

  # RR -------
  rr <- ifelse(is.na(rr) & !is.na(logrr), exp(logrr), rr)
  logRR <- suppressWarnings(log(rr))
  logrr_se <- abs(logRR / logreg_t)

  es_rr <- es_from_rr_se(
    rr = rr, logrr_se = logrr_se,
    baseline_risk = baseline_risk,
    n_exp = n_exp, n_nexp = n_nexp,
    n_cases = n_cases, n_controls = n_controls,
    rr_to_or = rr_to_or,
    reverse_rr = reverse_logreg_t
  )

  use_rr <- has_rr & !has_or

  if (!any(use_rr)) {
    es_or$info_used <- "logreg_t"
    return(es_or)
  } else if (all(use_rr)) {
    es_rr$info_used <- "logreg_t"
    return(es_rr)
  }

  all_cols <- union(names(es_or), names(es_rr))
  es <- data.frame(matrix(NA_real_, nrow = len, ncol = 0))
  for (col in all_cols) {
    if (col == "info_used") next
    or_val <- if (col %in% names(es_or)) es_or[[col]] else rep(NA_real_, len)
    rr_val <- if (col %in% names(es_rr)) es_rr[[col]] else rep(NA_real_, len)
    es[[col]] <- ifelse(use_rr, rr_val, or_val)
  }
  es$info_used <- "logreg_t"

  return(es)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
library(metaConvert)
library(DT)

## ----eval = FALSE-------------------------------------------------------------
# see_input_data(measure = "or")

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(see_input_data(measure = "or", type_of_measure = "natural", extension="data.frame"), options = list(  
    scrollX = TRUE,
    dom = c('t'),
    scrollY = "300px", 
    pageLength = 500,
    ordering = FALSE,
    rownames = FALSE,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center', 
                                     targets = "_all"))))

## ----eval = FALSE-------------------------------------------------------------
# data_extraction_sheet(measure = "or")

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(data_extraction_sheet(measure = "or", extension="data.frame"), options = list(  
    scrollX = TRUE,
    dom = c('t'),
   ordering = FALSE,
    rownames = FALSE,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center', 
                                     targets = "_all"))))

## ----message=FALSE, fig.width= 11---------------------------------------------
compare_df(
    df_extractor_1 = df.compare1,
    df_extractor_2 = df.compare2,
    output = "html")

## ----message=FALSE, fig.width= 11---------------------------------------------
compare_df(
    df_extractor_1 = df.compare1,
    df_extractor_2 = df.compare2,
    ordering_columns = c("author", "year"),
    output = "html")

## ----message=FALSE------------------------------------------------------------
res = convert_df(x = df.short, measure = "g")

## ----eval= FALSE--------------------------------------------------------------
# summary(res)

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(summary(res), options = list(  
    scrollX = TRUE,
    dom = c('t'),
    ordering = FALSE,
    rownames = FALSE,
    scrollY = "300px", 
    pageLength = 100,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center', 
                                     targets = "_all"))))

## ----message=FALSE------------------------------------------------------------
res_rd <- convert_df(df.haza, measure = "rd")
head(summary(res_rd)[, c("row_id", "es_crude", "se_crude",
                         "es_ci_lo_crude", "es_ci_up_crude", "info_used_crude")])

## ----message=FALSE------------------------------------------------------------
res_flags <- convert_df(df.short, measure = "g")
s_flags <- summary(res_flags, flags = TRUE, guidance = TRUE)
head(s_flags[, c("row_id", "es_crude", "info_used_crude",
                 "flags_crude", "es_guidance_crude")])

## ----message=FALSE------------------------------------------------------------
# Observed correlation with known scale reliabilities
es_disattenuate(r = 0.45, r_se = 0.08,
                reliability_x = 0.82, reliability_y = 0.78)

## -----------------------------------------------------------------------------
sem_res <- compute_sem(sd = 10, icc = 0.85, n_sample = 100)
sdc_res <- compute_sdc(sem = sem_res$sem, sem_se = sem_res$sem_se)
sem_res
sdc_res

#' Convert single-group proportion into an effect size measure
#'
#' @param prop proportion of cases in the single group (0-1)
#' @param n_sample total sample size
#' @param prop_to_es method used to compute the effect size from the proportion.
#'   Must be \code{"raw"} (default), \code{"logit"}, or \code{"freeman_tukey"}.
#' @param reverse_prop a logical value indicating whether the direction of the proportion should be flipped.
#'
#' @details
#' This function computes an effect size from a single-group proportion.
#'
#' 1. **When \code{prop_to_es = "raw"}** (default), the proportion is used as the effect size
#' and its standard error is:
#' \deqn{prop\_se = \sqrt{\frac{prop \times (1 - prop)}{n}}}
#'
#' 2. **When \code{prop_to_es = "logit"}**, the proportion is converted to a log odds:
#' \deqn{logit = \log\left(\frac{prop}{1 - prop}\right)}
#' \deqn{logit\_se = \sqrt{\frac{1}{n \times prop \times (1 - prop)}}}
#'
#' 3. **When \code{prop_to_es = "freeman_tukey"}**, the Freeman-Tukey double arcsine
#' transformation is applied (Barendregt et al., 2013):
#' \deqn{FT = \frac{1}{2}\left[\arcsin\left(\sqrt{\frac{x}{n+1}}\right) + \arcsin\left(\sqrt{\frac{x+1}{n+1}}\right)\right]}
#' with \eqn{x = n \times prop}, and:
#' \deqn{FT\_se = \frac{1}{\sqrt{4n + 2}}}
#'
#' When prop is equal to 0 or 1, a 0.5 correction is applied for the raw and logit methods:
#' \eqn{prop\_corrected = \frac{x + 0.5}{n + 1}}. For the raw method this shifts the
#' reported point estimate itself (a boundary proportion of exactly 0 or 1 is
#' returned as \eqn{(x + 0.5)/(n + 1)}, i.e. nudged toward the interior), matching
#' the convention of \code{metafor}'s \code{measure = "PR"}. The corrected
#' denominator \eqn{n + 1} also replaces \eqn{n} in the raw and logit standard
#' errors for these boundary rows (i.e.,
#' \eqn{\sqrt{p_c (1 - p_c) / (n + 1)}} and \eqn{\sqrt{1 / ((n + 1) \times p_c \times (1 - p_c))}}),
#' matching \code{metafor}'s \code{"PR"}/\code{"PLO"} convention throughout.
#'
#' Proportions outside \eqn{[0, 1]} are set to NA with a warning (the affected
#' rows return NA, but do not abort the run).
#'
#' @references
#' Barendregt, J. J., Doi, S. A., Lee, Y. Y., Norman, R. E., & Vos, T. (2013).
#' Meta-analysis of prevalence. Journal of Epidemiology and Community Health, 67(11), 974-978.
#'
#' Miller, J. J. (1978). The inverse of the Freeman-Tukey double arcsine transformation.
#' The American Statistician, 32(4), 138-138. (Back-transformation left to the
#' user; not applied here: Freeman-Tukey results stay on the transformed scale.)
#'
#' @return
#' This function estimates a single-group proportion.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab PROP\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab N/A\cr
#'  \tab \cr
#'  \code{required input data} \tab prop + n_sample\cr
#'  \tab \cr
#' }
#'
#' Note: when \code{prop_to_es = "logit"} or \code{"freeman_tukey"}, the returned
#' \code{prop}, \code{prop_se}, \code{prop_ci_lo} and \code{prop_ci_up} columns are
#' on the transformed (log-odds / Freeman-Tukey double-arcsine) scale, \strong{not}
#' the `[0, 1]` proportion scale, and are not back-transformed. Only
#' \code{prop_to_es = "raw"} returns values on the proportion scale.
#'
#' @export es_from_prop_single_group
#'
#' @md
#'
#' @examples
#' es_from_prop_single_group(prop = 0.30, n_sample = 100)
es_from_prop_single_group <- function(prop, n_sample, prop_to_es = "raw", reverse_prop) {
  if (missing(reverse_prop)) reverse_prop <- rep(FALSE, length(prop))
  reverse_prop[is.na(reverse_prop)] <- FALSE
  if (length(reverse_prop) == 1) reverse_prop <- rep(reverse_prop, length(prop))
  if (length(reverse_prop) != length(prop)) {
    stop("The length of the 'reverse_prop' argument is incorrectly specified.")
  }

  if (!prop_to_es %in% c("raw", "logit", "freeman_tukey")) {
    stop(paste0("'", prop_to_es, "' not in tolerated values for the 'prop_to_es' argument. ",
                "Possible inputs are: 'raw', 'logit', 'freeman_tukey'"))
  }

  # P9: per-row NA + warning instead of a hard stop. convert_df() calls this
  # function for every measure with no tryCatch, so a stop() here would abort
  # an entire pipeline run (including unrelated measures) because of one bad
  # cell. Tier-1 validation reports the issue in both correct_inputs modes.
  out_of_range <- !is.na(prop) & (prop < 0 | prop > 1)
  if (any(out_of_range)) {
    warning(sprintf(
      "es_from_prop_single_group: %d row(s) had a proportion outside [0, 1]; these proportions were set to NA.",
      sum(out_of_range)
    ), call. = FALSE)
    prop[out_of_range] <- NA_real_
  }

  nn_miss <- which(!is.na(prop) & !is.na(n_sample))

  n <- length(prop)
  prop_es <- rep(NA_real_, n)
  prop_es_se <- rep(NA_real_, n)

  if (length(nn_miss) != 0) {
    p <- prop[nn_miss]
    ns <- n_sample[nn_miss]

    p <- ifelse(reverse_prop[nn_miss], 1 - p, p)

    # correcting by 0.5 when prop = 0 or 1
    n_cases <- p * ns
    needs_correction <- (p == 0 | p == 1)
    p_corrected <- p
    p_corrected[needs_correction] <- (n_cases[needs_correction] + 0.5) / (ns[needs_correction] + 1)
    # P5: the continuity-corrected denominator n + 1 must also flow into the
    # SE for boundary rows (metafor PR/PLO convention: vi = 1/x_c + 1/(n_c - x_c)
    # with x_c = x + 0.5, n_c = n + 1); interior rows keep the raw n.
    ns_corrected <- ns
    ns_corrected[needs_correction] <- ns[needs_correction] + 1

    if (prop_to_es == "raw") {
      prop_es[nn_miss] <- p_corrected
      prop_es_se[nn_miss] <- sqrt(p_corrected * (1 - p_corrected) / ns_corrected)

    } else if (prop_to_es == "logit") {
      prop_es[nn_miss] <- log(p_corrected / (1 - p_corrected))
      prop_es_se[nn_miss] <- sqrt(1 / (ns_corrected * p_corrected * (1 - p_corrected)))

    } else if (prop_to_es == "freeman_tukey") {
      # no correction for FT
      x_ft <- ns * p
      prop_es[nn_miss] <- 0.5 * (asin(sqrt(x_ft / (ns + 1))) +
                                  asin(sqrt((x_ft + 1) / (ns + 1))))
      prop_es_se[nn_miss] <- 1 / sqrt(4 * ns + 2)
    }
  }

  # CIs
  prop_ci_lo <- prop_es - qnorm(0.975) * prop_es_se
  prop_ci_up <- prop_es + qnorm(0.975) * prop_es_se

  # bound to [0,1]
  if (prop_to_es == "raw") {
    prop_ci_lo <- pmax(0, prop_ci_lo)
    prop_ci_up <- pmin(1, prop_ci_up)
  }

  result <- data.frame(
    prop = prop_es,
    prop_se = prop_es_se,
    prop_ci_lo = prop_ci_lo,
    prop_ci_up = prop_ci_up,
    n_sample = n_sample,
    info_used = "prop_single_group"
  )

  return(result)
}


#' Convert single-group case counts into a proportion effect size measure
#'
#' @param n_cases number of cases in the single group
#' @param n_sample total sample size
#' @param prop_to_es method used to compute the effect size from the proportion.
#'   Must be \code{"raw"} (default), \code{"logit"}, or \code{"freeman_tukey"}.
#' @param reverse_prop a logical value indicating whether the direction of the proportion should be flipped.
#'
#' @details
#' This is a convenience function that converts case counts to proportions
#' and then calls \code{\link{es_from_prop_single_group}()}.
#'
#' The proportion is calculated as:
#' \deqn{prop = \frac{n\_cases}{n\_sample}}
#'
#' Then, calculations of the \code{\link{es_from_prop_single_group}()} are applied.
#'
#' Rows where \code{n_cases > n_sample} are set to NA (both values) with a
#' warning; the affected rows return NA, but do not abort the run.
#'
#' @references
#' Barendregt, J. J., Doi, S. A., Lee, Y. Y., Norman, R. E., & Vos, T. (2013).
#' Meta-analysis of prevalence. Journal of Epidemiology and Community Health, 67(11), 974-978.
#'
#' @return
#' This function returns the same output as \code{\link{es_from_prop_single_group}()}.
#' See that function's documentation for details.
#'
#' @export es_from_prop_single_group_counts
#'
#' @md
#'
#' @examples
#' es_from_prop_single_group_counts(n_cases = 25, n_sample = 100)
es_from_prop_single_group_counts <- function(n_cases, n_sample, prop_to_es = "raw", reverse_prop) {
  if (missing(reverse_prop)) reverse_prop <- rep(FALSE, length(n_cases))
  reverse_prop[is.na(reverse_prop)] <- FALSE

  # P9: per-row NA + warning instead of a hard stop (see
  # es_from_prop_single_group). Both members of an inconsistent pair are set
  # to NA, mirroring the Tier-1 part-total convention.
  inconsistent <- !is.na(n_cases) & !is.na(n_sample) & n_cases > n_sample
  if (any(inconsistent)) {
    warning(sprintf(
      "es_from_prop_single_group_counts: %d row(s) had n_cases > n_sample; both values were set to NA for these rows.",
      sum(inconsistent)
    ), call. = FALSE)
    n_cases[inconsistent] <- NA_real_
    n_sample[inconsistent] <- NA_real_
  }

  prop <- n_cases / n_sample

  result <- es_from_prop_single_group(prop = prop, n_sample = n_sample,
                                      prop_to_es = prop_to_es, reverse_prop = reverse_prop)

  result$info_used <- "prop_single_group_counts"

  return(result)
}

#' Data extraction sheet generator
#'
#' @param name Name of the file created
#' @param measure Target effect size measure (one of the 14 available in metaConvert). Default is "all".
#' @param extension Extension of the file created. Most common are ".xlsx", ".csv" or ".txt". It is also possible to generate an R dataframe object by using the "data.frame" extension.
#' @param type_of_measure One of "natural+converted" or "natural" (see details).
#' @param verbose logical variable indicating whether some information should be printed (e.g., the location where the sheet is created when using ".xlsx", ".csv" or ".txt" extensions)
#'
#' @details
#' This function generates, on your computer, a data extraction sheet that contains the name of columns
#' that can be used by our tools to estimate various effect size measures.
#'
#' If you select a specific measure (e.g., \code{measure = "g"}), you will be presented only with most common
#' information allowing to estimate this measure (e.g., you will not be provided with columns for contingency
#' tables if you request a data extraction sheet for \code{measure = "g"}).
#'
#'
#' ## Measure
#' You can specify a specific effect size measures (among those available in the \code{\link{metaConvert-package}}).
#' Doing this, the data extraction sheet will contain only the columns of the input data allowing **a natural estimation
#' of the effect size measure**. For example, if you request \code{measure="d"} the data extraction
#' sheet will not contain the columns for the contingency table since, although the \code{\link{convert_df}}
#' function allows you to **convert** a contingency table into a "d", this requires to convert the "OR"
#' that is naturally estimated from the contingency table into a "d".
#'
#'
#' This table is designed to be used in combination with tables showing the combination of input data
#' leading to estimate each of the effect size measures (https://metaconvert.org/html/input.html)
#'
#' ## Extension
#' You can export a file in various formats outside R
#' (by indicating, for example, ".txt", ".xlsx", or ".csv") in the \code{extension} argument.
#' You can also visualise this dataset directly in R by setting \code{extension = "data.frame"}.
#'
#' @return
#' This function returns a data extraction sheet that contains all the information
#' necessary to estimate any effect size using the metaConvert tools.
#'
#' @md
#'
#' @export data_extraction_sheet
#'
#' @examples
#' data_extraction_sheet(measure = "md", extension = "data.frame")
data_extraction_sheet <- function(measure = c("d", "g", "md", "dw", "gw", "mdw",
                                              "or", "rr", "nnt", "rd",
                                              "r", "z", "logvr", "logcvr", "irr",
                                              "prop"),
                                  type_of_measure = c("natural", "natural+converted"),
                                  name = "mcv_data_extraction",
                                  extension = c("data.frame", ".txt", ".csv", ".xlsx"),
                                  verbose = TRUE
                                  ) {
  type_of_measure = type_of_measure[1]
  extension = extension[1]
  measure = measure[1]
  cols_req = c("study_id", "author", "year", "predictor", "outcome", "all_info_expected")
  inf_req = c("unique ID for each study - numeric/character",
              "first author name - numeric/character",
              "year of publication - numeric/character",
              "intervention/exposure - numeric/character",
              "outcome - numeric/character",
              "expected input data leading to an effect size estimate - character")

  cols_sample = "n_sample"
  inf_sample = c("total number of participants - numeric")

  cols_exposed = c("n_exp", "n_nexp")
  inf_exposed = c("number of participants in the exposed/experimental group - numeric",
                  "number of participants in the non-exposed/non-experimental group - numeric")

  cols_cases = c("n_cases", "n_controls")
  inf_cases = c("number of cases/events across exposed/non-exposed groups - numeric",
                "number of controls/no-event across exposed/non-exposed groups - numeric")

  cols_input_crude = c("user_es_original_measure_crude",
    "user_es_target_measure_crude",
    "user_es_crude",
    "user_se_crude",
    "user_ci_lo_crude",
    "user_ci_up_crude")
  inf_input_crude = c("type of ES provided: d, g, or, logor, rr, logrr, r, z, rd, etc. - character",
                      "target ES for conversion (defaults to measure param) - character",
                      "value of an effect size - numeric",
                      "standard error of the effect size - numeric",
                      "lower bound of the 95% CI of the effect size measure - numeric",
                      "upper bound of the 95% CI of the effect size measure - numeric")

  cols_input_adjusted = c(
    "user_es_original_measure_adj",
    "user_es_target_measure_adj",
    "user_es_adj",
    "user_se_adj",
    "user_ci_lo_adj",
    "user_ci_up_adj")
  inf_input_adjusted = c("type of adjusted ES provided: d, g, or, logor, rr, logrr, r, z, rd, etc. - character",
                      "target ES for conversion (defaults to measure param) - character",
                      "value of the adjusted effect size - numeric",
                      "standard error of the effect size - numeric",
                      "adjusted lower bound of the 95% CI of the effect size measure - numeric",
                      "adjusted upper bound of the 95% CI of the effect size measure - numeric")

  cols_means_post = c(
    # means post
    "reverse_means",
    "mean_exp", "mean_sd_exp", "mean_se_exp", "mean_ci_lo_exp", "mean_ci_up_exp",
    "mean_nexp", "mean_sd_nexp", "mean_se_nexp", "mean_ci_lo_nexp", "mean_ci_up_nexp",
    "mean_sd_pooled",

    # plot
    "reverse_plot_means",
    "plot_mean_exp", "plot_mean_nexp",
    "plot_mean_sd_lo_exp", "plot_mean_sd_lo_nexp",
    "plot_mean_sd_up_exp", "plot_mean_sd_up_nexp",
    "plot_mean_se_lo_exp", "plot_mean_se_lo_nexp",
    "plot_mean_se_up_exp", "plot_mean_se_up_nexp",
    "plot_mean_ci_lo_exp", "plot_mean_ci_lo_nexp",
    "plot_mean_ci_up_exp", "plot_mean_ci_up_nexp",
    "reverse_means_variability")

  # md
  cols_med = c(    # quarts
    "reverse_med", "min_exp", "q1_exp", "med_exp", "q3_exp", "max_exp",
    "min_nexp", "q1_nexp", "med_nexp", "q3_nexp", "max_nexp"
  )
  inf_med = c(
    "whether the direction of the effect size generated from the medians/quartiles/ranges should be flipped - logical",
    "minimum value of the experimental/exposed group - numeric",
    "first quartile of the experimental/exposed group - numeric",
    "median value of the experimental/exposed group - numeric",
    "third quartile of the experimental/exposed group - numeric",
    "maximum value of the experimental/exposed group - numeric",
    "minimum value of the non-experimental/non-exposed group - numeric",
    "first quartile of the non-experimental/non-exposed group - numeric",
    "median value of the non-experimental/non-exposed group - numeric",
    "third quartile of the non-experimental/non-exposed group - numeric",
    "maximum value of the non-experimental/non-exposed  group - numeric"
  )
  # means post
  inf_means_post = c("whether the direction of the effect size generated from the means should be flipped - logical",
    "mean of participants in the experimental/exposed group - numeric",
    "standard deviation of participants in the experimental/exposed group - numeric",
    "standard error of participants in the experimental/exposed group - numeric",
    "lower bound of the 95% CI of the mean of the experimental/exposed group - numeric",
    "upper bound of the 95% CI of the mean of the experimental/exposed group - numeric",
    "mean of participants in the non-experimental/non-exposed group - numeric",
    "standard deviation of participants in the non-experimental/non-exposed group - numeric",
    "standard error of participants in the non-experimental/non-exposed group - numeric",
    "lower bound of the 95% CI of the mean of the non-experimental/non-exposed group - numeric",
    "upper bound of the 95% CI of the mean of the non-experimental/non-exposed group - numeric",
    "pooled standard deviation across both groups - numeric",

    # plot
    "whether the direction of the effect size generated from the means extracted from a plot should be flipped - logical",

    "mean of participants in the experimental/exposed group (extracted from a plot) - numeric",
    "mean of participants in the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting -1 SD from the mean of the experimental/exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting -1 SD from the mean of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting +1 SD from the mean of the experimental/exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting +1 SD from the mean of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting -1 SE from the mean of the experimental/exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting -1 SE from the mean of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting +1 SE from the mean of the experimental/exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting +1 SE from the mean of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting the 95% CI of the mean of the experimental/exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting the 95% CI of the mean of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting the 95% CI of the mean of the experimental/exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting the 95% CI of the mean of the non-experimental/non-exposed group (extracted from a plot) - numeric",

    "whether the direction of the effect size generated from the means variability should be flipped - logical")

  cols_md =  c(
    "reverse_md", "md", "md_sd", "md_se", "md_ci_lo", "md_ci_up",
   "md_pval")

  inf_md = c("whether the direction of the effect size generated from the mean difference should be flipped - logical",
  "mean difference - numeric",
  "standard deviation of the mean difference - numeric",
  "standard error of the mean difference - numeric",
  "lower bound of the 95% CI of the mean difference - numeric",
  "upper bound of the 95% CI of the mean difference - numeric",
  "p-value of the mean difference - numeric")

  cols_anova = c(
    # smd/g
    "reverse_d", "cohen_d", "reverse_g", "hedges_g",
    # t f eta
    "reverse_student_t", "student_t",
    "reverse_student_t_pval", "student_t_pval",
    "reverse_anova_f", "anova_f",
    "reverse_anova_f_pval", "anova_f_pval",
    "reverse_etasq", "etasq",
    "reverse_pt_bis_r", "pt_bis_r", "reverse_pt_bis_r_pval", "pt_bis_r_pval")

  inf_anova = c(
    # smd/g
    "whether the direction of the cohen_d value should be flipped - logical",
    "value of the Cohen's d - numeric",
    "whether the direction of the hedges_g value should be flipped - logical",
    "value of the Hedges' g - numeric",

    # t f
    "whether the direction of the effect size generated from the Student's t-test should be flipped - logical",
    "Student's t-test value - numeric",
    "whether the direction of the effect size generated from the Student's t-test p-value should be flipped - logical",
    "two-tailed p-value of a Student's t-test - numeric",
    "whether the direction of the effect size generated from the ANOVA F should be flipped - logical",
    "ANOVA F-test value - numeric",
    "whether the direction of the effect size generated from the ANOVA p-value should be flipped - logical",
    "two-tailed p-value of an ANOVA - numeric",
    "whether the direction of the effect size generated from the eta-squared should be flipped - logical",
    "eta-squared value - numeric",
    "whether the direction of the effect size generated from the point-biserial correlation should be flipped - logical",
    "point-biserial correlation coefficient value - numeric",
    "whether the direction of the effect size generated from the point-biserial correlation p-value should be flipped - logical",
    "p-value of a point-biserial correlation - numeric")

  cols_regression = c("reverse_beta_std", "beta_std", "reverse_beta_unstd", "beta_unstd", "sd_dv")
  inf_regression = c("whether the direction of the effect size generated from the standardized beta should be flipped - logical",
    "a standardized regression coefficient value (the predictor of interest must be binary) - numeric",
    "whether the direction of the effect size generated from the unstandardized beta should be flipped - logical",
    "a non-standardized regression coefficient value (the predictor of interest must be binary) - numeric",
    "standard deviation of the dependent variable (not the standard deviation of the regression coefficient) - numeric")

  cols_linreg = c("reverse_linreg_t", "linreg_t", "n_covariates")
  inf_linreg = c(
    "whether the direction of the effect size generated from the regression t should be flipped - logical",
    "t-statistic from a linear regression model - numeric",
    "number of covariates in the regression model (excluding predictor of interest) - numeric")

  cols_linreg_b = c("reverse_linreg_b", "linreg_b", "linreg_b_se",
                    "linreg_b_ci_lo", "linreg_b_ci_up",
                    "reverse_linreg_b_pval", "linreg_b_pval")
  inf_linreg_b = c(
    "whether the direction of the effect size generated from the regression coefficient should be flipped - logical",
    "unstandardized regression coefficient from a linear regression model - numeric",
    "standard error of the regression coefficient - numeric",
    "lower bound of the 95% CI of the regression coefficient - numeric",
    "upper bound of the 95% CI of the regression coefficient - numeric",
    "whether the direction of the effect size generated from the regression coefficient p-value should be flipped - logical",
    "p-value of the regression coefficient - numeric")

  cols_pre_means = c("reverse_means_pre_post", "mean_pre_exp", "mean_pre_sd_exp", "mean_pre_se_exp", "mean_pre_se_nexp", "mean_pre_ci_lo_exp", "mean_pre_ci_up_exp",
                     "mean_pre_nexp", "mean_pre_sd_nexp", "mean_pre_ci_lo_nexp", "mean_pre_ci_up_nexp",
                     "reverse_mean_change", "mean_change_exp", "mean_change_nexp",
                     "mean_change_sd_exp", "mean_change_sd_nexp",
                     "mean_change_se_exp", "mean_change_se_nexp",
                     "mean_change_ci_lo_exp", "mean_change_ci_up_exp",
                     "mean_change_ci_lo_nexp", "mean_change_ci_up_nexp",
                     "mean_change_pval_exp", "mean_change_pval_nexp",
                     "r_pre_post_exp", "r_pre_post_nexp")

  inf_pre_means = c("whether the direction of the effect size generated from the pre/post means should be flipped - logical",
  "mean of the experimental/exposed group at baseline - numeric",
  "standard deviation of the experimental/exposed group at baseline - numeric",
  "standard error of the experimental/exposed group at baseline - numeric",
  "lower bound of the 95% CI of the mean of the experimental/exposed group at baseline - numeric",
  "upper bound of the 95% CI of the mean of the experimental/exposed group at baseline - numeric",
  "mean of the non-experimental/non-exposed group at baseline - numeric",
  "standard deviation of the non-experimental/non-exposed group at baseline - numeric",
  "standard error of the non-experimental/non-exposed group at baseline - numeric",
  "lower bound of the 95% CI of the mean of the non-experimental/non-exposed group at baseline - numeric",
  "upper bound of the 95% CI of the mean of the non-experimental/non-exposed group at baseline - numeric",

  "whether the direction of the effect size generated from the mean change should be flipped - logical",
  "mean change of participants in the experimental/exposed group - numeric",
  "mean change of participants in the non-experimental/non-exposed group - numeric",
  "standard deviation of the mean change for participants in the experimental/exposed group - numeric",
  "standard deviation of the mean change for participants in the non-experimental/non-exposed group - numeric",
  "standard error of the mean change for participants in the experimental/exposed group - numeric",
  "standard error of the mean change for participants in the non-experimental/non-exposed group - numeric",
  "lower bound of the 95% CI of the mean change for the experimental/exposed group - numeric",
  "upper bound of the 95% CI of the mean change for the experimental/exposed group - numeric",
  "lower bound of the 95% CI of the mean change for the non-experimental/non-exposed group - numeric",
  "upper bound of the 95% CI of the mean change for the non-experimental/non-exposed group - numeric",

  "p-value of the mean change for the experimental/exposed group - numeric",
  "p-value of the mean change for the non-experimental/non-exposed group - numeric",

  "pre-post correlation in the experimental/exposed group - numeric",
  "pre-post correlation in the non-experimental/non-exposed group - numeric")

  cols_paired_statistics = c("reverse_paired_t", "paired_t_exp", "paired_t_nexp",
                             "reverse_paired_t_pval", "paired_t_pval_exp", "paired_t_pval_nexp",
                             "reverse_paired_f", "paired_f_exp", "paired_f_nexp",
                             "reverse_paired_f_pval",  "paired_f_pval_exp", "paired_f_pval_nexp")
  inf_paired_statistics = c(
  "whether the direction of the effect size generated from the paired t-tests should be flipped - logical",
  "paired t-test value of the experimental/exposed group - numeric",
  "paired t-test value of the non-experimental/non-exposed group - numeric",

  "whether the direction of the effect size generated from the paired t-test p-values should be flipped - logical",
  "p-value of the paired t-test value of the experimental/exposed group - numeric",
  "p-value of the paired t-test value of the non-experimental/non-exposed group - numeric",

  "whether the direction of the effect size generated from the paired F-tests should be flipped - logical",
  "paired ANOVA F value of the experimental/exposed group - numeric",
  "paired ANOVA F value of the non-experimental/non-exposed group - numeric",

  "whether the direction of the effect size generated from the paired F-tests p-values should be flipped - logical",
  "p-value of the paired ANOVA-F of the experimental/exposed group - numeric",
  "p-value of the paired ANOVA-F of the non-experimental/non-exposed group - numeric")

  cols_prop_single = c("reverse_prop", "prop", "n_cases")
  inf_prop_single = c(
    "whether the direction of the proportion should be flipped - logical",
    "proportion in single group (0-1) - numeric",
    "number of cases (alternative to proportion) - numeric")

  cols_ancova_mean = c(
    "reverse_ancova_means", "ancova_mean_sd_pooled", "cov_outcome_r", "n_cov_ancova",
     "ancova_mean_exp", "ancova_mean_nexp",
     "ancova_mean_sd_exp", "ancova_mean_sd_nexp",
     "ancova_mean_se_exp", "ancova_mean_se_nexp",
     "ancova_mean_ci_lo_exp", "ancova_mean_ci_up_exp",
     "ancova_mean_ci_lo_nexp", "ancova_mean_ci_up_nexp",

     "reverse_plot_ancova_means",
     "plot_ancova_mean_exp",
     "plot_ancova_mean_sd_lo_exp", "plot_ancova_mean_sd_up_exp",
     "plot_ancova_mean_se_lo_exp", "plot_ancova_mean_se_up_exp",
     "plot_ancova_mean_ci_lo_exp",  "plot_ancova_mean_ci_up_exp",

     "plot_ancova_mean_nexp",
     "plot_ancova_mean_sd_lo_nexp", "plot_ancova_mean_sd_up_nexp",
     "plot_ancova_mean_se_lo_nexp", "plot_ancova_mean_se_up_nexp",
     "plot_ancova_mean_ci_lo_nexp", "plot_ancova_mean_ci_up_nexp")

  inf_ancova_mean = c(
    "whether the direction of the effect size generated from the ANCOVA means should be flipped - logical",
    "adjusted pooled standard deviation - numeric",
    "correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model) - numeric",
    "number of covariates in the ANCOVA model - numeric",

    "adjusted mean of participants in the experimental/exposed group - numeric",
    "adjusted standard deviation of participants in the experimental/exposed group - numeric",
    "adjusted standard error of participants in the experimental/exposed group - numeric",
    "lower bound of the adjusted 95% CI of the mean of the experimental/exposed group - numeric",
    "upper bound of the adjusted 95% CI of the mean of the experimental/exposed group - numeric",

    "adjusted mean of participants in the non-experimental/non-exposed group - numeric",
    "adjusted standard deviation of participants in the non-experimental/non-exposed group - numeric",
    "adjusted standard error of participants in the non-experimental/non-exposed group - numeric",
    "lower bound of the adjusted 95% CI of the mean of the non-experimental/non-exposed group - numeric",
    "upper bound of the adjusted 95% CI of the mean of the non-experimental/non-exposed group - numeric",

    "whether the direction of the effect size generated from the ANCOVA means extracted from a plot should be flipped - logical",
    "adjusted mean (from ANCOVA) of participants in the experimental/exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting -1 SD from the adjusted mean (from ANCOVA) of the experimental/exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting +1 SD from the adjusted mean (from ANCOVA) of the experimental/exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting -1 SE from the adjusted mean (from ANCOVA) of the experimental/exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting +1 SE from the adjusted mean (from ANCOVA) of the experimental/exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting the 95% CI of the adjusted mean (from ANCOVA) of the experimental/exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting the 95% CI of the adjusted mean (from ANCOVA) of the experimental/exposed group (extracted from a plot) - numeric",

    "adjusted mean (from ANCOVA) of participants in the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting -1 SD from the adjusted mean (from ANCOVA) of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting +1 SD from the adjusted mean (from ANCOVA) of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting -1 SE from the adjusted mean (from ANCOVA) of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting +1 SE from the adjusted mean (from ANCOVA) of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "lower bound of an error bar depicting the 95% CI of the adjusted mean (from ANCOVA) of the non-experimental/non-exposed group (extracted from a plot) - numeric",
    "upper bound of an error bar depicting the 95% CI of the adjusted mean (from ANCOVA) of the non-experimental/non-exposed group (extracted from a plot) - numeric"
  )
    # ANCOVA
  cols_ancova_stat = c("reverse_ancova_t", "ancova_t",
    "reverse_ancova_f", "ancova_f",
    "reverse_ancova_t_pval", "ancova_t_pval",
    "reverse_ancova_f_pval", "ancova_f_pval",
    "reverse_ancova_md", "ancova_md", "ancova_md_sd",
    "ancova_md_se", "ancova_md_ci_lo", "ancova_md_ci_up",
    "ancova_md_pval",
    "cohen_d_adj",
    "etasq_adj")

  inf_ancova_stat =  c("whether the direction of the effect size generated from the ANCOVA t-test should be flipped - logical",
    "a t-statistic from an ANCOVA - numeric",
    "whether the direction of the effect size generated from the ANCOVA F-test should be flipped - logical",
    "a F-statistic from an ANCOVA - numeric",
    "whether the direction of the effect size generated from the ANCOVA t-test p-value should be flipped - logical",
    "a two-tailed p-value from an ANCOVA - numeric",
    "whether the direction of the effect size generated from the ANCOVA F-test p-value should be flipped - logical",
    "a two-tailed p-value from an ANCOVA - numeric",

    "whether the direction of the effect size generated from the ANCOVA mean difference should be flipped - logical",
    "adjusted mean difference between two independent groups - numeric",
    "standard deviation of the adjusted mean difference - numeric",
    "standard error of the adjusted mean difference - numeric",
    "lower bound of the 95% CI around the adjusted mean difference - numeric",
    "upper bound of the 95% CI around the adjusted mean difference - numeric",

    "p-value of the adjusted mean difference - numeric",

    "value of an adjusted Cohen's d - numeric",
    "value of an adjusted eta-square - numeric")


    # 2x2 table
    cols_2x2 = c(
    "reverse_2x2", "baseline_risk", "small_margin_prop",
    "n_cases_exp", "n_cases_nexp", "n_controls_exp", "n_controls_nexp",
    "reverse_prop", "prop_cases_exp", "prop_cases_nexp",
    "reverse_chisq", "chisq", "reverse_chisq_pval", "chisq_pval",
    "yates_chisq",
    "reverse_phi", "phi")

    inf_2x2 = c(
      "whether the direction of the effect size generated from the contingency table should be flipped - logical",
      "proportion of cases/events in the non-exposed group - numeric",
      "smallest margin proportion of cases/events in the underlying 2x2 table - numeric",
      "number of cases/events in the exposed group - numeric",
      "number of cases/events in the non exposed group - numeric",
      "number of controls/no-event in the exposed group - numeric",
      "number of controls/no-event in the non exposed group - numeric",
      "whether the direction of the cohen_d value should be flipped - logical",
      "proportion of cases/events in the exposed group (ranging from 0 to 1) - numeric",
      "proportion of cases/events in the non-exposed group (ranging from 0 to 1) - numeric",
      "whether the direction of the effect size generated from the chi-square value should be flipped - logical",
      "chi-square value - numeric",
      "whether the direction of the effect size generated from the chi-square p-value should be flipped - logical",
      "chi-square p-value - numeric",
      "whether the chi-square statistic/p-value reported in the primary was computed with Yates' continuity correction - logical (default FALSE)",
      "whether the direction of the effect size generated from the phi value should be flipped - logical",
      "phi value - numeric"
    )

    cols_or = c(
    # or
    "reverse_or", "or", "logor", "logor_se", "or_ci_lo", "or_ci_up", "logor_ci_lo", "logor_ci_up",
    "reverse_or_pval", "or_pval",
    "reverse_logreg_t", "logreg_t")

    inf_or = c(
      "whether the direction of the effect size generated from the odds ratio should be flipped - logical",
      "odds ratio value - numeric",
      "log odds ratio value - numeric",
      "standard error of the log odds ratio - numeric",
      "lower bound of the 95% CI of the risk ratio - numeric",
      "upper bound of the 95% CI of the log risk ratio - numeric",
      "lower bound of the 95% CI of the risk ratio - numeric",
      "upper bound of the 95% CI of the log risk ratio - numeric",
      "whether the direction of the cohen_d value should be flipped - logical",
      "p-value of an odds ratio - numeric",
      "whether the direction of the effect size generated from the logistic regression t should be flipped - logical",
      "Wald t-statistic from a logistic regression model - numeric"
    )
    cols_rr = c(
    # rr
    "reverse_rr", "rr", "logrr", "logrr_se",
    "rr_ci_lo", "rr_ci_up", "logrr_ci_lo", "logrr_ci_up", "reverse_rr_pval", "rr_pval")
    inf_rr = c(
      "whether the direction of the effect size generated from the risk ratio should be flipped - logical",
      "risk ratio value - numeric",
      "log risk ratio value - numeric",
      "log risk ratio standard error - numeric",
      "lower bound of the 95% CI of the risk ratio - numeric",
      "upper bound of the 95% CI of the risk ratio - numeric",
      "lower bound of the 95% CI of the log risk ratio - numeric",
      "upper bound of the 95% CI of the log risk ratio - numeric",
      "whether the direction of the risk ratio generated from a p-value should be flipped - logical",
      "p-value of a risk ratio -  numeric"
    )

    cols_rd = c(
    # rd (risk difference)
    "reverse_rd", "rd", "rd_se",
    "rd_ci_lo", "rd_ci_up",
    "reverse_rd_pval", "rd_pval")

    inf_rd = c(
      "whether the direction of the effect size generated from the risk difference should be flipped - logical",
      "risk difference value (control risk minus treatment risk) - numeric",
      "standard error of the risk difference - numeric",
      "lower bound of the 95% CI of the risk difference - numeric",
      "upper bound of the 95% CI of the risk difference - numeric",
      "whether the direction of the effect size generated from the risk difference p-value should be flipped - logical",
      "p-value of the risk difference - numeric"
    )

    cols_r = c(
    "reverse_pearson_r", "pearson_r", "reverse_fisher_z",
    "fisher_z", "reverse_spearman_r", "spearman_r",
    "unit_increase_iv", "unit_type", "sd_iv")

    inf_r = c(
      "whether the direction of the effect size generated from the Pearson's correlation should be flipped - logical",
      "Pearson's correlation coefficient value  - numeric",
      "whether the direction of the effect size generated from the Fisher's z should be flipped - logical",
      "Fisher's r-to-z transformed correlation coefficient - numeric",
      "whether the direction of the effect size generated from the Spearman's correlation should be flipped - logical",
      "Spearman's rank correlation coefficient - numeric",
      "a value of the independent variable that will be used to estimate the Cohen's d - numeric",
      "type of unit for the unit_increase_iv variable. Must be either 'sd' or 'value' - character",
      "standard deviation of the independent variable - numeric"
      )

    cols_alpha = c("cronbach_alpha", "n_items")
    inf_alpha = c(
      "Cronbach's alpha reliability coefficient - numeric",
      "number of items in the scale - numeric")

    cols_icc = c("icc", "n_measurements", "icc_type")
    inf_icc = c(
      "intraclass correlation coefficient - numeric",
      "number of measurements or raters - numeric",
      "ICC type: 'agreement' (ICC(2,1)) or 'consistency' (ICC(3,1)) - character")

    cols_irr = c(
      # survival
    "reverse_irr", "time_exp", "time_nexp", "baseline_rate")

    inf_irr = c(
      "whether the direction of the effect size generated from the number of cases & times should be flipped - logical",
      "person-time of disease-free observation in the exposed group - numeric",
      "person-time of disease-free observation in the non-exposed group - numeric",
      "incidence rate of events in the non-exposed/control group (events per person-time) - numeric"
    )


    if (measure %in% c("d", "g")) {
      if (type_of_measure == "natural") {
        dat = data.frame(t(c(inf_req, inf_exposed, inf_means_post,
                             inf_md, inf_anova, inf_regression,
                             inf_pre_means, inf_paired_statistics,
                             inf_ancova_mean, inf_ancova_stat, inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_exposed, cols_means_post,
                           cols_md, cols_anova, cols_regression,
                           cols_pre_means, cols_paired_statistics,
                           cols_ancova_mean, cols_ancova_stat, cols_input_crude, cols_input_adjusted)

      } else {
        dat = data.frame(t(c(inf_req, inf_exposed, inf_means_post,
                             inf_md, inf_anova, inf_regression,
                             inf_pre_means, inf_med,
                             inf_paired_statistics,
                             inf_ancova_mean, inf_ancova_stat,
                             inf_or, inf_2x2, inf_cases,
                             inf_sample, inf_r,
                             inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_exposed, cols_means_post,
                           cols_md, cols_anova, cols_regression,
                           cols_pre_means, cols_med,
                           cols_paired_statistics,
                           cols_ancova_mean, cols_ancova_stat,
                           cols_or ,cols_2x2, cols_cases,
                           cols_sample, cols_r,
                           cols_input_crude, cols_input_adjusted)

      }
    } else if (measure == "md") {
      if (type_of_measure == "natural") {

      dat = data.frame(t(c(inf_req, inf_exposed, inf_md, inf_means_post,
                           inf_pre_means, inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_exposed, cols_md, cols_means_post,
                         cols_pre_means, cols_input_crude, cols_input_adjusted)
      } else {
        dat = data.frame(t(c(inf_req, inf_exposed, inf_md, inf_means_post,
                             inf_pre_means, inf_med, inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_exposed, cols_md, cols_means_post,
                           cols_pre_means, cols_med, cols_input_crude, cols_input_adjusted)

      }
    } else if (measure %in% c("dw", "gw", "mdw")) {
      if (type_of_measure == "natural") {
        dat = data.frame(t(c(inf_req, inf_exposed, inf_pre_means, inf_paired_statistics,
                             inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_exposed, cols_pre_means, cols_paired_statistics,
                           cols_input_crude, cols_input_adjusted)
      } else {
        dat = data.frame(t(c(inf_req, inf_exposed, inf_pre_means, inf_paired_statistics,
                             inf_means_post, inf_md, inf_anova, inf_regression,
                             inf_ancova_mean, inf_ancova_stat, inf_med,
                             inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_exposed, cols_pre_means, cols_paired_statistics,
                           cols_means_post, cols_md, cols_anova, cols_regression,
                           cols_ancova_mean, cols_ancova_stat, cols_med,
                           cols_input_crude, cols_input_adjusted)
      }
    } else if (measure %in% c("or")) {
      if (type_of_measure == "natural") {
      dat = data.frame(t(c(inf_req, inf_exposed, inf_cases, inf_2x2,
                           inf_or, inf_rr, inf_rd,
                           inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_exposed, cols_cases, cols_2x2,
                         cols_or, cols_rr, cols_rd,
                         cols_input_crude, cols_input_adjusted)
      } else {
        dat = data.frame(t(c(inf_req, inf_exposed, inf_cases, inf_2x2,
                             inf_or, inf_rr, inf_rd,
                             inf_means_post,
                             inf_md, inf_anova, inf_regression,
                             inf_pre_means, inf_paired_statistics,
                             inf_ancova_mean, inf_ancova_stat, inf_med,
                             inf_sample, inf_r,
                             inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_exposed, cols_cases, cols_2x2,
                           cols_or, cols_rr, cols_rd,
                           cols_means_post,
                           cols_md, cols_anova, cols_regression,
                           cols_pre_means, cols_paired_statistics,
                           cols_ancova_mean, cols_ancova_stat, cols_med,
                           cols_sample, cols_r,
                           cols_input_crude, cols_input_adjusted)

      }
    } else if (measure %in% c("rr", "nnt", "rd")) {
      if (type_of_measure == "natural") {
        dat = data.frame(t(c(inf_req, inf_exposed, inf_cases, inf_2x2,
                             inf_rr, inf_rd, inf_irr,
                             inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_exposed, cols_cases, cols_2x2,
                           cols_rr, cols_rd, cols_irr,
                           cols_input_crude, cols_input_adjusted)
      } else {
        dat = data.frame(t(c(inf_req, inf_exposed, inf_cases, inf_2x2,
                             inf_or, inf_rr, inf_rd, inf_irr,
                             inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_exposed, cols_cases, cols_2x2,
                           cols_or, cols_rr, cols_rd, cols_irr,
                           cols_input_crude, cols_input_adjusted)

      }
    } else if (measure %in% c("r", "z")) {
      if (type_of_measure == "natural") {

      dat = data.frame(t(c(inf_req, inf_sample, inf_r, inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_sample, cols_r, cols_input_crude, cols_input_adjusted)
      } else {
        dat = data.frame(t(c(inf_req, inf_sample, inf_r,
                             inf_cases, inf_2x2,
                             inf_or,
                             inf_means_post,
                             inf_md, inf_anova, inf_regression,
                             inf_pre_means, inf_paired_statistics,
                             inf_ancova_mean, inf_ancova_stat, inf_med,
                             inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_sample, cols_r,
                           cols_cases, cols_2x2,
                           cols_or,
                           cols_means_post,
                           cols_md, cols_anova, cols_regression,
                           cols_pre_means, cols_paired_statistics,
                           cols_ancova_mean, cols_ancova_stat, cols_med,
                           cols_input_crude, cols_input_adjusted)

      }
    } else if (measure %in% c("rp", "zp")) {

      dat = data.frame(t(c(inf_req, inf_sample, inf_linreg, inf_linreg_b, inf_r, inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_sample, cols_linreg, cols_linreg_b, cols_r, cols_input_crude, cols_input_adjusted)

    } else if (measure %in% c("logvr", "logcvr")) {
      if (type_of_measure == "natural") {

      dat = data.frame(t(c(inf_req, inf_exposed, inf_means_post, inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_exposed, cols_means_post, cols_input_crude, cols_input_adjusted)
      } else {
        dat = data.frame(t(c(inf_req, inf_exposed, inf_means_post,
                             inf_med,
                             inf_input_crude, inf_input_adjusted)))
        colnames(dat) <- c(cols_req, cols_exposed, cols_means_post,
                           cols_med,
                           cols_input_crude, cols_input_adjusted)

      }
    } else if (measure %in% c("irr")) {

      dat = data.frame(t(c(inf_req, inf_exposed, inf_irr, inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_exposed, cols_irr, cols_input_crude, cols_input_adjusted)

    } else if (measure %in% c("hr")) {

      # HR can only enter via user_input_crude / user_input_adj (no raw-data
      # methods, because HR requires per-subject time-to-event data).
      dat = data.frame(t(c(inf_req, inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_input_crude, cols_input_adjusted)

    } else if (measure %in% c("prop")) {

      dat = data.frame(t(c(inf_req, inf_sample, inf_prop_single, inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_sample, cols_prop_single, cols_input_crude, cols_input_adjusted)

    } else if (measure == "alpha") {

      dat = data.frame(t(c(inf_req, inf_sample, inf_alpha, inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_sample, cols_alpha, cols_input_crude, cols_input_adjusted)

    } else if (measure == "icc") {

      dat = data.frame(t(c(inf_req, inf_sample, inf_icc, inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_sample, cols_icc, cols_input_crude, cols_input_adjusted)

    } else if (measure %in% c("all")) {
      dat = data.frame(t(c(inf_req, inf_exposed, inf_means_post,
                           inf_md, inf_anova, inf_regression,
                           inf_linreg, inf_linreg_b,
                           inf_med,
                           inf_pre_means, inf_paired_statistics,
                           inf_ancova_mean, inf_ancova_stat,
                           inf_cases, inf_2x2,
                           inf_or, inf_rr, inf_rd,
                           inf_sample, inf_r,
                           inf_irr,
                           inf_prop_single,
                           inf_alpha, inf_icc,
                           inf_input_crude, inf_input_adjusted)))
      colnames(dat) <- c(cols_req, cols_exposed, cols_means_post,
                         cols_md, cols_anova, cols_regression,
                         cols_linreg, cols_linreg_b,
                         cols_med,
                         cols_pre_means, cols_paired_statistics,
                         cols_ancova_mean, cols_ancova_stat,
                         cols_cases, cols_2x2,
                         cols_or, cols_rr, cols_rd,
                         cols_sample, cols_r,
                         cols_irr,
                         cols_prop_single,
                         cols_alpha, cols_icc,
                         cols_input_crude, cols_input_adjusted)

    }

  if (missing(name)) name = "data_extract_metaConvert"
  if (extension == "data.frame") {
    dat
  } else if (extension != ".xlsx") {
    if (verbose) {
      message(cat(paste0("Sheet created at location:\n\n", getwd(), "/",
                       name, extension,
                       "\n\n Good luck with your research!")))
    }
    rio::export(dat, paste0(name, extension))
  } else {
    if (verbose) {
      message(cat(paste0("Sheet created at location:\n\n", getwd(), "/",
                       name, extension,
                       "\n\n Good luck with your research!")))
    }
    rio::export(dat, paste0(name, extension), overwrite = TRUE)
  }
}
#' Overview of effect size measures generated from each type of input data
#'
#' @param name Name of the file created
#' @param measure Target effect size measure (one of the 14 available in metaConvert). Default is "all".
#' @param extension Extension of the file created. Most common are ".xlsx", ".csv" or ".txt". It is also possible to generate an R dataframe object by using the "data.frame" extension.
#' @param type_of_measure One of "natural+converted" or "natural" (see details).
#' @param verbose logical variable indicating whether some information should be printed (e.g., the location where the sheet is created when using ".xlsx", ".csv" or ".txt" extensions)
#'
#' @details
#' This function generates, on your computer on in the console,
#' a dataset showing each effect size measure computed from each type of input data.
#' The exact combination and names of input data required are available in the links.
#'
#' The \code{measure} argument allows to filter the dataset created.
#' Only the input data allowing to estimate the selected effect size measure will be shown. Default is "all".

#' The \code{type_of_measure} argument allows to filter the dataset created.
#' - If "natural+converted" is selected, the dataset will contain all input data allowing to naturally estimate
#' and to convert the selected effect size measure
#' - If "natural" is selected, the dataset will contain all input data allowing to naturally estimate
#' the selected effect size measure
#'
#' ## Extension
#' You can export a file in various formats outside R
#' (by indicating, for example, ".txt", ".xlsx", or ".csv") in the \code{extension} argument.
#' You can also visualise this dataset directly in R by setting \code{extension = "R"}.
#'
#' This table is designed to be used in combination with tables showing the combination of input data
#' leading to estimate each of the effect size measures (https://metaconvert.org/html/input.html)
#'
#' @return
#' This function returns a table dataset presenting the input data enabling to
#' compute each effect size measure.
#'
#' @md
#'
#' @export see_input_data
#'
#' @examples
#' see_input_data(measure = "md", extension = "data.frame")

see_input_data <- function(measure = c("all", "d", "g", "md", "or", "rr", "nnt",
                                            "r", "z", "logvr", "logcvr", "irr"),
                          type_of_measure = c("natural", "natural+converted"),
                          name = "mcv_input_data",
                          extension = c("data.frame", ".txt", ".csv", ".xlsx"),
                          verbose = TRUE) {

  type_of_measure = type_of_measure[1]
  extension = extension[1]
  measure = measure[1]

  hier_list = convert_df(df.haza[1, ], measure = "d", verbose=FALSE)
  hier = as.character(sapply(hier_list, function(x) unique(x[, "info_used"])))

  dat = data.frame(
    hierarch_name = hier,
    name_input_data = rep(NA, length(hier)),
    list_input_data = rep(NA, length(hier)),
    corresponding_R_function = rep(NA, length(hier)),
    natural_effect_size_measure = rep(NA, length(hier)),
    converted_effect_size_measure = rep(NA, length(hier)),
    adjusted_input_data = rep(NA, length(hier))
  )

  dat[dat$hierarch_name == "user_input_crude",
      2:7] <- c("Crude effect size measure indicated by the user",
                "Section 23. https://metaconvert.org/html/input.html",
                "es_from_user_crude()",
                "Any effect size measure",
                "N/A",
                "Non-adjusted")
  dat[dat$hierarch_name == "user_input_adj",
      2:7] <- c("Adjusted effect size measure indicated by the user",
                "Section 24. https://metaconvert.org/html/input.html",
                "es_from_user_adj()",
                "Any adjusted effect size measure", "N/A",
                "Adjusted")
  dat[dat$hierarch_name == "cohen_d",
      2:7] <- c("Cohen's d value",
                "Section 1. https://metaconvert.org/html/input.html",
                "es_from_cohen_d()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "cohen_d_adj",
      2:7] <- c("Section 17. Adjusted Cohen's d value",
                "https://metaconvert.org/html/input.html",
                "es_from_cohen_d_adj()",
                "D+G", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "hedges_g",
      2:7] <- c("Hedges'g value",
                "Section 1. https://metaconvert.org/html/input.html",
                "es_from_hedges_g()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "or",
      2:7] <- c("Odds ratio value",
                "Section 2. https://metaconvert.org/html/input.html",
                "es_from_or()",
                "N/A", "OR+RR+NNT+D+G+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "or_se",
      2:7] <- c("Odds ratio value + standard error",
                "Section 2. https://metaconvert.org/html/input.html",
                "es_from_or_se()",
                "OR", "RR+NNT+D+G+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "or_ci",
      2:7] <- c("Odds ratio value + 95% confidence interval",
                "Section 2. https://metaconvert.org/html/input.html",
                "es_from_or_ci()",
                "OR", "RR+NNT+D+G+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "or_pval",
      2:7] <- c("Odds ratio value + p-value",
                "Section 2. https://metaconvert.org/html/input.html",
                "es_from_or_pval()",
                "OR", "RR+NNT+D+G+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "logreg_t",
      2:7] <- c("OR or RR + Wald t-statistic from logistic/log-binomial regression",
                "Section 2. https://metaconvert.org/html/input.html",
                "es_from_logreg_t()",
                "OR+RR", "NNT+D+G+R+Z",
                "Non-adjusted")

  dat[dat$hierarch_name == "pearson_r",
      2:7] <- c("Pearson's correlation coefficient",
                "Section 4. https://metaconvert.org/html/input.html",
                "es_from_pearson_r()",
                "R+Z", "D+G+OR",
                "Non-adjusted")
  dat[dat$hierarch_name == "fisher_z",
      2:7] <- c("Fisher's r-to-z correlation coefficient",
                "Section 4. https://metaconvert.org/html/input.html",
                "es_from_fisher_z()",
                "R+Z", "D+G+OR",
                "Non-adjusted")
  dat[dat$hierarch_name == "spearman_r",
      2:7] <- c("Spearman's rank correlation coefficient",
                "Section 4. https://metaconvert.org/html/input.html",
                "es_from_spearman_rho()",
                "R+Z", "D+G+OR",
                "Non-adjusted")

  dat[dat$hierarch_name == "means_sd",
      2:7] <- c("Means and standard devations of two independent groups",
                "Section 9. https://metaconvert.org/html/input.html",
                "es_from_means_sd()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "means_sd_pooled",
      2:7] <- c("Means and pooled standard devation of two independent groups",
                "Section 9. https://metaconvert.org/html/input.html",
                "es_from_means_sd_pooled()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "means_se",
      2:7] <- c("Means and standard errors of two independent groups",
                "Section 9. https://metaconvert.org/html/input.html",
                "es_from_means_se()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "means_ci",
      2:7] <- c("Means and 95% confidence intervals of two independent groups",
                "Section 9. https://metaconvert.org/html/input.html",
                "es_from_means_ci()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")


  dat[dat$hierarch_name == "mean_change_sd",
      2:7] <- c("Mean changes from pre- to post-test and standard devations of two independent groups",
                "Section 15. https://metaconvert.org/html/input.html",
                "es_from_mean_change_sd()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "mean_change_se",
      2:7] <- c("Mean changes from pre- to post-test and standard errors of two independent groups",
                "Section 15. https://metaconvert.org/html/input.html",
                "es_from_mean_change_se()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "mean_change_ci",
      2:7] <- c("Mean changes from pre- to post-test and 95% confidence intervals of two independent groups",
                "Section 15. https://metaconvert.org/html/input.html",
                "es_from_mean_change_ci()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "mean_change_pval",
      2:7] <- c("Mean changes from pre- to post-test and p-values of two independent groups",
                "Section 15. https://metaconvert.org/html/input.html",
                "es_from_mean_change_pval()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")

  dat[dat$hierarch_name == "means_sd_pre_post",
      2:7] <- c("Means and standard devations at pre- and post-test of two independent groups",
                "Section 14. https://metaconvert.org/html/input.html",
                "es_from_means_sd_pre_post()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "means_se_pre_post",
      2:7] <- c("Means and standard errors at pre- and post-test of two independent groups",
                "Section 14. https://metaconvert.org/html/input.html",
                "es_from_means_se_pre_post()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "means_ci_pre_post",
      2:7] <- c("Means and 95% confidence intervals at pre- and post-test of two independent groups",
                "Section 14. https://metaconvert.org/html/input.html",
                "es_from_means_ci_pre_post()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")

  dat[dat$hierarch_name == "paired_t",
      2:7] <- c("Paired t-tests of two independent groups",
                "Section 16. https://metaconvert.org/html/input.html",
                "es_from_paired_t()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "paired_t_pval",
      2:7] <- c("Paired t-test p-values of two independent groups",
                "Section 16. https://metaconvert.org/html/input.html",
                "es_from_paired_t_pval()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "paired_f",
      2:7] <- c("Paired F-tests of two independent groups",
                "Section 16. https://metaconvert.org/html/input.html",
                "es_from_paired_f()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "paired_f_pval",
      2:7] <- c("Paired F-test p-values of two independent groups",
                "Section 16. https://metaconvert.org/html/input.html",
                "es_from_paired_f_pval()",
                "D+G", "OR+R+Z",
                "Non-adjusted")

  dat[dat$hierarch_name == "student_t",
      2:7] <- c("Student t-test value",
                "Section 11. https://metaconvert.org/html/input.html",
                "es_from_student_t()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "student_t_pval",
      2:7] <- c("Student t-test p-values of two independent groups",
                "Section 11. https://metaconvert.org/html/input.html",
                "es_from_student_t_pval()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "anova_f",
      2:7] <- c("F-value from a one-way ANOVA",
                "Section 11. https://metaconvert.org/html/input.html",
                "es_from_anova_f()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "anova_f_pval",
      2:7] <- c("p-value from a one-way ANOVA",
                "Section 11. https://metaconvert.org/html/input.html",
                "es_from_anova_f_pval()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "etasq",
      2:7] <- c("Eta-squared value from an ANOVA model",
                "Section 11. https://metaconvert.org/html/input.html",
                "es_from_etasq",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "pt_bis_r",
      2:7] <- c("Point-biserial correlation coefficient value",
                "Section 11. https://metaconvert.org/html/input.html",
                "es_from_pt_bis_r",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "pt_bis_r_pval",
      2:7] <- c("P-value of a point-biserial correlation",
                "Section 11. https://metaconvert.org/html/input.html",
                "es_from_pt_bis_r_pval",
                "D+G", "OR+R+Z",
                "Non-adjusted")

  dat[dat$hierarch_name == "md_sd",
      2:7] <- c("Mean difference value between two independent groups and standard deviation",
                "Section 10. https://metaconvert.org/html/input.html",
                "es_from_md_sd()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "md_se",
      2:7] <- c("Mean difference value between two independent groups and standard error",
                "Section 10. https://metaconvert.org/html/input.html",
                "es_from_md_se()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "md_ci",
      2:7] <- c("Mean difference value between two independent groups and 95% confidence interval",
                "Section 10. https://metaconvert.org/html/input.html",
                "es_from_md_ci()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "md_pval",
      2:7] <- c("Mean difference value between two independent groups and p-value",
                "Section 10. https://metaconvert.org/html/input.html",
                "es_from_md_pval()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")

  dat[dat$hierarch_name == "med_quarts",
      2:7] <- c("Medians and quartiles of two independent groups",
                "Section 12. https://metaconvert.org/html/input.html",
                "es_from_med_quarts()",
                "N/A", "D+G+MD+OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "med_min_max",
      2:7] <- c("Medians and minimum+maximum values of two independent groups",
                "Section 12. https://metaconvert.org/html/input.html",
                "es_from_med_min_max()",
                "N/A", "D+G+MD+OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "med_min_max_quarts",
      2:7] <- c("Medians, quartiles and minimum+maximum values of two independent groups",
                "Section 12. https://metaconvert.org/html/input.html",
                "es_from_med_min_max_quarts()",
                "N/A", "D+G+MD+OR+R+Z",
                "Non-adjusted")

  dat[dat$hierarch_name == "means_plot",
      2:7] <- c("Means and bounds of standard deviations, standard errors, and/or 95% CIs of two independent groups extracted from a plot",
                "Section 21. https://metaconvert.org/html/input.html",
                "es_from_means_plot()",
                "D+G+MD", "OR+R+Z",
                "Non-adjusted")

  dat[dat$hierarch_name == "ancova_means_plot",
      2:7] <- c("Adjusted means and bounds of standard deviations, standard errors, and/or 95% CIs of two independent groups extracted from a plot",
                "Section 22. https://metaconvert.org/html/input.html",
                "es_from_ancova_means_plot()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")


  dat[dat$hierarch_name == "ancova_means_sd",
      2:7] <- c("Adjusted means and standard devations (from an ANCOVA) of two independent groups",
                "Section 19. https://metaconvert.org/html/input.html",
                "es_from_ancova_means_sd()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_means_se",
      2:7] <- c("Adjusted means and standard errors (from an ANCOVA) of two independent groups",
                "Section 19. https://metaconvert.org/html/input.html",
                "es_from_ancova_means_se()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_means_ci",
      2:7] <- c("Adjusted means and 95% confidence intervals (from an ANCOVA) of two independent groups",
                "Section 19. https://metaconvert.org/html/input.html",
                "es_from_ancova_means_ci()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_means_sd_pooled",
      2:7] <- c("Adjusted means (from an ANCOVA) and crude pooled standard deviation of two independent groups",
                "Section 19. https://metaconvert.org/html/input.html",
                "es_from_ancova_means_sd_pooled()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_means_sd_pooled_adj",
      2:7] <- c("Adjusted means and pooled standard deviation (from an ANCOVA) of two independent groups",
                "Section 19. https://metaconvert.org/html/input.html",
                "es_from_ancova_means_sd_pooled_adj()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")

  dat[dat$hierarch_name == "ancova_t",
      2:7] <- c("T-test value from an ANCOVA model",
                "Section 18. https://metaconvert.org/html/input.html",
                "es_from_ancova_t()",
                "D+G", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_f",
      2:7] <- c("F-value from an ANCOVA model",
                "Section 18. https://metaconvert.org/html/input.html",
                "es_from_ancova_f()",
                "D+G", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_t_pval",
      2:7] <- c("p-value of a t-test from an ANCOVA model",
                "Section 18. https://metaconvert.org/html/input.html",
                "es_from_ancova_t_pval()",
                "D+G", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_f_pval",
      2:7] <- c("p-value of an F-test from an ANCOVA model",
                "Section 18. https://metaconvert.org/html/input.html",
                "es_from_ancova_f_pval()",
                "D+G", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "etasq_adj",
      2:7] <- c("Eta-squared value from an ANOVA model",
                "https://metaconvert.org/html/input.html#ancova_statistics",
                "es_from_etasq_adj()",
                "D+G", "OR+R+Z",
                "Adjusted")

  dat[dat$hierarch_name == "ancova_md_sd",
      2:7] <- c("Adjusted mean difference value between two independent groups and standard deviation (from an ANCOVA model)",
                "Section 20. https://metaconvert.org/html/input.html",
                "es_from_ancova_md_sd()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_md_se",
      2:7] <- c("Adjusted mean difference value between two independent groups and standard error (from an ANCOVA model)",
                "Section 20. https://metaconvert.org/html/input.html",
                "es_from_ancova_md_se()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_md_ci",
      2:7] <- c("Adjusted mean difference value between two independent groups and 95% confidence interval (from an ANCOVA model)",
                "Section 20. https://metaconvert.org/html/input.html",
                "es_from_ancova_md_ci()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")
  dat[dat$hierarch_name == "ancova_md_pval",
      2:7] <- c("Adjusted mean difference value between two independent groups and p-value (from an ANCOVA model)",
                "Section 20. https://metaconvert.org/html/input.html",
                "es_from_ancova_md_pval()",
                "D+G+MD", "OR+R+Z",
                "Adjusted")

  dat[dat$hierarch_name == "chisq",
      2:7] <- c("Chi-square value (obtained from a 2x2 table)",
                "Section 8. https://metaconvert.org/html/input.html",
                "es_from_chisq()",
                "OR+RR+R+Z", "D+G",
                "Non-adjusted")
  dat[dat$hierarch_name == "chisq_pval",
      2:7] <- c("P-value of a Chi-square (obtained from a 2x2 table)",
                "Section 8. https://metaconvert.org/html/input.html",
                "es_from_chisq_pval()",
                "OR+RR+R+Z", "D+G",
                "Non-adjusted")
  dat[dat$hierarch_name == "phi",
      2:7] <- c("Phi coefficient",
                "Section 8. https://metaconvert.org/html/input.html",
                "es_from_phi()",
                "OR+RR+R+Z", "D+G",
                "Non-adjusted")


  dat[dat$hierarch_name == "2x2",
      2:7] <- c("Number of participants in each cell of a 2x2 table",
                "Section 7. https://metaconvert.org/html/input.html",
                "es_from_2x2()",
                "OR+RR+NNT", "D+G+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "2x2_sum",
      2:7] <- c("Number of participants 'developping the disease' and row marginal sums",
                "Section 7. https://metaconvert.org/html/input.html",
                "es_from_2x2_sum()",
                "OR+RR+NNT", "D+G+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "2x2_prop",
      2:7] <- c("Proportion of participants 'developping the disease' and row marginal proportions",
                "Section 7. https://metaconvert.org/html/input.html",
                "es_from_2x2_prop()",
                "OR+RR+NNT", "D+G+R+Z",
                "Non-adjusted")


  dat[dat$hierarch_name == "rr_se",
      2:7] <- c("Risk ratio value + standard error",
                "Section 3. https://metaconvert.org/html/input.html",
                "es_from_rr_se()",
                "RR", "OR+NNT",
                "Non-adjusted")
  dat[dat$hierarch_name == "rr_ci",
      2:7] <- c("Risk ratio value + 95% confidence interval",
                "Section 3. https://metaconvert.org/html/input.html",
                "es_from_rr_ci()",
                "RR", "OR+NNT",
                "Non-adjusted")
  dat[dat$hierarch_name == "rr_pval",
      2:7] <- c("Risk ratio value + p-value",
                "Section 3. https://metaconvert.org/html/input.html",
                "es_from_rr_pval",
                "RR", "OR+NNT",
                "Non-adjusted")

  dat[dat$hierarch_name == "beta_std",
      2:7] <- c("Standardized regression estimate (with a dichotomous predictor)",
                "Section 13. https://metaconvert.org/html/input.html",
                "es_from_beta_std()",
                "D+G", "OR+R+Z",
                "Non-adjusted")
  dat[dat$hierarch_name == "beta_unstd",
      2:7] <- c("Unstandardized regression estimate (with a dichotomous predictor)",
                "Section 13. https://metaconvert.org/html/input.html",
                "es_from_beta_unstd()",
                "D+G", "OR+R+Z",
                "Non-adjusted")

  dat[dat$hierarch_name == "cases_time",
      2:7] <- c("Number of cases and time of disee-free observation time in two independent groups",
                "Section 5. https://metaconvert.org/html/input.html",
                "es_from_cases_time()",
                "IRR", "N/A",
                "Non-adjusted")

  dat[dat$hierarch_name == "variability_means_sd",
      2:7] <- c("Standard devations (and means) of two independent groups",
                "Section 6. https://metaconvert.org/html/input.html",
                "es_from_variability_means_sd()",
                "VR+CVR", "N/A",
                "Non-adjusted")
  dat[dat$hierarch_name == "variability_means_ci",
      2:7] <- c("95% CIs around means (and means) of two independent groups",
                "Section 6. https://metaconvert.org/html/input.html",
                "es_from_variability_means_ci()",
                "VR+CVR", "N/A",
                "Non-adjusted")
  dat[dat$hierarch_name == "variability_means_se",
      2:7] <- c("Standard errors (and means) of two independent groups",
                "Section 6. https://metaconvert.org/html/input.html",
                "es_from_variability_means_se()",
                "VR+CVR", "N/A",
                "Non-adjusted")

  if (type_of_measure == "natural+converted") {
    dat = switch(measure,
                 "d" = subset(dat, grepl("D+G", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "g" = subset(dat, grepl("G", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "md" = subset(dat, grepl("MD", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "or" = subset(dat, grepl("OR", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "rr" = subset(dat, grepl("RR", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "nnt" = subset(dat, grepl("NNT", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "r" = subset(dat, grepl("R+Z", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "Z" = subset(dat, grepl("R+Z", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "irr" = subset(dat, grepl("IRR", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "logvr" = subset(dat, grepl("VR", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "logcvr" = subset(dat, grepl("CVR", paste0(dat[, 6], dat[, 5]), fixed = TRUE)),
                 "all" = dat
    )
  } else if (type_of_measure == "natural") {
    dat = switch(measure,
                 "d" = subset(dat, grepl("D+G", paste0(dat[, 5]), fixed = TRUE)),
                 "g" = subset(dat, grepl("G", paste0(dat[, 5]), fixed = TRUE)),
                 "md" = subset(dat, grepl("MD", paste0(dat[, 5]), fixed = TRUE)),
                 "or" = subset(dat, grepl("OR", paste0(dat[, 5]), fixed = TRUE)),
                 "rr" = subset(dat, grepl("RR", paste0(dat[, 5]), fixed = TRUE)),
                 "nnt" = subset(dat, grepl("NNT", paste0(dat[, 5]), fixed = TRUE)),
                 "r" = subset(dat, grepl("R+Z", paste0(dat[, 5]), fixed = TRUE)),
                 "Z" = subset(dat, grepl("R+Z", paste0(dat[, 5]), fixed = TRUE)),
                 "irr" = subset(dat, grepl("IRR", paste0(dat[, 5]), fixed = TRUE)),
                 "logvr" = subset(dat, grepl("VR", paste0(dat[, 5]), fixed = TRUE)),
                 "logcvr" = subset(dat, grepl("CVR", paste0(dat[, 5]), fixed = TRUE)),
                 "all" = subset(dat, dat[, 5] != "N/A")
    )
  } else {
    stop(paste0("Incorrect type_of_measure: ", type_of_measure))
  }

  dat = dat[order(dat$list_input_data), ]
  dat = dat[order(dat$adjusted_input_data, decreasing=TRUE), ]

  if (missing(name)) name = "data_extract_metaConvert"
  if (extension == "data.frame") {
    dat
  } else if (extension != ".xlsx") {
    rio::export(dat, paste0(name, extension))
    if (verbose) {
      message(cat(paste0("Sheet created at location:\n\n", getwd(), "/",
                         name, extension,
                         "\n\n Good luck with your research!")))
    }
  } else {
    rio::export(dat, paste0(name, extension), overwrite = TRUE)
    if (verbose) {
      message(cat(paste0("Sheet created at location:\n\n", getwd(), "/",
                         name, extension,
                         "\n\n Good luck with your research!")))
    }
  }
}

#' This function checks the input data to have the correct format and that it has no incongruencies
#'
#' @param x a data.frame passed to convert_df() to be checked for correct format and consistency
#'
#' @noRd
.check_data <- function(x, split_adjusted = TRUE, format = "wide", main_es = TRUE) {
  #### Check input type ----------------

  # we check if the input object is a dataframe. if it is not, we display an error message
  if (!("data.frame" %in% class(x))) {
    stop("The object passed to the convert_df() function is not a data.frame object.")
  } else if (length(class(x)) > 1 & "data.frame" %in% class(x)) { # if it is not only a dataframe, we convert it
    x <- data.frame(x)
  }

  if (length(colnames(x)) == 0) {
    stop("Dataframe passed to the convert_df() function has no column. Check format of the dataset.")
  } else if (nrow(x) == 0) {
    stop("Dataframe passed to the convert_df() function has no row. Check format of the dataset.")
  }


  #### Initialize some settings ------
  status <- "OK" # this is the general status of the whole analysis
  error_msgs <- c() # this will contain all errors and warnings for each study
  row_wrong <- c() # row of error
  col_wrong <- c() # col of error
  column_errors <- rep("", nrow(x))
  column_type_errors <- rep("", nrow(x))
  user_input_es <- user_input_se <-
    user_input_es_crd <- user_input_se_crd <-
    user_input_es_adj <- user_input_se_adj <- rep(NA_real_, nrow(x))
  x$row_id <- 1:nrow(x)

  # Backwards compatibility: map old user_es_measure column names to new names
  if ("user_es_measure_crude" %in% colnames(x) && !"user_es_original_measure_crude" %in% colnames(x)) {
    x$user_es_original_measure_crude <- x$user_es_measure_crude
  }
  if ("user_es_measure_adj" %in% colnames(x) && !"user_es_original_measure_adj" %in% colnames(x)) {
    x$user_es_original_measure_adj <- x$user_es_measure_adj
  }

  returned <- paste0("Checkings finished.") # object returned


  expected_cols <- c(
    "study_id", "author", "year", "predictor", "outcome", "info_expected",
    # sample size
    "n_sample", "n_exp", "n_nexp",
    # CRUDE input
    "user_es_original_measure_crude",
    "user_es_target_measure_crude",
    "user_es_crude",
    "user_se_crude",
    "user_ci_lo_crude",
    "user_ci_up_crude",
    # smd/g
    "reverse_d", "cohen_d", "reverse_g", "hedges_g",
    # regression
    "reverse_beta_std", "beta_std", "reverse_beta_unstd", "beta_unstd", "sd_dv",
    # regression t-statistic (partial correlation)
    "reverse_linreg_t", "linreg_t", "n_covariates",
    # regression coefficient (partial correlation)
    "reverse_linreg_b", "linreg_b", "linreg_b_se",
    "linreg_b_ci_lo", "linreg_b_ci_up",
    "reverse_linreg_b_pval", "linreg_b_pval",
    # md
    "reverse_md", "md", "md_sd", "md_se", "md_ci_lo", "md_ci_up",
    "md_pval",
    # quarts
    "reverse_med", "min_exp", "q1_exp", "med_exp", "q3_exp", "max_exp",
    "min_nexp", "q1_nexp", "med_nexp", "q3_nexp", "max_nexp",
    # means post
    "reverse_means", "mean_sd_pooled",
    "mean_exp", "mean_sd_exp", "mean_se_exp", "mean_ci_lo_exp", "mean_ci_up_exp",
    "mean_nexp", "mean_sd_nexp", "mean_se_nexp", "mean_ci_lo_nexp", "mean_ci_up_nexp",
    # t f eta
    "reverse_student_t", "student_t",
    "reverse_student_t_pval", "student_t_pval",
    "reverse_anova_f", "anova_f",
    "reverse_anova_f_pval", "anova_f_pval",
    "reverse_etasq", "etasq",
    "reverse_pt_bis_r", "pt_bis_r", "reverse_pt_bis_r_pval", "pt_bis_r_pval",

    # pre/post
    "reverse_means_pre_post", "mean_pre_exp", "mean_pre_nexp", "mean_pre_sd_exp", "mean_pre_sd_nexp",
    "mean_pre_se_exp", "mean_pre_se_nexp", "mean_pre_ci_lo_exp", "mean_pre_ci_up_exp", "mean_pre_ci_lo_nexp", "mean_pre_ci_up_nexp",
    "reverse_mean_change", "mean_change_exp", "mean_change_nexp",
    "mean_change_sd_exp", "mean_change_sd_nexp",
    "mean_change_se_exp", "mean_change_se_nexp",
    "mean_change_ci_lo_exp", "mean_change_ci_up_exp",
    "mean_change_ci_lo_nexp", "mean_change_ci_up_nexp",
    "mean_change_pval_exp", "mean_change_pval_exp",
    "mean_change_pval_nexp", "mean_change_pval_nexp",

    "r_pre_post_exp", "r_pre_post_nexp",
    "paired_t_exp", "paired_t_nexp", "reverse_paired_t",
    "paired_t_pval_exp", "paired_t_pval_nexp", "reverse_paired_t_pval",
    "paired_f_exp", "paired_f_nexp", "reverse_paired_f",
    "paired_f_pval_exp", "paired_f_pval_nexp", "reverse_paired_f_pval",
    # plot
    "reverse_plot_means",
    "plot_mean_exp", "plot_mean_nexp",
    "plot_mean_sd_lo_exp", "plot_mean_sd_lo_nexp",
    "plot_mean_sd_up_exp", "plot_mean_sd_up_nexp",
    "plot_mean_se_lo_exp", "plot_mean_se_lo_nexp",
    "plot_mean_se_up_exp", "plot_mean_se_up_nexp",
    "plot_mean_ci_lo_exp", "plot_mean_ci_lo_nexp",
    "plot_mean_ci_up_exp", "plot_mean_ci_up_nexp",
    # ADJUSTED input
    "user_es_original_measure_adj",
    "user_es_target_measure_adj",
    "user_es_adj",
    "user_se_adj",
    "user_ci_lo_adj",
    "user_ci_up_adj",
    "cohen_d_adj",
    "etasq_adj",
    # ANCOVA
    "reverse_ancova_means", "ancova_mean_sd_pooled", "cov_outcome_r", "n_cov_ancova",
    "ancova_mean_exp", "ancova_mean_nexp",
    "ancova_mean_sd_exp", "ancova_mean_sd_nexp",
    "ancova_mean_se_exp", "ancova_mean_se_nexp",
    "ancova_mean_ci_lo_exp", "ancova_mean_ci_up_exp",
    "ancova_mean_ci_lo_nexp", "ancova_mean_ci_up_nexp",
    "reverse_ancova_t", "ancova_t",
    "reverse_ancova_f", "ancova_f",
    "reverse_ancova_t_pval", "ancova_t_pval",
    "reverse_ancova_f_pval", "ancova_f_pval",
    "reverse_ancova_md", "ancova_md", "ancova_md_sd", "ancova_md_se",
    "ancova_md_ci_lo", "ancova_md_ci_up",
    "ancova_md_pval",
    # plot
    "reverse_plot_ancova_means",
    "plot_ancova_mean_exp", "plot_ancova_mean_nexp",
    "plot_ancova_mean_sd_lo_exp", "plot_ancova_mean_sd_lo_nexp",
    "plot_ancova_mean_sd_up_exp", "plot_ancova_mean_sd_up_nexp",
    "plot_ancova_mean_se_lo_exp", "plot_ancova_mean_se_lo_nexp",
    "plot_ancova_mean_se_up_exp", "plot_ancova_mean_se_up_nexp",
    "plot_ancova_mean_ci_lo_exp", "plot_ancova_mean_ci_lo_nexp",
    "plot_ancova_mean_ci_up_exp", "plot_ancova_mean_ci_up_nexp",

    # variability
    "reverse_means_variability", "reverse_means_change_variability",

    # 2x2 table
    "reverse_2x2", "baseline_risk", "baseline_rate", "small_margin_prop",
    "n_cases_exp", "n_cases_nexp", "n_controls_exp", "n_controls_nexp",
    "reverse_prop", "prop_cases_exp", "prop_cases_nexp",
    "n_cases", "n_controls",
    # single-group proportion
    "prop",
    # or
    "reverse_or", "or", "logor", "logor_se", "or_ci_lo", "or_ci_up", "logor_ci_lo", "logor_ci_up",
    "reverse_or_pval", "or_pval",
    "reverse_logreg_t", "logreg_t",
    # rr
    "reverse_rr", "rr", "logrr", "logrr_se",
    "rr_ci_lo", "rr_ci_up", "logrr_ci_lo", "logrr_ci_up",
    "reverse_rr_pval", "rr_pval",
    # rd (risk difference)
    "reverse_rd", "rd", "rd_se",
    "rd_ci_lo", "rd_ci_up",
    "reverse_rd_pval", "rd_pval",
    # X2, PHI, COR PB
    "reverse_chisq", "chisq", "reverse_chisq_pval", "chisq_pval",
    "yates_chisq",
    "reverse_phi", "phi", #"reverse_phi_pval", "phi_pval",
    # r
    "reverse_pearson_r", "pearson_r", "reverse_fisher_z", "fisher_z",
    "unit_increase_iv",
    "unit_type",
    "sd_iv",
    # survival
    "time_exp", "time_nexp", "reverse_irr",
    # correlation (Spearman)
    "reverse_spearman_r", "spearman_r",
    # psychometrics
    "cronbach_alpha", "n_items",
    "icc", "n_measurements", "icc_type",
    "discard",
    # multi-arm trial pooling
    "pool_side"
  )
  # res = data.frame(Comment = rep("N/A", 5))
  # res[, c(expected_cols)] <- NA
  # res$study_id = paste("study ", 1:5)
  # res$author = c("Bern", "Smith", "Jonh", "Doe", "Bakari")
  # res$year = c(2018, 2011, 2023, 2001, 2015)
  # res$factor = "Early Intensive Behavioral Intervention"
  # res$outcome = "Vineland-III"
  # rio::export(res, "C:/Users/Corentin Gosling/drive_gmail/Recherche/es.utils/web/background/extraction_sheet.xlsx", overwrite=TRUE)

  expected_cols_type <- ifelse(grepl("study_id", expected_cols, fixed = TRUE) |
    grepl("info_expected", expected_cols, fixed = TRUE) |
    grepl("author", expected_cols, fixed = TRUE) |
    grepl("year", expected_cols, fixed = TRUE) |
    grepl("predictor", expected_cols, fixed = TRUE) |
    grepl("outcome", expected_cols, fixed = TRUE) |
    grepl("measure", expected_cols, fixed = TRUE) |
    grepl("unit_type", expected_cols, fixed = TRUE) |
    grepl("icc_type", expected_cols, fixed = TRUE) |
    grepl("discard", expected_cols, fixed = TRUE),
  "char", "numeric"
  )

  # Set reverse_* and discard columns to logical type
  expected_cols_type[grepl("reverse", expected_cols, fixed = TRUE)] <- "logical"
  expected_cols_type[grepl("discard", expected_cols, fixed = TRUE)] <- "logical"

  expected_cols_type[which(expected_cols == "cov_outcome_r")] <- "numeric"
  # n_measurements contains "measure", so the grepl above would otherwise type it "char";
  # force it numeric. n_items does NOT match any char-triggering grepl, so its override
  # is a harmless belt-and-braces guard (kept for symmetry).
  expected_cols_type[which(expected_cols == "n_measurements")] <- "numeric"
  expected_cols_type[which(expected_cols == "n_items")] <- "numeric"
  expected_cols_type[which(expected_cols == "pool_side")] <- "char"
  # per-row Yates flag for chi-square back-derivation
  expected_cols_type[which(expected_cols == "yates_chisq")] <- "logical"
  # View(cbind(expected_cols, expected_cols_type))
  # colnames(x) <- tolower(colnames(x))

  # remove all rows that should be discarded from analyses
  if (any(x$discard %in% c("yes", "Yes", "remove", "removed", "TRUE", TRUE))) {
    removed_rows <- which(x$discard %in% c("yes", "Yes", "remove", "removed", "TRUE"))
    status <- ifelse(status == "ERROR", "ERROR", "WARNING")
    error_msgs <- append(error_msgs, paste0("Some rows of the original dataset have been removed based on the 'discard' column inputs: rows = ", paste(removed_rows, collapse = ", "), " (only a warning, not an error)."))
    column_errors <- column_errors[-removed_rows]
    column_type_errors <- column_type_errors[-removed_rows]
    x <- subset(x, !discard %in% c("yes", "Yes", "remove", "removed", "TRUE", TRUE))
    # NB: a stray `situation <- situation[-removed_rows]` was removed here. `situation`
    # was never defined anywhere in this function, so any truthy value in the documented
    # `discard` column aborted the whole convert_df() run with "object 'situation' not
    # found". The actual row removal is performed by the subset() call above.
    # row_id was assigned as 1:nrow() before the discard drop, so it is now
    # non-contiguous (e.g. c(1, 3)). Downstream code (the dispersion merge in
    # .generate_df) requires row_id == seq_len(nrow(x)); re-sequence it here so the
    # retained rows are renumbered and the invariant holds.
    x$row_id <- seq_len(nrow(x))
  }

  # set as NA columns not included in the dataset
  for (col in expected_cols) {
    if (!(col %in% colnames(x))) {
      x[, col] <- NA
    }
  }

  #### set "na", "inf" or blank as NA
  x[x == "" | x == " " | x == "na" | x == "NA" | x == "n/a" | x == "N/A" |
      x == "inf" | x == "infinity" | x == "Infinity" | x == "INFINITY" | x == "INF" |
      x == "Inf"] <- NA

  #### Convert numerical and logical columns to correct type
  for (j in 1:length(expected_cols)) {
    idx <- which(colnames(x) == expected_cols[j])
    x[, idx] <- as.character(x[, idx])
    if (expected_cols_type[j] == "numeric") {
      # check the presence of . and , in numeric columns
      if (any(grepl("\\.", x[, idx]) & grepl(",", x[, idx]))) {
        commas_and_points <- which(grepl("\\.", x[, idx]) & grepl(",", x[, idx]))
        status <- "ERROR"
        # column_errors[commas_and_points] = paste(column_errors[commas_and_points], "Column '", paste(colnames(x)[idx]), "' contains both '.' and ','. //")
        stop(paste("Column '", paste(colnames(x)[idx]), "' contains both '.' and ','. //"))
        # column_type_errors[commas_and_points] = "ERROR"
      } else if (any(grepl(",", x[, idx]))) {
        commas <- which(grepl(",", x[, idx]))
        status <- ifelse(status == "ERROR", "ERROR", "WARNING")
        column_errors[commas] <- paste(column_errors[commas], "A ',' has been converted to a '.' in column '", paste(colnames(x)[idx]), "' (this is only a warning, not an error). //")
        warning("A ',' has been converted to a '.' in column '", paste(colnames(x)[idx]), "' (this is only a warning, not an error).")
        x[, idx] <- gsub(",", ".", x[, idx])
      }
      # check the presence of non-numeric characters in numeric columns
      if (any(suppressWarnings(is.na(as.numeric(as.character(na.omit(x[, idx]))))))) {
        not_num <- x[which(!is.na(x[, idx])), ][suppressWarnings(is.na(as.numeric(as.character(na.omit(x[, idx]))))), ]$row_id
        stop("Non-numeric characters ('", paste(unique(x[not_num, idx]), collapse = " ' / ' "), "') in column '", paste(colnames(x)[idx], "'. //"))
      }
      # convert numeric columns to numeric format
      x[, idx] <- as.numeric(as.character(x[, idx]))
    } else if (expected_cols_type[j] == "logical") {
      # convert logical columns to logical format
      x[, idx] <- as.logical(x[, idx])
      # Set NA values to FALSE for logical columns - except yates_chisq,
      # where NA means "use the scalar yates_chisq default supplied to convert_df()"
      # (handled downstream in main_convert_df.R).
      if (expected_cols[j] != "yates_chisq") {
        x[is.na(x[, idx]), idx] <- FALSE
      }
    }
  }



  x$n_cases <- with(x, ifelse(is.na(n_cases) & !is.na(n_cases_exp) & !is.na(n_cases_nexp),
    n_cases_exp + n_cases_nexp,
    n_cases
  ))
  x$n_controls <- with(x, ifelse(is.na(n_controls) & !is.na(n_controls_exp) & !is.na(n_controls_nexp),
    n_controls_exp + n_controls_nexp,
    n_controls
  ))
  x$n_exp <- with(x, ifelse(is.na(n_exp) & !is.na(n_cases_exp) & !is.na(n_controls_exp),
    n_cases_exp + n_controls_exp,
    n_exp
  ))
  x$n_nexp <- with(x, ifelse(is.na(n_nexp) & !is.na(n_cases_nexp) & !is.na(n_controls_nexp),
    n_cases_nexp + n_controls_nexp,
    n_nexp
  ))
  x$n_sample <- with(x, ifelse(is.na(n_sample) & !is.na(n_exp) & !is.na(n_nexp),
    n_exp + n_nexp,
    ifelse(is.na(n_sample) & !is.na(n_cases) & !is.na(n_controls),
      n_cases + n_controls, n_sample
    )
  ))

  col_names <- c(
    "all_info", "measure", "info_measure", "n_estimations", "es_selected", "info_used",
    "es", "se", "es_ci_lo", "es_ci_up",
    "min_info", "min_es_value", "min_es_se", "min_es_ci_lo", "min_es_ci_up",
    "max_info", "max_es_value", "max_es_se", "max_es_ci_lo", "max_es_ci_up",
    "diff_min_max", "overlap_min_max", "dispersion_es"
  )

  if (split_adjusted == TRUE & format == "wide" & main_es == TRUE) {
    res_crude <- data.frame(matrix(NA, nrow = nrow(x), ncol = length(col_names)))
    colnames(res_crude) <- paste0(col_names, "_crude")

    res_adjusted <- data.frame(matrix(NA, nrow = nrow(x), ncol = length(col_names)))
    colnames(res_adjusted) <- paste0(col_names, "_adjusted")

    res <- cbind(
      x[, c("row_id", "study_id", "author", "year", "predictor", "outcome", "info_expected")],
      res_crude, res_adjusted,
      subset(x, select = -c(row_id, study_id, author, year, predictor, outcome, info_expected))
    )
  } else {#if ((split_adjusted == TRUE & format == "long") | main_es == FALSE) {
    res_crude <- data.frame(matrix(NA, nrow = nrow(x), ncol = length(col_names)))
    colnames(res_crude) <- col_names
    res <- cbind(
      x[, c("row_id", "study_id", "author", "year", "predictor", "outcome", "info_expected")],
      adjusted_input = NA,
      res_crude,
      subset(x, select = -c(row_id, study_id, author, year, predictor, outcome, info_expected))
    )
  }

  return(res)
}
# rio::export(data.frame(do.call(rbind, lapply(object, function(x) x[1, "info_used"]))),
#             "possible_inputs.xlsx", overwrite = TRUE)

# if (es_adj == "separate" & !is.na(es_adj)) {
#
#   # we extract information on ES already given by the users
#   if ("es_crude" %in% colnames(x) & "se_crude" %in% colnames(x)) {
#     user_input_es_crd = x$es_crude
#     user_input_se_crd = x$se_crude
#   } else {
#     es_crude = se_crude = rep(NA_real_, nrow(x))
#   }
#
#   if ("es_adjusted" %in% colnames(x) & "se_adjusted" %in% colnames(x)) {
#     user_input_es_adj = x$es_adjusted
#     user_input_se_adj = x$se_adjusted
#   } else {
#     es_adjusted = se_adjusted = rep(NA_real_, nrow(x))
#   }
#
#   res_return_crude <- data.frame(matrix(NA, nrow = nrow(x), ncol = length(col_names)))
#   colnames(res_return_crude) <- paste0(col_names, "_crude")
#
#   res_return_adjusted <- data.frame(matrix(NA, nrow = nrow(x), ncol = length(col_names)))
#   colnames(res_return_adjusted) <- paste0(col_names, "_adjusted")
#
#   res_return = cbind(res_return_crude, res_return_adjusted)
#
# } else {
#
#   # we extract information on ES already given by the users
#   if ("es" %in% colnames(x) & "se" %in% colnames(x)) {
#     user_input_es = x$es
#     user_input_se = x$se
#   } else {
#     es_crude = se_crude = rep(NA_real_, nrow(x))
#   }
#   res_return <- data.frame(matrix(NA, nrow = nrow(x), ncol = length(col_names)))
#   colnames(res_return) <- col_names
# }
#
# res = cbind(res_return, subset(x, select = -c(row_id)))
# attr(res, "user_input_es") <- user_input_es
# attr(res, "user_input_se") <- user_input_se
# attr(res, "user_input_es_crd") <- user_input_es_crd
# attr(res, "user_input_se_crd") <- user_input_se_crd
# attr(res, "user_input_es_adj") <- user_input_es_adj
# attr(res, "user_input_se_adj") <- user_input_se_adj
# attr(res, "user_input_se_adj") <- user_input_se_adj

#' Estimate sample size and time for the number of cases exposed and non-exposed from IRR, variance and overall number of cases
#'
#' @param irr IRR
#' @param var variance
#' @param n_cases number of cases
#' @param time time
#' @param time_exp time of exposed group
#' @param time_nexp time of non exposed group
#'
#' @noRd
.estimate_n_from_irr <- function(irr, var, n_cases, time = NA, time_exp = NA, time_nexp = NA) {
  # (re)-estimate n and time from irr and var
  n_cases_exp <- optimize(function(n_cases_exp, var, n_cases) {
    (1 / n_cases_exp + 1 / (n_cases - n_cases_exp) - var)^2
  }, c(0, n_cases), var, n_cases)$minimum
  n_cases_nexp <- n_cases - n_cases_exp
  if (is.na(time) && is.na(time_exp) && is.na(time_nexp)) {
    time <- 1
  }
  if (!is.na(time)) {
    time_exp <- n_cases_exp / (n_cases_exp + irr * n_cases_nexp) * time
    time_nexp <- time - time_exp
  } else if (!is.na(time_exp)) {
    time <- time_exp * (n_cases_exp + irr * n_cases_nexp) / n_cases_exp
    time_nexp <- time - time_exp
  } else {
    time_exp <- (n_cases_exp * time_nexp) / (irr * n_cases_nexp)
    time <- time_exp + time_nexp
  }
  return(data.frame(n_cases_exp, time_exp, n_cases_nexp, time_nexp))
}

#' Estimate n value from OR and the number of cases and controls
#'
#' @param or OR
#' @param var variance
#' @param n_cases number of cases
#' @param n_controls number of controls
#'
#' @noRd
.estimate_n_from_or_and_n_cases <- function(or, var, n_cases, n_controls) {
  # Create all possibilites of n
  n_cases_nexp_sim1 <- 0:n_cases
  n_controls_nexp_sim1 <- round(n_controls * (1 - (n_cases - n_cases_nexp_sim1) / (n_cases + (or - 1) * n_cases_nexp_sim1)))
  n_cases_exp_sim1 <- n_cases - n_cases_nexp_sim1
  n_controls_exp_sim1 <- n_controls - n_controls_nexp_sim1
  # sim1: possiblities with strictly positive n
  idx_non_zero <- which(
    n_cases_nexp_sim1 > 0 &
      n_controls_nexp_sim1 > 0 &
      n_cases_exp_sim1 > 0 &
      n_controls_exp_sim1 > 0
  )
  n_cases_nexp_sim1 <- n_cases_nexp_sim1[idx_non_zero]
  n_controls_nexp_sim1 <- n_controls_nexp_sim1[idx_non_zero]
  n_cases_exp_sim1 <- n_cases_exp_sim1[idx_non_zero]
  n_controls_exp_sim1 <- n_controls_exp_sim1[idx_non_zero]

  # sim2: possiblities with positive n and at least one zero (Add 0.5 to the possiblities with any 0)
  n_cases_nexp_sim2 <- 0:n_cases
  n_controls_nexp_sim2 <- round((n_controls + 0.5) - (n_controls + 1) * (n_cases - n_cases_nexp_sim2 + 0.5) / ((n_cases_nexp_sim2 + 0.5) * or + n_cases - n_cases_nexp_sim2 + 0.5))
  n_cases_exp_sim2 <- n_cases - n_cases_nexp_sim2
  n_controls_exp_sim2 <- n_controls - n_controls_nexp_sim2
  # (n_cases_exp_sim2 + 0.5) / (n_cases_nexp_sim2 + 0.5) / (n_controls_exp_sim2 + 0.5) * (n_controls_nexp_sim2 + 0.5)
  # select the ones with some 0 but non-negative
  idx_some_zero <- which(
    (n_cases_nexp_sim2 == 0 | n_controls_nexp_sim2 == 0 | n_cases_exp_sim2 == 0 | n_controls_exp_sim2 == 0) &
      (n_cases_nexp_sim2 >= 0 & n_controls_nexp_sim2 >= 0 & n_cases_exp_sim2 >= 0 & n_controls_exp_sim2 >= 0)
  )
  n_cases_nexp_sim2 <- n_cases_nexp_sim2[idx_some_zero]
  n_controls_nexp_sim2 <- n_controls_nexp_sim2[idx_some_zero]
  n_cases_exp_sim2 <- n_cases_exp_sim2[idx_some_zero]
  n_controls_exp_sim2 <- n_controls_exp_sim2[idx_some_zero]

  # join both previous vectors
  n_cases_nexp_sim <- append(n_cases_nexp_sim1, n_cases_nexp_sim2)
  n_controls_nexp_sim <- append(n_controls_nexp_sim1, n_controls_nexp_sim2)
  n_cases_exp_sim <- append(n_cases_exp_sim1, n_cases_exp_sim2)
  n_controls_exp_sim <- append(n_controls_exp_sim1, n_controls_exp_sim2)

  some_zero <- n_cases_exp_sim == 0 | n_controls_exp_sim == 0 | n_cases_nexp_sim == 0 | n_controls_nexp_sim == 0

  var_sim <- ifelse(some_zero,
    1 / ((n_cases + 1) - (n_cases_nexp_sim + 0.5)) + 1 / ((n_controls + 1) - (n_controls_nexp_sim + 0.5)) + 1 / (n_cases_nexp_sim + 0.5) + 1 / (n_controls_nexp_sim + 0.5),
    1 / (n_cases - n_cases_nexp_sim) + 1 / (n_controls - n_controls_nexp_sim) + 1 / n_cases_nexp_sim + 1 / n_controls_nexp_sim
  )

  # var_sim2 = 1 / ((n_cases+1) - (n_cases_nexp_sim+0.5)) + 1 / ((n_controls+1) - (n_controls_nexp_sim+0.5)) + 1 / (n_cases_nexp_sim+0.5) + 1 / (n_controls_nexp_sim+0.5)

  best <- order((var_sim - var)^2)[1]
  n_cases_nexp <- n_cases_nexp_sim[best]
  n_controls_nexp <- n_controls_nexp_sim[best]

  n_cases_exp <- n_cases - n_cases_nexp
  n_controls_exp <- n_controls - n_controls_nexp

  return(data.frame(n_cases_exp, n_cases_nexp, n_controls_exp, n_controls_nexp))
}

#' Estimate the n, using the variance, the number of exposed and non-exposed subjects
#'
#' @param or OR
#' @param var variance
#' @param n_exp number of exposed participants
#' @param n_nexp number of non exposed participants
#'
#' @noRd
.estimate_n_from_or_and_n_exp <- function(or, var, n_exp, n_nexp) {
  # first: uncorrected values with 0
  n_controls_exp_sim1 <- 0:n_exp
  n_controls_nexp_sim1 <- round(n_nexp / (1 + (n_exp - n_controls_exp_sim1) / (or * n_controls_exp_sim1)))
  n_cases_exp_sim1 <- n_exp - n_controls_exp_sim1
  n_cases_nexp_sim1 <- n_nexp - n_controls_nexp_sim1
  # we take the ones without 0 and non-negative
  idx_non_zero <- which(n_cases_nexp_sim1 > 0 & n_controls_nexp_sim1 > 0 & n_cases_exp_sim1 > 0 & n_controls_exp_sim1 > 0) # Posem ">" i no "!=" per treure els negatius!
  n_cases_nexp_sim1 <- n_cases_nexp_sim1[idx_non_zero]
  n_controls_nexp_sim1 <- n_controls_nexp_sim1[idx_non_zero]
  n_cases_exp_sim1 <- n_cases_exp_sim1[idx_non_zero]
  n_controls_exp_sim1 <- n_controls_exp_sim1[idx_non_zero]

  # correcting by 0.5
  n_controls_exp_sim2 <- 0:n_exp
  n_controls_nexp_sim2 <- round((n_nexp + 0.5) - ((n_nexp + 1) * (n_exp - n_controls_exp_sim2 + 0.5)) / ((n_controls_exp_sim2 + 0.5) * or + n_exp - n_controls_exp_sim2 + 0.5))
  n_cases_exp_sim2 <- n_exp - n_controls_exp_sim2
  n_cases_nexp_sim2 <- n_nexp - n_controls_nexp_sim2

  # SELECT THE ONES THAT HAS SOME 0 BUT NO NEGATIVE ONES
  idx_some_zero <- which(
    (n_cases_nexp_sim2 == 0 | n_controls_nexp_sim2 == 0 | n_cases_exp_sim2 == 0 | n_controls_exp_sim2 == 0) &
      (n_cases_nexp_sim2 >= 0 & n_controls_nexp_sim2 >= 0 & n_cases_exp_sim2 >= 0 & n_controls_exp_sim2 >= 0)
  )
  n_cases_nexp_sim2 <- n_cases_nexp_sim2[idx_some_zero]
  n_controls_nexp_sim2 <- n_controls_nexp_sim2[idx_some_zero]
  n_cases_exp_sim2 <- n_cases_exp_sim2[idx_some_zero]
  n_controls_exp_sim2 <- n_controls_exp_sim2[idx_some_zero]

  n_controls_exp_sim <- append(n_controls_exp_sim1, n_controls_exp_sim2)
  n_controls_nexp_sim <- append(n_controls_nexp_sim1, n_controls_nexp_sim2)
  n_cases_exp_sim <- append(n_cases_exp_sim1, n_cases_exp_sim2)
  n_cases_nexp_sim <- append(n_cases_nexp_sim1, n_cases_nexp_sim2)


  some_zero <- n_cases_exp_sim == 0 | n_controls_exp_sim == 0 | n_cases_nexp_sim == 0 | n_controls_nexp_sim == 0
  var_sim <- ifelse(some_zero,
    # Continuity-corrected log-OR variance is a SUM of four reciprocals. Kept bit-for-bit
    # in sync with the live copy in internal_multiple_formulas.R (this file's definition is
    # shadowed by that one under the current source order); the missing ')' after the first
    # term previously folded the whole sum into a single denominator -- fixed so a load-order
    # change cannot reactivate the wrong formula (mirrors the .estimate_n_from_rr twin policy).
    1 / ((n_exp + 1) - (n_controls_exp_sim + 0.5)) + 1 / (n_controls_exp_sim + 0.5) + 1 / ((n_nexp + 1) - (n_controls_nexp_sim + 0.5)) + 1 / (n_controls_nexp_sim + 0.5),
    1 / (n_exp - n_controls_exp_sim) + 1 / n_controls_exp_sim + 1 / (n_nexp - n_controls_nexp_sim) + 1 / n_controls_nexp_sim
  )

  # var_sim = 1 / (n_exp - n_controls_exp_sim) + 1 / n_controls_exp_sim + 1 / (n_nexp - n_controls_nexp_sim) + 1 / n_controls_nexp_sim
  best <- order((var_sim - var)^2)[1]
  n_controls_exp <- n_controls_exp_sim[best]
  n_controls_nexp <- n_controls_nexp_sim[best]
  n_cases_exp <- n_exp - n_controls_exp
  n_cases_nexp <- n_nexp - n_controls_nexp

  return(data.frame(n_cases_exp, n_cases_nexp, n_controls_exp, n_controls_nexp))
}
#' Estimate the number of cases and controls exposed and non-exposed given the risk ratio
#'
#' @param rr RR
#' @param var variance
#' @param n_cases number of cases
#' @param n_controls number of controls
#'
#' @noRd
.estimate_n_from_rr <- function(rr, var, n_cases, n_controls) {
  # uncorrected
  n_cases_nexp_sim1 <- 0:n_cases
  n_controls_nexp_sim1 <- round(n_cases_nexp_sim1 * ((rr * (n_cases + n_controls)) / (n_cases + (rr - 1) * n_cases_nexp_sim1) - 1))
  n_cases_exp_sim1 <- n_cases - n_cases_nexp_sim1
  n_controls_exp_sim1 <- n_controls - n_controls_nexp_sim1

  # we take only positives (no-zero)
  idx_non_zero <- which(n_cases_nexp_sim1 > 0 & n_controls_nexp_sim1 > 0 & n_cases_exp_sim1 > 0 & n_controls_exp_sim1 > 0) # Posem ">" i no "!=" per treure els negatius!
  n_cases_nexp_sim1 <- n_cases_nexp_sim1[idx_non_zero]
  n_controls_nexp_sim1 <- n_controls_nexp_sim1[idx_non_zero]
  n_cases_exp_sim1 <- n_cases_exp_sim1[idx_non_zero]
  n_controls_exp_sim1 <- n_controls_exp_sim1[idx_non_zero]

  # corregint 0.5
  n_cases_nexp_sim2 <- 0:n_cases
  n_controls_nexp_sim2 <- ((n_cases + n_controls - n_cases_nexp_sim2 + 1) - (n_cases + n_controls + 2) * (n_cases - n_cases_nexp_sim2 + 0.5) / ((n_cases_nexp_sim2 + 0.5) * rr + n_cases - n_cases_nexp_sim2 + 0.5))
  n_cases_exp_sim2 <- n_cases - n_cases_nexp_sim2
  n_controls_exp_sim2 <- n_controls - n_controls_nexp_sim2

  # we take the ones with some 0 but non negative
  idx_some_zero <- which(
    (n_cases_nexp_sim2 == 0 | n_controls_nexp_sim2 == 0 | n_cases_exp_sim2 == 0 | n_controls_exp_sim2 == 0) &
      (n_cases_nexp_sim2 >= 0 & n_controls_nexp_sim2 >= 0 & n_cases_exp_sim2 >= 0 & n_controls_exp_sim2 >= 0)
  )
  n_cases_nexp_sim2 <- n_cases_nexp_sim2[idx_some_zero]
  n_controls_nexp_sim2 <- n_controls_nexp_sim2[idx_some_zero]
  n_cases_exp_sim2 <- n_cases_exp_sim2[idx_some_zero]
  n_controls_exp_sim2 <- n_controls_exp_sim2[idx_some_zero]

  #########
  n_controls_exp_sim <- append(n_controls_exp_sim1, n_controls_exp_sim2)
  n_controls_nexp_sim <- append(n_controls_nexp_sim1, n_controls_nexp_sim2)
  n_cases_nexp_sim <- append(n_cases_nexp_sim1, n_cases_nexp_sim2)
  n_cases_exp_sim <- append(n_cases_exp_sim1, n_cases_exp_sim2)

  some_zero <- n_cases_exp_sim == 0 | n_controls_exp_sim == 0 | n_cases_nexp_sim == 0 | n_controls_nexp_sim == 0
  # var(log RR) = 1/a - 1/n1 + 1/c - 1/n2 (arm totals SUBTRACTED); mirrors the fixed
  # live copy .metaumbrella_rr_se_to_or in R/internal_multiple_formulas.R. This helper
  # is currently unused, but is kept in sync so a future load-order change cannot
  # silently reintroduce the added-sign bug. (Also drops the old unconditional
  # var_sim overwrite that had discarded the continuity-corrected branch.)
  var_sim <- ifelse(some_zero,
    1 / ((n_cases + 1) - (n_cases_nexp_sim + 0.5)) - 1 / ((n_cases + 1) + (n_controls + 1) - ((n_cases_nexp_sim + 0.5) + (n_controls_nexp_sim + 0.5))) +
      1 / (n_cases_nexp_sim + 0.5) - 1 / ((n_cases_nexp_sim + 0.5) + (n_controls_nexp_sim + 0.5)),
    1 / (n_cases - n_cases_nexp_sim) - 1 / (n_cases + n_controls - (n_cases_nexp_sim + n_controls_nexp_sim)) +
      1 / n_cases_nexp_sim - 1 / (n_cases_nexp_sim + n_controls_nexp_sim)
  )

  best <- order((var_sim - var)^2)[1]
  n_cases_nexp <- n_cases_nexp_sim[best]
  n_controls_nexp <- n_controls_nexp_sim[best]
  n_cases_exp <- n_cases - n_cases_nexp
  n_controls_exp <- n_controls - n_controls_nexp
  n_exp <- n_cases_exp + n_controls_exp
  n_nexp <- n_cases_nexp + n_controls_nexp

  return(data.frame(n_cases_exp, n_exp, n_cases_nexp, n_nexp))
}

#' Convert two paired t-test value of two independent groups into several effect size measures
#'
#' @param paired_t_exp Paired t-test value of the experimental/exposed group.
#' @param paired_t_nexp Paired t-test value of the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the paired t-test value into a SMD (see details).
#' @param reverse_paired_t a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts paired t-tests of two independent groups value into a Cohen's d (D) and Hedges' g (G) (table 12.2 in Cooper).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate the Cohen's d,** the following formulas are used (Cooper et al., 2019):
#' \deqn{cohen\_d\_exp = paired\_t\_exp * \sqrt{\frac{2 * (1 - r\_pre\_post\_exp)}{n\_exp}}}
#' \deqn{cohen\_d\_nexp = paired\_t\_nexp * \sqrt{\frac{2 * (1 - r\_pre\_post\_nexp)}{n\_nexp}}}
#' \deqn{cohen\_d\_se\_exp = \sqrt{\frac{2 * (1 - r\_pre\_post\_exp)}{n\_exp} + \frac{d\_exp^2}{2 * n\_exp}}}
#' \deqn{cohen\_d\_se\_nexp = \sqrt{\frac{2 * (1 - r\_pre\_post\_nexp)}{n\_nexp} + \frac{d\_nexp^2}{2 * n\_nexp}}}
#' \deqn{cohen\_d = d\_exp - d\_nexp}
#' \deqn{d\_se = \sqrt{cohen\_d\_se\_exp^2 + cohen\_d\_se\_nexp^2}}
#'
#' When \code{pre_post_to_smd = "morris_dz"}, the mean difference is standardized by the standard deviation
#' of the change score and the pre-post correlation is no longer involved (Morris & DeShon, 2002):
#' \deqn{cohen\_d\_exp = \frac{paired\_t\_exp}{\sqrt{n\_exp}}}
#' \deqn{cohen\_d\_nexp = \frac{paired\_t\_nexp}{\sqrt{n\_nexp}}}
#'
#' Note that the Cohen's d obtained from a paired t-test strongly depends on the pre-post correlation.
#' When \code{r_pre_post_exp} / \code{r_pre_post_nexp} are not indicated, a value of 0.8 is assumed and
#' users should conduct sensitivity analyses with other plausible values.
#'
#' **No \code{pool_sd} argument.** Unlike the means and mean-change converters, this function cannot
#' pool the standardizing SD across arms: a paired t-statistic identifies each arm's
#' \eqn{mean\_change / sd\_change} ratio but NOT the ratio of the two arms' SDs, so the pooled
#' standardizer is not recoverable from the reported statistic. Each arm is therefore standardized by
#' its own SD and the two within-group values are subtracted -- a construction that coincides with the
#' pooled one only when the arms' SDs are equal. When rows from this route are combined in one pool
#' with rows that DO use a pooled standardizer, \code{summary(..., flags = TRUE)} raises an
#' informational flag. If the arm SDs are reported, prefer \code{\link{es_from_means_sd_pre_post}} or
#' \code{\link{es_from_mean_change_sd}}.
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 16. Paired: Paired F- or t-test'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @export es_from_paired_t
#'
#' @md
#'
#' @examples
#' es_from_paired_t(paired_t_exp = 2.1, paired_t_nexp = 4.2, n_exp = 20, n_nexp = 22)
es_from_paired_t <- function(paired_t_exp, paired_t_nexp, n_exp, n_nexp,
                             r_pre_post_exp, r_pre_post_nexp,
                             smd_to_cor = "viechtbauer",
                             pre_post_to_smd = "cooper",
                             reverse_paired_t) {
  if (missing(reverse_paired_t)) reverse_paired_t <- rep(FALSE, length(paired_t_exp))
  reverse_paired_t[is.na(reverse_paired_t)] <- FALSE
  if (length(reverse_paired_t) == 1) reverse_paired_t = c(rep(reverse_paired_t, length(paired_t_exp)))
  if (length(reverse_paired_t) != length(paired_t_exp)) stop("The length of the 'reverse_paired_t' argument is incorrectly specified.")

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "paired_t",
    func_name = "es_from_paired_t"
  )

  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(paired_t_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(paired_t_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  r_pre_post_exp <- .guard_r_pre_post(r_pre_post_exp)
  r_pre_post_nexp <- .guard_r_pre_post(r_pre_post_nexp)

  J_exp <- .d_j(n_exp - 1)
  J_nexp <- .d_j(n_nexp - 1)

  # per-row method selection: a vector mixing "morris_dz" and "morris_drm" rows
  # previously fell wholesale into the drm branch (all(...) is a single logical).
  # Recycled to the input length so a scalar method still yields one value per row.
  dz <- rep(pre_post_to_smd == "morris_dz", length.out = length(paired_t_exp))

  # morris dz. metafor SMCC convention (variance built from the corrected g),
  # matching .single_group_pre_post_to_smd so the paired-t and mean-change
  # routes return identical SEs on equivalent inputs
  d_exp_dz <- paired_t_exp / sqrt(n_exp)
  d_nexp_dz <- paired_t_nexp / sqrt(n_nexp)
  d_var_exp_dz <- (1 / n_exp + (J_exp * d_exp_dz)^2 / (2 * n_exp)) / J_exp^2
  d_var_nexp_dz <- (1 / n_nexp + (J_nexp * d_nexp_dz)^2 / (2 * n_nexp)) / J_nexp^2

  # morris drm
  d_exp_drm <- paired_t_exp * sqrt((2 * (1 - r_pre_post_exp)) / n_exp)
  d_nexp_drm <- paired_t_nexp * sqrt((2 * (1 - r_pre_post_nexp)) / n_nexp)
  d_var_exp_drm <- 2 * (1 - r_pre_post_exp) / n_exp + d_exp_drm^2 / (2 * n_exp)
  d_var_nexp_drm <- 2 * (1 - r_pre_post_nexp) / n_nexp + d_nexp_drm^2 / (2 * n_nexp)

  d_exp <- ifelse(dz, d_exp_dz, d_exp_drm)
  d_nexp <- ifelse(dz, d_nexp_dz, d_nexp_drm)
  d_var_exp <- ifelse(dz, d_var_exp_dz, d_var_exp_drm)
  d_var_nexp <- ifelse(dz, d_var_nexp_dz, d_var_nexp_drm)

  # A within-subject arm needs n >= 2 to have any estimable variance. The morris_drm
  # branch carries no J(n-1) factor, so unlike morris_dz it stays finite at n = 1 (and
  # blows up to Inf at n = 0), leaking a spurious d/g/logOR/r downstream. NA such arms
  # so this route matches the mean-change route (which NAs via the n >= 2 kernel guard).
  bad_exp <- !(is.finite(n_exp) & n_exp >= 2)
  bad_nexp <- !(is.finite(n_nexp) & n_nexp >= 2)
  d_exp[bad_exp] <- NA_real_; d_var_exp[bad_exp] <- NA_real_
  d_nexp[bad_nexp] <- NA_real_; d_var_nexp[bad_nexp] <- NA_real_

  g_exp <- J_exp * d_exp
  g_nexp <- J_nexp * d_nexp

  g_var_exp <- J_exp^2 * d_var_exp
  g_var_nexp <- J_nexp^2 * d_var_nexp

  d <- d_exp - d_nexp
  d_se <- sqrt(d_var_exp + d_var_nexp)

  es <- .es_from_d(
    d = d, d_se = d_se, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, reverse = reverse_paired_t
  )

  es$g <- ifelse(reverse_paired_t, g_nexp - g_exp, g_exp - g_nexp)
  es$g_se <- sqrt(g_var_exp + g_var_nexp)
  es$g_ci_lo <- es$g - qt(.975, n_exp + n_nexp - 2) * es$g_se
  es$g_ci_up <- es$g + qt(.975, n_exp + n_nexp - 2) * es$g_se

  es$info_used <- "paired_t"
  return(es)
}

#' Convert two paired t-test p-value obtained from two independent groups into several effect size measures
#'
#' @param paired_t_pval_exp P-value of the paired t-test value of the experimental/exposed group.
#' @param paired_t_pval_nexp P-value of the paired t-test value of the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the paired t-test value into a SMD (see details).
#' @param reverse_paired_t_pval a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the p-values of two paired t-test obtained from two independent groups value into a Cohen's d (D) and Hedges' g (G) (table 12.2 in Cooper).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate the Cohen's d,** the following formulas are used (Cooper et al., 2019):
#' This function converts a Student's t-test value into a Cohen's d (table 12.2 in Cooper).
#' \deqn{paired\_t\_exp = qt(\frac{paired\_t\_pval\_exp}{2}, df = n\_exp - 1) * \sqrt{\frac{2 * (1 - r\_pre\_post\_exp)}{n\_exp}}}
#' \deqn{paired\_t\_nexp = qt(\frac{paired\_t\_pval\_nexp}{2}, df = n\_nexp - 1) * \sqrt{\frac{2 * (1 - r\_pre\_post\_nexp)}{n\_nexp}}}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 16. Paired: Paired F- or t-test'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_paired_t_pval
#'
#' @md
#'
#' @examples
#' es_from_paired_t_pval(paired_t_pval_exp = 0.4, paired_t_pval_nexp = 0.01, n_exp = 19, n_nexp = 22)
es_from_paired_t_pval <- function(paired_t_pval_exp, paired_t_pval_nexp, n_exp, n_nexp,
                                  r_pre_post_exp, r_pre_post_nexp,
                                  smd_to_cor = "viechtbauer",
                                  pre_post_to_smd = "cooper",
                                  reverse_paired_t_pval) {
  if (missing(reverse_paired_t_pval)) reverse_paired_t_pval <- rep(FALSE, length(paired_t_pval_exp))
  reverse_paired_t_pval[is.na(reverse_paired_t_pval)] <- FALSE
  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(paired_t_pval_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(paired_t_pval_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  paired_t_exp <- qt(p = paired_t_pval_exp / 2, df = n_exp - 1, lower.tail = FALSE)

  paired_t_nexp <- qt(p = paired_t_pval_nexp / 2, df = n_nexp - 1, lower.tail = FALSE)

  es <- es_from_paired_t(
    paired_t_exp = paired_t_exp, paired_t_nexp = paired_t_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp, r_pre_post_nexp = r_pre_post_nexp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    reverse_paired_t = reverse_paired_t_pval
  )

  es$info_used <- "paired_t_pval"
  return(es)
}

#' Convert two paired ANOVA f value of two independent groups into several effect size measures
#'
#' @param paired_f_exp Paired ANOVA F value of the experimental/exposed group.
#' @param paired_f_nexp Paired ANOVA F value of the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the paired t-test value into a SMD (see details).
#' @param reverse_paired_f a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the paired F-test obtained from two independent groups value into a Cohen's d (D) and Hedges' g (G) (table 12.2 in Cooper).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate the Cohen's d,** the following formulas are used (Cooper et al., 2019):
#' This function converts a Student's t-test value into a Cohen's d (table 12.2 in Cooper).
#' \deqn{paired\_t\_exp = \sqrt{paired\_f\_exp}}
#' \deqn{paired\_t\_nexp = \sqrt{paired\_f\_nexp}}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_paired_t}()} are applied.
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 16. Paired: Paired F- or t-test'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_paired_f
#'
#' @md
#'
#' @examples
#' es_from_paired_f(paired_f_exp = 2.1, paired_f_nexp = 4.2, n_exp = 20, n_nexp = 22)
es_from_paired_f <- function(paired_f_exp, paired_f_nexp, n_exp, n_nexp,
                             r_pre_post_exp, r_pre_post_nexp,
                             smd_to_cor = "viechtbauer",
                             pre_post_to_smd = "cooper",
                             reverse_paired_f) {
  if (missing(reverse_paired_f)) reverse_paired_f <- rep(FALSE, length(paired_f_exp))
  reverse_paired_f[is.na(reverse_paired_f)] <- FALSE
  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(paired_f_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(paired_f_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8


  paired_t_exp <- sqrt(paired_f_exp)

  paired_t_nexp <- sqrt(paired_f_nexp)

  es <- es_from_paired_t(
    paired_t_exp = paired_t_exp, paired_t_nexp = paired_t_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp,
    r_pre_post_nexp = r_pre_post_nexp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    reverse_paired_t = reverse_paired_f
  )

  es$info_used <- "paired_f"
  return(es)
}

#' Convert two paired ANOVA f p-value of two independent groups into several effect size measures
#'
#' @param paired_f_pval_exp P-value of the paired ANOVA F of the experimental/exposed group.
#' @param paired_f_pval_nexp P-value of the paired ANOVA F of the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the paired t-test value into a SMD (see details).
#' @param reverse_paired_f_pval a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the p-values of two paired F-test obtained from two independent groups value into a Cohen's d (D) and Hedges' g (G) (table 12.2 in Cooper).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate the Cohen's d,** the following formulas are used (Cooper et al., 2019):
#' This function converts a Student's t-test value into a Cohen's d (table 12.2 in Cooper).
#' \deqn{paired\_t\_exp = qt(\frac{paired\_f\_pval\_exp}{2}, df = n\_exp - 1) * \sqrt{\frac{2 * (1 - r_pre_post_exp)}{n_exp}}}
#' \deqn{paired\_t\_nexp = qt(\frac{paired\_f\_pval\_nexp}{2}, df = n\_nexp - 1) * \sqrt{\frac{2 * (1 - r_pre_post_nexp)}{n_nexp}}}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_paired_t}()} are applied.
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 16. Paired: Paired F- or t-test'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_paired_f_pval
#'
#' @md
#'
#' @examples
#' es_from_paired_f_pval(paired_f_pval_exp = 0.4, paired_f_pval_nexp = 0.01, n_exp = 19, n_nexp = 22)
es_from_paired_f_pval <- function(paired_f_pval_exp, paired_f_pval_nexp, n_exp, n_nexp,
                                  r_pre_post_exp, r_pre_post_nexp,
                                  smd_to_cor = "viechtbauer",
                                  pre_post_to_smd = "cooper",
                                  reverse_paired_f_pval) {
  if (missing(reverse_paired_f_pval)) reverse_paired_f_pval <- rep(FALSE, length(paired_f_pval_exp))
  reverse_paired_f_pval[is.na(reverse_paired_f_pval)] <- FALSE
  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(paired_f_pval_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(paired_f_pval_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  es <- es_from_paired_t_pval(
    paired_t_pval_exp = paired_f_pval_exp,
    paired_t_pval_nexp = paired_f_pval_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp,
    r_pre_post_nexp = r_pre_post_nexp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    reverse_paired_t_pval = reverse_paired_f_pval
  )

  es$info_used <- "paired_f_pval"
  return(es)
}

# Bounds-safe lookup into a Wan (2014) / Hozo (2005) constant table. At index 0 R
# silently DROPS the element (and does negative indexing at index < 0), which shortens
# the result vector so the enclosing ifelse() recycles a constant onto the WRONG (valid)
# row -- silently corrupting a neighbouring study's SD/effect size. Mapping every
# out-of-range index to NA keeps the result the same length as the input, so each row
# keeps its own constant and an out-of-range row is NA (n <= 1 for the IQR/quartile
# tables, n <= 0 for the range table). This complements the Tier-1 V16 check in
# internal_flags.R, which flags n == 1 rows as invalid (within-group variance undefined).
.wan_table_lookup <- function(tbl, idx) {
  idx <- as.integer(idx)
  idx[!is.na(idx) & (idx < 1L | idx > length(tbl))] <- NA_integer_
  tbl[idx]
}

#' Convert median and interquartile range of two independent groups into several effect size measures
#'
#' @param q1_exp first quartile of the experimental/exposed group.
#' @param med_exp median value of the experimental/exposed group.
#' @param q3_exp third quartile of the experimental/exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param q1_nexp first quartile of the non-experimental/non-exposed group.
#' @param med_nexp median value of the non-experimental/non-exposed group.
#' @param q3_nexp third quartile of the non-experimental/non-exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the generated \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_med a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function first converts a Cohen's d (D), Hedges' g (G) and mean difference (MD)
#' from the medians and interquartile ranges of two independent groups.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **This function recreates means+SD of the two groups**  (Wan et al., 2014):
#' \deqn{mean\_exp = \frac{q1\_exp + med\_exp + q3\_exp}{3}}
#' \deqn{mean\_nexp = \frac{q1\_nexp + med\_nexp + q3\_nexp}{3}}
#' \deqn{mean\_sd\_exp = \frac{q3\_exp - q1\_exp}{2*qnorm(\frac{0.75*n\_exp - 0.125}{n\_exp+0.25})}}
#' \deqn{mean\_sd\_nexp = \frac{q3\_nexp - q1\_nexp}{2*qnorm(\frac{0.75*n\_nexp - 0.125}{n\_nexp+0.25})}}
#'
#' Note that if the group sample size is inferior to 50, a correction is applied to estimate the standard deviation.
#'
#' **From these means+SD, the function computes MD, D and G** using formulas
#' described in \code{\link{es_from_means_sd}()}.
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @references
#' Wan, X., Wang, W., Liu, J. et al. Estimating the sample mean and standard deviation from the sample size, median, range and/or interquartile range. BMC Med Res Methodol 14, 135 (2014). https://doi.org/10.1186/1471-2288-14-135
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' @export es_from_med_quarts
#'
#' @md
#'
#' @examples
#' es_from_med_quarts(
#'   q1_exp = 1335, med_exp = 1400,
#'   q3_exp = 1765, n_exp = 40,
#'   q1_nexp = 1481, med_nexp = 1625,
#'   q3_nexp = 1800, n_nexp = 40
#' )
es_from_med_quarts <- function(q1_exp, med_exp, q3_exp, n_exp,
                               q1_nexp, med_nexp, q3_nexp, n_nexp,
                               smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_med) {
  if (missing(reverse_med)) reverse_med <- rep(FALSE, length(q1_exp))
  reverse_med[is.na(reverse_med)] <- FALSE


  e <- c(
    0.990, 1.144, 1.206, 1.239, 1.260,
    1.274, 1.284, 1.292, 1.298, 1.303,
    1.307, 1.311, 1.313, 1.316, 1.318,
    1.320, 1.322, 1.323, 1.324, 1.326,
    1.327, 1.328, 1.329, 1.330, 1.330,
    1.331, 1.332, 1.332, 1.333, 1.333,
    1.334, 1.334, 1.335, 1.335, 1.336,
    1.336, 1.336, 1.337, 1.337, 1.337,
    1.338, 1.338, 1.338, 1.338, 1.339,
    1.339, 1.339, 1.339, 1.339, 1.340
  )

  mean_exp <- (q1_exp + med_exp + q3_exp) / 3
  Q_exp <- (n_exp - 1) / 4
  cor_exp <- ifelse(Q_exp <= 50,
    .wan_table_lookup(e, ceiling(Q_exp)),
    (2 * qnorm((0.75 * n_exp - 0.125) / (n_exp + 0.25)))
  )
  sd_exp <- (q3_exp - q1_exp) / cor_exp

  mean_nexp <- (q1_nexp + med_nexp + q3_nexp) / 3
  Q_nexp <- (n_nexp - 1) / 4
  cor_nexp <- ifelse(Q_nexp <= 50,
    .wan_table_lookup(e, ceiling(Q_nexp)),
    (2 * qnorm((0.75 * n_nexp - 0.125) / (n_nexp + 0.25)))
  )
  sd_nexp <- (q3_nexp - q1_nexp) / cor_nexp

  es <- es_from_means_sd(
    mean_exp = mean_exp, mean_nexp = mean_nexp,
    mean_sd_exp = sd_exp, mean_sd_nexp = sd_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_means = reverse_med
  )

  es$info_used <- "med_quarts"

  return(es)
}

#' Convert median, range and interquartile range of two independent groups into several effect size measures
#'
#' @param min_exp minimum value of the experimental/exposed group.
#' @param q1_exp first quartile of the experimental/exposed group.
#' @param med_exp median value of the experimental/exposed group.
#' @param q3_exp third quartile of the experimental/exposed group.
#' @param max_exp maximum value of the experimental/exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param min_nexp minimum value of the non-experimental/non-exposed group.
#' @param q1_nexp first quartile of the non-experimental/non-exposed group.
#' @param med_nexp median value of the non-experimental/non-exposed group.
#' @param q3_nexp third quartile of the non-experimental/non-exposed group.
#' @param max_nexp maximum value of the non-experimental/non-exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the generated \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_med a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function first converts a Cohen's d (D), Hedges' g (G) and mean difference (MD)
#' from the medians, ranges, and interquartile ranges of two independent groups.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **This function recreates means+SD of the two groups**  (Wan et al., 2014):
#' \deqn{mean\_exp = \frac{min\_exp + 2*q1\_exp + 2*med\_exp + 2*q3\_exp + max\_exp}{8}}
#' \deqn{mean\_nexp = \frac{min\_nexp + 2*q1\_nexp + 2*med\_nexp + 2*q3\_nexp + max\_nexp}{8}}
#' \deqn{mean\_sd\_exp = \frac{max\_exp - min\_exp}{4*qnorm(\frac{n\_exp-0.375}{n\_exp+0.25})} + \frac{q3\_exp-q1\_exp}{4*qnorm(\frac{0.75*n\_exp-0.125}{n\_exp+0.25})}}
#' \deqn{mean\_sd\_nexp = \frac{max\_nexp - min\_nexp}{4*qnorm(\frac{n\_nexp-0.375}{n\_nexp+0.25})} + \frac{q3\_nexp-q1\_nexp}{4*qnorm(\frac{0.75*n\_nexp-0.125}{n\_nexp+0.25})}}
#'
#' Note that if the group sample size is inferior to 50, a correction is applied to estimate the standard deviation.
#'
#' **From these means+SD, the function computes MD, D and G** using formulas
#' described in \code{\link{es_from_means_sd}()}.
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab \cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab MD + D + G\cr
#'  \tab OR + R + Z \cr
#'  \code{required input data} \tab See 'Section 12. Median, range and/or interquartile range'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Wan, X., Wang, W., Liu, J. et al. Estimating the sample mean and standard deviation from the sample size, median, range and/or interquartile range. BMC Med Res Methodol 14, 135 (2014). https://doi.org/10.1186/1471-2288-14-135
#'
#' @export es_from_med_min_max_quarts
#'
#' @md
#'
#' @examples
#' es_from_med_min_max_quarts(
#'   min_exp = 1102, q1_exp = 1335,
#'   med_exp = 1400, q3_exp = 1765,
#'   max_exp = 1899, n_exp = 40,
#'   min_nexp = 1181, q1_nexp = 1481,
#'   med_nexp = 1625, q3_nexp = 1800,
#'   max_nexp = 1910, n_nexp = 40
#' )
es_from_med_min_max_quarts <- function(q1_exp, med_exp, q3_exp,
                                       min_exp, max_exp, n_exp,
                                       q1_nexp, med_nexp, q3_nexp,
                                       min_nexp, max_nexp, n_nexp,
                                       smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_med) {
  if (missing(reverse_med)) reverse_med <- rep(FALSE, length(min_exp))
  reverse_med[is.na(reverse_med)] <- FALSE

  list1 <- c(
    0.000, 1.128, 1.693, 2.059, 2.326, 2.534, 2.704,
    2.847, 2.970, 3.078, 3.173, 3.259, 3.336, 3.407,
    3.472, 3.532, 3.588, 3.640, 3.689, 3.735, 3.778,
    3.819, 3.858, 3.895, 3.931, 3.964, 3.997, 4.027,
    4.057, 4.086, 4.113, 4.139, 4.165, 4.189, 4.213,
    4.236, 4.259, 4.280, 4.301, 4.322, 4.341, 4.361,
    4.379, 4.398, 4.415, 4.433, 4.450, 4.466, 4.482,
    4.498
  )

  list2 <- c(
    0.990, 1.144, 1.206, 1.239, 1.260,
    1.274, 1.284, 1.292, 1.298, 1.303,
    1.307, 1.311, 1.313, 1.316, 1.318,
    1.320, 1.322, 1.323, 1.324, 1.326,
    1.327, 1.328, 1.329, 1.330, 1.330,
    1.331, 1.332, 1.332, 1.333, 1.333,
    1.334, 1.334, 1.335, 1.335, 1.336,
    1.336, 1.336, 1.337, 1.337, 1.337,
    1.338, 1.338, 1.338, 1.338, 1.339,
    1.339, 1.339, 1.339, 1.339, 1.340
  )


  mean_exp <- (min_exp + 2 * q1_exp + 2 * med_exp + 2 * q3_exp + max_exp) / 8
  cor_m_exp <- ifelse(n_exp <= 50,
    .wan_table_lookup(list1, n_exp),
    2 * qnorm((n_exp - 0.375) / (n_exp + 0.25))
  )
  Q_exp <- (n_exp - 1) / 4
  cor_q_exp <- ifelse(Q_exp <= 50,
    .wan_table_lookup(list2, ceiling(Q_exp)),
    (2 * qnorm((0.75 * n_exp - 0.125) / (n_exp + 0.25)))
  )

  sd_exp <- ((max_exp - min_exp) / cor_m_exp + (q3_exp - q1_exp) / cor_q_exp) / 2

  mean_nexp <- (min_nexp + 2 * q1_nexp + 2 * med_nexp + 2 * q3_nexp + max_nexp) / 8
  cor_m_nexp <- ifelse(n_nexp <= 50,
    .wan_table_lookup(list1, n_nexp),
    2 * qnorm((n_nexp - 0.375) / (n_nexp + 0.25))
  )
  Q_nexp <- (n_nexp - 1) / 4
  cor_q_nexp <- ifelse(Q_nexp <= 50,
    .wan_table_lookup(list2, ceiling(Q_nexp)),
    (2 * qnorm((0.75 * n_nexp - 0.125) / (n_nexp + 0.25)))
  )

  sd_nexp <- ((max_nexp - min_nexp) / cor_m_nexp + (q3_nexp - q1_nexp) / cor_q_nexp) / 2

  es <- es_from_means_sd(
    mean_exp = mean_exp, mean_nexp = mean_nexp,
    mean_sd_exp = sd_exp, mean_sd_nexp = sd_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_means = reverse_med
  )

  es$info_used <- "med_min_max_quarts"

  return(es)
}

#' Convert median, quartiles, and range of two independent groups into several effect size measures
#'
#' @param min_exp minimum value of the experimental/exposed group.
#' @param med_exp median value of the experimental/exposed group.
#' @param max_exp maximum value of the experimental/exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param min_nexp minimum value of the non-experimental/non-exposed group.
#' @param med_nexp median value of the non-experimental/non-exposed group.
#' @param max_nexp maximum value of the non-experimental/non-exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the generated \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_med a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function first converts a Cohen's d (D), Hedges' g (G)
#' and mean difference (MD) from the medians and ranges of two independent groups.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **This function recreates means+SD of the two groups**  (Wan et al., 2014):
#' \deqn{mean\_exp = \frac{min\_exp + 2*med\_exp + max\_exp}{4} + \frac{min\_exp - 2*med\_exp + max\_exp}{4*n\_exp}}
#' \deqn{mean\_nexp = \frac{min\_nexp + 2*med\_nexp + max\_nexp}{4} + \frac{min\_nexp - 2*med\_nexp + max\_nexp}{4*n\_nexp}}
#' \deqn{mean\_sd\_exp = \frac{max\_exp - min\_exp}{2*qnorm((n\_exp-0.375) / (n\_exp+0.25))}}
#' \deqn{mean\_sd\_nexp = \frac{max\_nexp - min\_nexp}{2*qnorm((n\_nexp-0.375) / (n\_nexp+0.25))}}
#'
#' Note that if the group sample size is inferior to 50, a correction is applied to estimate the standard deviation.
#'
#' **From these means+SD, the function computes MD, D and G** using formulas
#' described in \code{\link{es_from_means_sd}()}.
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' **Importantly,**, authors of the Cochrane Handbook stated
#' "As a general rule, we recommend that ranges should not be used
#' to estimate SDs." (see section 6.5.2.6).
#' It is thus a good practice to explore the consequences of
#' the use of this conversion in sensitivity analyses.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab \cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab MD + D + G\cr
#'  \tab OR + R + Z \cr
#'  \code{required input data} \tab See 'Section 12. Median, range and/or interquartile range'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab \cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab MD + D + G\cr
#'  \tab OR + R + Z \cr
#'  \code{required input data} \tab See 'Section 12. Median, range and/or interquartile range'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Wan, X., Wang, W., Liu, J. et al. Estimating the sample mean and standard deviation from the sample size, median, range and/or interquartile range. BMC Med Res Methodol 14, 135 (2014). https://doi.org/10.1186/1471-2288-14-135
#'
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @export es_from_med_min_max
#'
#' @md
#'
#' @examples
#' es_from_med_min_max(
#'   min_exp = 1335, med_exp = 1400,
#'   max_nexp = 1765, n_exp = 40,
#'   min_nexp = 1481, med_nexp = 1625,
#'   max_exp = 1800, n_nexp = 40
#' )
es_from_med_min_max <- function(min_exp, med_exp, max_exp, n_exp,
                                min_nexp, med_nexp, max_nexp, n_nexp,
                                smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_med) {
  if (missing(reverse_med)) reverse_med <- rep(FALSE, length(min_exp))
  reverse_med[is.na(reverse_med)] <- FALSE

  list1 <- c(
    0.000, 1.128, 1.693, 2.059, 2.326, 2.534, 2.704,
    2.847, 2.970, 3.078, 3.173, 3.259, 3.336, 3.407,
    3.472, 3.532, 3.588, 3.640, 3.689, 3.735, 3.778,
    3.819, 3.858, 3.895, 3.931, 3.964, 3.997, 4.027,
    4.057, 4.086, 4.113, 4.139, 4.165, 4.189, 4.213,
    4.236, 4.259, 4.280, 4.301, 4.322, 4.341, 4.361,
    4.379, 4.398, 4.415, 4.433, 4.450, 4.466, 4.482,
    4.498
  )

  mean_exp <- (min_exp + 2 * med_exp + max_exp) / 4 + (min_exp - 2 * med_exp + max_exp) / (4 * n_exp)
  cor_exp <- ifelse(n_exp <= 50, .wan_table_lookup(list1, n_exp), 2 * qnorm((n_exp - 0.375) / (n_exp + 0.25)))
  sd_exp <- (max_exp - min_exp) / cor_exp

  mean_nexp <- (min_nexp + 2 * med_nexp + max_nexp) / 4 + (min_nexp - 2 * med_nexp + max_nexp) / (4 * n_nexp)
  cor_nexp <- ifelse(n_nexp <= 50, .wan_table_lookup(list1, n_nexp), 2 * qnorm((n_nexp - 0.375) / (n_nexp + 0.25)))
  sd_nexp <- (max_nexp - min_nexp) / cor_nexp

  es <- es_from_means_sd(
    mean_exp = mean_exp, mean_nexp = mean_nexp,
    mean_sd_exp = sd_exp, mean_sd_nexp = sd_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_means = reverse_med
  )

  es$info_used <- "med_min_max"

  return(es)
}

#' Convert a 2x2 table into several effect size measures
#'
#' @param n_cases_exp number of cases/events in the exposed group
#' @param n_cases_nexp number of cases/events in the non exposed group
#' @param n_controls_exp number of controls/no-event in the exposed group
#' @param n_controls_nexp number of controls/no-event in the non exposed group
#' @param table_2x2_to_cor formula used to obtain a correlation coefficient from the contingency table (see details).
#' @param reverse_2x2 a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function first computes (log) odds ratio (OR), (log) risk ratio (RR) and number needed to treat (NNT)
#' from the 2x2 table. Note that if a cell is equal to 0, we applied the typical adjustment (add 0.5) to all cells.
#' This adjustment is used for the OR/RR only; the RD and NNT are obtained from the raw cell counts.
#' Cohen's d (D), Hedges' g (G) and correlation coefficients (R/Z) are then estimated from the OR.
#'
#' **To estimate an OR**, the formulas used (Box 6.4.a in the Cochrane Handbook) are:
#' \deqn{logor = log(\frac{n\_cases\_exp / n\_cases\_nexp}{n\_controls\_exp / n\_controls\_nexp})}
#' \deqn{logor\_se = \sqrt{\frac{1}{n\_cases\_exp} + \frac{1}{n\_cases\_nexp} + \frac{1}{n\_controls\_exp} + \frac{1}{n\_controls\_nexp}}}
#'
#' **To estimate an RR**, the formulas used (Box 6.4.a in the Cochrane Handbook) are:
#' \deqn{logrr = log(\frac{n\_cases\_exp / n\_exp}{n\_cases\_nexp / n\_nexp})}
#' \deqn{logrr\_se = \sqrt{\frac{1}{n\_cases\_exp} - \frac{1}{n\_exp} + \frac{1}{n\_cases\_nexp} - \frac{1}{n\_nexp}}}
#'
#' **To estimate a risk difference (RD) and NNT**, the formulas used are (Wen et al., 2005; Altman, 1998):
#' \deqn{pt = \frac{n\_cases\_exp}{n\_cases\_exp + n\_controls\_exp}}
#' \deqn{pc = \frac{n\_cases\_nexp}{n\_cases\_nexp + n\_controls\_nexp}}
#' \deqn{rd = pc - pt}
#' \deqn{rd\_se = \sqrt{\frac{pt(1-pt)}{n\_exp} + \frac{pc(1-pc)}{n\_nexp}}}
#' \deqn{nnt = \frac{1}{rd}}
#' \deqn{nnt\_se = \frac{rd\_se}{rd^2}}
#' Note that NNT confidence intervals are set to NA when the RD confidence interval crosses zero
#' (discontinuous CI; Altman, 1998).
#'
#' **Direction convention.** The risk difference is defined as \eqn{rd = pc - pt}
#' (control risk minus exposed risk), so a POSITIVE RD means the control group has the
#' higher risk. This is the OPPOSITE direction to the OR and RR produced from the same
#' 2x2 table, which are exposed-over-non-exposed (an OR/RR \eqn{> 1} means the exposed
#' group has the higher risk). Consequently, for the same table, a protective exposure
#' yields \eqn{OR < 1}, \eqn{RR < 1} but \eqn{RD > 0}; keep this in mind when pooling RD
#' alongside OR/RR, and use \code{reverse_2x2} if you need to align the directions.
#'
#' **To convert the 2x2 table into a SMD**,
#' the function estimates an OR value from the 2x2 table (formula above)
#' that is then converted to a SMD
#' (see formula in \code{\link{es_from_or_se}()}).
#'
#' **To convert the 2x2 table into a correlation coefficient**,
#' For now, only the tetrachoric correlation is currently proposed
#' - \code{table_2x2_to_cor = "tetrachoric"}.
#' Given the heavy calculations required for this effect size measure,
#' we relied on the implementation of the formulas of the 'metafor' package. More
#' information can be retrieved here
#' (https://wviechtb.github.io/metafor/reference/escalc.html#-b-measures-for-two-dichotomous-variables).
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR + RR + NNT + RD\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 7. Contingency (2x2) table or proportions'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @export es_from_2x2
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Available from www.training.cochrane.org/handbook.
#'
#' Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
#'
#' Sedgwick, P. (2013). What is number needed to treat (NNT)? Bmj, 347.
#'
#' Altman, D. G. (1998). Confidence intervals for the number needed to treat. BMJ, 317(7168), 1309-1312.
#'
#' Wen, S., Zhang, L., & Yang, B. (2005). Two approaches to incorporate clinical data uncertainty into number needed to treat. Journal of Clinical Pharmacy and Therapeutics, 30(2), 105-109.
#'
#' @examples
#' es_from_2x2(n_cases_exp = 467, n_cases_nexp = 22087, n_controls_exp = 261, n_controls_nexp = 8761)
es_from_2x2 <- function(n_cases_exp, n_cases_nexp,
                        n_controls_exp, n_controls_nexp,
                        table_2x2_to_cor = "tetrachoric", reverse_2x2) {
  if (missing(reverse_2x2)) reverse_2x2 <- rep(FALSE, length(n_cases_exp))
  reverse_2x2[is.na(reverse_2x2)] <- FALSE
  if (length(reverse_2x2) == 1) reverse_2x2 = c(rep(reverse_2x2, length(n_cases_exp)))
  if (length(reverse_2x2) != length(n_cases_exp)) stop("The length of the 'reverse_2x2' argument is incorrectly specified.")

  if (!all(table_2x2_to_cor %in% c("tetrachoric"))) { # , "cooper", "lipsey"
    stop(paste0(
      "'",
      unique(table_2x2_to_cor[!table_2x2_to_cor %in% c("tetrachoric")]), #, "cooper", "lipsey"
      "' not in tolerated values for the 'table_2x2_to_cor' argument.",
      " Possible inputs are: 'tetrachoric', "#'cooper', 'lipsey'
    ))
  }

  # rd from raw counts
  n_exp_raw <- n_cases_exp + n_controls_exp
  n_nexp_raw <- n_cases_nexp + n_controls_nexp
  pc_raw <- n_cases_nexp / n_nexp_raw
  pt_raw <- n_cases_exp / n_exp_raw
  rd <- pc_raw - pt_raw
  rd_se <- sqrt(pt_raw * (1 - pt_raw) / n_exp_raw + pc_raw * (1 - pc_raw) / n_nexp_raw)

  # 0.5 correction for or/rr
  zero <- which(n_cases_exp == 0 | n_cases_nexp == 0 | n_controls_exp == 0 | n_controls_nexp == 0)
  n_cases_exp[zero] <- n_cases_exp[zero] + 0.5
  n_cases_nexp[zero] <- n_cases_nexp[zero] + 0.5
  n_controls_exp[zero] <- n_controls_exp[zero] + 0.5
  n_controls_nexp[zero] <- n_controls_nexp[zero] + 0.5
  n_exp <- n_cases_exp + n_controls_exp
  n_nexp <- n_cases_nexp + n_controls_nexp

  or_raw <- suppressWarnings((n_cases_exp * n_controls_nexp) / (n_controls_exp * n_cases_nexp))
  or <- ifelse(reverse_2x2, 1 / or_raw, or_raw)
  se_or <- suppressWarnings(sqrt(1 / n_cases_exp + 1 / n_controls_exp + 1 / n_cases_nexp + 1 / n_controls_nexp))

  rr_raw <- suppressWarnings((n_cases_exp / n_exp) / (n_cases_nexp / n_nexp))
  rr <- ifelse(reverse_2x2, 1 / rr_raw, rr_raw)
  se_rr <- suppressWarnings(sqrt(1 / n_cases_exp - 1 / n_exp + 1 / n_cases_nexp - 1 / n_nexp))

  logOR <- suppressWarnings(log(or))
  d <- logOR * sqrt(3) / pi
  d_se <- sqrt(se_or^2 * 3 / (pi^2))

  es <- .es_from_d(
    d = d, d_se = d_se,
    n_exp = n_exp,
    n_nexp = n_nexp,
    smd_to_cor = rep("lipsey_cooper", length(d))
  )

  es$logor <- logOR
  es$logor_se <- se_or
  es$logor_ci_lo <- es$logor - qnorm(.975) * es$logor_se
  es$logor_ci_up <- es$logor + qnorm(.975) * es$logor_se

  es$logrr <- log(rr)
  es$logrr_se <- se_rr
  es$logrr_ci_lo <- es$logrr - qnorm(.975) * es$logrr_se
  es$logrr_ci_up <- es$logrr + qnorm(.975) * es$logrr_se

  dat2x2 <- data.frame(
    n_cases_exp = n_cases_exp, n_controls_exp = n_controls_exp,
    n_cases_nexp = n_cases_nexp, n_controls_nexp = n_controls_nexp,
    n_exp = n_exp, n_nexp = n_nexp,
    table_2x2_to_cor = table_2x2_to_cor, reverse_2x2 = reverse_2x2
  )

  nn_miss <- which(
    (dat2x2$table_2x2_to_cor == "lipsey" |
       dat2x2$table_2x2_to_cor == "tetrachoric") &
      !is.na(dat2x2$n_cases_exp) & !is.na(dat2x2$n_controls_exp) &
      !is.na(dat2x2$n_cases_nexp) & !is.na(dat2x2$n_controls_nexp)
  )

  if (length(nn_miss) != 0) {
    # no_cooper = which(dat2x2$table_2x2_to_cor != "cooper")
    #
    # es$r[no_cooper] <- es$r_se[no_cooper] <-
    #   es$r_ci_lo[no_cooper] <- es$r_ci_up[no_cooper] <-
    #   es$z[no_cooper] <- es$z_se[no_cooper] <-
    #   es$z_ci_lo[no_cooper] <- es$z_ci_up[no_cooper] <- NA

    res_tet <- suppressWarnings(t(mapply(.contingency_to_cor,
      n_cases_exp = dat2x2$n_cases_exp[nn_miss],
      n_controls_exp = dat2x2$n_controls_exp[nn_miss],
      n_cases_nexp = dat2x2$n_cases_nexp[nn_miss],
      n_controls_nexp = dat2x2$n_controls_nexp[nn_miss],
      table_2x2_to_cor = dat2x2$table_2x2_to_cor[nn_miss],
      reverse_2x2 = dat2x2$reverse_2x2[nn_miss]
    )))

    es$r[nn_miss] <- res_tet[, 1]
    es$r_se[nn_miss] <- suppressWarnings(sqrt(res_tet[, 2]))
    es$r_ci_lo[nn_miss] <- res_tet[, 3]
    es$r_ci_up[nn_miss] <- res_tet[, 4]

    es$z[nn_miss] <- res_tet[, 5]
    es$z_se[nn_miss] <- suppressWarnings(sqrt(res_tet[, 6]))
    es$z_ci_lo[nn_miss] <- res_tet[, 7]
    es$z_ci_up[nn_miss] <- res_tet[, 8]
  }

  es$rd <- ifelse(reverse_2x2, -rd, rd)
  es$rd_se <- rd_se
  es$rd_ci_lo <- es$rd - qnorm(.975) * rd_se
  es$rd_ci_up <- es$rd + qnorm(.975) * rd_se

  es$nnt <- ifelse(rd == 0, NA, ifelse(reverse_2x2, -1 / rd, 1 / rd))
  es$nnt_se <- ifelse(rd == 0, NA, rd_se / rd^2)
  rd_ci_lo_raw <- rd - qnorm(.975) * rd_se
  rd_ci_up_raw <- rd + qnorm(.975) * rd_se
  crosses_zero <- (rd_ci_lo_raw < 0 & rd_ci_up_raw > 0)
  es$nnt_ci_lo <- ifelse(crosses_zero | rd == 0, NA,
                          ifelse(reverse_2x2, -1 / rd_ci_lo_raw, 1 / rd_ci_up_raw))
  es$nnt_ci_up <- ifelse(crosses_zero | rd == 0, NA,
                          ifelse(reverse_2x2, -1 / rd_ci_up_raw, 1 / rd_ci_lo_raw))

  es$info_used <- "2x2"
  return(es)
}

#' Convert a table with the number of cases and row marginal sums into several effect size measures
#'
#' @param n_cases_exp number of cases/events in the exposed group
#' @param n_cases_nexp number of cases/events in the non exposed group
#' @param n_exp total number of participants in the exposed group
#' @param n_nexp total number of participants in the non exposed group
#' @param table_2x2_to_cor formula used to obtain a correlation coefficient from the contingency table (see details).
#' @param reverse_2x2 a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function uses the number of cases in both the exposed
#' and non-exposed groups and the total number of participants exposed and non-exposed
#' to recreate a 2x2 table.
#' Then relies on the calculations of the \code{\link{es_from_2x2}} function.
#' \deqn{n\_controls\_exp = n\_exp - n\_cases\_exp}
#' \deqn{n\_controls\_nexp = n\_nexp - n\_cases\_nexp}
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR + RR + NNT + RD\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 7. Contingency (2x2) table or proportions'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#'
#' @md
#'
#' @export es_from_2x2_sum
#'
#' @examples
#' es_from_2x2_sum(n_cases_exp = 10, n_exp = 40, n_cases_nexp = 25, n_nexp = 47)
es_from_2x2_sum <- function(n_cases_exp, n_exp, n_cases_nexp, n_nexp,
                            table_2x2_to_cor = "tetrachoric", reverse_2x2) {
  if (missing(reverse_2x2)) reverse_2x2 <- rep(FALSE, length(n_cases_exp))
  reverse_2x2[is.na(reverse_2x2)] <- FALSE

  es <- es_from_2x2(
    n_cases_exp = n_cases_exp,
    n_cases_nexp = n_cases_nexp,
    n_controls_exp = n_exp - n_cases_exp,
    n_controls_nexp = n_nexp - n_cases_nexp,
    table_2x2_to_cor = table_2x2_to_cor,
    reverse_2x2 = reverse_2x2
  )

  es$info_used <- "2x2_sum"

  return(es)
}

#' Convert the proportion of occurrence of a binary event in two independent groups into several effect size measures
#'
#' @param prop_cases_exp proportion of cases/events in the exposed group (ranging from 0 to 1)
#' @param prop_cases_nexp proportion of cases/events in the non-exposed group (ranging from 0 to 1)
#' @param n_exp total number of participants in the exposed group
#' @param n_nexp total number of participants in the non exposed group
#' @param table_2x2_to_cor formula used to obtain a correlation coefficient from the contingency table (see details).
#' @param reverse_prop a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#'
#' @details
#' This function uses the proportions and sample size to
#' recreate the 2x2 table, and
#' then relies on the calculations of the \code{\link{es_from_2x2_sum}()} function.
#'
#' The formulas used is to obtain the 2x2 table are
#' \deqn{n\_cases\_exp = prop\_cases\_exp * n\_exp}
#' \deqn{n\_cases\_nexp = prop\_cases\_nexp * n\_nexp}
#' \deqn{n\_controls\_exp = (1 - prop\_cases\_exp) * n\_exp}
#' \deqn{n\_controls\_nexp = (1 - prop\_cases\_nexp) * n\_nexp}
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR + RR + NNT + RD\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 7. Contingency (2x2) table or proportions'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_2x2_prop
#'
#'
#' @md
#'
#' @examples
#' es_from_2x2_prop(prop_cases_exp = 0.80, prop_cases_nexp = 0.60, n_exp = 10, n_nexp = 20)
es_from_2x2_prop <- function(prop_cases_exp, prop_cases_nexp, n_exp, n_nexp,
                             table_2x2_to_cor = "tetrachoric", reverse_prop) {
  if (missing(reverse_prop)) reverse_prop <- rep(FALSE, length(prop_cases_exp))
  reverse_prop[is.na(reverse_prop)] <- FALSE


  n_cases_exp <- round(prop_cases_exp * n_exp)
  n_cases_nexp <- round(prop_cases_nexp * n_nexp)
  n_controls_exp <- n_exp - n_cases_exp
  n_controls_nexp <- n_nexp - n_cases_nexp

  es <- es_from_2x2(
    n_cases_exp = n_cases_exp, n_cases_nexp = n_cases_nexp,
    n_controls_exp = n_controls_exp, n_controls_nexp = n_controls_nexp,
    table_2x2_to_cor = table_2x2_to_cor, reverse_2x2 = reverse_prop
  )

  es$info_used <- "2x2_prop"

  return(es)
}

#' Convert means and standard deviations of two independent groups obtained from an ANCOVA model into several effect size measures
#'
#' @param ancova_mean_exp adjusted mean of participants in the experimental/exposed group.
#' @param ancova_mean_nexp adjusted mean of participants in the non-experimental/non-exposed group.
#' @param ancova_mean_sd_exp adjusted standard deviation of participants in the experimental/exposed group.
#' @param ancova_mean_sd_nexp adjusted standard deviation of participants in the non-experimental/non-exposed group.
#' @param cov_outcome_r correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function first computes an "adjusted" mean difference (MD),
#' Cohen's d (D) and Hedges' g (G) from the adjusted means and standard deviations.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' This function start by estimating the non-adjusted standard deviation of the two groups (formula 12.24 in Cooper);
#' \deqn{mean\_sd\_exp = \frac{ancova\_mean\_sd\_exp}{\sqrt{1 - cov\_outcome\_r^2}}}
#' \deqn{mean\_sd\_nexp = \frac{ancova\_mean\_sd\_nexp}{\sqrt{1 - cov\_outcome\_r^2}}}
#'
#' **To obtain the mean difference**, the following formulas are used (authors calculations):
#' \deqn{md = ancova\_mean\_exp - ancova\_mean\_nexp}
#' \deqn{md\_se = \sqrt{\frac{ancova\_mean\_sd\_exp^2}{n\_exp} + \frac{ancova\_mean\_sd\_nexp^2}{n\_nexp}}}
#' \deqn{md\_ci\_lo = md - md\_se * qt(.975, n\_exp+n\_nexp-2-n\_cov\_ancova)}
#' \deqn{md\_ci\_up = md + md\_se * qt(.975, n\_exp+n\_nexp-2-n\_cov\_ancova)}
#'
#' **To obtain the Cohen's d**, the following formulas are used (table 12.3 in Cooper):
#' \deqn{mean\_sd\_pooled = \sqrt{\frac{(n\_exp - 1) * ancova\_mean\_sd\_exp^2 + (n\_nexp - 1) * ancova\_mean\_sd\_nexp^2}{n\_exp+n\_nexp-2}}}
#' \deqn{cohen\_d =  \frac{ancova\_mean\_exp - ancova\_mean\_nexp}{mean\_sd\_pooled}}
#' \deqn{cohen\_d\_se = \sqrt{\frac{(n\_exp+n\_nexp)*(1-cov\_outcome\_r^2)}{n\_exp*n\_nexp} + \frac{cohen\_d^2}{2(n\_exp+n\_nexp)}}}
#' \deqn{cohen\_d\_ci\_lo = cohen\_d - cohen\_d\_se * qt(.975, n\_exp + n\_nexp - 2 - n\_cov\_ancova)}
#' \deqn{cohen\_d\_ci\_up = cohen\_d + cohen\_d\_se * qt(.975, n\_exp + n\_nexp - 2 - n\_cov\_ancova)}
#'
#' **To estimate other effect size measures**,
#' Calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @note
#' The sampling variance follows Cooper's eq. 12.26 and assumes the covariate is
#' balanced across groups. It omits the covariate-imbalance ("leverage") term of
#' the exact ANCOVA variance,
#' \eqn{\sigma^2_{res}\,(1/n\_exp + 1/n\_nexp + (\bar{x}\_exp-\bar{x}\_nexp)^2 / SS_x)},
#' which is not recoverable from summary statistics, and treats
#' \code{cov_outcome_r} as known. In balanced/randomised designs the omission is
#' negligible; in observational or otherwise covariate-imbalanced designs the
#' standard error is a lower bound (anti-conservative), while the point estimate
#' remains unbiased. See Lai and Kelley (2012).
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Lai, K., & Kelley, K. (2012). Accuracy in parameter estimation for ANCOVA and ANOVA contrasts: Sample size planning via narrow confidence intervals. British Journal of Mathematical and Statistical Psychology, 65(2), 350-370.
#'
#' @export es_from_ancova_means_sd
#'
#' @md
#'
#' @examples
#' es_from_ancova_means_sd(
#'   n_exp = 55, n_nexp = 55,
#'   ancova_mean_exp = 2.3, ancova_mean_sd_exp = 1.2,
#'   ancova_mean_nexp = 1.9, ancova_mean_sd_nexp = 0.9,
#'   cov_outcome_r = 0.2, n_cov_ancova = 3
#' )
#'
es_from_ancova_means_sd <- function(n_exp, n_nexp,
                                    ancova_mean_exp, ancova_mean_nexp,
                                    ancova_mean_sd_exp, ancova_mean_sd_nexp,
                                    cov_outcome_r, n_cov_ancova,
                                    smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_ancova_means) {
  if (missing(reverse_ancova_means)) reverse_ancova_means <- rep(FALSE, length(ancova_mean_exp))
  reverse_ancova_means[is.na(reverse_ancova_means)] <- FALSE
  if (length(reverse_ancova_means) == 1) reverse_ancova_means = c(rep(reverse_ancova_means, length(ancova_mean_exp)))
  if (length(reverse_ancova_means) != length(ancova_mean_exp)) stop("The length of the 'reverse_ancova_means' argument is incorrectly specified.")

  df <- n_exp + n_nexp - 2 - n_cov_ancova
  ancova_mean_sd_pooled <- sqrt(((n_exp - 1) * ancova_mean_sd_exp^2 +
    (n_nexp - 1) * ancova_mean_sd_nexp^2) / (n_exp + n_nexp - 2))

  es <- es_from_ancova_means_sd_pooled_adj(
    ancova_mean_exp = ancova_mean_exp, ancova_mean_nexp = ancova_mean_nexp,
    ancova_mean_sd_pooled = ancova_mean_sd_pooled, cov_outcome_r = cov_outcome_r,
    n_cov_ancova = n_cov_ancova, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_means = reverse_ancova_means
  )


  es$md <- ifelse(reverse_ancova_means,
                  ancova_mean_nexp - ancova_mean_exp,
                  ancova_mean_exp - ancova_mean_nexp)
  es$md_se <- sqrt(ancova_mean_sd_exp^2/n_exp + ancova_mean_sd_nexp^2/n_nexp)
  es$md_ci_lo <- es$md - qt(.975, df) * es$md_se
  es$md_ci_up <- es$md + qt(.975, df) * es$md_se

  es$info_used <- "ancova_means_sd"

  return(es)
}
#' Convert means and standard errors of two independent groups obtained from an ANCOVA model into several effect size measures
#'
#' @param ancova_mean_exp adjusted mean of participants in the experimental/exposed group.
#' @param ancova_mean_nexp adjusted mean of participants in the non-experimental/non-exposed group.
#' @param ancova_mean_se_exp adjusted standard error of participants in the experimental/exposed group.
#' @param ancova_mean_se_nexp adjusted standard error of participants in the non-experimental/non-exposed group.
#' @param cov_outcome_r correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the adjusted means standard errors of two independent groups
#' into standard deviations,
#' and then relies on the calculations of the \code{\link{es_from_ancova_means_sd}} function.
#'
#' **To convert the standard errors into standard deviations**, the following formula is used.
#' \deqn{ancova\_mean\_sd\_exp = ancova\_mean\_se\_exp * \sqrt{n\_exp}}
#' \deqn{ancova\_mean\_sd\_nexp = ancova\_mean\_se\_nexp * \sqrt{n\_nexp}}
#' Calculations of the \code{\link{es_from_ancova_means_sd}()} are then applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
#'
#' @export es_from_ancova_means_se
#'
#' @md
#'
#' @examples
#' es_from_ancova_means_se(
#'   n_exp = 55, n_nexp = 55,
#'   ancova_mean_exp = 2.3, ancova_mean_se_exp = 1.2,
#'   ancova_mean_nexp = 1.9, ancova_mean_se_nexp = 0.9,
#'   cov_outcome_r = 0.2, n_cov_ancova = 3
#' )
es_from_ancova_means_se <- function(n_exp, n_nexp,
                                    ancova_mean_exp, ancova_mean_nexp,
                                    ancova_mean_se_exp, ancova_mean_se_nexp,
                                    cov_outcome_r, n_cov_ancova,
                                    smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_ancova_means) {
  if (missing(reverse_ancova_means)) reverse_ancova_means <- rep(FALSE, length(ancova_mean_exp))
  reverse_ancova_means[is.na(reverse_ancova_means)] <- FALSE



  ancova_mean_sd_exp <- ancova_mean_se_exp * sqrt(n_exp)
  ancova_mean_sd_nexp <- ancova_mean_se_nexp * sqrt(n_nexp)


  es <- es_from_ancova_means_sd(
    n_exp = n_exp, n_nexp = n_nexp,
    ancova_mean_exp = ancova_mean_exp, ancova_mean_nexp = ancova_mean_nexp,
    ancova_mean_sd_exp = ancova_mean_sd_exp, ancova_mean_sd_nexp = ancova_mean_sd_nexp,
    cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_means = reverse_ancova_means
  )

  es$info_used <- "ancova_means_se"

  return(es)
}
#' Convert means and 95% CIs of two independent groups obtained from an ANCOVA model into several effect size measures
#'
#' @param ancova_mean_exp adjusted mean of participants in the experimental/exposed group.
#' @param ancova_mean_nexp adjusted mean of participants in the non-experimental/non-exposed group.
#' @param ancova_mean_ci_lo_exp lower bound of the adjusted 95% CI of the mean of the experimental/exposed group
#' @param ancova_mean_ci_up_exp upper bound of the adjusted 95% CI of the mean of the experimental/exposed group
#' @param ancova_mean_ci_lo_nexp lower bound of the adjusted 95% CI of the mean of the non-experimental/non-exposed group.
#' @param ancova_mean_ci_up_nexp upper bound of the adjusted 95% CI of the mean of the non-experimental/non-exposed group.
#' @param cov_outcome_r correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the adjusted means 95% CI of two independent groups
#' into a standard error,
#' and then relies on the calculations of the \code{\link{es_from_ancova_means_se}()} function.
#'
#' **To convert the 95% CIs into standard errors,** the following formula is used (table 12.3 in Cooper):
#' \deqn{ancova\_mean\_se\_exp = \frac{ancova\_mean\_ci\_up\_exp - ancova\_mean\_ci\_lo\_exp}{2 * qt(0.975, df = n\_exp + n\_nexp - 2 - n\_cov\_ancova)}}
#' \deqn{ancova\_mean\_se\_nexp = \frac{ancova\_mean\_ci\_up\_nexp - ancova\_mean\_ci\_lo\_nexp}{2 * qt(0.975, df = n\_exp + n\_nexp - 2 - n\_cov\_ancova)}}
#' Calculations of the \code{\link{es_from_ancova_means_se}()} are then applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_ancova_means_ci
#'
#' @md
#'
#' @examples
#' es_from_ancova_means_ci(
#'   n_exp = 55, n_nexp = 55, cov_outcome_r = 0.5, n_cov_ancova = 4,
#'   ancova_mean_exp = 25, ancova_mean_ci_lo_exp = 15, ancova_mean_ci_up_exp = 35,
#'   ancova_mean_nexp = 18, ancova_mean_ci_lo_nexp = 12, ancova_mean_ci_up_nexp = 24
#' )
es_from_ancova_means_ci <- function(n_exp, n_nexp,
                                    ancova_mean_exp, ancova_mean_ci_lo_exp, ancova_mean_ci_up_exp,
                                    ancova_mean_nexp, ancova_mean_ci_lo_nexp, ancova_mean_ci_up_nexp,
                                    cov_outcome_r, n_cov_ancova,
                                    smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_ancova_means) {
  if (missing(reverse_ancova_means)) reverse_ancova_means <- rep(FALSE, length(ancova_mean_exp))
  reverse_ancova_means[is.na(reverse_ancova_means)] <- FALSE

  df_model <- n_exp + n_nexp - 2 - n_cov_ancova

  ancova_se_exp <- (ancova_mean_ci_up_exp - ancova_mean_ci_lo_exp) / (2 * qt(0.975, df_model))
  ancova_se_nexp <- (ancova_mean_ci_up_nexp - ancova_mean_ci_lo_nexp) / (2 * qt(0.975, df_model))

  es <- es_from_ancova_means_se(
    n_exp = n_exp, n_nexp = n_nexp,
    ancova_mean_exp = ancova_mean_exp,
    ancova_mean_se_exp = ancova_se_exp,
    ancova_mean_nexp = ancova_mean_nexp,
    ancova_mean_se_nexp = ancova_se_nexp,
    cov_outcome_r = cov_outcome_r, n_cov_ancova = n_cov_ancova,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse_ancova_means = reverse_ancova_means
  )

  es$info_used <- "ancova_means_ci"

  return(es)
}
#' Convert means and adjusted pooled standard deviation of two independent groups obtained from an ANCOVA model into several effect size measures
#'
#' @param ancova_mean_exp adjusted mean of participants in the experimental/exposed group.
#' @param ancova_mean_nexp adjusted mean of participants in the non-experimental/non-exposed group.
#' @param ancova_mean_sd_pooled adjusted pooled standard deviation.
#' @param cov_outcome_r correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the adjusted pooled standard deviations of two independent groups
#' into a crude pooled standard deviation.
#' and then relies on the calculations of the \code{\link{es_from_ancova_means_sd_pooled_crude}()} function.
#'
#' **To convert the adjusted pooled SD into a crude pooled SD** (table 12.3 in Cooper):
#' \deqn{mean\_sd\_pooled =  \frac{ancova\_mean\_sd\_pooled}{\sqrt{1 - cov\_outcome\_r^2}}}
#'
#' Calculations of the \code{\link{es_from_ancova_means_sd_pooled_crude}()} are then applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_ancova_means_sd_pooled_adj
#'
#' @md
#'
#' @examples
#' es_from_ancova_means_sd_pooled_adj(
#'   ancova_mean_exp = 98, ancova_mean_nexp = 87,
#'   ancova_mean_sd_pooled = 17, cov_outcome_r = 0.2,
#'   n_cov_ancova = 3, n_exp = 20, n_nexp = 20
#' )
es_from_ancova_means_sd_pooled_adj <- function(ancova_mean_exp, ancova_mean_nexp,
                                               ancova_mean_sd_pooled, cov_outcome_r,
                                               n_cov_ancova, n_exp, n_nexp,
                                               smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_ancova_means) {
  if (missing(reverse_ancova_means)) reverse_ancova_means <- rep(FALSE, length(ancova_mean_exp))
  reverse_ancova_means[is.na(reverse_ancova_means)] <- FALSE
  if (length(reverse_ancova_means) == 1) reverse_ancova_means = c(rep(reverse_ancova_means, length(ancova_mean_exp)))
  if (length(reverse_ancova_means) != length(ancova_mean_exp)) stop("The length of the 'reverse_ancova_means' argument is incorrectly specified.")

  sd_within <- ancova_mean_sd_pooled / sqrt(1 - cov_outcome_r^2)

  d <- (ancova_mean_exp - ancova_mean_nexp) / sd_within

  es <- .es_from_d(
    d = d, cov_outcome_r = cov_outcome_r,
    adjusted = TRUE,
    n_cov_ancova = n_cov_ancova, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_ancova_means
  )

  es$info_used <- "ancova_means_sd_pooled_adj"

  es$md <- ifelse(reverse_ancova_means,
    ancova_mean_nexp - ancova_mean_exp,
    ancova_mean_exp - ancova_mean_nexp
  )
  es$md_se <- sqrt((n_exp + n_nexp) / (n_exp * n_nexp) * (1 - cov_outcome_r^2) * sd_within^2)
  es$md_ci_lo <- es$md - qt(.975, n_exp + n_nexp - 2 - n_cov_ancova) * es$md_se
  es$md_ci_up <- es$md + qt(.975, n_exp + n_nexp - 2 - n_cov_ancova) * es$md_se

  return(es)
}

#' Convert adjusted means obtained from an ANCOVA model and crude pooled standard deviation of two independent groups into several effect size measures
#'
#' @param ancova_mean_exp adjusted mean of participants in the experimental/exposed group.
#' @param ancova_mean_nexp adjusted mean of participants in the non-experimental/non-exposed group.
#' @param mean_sd_pooled crude pooled standard deviation.
#' @param cov_outcome_r correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).
#' @param n_cov_ancova number of covariates in the ANCOVA model.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_ancova_means a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function first computes an "adjusted" mean difference (MD) and Cohen's d (D)
#' from the adjusted means and crude pooled standard deviation of two independent groups.
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate the Cohen's d**:
#' \deqn{d =  \frac{ancova\_mean\_exp - ancova\_mean\_nexp\_adj}{mean\_sd\_pooled}}
#'
#' **To estimate the mean difference**:
#' \deqn{md =  ancova\_mean\_exp - ancova\_mean\_nexp\_adj}
#' \deqn{md\_se =  \sqrt{\frac{n\_exp + n\_nexp}{n\_exp * n\_nexp} * (1 - cov\_outcome\_r^2) * mean\_sd\_pooled^2}}
#'
#' Then, calculations of the \code{\link{es_from_ancova_means_sd}()} and \code{\link{es_from_cohen_d_adj}()} are applied.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @export es_from_ancova_means_sd_pooled_crude
#'
#' @md
#'
#' @examples
#' es_from_ancova_means_sd_pooled_crude(
#'   ancova_mean_exp = 29, ancova_mean_nexp = 34,
#'   mean_sd_pooled = 7, cov_outcome_r = 0.2,
#'   n_cov_ancova = 3, n_exp = 20, n_nexp = 20
#' )
es_from_ancova_means_sd_pooled_crude <- function(ancova_mean_exp, ancova_mean_nexp,
                                                 mean_sd_pooled, cov_outcome_r, n_cov_ancova,
                                                 n_exp, n_nexp, smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_ancova_means) {
  if (missing(reverse_ancova_means)) reverse_ancova_means <- rep(FALSE, length(ancova_mean_exp))
  reverse_ancova_means[is.na(reverse_ancova_means)] <- FALSE
  if (length(reverse_ancova_means) == 1) reverse_ancova_means = c(rep(reverse_ancova_means, length(ancova_mean_exp)))
  if (length(reverse_ancova_means) != length(ancova_mean_exp)) stop("The length of the 'reverse_ancova_means' argument is incorrectly specified.")

  d <- (ancova_mean_exp - ancova_mean_nexp) / mean_sd_pooled

  es <- .es_from_d(
    d = d, cov_outcome_r = cov_outcome_r,
    adjusted = TRUE,
    n_cov_ancova = n_cov_ancova, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_ancova_means
  )

  es$info_used <- "ancova_means_sd_pooled"

  es$md <- ifelse(reverse_ancova_means,
    ancova_mean_nexp - ancova_mean_exp,
    ancova_mean_exp - ancova_mean_nexp
  )

  es$md_se <- sqrt((n_exp + n_nexp) / (n_exp * n_nexp) * (1 - cov_outcome_r^2) * mean_sd_pooled^2)
  es$md_ci_lo <- es$md - qt(.975, n_exp + n_nexp - 2 - n_cov_ancova) * es$md_se
  es$md_ci_up <- es$md + qt(.975, n_exp + n_nexp - 2 - n_cov_ancova) * es$md_se

  return(es)
}

.quick_val_ci <- function(value, ci_lo, ci_up) {
  # First check if any inputs are length 0
  if (length(value) == 0 || length(ci_lo) == 0 || length(ci_up) == 0) {
    return(FALSE)
  }

  lengths <- c(length(value), length(ci_lo), length(ci_up))
  if (length(unique(lengths)) > 1) {
    return(FALSE)
  }

  # Proceed with the original logic
  mapply(function(v, l, u) {
    if (is.na(v) || is.na(l) || is.na(u)) return(FALSE)
    return((v - l) > 0 && (u - v) > 0)
  }, value, ci_lo, ci_up)
}

.extract_info <- function(x) {
  row <- which((!is.na(x$d) & !is.na(x$d_se) & .quick_val_ci(x$d, x$d_ci_lo, x$d_ci_up)) |
    (!is.na(x$g) & !is.na(x$g_se) & .quick_val_ci(x$g, x$g_ci_lo, x$g_ci_up)) |
    (!is.na(x$md) & !is.na(x$md_se) & .quick_val_ci(x$md, x$md_ci_lo, x$md_ci_up)) |
    (!is.na(x$r) & !is.na(x$r_se) & .quick_val_ci(x$r, x$r_ci_lo, x$r_ci_up)) |
    (!is.na(x$z) & !is.na(x$z_se) & .quick_val_ci(x$z, x$z_ci_lo, x$z_ci_up)) |
    (!is.na(x$logor) & !is.na(x$logor_se) & .quick_val_ci(x$logor, x$logor_ci_lo, x$logor_ci_up)) |
    (!is.na(x$logrr) & !is.na(x$logrr_se) & .quick_val_ci(x$logrr, x$logrr_ci_lo, x$logrr_ci_up)) |
    (!is.na(x$logirr) & !is.na(x$logirr_se) & .quick_val_ci(x$logirr, x$logirr_ci_lo, x$logirr_ci_up)) |
    (!is.na(x$logcvr) & !is.na(x$logcvr_se) & .quick_val_ci(x$logcvr, x$logcvr_ci_lo, x$logcvr_ci_up)) |
    (!is.na(x$logvr) & !is.na(x$logvr_se) & .quick_val_ci(x$logvr, x$logvr_ci_lo, x$logvr_ci_up)) |
    (!is.na(x$rp) & !is.na(x$rp_se) & .quick_val_ci(x$rp, x$rp_ci_lo, x$rp_ci_up)) |
    (!is.na(x$zp) & !is.na(x$zp_se) & .quick_val_ci(x$zp, x$zp_ci_lo, x$zp_ci_up)) |
    ("nnt_se" %in% colnames(x) & !is.na(x$nnt) & !is.na(x$nnt_se)) |
    ("rd_se" %in% colnames(x) & !is.na(x$rd) & !is.na(x$rd_se)) |
    ("prop_se" %in% colnames(x) & !is.na(x$prop) & !is.na(x$prop_se)) |
    ("alpha_se" %in% colnames(x) & !is.na(x$alpha) & !is.na(x$alpha_se)) |
    ("icc_se" %in% colnames(x) & !is.na(x$icc) & !is.na(x$icc_se)))
  info <- rep(NA, nrow(x))
  if (length(row) > 0) {
    info[row] <- x$info_used[row]
  }
  return(info)
}

.add_columns <- function(x, y) {
  cols <- y[which(!y %in% colnames(x))]

  if (length(cols) > 0) {
    x[, cols] <- NA
  }
  return(x)
}

.extract_es <- function(x, measure, exp) {
  if (measure == "d") {
    if ("d" %in% colnames(x)) {
      x$value <- x$d
      x$se_value <- x$d_se
      x$value_ci_lo <- x$d_ci_lo
      x$value_ci_up <- x$d_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "g") {
    if ("g" %in% colnames(x)) {
      x$value <- x$g
      x$se_value <- x$g_se
      x$value_ci_lo <- x$g_ci_lo
      x$value_ci_up <- x$g_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "z") {
    if ("z" %in% colnames(x)) {
      x$value <- x$z
      x$se_value <- x$z_se
      x$value_ci_lo <- x$z_ci_lo
      x$value_ci_up <- x$z_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "r") {
    if ("r" %in% colnames(x)) {
      x$value <- x$r
      x$se_value <- x$r_se
      x$value_ci_lo <- x$r_ci_lo
      x$value_ci_up <- x$r_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "logor") {
    if (exp) {
      if ("logor" %in% colnames(x)) {
        x$value <- exp(as.numeric(as.character(x$logor)))
        x$se_value <- x$logor_se
        x$value_ci_lo <- exp(as.numeric(as.character(x$logor_ci_lo)))
        x$value_ci_up <- exp(as.numeric(as.character(x$logor_ci_up)))
      } else {
        x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
      }
    } else {
      if ("logor" %in% colnames(x)) {
        x$value <- x$logor
        x$se_value <- x$logor_se
        x$value_ci_lo <- x$logor_ci_lo
        x$value_ci_up <- x$logor_ci_up
      } else {
        x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
      }
    }
  } else if (measure == "logrr") {
    if (exp) {
      if ("logrr" %in% colnames(x)) {
        x$value <- exp(as.numeric(as.character(x$logrr)))
        x$se_value <- x$logrr_se
        x$value_ci_lo <- exp(as.numeric(as.character(x$logrr_ci_lo)))
        x$value_ci_up <- exp(as.numeric(as.character(x$logrr_ci_up)))
      } else {
        x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
      }
    } else {
      if ("logrr" %in% colnames(x)) {
        x$value <- x$logrr
        x$se_value <- x$logrr_se
        x$value_ci_lo <- x$logrr_ci_lo
        x$value_ci_up <- x$logrr_ci_up
      } else {
        x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
      }
    }
  } else if (measure == "logirr") {
    if (exp) {
      if ("logirr" %in% colnames(x)) {
        x$value <- exp(as.numeric(as.character(x$logirr)))
        x$se_value <- x$logirr_se
        x$value_ci_lo <- exp(as.numeric(as.character(x$logirr_ci_lo)))
        x$value_ci_up <- exp(as.numeric(as.character(x$logirr_ci_up)))
      } else {
        x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
      }
    } else {
      if ("logirr" %in% colnames(x)) {
        x$value <- x$logirr
        x$se_value <- x$logirr_se
        x$value_ci_lo <- x$logirr_ci_lo
        x$value_ci_up <- x$logirr_ci_up
      } else {
        x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
      }
    }
  } else if (measure == "loghr") {
    if (exp) {
      if ("loghr" %in% colnames(x)) {
        x$value <- exp(as.numeric(as.character(x$loghr)))
        x$se_value <- x$loghr_se
        x$value_ci_lo <- exp(as.numeric(as.character(x$loghr_ci_lo)))
        x$value_ci_up <- exp(as.numeric(as.character(x$loghr_ci_up)))
      } else {
        x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
      }
    } else {
      if ("loghr" %in% colnames(x)) {
        x$value <- x$loghr
        x$se_value <- x$loghr_se
        x$value_ci_lo <- x$loghr_ci_lo
        x$value_ci_up <- x$loghr_ci_up
      } else {
        x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
      }
    }
  } else if (measure == "logcvr") {
    if ("logcvr" %in% colnames(x)) {
      x$value <- x$logcvr
      x$se_value <- x$logcvr_se
      x$value_ci_lo <- x$logcvr_ci_lo
      x$value_ci_up <- x$logcvr_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "logvr") {
    if ("logvr" %in% colnames(x)) {
      x$value <- x$logvr
      x$se_value <- x$logvr_se
      x$value_ci_lo <- x$logvr_ci_lo
      x$value_ci_up <- x$logvr_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "md") {
    if ("md" %in% colnames(x)) {
      x$value <- x$md
      x$se_value <- x$md_se
      x$value_ci_lo <- x$md_ci_lo
      x$value_ci_up <- x$md_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "nnt") {
    if ("nnt" %in% colnames(x)) {
      x$value <- x$nnt
      x$se_value <- if ("nnt_se" %in% colnames(x)) x$nnt_se else rep(NA, nrow(x))
      x$value_ci_lo <- if ("nnt_ci_lo" %in% colnames(x)) x$nnt_ci_lo else rep(NA, nrow(x))
      x$value_ci_up <- if ("nnt_ci_up" %in% colnames(x)) x$nnt_ci_up else rep(NA, nrow(x))
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "rd") {
    if ("rd" %in% colnames(x)) {
      x$value <- x$rd
      x$se_value <- if ("rd_se" %in% colnames(x)) x$rd_se else rep(NA, nrow(x))
      x$value_ci_lo <- if ("rd_ci_lo" %in% colnames(x)) x$rd_ci_lo else rep(NA, nrow(x))
      x$value_ci_up <- if ("rd_ci_up" %in% colnames(x)) x$rd_ci_up else rep(NA, nrow(x))
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "dw") {
    if ("d" %in% colnames(x)) {
      x$value <- x$d
      x$se_value <- x$d_se
      x$value_ci_lo <- x$d_ci_lo
      x$value_ci_up <- x$d_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "gw") {
    if ("g" %in% colnames(x)) {
      x$value <- x$g
      x$se_value <- x$g_se
      x$value_ci_lo <- x$g_ci_lo
      x$value_ci_up <- x$g_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "mdw") {
    if ("mdw" %in% colnames(x)) {
      x$value <- x$mdw
      x$se_value <- x$mdw_se
      x$value_ci_lo <- x$mdw_ci_lo
      x$value_ci_up <- x$mdw_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "rp") {
    if ("rp" %in% colnames(x)) {
      x$value <- x$rp
      x$se_value <- x$rp_se
      x$value_ci_lo <- x$rp_ci_lo
      x$value_ci_up <- x$rp_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "zp") {
    if ("zp" %in% colnames(x)) {
      x$value <- x$zp
      x$se_value <- x$zp_se
      x$value_ci_lo <- x$zp_ci_lo
      x$value_ci_up <- x$zp_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "prop") {
    if ("prop" %in% colnames(x)) {
      x$value <- x$prop
      x$se_value <- x$prop_se
      x$value_ci_lo <- x$prop_ci_lo
      x$value_ci_up <- x$prop_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "alpha") {
    if ("alpha" %in% colnames(x)) {
      x$value <- x$alpha
      x$se_value <- x$alpha_se
      x$value_ci_lo <- x$alpha_ci_lo
      x$value_ci_up <- x$alpha_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  } else if (measure == "icc") {
    if ("icc" %in% colnames(x)) {
      x$value <- x$icc
      x$se_value <- x$icc_se
      x$value_ci_lo <- x$icc_ci_lo
      x$value_ci_up <- x$icc_ci_up
    } else {
      x$value <- x$se_value <- x$value_ci_lo <- x$value_ci_up <- rep(NA_real_, nrow(x))
    }
  }
  return(x)
}


##########################################################################################
##########################################################################################
##########################################################################################
##########################################################################################
.generate_df <- function(x, main_es = TRUE, list_df_es_enh, list_df, ordering,
                         measure, suffix = "",
                         es_selected,
                         exp, digits) {
  # Early return if ordering is empty (e.g., no adjusted entries for single-group measures)
  if (length(ordering) == 0) return(x)

  list_keep <- do.call(cbind, lapply(list_df, function(x) x$info_used))
  cols_keep <- which(as.character(list_keep[1, ]) %in% ordering)
  list_df_restrict <- list_df[cols_keep]

  # build datasets with all ES / SE / information ----------------------------
  df_value_min_max <- do.call(cbind, lapply(list_df_restrict, function(x) x$value))
  df_se_min_max <- do.call(cbind, lapply(list_df_restrict, function(x) x$se_value))
  df_ci_lo_min_max <- do.call(cbind, lapply(list_df_restrict, function(x) x$value_ci_lo))
  df_ci_up_min_max <- do.call(cbind, lapply(list_df_restrict, function(x) x$value_ci_up))
  df_info_min_max <- do.call(cbind, lapply(list_df_restrict, function(x) x$info_used))


  # select the requested ES to highlight BASED ON HIERARCHY -------------------------------------------------
  # we start by the first ES requested, and if NA, try the second one, etc...
  for (i in 1:length(ordering)) {
    # i = 1
    # print(i);
    # print(ordering[i]);
    df_temp <- list_df_restrict[[which(df_info_min_max[1, ] == ordering[i])[1]]]
    row <- which(
      # info non-missing in the selected results
      !is.na(df_temp$value) & (!is.na(df_temp$se_value) | measure == "nnt") &
      # info missing in user datasets
        ((((is.na(x[, paste0("es", suffix)]) | is.na(x[, paste0("se", suffix)]))) & measure != "nnt") |
        (is.na(x[, paste0("es", suffix)]) & measure == "nnt"))
      )

    # print(length(row))
    if (length(row) >= 1) {
      x[row, paste0("es", suffix)] <- df_temp$value[row]
      x[row, paste0("se", suffix)] <- df_temp$se_value[row]
      x[row, paste0("es_ci_lo", suffix)] <- df_temp$value_ci_lo[row]
      x[row, paste0("es_ci_up", suffix)] <- df_temp$value_ci_up[row]
      x[row, paste0("info_used", suffix)] <- df_temp$info_used[row]
      x[row, paste0("measure", suffix)] <- measure
      if (grepl("user_input", unique(df_temp$info_used), fixed = TRUE)) {
        x[row, paste0("measure", suffix)] <- attr(df_temp, "measure")[row]
      }
    }
  }

  # rename the measure in case of exp selection
  if (exp & measure %in% c("logrr", "logor", "logirr")) {
    x[, paste0("measure", suffix)][x[, paste0("measure", suffix)] == "logrr" & !is.na(x[, paste0("measure", suffix)])] <- "rr"
    x[, paste0("measure", suffix)][x[, paste0("measure", suffix)] == "logor" & !is.na(x[, paste0("measure", suffix)])] <- "or"
    x[, paste0("measure", suffix)][x[, paste0("measure", suffix)] == "logirr" & !is.na(x[, paste0("measure", suffix)])] <- "irr"
  }

  # From there, the info_used+ES+SE+CIs have been inserted
  # this is for info_measure
  # message("OK")
  for (col in 1:length(ordering)) {
    # for (col in 1:38) {
    for (row in 1:nrow(df_value_min_max)) {
      # print(col)
      # for a given information, if a calculation (ES or SE) failed, we delete the others (CI are necessarily NA)----------------------------
      if ((!is.na(df_value_min_max[row, col]) & is.na(df_se_min_max[row, col]) & measure != "nnt") |
        (is.na(df_value_min_max[row, col]) & !is.na(df_se_min_max[row, col]))) {
        df_value_min_max[row, col] <- df_se_min_max[row, col] <- NA
      }
      # we identify all information from which we can derive an effect size ----------------------------
      if (!is.na(df_value_min_max[row, col]) & (!is.na(df_se_min_max[row, col]) | measure == "nnt")) {
        x[row, paste0("info_measure", suffix)] <- paste(x[row, paste0("info_measure", suffix)],
                                                        df_info_min_max[row, col], sep = " + ")
      }
    }
  }

  x[, paste0("info_measure", suffix)] <- sub("NA", "", x[, paste0("info_measure", suffix)], fixed = TRUE)
  x[, paste0("info_measure", suffix)] <- ifelse(grepl(" + ", substring(x[, paste0("info_measure", suffix)], 1, 3), fixed = TRUE),
    sub(" \\+ ", "", x[, paste0("info_measure", suffix)]),
    x[, paste0("info_measure", suffix)]
  )

  # This is for "all_info"
  info_list_all <- do.call(cbind, lapply(list_df_es_enh[cols_keep], .extract_info))
  all_info <- apply(info_list_all, 1, function(x) paste(na.omit(x), collapse = " + "))
  all_info[all_info == ""] <- "-- None available --"
  no_avlble <- c("", "-- None available --", NA)

  x[, paste0("all_info", suffix)] <- ifelse(
    x[, paste0("all_info", suffix)] %in% no_avlble & all_info %in% no_avlble,
   "-- None available --",
   ifelse(x[, paste0("all_info", suffix)] %in% no_avlble & !all_info %in% no_avlble,
          all_info,
          ifelse(!x[, paste0("all_info", suffix)] %in% no_avlble & all_info %in% no_avlble,
                 x[, paste0("all_info", suffix)],
                 ifelse(!x[, paste0("all_info", suffix)] %in% no_avlble & !all_info %in% no_avlble,
                        paste0(x[, paste0("all_info", suffix)], " + ", all_info),
                        "Error - contact corentin.gosling@parisnanterre.fr for more information"
                 )
          )
     )
    )

  # This is for "n_estimation"
  x[, paste0("n_estimations", suffix)] <- ncol(df_value_min_max) - rowSums(is.na(df_value_min_max))

  # This is for min_** and max_***
  col_min <- apply(df_value_min_max, 1, function(x) which.min(x)[1])
  col_max <- apply(df_value_min_max, 1, function(x) which.max(x)[1])

  cols_equal = which(col_min == col_max)
  if (length(cols_equal) > 0 && ncol(df_ci_lo_min_max) > 1) {
    # drop = FALSE keeps the matrix/data.frame shape when df_ci_lo_min_max
    # has 1 column (e.g. measure = "hr" with only user_input_crude),
    # which otherwise collapses to a vector and breaks apply().
    col_min[cols_equal] <- apply(data.frame(df_ci_lo_min_max)[cols_equal, , drop = FALSE], 1, function(x) which.min(x)[1])
    col_max[cols_equal] <- apply(data.frame(df_ci_lo_min_max)[cols_equal, , drop = FALSE], 1, function(x) which.max(x)[1])
  }

  for (i in 1:length(col_min)) {
    x[i, paste0("min_info", suffix)] <- df_info_min_max[i, col_min[i]]
    x[i, paste0("min_es_value", suffix)] <- as.numeric(as.character(df_value_min_max[i, col_min[i]]))
    x[i, paste0("min_es_se", suffix)] <- as.numeric(as.character(df_se_min_max[i, col_min[i]]))
    x[i, paste0("min_es_ci_lo", suffix)] <- as.numeric(as.character(df_ci_lo_min_max[i, col_min[i]]))
    x[i, paste0("min_es_ci_up", suffix)] <- as.numeric(as.character(df_ci_up_min_max[i, col_min[i]]))

    x[i, paste0("max_info", suffix)] <- df_info_min_max[i, col_max[i]]
    x[i, paste0("max_es_value", suffix)] <- as.numeric(as.character(df_value_min_max[i, col_max[i]]))
    x[i, paste0("max_es_se", suffix)] <- as.numeric(as.character(df_se_min_max[i, col_max[i]]))
    x[i, paste0("max_es_ci_lo", suffix)] <- as.numeric(as.character(df_ci_lo_min_max[i, col_max[i]]))
    x[i, paste0("max_es_ci_up", suffix)] <- as.numeric(as.character(df_ci_up_min_max[i, col_max[i]]))
  }

  # This is for "diff_min_max"
  x[, paste0("diff_min_max", suffix)] <- ifelse(x[, paste0("n_estimations", suffix)] > 1,
                                                x[, paste0("max_es_value", suffix)] - x[, paste0("min_es_value", suffix)],
                                                "< 2 types of input data available"
  )

  # This is for "overlap_min_max" -- intersection / union ratio of the two CIs.
  # Always in [0, 1]: correctly handles nested CIs (one CI contained in the
  # other) which the prior |max_lo - min_up| / |max_up - min_lo| formula
  # mis-handled (could return values > 1 or hide nesting at 100%).
  min_lo <- as.numeric(x[, paste0("min_es_ci_lo", suffix)])
  min_up <- as.numeric(x[, paste0("min_es_ci_up", suffix)])
  max_lo <- as.numeric(x[, paste0("max_es_ci_lo", suffix)])
  max_up <- as.numeric(x[, paste0("max_es_ci_up", suffix)])
  inter_w <- pmax(0, pmin(min_up, max_up) - pmax(min_lo, max_lo))
  union_w <- pmax(min_up, max_up) - pmin(min_lo, max_lo)
  overlap_val <- ifelse(is.finite(union_w) & union_w > 0, inter_w / union_w, 0)
  x[, paste0("overlap_min_max", suffix)] <- ifelse(
    x[, paste0("n_estimations", suffix)] > 1,
    overlap_val,
    "< 2 types of input data available"
  )
  # This is for dispersion_es
  dat_long = rep(NA, nrow(df_value_min_max))
  i = 0
  for (dat in c("df_value_min_max", "df_se_min_max",
                "df_ci_lo_min_max", "df_ci_up_min_max",
                "df_info_min_max")) {
    # print(dat)
    i = i+1
    df_tot = data.frame(mget(dat))
    df_tot$row_id = x$row_id
    dat_long_transit = reshape(
      df_tot, direction = "long",
      varying = names(df_tot)[which(names(df_tot) != "row_id")],
      times = names(df_tot)[which(names(df_tot) != "row_id")],
      v.names = "es",
      timevar = "row_id")
    colnames(dat_long_transit) <- c(c("info_measure_es_used", "info_measure_se",
                                      "info_measure_ci_lo", "info_measure_ci_up", "info_measure_used")[i],
                                    c("es", "se", "es_ci_lo", "es_ci_up", "info_used")[i],
                                    c("row_id", "row_id2", "row_id3", "row_id4", "row_id5")[i])
    dat_long = cbind(dat_long, dat_long_transit)
  }
  dat_long = dat_long[(!is.na(dat_long$es) & !is.na(dat_long$se) &
                         !is.na(dat_long$es_ci_lo) & !is.na(dat_long$es_ci_up) &
                         measure != "nnt") |
                        (!is.na(dat_long$es) & measure == "nnt"), ]
  dat_long = dat_long[order(dat_long$row_id),]

  # Cross-method dispersion (E1 signal). Uses the k-robust max-absolute-deviation
  # -from-median (.dispersion_stat) rather than a raw sample SD: sd() shrinks
  # ~1/sqrt(k) as agreeing methods are added, so a lone discordant estimate
  # becomes progressively harder to flag in data-rich rows, and for k = 2 it
  # collapses to |a - b|/sqrt(2), duplicating E3 with a mismatched threshold. The
  # MaxAD is k-invariant and, because diff_max = 2*dispersion_max for every
  # measure, coincides exactly with E3 at k = 2 while complementing it for k > 2.
  res_dispersion = data.frame(tapply(dat_long$es, dat_long$row_id, .dispersion_stat))
  dispersion = data.frame(dispersion_es = res_dispersion[,1],
                          row_id = rownames(res_dispersion))

  dat_long = merge(x = dat_long,
                   y = dispersion)

  if (main_es == TRUE & nrow(dat_long) != 0) {
    dispersion$blank = NA
    dispersion$row_id = as.numeric(as.character(dispersion$row_id))
    x_transit = merge(x = dispersion[,c("row_id", "blank")],
                      y = x)
    if (all(dispersion$row_id == x_transit$row_id)) {
      x_transit[, paste0("dispersion_es", suffix)] <- dispersion$dispersion_es
    } else {
      stop("an error occured when estimating the 'dispersion_es' variable")
    }

    x_transit <- x_transit[, -which(names(x_transit) == "blank")]

    x_empty = x[!x$row_id %in% x_transit$row_id, ]

    x = rbind(x_transit, x_empty)

    x = x[order(x$row_id),]

  } else if (nrow(dat_long) != 0) {
    x_transit = merge(x = dat_long[, c("dat_long", "row_id")], y = x)

    x_transit[, paste0("info_used", suffix)] <- dat_long$info_used
    x_transit[, paste0("es", suffix)] <- dat_long$es
    x_transit[, paste0("se", suffix)] <- dat_long$se
    x_transit[, paste0("es_ci_up", suffix)] <- dat_long$es_ci_up
    x_transit[, paste0("es_ci_lo", suffix)] <- dat_long$es_ci_lo
    x_transit[, paste0("dispersion_es", suffix)] <- dat_long$dispersion_es

    x_transit <- x_transit[, -which(names(x_transit) == "dat_long")]

    x_empty = x[!x$row_id %in% x_transit$row_id, ]

    x = rbind(x_transit, x_empty)

    x = x[order(x$row_id),]

  }

  # INSERT FINAL ES if users prefer using minimum/maximum rather than hierarchy
  if (es_selected == "minimum") {
    x[, paste0("es_selected", suffix)] <- "minimum"
    x[, paste0("es", suffix)] <- x[, paste0("min_es_value", suffix)]
    x[, paste0("se", suffix)] <- x[, paste0("min_es_se", suffix)]
    x[, paste0("es_ci_lo", suffix)] <- x[, paste0("min_es_ci_lo", suffix)]
    x[, paste0("es_ci_up", suffix)] <- x[, paste0("min_es_ci_up", suffix)]
    x[, paste0("info_used", suffix)] <- x[, paste0("min_info", suffix)]
    x[, paste0("measure", suffix)] <- x[, paste0("measure", suffix)]
  } else if (es_selected == "maximum") {
    x[, paste0("es_selected", suffix)] <- "maximum"
    x[, paste0("es", suffix)] <- x[, paste0("max_es_value", suffix)]
    x[, paste0("se", suffix)] <- x[, paste0("max_es_se", suffix)]
    x[, paste0("es_ci_lo", suffix)] <- x[, paste0("max_es_ci_lo", suffix)]
    x[, paste0("es_ci_up", suffix)] <- x[, paste0("max_es_ci_up", suffix)]
    x[, paste0("info_used", suffix)] <- x[, paste0("max_info", suffix)]
    x[, paste0("measure", suffix)] <- x[, paste0("measure", suffix)]
  } else {
    x[, paste0("es_selected", suffix)] <- "hierarchy"
  }

  # this is for securizing certain columns.
  # set NA when n_estimations = 0
  row_miss <- which(x[, paste0("n_estimations", suffix)] == 0)
  x[row_miss, paste0(
    c(
      "overlap_min_max", "diff_min_max",
      "min_info", "min_es_value", "min_es_se", "min_es_ci_lo", "min_es_ci_up",
      "max_info", "max_es_value", "max_es_se", "max_es_ci_lo", "max_es_ci_up"
    ),
    suffix
  )] <- NA
  # set "< 2 types of input data available" when n_estimations = 1
  row_1 <- which(x[, paste0("n_estimations", suffix)] == 1)
  x[row_1, paste0(
    c("dispersion_es",
      "min_info", "min_es_value", "min_es_se", "min_es_ci_lo", "min_es_ci_up",
      "max_info", "max_es_value", "max_es_se", "max_es_ci_lo", "max_es_ci_up"
    ),
    suffix
  )] <- "< 2 types of input data available"

  # Note: rounding was moved to summary() so that quality flags operate on
  # full-precision values. See .round_numeric_cols() called after flag checks.

  return(x)
}

#' Convert a Cohen's d value to several effect size measures
#'
#' @param cohen_d Cohen's d (i.e., standardized mean difference) value.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_d a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function estimates the standard error of a Cohen's d value and computes a Hedges' g (G).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate the standard error of Cohen's d**, the following formula is used (formula 12.13 in Cooper):
#' \deqn{cohen\_d\_se =  \sqrt{\frac{n\_exp+n\_nexp}{n\_exp*n\_nexp} + \frac{cohen\_d^2}{2*(n\_exp+n\_nexp)}}}
#' \deqn{cohen\_d\_ci\_lo = cohen\_d - cohen\_d\_se * qt(.975, df = n\_exp+n\_nexp-2)}
#' \deqn{cohen\_d\_ci\_up = cohen\_d + cohen\_d\_se * qt(.975, df = n\_exp+n\_nexp-2)}
#'
#' **To estimate the Hedges' g and its standard error**, the following formulas are used (Hedges, 1981):
#' \deqn{df = n\_exp + n\_nexp - 2}
#' \deqn{J = exp(\log_{gamma}(\frac{df}{2}) - 0.5 * \log(\frac{df}{2}) - \log_{gamma}(\frac{df - 1}{2}))}
#' \deqn{hedges\_g = cohen\_d * J}
#' \deqn{hedges\_g\_se = \sqrt{cohen\_d\_se^2 * J^2}}
#' \deqn{hedges\_g\_ci\_lo = hedges\_g - hedges\_g\_se * qt(.975, df = n\_exp+n\_nexp-2)}
#' \deqn{hedges\_g\_ci\_up = hedges\_g + hedges\_g\_se * qt(.975, df = n\_exp+n\_nexp-2)}
#'
#' **To estimate the log odds ratio and its standard error**, the following formulas are used (formulas 12.34-12.35 in Cooper):
#' \deqn{logor = \frac{cohen\_d * \pi}{\sqrt{3}}}
#' \deqn{logor\_se = \sqrt{\frac{cohen\_d\_se^2 * \pi^2}{3}}}
#' \deqn{logor\_lo = logor - logor\_se * qnorm(.975)}
#' \deqn{logor\_up = logor + logor\_se * qnorm(.975)}
#' Note that this conversion assumes that responses within the two groups follow logistic distributions.
#'
#' **To estimate the correlation coefficient and its standard error**, various formulas can be used.
#'
#' **A.** To estimate the 'biserial' correlation (\code{smd_to_cor="viechtbauer"}), the following formulas are used (formulas 5, 8, 13, 17, 18, 19 in Viechtbauer):
#' \deqn{df = n\_exp + n\_nexp - 2 - n\_cov\_ancova}
#' \deqn{h = \frac{df}{n\_exp} + \frac{df}{n\_nexp}}
#' \deqn{r.pb = \frac{cohen\_d}{\sqrt{cohen\_d^2 + h}}}
#' \deqn{p = \frac{n\_exp}{n\_exp + n\_nexp}}
#' \deqn{q = 1 - p}
#' \deqn{f = dnorm(qnorm(1-p))}
#' \deqn{R = \frac{\sqrt{p*q}}{f} * r.pb}
#' \deqn{R\_var = \frac{1}{n\_exp + n\_nexp - 1} * (\frac{p*q}{f^2} - (\frac{3}{2} + (1 - \frac{p*qnorm(1-p)}{f})(1 + \frac{q*qnorm(1-p)}{f})) R^2 + R^4)}
#' \deqn{R\_se = \sqrt{R\_var}}
#' \deqn{a = \frac{\sqrt{f}}{(p*q)^\frac{1}{4}}}
#' \deqn{Z = \frac{a}{2} * \log(\frac{1+a*R}{1-a*R})}
#' \deqn{Z\_var = \frac{1}{n - 1}}
#' \deqn{Z\_se = \sqrt{Z\_var}}
#' \deqn{Z\_ci\_lo = Z - qnorm(.975) * Z\_se}
#' \deqn{Z\_ci\_up = Z + qnorm(.975) * Z\_se}
#' \deqn{R\_ci\_lo = \frac{1}{a} * tanh(\frac{Z\_lo}{a})}
#' \deqn{R\_ci\_up = \frac{1}{a} * tanh(\frac{Z\_up}{a})}
#'
#' Note: this \eqn{Z} is Jacobs & Viechtbauer's (2017) variance-stabilizing transform of the
#' biserial correlation (with variance \eqn{1/(n-1)}), which is analogous to -- but NOT the
#' same as -- Fisher's \eqn{atanh(R)} z-transform for a Pearson correlation (the two coincide
#' only as the effect approaches 0). It is intended for constructing the confidence interval
#' of a single coefficient. For a meta-analysis that mixes standardized-mean-difference and
#' genuine-correlation studies, pool on the correlation (\eqn{R}) scale, where biserial and
#' product-moment correlations are directly comparable (Jacobs & Viechtbauer, 2017), rather
#' than on this \eqn{Z} scale (which summary() labels "Fisher's z" for all correlation inputs).
#'
#' **B.** To estimate the correlation coefficient according to Cooper et al. (2019) (formulas 12.40-42)
#' and Borenstein et al. (2009) (formulas 54-56),
#' the following formulas are used (\code{smd_to_cor="lipsey_cooper"}):
#' \deqn{p = \frac{n\_exp}{n\_exp + n\_nexp}}
#' \deqn{R = \frac{cohen\_d}{\sqrt{cohen\_d^2 + 1 / (p * (1 - p))}}}
#' \deqn{a = \frac{(n\_exp + n\_nexp)^2}{(n\_exp*n\_nexp)}}
#' \deqn{var\_R = \frac{a^2 * cohen\_d\_se^2}{(cohen\_d^2 + a)^3}}
#' \deqn{R\_se = \sqrt{R\_var}}
#' \deqn{R\_ci\_lo = R - qt(.975, n\_exp+n\_nexp- 2) * R\_se}
#' \deqn{R\_ci\_up = R + qt(.975, n\_exp+n\_nexp- 2) * R\_se}
#' \deqn{Z = atanh(R)}
#' \deqn{Z\_var = \frac{cohen\_d\_se^2}{cohen\_d^2 + (1 / p*(1-p))}}
#' \deqn{Z\_se = \sqrt{Z\_var}}
#' \deqn{Z\_ci\_lo = Z - qnorm(.975) * Z\_se}
#' \deqn{Z\_ci\_up = Z + qnorm(.975) * Z\_se}
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Borenstein, M., Hedges, L. V., Higgins, J. P., & Rothstein, H. R. (2021). Introduction to meta-analysis. John Wiley & Sons.
#'
#' Hedges LV (1981): Distribution theory for Glass's estimator of effect size and related estimators. Journal of Educational and Behavioral Statistics, 6, 107-28
#'
#' Jacobs, P., & Viechtbauer, W. (2017). Estimation of the biserial correlation and its sampling variance for use in meta-analysis. Research synthesis methods, 8(2), 161-180.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 1. Cohen's d or Hedges' g'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_cohen_d
#'
#' @md
#'
#' @examples
#' es_from_cohen_d(cohen_d = 1, n_exp = 20, n_nexp = 20)
es_from_cohen_d <- function(cohen_d, n_exp, n_nexp, smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_d) {
  if (missing(reverse_d)) reverse_d <- rep(FALSE, length(n_exp))
  reverse_d[is.na(reverse_d)] <- FALSE


  es <- .es_from_d(
    d = cohen_d, n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_d
  )

  es$info_used <- "cohen_d"
  return(es)
}

#' Convert an adjusted Cohen's d value to several effect size measures
#'
#' @param cohen_d_adj Adjusted Cohen's d (i.e., standardized mean difference) value.
#' @param n_cov_ancova number of covariates
#' @param cov_outcome_r covariate-outcome correlation (in case of multiple covariates, the multiple correlation)
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_d a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function estimates the standard error of an adjusted Cohen's d value and Hedges' g (G), and
#' converts an odds ratio (OR) and correlation coefficients (R/Z).
#'
#' **To estimate the standard error of Cohen's d**, the following formula is used (table 12.3 in Cooper):
#' \deqn{d\_se = \sqrt{\frac{n\_exp+n\_nexp}{n\_exp*n\_nexp} * (1 - cov\_outcome\_r^2) + \frac{cohen\_d\_adj^2}{2*(n\_exp+n\_nexp)}}}
#'
#' **To estimate other effect size measures**, calculations of the
#'  \code{\link{es_from_cohen_d}()} function are used (with the exception of the degree of freedom
#'  that is estimated as \code{df = n_exp + n_nexp - 2 - n_cov_ancova}).
#'
#' @references
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 1. Cohen's d or Hedges' g'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_cohen_d_adj
#'
#' @md
#'
#' @examples
#' es_from_cohen_d_adj(cohen_d_adj = 1, n_cov_ancova = 4, cov_outcome_r = .30, n_exp = 20, n_nexp = 20)
es_from_cohen_d_adj <- function(cohen_d_adj, n_cov_ancova, cov_outcome_r, n_exp, n_nexp,
                                smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_d) {
  if (missing(reverse_d)) reverse_d <- rep(FALSE, length(n_exp))
  reverse_d[is.na(reverse_d)] <- FALSE

  es <- .es_from_d(
    d = cohen_d_adj, adjusted = TRUE, n_cov_ancova = n_cov_ancova,
    cov_outcome_r = cov_outcome_r,
    n_exp = n_exp, n_nexp = n_nexp,
    smd_to_cor = smd_to_cor, smd_var = smd_var, reverse = reverse_d
  )

  es$info_used <- "cohen_d_adj"
  return(es)
}

#' Convert a Hedges' g value to other effect size measures (G, OR, COR)
#'
#' @param hedges_g Hedges' g value
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param smd_to_cor formula used to convert the \code{hedges_g} value into a coefficient correlation (see details).
#' @param smd_var name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.
#' @param reverse_g a logical value indicating whether the direction of the \code{hedges_g} value should be flipped.
#'
#' @details
#' This function estimates the standard error of the Hedges' g and the Cohen's d (D).
#' Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.
#'
#' **To estimate standard error of Hedges'g**, the following formula is used (Hedges, 1981):
#' \deqn{df = n\_exp + n\_nexp - 2}
#' \deqn{hedges\_g\_se = \sqrt{cohen\_d\_se^2 * J^2}}
#' \deqn{hedges\_g\_ci\_lo = hedges\_g - hedges\_g\_se * qt(.975, df = n\_exp+n\_nexp-2)}
#' \deqn{hedges\_g\_ci\_up = hedges\_g + hedges\_g\_se * qt(.975, df = n\_exp+n\_nexp-2)}
#'
#' **To estimate the Cohen's d value**, the following formula is used (Hedges, 1981):
#' \deqn{J = exp(\log_{gamma}(\frac{df}{2}) - 0.5 * \log(\frac{df}{2}) - \log_{gamma}(\frac{df - 1}{2}))}
#' \deqn{cohen\_d = \frac{hedges\_g}{J}}
#' \deqn{cohen\_d\_se = \sqrt{(\frac{n\_exp+n\_nexp}{n\_exp*n\_nexp} + \frac{cohen\_d^2}{2*(n\_exp+n\_nexp)})}}
#'
#' **To estimate other effect size measures**,
#' calculations of the \code{\link{es_from_cohen_d}()} are applied.
#'
#' @references
#' Hedges LV (1981): Distribution theory for Glass's estimator of effect size and related estimators. Journal of Educational and Behavioral Statistics, 6, 107-28
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 1. Cohen's d or Hedges' g'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_hedges_g
#'
#' @md
#'
#' @examples
#' es_from_hedges_g(hedges_g = 0.243, n_exp = 20, n_nexp = 20)
es_from_hedges_g <- function(hedges_g, n_exp, n_nexp, smd_to_cor = "viechtbauer", smd_var = "borenstein", reverse_g) {
  if (missing(reverse_g)) reverse_g <- rep(FALSE, length(n_exp))
  reverse_g[is.na(reverse_g)] <- FALSE
  if (length(reverse_g) == 1) reverse_g = c(rep(reverse_g, length(hedges_g)))
  if (length(reverse_g) != length(hedges_g)) stop("The length of the 'reverse_g' argument is incorrectly specified.")

  df <- n_exp + n_nexp - 2

  hedges_g <- ifelse(reverse_g, -hedges_g, hedges_g)

  J <- .d_j(df)
  d <- hedges_g / J

  es <- .es_from_d(d = d, n_exp = n_exp, n_nexp = n_nexp, smd_to_cor = smd_to_cor, smd_var = smd_var)

  es$info_used <- "hedges_g"
  return(es)
}

#' Flag the differences between two dataframes.
#'
#' @param df_extractor_1 a first dataset. Differences with the second dataset will be flagged in green.
#' @param df_extractor_2 a second dataset. Differences with the first dataset will be flagged in red.
#' @param tolerance the cut-off value used to flag differences between two numeric values
#' @param tolerance_type must be either 'difference' or 'ratio'
#' @param output type of object returned by the function (see 'Value' section). Must be either 'wide', 'long', 'html', 'html2' or 'xlsx'.
#' @param file_name the name of the generated file (only used when \code{output="xlsx"})
#' @param ordering_columns column names that should be used to re-order the two datasets before running the comparisons
#'
#' @details
#' This function aims to facilitate the comparison of two datasets created by blind data extractors during a systematic review.
#' It is a wrapper of several functions from the 'compareDF' package.
#'
#' @return
#' This function returns a dataframe composed of the rows that include a
#' difference (all identical rows are removed).
#' Several outputs can be requested :
#' 1. setting \code{output="xlsx"} returns an excel file. A message indicates the location of the generated file on your computer.
#' 1. setting \code{output="html"} returns an html file
#' 1. setting \code{output="html2"} returns an html file (only useful when the "html" command did not make the html pane appear in R studio).
#' 1. setting \code{output="wide"} a wide dataframe
#' 1. setting \code{output="long"} a long dataframe
#'
#' @md
#'
#' @export compare_df
#'
#' @references
#' Alex Joseph (2022). compareDF: Do a Git Style Diff of the Rows Between Two Dataframes with Similar Structure. R package version 2.3.3. https://CRAN.R-project.org/package=compareDF
#'
#' @examples
#' df.compare1 = df.compare1[order(df.compare1$author), ]
#' df.compare2 = df.compare2[order(df.compare2$year), ]
#'
#' compare_df(
#'   df_extractor_1 = df.compare1,
#'   df_extractor_2 = df.compare2,
#'   ordering_columns = c("author", "year")
#' )

compare_df <- function(df_extractor_1, df_extractor_2,
                       ordering_columns = NULL,
                       tolerance = 0,
                       tolerance_type = "ratio",
                       output = "html",
                       file_name = "comparison.xlsx") {

  df_extractor_1 = data.frame(df_extractor_1)
  df_extractor_2 = data.frame(df_extractor_2)


  cols_available = unique(append(names(df_extractor_1), names(df_extractor_2)))
  cols_retained = intersect(names(df_extractor_1), names(df_extractor_2))
  cols_unique = cols_available[!cols_available %in% cols_retained]
  if (length(cols_unique) > 0) {
    warning("\nThe columns ", paste(cols_unique, collapse = " / "),
            " were removed from the comparison since they were included only in one of the two datasets.\n")
  }

  if (!is.null(ordering_columns)) {
    if (!all(ordering_columns %in% cols_retained)) {
      stop("The ordering columns should be present in the two datasets")
    } else if (length(ordering_columns) > 1) {

      create_unique_id <- function(df, columns) {
        do.call(paste, c(df[, columns, drop = FALSE], sep = "_"))
      }

      df_extractor_1$ID_metaConvert <- create_unique_id(df_extractor_1, ordering_columns)
      df_extractor_2$ID_metaConvert <- create_unique_id(df_extractor_2, ordering_columns)

      ordering_columns = "ID_metaConvert"
      cols_retained = append(cols_retained, ordering_columns)

    }
  }



  if (!is.null(ordering_columns)) {
    df_extractor_1 = suppressWarnings(df_extractor_1[order(df_extractor_1[, ordering_columns]), cols_retained])
    df_extractor_2 = suppressWarnings(df_extractor_2[order(df_extractor_2[, ordering_columns]), cols_retained])

    combined_values <- unique(c(df_extractor_1[, ordering_columns], df_extractor_2[, ordering_columns]))
    combined_values <- combined_values[order(combined_values)]

    insert_blank_rows <- function(df, combined_values, ordering_columns) {
      new_df <- data.frame(matrix(ncol = ncol(df), nrow = length(combined_values)))
      colnames(new_df) <- colnames(df)

      for (i in seq_along(combined_values)) {
        match_row <- which(df[, ordering_columns] == combined_values[i])
        if (length(match_row) > 0) {
          new_df[i, ] <- df[match_row, ]
        }
      }

      return(new_df)
    }

    # Apply the function to both dataframes
    df_extractor_1 <- insert_blank_rows(df_extractor_1, combined_values, ordering_columns)
    df_extractor_2 <- insert_blank_rows(df_extractor_2, combined_values, ordering_columns)
    df_extractor_1$rowname <- 1:nrow(df_extractor_1)
    df_extractor_2$rowname <- 1:nrow(df_extractor_2)

    } else {
    df_extractor_1 = df_extractor_1[, cols_retained]
    df_extractor_2 = df_extractor_2[, cols_retained]
    df_extractor_1$rowname <- 1:nrow(df_extractor_1)
    df_extractor_2$rowname <- 1:nrow(df_extractor_2)
    }


  comp <- suppressMessages(compareDF::compare_df(
    df_new = df_extractor_1,
    df_old = df_extractor_2,
    group_col = "rowname",
    exclude = NULL,
    tolerance = tolerance,
    tolerance_type = tolerance_type,
    stop_on_error = TRUE,
    change_markers = c("df_extractor_1", "df_extractor_2", "="),
    round_output_to = 3
  ))

  if (output == "long") {
    res <- comp$comparison_df[order(as.numeric(as.character(comp$comparison_df$rowname))), ]
  } else if (output == "wide") {
    res <- compareDF::create_wide_output(comp)
    res <- res[order(as.numeric(as.character(res$rowname))), ]
  } else if (output == "html") {
    res <- suppressMessages(compareDF::create_output_table(comp, output_type = "html", limit = 5000))
  } else if (output == "html2") {
    res <- suppressMessages(compareDF::view_html(comp))
  } else if (output == "xlsx") {
    if (!grepl(".xlsx", file_name)) {
      file_name <- paste0(file_name, ".xlsx")
    }
    message("File generated at '", paste0(getwd(), "/", file_name, "'"))
    res <- compareDF::create_output_table(comp,
      output_type = "xlsx",
      file_name = file_name, limit = 5000
    )
  }
  return(res)
}

#' Convert a phi value to several effect size measures
#'
#' @param phi phi value
#' @param n_sample total number of participants in the sample
#' @param n_cases total number of cases/events
#' @param n_exp total number of participants in the exposed group
#' @param reverse_phi a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' The functions computes an odds ratio (OR), risk ratio (RR), and number needed to treat (NNT)
#' from the the phi coefficient, the total number of participants,
#' the total number of cases and the total number of people exposed.
#' Cohen's d (D) and Hedges' g (G) are tried to be obtained from the OR, or are converted
#' using the approach by Lipsey et al. (2001).
#' The correlation coefficients (R/Z) are converted by assuming that the phi coefficient
#' is equal to a R, and the variances of R and Z are obtained using the approach proposed
#' by Lipsey et al. (2001) as well as by our own calculations.
#'
#' **To estimate the OR, RR, NNT,**,
#' this function reconstructs a 2x2 table (using the approach proposed by Viechtbauer, 2023).
#'
#' Then, the calculations of the \code{\link{es_from_2x2}()} function are applied.
#'
#' **To estimate D, G and the correlation coefficients (R/Z) when the 2x2 table cannot be
#' reconstructed** (e.g., the number of cases or exposed participants is missing), the phi
#' coefficient -- which is the Pearson correlation of the two binary variables -- is
#' treated directly as a correlation coefficient and passed to
#' \code{\link{es_from_pearson_r}()}. This yields R and Z with their standard large-sample
#' sampling variances,
#' \deqn{r = phi, \quad r\_se = \frac{1 - r^2}{\sqrt{n\_sample - 1}}}
#' \deqn{z = atanh(r), \quad z\_se = \frac{1}{\sqrt{n\_sample - 3}}}
#' and D and G are then obtained from R exactly as in \code{\link{es_from_pearson_r}()}
#' (the R to D step assumes balanced groups). In this situation the OR, RR and NNT require
#' the 2x2 margins and are returned as NA. When the 2x2 table can be reconstructed, D, G,
#' R and Z are instead derived from it (see \code{\link{es_from_2x2}()}).
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR + RR + NNT\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 8. Phi or chi-square'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Viechtbauer (2023). Accessed at https://wviechtb.github.io/metafor/reference/conv.2x2.html.
#' Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
#'
#' @export es_from_phi
#'
#' @md
#'
#' @examples
#' es_from_phi(phi = 0.3, n_sample = 120, n_cases = 20, n_exp = 40)
es_from_phi <- function(phi, n_cases, n_exp,
                        n_sample,
                        reverse_phi) {
  if (missing(reverse_phi)) reverse_phi <- rep(FALSE, length(phi))
  reverse_phi[is.na(reverse_phi)] <- FALSE

  if (missing(n_sample)) n_sample <- rep(NA, length(phi))
  if (missing(n_exp)) n_exp <- rep(NA, length(phi))
  if (missing(n_cases)) n_cases <- rep(NA, length(phi))

  if (length(reverse_phi) == 1) reverse_phi = c(rep(reverse_phi, length(phi)))
  if (length(reverse_phi) != length(phi)) stop("The length of the 'reverse_phi' argument is incorrectly specified.")

  # A phi coefficient is a correlation and must lie in [-1, 1]; metafor::conv.2x2()
  # raises a HARD ERROR (not a warning) on |phi| > 1, which would abort an entire
  # convert_df() run over the other (valid) rows. Guard here so a single out-of-range
  # value degrades to an all-NA row (the package's one-bad-cell-cannot-abort contract).
  # convert_df()'s Tier-1 validation already NA's such values under correct_inputs =
  # TRUE; this is the backstop for direct es_from_phi() calls.
  phi_valid <- phi
  phi_valid[!is.na(phi_valid) & abs(phi_valid) > 1] <- NA_real_

  cont_table <- suppressWarnings(
    metafor::conv.2x2(
      ri = phi_valid, ni = n_sample,
      n1i = n_exp, n2i = n_cases
    )
  )
  es <- es_from_2x2(
    n_cases_exp = cont_table$ai,
    n_controls_exp = cont_table$bi,
    n_cases_nexp = cont_table$ci,
    n_controls_nexp = cont_table$di,
    reverse_2x2 = reverse_phi
  )

  # Fallback for rows where the 2x2 table could NOT be reconstructed (n_cases / n_exp
  # missing, so conv.2x2 returns NA cells and OR/RR/NNT are NA): a phi coefficient IS the
  # Pearson correlation of two binary variables, so deliver R/Z/D/G by treating phi as a
  # Pearson r and delegating to the vetted es_from_pearson_r() engine. This gives the
  # STANDARD sampling variances (z_se = 1/sqrt(n-3), r_se = (1-r^2)/sqrt(n-1), matching
  # metafor) rather than the earlier ad hoc formula that inflated the SE by up to ~22% at
  # large phi. A phi-only row therefore yields the SAME R/Z/D/G as an equivalent
  # pearson_r row (internal consistency). NB: the R -> D step assumes balanced groups;
  # OR/RR/NNT require the 2x2 margins and remain NA when they are missing.
  miss <- which(is.na(es$r) & !is.na(phi) & !is.na(n_sample) & abs(phi) < 1)
  if (length(miss) > 0) {
    pr <- es_from_pearson_r(
      pearson_r = phi[miss], n_sample = n_sample[miss],
      reverse_pearson_r = reverse_phi[miss]
    )
    fb_cols <- intersect(
      c("d", "d_se", "d_ci_lo", "d_ci_up", "g", "g_se", "g_ci_lo", "g_ci_up",
        "r", "r_se", "r_ci_lo", "r_ci_up", "z", "z_se", "z_ci_lo", "z_ci_up"),
      intersect(colnames(es), colnames(pr))
    )
    es[miss, fb_cols] <- pr[, fb_cols]
  }

  es$info_used <- "phi"

  return(es)
}

#' Convert a chi-square value to several effect size measures
#'
#' @param chisq value of the chi-squared
#' @param n_sample total number of participants in the sample
#' @param n_cases total number of cases/events
#' @param n_exp total number of participants in the exposed group
#' @param yates_chisq logical value (or vector of length \code{length(chisq)})
#'   indicating whether the chi-square has been performed using Yates' correction
#'   for continuity. When a vector is supplied, each row is back-transformed using
#'   its own setting. Missing values are treated as \code{FALSE}.
#' @param reverse_chisq a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts a chi-square value (with one degree of freedom)
#' into a phi coefficient (Lipsey et al. 2001):
#' \deqn{phi = \sqrt{\frac{chisq}{n\_sample}}}
#' and then converts it to other effect size measures exactly as in
#' \code{\link{es_from_phi}()} (including the correlation-based R/Z/D/G fallback with
#' standard sampling variances when the 2x2 table cannot be reconstructed).
#'
#' Note that if \code{yates_chisq = TRUE}, the chi-square value is interpreted
#' as Yates-corrected when back-transforming to a 2x2 contingency table; this is
#' propagated row by row when a vector is supplied.
#'
#' Then, the phi coefficient is converted to other effect size measures (see \code{\link{es_from_phi}}).
#'
#' @references
#' Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR + RR + NNT\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 8. Phi or chi-square'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_chisq
#'
#' @md
#'
#' @examples
#' es_from_chisq(chisq = 4.21, n_sample = 78, n_cases = 51, n_exp = 50)
es_from_chisq <- function(chisq, n_sample, n_cases, n_exp,
                          yates_chisq = FALSE,
                          reverse_chisq) {
  if (missing(reverse_chisq)) reverse_chisq <- rep(FALSE, length(chisq))
  if (missing(n_sample)) n_sample <- rep(NA, length(chisq))
  if (missing(n_exp)) n_exp <- rep(NA, length(chisq))
  if (missing(n_cases)) n_cases <- rep(NA, length(chisq))
  reverse_chisq[is.na(reverse_chisq)] <- FALSE

  if (length(reverse_chisq) == 1) reverse_chisq = c(rep(reverse_chisq, length(chisq)))
  if (length(reverse_chisq) != length(chisq)) stop("The length of the 'reverse_chisq' argument is incorrectly specified.")

  if (length(yates_chisq) == 1) yates_chisq <- rep(yates_chisq, length(chisq))
  if (length(yates_chisq) != length(chisq)) stop("The length of the 'yates_chisq' argument is incorrectly specified.")
  yates_chisq[is.na(yates_chisq)] <- FALSE

  cont_table <- suppressWarnings(
    metafor::conv.2x2(
      x2i = chisq, ni = n_sample,
      n1i = n_exp, n2i = n_cases,
      correct = yates_chisq
    )
  )

  es <- es_from_2x2(
    n_cases_exp = cont_table$ai,
    n_controls_exp = cont_table$bi,
    n_cases_nexp = cont_table$ci,
    n_controls_nexp = cont_table$di,
    reverse_2x2 = reverse_chisq
  )

  # Fallback for rows where the 2x2 table could NOT be reconstructed (n_cases / n_exp
  # missing). For a 1-df chi-square, phi = sqrt(chisq / n) is the Pearson correlation of
  # the two binary variables (its sign is not identified by chisq alone, so it is taken
  # positive and flipped by reverse_chisq). Deliver R/Z/D/G by treating phi as a Pearson
  # r via the vetted es_from_pearson_r() engine, giving the STANDARD SEs (matching
  # metafor) rather than the earlier ad hoc, SE-inflating formula. OR/RR/NNT require the
  # 2x2 margins and remain NA; the R -> D step assumes balanced groups.
  r_chi <- suppressWarnings(sqrt(chisq / n_sample))
  miss <- which(is.na(es$r) & !is.na(r_chi) & is.finite(r_chi) & r_chi < 1 & !is.na(n_sample))
  if (length(miss) > 0) {
    pr <- es_from_pearson_r(
      pearson_r = r_chi[miss], n_sample = n_sample[miss],
      reverse_pearson_r = reverse_chisq[miss]
    )
    fb_cols <- intersect(
      c("d", "d_se", "d_ci_lo", "d_ci_up", "g", "g_se", "g_ci_lo", "g_ci_up",
        "r", "r_se", "r_ci_lo", "r_ci_up", "z", "z_se", "z_ci_lo", "z_ci_up"),
      intersect(colnames(es), colnames(pr))
    )
    es[miss, fb_cols] <- pr[, fb_cols]
  }

  es$info_used <- "chisq"

  return(es)
}

#' Convert a p-value of a chi-square to several effect size measures
#'
#' @param chisq_pval p-value of a chi-square coefficient
#' @param n_sample total number of participants in the sample
#' @param n_cases total number of cases/events
#' @param n_exp total number of participants in the exposed group
#' @param yates_chisq logical value (or vector of length \code{length(chisq_pval)})
#'   indicating whether the chi-square has been performed using Yates' correction
#'   for continuity. When a vector is supplied, each row is back-transformed using
#'   its own setting. Missing values are treated as \code{FALSE}.
#' @param reverse_chisq_pval a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts a chi-square value (with one degree of freedom)
#' into a chi-square coefficient (Section 3.12 in Lipsey et al., 2001):
#' \deqn{chisq = qchisq(chisq\_pval, df = 1, lower.tail = FALSE)}
#'
#' Note that if \code{yates_chisq = TRUE}, the chi-square value is interpreted
#' as Yates-corrected when back-transforming to a 2x2 contingency table; this is
#' propagated row by row when a vector is supplied.
#'
#' Then, the chisq coefficient is converted to other effect size measures (see \code{\link{es_from_chisq}}).
#'
#' @references
#' Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab OR + RR + NNT\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab D + G + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 8. Phi or chi-square'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_chisq_pval
#'
#' @md
#'
#' @examples
#' es_from_chisq_pval(chisq_pval = 0.2, n_sample = 42, n_exp = 25, n_cases = 13)
es_from_chisq_pval <- function(chisq_pval, n_sample, n_cases, n_exp,
                               yates_chisq = FALSE,
                               reverse_chisq_pval) {
  if (missing(reverse_chisq_pval)) reverse_chisq_pval <- rep(FALSE, length(chisq_pval))
  reverse_chisq_pval[is.na(reverse_chisq_pval)] <- FALSE

  chisq <- stats::qchisq(p = chisq_pval, df = 1, lower.tail = FALSE)

  es <- es_from_chisq(
    chisq = chisq, n_sample = n_sample,
    yates_chisq = yates_chisq,
    n_cases = n_cases,
    n_exp = n_exp,
    reverse_chisq = reverse_chisq_pval
  )

  es$info_used <- "chisq_pval"
  return(es)
}

# es_d <- .es_from_d(d = d, d_se = d_se, n_sample = n_sample,
#                    reverse = reverse_chisq)
# OR
# px = n_exp/n_sample
# py = n_cases/n_sample
# n_cases_exp = suppressWarnings(
#   n_sample * px * py *
#     sqrt((chisq * px * py * (1-px) * (1-py))/n_sample))
#
# n_controls_exp = n_exp - n_cases_exp
# n_cases_nexp = n_cases - n_cases_exp
# n_controls_nexp = n_sample - (n_cases_exp + n_controls_exp + n_cases_nexp)
#
# es = es_from_2x2(n_cases_exp = n_cases_exp, n_controls_exp = n_controls_exp,
#                  n_cases_nexp = n_cases_nexp, n_controls_nexp = n_controls_nexp,
#                  reverse_2x2 = reverse_chisq)
#
# col.es = which(colnames(es) %in% c("d", "d_se", "d_ci_lo", "d_ci_up",
#                                    "g", "g_se", "g_ci_lo", "g_ci_up"))
# col.esd = which(colnames(es_d) %in% c("d", "d_se", "d_ci_lo", "d_ci_up",
#                                    "g", "g_se", "g_ci_lo", "g_ci_up"))
#
# es[, col.es] <- es_d[, col.esd]
#
# #COR
# r = r_se = r_ci_lo = r_ci_up =
#   z = z_se = z_ci_lo = z_ci_up = rep(NA, length(chisq))
#
# dat_cor = data.frame(n_cases_exp = n_cases_exp, n_controls_exp = n_controls_exp,
#                      n_cases_nexp = n_cases_nexp, n_controls_nexp = n_controls_nexp,
#                      n_sample = n_sample, reverse_chisq = reverse_chisq,
#                      chisq = chisq, chisq_to_cor = chisq_to_cor)
#
# nn_miss = which(
#   (!is.na(n_cases_exp) & !is.na(n_controls_exp) & !is.na(n_cases_nexp) & !is.na(n_controls_nexp)) |
#     (!is.na(chisq) & !is.na(n_sample))
# )
#
# if (length(nn_miss) != 0) {
#   cor = t(mapply(.chi_to_cor,
#                  chisq = dat_cor$chisq[nn_miss],
#                  n_sample = dat_cor$n_sample[nn_miss],
#                  n_cases_exp = dat_cor$n_cases_exp[nn_miss],
#                  n_controls_exp = dat_cor$n_controls_exp[nn_miss],
#                  n_cases_nexp = dat_cor$n_cases_nexp[nn_miss],
#                  n_controls_nexp = dat_cor$n_controls_nexp[nn_miss],
#                  reverse_chisq = dat_cor$reverse_chisq[nn_miss],
#                  chisq_to_cor = dat_cor$chisq_to_cor[nn_miss]))
#
#   r[nn_miss] = cor[, 1]
#   r_se[nn_miss] = sqrt(cor[, 2])
#   r_ci_lo[nn_miss] = cor[, 3]
#   r_ci_up[nn_miss] = cor[, 4]
#   z[nn_miss] = cor[, 5]
#   z_se[nn_miss] = sqrt(cor[, 6])
#   z_ci_lo[nn_miss] = cor[, 7]
#   z_ci_up[nn_miss] = cor[, 8]
# }
#
# col = which(colnames(es) %in% c("r", "r_se", "r_ci_lo", "r_ci_up",
#                                 "z", "z_se", "z_ci_lo", "z_ci_up"))
# es[, col] <- cbind(r, r_se, r_ci_lo, r_ci_up,
#                   z, z_se, z_ci_lo, z_ci_up)







# es_d <- .es_from_d(d = d, d_se = d_se, n_sample = n_sample, reverse = reverse_phi)

# px = n_exp/n_sample
# py = n_cases/n_sample
# n_cases_exp = n_sample * suppressWarnings(px*py + phi*sqrt(px*py*(1-px)*(1-py)))
# n_controls_exp = n_exp - n_cases_exp
# n_cases_nexp = n_cases - n_cases_exp
# n_controls_nexp = n_sample - (n_cases_exp + n_controls_exp + n_cases_nexp)
#
# es = es_from_2x2(n_cases_exp = n_cases_exp, n_controls_exp = n_controls_exp,
#                  n_cases_nexp = n_cases_nexp, n_controls_nexp = n_controls_nexp,
#                  reverse_2x2 = reverse_phi)
# col.es = which(colnames(es) %in% c("d", "d_se", "d_ci_lo", "d_ci_up",
#                                    "g", "g_se", "g_ci_lo", "g_ci_up"))
# col.esd = which(colnames(es_d) %in% c("d", "d_se", "d_ci_lo", "d_ci_up",
#                                       "g", "g_se", "g_ci_lo", "g_ci_up"))
#
# es[, col.es] <- es_d[, col.esd]
# #COR
# r = r_se = r_ci_lo = r_ci_up =
# z = z_se = z_ci_lo = z_ci_up = rep(NA, length(phi))
#
# dat_cor = data.frame(n_cases_exp = n_cases_exp, n_controls_exp = n_controls_exp,
#                      n_cases_nexp = n_cases_nexp, n_controls_nexp = n_controls_nexp,
#                      n_sample = n_sample, reverse_phi = reverse_phi,
#                      phi = phi, phi_to_cor = phi_to_cor)
#
# nn_miss = which(
#   (!is.na(n_cases_exp) & !is.na(n_controls_exp) & !is.na(n_cases_nexp) & !is.na(n_controls_nexp)) |
#     (!is.na(phi) & !is.na(n_sample))
# )
#
# if (length(nn_miss) != 0) {
#   cor = t(mapply(.phi_to_cor,
#                  phi = dat_cor$phi[nn_miss],
#                  n_sample = dat_cor$n_sample[nn_miss],
#                  n_cases_exp = dat_cor$n_cases_exp[nn_miss],
#                  n_controls_exp = dat_cor$n_controls_exp[nn_miss],
#                  n_cases_nexp = dat_cor$n_cases_nexp[nn_miss],
#                  n_controls_nexp = dat_cor$n_controls_nexp[nn_miss],
#                  reverse_phi = dat_cor$reverse_phi[nn_miss],
#                  phi_to_cor = dat_cor$phi_to_cor[nn_miss]))
#
#   r[nn_miss] = cor[, 1]
#   r_se[nn_miss] = sqrt(cor[, 2])
#   r_ci_lo[nn_miss] = cor[, 3]
#   r_ci_up[nn_miss] = cor[, 4]
#   z[nn_miss] = cor[, 5]
#   z_se[nn_miss] = sqrt(cor[, 6])
#   z_ci_lo[nn_miss] = cor[, 7]
#   z_ci_up[nn_miss] = cor[, 8]
# }
#
# col = which(colnames(es) %in% c("r", "r_se", "r_ci_lo", "r_ci_up",
#                                 "z", "z_se", "z_ci_lo", "z_ci_up"))
# es[, col] <- cbind(r, r_se, r_ci_lo, r_ci_up,
#                    z, z_se, z_ci_lo, z_ci_up)

#' Convert mean changes and standard deviations of two independent groups into standard effect size measures
#'
#' @param mean_change_exp mean change of participants in the experimental/exposed group.
#' @param mean_change_sd_exp standard deviation of the mean change (i.e., SD of the difference scores)
#'   for participants in the experimental/exposed group.
#' @param mean_change_nexp mean change of participants in the non-experimental/non-exposed group.
#' @param mean_change_sd_nexp standard deviation of the mean change for participants in the
#'   non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the mean change into a SMD ("morris_drm" or "morris_dz", see details).
#' @param pool_sd a logical value indicating whether the standardizing SD should be pooled across the two
#'   groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
#'   equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
#'   which to prefer, so this is a deliberate choice and not a technical detail.
#'   \itemize{
#'     \item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
#'       within-group values are subtracted, their variances adding because the arms are independent. This
#'       is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
#'       SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
#'       broadly applicable of the two.
#'     \item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
#'       is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
#'       assumes the two arms' true standardizing SDs are equal.
#'   }
#' @param reverse_mean_change a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function computes a Cohen's d (D) and Hedges' g (G) from the mean change and standard deviation
#' of change scores of two independent groups. Odds ratio (OR) and correlation coefficients (R/Z) are
#' then converted from Cohen's d.
#'
#' Two formulas can be used to obtain the SMD (Morris & DeShon, 2002):
#' \deqn{d_{z} = \frac{mean\_change}{sd\_change}}
#' \deqn{d_{rm} = d_{z} \times \sqrt{2 \times (1 - r\_pre\_post)}}
#' The 'morris_drm' formula (default, alias 'cooper') requires the pre-post correlation while
#' 'morris_dz' does not. Note that d_rm and d_z are not expressed on the same scale and should not be
#' combined in a same meta-analysis (Morris & DeShon, 2002). Under 'morris_drm', the resulting effect
#' size directly depends on the r_pre_post value entered; if r is unknown, the 'morris_dz' formula
#' can be used instead. The 'bonett' and 'morris_dav' formulas require separate pre/post SDs and are
#' thus not available for mean change data.
#'
#' This function simply internally calls the \code{\link{es_from_means_sd_pre_post}} function but setting:
#' \deqn{mean\_pre\_exp = 0}
#' \deqn{mean\_pre\_sd\_exp = 0}
#' \deqn{mean\_exp = mean\_change\_exp}
#' \deqn{mean\_sd\_exp = mean\_change\_sd\_exp}
#' \deqn{mean\_pre\_nexp = 0}
#' \deqn{mean\_pre\_sd\_nexp = 0}
#' \deqn{mean\_nexp = mean\_change\_nexp}
#' \deqn{mean\_sd\_nexp = mean\_change\_sd\_nexp}
#'
#' To know more about the calculations, see \code{\link{es_from_means_sd_pre_post}} function.
#'
#' @references
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386. https://doi.org/10.1177/1094428106291059
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_mean_change_sd
#'
#' @md
#'
#' @examples
#' es_from_mean_change_sd(
#'   n_exp = 36, n_nexp = 35,
#'   mean_change_exp = 8.4, mean_change_sd_exp = 9.13,
#'   mean_change_nexp = 2.43, mean_change_sd_nexp = 6.61,
#'   r_pre_post_exp = 0.2, r_pre_post_nexp = 0.2
#' )
es_from_mean_change_sd <- function(mean_change_exp, mean_change_sd_exp,
                                   mean_change_nexp, mean_change_sd_nexp,
                                   r_pre_post_exp, r_pre_post_nexp,
                                   n_exp, n_nexp,
                                   smd_to_cor = "viechtbauer",
                                   pre_post_to_smd = "cooper",
                                   pool_sd = FALSE,
                                   reverse_mean_change) {
  if (missing(reverse_mean_change)) reverse_mean_change <- rep(FALSE, length(mean_change_exp))
  reverse_mean_change[is.na(reverse_mean_change)] <- FALSE
  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(mean_change_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_change_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "mean_change",
    func_name = "es_from_mean_change_sd"
  )

  es <- es_from_means_sd_pre_post(
    mean_pre_exp = rep(0, length(mean_change_exp)),
    mean_exp = mean_change_exp,
    mean_pre_sd_exp = rep(0, length(mean_change_exp)),
    mean_sd_exp = mean_change_sd_exp,
    mean_pre_nexp = rep(0, length(mean_change_exp)),
    mean_nexp = mean_change_nexp,
    mean_pre_sd_nexp = rep(0, length(mean_change_exp)),
    mean_sd_nexp = mean_change_sd_nexp,
    n_exp = n_exp,
    n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp,
    r_pre_post_nexp = r_pre_post_nexp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    pool_sd = pool_sd,
    reverse_means_pre_post = reverse_mean_change
  )

  es$info_used <- "mean_change_sd"

  return(es)
}
#' Convert mean changes and standard errors of two independent groups into standard effect size measures
#'
#' @param mean_change_exp mean change of participants in the experimental/exposed group.
#' @param mean_change_se_exp standard error of the mean change for participants in the experimental/exposed group.
#' @param mean_change_nexp mean change of participants in the non-experimental/non-exposed group.
#' @param mean_change_se_nexp standard error of the mean change for participants in the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pre_post_to_smd formula used to convert the mean change into a SMD ("morris_drm" or "morris_dz", see details).
#' @param pool_sd a logical value indicating whether the standardizing SD should be pooled across the two
#'   groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
#'   equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
#'   which to prefer, so this is a deliberate choice and not a technical detail.
#'   \itemize{
#'     \item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
#'       within-group values are subtracted, their variances adding because the arms are independent. This
#'       is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
#'       SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
#'       broadly applicable of the two.
#'     \item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
#'       is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
#'       assumes the two arms' true standardizing SDs are equal.
#'   }
#' @param reverse_mean_change a logical value indicating whether the direction of generated effect sizes should be flipped.
#'
#' @details
#' This function converts the mean change and standard errors of two independent groups into a Cohen's d.
#' The Cohen's d is then converted to other effect size measures.
#'
#' This function simply internally calls the \code{\link{es_from_means_se_pre_post}} function but setting:
#' \deqn{mean\_pre\_exp = 0}
#' \deqn{mean\_pre\_se\_exp = 0}
#' \deqn{mean\_exp = mean\_change\_exp}
#' \deqn{mean\_se\_exp = mean\_change\_se\_exp}
#' \deqn{mean\_pre\_nexp = 0}
#' \deqn{mean\_pre\_se\_nexp = 0}
#' \deqn{mean\_nexp = mean\_change\_nexp}
#' \deqn{mean\_se\_nexp = mean\_change\_se\_nexp}
#'
#' To know more about the calculations, see \code{\link{es_from_means_se_pre_post}} function.
#'
#' @references
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386. https://doi.org/10.1177/1094428106291059
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_from_mean_change_se
#'
#' @md
#'
#' @examples
#' es_from_mean_change_se(
#'   n_exp = 36, n_nexp = 35,
#'   mean_change_exp = 8.4, mean_change_se_exp = 9.13,
#'   mean_change_nexp = 2.43, mean_change_se_nexp = 6.61,
#'   r_pre_post_exp = 0.2, r_pre_post_nexp = 0.2
#' )
es_from_mean_change_se <- function(mean_change_exp, mean_change_se_exp,
                                   mean_change_nexp, mean_change_se_nexp,
                                   r_pre_post_exp, r_pre_post_nexp,
                                   n_exp, n_nexp,
                                   smd_to_cor = "viechtbauer",
                                   pre_post_to_smd = "cooper",
                                   pool_sd = FALSE,
                                   reverse_mean_change) {
  if (missing(reverse_mean_change)) reverse_mean_change <- rep(FALSE, length(mean_change_exp))
  reverse_mean_change[is.na(reverse_mean_change)] <- FALSE
  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(mean_change_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_change_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "mean_change",
    func_name = "es_from_mean_change_se"
  )

  es <- es_from_means_se_pre_post(
    mean_pre_exp = rep(0, length(mean_change_exp)),
    mean_exp = mean_change_exp,
    mean_pre_se_exp = rep(0, length(mean_change_exp)),
    mean_se_exp = mean_change_se_exp,
    mean_pre_nexp = rep(0, length(mean_change_exp)),
    mean_nexp = mean_change_nexp,
    mean_pre_se_nexp = rep(0, length(mean_change_exp)),
    mean_se_nexp = mean_change_se_nexp,
    n_exp = n_exp,
    n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp,
    r_pre_post_nexp = r_pre_post_nexp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    pool_sd = pool_sd,
    reverse_means_pre_post = reverse_mean_change
  )

  es$info_used <- "mean_change_se"

  return(es)
}
#' Convert mean changes and 95% CI of two independent groups into standard effect size measures
#'
#' @param mean_change_exp mean change of participants in the experimental/exposed group.
#' @param mean_change_ci_lo_exp lower bound of the 95% CI around the mean change of the experimental/exposed group.
#' @param mean_change_ci_up_exp upper bound of the 95% CI around the mean change of the experimental/exposed group.
#' @param mean_change_nexp mean change of participants in the non-experimental/non-exposed group.
#' @param mean_change_ci_lo_nexp lower bound of the 95% CI around the mean change of the non-experimental/non-exposed group.
#' @param mean_change_ci_up_nexp upper bound of the 95% CI around the mean change of the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param max_asymmetry A percentage indicating the tolerance before detecting asymmetry in the 95% CI bounds.
#' @param pool_sd a logical value indicating whether the standardizing SD should be pooled across the two
#'   groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
#'   equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
#'   which to prefer, so this is a deliberate choice and not a technical detail.
#'   \itemize{
#'     \item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
#'       within-group values are subtracted, their variances adding because the arms are independent. This
#'       is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
#'       SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
#'       broadly applicable of the two.
#'     \item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
#'       is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
#'       assumes the two arms' true standardizing SDs are equal.
#'   }
#' @param reverse_mean_change a logical value indicating whether the direction of generated effect sizes should be flipped.
#' @param pre_post_to_smd formula used to convert the mean change into a SMD ("morris_drm" or "morris_dz", see details).
#'
#' @details
#' This function converts the mean change and 95% CI of two independent groups into a Cohen's d.
#' The Cohen's d is then converted to other effect size measures.
#'
#' This function simply internally calls the \code{\link{es_from_means_ci_pre_post}} function but setting:
#' \deqn{mean\_pre\_exp = 0}
#' \deqn{mean\_pre\_ci\_lo\_exp = 0}
#' \deqn{mean\_pre\_ci\_up\_exp = 0}
#'
#' \deqn{mean\_exp = mean\_change\_exp}
#' \deqn{mean\_ci\_lo\_exp = mean\_change\_ci\_lo\_exp}
#' \deqn{mean\_ci\_up\_exp = mean\_change\_ci\_up\_exp}
#'
#' \deqn{mean\_pre\_nexp = 0}
#' \deqn{mean\_pre\_ci\_lo\_nexp = 0}
#' \deqn{mean\_pre\_ci\_up\_nexp = 0}
#'
#' \deqn{mean\_nexp = mean\_change\_nexp}
#' \deqn{mean\_ci\_lo\_nexp = mean\_change\_ci\_lo\_nexp}
#' \deqn{mean\_ci\_up\_nexp = mean\_change\_ci\_up\_nexp}
#'
#' To know more about the calculations, see \code{\link{es_from_means_sd_pre_post}} function.
#'
#' @references
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386. https://doi.org/10.1177/1094428106291059
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @export es_from_mean_change_ci
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_mean_change_ci(
#'   n_exp = 36, n_nexp = 35,
#'   mean_change_exp = 8.4,
#'   mean_change_ci_lo_exp = 6.4, mean_change_ci_up_exp = 10.4,
#'   mean_change_nexp = 2.43,
#'   mean_change_ci_lo_nexp = 1.43, mean_change_ci_up_nexp = 3.43,
#'   r_pre_post_exp = 0.2, r_pre_post_nexp = 0.2
#' )
es_from_mean_change_ci <- function(mean_change_exp,
                                   mean_change_ci_lo_exp, mean_change_ci_up_exp,
                                   mean_change_nexp,
                                   mean_change_ci_lo_nexp, mean_change_ci_up_nexp,
                                   r_pre_post_exp, r_pre_post_nexp,
                                   n_exp, n_nexp, max_asymmetry = 10,
                                   smd_to_cor = "viechtbauer", pre_post_to_smd = "cooper",
                                   pool_sd = FALSE, reverse_mean_change) {
  if (missing(reverse_mean_change)) reverse_mean_change <- rep(FALSE, length(mean_change_exp))
  reverse_mean_change[is.na(reverse_mean_change)] <- FALSE
  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(mean_change_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_change_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "mean_change",
    func_name = "es_from_mean_change_ci"
  )

  es <- es_from_means_ci_pre_post(
    mean_pre_exp = rep(0, length(mean_change_exp)),
    mean_pre_ci_lo_exp = rep(0, length(mean_change_exp)),
    mean_pre_ci_up_exp = rep(0, length(mean_change_exp)),

    mean_pre_nexp = rep(0, length(mean_change_exp)),
    mean_pre_ci_lo_nexp = rep(0, length(mean_change_exp)),
    mean_pre_ci_up_nexp = rep(0, length(mean_change_exp)),

    mean_exp = mean_change_exp, mean_ci_lo_exp = mean_change_ci_lo_exp, mean_ci_up_exp = mean_change_ci_up_exp,
    mean_nexp = mean_change_nexp, mean_ci_lo_nexp = mean_change_ci_lo_nexp, mean_ci_up_nexp = mean_change_ci_up_nexp,
    n_exp = n_exp,
    n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp,
    r_pre_post_nexp = r_pre_post_nexp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    pool_sd = pool_sd,
    max_asymmetry = max_asymmetry,
    reverse_means_pre_post = reverse_mean_change
  )

  es$info_used <- "mean_change_ci"

  return(es)
}


#' Convert mean changes and p-values of two independent groups into standard effect size measures
#'
#' @param mean_change_exp mean change of participants in the experimental/exposed group.
#' @param mean_change_pval_exp p-value of the mean change for participants in the experimental/exposed group.
#' @param mean_change_nexp mean change of participants in the non-experimental/non-exposed group.
#' @param mean_change_pval_nexp p-value of the mean change for participants in the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param r_pre_post_exp pre-post correlation in the experimental/exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).
#' @param r_pre_post_nexp pre-post correlation in the non-experimental/non-exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).
#' @param smd_to_cor formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).
#' @param pool_sd a logical value indicating whether the standardizing SD should be pooled across the two
#'   groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
#'   equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
#'   which to prefer, so this is a deliberate choice and not a technical detail.
#'   \itemize{
#'     \item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
#'       within-group values are subtracted, their variances adding because the arms are independent. This
#'       is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
#'       SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
#'       broadly applicable of the two.
#'     \item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
#'       is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
#'       assumes the two arms' true standardizing SDs are equal.
#'   }
#' @param reverse_mean_change a logical value indicating whether the direction of generated effect sizes should be flipped.
#' @param pre_post_to_smd formula used to convert the mean change into a SMD ("morris_drm" or "morris_dz", see details).
#'
#' @details
#' This function converts the mean change and associated p-values of two independent groups into a Cohen's d.
#' The Cohen's d is then converted to other effect size measures.
#'
#' To start, this function estimates the mean change standard errors from the p-values:
#' \deqn{t\_exp <- qt(p = mean\_change\_pval\_exp / 2, df = n\_exp - 1, lower.tail = FALSE)}
#' \deqn{t\_nexp <- qt(p = mean\_change\_pval\_nexp / 2, df = n\_nexp - 1, lower.tail = FALSE)}
#' \deqn{mean\_change\_se\_exp <- |\frac{mean\_change\_exp}{t\_exp}|}
#' \deqn{mean\_change\_se\_nexp <- |\frac{mean\_change\_nexp}{t\_nexp}|}
#'
#' Then, this function simply internally calls the \code{\link{es_from_means_se_pre_post}} function but setting:
#' \deqn{mean\_pre\_exp = 0}
#' \deqn{mean\_pre\_se\_exp = 0}
#' \deqn{mean\_exp = mean\_change\_exp}
#' \deqn{mean\_se\_exp = mean\_change\_se\_exp}
#' \deqn{mean\_pre\_nexp = 0}
#' \deqn{mean\_pre\_se\_nexp = 0}
#' \deqn{mean\_nexp = mean\_change\_nexp}
#' \deqn{mean\_se\_nexp = mean\_change\_se\_nexp}
#'
#' To know more about other calculations, see \code{\link{es_from_means_sd_pre_post}} function.
#'
#' @references
#' Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386. https://doi.org/10.1177/1094428106291059
#'
#' Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
#'
#' Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
#'
#' @export es_from_mean_change_pval
#'
#' @return
#' This function estimates and converts between several effect size measures.
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab MD + D + G\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab OR + R + Z \cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
#'  \tab https://metaconvert.org/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_from_mean_change_pval(
#'   n_exp = 36, n_nexp = 35,
#'   mean_change_exp = 8.4, mean_change_pval_exp = 0.13,
#'   mean_change_nexp = 2.43, mean_change_pval_nexp = 0.61,
#'   r_pre_post_exp = 0.8, r_pre_post_nexp = 0.8
#' )
es_from_mean_change_pval <- function(mean_change_exp, mean_change_pval_exp,
                                   mean_change_nexp, mean_change_pval_nexp,
                                   r_pre_post_exp, r_pre_post_nexp,
                                   n_exp, n_nexp,
                                   smd_to_cor = "viechtbauer", pre_post_to_smd = "cooper",
                                   pool_sd = FALSE, reverse_mean_change) {
  if (missing(reverse_mean_change)) reverse_mean_change <- rep(FALSE, length(mean_change_exp))
  reverse_mean_change[is.na(reverse_mean_change)] <- FALSE
  if (missing(r_pre_post_nexp)) r_pre_post_nexp <- rep(0.8, length(mean_change_exp))
  r_pre_post_nexp[is.na(r_pre_post_nexp)] <- 0.8
  if (missing(r_pre_post_exp)) r_pre_post_exp <- rep(0.8, length(mean_change_exp))
  r_pre_post_exp[is.na(r_pre_post_exp)] <- 0.8

  pre_post_to_smd <- .validate_pre_post_to_smd(
    pre_post_to_smd,
    allowed_methods = c("morris_drm", "morris_dz"),
    context = "mean_change",
    func_name = "es_from_mean_change_pval"
  )

  t_exp <- qt(p = mean_change_pval_exp / 2, df = n_exp - 1, lower.tail = FALSE)
  t_nexp <- qt(p = mean_change_pval_nexp / 2, df = n_nexp - 1, lower.tail = FALSE)

  mean_change_se_exp <- abs(mean_change_exp / t_exp)
  mean_change_se_nexp <- abs(mean_change_nexp / t_nexp)

  es <- es_from_means_se_pre_post(
    mean_pre_exp = rep(0, length(mean_change_exp)),
    mean_exp = mean_change_exp,
    mean_pre_se_exp = rep(0, length(mean_change_exp)),
    mean_se_exp = mean_change_se_exp,
    mean_pre_nexp = rep(0, length(mean_change_exp)),
    mean_nexp = mean_change_nexp,
    mean_pre_se_nexp = rep(0, length(mean_change_exp)),
    mean_se_nexp = mean_change_se_nexp,
    n_exp = n_exp,
    n_nexp = n_nexp,
    r_pre_post_exp = r_pre_post_exp,
    r_pre_post_nexp = r_pre_post_nexp,
    smd_to_cor = smd_to_cor,
    pre_post_to_smd = pre_post_to_smd,
    pool_sd = pool_sd,
    reverse_means_pre_post = reverse_mean_change
  )

  es$info_used <- "mean_change_pval"

  return(es)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----echo=FALSE, message=FALSE, warning=FALSE, results='hide'-----------------
library(metaConvert)
library(metafor)
library(DT)

## -----------------------------------------------------------------------------
data(df.psychom)

## ----eval=FALSE---------------------------------------------------------------
# str(df.psychom)

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(df.psychom, options = list(
    scrollX = TRUE,
    dom = c('t'),
    scrollY = "300px",
    pageLength = 50,
    ordering = FALSE,
    rownames = FALSE,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center',
                                     targets = "_all"))))

## ----eval=FALSE---------------------------------------------------------------
# # Extraction sheet for Cronbach's alpha
# data_extraction_sheet(measure = "alpha", extension = "data.frame")
# 
# # Extraction sheet for ICC
# data_extraction_sheet(measure = "icc", extension = "data.frame")
# 
# # Extraction sheet for correlations (Pearson or Spearman)
# data_extraction_sheet(measure = "r", extension = "data.frame")
# 
# # Extraction sheet for proportions
# data_extraction_sheet(measure = "prop", extension = "data.frame")

## -----------------------------------------------------------------------------
# Subset internal consistency studies
dat_alpha <- df.psychom[df.psychom$outcome == "internal_consistency", ]

# Compute effect sizes with Bonett transformation (default)
res_alpha <- convert_df(dat_alpha, measure = "alpha",
                        verbose = FALSE, split_adjusted = FALSE)
summary_alpha <- summary(res_alpha, flags = TRUE, guidance = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# summary_alpha[, c("author", "es", "se", "es_ci_lo", "es_ci_up", "info_used")]

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(summary_alpha[, c("author", "es", "se", "es_ci_lo", "es_ci_up", "info_used")],
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '130px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))

## ----fig.width=7, fig.height=4------------------------------------------------
# Random-effects meta-analysis on Bonett-transformed scale
ma_alpha <- metafor::rma(yi = es, sei = se, data = summary_alpha,
                         method = "REML")

# Forest plot
metafor::forest(ma_alpha, slab = paste0(summary_alpha$author, " (", dat_alpha$year, ")"),
                xlab = "Bonett-transformed alpha: ln(1 - alpha)",
                header = TRUE)

## -----------------------------------------------------------------------------
# Back-transform pooled estimate and CI
pooled_alpha    <- 1 - exp(ma_alpha$beta[1])
pooled_alpha_lo <- 1 - exp(ma_alpha$ci.ub)
pooled_alpha_up <- 1 - exp(ma_alpha$ci.lb)

cat(sprintf("Pooled alpha = %.3f [%.3f, %.3f]\n",
            pooled_alpha, pooled_alpha_lo, pooled_alpha_up))

## ----eval=FALSE---------------------------------------------------------------
# # Raw alpha (not recommended for meta-analysis)
# res_alpha_raw <- convert_df(dat_alpha, measure = "alpha", alpha_to_es = "raw",
#                             verbose = FALSE, split_adjusted = FALSE)
# summary(res_alpha_raw)

## -----------------------------------------------------------------------------
# Subset test-retest studies
dat_icc <- df.psychom[df.psychom$outcome == "test_retest", ]

# Compute effect sizes with Bonett transformation (default)
res_icc <- convert_df(dat_icc, measure = "icc",
                      verbose = FALSE, split_adjusted = FALSE)
summary_icc <- summary(res_icc, flags = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# summary_icc[, c("author", "es", "se", "es_ci_lo", "es_ci_up",
#                  "icc_type", "info_used")]

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(summary_icc[, c("author", "es", "se", "es_ci_lo", "es_ci_up",
                                "icc_type", "info_used")],
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '130px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))

## ----fig.width=7, fig.height=4------------------------------------------------
# Random-effects meta-analysis
ma_icc <- metafor::rma(yi = es, sei = se, data = summary_icc,
                       method = "REML")

# Forest plot
metafor::forest(ma_icc, slab = paste0(summary_icc$author, " (", dat_icc$year, ")"),
                xlab = "Bonett-transformed ICC: ln(1 - ICC)",
                header = TRUE)

## -----------------------------------------------------------------------------
# Back-transform pooled estimate
pooled_icc    <- 1 - exp(ma_icc$beta[1])
pooled_icc_lo <- 1 - exp(ma_icc$ci.ub)
pooled_icc_up <- 1 - exp(ma_icc$ci.lb)

cat(sprintf("Pooled ICC = %.3f [%.3f, %.3f]\n",
            pooled_icc, pooled_icc_lo, pooled_icc_up))

## -----------------------------------------------------------------------------
# Use test-retest studies that report both ICC and SD
dat_me <- df.psychom[df.psychom$outcome == "test_retest" &
                     !is.na(df.psychom$sd_scores), ]

# Compute SEM for each study (delta-method SE included)
sem_results <- compute_sem(
  sd       = dat_me$sd_scores,
  icc      = dat_me$icc,
  n_sample = dat_me$n_sample
)

# Chain to SDC
sdc_results <- compute_sdc(
  sem    = sem_results$sem,
  sem_se = sem_results$sem_se
)

## ----eval=FALSE---------------------------------------------------------------
# # Combined measurement error table
# measurement_error <- data.frame(
#   study  = dat_me$study_id,
#   n      = dat_me$n_sample,
#   sd     = dat_me$sd_scores,
#   icc    = dat_me$icc,
#   sem    = round(sem_results$sem, 3),
#   sem_se = round(sem_results$sem_se, 3),
#   sdc    = round(sdc_results$sdc, 3),
#   sdc_se = round(sdc_results$sdc_se, 3)
# )
# measurement_error

## ----echo=FALSE---------------------------------------------------------------
measurement_error <- data.frame(
  study  = dat_me$study_id,
  n      = dat_me$n_sample,
  sd     = dat_me$sd_scores,
  icc    = dat_me$icc,
  sem    = round(sem_results$sem, 3),
  sem_se = round(sem_results$sem_se, 3),
  sdc    = round(sdc_results$sdc, 3),
  sdc_se = round(sdc_results$sdc_se, 3)
)
DT::datatable(measurement_error,
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '100px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))

## ----fig.width=7, fig.height=4------------------------------------------------
ma_sem <- metafor::rma(yi = sem_results$sem, sei = sem_results$sem_se,
                       method = "REML")

metafor::forest(ma_sem,
                slab = paste0(dat_me$author, " (", dat_me$year, ")"),
                xlab = "Standard Error of Measurement (SEM)",
                header = TRUE)

## -----------------------------------------------------------------------------
cat(sprintf("Pooled SEM = %.3f [%.3f, %.3f]\n",
            ma_sem$beta[1], ma_sem$ci.lb, ma_sem$ci.ub))

# Derive pooled SDC from pooled SEM
pooled_sdc <- 1.96 * sqrt(2) * ma_sem$beta[1]
cat(sprintf("Pooled SDC = %.3f\n", pooled_sdc))

## -----------------------------------------------------------------------------
# Subset criterion and construct validity studies
dat_validity <- df.psychom[df.psychom$outcome %in%
                           c("criterion_validity", "construct_validity"), ]

# convert_df automatically handles both Pearson r and Spearman rho
res_r <- convert_df(dat_validity, measure = "r",
                    verbose = FALSE, split_adjusted = FALSE)
summary_r <- summary(res_r, flags = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# summary_r[, c("author", "es", "se", "info_used")]

## ----echo=FALSE---------------------------------------------------------------
DT::datatable(summary_r[, c("author", "es", "se", "es_ci_lo", "es_ci_up", "info_used")],
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '130px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))

## ----fig.width=7, fig.height=5------------------------------------------------
# Pool observed (uncorrected) correlations on Fisher's z scale
z_obs <- atanh(summary_r$es)
r_bounded <- pmin(pmax(summary_r$es, -0.9999), 0.9999)
z_se  <- summary_r$se / (1 - r_bounded^2)

ma_r_obs <- metafor::rma(yi = z_obs, sei = z_se, method = "REML")

# Forest plot on correlation scale
metafor::forest(ma_r_obs,
                slab = paste0(summary_r$author, " (", dat_validity$year, ")"),
                xlab = "Fisher's z (observed correlation)",
                header = TRUE)

## -----------------------------------------------------------------------------
# Back-transform to correlation scale
pooled_r_obs <- tanh(ma_r_obs$beta[1])
pooled_r_lo  <- tanh(ma_r_obs$ci.lb)
pooled_r_up  <- tanh(ma_r_obs$ci.ub)

cat(sprintf("(a) Pooled uncorrected r = %.3f [%.3f, %.3f]\n",
            pooled_r_obs, pooled_r_lo, pooled_r_up))

## -----------------------------------------------------------------------------
# Disattenuate using study-specific reliabilities
corrected <- es_disattenuate(
  r             = summary_r$es,
  r_se          = summary_r$se,
  reliability_x = dat_validity$reliability_target,
  reliability_y = dat_validity$reliability_comparator,
  n_sample      = dat_validity$n_sample
)

## ----eval=FALSE---------------------------------------------------------------
# # Comparison table
# data.frame(
#   study              = dat_validity$study_id,
#   info_used          = summary_r$info_used,
#   r_observed         = round(summary_r$es, 3),
#   r_corrected        = round(corrected$r_corrected, 3),
#   attenuation_factor = round(corrected$attenuation_factor, 3)
# )

## ----echo=FALSE---------------------------------------------------------------
comp <- data.frame(
  study              = dat_validity$study_id,
  info_used          = summary_r$info_used,
  r_observed         = round(summary_r$es, 3),
  r_corrected        = round(corrected$r_corrected, 3),
  attenuation_factor = round(corrected$attenuation_factor, 3)
)
DT::datatable(comp,
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '130px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))

## ----fig.width=7, fig.height=5------------------------------------------------
# Pool corrected correlations on Fisher's z scale
ma_r_corr <- metafor::rma(yi = corrected$z_corrected,
                           sei = corrected$z_corrected_se,
                           method = "REML")

metafor::forest(ma_r_corr,
                slab = paste0(summary_r$author, " (", dat_validity$year, ")"),
                xlab = "Fisher's z (disattenuated correlation)",
                header = TRUE)

## -----------------------------------------------------------------------------
pooled_r_corr <- tanh(ma_r_corr$beta[1])
pooled_r_corr_lo <- tanh(ma_r_corr$ci.lb)
pooled_r_corr_up <- tanh(ma_r_corr$ci.ub)

cat(sprintf("(b) Pooled disattenuated r = %.3f [%.3f, %.3f]\n",
            pooled_r_corr, pooled_r_corr_lo, pooled_r_corr_up))

## ----eval=FALSE---------------------------------------------------------------
# # Use pooled alpha and ICC from earlier sections as proxies
# es_disattenuate(
#   r = 0.65,
#   r_se = 0.08,
#   reliability_x = pooled_alpha,  # from Section 2
#   reliability_y = 0.85,          # from external published data
#   n_sample = 150
# )

## -----------------------------------------------------------------------------
# Subset responsiveness studies
dat_resp <- df.psychom[df.psychom$outcome == "responsiveness", ]

# Step 1: Compute change-score reliability
change_rel <- reliability_change_score(
  reliability = dat_resp$single_occasion_reliability,
  r_pre_post  = dat_resp$r_pre_post
)

# Step 2: Get responsiveness correlations via convert_df
res_resp <- convert_df(dat_resp, measure = "r",
                       verbose = FALSE, split_adjusted = FALSE)
summary_resp <- summary(res_resp)

# Step 3: Disattenuate using CHANGE-SCORE reliability
corrected_resp <- es_disattenuate(
  r             = summary_resp$es,
  r_se          = summary_resp$se,
  reliability_x = change_rel$rel_change,
  reliability_y = dat_resp$reliability_comparator,
  n_sample      = dat_resp$n_sample
)

## ----eval=FALSE---------------------------------------------------------------
# data.frame(
#   study        = dat_resp$study_id,
#   r_observed   = round(summary_resp$es, 3),
#   rel_single   = dat_resp$single_occasion_reliability,
#   r_pre_post   = dat_resp$r_pre_post,
#   rel_change   = round(change_rel$rel_change, 3),
#   r_corrected  = round(corrected_resp$r_corrected, 3)
# )

## ----echo=FALSE---------------------------------------------------------------
resp_table <- data.frame(
  study        = dat_resp$study_id,
  r_observed   = round(summary_resp$es, 3),
  rel_single   = dat_resp$single_occasion_reliability,
  r_pre_post   = dat_resp$r_pre_post,
  rel_change   = round(change_rel$rel_change, 3),
  r_corrected  = round(corrected_resp$r_corrected, 3)
)
DT::datatable(resp_table,
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '110px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))

## ----fig.width=7, fig.height=4------------------------------------------------
# Pool observed change-score correlations
z_resp     <- atanh(summary_resp$es)
r_resp_bounded <- pmin(pmax(summary_resp$es, -0.9999), 0.9999)
z_resp_se  <- summary_resp$se / (1 - r_resp_bounded^2)

ma_resp_obs <- metafor::rma(yi = z_resp, sei = z_resp_se, method = "REML")

cat(sprintf("Pooled observed r = %.3f [%.3f, %.3f]\n",
            tanh(ma_resp_obs$beta[1]),
            tanh(ma_resp_obs$ci.lb),
            tanh(ma_resp_obs$ci.ub)))

# Pool disattenuated change-score correlations
ma_resp_corr <- metafor::rma(yi = corrected_resp$z_corrected,
                              sei = corrected_resp$z_corrected_se,
                              method = "REML")

cat(sprintf("Pooled disattenuated r = %.3f [%.3f, %.3f]\n",
            tanh(ma_resp_corr$beta[1]),
            tanh(ma_resp_corr$ci.lb),
            tanh(ma_resp_corr$ci.ub)))

## -----------------------------------------------------------------------------
# Example: one study with r_observed = 0.50, rel_single = 0.85
r_pre_post_grid <- c(0.20, 0.40, 0.60, 0.80)

sensitivity <- data.frame(
  r_pre_post = r_pre_post_grid,
  rel_change = sapply(r_pre_post_grid, function(rpp) {
    reliability_change_score(reliability = 0.85, r_pre_post = rpp)$rel_change
  }),
  r_corrected = sapply(r_pre_post_grid, function(rpp) {
    rc <- reliability_change_score(reliability = 0.85, r_pre_post = rpp)$rel_change
    es_disattenuate(r = 0.50, r_se = 0.08, reliability_x = rc,
                    reliability_y = 0.80, n_sample = 100)$r_corrected
  })
)
sensitivity

## -----------------------------------------------------------------------------
# Subset floor/ceiling studies
dat_prop <- df.psychom[df.psychom$outcome == "floor_ceiling", ]

# Freeman-Tukey transformation (recommended)
res_prop <- convert_df(dat_prop, measure = "prop", prop_to_es = "freeman_tukey",
                       verbose = FALSE, split_adjusted = FALSE)
summary_prop <- summary(res_prop)

## ----fig.width=7, fig.height=3.5----------------------------------------------
ma_prop <- metafor::rma(yi = es, sei = se, data = summary_prop,
                        method = "REML")

metafor::forest(ma_prop,
                slab = paste0(summary_prop$author, " (", dat_prop$year, ")"),
                xlab = "Freeman-Tukey transformed proportion",
                header = TRUE)

## -----------------------------------------------------------------------------
# Approximate back-transformation (for the pooled estimate)
pooled_prop <- sin(ma_prop$beta[1])^2
cat(sprintf("Pooled proportion (approx.) = %.3f\n", pooled_prop))

#' Convert means and/or standard deviations of two independent groups into two effect measures (VR/CVR)
#'
#' @param mean_exp mean of participants in the experimental/exposed group.
#' @param mean_nexp mean of participants in the non-experimental/non-exposed group.
#' @param mean_sd_exp standard deviation of participants in the experimental/exposed group.
#' @param mean_sd_nexp standard deviation of participants in the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param reverse_means_variability a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the means and standard deviations of two independent groups into a log variability ratio (VR) and a log coefficient of variation ratio (CVR).
#'
#' The formulas used to obtain the log VR are (formulas 5 and 15, Senior et al. 2020):
#' \deqn{logvr = log(\frac{mean\_sd\_exp}{mean\_sd\_nexp}) + \frac{1}{2 * (n\_exp - 1)} - \frac{1}{2 * (n\_nexp - 1)}}
#' \deqn{logvr\_se = \sqrt{\frac{1}{2} * (\frac{n\_nexp}{(n\_nexp - 1)^2} + \frac{n\_exp}{(n\_exp - 1)^2})}}
#' \deqn{logvr\_ci\_lo = logvr - qnorm(.975) * logvr\_se}
#' \deqn{logvr\_ci\_up = logvr + qnorm(.975) * logvr\_se}
#'
#' The formulas used to obtain the log CVR are (formulas 6 and 16, Senior et al. 2020):
#' \deqn{cvt = mean\_sd\_exp / mean\_exp}
#' \deqn{cvc = mean\_sd\_nexp / mean\_nexp}
#' \deqn{logcvr = log(\frac{cvt}{cvc}) + \frac{1}{2} * (\frac{1}{n\_exp - 1} - \frac{1}{n\_nexp - 1}) + \frac{1}{2} * (\frac{mean\_sd\_nexp^2}{n\_nexp * mean\_nexp^2} - \frac{mean\_sd\_exp^2}{n\_exp * mean\_exp^2})}
#' \deqn{vt\_exp = \frac{mean\_sd\_exp^2}{n\_exp * mean\_exp^2} + \frac{mean\_sd\_exp^4}{2 * n\_exp^2 * mean\_exp^4} + \frac{n\_exp}{2 * (n\_exp - 1)^2}}
#' \deqn{vt\_nexp = \frac{mean\_sd\_nexp^2}{n\_nexp * mean\_nexp^2} + \frac{mean\_sd\_nexp^4}{2 * n\_nexp^2 * mean\_nexp^4} + \frac{n\_nexp}{2 * (n\_nexp - 1)^2}}
#' \deqn{logcvr\_se = \sqrt{vt\_exp + vt\_nexp}}
#' \deqn{logcvr\_ci\_lo = logcvr - qnorm(.975) * logcvr\_se}
#' \deqn{logcvr\_ci\_up = logcvr + qnorm(.975) * logcvr\_se}
#'
#' @references
#' Senior, A. M., Viechtbauer, W., & Nakagawa, S. (2020). Revisiting and expanding the meta-analysis of variation: The log coefficient of variation ratio. Research Synthesis Methods, 11(4), 553-567. https://doi.org/10.1002/jrsm.1423
#'
#' @return
#' This function estimates VR and CVR
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab VR + CVR\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab No conversion performed\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 23. User's input (crude)'\cr
#'  \tab https://metaconvert.org/html/input.html\cr
#'  \tab \cr
#' }
#'
#' @export es_variab_from_means_sd
#'
#' @md
#'
#' @examples
#' es_variab_from_means_sd(
#'   n_exp = 55, n_nexp = 55,
#'   mean_exp = 2.3, mean_sd_exp = 1.2,
#'   mean_nexp = 1.9, mean_sd_nexp = 0.9
#' )
es_variab_from_means_sd <- function(mean_exp, mean_nexp, mean_sd_exp, mean_sd_nexp,
                                    n_exp, n_nexp, reverse_means_variability) {
  if (missing(reverse_means_variability)) reverse_means_variability <- rep(FALSE, length(mean_sd_exp))
  reverse_means_variability[is.na(reverse_means_variability)] <- FALSE
  if (length(reverse_means_variability) == 1) reverse_means_variability = c(rep(reverse_means_variability, length(mean_sd_exp)))
  if (length(reverse_means_variability) != length(mean_sd_exp)) stop("The length of the 'reverse_means_variability' argument is incorrectly specified.")

  es <- data.frame(logcvr = rep(NA, length(mean_sd_exp)))
  # es$logcvr = log(mean_sd_exp/mean_exp) + 1/(2 * (n_exp - 1)) - log(mean_sd_nexp/mean_nexp) - 1/(2 * (n_nexp - 1))
  # yi <- log(sd1i/m1i) + 1/(2 * (n1i - 1)) - log(sd2i/m2i) -
  #   1/(2 * (n2i - 1))

  # lnCVRIND2 (equation 6) Nakagawa 202
  CVT = mean_sd_exp / mean_exp
  CVC = mean_sd_nexp / mean_nexp
  logcvr <- suppressWarnings(
    log(CVT / CVC) +
      1/2 * (1/(n_exp-1) - 1/(n_nexp-1)) +
      1/2 * (mean_sd_nexp^2 / (n_nexp * mean_nexp^2) - mean_sd_exp^2 / (n_exp * mean_exp^2))
  )
  es$logcvr <- ifelse(reverse_means_variability, -logcvr, logcvr)

  # s2IND2 (equation 16) Senior et al. 2020.
  # NB: the Var(ln s) term carries a 1/2 factor -- Var(ln s) ~= trigamma((n-1)/2)/4
  # ~= 0.5 * n/(n-1)^2 for a normal sample -- and the SAME quantity in logvr_se below is
  # correctly written 1/2 * n/(n-1)^2. Without the 1/2 the lnCVR variance is ~2x too
  # large (SE ~40% too wide, inverse-variance weights ~2x off).
  es$logcvr_se <- sqrt(
    mean_sd_exp^2 / (n_exp * mean_exp^2) +
    mean_sd_exp^4 / (2 * n_exp^2 * mean_exp^4) +
    0.5 * n_exp / ((n_exp - 1)^2) +
    mean_sd_nexp^2 / (n_nexp * mean_nexp^2) +
    mean_sd_nexp^4 / (2 * n_nexp^2 * mean_nexp^4) +
    0.5 * n_nexp / ((n_nexp - 1)^2))

  es$logcvr_ci_lo <- es$logcvr - qnorm(.975) * es$logcvr_se
  es$logcvr_ci_up <- es$logcvr + qnorm(.975) * es$logcvr_se

  logvr <- suppressWarnings(
    log(mean_sd_exp / mean_sd_nexp) + 1 / (2 * (n_exp - 1)) - 1 / (2 * (n_nexp - 1))
  )
  es$logvr <- ifelse(reverse_means_variability, -logvr, logvr)
  es$logvr_se <- sqrt(1/2 * (n_nexp/((n_nexp - 1)^2) + n_exp/((n_exp - 1)^2)))
  # es$logvr_se <- sqrt(1 / (2 * (n_exp - 1)) + 1 / (2 * (n_nexp - 1)))
  es$logvr_ci_lo <- es$logvr - qnorm(.975) * es$logvr_se
  es$logvr_ci_up <- es$logvr + qnorm(.975) * es$logvr_se

  es$info_used <- "variability_means_sd"

  return(es)
}

#' Convert means and/or standard errors of two independent groups into two effect measures (VR/CVR)
#'
#' @param mean_exp mean of participants in the experimental/exposed group.
#' @param mean_nexp mean of participants in the non-experimental/non-exposed group.
#' @param mean_se_exp standard error of participants in the experimental/exposed group.
#' @param mean_se_nexp standard error of participants in the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param reverse_means_variability a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the standard errors into standard deviations
#' (\eqn{SE = SD / \sqrt{n}}, so \eqn{SD = SE \times \sqrt{n}}).
#' \deqn{mean\_sd\_exp = mean\_se\_exp * \sqrt{n\_exp}}
#' \deqn{mean\_sd\_nexp = mean\_se\_nexp * \sqrt{n\_nexp}}
#'
#' Then, calculations of the \code{\link{es_variab_from_means_sd}} are applied.
#'
#' @return
#' This function estimates VR and CVR
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab VR + CVR\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab No conversion performed\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 23. User's input (crude)'\cr
#'  \tab https://metaconvert.org/html/input.html\cr
#'  \tab \cr
#' }
#'
#' @references
#' Senior, A. M., Viechtbauer, W., & Nakagawa, S. (2020). Revisiting and expanding the meta-analysis of variation: The log coefficient of variation ratio. Research Synthesis Methods, 11(4), 553-567. https://doi.org/10.1002/jrsm.1423
#'
#' @export es_variab_from_means_se
#'
#' @md
#'
#' @examples
#' es_variab_from_means_se(
#'   mean_exp = 42, mean_se_exp = 11,
#'   mean_nexp = 42, mean_se_nexp = 15,
#'   n_exp = 43, n_nexp = 34
#' )
es_variab_from_means_se <- function(mean_exp, mean_nexp, mean_se_exp, mean_se_nexp,
                                    n_exp, n_nexp, reverse_means_variability) {
  if (missing(reverse_means_variability)) reverse_means_variability <- rep(FALSE, length(mean_se_exp))
  reverse_means_variability[is.na(reverse_means_variability)] <- FALSE

  sd_exp <- mean_se_exp * sqrt(n_exp)
  sd_nexp <- mean_se_nexp * sqrt(n_nexp)

  es <- es_variab_from_means_sd(
    mean_exp = mean_exp, mean_nexp = mean_nexp,
    mean_sd_exp = sd_exp, mean_sd_nexp = sd_nexp,
    n_exp = n_exp, n_nexp = n_nexp, reverse_means_variability = reverse_means_variability
  )
  es$info_used <- "variability_means_se"

  return(es)
}

#' Title
#'
#' @param mean_exp mean of participants in the experimental/exposed group.
#' @param mean_nexp mean of participants in the non-experimental/non-exposed group.
#' @param mean_ci_lo_exp lower bound of the 95% CI of the mean of the experimental/exposed group
#' @param mean_ci_up_exp upper bound of the 95% CI of the mean of the experimental/exposed group
#' @param mean_ci_lo_nexp lower bound of the 95% CI of the mean of the non-experimental/non-exposed group.
#' @param mean_ci_up_nexp upper bound of the 95% CI of the mean of the non-experimental/non-exposed group.
#' @param n_exp number of participants in the experimental/exposed group.
#' @param n_nexp number of participants in the non-experimental/non-exposed group.
#' @param reverse_means_variability a logical value indicating whether the direction of the generated effect sizes should be flipped.
#'
#' @details
#' This function converts the bounds of the 95% CI of the means of two independent groups into standard errors.
#' \deqn{mean\_se\_exp = \frac{mean\_ci\_up\_exp - mean\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
#' \deqn{mean\_se\_nexp = \frac{mean\_ci\_up\_nexp - mean\_ci\_lo\_nexp}{2 * qt{(0.975, df = n\_nexp - 1)}}}
#'
#' Then, calculations of the \code{\link{es_variab_from_means_se}} are applied.
#'
#' @references
#' Senior, A. M., Viechtbauer, W., & Nakagawa, S. (2020). Revisiting and expanding the meta-analysis of variation: The log coefficient of variation ratio. Research Synthesis Methods, 11(4), 553-567. https://doi.org/10.1002/jrsm.1423
#'
#' @export es_variab_from_means_ci
#'
#' @return
#' This function estimates VR and CVR
#'
#' \tabular{ll}{
#'  \code{natural effect size measure} \tab VR + CVR\cr
#'  \tab \cr
#'  \code{converted effect size measure} \tab No conversion performed\cr
#'  \tab \cr
#'  \code{required input data} \tab See 'Section 23. User's input (crude)'\cr
#'  \tab https://metaconvert.org/html/input.html\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @examples
#' es_variab_from_means_ci(
#'   mean_exp = 42, mean_ci_lo_exp = 32, mean_ci_up_exp = 52,
#'   mean_nexp = 42, mean_ci_lo_nexp = 37, mean_ci_up_nexp = 47,
#'   n_exp = 43, n_nexp = 34
#' )
es_variab_from_means_ci <- function(mean_exp, mean_nexp,
                                    mean_ci_lo_exp, mean_ci_up_exp,
                                    mean_ci_lo_nexp, mean_ci_up_nexp,
                                    n_exp, n_nexp, reverse_means_variability) {
  if (missing(reverse_means_variability)) reverse_means_variability <- rep(FALSE, length(mean_ci_up_exp))
  reverse_means_variability[is.na(reverse_means_variability)] <- FALSE

  se_exp <- (mean_ci_up_exp - mean_ci_lo_exp) / (2 * qt(0.975, n_exp - 1))
  se_nexp <- (mean_ci_up_nexp - mean_ci_lo_nexp) / (2 * qt(0.975, n_nexp - 1))

  es <- es_variab_from_means_se(
    mean_exp = mean_exp, mean_nexp = mean_nexp,
    mean_se_exp = se_exp, mean_se_nexp = se_nexp,
    n_exp = n_exp, n_nexp = n_nexp, reverse_means_variability = reverse_means_variability
  )
  es$info_used <- "variability_means_ci"

  return(es)
}

#' metaConvert: An R Package Dedicated to Automated Effect Size Calculations
#'
#' The \pkg{metaConvert} package automatically estimates 14 effect size measures from a well-formatted dataframe.
#' Various other functions can help, for example, removing dependency between several effect sizes,
#' or identifying differences between two dataframes.
#' This package is mainly designed to assist in conducting a systematic review with a meta-analysis, but it can be
#' useful to any researcher interested in estimating an effect size.
#'
#' # Overview of the package
#' To visualize all the types of input data that can be used to estimate the 14 effect size measures available
#' in metaConvert, you can use the \code{\link{see_input_data}()} function.
#'
#' # Estimate effect sizes
#' To automatically estimate effect sizes directly from a dataset, you can use the \code{\link{convert_df}()} function.
#'
#' # Aggregate dependent effect sizes
#' To automatically aggregate dependent effect sizes using Borenstein's formulas,
#' you can use the \code{\link{aggregate_df}()} function. This function can handle dependent effect sizes
#' from multiple subgroups, or dependent effect sizes from the same participants.
#'
#' # Flag differences between two datasets
#' If pairs of data extractors have generated similar datasets that should be compared,
#' you can use the \code{\link{compare_df}()} function.
#'
#' # Prepare a dataset extraction sheet
#' If you have not started data extraction yet,
#' you can use the \code{\link{data_extraction_sheet}()} function to obtain a
#' perfectly formatted data extraction sheet.
#'
#' @details
#'
#' # Well-formatted dataset
#' One of the specificities of the \pkg{metaConvert} package is that its core function (\code{\link{convert_df}})
#' does not have arguments to specify the names of the variables contained in the dataset.
#' While this allow using a convenient automatic process in the calculations, this requires that the datasets
#' passed to this function respect a very precise formatting (which we will refer to as \code{well-formatted dataset}).
#'
#' Rather than a long description of all column names, we built several tools
#' that help you find required information.
#' 1. You can use the \code{\link{data_extraction_sheet}()} function that
#' generates an excel/csv/txt file containing all the column names available,
#' as well as a description of the information it should contain.
#' 2. You can use the \code{\link{see_input_data}()} function that generates
#' a list of all available types of input data as well as their estimated/converted
#' effect size measures. This function also points out to the corresponding helper tables
#' available in https://metaconvert.org
#'
#'
#' # Effect size measures available
#'
#' Eleven effect size measures are accepted:
#'
#' \itemize{
#'  \item \bold{"d"}: standardized mean difference (i.e., Cohen's d)
#'  \item \bold{"g"}: Hedges' g
#'  \item \bold{"md"}: mean difference
#'  \item \bold{"r"}: Correlation coefficient
#'  \item \bold{"z"}: Fisher's r-to-z correlation
#'  \item \bold{"or"} or \bold{"logor"}: odds ratio  or its logarithm
#'  \item \bold{"rr"} or \bold{"logrr"}: risk ratio or its logarithm
#'  \item \bold{"irr"} or \bold{"logirr"}: incidence rate ratio or its logarithm
#'  \item \bold{"nnt"}: number needed to treat
#'  \item \bold{"logcvr"}: log coefficient of variation
#'  \item \bold{"logvr"}: log variability ratio
#' }
#'
#' # Output
#' All the functions of the \pkg{metaConvert} package that are dedicated to effect size calculations
#' (i.e., all the functions named \code{es_from_*}) return a dataframe that contain,
#' depending on the function - some of the following columns:
#' \tabular{ll}{
#'  \code{info_used} \tab input data used to generate the effect size.\cr
#'  \tab \cr
#'  \code{md} \tab value of the mean difference.\cr
#'  \tab \cr
#'  \code{md_se} \tab standard error of the mean difference.\cr
#'  \tab \cr
#'  \code{md_ci_lo} \tab lower bound of the 95% CI of the mean difference.\cr
#'  \tab \cr
#'  \code{md_ci_up} \tab upper bound of the 95% CI of the mean difference.\cr
#'  \tab \cr
#'  \code{d} \tab value of the Cohen's d.\cr
#'  \tab \cr
#'  \code{d_se} \tab standard error of the Cohen's d.\cr
#'  \tab \cr
#'  \code{d_ci_lo} \tab lower bound of the 95% CI of the Cohen's d.\cr
#'  \tab \cr
#'  \code{d_ci_up} \tab upper bound of the 95% CI of the Cohen's d.\cr
#'  \tab \cr
#'  \code{g} \tab value of the Hedges' g.\cr
#'  \tab \cr
#'  \code{g_se} \tab standard error of the Hedges' g.\cr
#'  \tab \cr
#'  \code{g_ci_lo} \tab lower bound of the 95% CI of the Hedges' g.\cr
#'  \tab \cr
#'  \code{g_ci_up} \tab upper bound of the 95% CI of the Hedges' g.\cr
#'  \tab \cr
#'  \code{r} \tab value of the correlation coefficient.\cr
#'  \tab \cr
#'  \code{r_se} \tab standard error of the correlation coefficient.\cr
#'  \tab \cr
#'  \code{r_ci_lo} \tab lower bound of the 95% CI of the correlation coefficient.\cr
#'  \tab \cr
#'  \code{r_ci_up} \tab upper bound of the 95% CI of the correlation coefficient.\cr
#'  \tab \cr
#'  \code{z} \tab value of the r-to-z transformed correlation coefficient.\cr
#'  \tab \cr
#'  \code{z_se} \tab standard error of the r-to-z transformed correlation coefficient.\cr
#'  \tab \cr
#'  \code{z_ci_lo} \tab lower bound of the 95% CI of the r-to-z transformed correlation coefficient.\cr
#'  \tab \cr
#'  \code{z_ci_up} \tab upper bound of the 95% CI of the r-to-z transformed correlation coefficient.\cr
#'  \tab \cr
#'  \code{logor} \tab value of the log odds ratio.\cr
#'  \tab \cr
#'  \code{logor_se} \tab standard error of the log odds ratio.\cr
#'  \tab \cr
#'  \code{logor_ci_lo} \tab lower bound of the 95% CI of the log odds ratio.\cr
#'  \tab \cr
#'  \code{logor_ci_up} \tab upper bound of the 95% CI of the log odds ratio.\cr
#'  \tab \cr
#'  \code{logrr} \tab value of the log risk ratio.\cr
#'  \tab \cr
#'  \code{logrr_se} \tab standard error of the log risk ratio.\cr
#'  \tab \cr
#'  \code{logrr_ci_lo} \tab lower bound of the 95% CI of the log risk ratio.\cr
#'  \tab \cr
#'  \code{logrr_ci_up} \tab upper bound of the 95% CI of the log risk ratio.\cr
#'  \tab \cr
#'  \code{logirr} \tab value of the log incidence rate ratio.\cr
#'  \tab \cr
#'  \code{logirr_se} \tab standard error of the log incidence rate ratio.\cr
#'  \tab \cr
#'  \code{logirr_ci_lo} \tab lower bound of the 95% CI of the log incidence rate ratio.\cr
#'  \tab \cr
#'  \code{logirr_ci_up} \tab upper bound of the 95% CI of the log incidence rate ratio.\cr
#'  \tab \cr
#'  \code{logvr} \tab value of the log variability ratio.\cr
#'  \tab \cr
#'  \code{logvr_se} \tab standard error of the log variability ratio.\cr
#'  \tab \cr
#'  \code{logvr_ci_lo} \tab lower bound of the 95% CI of the log variability ratio.\cr
#'  \tab \cr
#'  \code{logvr_ci_up} \tab upper bound of the 95% CI of the log variability ratio.\cr
#'  \tab \cr
#'  \code{logcvr} \tab value of the log coefficient of variation.\cr
#'  \tab \cr
#'  \code{logcvr_se} \tab standard error of the log coefficient of variation.\cr
#'  \tab \cr
#'  \code{logcvr_ci_lo} \tab lower bound of the 95% CI of the log coefficient of variation.\cr
#'  \tab \cr
#'  \code{logcvr_ci_up} \tab upper bound of the 95% CI of the log coefficient of variation.\cr
#'  \tab \cr
#'  \code{nnt} \tab number needed to treat.\cr
#'  \tab \cr
#' }
#'
#' @md
#'
#' @name metaConvert-package
"_PACKAGE"

#' @importFrom stats sd aggregate.data.frame df dnorm na.omit optimize pnorm pt qnorm qt
#' @importFrom utils packageVersion
NULL
utils::globalVariables(c(
  "y", "discard", "reshape",
  "row_id", "study_id", "author", "year", "predictor",
  "outcome", "info_expected", "adjusted_input", "df.haza"
))

---
title: "Vignette 2. Psychometric meta-analysis with metaConvert (COSMIN framework)"
author: "Gosling CJ, Cortese S, Solmi M, Haza B, Vieta E, Delorme R, Fusar-Poli P, Radua J"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Vignette 2. Psychometric meta-analysis with metaConvert (COSMIN framework)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{=html}
<style type="text/css">

*{
  font-family: "Gill Sans", sans-serif;
}

h1.title {
  font-weight: 700;
  font-size: 2.2rem;
  padding-top: 0rem;
  margin-top: 0rem;
  border-top: none;
}

#TOC {
  width: 100%;
}

h1{
  font-weight: 550;
  font-size: 1.9rem;
  border-top: 1px solid black;
  margin-top: 3rem;
  padding-top: 2rem;
}


p{
  line-height: 1.4rem;
}
</style>
```

```{r echo=FALSE, message=FALSE, warning=FALSE, results='hide'}
library(metaConvert)
library(metafor)
library(DT)
```

# Introduction

The **COSMIN** (COnsensus-based Standards for the selection of health Measurement INstruments) framework provides a standardised taxonomy of measurement properties for evaluating patient-reported outcome measures (PROMs). Conducting a **meta-analysis of measurement properties** requires computing, transforming, and pooling a variety of effect sizes that are specific to psychometric studies.

The `metaConvert` package provides functions for several COSMIN measurement properties. The table below summarises the coverage:

| COSMIN property | Coverage | metaConvert function(s) |
|:---|:---:|:---|
| Internal consistency | Full | `convert_df(measure = "alpha")` |
| Test-retest reliability | Full | `convert_df(measure = "icc")` |
| Measurement error | Full | `compute_sem()`, `compute_sdc()` |
| Criterion validity | Full | `convert_df(measure = "r")` + `es_disattenuate()` |
| Construct validity | Full | `convert_df(measure = "r")` + `es_disattenuate()` |
| Responsiveness | Full | `reliability_change_score()` + `es_disattenuate()` |
| Floor/ceiling effects | Full | `convert_df(measure = "prop")` |
| Structural validity | Not covered | Requires OSMASEM (metaSEM / OpenMx) |
| Diagnostic accuracy | Not covered | Requires bivariate GLMM (mada / lme4) |
| Cross-cultural validity | Partial | DIF log-odds via OR pipeline; invariance is descriptive |
| Interpretability (MIC) | Partial | Raw MIC via user-provided ES columns |

Throughout this vignette, we use the **`df.psychom`** dataset distributed with metaConvert. It simulates a COSMIN systematic review of a fictional 9-item self-report PROM called the **Mental Health Wellbeing Scale (MHWS)**, scored 0--27, validated across multiple populations and languages.

```{r}
data(df.psychom)
```

```{r, eval=FALSE}
str(df.psychom)
```

```{r, echo=FALSE}
DT::datatable(df.psychom, options = list(
    scrollX = TRUE,
    dom = c('t'),
    scrollY = "300px",
    pageLength = 50,
    ordering = FALSE,
    rownames = FALSE,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center',
                                     targets = "_all"))))
```

<br>

Each row represents one study reporting one measurement property. The `outcome` column identifies the COSMIN property assessed. Columns not relevant to a study's property are `NA`, reflecting the typical sparsity of real-world data extraction sheets.

# Data preparation

Before data extraction, you can generate a template extraction sheet for each psychometric measure. These templates list all column names that `metaConvert` recognises.

```{r, eval=FALSE}
# Extraction sheet for Cronbach's alpha
data_extraction_sheet(measure = "alpha", extension = "data.frame")

# Extraction sheet for ICC
data_extraction_sheet(measure = "icc", extension = "data.frame")

# Extraction sheet for correlations (Pearson or Spearman)
data_extraction_sheet(measure = "r", extension = "data.frame")

# Extraction sheet for proportions
data_extraction_sheet(measure = "prop", extension = "data.frame")
```

The key psychometric columns are:

| Column | Type | Used by |
|:---|:---|:---|
| `cronbach_alpha` | numeric | `es_from_cronbach_alpha()` |
| `n_items` | numeric | `es_from_cronbach_alpha()` |
| `icc` | numeric | `es_from_icc()` |
| `n_measurements` | numeric | `es_from_icc()` |
| `icc_type` | character | `es_from_icc()` -- `"agreement"` or `"consistency"` |
| `spearman_r` | numeric | `es_from_spearman_rho()` |
| `prop` | numeric | `es_from_prop_single_group()` |
| `n_cases` | numeric | `es_from_prop_single_group_counts()` |


# Internal consistency

Internal consistency is typically assessed using **Cronbach's alpha**. For meta-analysis, the **Bonett (2002) variance-stabilising transformation** is recommended:

$$T(\alpha) = \ln(1 - \alpha), \quad \text{SE} = \sqrt{\frac{2k}{(k-1)(n-2)}}$$

where $k$ is the number of items and $n$ is the sample size (Bonett, 2002). This transformation normalises the sampling distribution and stabilises the variance, which is particularly important when alpha values are close to 1.

## Effect size computation

```{r}
# Subset internal consistency studies
dat_alpha <- df.psychom[df.psychom$outcome == "internal_consistency", ]

# Compute effect sizes with Bonett transformation (default)
res_alpha <- convert_df(dat_alpha, measure = "alpha",
                        verbose = FALSE, split_adjusted = FALSE)
summary_alpha <- summary(res_alpha, flags = TRUE, guidance = TRUE)
```

```{r, eval=FALSE}
summary_alpha[, c("author", "es", "se", "es_ci_lo", "es_ci_up", "info_used")]
```

```{r, echo=FALSE}
DT::datatable(summary_alpha[, c("author", "es", "se", "es_ci_lo", "es_ci_up", "info_used")],
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '130px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))
```

<br>

The `es` column contains Bonett-transformed values $\ln(1 - \alpha)$, which are always negative (since $\alpha \in [0, 1)$). To **back-transform** for reporting:

$$\hat{\alpha} = 1 - \exp(T)$$

## Meta-analysis

```{r, fig.width=7, fig.height=4}
# Random-effects meta-analysis on Bonett-transformed scale
ma_alpha <- metafor::rma(yi = es, sei = se, data = summary_alpha,
                         method = "REML")

# Forest plot
metafor::forest(ma_alpha, slab = paste0(summary_alpha$author, " (", dat_alpha$year, ")"),
                xlab = "Bonett-transformed alpha: ln(1 - alpha)",
                header = TRUE)
```

```{r}
# Back-transform pooled estimate and CI
pooled_alpha    <- 1 - exp(ma_alpha$beta[1])
pooled_alpha_lo <- 1 - exp(ma_alpha$ci.ub)
pooled_alpha_up <- 1 - exp(ma_alpha$ci.lb)

cat(sprintf("Pooled alpha = %.3f [%.3f, %.3f]\n",
            pooled_alpha, pooled_alpha_lo, pooled_alpha_up))
```

Note that the CI bounds are **swapped** when back-transforming because $1 - \exp(x)$ is a decreasing function of $x$.

The pooled reliability estimate can later serve as input for **disattenuation corrections** in criterion validity, construct validity, and responsiveness analyses (see below).

## Alternative: raw alpha

An alternative is to meta-analyse raw Cronbach's alpha values directly (`alpha_to_es = "raw"`), using the SE formula from Feldt et al. (1987). This is simpler to interpret but the sampling distribution may be skewed when alpha is close to 1. The Bonett transformation is generally recommended.

```{r, eval=FALSE}
# Raw alpha (not recommended for meta-analysis)
res_alpha_raw <- convert_df(dat_alpha, measure = "alpha", alpha_to_es = "raw",
                            verbose = FALSE, split_adjusted = FALSE)
summary(res_alpha_raw)
```


# Test-retest reliability

Test-retest reliability is assessed using the **intraclass correlation coefficient (ICC)**. The same **Bonett (2002) transformation** is applied:

$$T(\text{ICC}) = \ln(1 - \text{ICC})$$

The SE of the **transformed** ICC is obtained by the delta method: the $(1-\rho)^2$ term from the raw ICC variance cancels with the $1/(1-\rho)^2$ from the transformation derivative. For a single-measure ICC, this leading-order variance is the **same** for the two-way absolute-agreement ICC(2,1) and the two-way consistency ICC(3,1):

- **ICC(2,1) -- absolute agreement** (`icc_type = "agreement"`) and **ICC(3,1) -- consistency** (`icc_type = "consistency"`):
$$\text{SE}(T) = \sqrt{\frac{2(1+(k-1)\rho)^2}{k(k-1)(n-1)}}$$

where $k$ is the number of measurement occasions and $\rho$ is the ICC value. Deriving the ICC(3,1) variance directly from $F_0 = \text{MSR}/\text{MSE}$ (degrees of freedom $n-1$ and $(n-1)(k-1)$) yields $\left[(1+(k-1)\rho)/k\right]^2 \cdot 2k/((k-1)(n-1))$, which reduces to the same expression --- so, contrary to a common simplification, the ICC(3,1) SE is **not** independent of $\rho$ (a Monte Carlo check confirms the $\rho$-dependent form; a $\rho$-free $\sqrt{2/((k-1)(n-1))}$ under-states the SE by up to ~2--3$\times$ in variance for high reliabilities).

**Technical note on the ICC(2,1) formula**: The SE formula labelled "ICC(2,1)" above is technically the analytical variance for the **one-way random ICC(1,1)** model (Shrout & Fleiss, 1979), as derived by Bonett (2002). The exact two-way random ICC(2,1) variance formula involves separate between-rater and residual variance components that are typically unavailable without the raw ANOVA mean squares. Using the one-way formula as a proxy for ICC(2,1) is standard practice in psychometric meta-analysis (see e.g., Bonett, 2002; Donner & Eliasziw, 1987) and provides a reasonable approximation.

## Effect size computation

```{r}
# Subset test-retest studies
dat_icc <- df.psychom[df.psychom$outcome == "test_retest", ]

# Compute effect sizes with Bonett transformation (default)
res_icc <- convert_df(dat_icc, measure = "icc",
                      verbose = FALSE, split_adjusted = FALSE)
summary_icc <- summary(res_icc, flags = TRUE)
```

```{r, eval=FALSE}
summary_icc[, c("author", "es", "se", "es_ci_lo", "es_ci_up",
                 "icc_type", "info_used")]
```

```{r, echo=FALSE}
DT::datatable(summary_icc[, c("author", "es", "se", "es_ci_lo", "es_ci_up",
                                "icc_type", "info_used")],
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '130px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))
```

<br>

Note that studies using different ICC models (agreement vs. consistency) can coexist in the same analysis. Both types share the same leading-order SE formula: it is exact for consistency ICC(3,1), but for agreement ICC(2,1) it is a one-way approximation that assumes negligible between-rater variance -- when raters differ systematically, the reported SE is anti-conservative, and `summary()` marks agreement-type rows with an informational flag (see `?es_from_icc`). When pooling, `icc_type` can be examined as a **moderator** variable.

## Meta-analysis

```{r, fig.width=7, fig.height=4}
# Random-effects meta-analysis
ma_icc <- metafor::rma(yi = es, sei = se, data = summary_icc,
                       method = "REML")

# Forest plot
metafor::forest(ma_icc, slab = paste0(summary_icc$author, " (", dat_icc$year, ")"),
                xlab = "Bonett-transformed ICC: ln(1 - ICC)",
                header = TRUE)
```

```{r}
# Back-transform pooled estimate
pooled_icc    <- 1 - exp(ma_icc$beta[1])
pooled_icc_lo <- 1 - exp(ma_icc$ci.ub)
pooled_icc_up <- 1 - exp(ma_icc$ci.lb)

cat(sprintf("Pooled ICC = %.3f [%.3f, %.3f]\n",
            pooled_icc, pooled_icc_lo, pooled_icc_up))
```

The pooled ICC will feed into the **measurement error** derivation (next section) and **disattenuation** corrections for validity and responsiveness.

Studies that report only Pearson correlations for test-retest reliability (rather than ICC) can be pooled separately using `convert_df(..., measure = "r")`.


# Measurement error

Measurement error is characterised by the **standard error of measurement (SEM)** and the **smallest detectable change (SDC)**. These are not traditional effect sizes but are essential COSMIN measurement properties.

The SEM is derived from matched ICC and SD data:

$$\text{SEM} = \text{SD} \times \sqrt{1 - \text{ICC}}$$

The SDC is derived from the SEM:

$$\text{SDC} = 1.96 \times \sqrt{2} \times \text{SEM}$$

These are computed using **standalone utility functions** (not `convert_df()`), because they require combining reliability and variability from the same sample.

## Study-specific SEM and SDC

```{r}
# Use test-retest studies that report both ICC and SD
dat_me <- df.psychom[df.psychom$outcome == "test_retest" &
                     !is.na(df.psychom$sd_scores), ]

# Compute SEM for each study (delta-method SE included)
sem_results <- compute_sem(
  sd       = dat_me$sd_scores,
  icc      = dat_me$icc,
  n_sample = dat_me$n_sample
)

# Chain to SDC
sdc_results <- compute_sdc(
  sem    = sem_results$sem,
  sem_se = sem_results$sem_se
)
```

```{r, eval=FALSE}
# Combined measurement error table
measurement_error <- data.frame(
  study  = dat_me$study_id,
  n      = dat_me$n_sample,
  sd     = dat_me$sd_scores,
  icc    = dat_me$icc,
  sem    = round(sem_results$sem, 3),
  sem_se = round(sem_results$sem_se, 3),
  sdc    = round(sdc_results$sdc, 3),
  sdc_se = round(sdc_results$sdc_se, 3)
)
measurement_error
```

```{r, echo=FALSE}
measurement_error <- data.frame(
  study  = dat_me$study_id,
  n      = dat_me$n_sample,
  sd     = dat_me$sd_scores,
  icc    = dat_me$icc,
  sem    = round(sem_results$sem, 3),
  sem_se = round(sem_results$sem_se, 3),
  sdc    = round(sdc_results$sdc, 3),
  sdc_se = round(sdc_results$sdc_se, 3)
)
DT::datatable(measurement_error,
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '100px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))
```

## Pooling SEM

The study-specific SEM values can be pooled using inverse-variance random-effects meta-analysis, with the delta-method SE providing the sampling variance:

```{r, fig.width=7, fig.height=4}
ma_sem <- metafor::rma(yi = sem_results$sem, sei = sem_results$sem_se,
                       method = "REML")

metafor::forest(ma_sem,
                slab = paste0(dat_me$author, " (", dat_me$year, ")"),
                xlab = "Standard Error of Measurement (SEM)",
                header = TRUE)
```

```{r}
cat(sprintf("Pooled SEM = %.3f [%.3f, %.3f]\n",
            ma_sem$beta[1], ma_sem$ci.lb, ma_sem$ci.ub))

# Derive pooled SDC from pooled SEM
pooled_sdc <- 1.96 * sqrt(2) * ma_sem$beta[1]
cat(sprintf("Pooled SDC = %.3f\n", pooled_sdc))
```

For interpretability, the ratio of pooled SDC to pooled MIC (minimal important change) indicates whether the instrument can reliably detect changes that patients consider meaningful. A ratio $< 1$ (SDC < MIC) is desirable.

**Important**: SEM and SDC are expressed in the **raw units of the measurement instrument**. Pooling is only valid when all primary studies use the exact same version of the instrument with the same scoring range. If some studies use the original 0--27 MHWS and others use a modified 0--100 version, their SEMs are on different scales and cannot be pooled directly. In such cases, consider standardising by the score SD (computing the coefficient of variation SEM/SD) or pooling only within homogeneous subgroups.


# Criterion and construct validity

Criterion and construct validity are assessed by pooling **correlations** between the MHWS and comparator instruments. Some studies report Pearson's $r$, others report Spearman's $\rho$. The `metaConvert` pipeline handles both: Spearman correlations are automatically converted to Pearson-equivalent values using the **Rupinski & Dunlap (1996)** formula:

$$r_{\text{Pearson}} = 2 \sin\!\left(\frac{\pi}{6} \times r_{\text{Spearman}}\right)$$

The SE is derived via the delta method, correctly propagating the uncertainty from the conversion.

Results should be reported three ways:

- **(a) Uncorrected**: observed correlations pooled directly
- **(b) Individually corrected**: study-specific disattenuation, then pooled
- **(c) Artifact-distribution corrected**: using the distribution of reliabilities across studies (Hunter & Schmidt method; see the `psychmeta` package)

This vignette demonstrates approaches (a) and (b).

## Effect size computation

```{r}
# Subset criterion and construct validity studies
dat_validity <- df.psychom[df.psychom$outcome %in%
                           c("criterion_validity", "construct_validity"), ]

# convert_df automatically handles both Pearson r and Spearman rho
res_r <- convert_df(dat_validity, measure = "r",
                    verbose = FALSE, split_adjusted = FALSE)
summary_r <- summary(res_r, flags = TRUE)
```

```{r, eval=FALSE}
summary_r[, c("author", "es", "se", "info_used")]
```

```{r, echo=FALSE}
DT::datatable(summary_r[, c("author", "es", "se", "es_ci_lo", "es_ci_up", "info_used")],
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '130px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))
```

<br>

The `info_used` column distinguishes between `"pearson_r"` and `"spearman_r"` sources, providing traceability for each study's conversion pathway.

## (a) Pooling uncorrected correlations

Meta-analysis of correlations is typically performed on the **Fisher's z** scale, then back-transformed. Here, `convert_df` with `measure = "r"` returns correlations in the `es` column. The SE on the Fisher's z scale is derived via the delta method: $\text{SE}(z) = \text{SE}(r) / (1 - r^2)$, which correctly propagates the uncertainty from any upstream conversion (e.g., Spearman to Pearson):

```{r, fig.width=7, fig.height=5}
# Pool observed (uncorrected) correlations on Fisher's z scale
z_obs <- atanh(summary_r$es)
r_bounded <- pmin(pmax(summary_r$es, -0.9999), 0.9999)
z_se  <- summary_r$se / (1 - r_bounded^2)

ma_r_obs <- metafor::rma(yi = z_obs, sei = z_se, method = "REML")

# Forest plot on correlation scale
metafor::forest(ma_r_obs,
                slab = paste0(summary_r$author, " (", dat_validity$year, ")"),
                xlab = "Fisher's z (observed correlation)",
                header = TRUE)
```

```{r}
# Back-transform to correlation scale
pooled_r_obs <- tanh(ma_r_obs$beta[1])
pooled_r_lo  <- tanh(ma_r_obs$ci.lb)
pooled_r_up  <- tanh(ma_r_obs$ci.ub)

cat(sprintf("(a) Pooled uncorrected r = %.3f [%.3f, %.3f]\n",
            pooled_r_obs, pooled_r_lo, pooled_r_up))
```

## (b) Individual disattenuation then pooling

The `es_disattenuate()` function corrects each observed correlation for measurement error in both the target PROM and the comparator instrument:

$$r_{\text{corrected}} = \frac{r_{\text{observed}}}{\sqrt{\text{rel}_X \times \text{rel}_Y}}$$

```{r}
# Disattenuate using study-specific reliabilities
corrected <- es_disattenuate(
  r             = summary_r$es,
  r_se          = summary_r$se,
  reliability_x = dat_validity$reliability_target,
  reliability_y = dat_validity$reliability_comparator,
  n_sample      = dat_validity$n_sample
)
```

```{r, eval=FALSE}
# Comparison table
data.frame(
  study              = dat_validity$study_id,
  info_used          = summary_r$info_used,
  r_observed         = round(summary_r$es, 3),
  r_corrected        = round(corrected$r_corrected, 3),
  attenuation_factor = round(corrected$attenuation_factor, 3)
)
```

```{r, echo=FALSE}
comp <- data.frame(
  study              = dat_validity$study_id,
  info_used          = summary_r$info_used,
  r_observed         = round(summary_r$es, 3),
  r_corrected        = round(corrected$r_corrected, 3),
  attenuation_factor = round(corrected$attenuation_factor, 3)
)
DT::datatable(comp,
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '130px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))
```

<br>

The corrected correlations are systematically larger in magnitude, reflecting the removal of attenuation due to measurement error.

**Note on bounds**: Because sampling error exists in both the observed correlation and the reliability estimates, `r_corrected` can occasionally exceed 1.0 in absolute value. When this happens, `es_disattenuate()` internally caps the corrected correlation at $\pm$0.9999 before computing Fisher's z, so the meta-analytic pipeline will not produce `NaN` values.

However, a corrected correlation exceeding 1.0 is a **red flag** that warrants investigation. Common causes include: (1) the reliability coefficient was estimated from a different population than the study sample; (2) the reliability was based on a very small sample and is itself imprecise; or (3) range restriction artefacts interact with the correction. We recommend running a **sensitivity analysis** excluding studies where $|r_c| > 1$ and comparing results. If multiple studies produce impossible values, an artifact-distribution method (Hunter & Schmidt, 2004; see the `psychmeta` package) may be more appropriate than individual study-level correction, as it uses the *distribution* of reliability estimates rather than point values.

```{r, fig.width=7, fig.height=5}
# Pool corrected correlations on Fisher's z scale
ma_r_corr <- metafor::rma(yi = corrected$z_corrected,
                           sei = corrected$z_corrected_se,
                           method = "REML")

metafor::forest(ma_r_corr,
                slab = paste0(summary_r$author, " (", dat_validity$year, ")"),
                xlab = "Fisher's z (disattenuated correlation)",
                header = TRUE)
```

```{r}
pooled_r_corr <- tanh(ma_r_corr$beta[1])
pooled_r_corr_lo <- tanh(ma_r_corr$ci.lb)
pooled_r_corr_up <- tanh(ma_r_corr$ci.ub)

cat(sprintf("(b) Pooled disattenuated r = %.3f [%.3f, %.3f]\n",
            pooled_r_corr, pooled_r_corr_lo, pooled_r_corr_up))
```

## Using pooled reliability when study-specific data are unavailable

When study-specific reliability data are unavailable, the pooled reliability estimates from the internal consistency (Section 2) and test-retest (Section 3) meta-analyses can be used:

```{r, eval=FALSE}
# Use pooled alpha and ICC from earlier sections as proxies
es_disattenuate(
  r = 0.65,
  r_se = 0.08,
  reliability_x = pooled_alpha,  # from Section 2
  reliability_y = 0.85,          # from external published data
  n_sample = 150
)
```


# Responsiveness

Responsiveness is assessed by correlating **change scores** on the MHWS with change scores on a comparator instrument (e.g., CGI-Change, PGIC). Because change scores have lower reliability than single-occasion scores, **disattenuation requires change-score reliability**, not single-occasion reliability.

The **Lord (1963)** formula derives change-score reliability from single-occasion reliability and the pre-post correlation:

$$\text{rel}_{\text{change}} = \frac{\text{rel} - r_{\text{pre-post}}}{1 - r_{\text{pre-post}}}$$

## Effect size computation with change-score correction

```{r}
# Subset responsiveness studies
dat_resp <- df.psychom[df.psychom$outcome == "responsiveness", ]

# Step 1: Compute change-score reliability
change_rel <- reliability_change_score(
  reliability = dat_resp$single_occasion_reliability,
  r_pre_post  = dat_resp$r_pre_post
)

# Step 2: Get responsiveness correlations via convert_df
res_resp <- convert_df(dat_resp, measure = "r",
                       verbose = FALSE, split_adjusted = FALSE)
summary_resp <- summary(res_resp)

# Step 3: Disattenuate using CHANGE-SCORE reliability
corrected_resp <- es_disattenuate(
  r             = summary_resp$es,
  r_se          = summary_resp$se,
  reliability_x = change_rel$rel_change,
  reliability_y = dat_resp$reliability_comparator,
  n_sample      = dat_resp$n_sample
)
```

```{r, eval=FALSE}
data.frame(
  study        = dat_resp$study_id,
  r_observed   = round(summary_resp$es, 3),
  rel_single   = dat_resp$single_occasion_reliability,
  r_pre_post   = dat_resp$r_pre_post,
  rel_change   = round(change_rel$rel_change, 3),
  r_corrected  = round(corrected_resp$r_corrected, 3)
)
```

```{r, echo=FALSE}
resp_table <- data.frame(
  study        = dat_resp$study_id,
  r_observed   = round(summary_resp$es, 3),
  rel_single   = dat_resp$single_occasion_reliability,
  r_pre_post   = dat_resp$r_pre_post,
  rel_change   = round(change_rel$rel_change, 3),
  r_corrected  = round(corrected_resp$r_corrected, 3)
)
DT::datatable(resp_table,
              options = list(scrollX = TRUE, dom = 't', ordering = FALSE,
                             rownames = FALSE, pageLength = 50,
                             columnDefs = list(
                               list(width = '110px', targets = "_all"),
                               list(className = 'dt-center', targets = "_all"))))
```

<br>

Note that change-score reliability (`rel_change`) is always lower than single-occasion reliability (`rel_single`), which leads to a **larger disattenuation correction**.

## Meta-analysis

```{r, fig.width=7, fig.height=4}
# Pool observed change-score correlations
z_resp     <- atanh(summary_resp$es)
r_resp_bounded <- pmin(pmax(summary_resp$es, -0.9999), 0.9999)
z_resp_se  <- summary_resp$se / (1 - r_resp_bounded^2)

ma_resp_obs <- metafor::rma(yi = z_resp, sei = z_resp_se, method = "REML")

cat(sprintf("Pooled observed r = %.3f [%.3f, %.3f]\n",
            tanh(ma_resp_obs$beta[1]),
            tanh(ma_resp_obs$ci.lb),
            tanh(ma_resp_obs$ci.ub)))

# Pool disattenuated change-score correlations
ma_resp_corr <- metafor::rma(yi = corrected_resp$z_corrected,
                              sei = corrected_resp$z_corrected_se,
                              method = "REML")

cat(sprintf("Pooled disattenuated r = %.3f [%.3f, %.3f]\n",
            tanh(ma_resp_corr$beta[1]),
            tanh(ma_resp_corr$ci.lb),
            tanh(ma_resp_corr$ci.ub)))
```

## Sensitivity analysis for unknown pre-post correlations

When pre-post correlations are not reported, a sensitivity analysis using plausible values illustrates the impact on the corrected estimates:

```{r}
# Example: one study with r_observed = 0.50, rel_single = 0.85
r_pre_post_grid <- c(0.20, 0.40, 0.60, 0.80)

sensitivity <- data.frame(
  r_pre_post = r_pre_post_grid,
  rel_change = sapply(r_pre_post_grid, function(rpp) {
    reliability_change_score(reliability = 0.85, r_pre_post = rpp)$rel_change
  }),
  r_corrected = sapply(r_pre_post_grid, function(rpp) {
    rc <- reliability_change_score(reliability = 0.85, r_pre_post = rpp)$rel_change
    es_disattenuate(r = 0.50, r_se = 0.08, reliability_x = rc,
                    reliability_y = 0.80, n_sample = 100)$r_corrected
  })
)
sensitivity
```

As the pre-post correlation increases, change-score reliability decreases, and the disattenuation correction becomes larger. This highlights why reporting pre-post correlations in primary studies is essential.


# Floor and ceiling effects

Floor and ceiling effects can be assessed by meta-analysing the **proportion** of respondents scoring at the minimum or maximum of the scale. Three transformations are available:

- **`"raw"`** (default): raw proportion, SE = $\sqrt{p(1-p)/n}$. CIs bounded to [0, 1].
- **`"logit"`**: logit transform $\log(p/(1-p))$, SE = $\sqrt{1/(np(1-p))}$
- **`"freeman_tukey"`** (recommended): Freeman-Tukey double arcsine. Handles zero proportions naturally and stabilises variance.

## Effect size computation and pooling

```{r}
# Subset floor/ceiling studies
dat_prop <- df.psychom[df.psychom$outcome == "floor_ceiling", ]

# Freeman-Tukey transformation (recommended)
res_prop <- convert_df(dat_prop, measure = "prop", prop_to_es = "freeman_tukey",
                       verbose = FALSE, split_adjusted = FALSE)
summary_prop <- summary(res_prop)
```

```{r, fig.width=7, fig.height=3.5}
ma_prop <- metafor::rma(yi = es, sei = se, data = summary_prop,
                        method = "REML")

metafor::forest(ma_prop,
                slab = paste0(summary_prop$author, " (", dat_prop$year, ")"),
                xlab = "Freeman-Tukey transformed proportion",
                header = TRUE)
```

```{r}
# Approximate back-transformation (for the pooled estimate)
pooled_prop <- sin(ma_prop$beta[1])^2
cat(sprintf("Pooled proportion (approx.) = %.3f\n", pooled_prop))
```

**Note on the Freeman-Tukey back-transformation**: The `sin(x)^2` formula is an approximation. Recent methodological work (Schwarzer et al., 2019, *Research Synthesis Methods*) has shown that the Freeman-Tukey double arcsine back-transformation can distort pooled results when sample sizes are heterogeneous across studies. For proportions near 0 or 1 (as is typical for floor/ceiling effects), a generalised linear mixed model (GLMM) with logit link --- available via `metafor::rma.glmm()` --- may be preferable as it avoids the transformation entirely and directly models the binomial counts.


# Beyond metaConvert

Several COSMIN measurement properties require specialised methods not covered by `metaConvert`:

**Structural validity** requires meta-analytic structural equation modelling (OSMASEM; Jak & Cheung, 2020) to pool inter-item correlation matrices and fit confirmatory factor models. See the `metaSEM` and `OpenMx` packages.

**Diagnostic and screening accuracy** requires the bivariate generalised linear mixed model (Chu & Cole, 2006) for jointly modelling sensitivity and specificity. See the `mada` and `lme4` packages, and the `HSROC` model for summary ROC curves.

**Cross-cultural validity and measurement invariance** is primarily a descriptive synthesis (tabulating configural, metric, scalar invariance levels). When DIF statistics (log-odds ratios) are available, they can be pooled using metaConvert's existing OR pipeline via `convert_df(measure = "or")`.

**Interpretability (MIC)** can be partially handled: if you have MIC estimates and their SEs, enter them as user-provided effect sizes via `user_es_measure_crude`, `user_es_crude`, and `user_se_crude` columns, then use `convert_df()` to organise them alongside other effect sizes.

**Feasibility and acceptability** are synthesised narratively, without quantitative pooling.


# Summary

| COSMIN property | metaConvert step | Key function(s) |
|:---|:---|:---|
| Internal consistency | `convert_df(measure = "alpha")` | `es_from_cronbach_alpha()` |
| Test-retest reliability | `convert_df(measure = "icc")` | `es_from_icc()` |
| Measurement error | Standalone | `compute_sem()` + `compute_sdc()` |
| Criterion / construct validity | `convert_df(measure = "r")` then `es_disattenuate()` | `es_from_pearson_r()`, `es_from_spearman_rho()`, `es_disattenuate()` |
| Responsiveness | `reliability_change_score()` then `es_disattenuate()` | Full chain |
| Floor/ceiling effects | `convert_df(measure = "prop")` | `es_from_prop_single_group()` |

---

---
title: "Vignette 1. General guidance about metaConvert"
author: "Gosling CJ, Cortese S, Solmi M, Haza B, Vieta E, Delorme R, Fusar-Poli P, Radua J"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Vignette 1. General guidance about metaConvert}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{=html}
<style type="text/css">

*{
  font-family: "Gill Sans", sans-serif;
}

h1.title {
  font-weight: 700;
  font-size: 2.2rem;
  padding-top: 0rem;
  margin-top: 0rem;
  border-top: none;
}

#TOC {
  width: 100%;
}

h1{
  font-weight: 550;
  font-size: 1.9rem;
  border-top: 1px solid black;
  margin-top: 3rem;
  padding-top: 2rem;
}


p{
  line-height: 1.4rem;
}
</style>
```

```{r echo=FALSE, message=FALSE, warning=FALSE, results='hide'}
library(metaConvert)
library(DT)
```


# Step 1. Protocol stage

If you have not yet registered your protocol, you can benefit of our tools to select **a priori** the type of input data that could be extracted to estimate an effect size. 

- start by determining the effect size measure (SMD, OR, RR, etc.) you plan to estimate
- then, identify all types of input data that can be used to estimate this effect size measure
```{r, eval = FALSE}
see_input_data(measure = "or")
```

```{r, echo=FALSE}
DT::datatable(see_input_data(measure = "or", type_of_measure = "natural", extension="data.frame"), options = list(  
    scrollX = TRUE,
    dom = c('t'),
    scrollY = "300px", 
    pageLength = 500,
    ordering = FALSE,
    rownames = FALSE,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center', 
                                     targets = "_all"))))
```

<br><br>

- last, generate a data extraction sheet for this effect size measure
```{r, eval = FALSE}
data_extraction_sheet(measure = "or")
```

```{r, echo=FALSE}
DT::datatable(data_extraction_sheet(measure = "or", extension="data.frame"), options = list(  
    scrollX = TRUE,
    dom = c('t'),
   ordering = FALSE,
    rownames = FALSE,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center', 
                                     targets = "_all"))))
```


# Step 2. Dataset comparison

When data extraction has been performed in duplicate, our tools offer the possibility to flag the differences between the two datasets. For this example, we will use two datasets (df.compare1 and df.compare2) distributed with metaConvert. 
```{r, message=FALSE, fig.width= 11}
compare_df(
    df_extractor_1 = df.compare1,
    df_extractor_2 = df.compare2,
    output = "html")
```

In grey, values that are consistent between the two data extractors. In green/red, the values that differ.  

Only rows with differences between the two datasets are identified, and you can easily retrieve the row number by looking at the ID in the rowname column. If your dataset have rows in a different order (like in the example above), you can automatically reorder your datasets by indicating to the function the columns that store this information

```{r, message=FALSE, fig.width= 11}
compare_df(
    df_extractor_1 = df.compare1,
    df_extractor_2 = df.compare2,
    ordering_columns = c("author", "year"),
    output = "html")
```


# Step 3. Effect size computation

## Basic usage

To generate an effect size from a dataset that contains approriate column names and information, you simply need to :<ul>
<li> pass this dataset to the **convert_df()** function </li>
<li> indicate the effect size measure that should be estimated </li>
</ul>

For this example, we will generate effect sizes from the **df.short** dataset, and we will estimate Hedges' g.

```{r, message=FALSE}
res = convert_df(x = df.short, measure = "g")
```

```{r, eval= FALSE}
summary(res)
```

```{r echo=FALSE}
DT::datatable(summary(res), options = list(  
    scrollX = TRUE,
    dom = c('t'),
    ordering = FALSE,
    rownames = FALSE,
    scrollY = "300px", 
    pageLength = 100,
    columnDefs = list(
                  list(width = '130px',
                       targets = "_all"),
                  list(className = 'dt-center', 
                                     targets = "_all"))))
```

To know more about the information stored in each column, refer to documentation of the summary.metaConvert function, available in the R manual of this package.

## More advanced usage

A tutorial on a more advanced usage will be proposed in the companion paper of this tool; the link will be inserted as soon as the paper will be published.

For now, you can refer to the documentation of the convert_df function in the R manual of this package, in which all options are described.

## Risk difference and NNT

Since version 1.1.0 the package natively supports **risk difference** (`measure = "rd"`) and **number needed to treat** (`measure = "nnt"`) alongside ratio measures. Both carry a proper standard error and a confidence interval. When the RD CI crosses zero, the NNT CI is set to NA to preserve the Altman (1998) discontinuous-CI convention - the point estimate and SE remain available.

```{r, message=FALSE}
res_rd <- convert_df(df.haza, measure = "rd")
head(summary(res_rd)[, c("row_id", "es_crude", "se_crude",
                         "es_ci_lo_crude", "es_ci_up_crude", "info_used_crude")])
```

## Quality flags and missing-data guidance

`summary()` can surface two diagnostic layers. `flags = TRUE` adds a `flags_crude` / `flags_adjusted` column with validation errors (Tier 1: input integrity checks such as negative SDs or inverted CIs) and plausibility warnings (Tier 2: e.g. implausibly large SMD, RD outside `[-1, 1]`, ES/SE outliers across rows). `guidance = TRUE` adds an `es_guidance_crude` column that, for rows where the effect size could not be estimated, explains which input columns the user would need to add to unlock additional estimators.

```{r, message=FALSE}
res_flags <- convert_df(df.short, measure = "g")
s_flags <- summary(res_flags, flags = TRUE, guidance = TRUE)
head(s_flags[, c("row_id", "es_crude", "info_used_crude",
                 "flags_crude", "es_guidance_crude")])
```

## Correction for attenuation due to measurement error

`es_disattenuate()` corrects an observed correlation for unreliability in one or both measures (Spearman, 1904; Hunter & Schmidt, 2004) and propagates the uncertainty into the corrected SE and Fisher's z SE via the delta method. Apply it **after** `summary(convert_df(..., measure = "r"))`:

```{r, message=FALSE}
# Observed correlation with known scale reliabilities
es_disattenuate(r = 0.45, r_se = 0.08,
                reliability_x = 0.82, reliability_y = 0.78)
```

Set `reliability_y = 1.0` if only one side of the correlation needs correcting (e.g. a reliable criterion measure).

## Psychometric utilities: SEM and SDC

Standalone functions compute the **standard error of measurement** and the **smallest detectable change** for meta-analyses of measurement properties. They chain naturally:

```{r}
sem_res <- compute_sem(sd = 10, icc = 0.85, n_sample = 100)
sdc_res <- compute_sdc(sem = sem_res$sem, sem_se = sem_res$sem_se)
sem_res
sdc_res
```

# I do not like R

If you prefer having a graphical user interface (GUI) when performing data analysis, 
we are please to introduce you to our web-app that enables to perform ALL calculations of this
package using an interactive GUI <a href="https://metaconvert.org/" target="_blank">https://metaconvert.org/</a>

---

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_2x2.R
\name{es_from_2x2_prop}
\alias{es_from_2x2_prop}
\title{Convert the proportion of occurrence of a binary event in two independent groups into several effect size measures}
\usage{
es_from_2x2_prop(
  prop_cases_exp,
  prop_cases_nexp,
  n_exp,
  n_nexp,
  table_2x2_to_cor = "tetrachoric",
  reverse_prop
)
}
\arguments{
\item{prop_cases_exp}{proportion of cases/events in the exposed group (ranging from 0 to 1)}

\item{prop_cases_nexp}{proportion of cases/events in the non-exposed group (ranging from 0 to 1)}

\item{n_exp}{total number of participants in the exposed group}

\item{n_nexp}{total number of participants in the non exposed group}

\item{table_2x2_to_cor}{formula used to obtain a correlation coefficient from the contingency table (see details).}

\item{reverse_prop}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab OR + RR + NNT + RD\cr
\tab \cr
\code{converted effect size measure} \tab D + G + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 7. Contingency (2x2) table or proportions'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert the proportion of occurrence of a binary event in two independent groups into several effect size measures
}
\details{
This function uses the proportions and sample size to
recreate the 2x2 table, and
then relies on the calculations of the \code{\link{es_from_2x2_sum}()} function.

The formulas used is to obtain the 2x2 table are
\deqn{n\_cases\_exp = prop\_cases\_exp * n\_exp}
\deqn{n\_cases\_nexp = prop\_cases\_nexp * n\_nexp}
\deqn{n\_controls\_exp = (1 - prop\_cases\_exp) * n\_exp}
\deqn{n\_controls\_nexp = (1 - prop\_cases\_nexp) * n\_nexp}
}
\examples{
es_from_2x2_prop(prop_cases_exp = 0.80, prop_cases_nexp = 0.60, n_exp = 10, n_nexp = 20)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_OR.R
\name{es_from_or_ci}
\alias{es_from_or_ci}
\title{Convert an odds ratio value and its 95\% confidence interval to several effect size measures}
\usage{
es_from_or_ci(
  or,
  or_ci_lo,
  or_ci_up,
  logor,
  logor_ci_lo,
  logor_ci_up,
  baseline_risk,
  small_margin_prop,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  n_sample,
  max_asymmetry = 10,
  or_to_cor = "bonett",
  or_to_rr = "metaumbrella_cases",
  reverse_or
)
}
\arguments{
\item{or}{odds ratio value}

\item{or_ci_lo}{lower bound of the 95\% CI around the odds ratio value}

\item{or_ci_up}{upper bound of the 95\% CI around the odds ratio value}

\item{logor}{log odds ratio value}

\item{logor_ci_lo}{lower bound of the 95\% CI around the log odds ratio value}

\item{logor_ci_up}{upper bound of the 95\% CI around the log odds ratio value}

\item{baseline_risk}{proportion of cases in the non-exposed group (only required for the \code{or_to_rr = "grant"} argument).}

\item{small_margin_prop}{smallest margin proportion of the underlying 2x2 table}

\item{n_exp}{number of participants in the exposed group (only required for the \code{or_to_rr = "grant"}, and \code{or_to_rr = "metaumbrella_exp"} arguments)}

\item{n_nexp}{number of participants in the non-exposed group (only required for the \code{or_to_rr = "grant"}, and \code{or_to_rr = "metaumbrella_exp"} arguments)}

\item{n_cases}{number of cases/events}

\item{n_controls}{number of controls/no-event}

\item{n_sample}{total number of participants in the sample}

\item{max_asymmetry}{A percentage indicating the tolerance before detecting asymmetry in the 95\% CI bounds.}

\item{or_to_cor}{formula used to convert the \code{or} value into a correlation coefficient (see details).}

\item{or_to_rr}{formula used to convert the \code{or} value into a risk ratio (see details).}

\item{reverse_or}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab OR\cr
\tab \cr
\code{converted effect size measure} \tab RR + NNT\cr
\code{} \tab D + G + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 2. Odds Ratio'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an odds ratio value and its 95\% confidence interval to several effect size measures
}
\details{
This function computes the standard error of the (log) odds ratio
into a standard error (Section 6.5.2.2 in the Cochrane Handbook).
\deqn{logor\_se = \frac{\log{or\_ci\_up} - \log{or\_ci\_lo}}{2 * qnorm(.975)}}

Then, calculations of \code{\link{es_from_or_se}} are applied.
}
\examples{
es_or <- es_from_or_ci(
  or = 1, or_ci_lo = 0.5, or_ci_up = 2,
  n_cases = 42, n_controls = 38, baseline_risk = 0.08,
  or_to_rr = "grant"
)
}
\references{
Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANOVA_means.R
\name{es_from_means_se}
\alias{es_from_means_se}
\title{Convert means and standard errors of two independent groups several effect size measures}
\usage{
es_from_means_se(
  mean_exp,
  mean_se_exp,
  mean_nexp,
  mean_se_nexp,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  smd_denom = "pooled",
  reverse_means
)
}
\arguments{
\item{mean_exp}{mean of participants in the experimental/exposed group.}

\item{mean_se_exp}{standard error of participants in the experimental/exposed group.}

\item{mean_nexp}{mean of participants in the non-experimental/non-exposed group.}

\item{mean_se_nexp}{standard error of participants in the non-experimental/non-exposed group.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{smd_denom}{standardizer for the standardized mean difference. "pooled" (default) uses the pooled endpoint SD (Cohen's d / Hedges' g); "glass" (alias "control") uses the control (non-experimental) endpoint SD (Glass's delta); "glass_robust" (alias "control_robust") is Glass's delta with a heteroscedasticity-consistent sampling variance.}

\item{reverse_means}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 9. Means and dispersion (crude)'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert means and standard errors of two independent groups several effect size measures
}
\details{
This function converts the standard errors of two independent groups into standard deviations,
and then relies on the calculations of the \code{\link{es_from_means_sd}()} function.

\strong{To convert the standard errors into standard deviations}, the following formula is used.
\deqn{mean\_sd\_exp = mean\_se\_exp * \sqrt{n\_exp}}
\deqn{mean\_sd\_nexp = mean\_se\_nexp * \sqrt{n\_nexp}}
Then, calculations of the \code{\link{es_from_means_sd}()} are applied.

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_means_se(
  mean_exp = 42, mean_se_exp = 11,
  mean_nexp = 42, mean_se_nexp = 15,
  n_exp = 43, n_nexp = 34
)
}
\references{
Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_variability_from_dispersion.R
\name{es_variab_from_means_ci}
\alias{es_variab_from_means_ci}
\title{Title}
\usage{
es_variab_from_means_ci(
  mean_exp,
  mean_nexp,
  mean_ci_lo_exp,
  mean_ci_up_exp,
  mean_ci_lo_nexp,
  mean_ci_up_nexp,
  n_exp,
  n_nexp,
  reverse_means_variability
)
}
\arguments{
\item{mean_exp}{mean of participants in the experimental/exposed group.}

\item{mean_nexp}{mean of participants in the non-experimental/non-exposed group.}

\item{mean_ci_lo_exp}{lower bound of the 95\% CI of the mean of the experimental/exposed group}

\item{mean_ci_up_exp}{upper bound of the 95\% CI of the mean of the experimental/exposed group}

\item{mean_ci_lo_nexp}{lower bound of the 95\% CI of the mean of the non-experimental/non-exposed group.}

\item{mean_ci_up_nexp}{upper bound of the 95\% CI of the mean of the non-experimental/non-exposed group.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{reverse_means_variability}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates VR and CVR

\tabular{ll}{
\code{natural effect size measure} \tab VR + CVR\cr
\tab \cr
\code{converted effect size measure} \tab No conversion performed\cr
\tab \cr
\code{required input data} \tab See 'Section 23. User's input (crude)'\cr
\tab https://metaconvert.org/html/input.html\cr
\tab \cr
}
}
\description{
Title
}
\details{
This function converts the bounds of the 95\% CI of the means of two independent groups into standard errors.
\deqn{mean\_se\_exp = \frac{mean\_ci\_up\_exp - mean\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
\deqn{mean\_se\_nexp = \frac{mean\_ci\_up\_nexp - mean\_ci\_lo\_nexp}{2 * qt{(0.975, df = n\_nexp - 1)}}}

Then, calculations of the \code{\link{es_variab_from_means_se}} are applied.
}
\examples{
es_variab_from_means_ci(
  mean_exp = 42, mean_ci_lo_exp = 32, mean_ci_up_exp = 52,
  mean_nexp = 42, mean_ci_lo_nexp = 37, mean_ci_up_nexp = 47,
  n_exp = 43, n_nexp = 34
)
}
\references{
Senior, A. M., Viechtbauer, W., & Nakagawa, S. (2020). Revisiting and expanding the meta-analysis of variation: The log coefficient of variation ratio. Research Synthesis Methods, 11(4), 553-567. https://doi.org/10.1002/jrsm.1423
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_MD.R
\name{es_from_md_pval}
\alias{es_from_md_pval}
\title{Convert a mean difference between two independent groups and its p-value into several effect size measures}
\usage{
es_from_md_pval(
  md,
  md_pval,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_md
)
}
\arguments{
\item{md}{mean difference between two independent groups}

\item{md_pval}{p-value of the mean difference}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer").}

\item{reverse_md}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 10. Mean difference and dispersion (crude)'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a mean difference between two independent groups and its p-value into several effect size measures
}
\details{
This function converts the p-value of a mean difference
into a standard error (Cochrane Handbook section 6.5.2.3):
\deqn{t = qt(\frac{md\_pval}{2}, df = n\_exp + n\_nexp - 2)}
\deqn{md\_se = |\frac{md}{t}|}
Calculations of the \code{\link{es_from_md_se}} function are then used to estimate the Cohen's d and other effect size measures.
}
\examples{
es_from_md_pval(md = 4, md_pval = 0.024, n_exp = 20, n_nexp = 22)
}
\references{
Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_RR.R
\name{es_from_rr_ci}
\alias{es_from_rr_ci}
\title{Convert a risk ratio value and 95\% confidence interval to various effect size measures}
\usage{
es_from_rr_ci(
  rr,
  rr_ci_lo,
  rr_ci_up,
  logrr,
  logrr_ci_lo,
  logrr_ci_up,
  baseline_risk,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  rr_to_or = "metaumbrella",
  max_asymmetry = 10,
  reverse_rr
)
}
\arguments{
\item{rr}{risk ratio value}

\item{rr_ci_lo}{lower bound of the 95\% CI around the risk ratio value}

\item{rr_ci_up}{upper bound of the 95\% CI around the risk ratio value}

\item{logrr}{log risk ratio value}

\item{logrr_ci_lo}{lower bound of the 95\% CI around the log risk ratio value}

\item{logrr_ci_up}{upper bound of the 95\% CI around the log risk ratio value}

\item{baseline_risk}{proportion of cases in the non-exposed group (only required for the \code{rr_to_or = "grant_CI"} and \code{rr_to_or = "grant_2x2"} arguments).}

\item{n_exp}{number of participants in the exposed group (only required for the \code{rr_to_or = "grant_CI"}, \code{rr_to_or = "grant_2x2"} arguments).}

\item{n_nexp}{number of participants in the non-exposed group (only required for the \code{rr_to_or = "grant_CI"}, \code{rr_to_or = "grant_2x2"} arguments).}

\item{n_cases}{number of cases/events}

\item{n_controls}{number of controls/no-event}

\item{rr_to_or}{formula used to convert the \code{rr} value into an odds ratio (see details).}

\item{max_asymmetry}{A percentage indicating the tolerance before detecting asymmetry in the 95\% CI bounds.}

\item{reverse_rr}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab RR\cr
\tab \cr
\code{converted effect size measure} \tab OR + NNT\cr
\tab \cr
\code{required input data} \tab See 'Section 3. Risk Ratio'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a risk ratio value and 95\% confidence interval to various effect size measures
}
\details{
This function uses the 95\% CI of the (log) risk ratio to obtain the standard error (Section 6.5.2.2 in the Cochrane Handbook).
\deqn{logrr\_se = \frac{\log{rr\_ci\_up} - \log{rr\_ci\_lo}}{2 * qnorm(.975)}}

Then, calculations of the \code{\link{es_from_rr_se}()} are applied.
}
\examples{
es_from_rr_ci(
  rr = 1, rr_ci_lo = 0.5, rr_ci_up = 2,
  n_cases = 42, n_controls = 38, baseline_risk = 0.08
)
}
\references{
Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_means.R
\name{es_from_ancova_means_sd_pooled_adj}
\alias{es_from_ancova_means_sd_pooled_adj}
\title{Convert means and adjusted pooled standard deviation of two independent groups obtained from an ANCOVA model into several effect size measures}
\usage{
es_from_ancova_means_sd_pooled_adj(
  ancova_mean_exp,
  ancova_mean_nexp,
  ancova_mean_sd_pooled,
  cov_outcome_r,
  n_cov_ancova,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_means
)
}
\arguments{
\item{ancova_mean_exp}{adjusted mean of participants in the experimental/exposed group.}

\item{ancova_mean_nexp}{adjusted mean of participants in the non-experimental/non-exposed group.}

\item{ancova_mean_sd_pooled}{adjusted pooled standard deviation.}

\item{cov_outcome_r}{correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_means}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert means and adjusted pooled standard deviation of two independent groups obtained from an ANCOVA model into several effect size measures
}
\details{
This function converts the adjusted pooled standard deviations of two independent groups
into a crude pooled standard deviation.
and then relies on the calculations of the \code{\link{es_from_ancova_means_sd_pooled_crude}()} function.

\strong{To convert the adjusted pooled SD into a crude pooled SD} (table 12.3 in Cooper):
\deqn{mean\_sd\_pooled =  \frac{ancova\_mean\_sd\_pooled}{\sqrt{1 - cov\_outcome\_r^2}}}

Calculations of the \code{\link{es_from_ancova_means_sd_pooled_crude}()} are then applied.
}
\examples{
es_from_ancova_means_sd_pooled_adj(
  ancova_mean_exp = 98, ancova_mean_nexp = 87,
  ancova_mean_sd_pooled = 17, cov_outcome_r = 0.2,
  n_cov_ancova = 3, n_exp = 20, n_nexp = 20
)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_OR.R
\name{es_from_logreg_t}
\alias{es_from_logreg_t}
\title{Convert an odds ratio or risk ratio and a Wald t-statistic from a regression model into several effect size measures}
\usage{
es_from_logreg_t(
  or,
  logor,
  rr,
  logrr,
  logreg_t,
  baseline_risk,
  small_margin_prop,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  n_sample,
  or_to_rr = "metaumbrella_cases",
  or_to_cor = "bonett",
  rr_to_or = "metaumbrella",
  reverse_logreg_t
)
}
\arguments{
\item{or}{odds ratio value (from logistic regression)}

\item{logor}{log odds ratio value}

\item{rr}{risk ratio value (from log-binomial or modified Poisson regression)}

\item{logrr}{log risk ratio value}

\item{logreg_t}{a t-statistic (Wald statistic) from a logistic, log-binomial, or modified Poisson regression model}

\item{baseline_risk}{proportion of cases in the non-exposed group}

\item{small_margin_prop}{smallest margin proportion of cases/events in the underlying 2x2 table}

\item{n_exp}{number of participants in the exposed group}

\item{n_nexp}{number of participants in the non-exposed group}

\item{n_cases}{number of cases/events across exposed/non-exposed groups}

\item{n_controls}{number of controls/no-event across exposed/non-exposed groups}

\item{n_sample}{total number of participants in the sample}

\item{or_to_rr}{formula used to convert the \code{or} value into a risk ratio (see details).}

\item{or_to_cor}{formula used to convert the \code{or} value into a correlation coefficient (see details).}

\item{rr_to_or}{formula used to convert the \code{rr} value into an odds ratio (see details).}

\item{reverse_logreg_t}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.
\tabular{ll}{
\code{natural effect size measure} \tab OR + RR \cr
\tab \cr
\code{converted effect size measure} \tab OR inputs: D + G + R + Z (+ RR + NNT + RD); RR inputs: OR + NNT + RD \cr
}
}
\description{
Convert an odds ratio or risk ratio and a Wald t-statistic from a regression model into several effect size measures
}
\details{
This function derives the standard error of the log odds ratio (or log risk ratio) from a
Wald t-statistic reported in a regression model.

\strong{To estimate the standard error of the log OR (or log RR)}, the formulas used are:
\deqn{t = \frac{\beta}{SE(\beta)}}
\deqn{SE(\beta) = \frac{|\beta|}{|t|}}
where \eqn{\beta} is \eqn{\log(OR)} or \eqn{\log(RR)} depending on the model.

Then, if an OR (or logOR) is entered, calculations of \code{\link{es_from_or_se}()} are applied.
If a RR (or logRR) is entered, calculations of \code{\link{es_from_rr_se}()} are applied.

Note that the standardized-mean-difference and correlation conversions (D, G, R, Z) are
produced for \strong{OR inputs only}. RR inputs are treated as a ratio measure and yield
RR + OR + NNT + RD: RR is not converted to a standardized mean difference or correlation,
because that would require going through the OR and the baseline risk (see
\code{\link{es_from_rr_se}}). To obtain a SMD or correlation from an RR, convert it to an
OR first (supplying the baseline risk) and then use the OR path.
}
\examples{
es_or <- es_from_logreg_t(
  or = 2.12, logreg_t = 3.21,
  n_cases = 50, n_controls = 150
)

es_rr <- es_from_logreg_t(
  rr = 1.5, logreg_t = 2.8,
  n_exp = 100, n_nexp = 100
)
}
\references{
Sanchez-Meca, J., Marin-Martinez, F., & Chacon-Moscoso, S. (2003). Effect-size indices for
dichotomized outcomes in meta-analysis. \emph{Psychological Methods}, 8(4), 448--467.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_pool_arms.R
\name{pool_arms}
\alias{pool_arms}
\title{Pool or split arms in multi-arm trials}
\usage{
pool_arms(x, study_id, pool_side, method = c("pool", "split"), verbose = TRUE)
}
\arguments{
\item{x}{a data.frame in metaConvert wide format (as used by \code{\link{convert_df}}).}

\item{study_id}{character string: column name identifying which rows belong
to the same multi-arm trial.}

\item{pool_side}{character string: column name whose values indicate which
side to pool (\code{"exp"} or \code{"nexp"}) or \code{NA} for single-comparison rows
that should be left unchanged.}

\item{method}{either \code{"pool"} (Cochrane-recommended pooling of summary
statistics) or \code{"split"} (divide the shared arm's sample size).
See details.}

\item{verbose}{logical: whether to print warnings about columns set to NA
and non-identical shared-side values. Default is \code{TRUE}.}
}
\value{
A data.frame suitable for \code{\link{convert_df}}. The
\code{pool_side} column is removed from the output.
}
\description{
Pool or split arms in multi-arm trials
}
\details{
Multi-arm trials create dependency when multiple experimental arms share a
common control group (or vice versa). This function resolves the dependency
before effect size computation with \code{\link{convert_df}}.

With \code{method = "pool"} (recommended), the arms to be pooled are combined
into a single composite arm using Cochrane Handbook formulas (Table 23.3.a):
\itemize{
\item Sample sizes and event counts: summed
\item Means: weighted average by sample size
\item SDs: Cochrane pooling formula
\item SEs: recomputed from pooled SD and pooled n
\item CIs, medians, paired statistics: set to NA (cannot be pooled)
}
Returns one row per multi-arm group.

With \code{method = "split"}, the shared arm's sample size (and event counts)
is divided equally across comparisons. Each comparison retains its own arm
data unchanged, so the output has the same number of rows as the input but
with adjusted sample sizes on the shared side. Simpler but slightly
conservative.
}
\examples{
# Two experimental arms vs one shared control
dat <- data.frame(
  study_id = c("Study1", "Study1", "Study2"),
  pool_side = c("exp", "exp", NA),
  n_exp = c(30, 25, 40),
  n_nexp = c(28, 28, 35),
  mean_exp = c(12.5, 14.2, 10.0),
  mean_sd_exp = c(3.1, 2.8, 4.0),
  mean_nexp = c(10.1, 10.1, 9.5),
  mean_sd_nexp = c(3.0, 3.0, 3.8)
)

# Pool experimental arms into one composite arm
pooled <- pool_arms(dat, study_id = "study_id",
                    pool_side = "pool_side", method = "pool")

# Or split the shared control's sample size
split_dat <- pool_arms(dat, study_id = "study_id",
                       pool_side = "pool_side", method = "split")

}
\references{
Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA
(editors). Cochrane Handbook for Systematic Reviews of Interventions.
Chapter 23, Table 23.3.a.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PAIRED_STATISTICS.R
\name{es_from_paired_t_pval}
\alias{es_from_paired_t_pval}
\title{Convert two paired t-test p-value obtained from two independent groups into several effect size measures}
\usage{
es_from_paired_t_pval(
  paired_t_pval_exp,
  paired_t_pval_nexp,
  n_exp,
  n_nexp,
  r_pre_post_exp,
  r_pre_post_nexp,
  smd_to_cor = "viechtbauer",
  pre_post_to_smd = "cooper",
  reverse_paired_t_pval
)
}
\arguments{
\item{paired_t_pval_exp}{P-value of the paired t-test value of the experimental/exposed group.}

\item{paired_t_pval_nexp}{P-value of the paired t-test value of the non-experimental/non-exposed group.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{r_pre_post_exp}{pre-post correlation in the experimental/exposed group}

\item{r_pre_post_nexp}{pre-post correlation in the non-experimental/non-exposed group}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{pre_post_to_smd}{formula used to convert the paired t-test value into a SMD (see details).}

\item{reverse_paired_t_pval}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 16. Paired: Paired F- or t-test'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert two paired t-test p-value obtained from two independent groups into several effect size measures
}
\details{
This function converts the p-values of two paired t-test obtained from two independent groups value into a Cohen's d (D) and Hedges' g (G) (table 12.2 in Cooper).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate the Cohen's d,} the following formulas are used (Cooper et al., 2019):
This function converts a Student's t-test value into a Cohen's d (table 12.2 in Cooper).
\deqn{paired\_t\_exp = qt(\frac{paired\_t\_pval\_exp}{2}, df = n\_exp - 1) * \sqrt{\frac{2 * (1 - r\_pre\_post\_exp)}{n\_exp}}}
\deqn{paired\_t\_nexp = qt(\frac{paired\_t\_pval\_nexp}{2}, df = n\_nexp - 1) * \sqrt{\frac{2 * (1 - r\_pre\_post\_nexp)}{n\_nexp}}}

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_paired_t_pval(paired_t_pval_exp = 0.4, paired_t_pval_nexp = 0.01, n_exp = 19, n_nexp = 22)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/psychometric_utils.R
\name{reliability_change_score}
\alias{reliability_change_score}
\title{Compute change-score reliability}
\usage{
reliability_change_score(reliability, r_pre_post)
}
\arguments{
\item{reliability}{single-occasion reliability coefficient (e.g., Cronbach's alpha or ICC)}

\item{r_pre_post}{pre-post correlation of observed scores}
}
\value{
A data.frame containing the change-score reliability (\code{rel_change}).
}
\description{
Compute change-score reliability
}
\details{
Computes the reliability of a change (difference) score from a single-occasion
reliability coefficient and the pre-post correlation of observed scores,
assuming equal variances AND equal reliabilities at pre and post
(Lord, 1963; Cronbach & Furby, 1970):

\deqn{rel_{change} = \frac{rel - r_{12}}{1 - r_{12}}}

This is the equal-variance/equal-reliability special case of the general
Lord (1963) formula; when the pre and post variances (or reliabilities)
differ materially, the general formula should be used instead.

This is useful for disattenuating change-score correlations, where the
reliability of the change score is needed rather than the single-occasion reliability.

A negative result (which occurs when \code{r_pre_post} exceeds
\code{reliability}) is population-impossible under classical test theory
and signals inconsistent inputs; it is returned as computed but triggers a
warning, since a negative value fed to \code{\link{es_disattenuate}} as a
reliability would produce \code{NaN} (square root of a negative number).
}
\examples{
reliability_change_score(reliability = 0.85, r_pre_post = 0.60)
}
\references{
Lord, F. M. (1963). Elementary models for measuring change.
In C. W. Harris (Ed.), Problems in measuring change. University of Wisconsin Press.

Cronbach, L. J., & Furby, L. (1970). How we should measure "change" - or should we?
Psychological Bulletin, 74(1), 68-80.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_OR.R
\name{es_from_or_pval}
\alias{es_from_or_pval}
\title{Convert an odds ratio value and its standard error to several effect size measures}
\usage{
es_from_or_pval(
  or,
  logor,
  or_pval,
  baseline_risk,
  small_margin_prop,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  n_sample,
  or_to_rr = "metaumbrella_cases",
  or_to_cor = "bonett",
  reverse_or_pval
)
}
\arguments{
\item{or}{odds ratio value}

\item{logor}{log odds ratio value}

\item{or_pval}{p-value of the (log) odds ratio}

\item{baseline_risk}{proportion of cases in the non-exposed group (only required for the \code{or_to_rr = "grant"} argument).}

\item{small_margin_prop}{smallest margin proportion of the underlying 2x2 table}

\item{n_exp}{number of participants in the exposed group (only required for the \code{or_to_rr = "grant"}, and \code{or_to_rr = "metaumbrella_exp"} arguments)}

\item{n_nexp}{number of participants in the non-exposed group (only required for the \code{or_to_rr = "grant"}, and \code{or_to_rr = "metaumbrella_exp"} arguments)}

\item{n_cases}{number of cases/events}

\item{n_controls}{number of controls/no-event}

\item{n_sample}{total number of participants in the sample}

\item{or_to_rr}{formula used to convert the \code{or} value into a risk ratio (see details).}

\item{or_to_cor}{formula used to convert the \code{or} value into a correlation coefficient (see details).}

\item{reverse_or_pval}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab OR\cr
\tab \cr
\code{converted effect size measure} \tab RR + NNT\cr
\code{} \tab D + G + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 2. Odds Ratio'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an odds ratio value and its standard error to several effect size measures
}
\details{
This function computes the standard error of the (log) odds ratio into
from a p-value (Section 6.3.2 in the Cochrane Handbook).
\deqn{logor\_z = qnorm(or_pval/2, lower.tail=FALSE)}
\deqn{logor\_se = |\frac{\log(or)}{logor\_z}|}

Then, calculations of \code{\link{es_from_or_se}()} are applied.
}
\examples{
es_or <- es_from_or_pval(
  or = 3.51, or_pval = 0.001,
  n_cases = 12, n_controls = 68
)
}
\references{
Higgins, J. P., Thomas, J., Chandler, J., Cumpston, M., Li, T., Page, M. J., & Welch, V. A. (Eds.). (2019). Cochrane handbook for systematic reviews of interventions. John Wiley & Sons.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ETASQ.R
\name{es_from_etasq_adj}
\alias{es_from_etasq_adj}
\title{Convert an adjusted eta-squared value (i.e., from an ANCOVA) to various effect size measures}
\usage{
es_from_etasq_adj(
  etasq_adj,
  n_exp,
  n_nexp,
  n_cov_ancova,
  cov_outcome_r,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_etasq
)
}
\arguments{
\item{etasq_adj}{an adjusted eta-squared value obtained from an ANCOVA model. This
must be the \strong{partial} eta-squared of the group effect,
\eqn{\eta_p^2 = SS_{group} / (SS_{group} + SS_{error})} — the quantity that
most software (SPSS, \code{car}, \code{effectsize}) prints for an ANCOVA term.
Do \strong{not} supply the classical eta-squared \eqn{SS_{group} / SS_{total}}: in an
ANCOVA the total sum of squares also contains the covariate variance, so the
classical value is smaller than the partial one and would be inverted into too
small an F, biasing the effect size toward zero. (In a covariate-free one-way
ANOVA the two definitions coincide; see \code{\link{es_from_etasq}()}.)}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{cov_outcome_r}{correlation between the outcome and covariate (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_etasq}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 18. Adjusted: ANCOVA statistics, eta-squared'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an adjusted eta-squared value (i.e., from an ANCOVA) to various effect size measures
}
\details{
This function converts the adjusted (partial) eta-squared of a binary predictor
(ANCOVA model) into the ANCOVA F-statistic it implies, and then relies on the
calculations of the \code{\link{es_from_ancova_f}()} function. The returned
Cohen's d (D) and Hedges' g (G) are therefore expressed on the
\strong{marginal (unadjusted) SD scale} — consistent with the other
\code{es_from_ancova_*} functions — not on the residual (covariate-adjusted)
SD scale on which a partial eta-squared is natively defined.
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To convert the adjusted eta-squared into an ANCOVA F-statistic},
the partial-eta-squared identity \eqn{\eta_p^2 = F / (F + df)} is inverted
(this is why a \emph{partial} eta-squared, not a classical one, is required):
\deqn{df = n\_exp + n\_nexp - 2 - n\_cov\_ancova}
\deqn{ancova\_f = \frac{etasq\_adj * df}{1 - etasq\_adj}}

\strong{To estimate a Cohen's d} the formula used is (table 12.3 in Cooper):
\deqn{cohen\_d = \sqrt{ancova\_f * \frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp}} * \sqrt{1 - cov\_out\_cor^2}}

Note that the back-transformation to the marginal SD scale requires
\code{cov_outcome_r}; when it is missing, the effect size estimates are
returned as NA (the marginal scale is not identified from the adjusted
eta-squared alone).

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
}
\note{
Two cautions apply. (1) \code{etasq_adj} must be the \emph{partial} eta-squared (see
the parameter description); a classical eta-squared biases the effect size
toward zero. (2) The sampling variance is rebuilt from Cooper's eq. 12.26, which
assumes a covariate balanced across groups and treats \code{cov_outcome_r} as
known; in covariate-imbalanced designs the standard error is a lower bound
(anti-conservative), while the point estimate remains unbiased. See Lai and
Kelley (2012).
}
\examples{
es_from_etasq_adj(etasq_adj = 0.28, n_cov_ancova = 3, cov_outcome_r = 0.2, n_exp = 20, n_nexp = 22)
}
\references{
Cooper, H., Hedges, L. V., & Valentine, J. C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Lai, K., & Kelley, K. (2012). Accuracy in parameter estimation for ANCOVA and ANOVA contrasts: Sample size planning via narrow confidence intervals. British Journal of Mathematical and Statistical Psychology, 65(2), 350-370.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions_summary.R
\name{as.data.frame.metaConvert}
\alias{as.data.frame.metaConvert}
\title{Convert a \dQuote{metaConvert} object to a dataframe}
\usage{
\method{as.data.frame}{metaConvert}(x, ...)
}
\arguments{
\item{x}{an object of class \dQuote{metaConvert}}

\item{...}{other arguments passed to \code{\link{summary.metaConvert}}}
}
\value{
A dataframe containing the effect size results, diagnostics, and all input data columns.
}
\description{
Convert a \dQuote{metaConvert} object to a dataframe
}
\details{
Convenience wrapper around \code{summary.metaConvert(x, ...)}.
}
\examples{
### get the full output as a dataframe
as.data.frame(convert_df(df.haza, measure = "g"))
}
\seealso{
\code{\link{summary.metaConvert}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{df.psychom}
\alias{df.psychom}
\title{Simulated dataset for a COSMIN-based systematic review of PROMs}
\format{
An object of class \code{data.frame} with 30 rows and 25 columns.
}
\usage{
df.psychom
}
\description{
Simulated dataset of a systematic review evaluating measurement properties
of a fictional patient-reported outcome measure (the Mental Health Wellbeing
Scale, MHWS) following the COSMIN framework. Designed to demonstrate the
psychometric functions of the metaConvert package.
}
\details{
The dataset contains studies reporting:
\itemize{
  \item Internal consistency (Cronbach's alpha)
  \item Test-retest reliability (ICC)
  \item Criterion and construct validity (Pearson and Spearman correlations)
  \item Responsiveness (change-score correlations)
  \item Floor/ceiling effects (proportions)
}

Additional columns support standalone psychometric utility functions
(SEM, SDC, disattenuation, change-score reliability).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_spearman_COR.R
\name{es_from_spearman_rho}
\alias{es_from_spearman_rho}
\title{Convert a Spearman's rank correlation coefficient to several effect size measures}
\usage{
es_from_spearman_rho(
  spearman_r,
  n_sample,
  n_exp,
  n_nexp,
  cor_to_smd = "viechtbauer",
  sd_iv,
  unit_increase_iv,
  unit_type = "raw_scale",
  reverse_spearman_r
)
}
\arguments{
\item{spearman_r}{a Spearman's rank correlation coefficient value}

\item{n_sample}{the total number of participants}

\item{n_exp}{number of the experimental/exposed group}

\item{n_nexp}{number of the non-experimental/non-exposed group}

\item{cor_to_smd}{formula used to convert the derived Pearson's r value into a SMD.}

\item{sd_iv}{the standard deviation of the independent variable}

\item{unit_increase_iv}{a value of the independent variable that will be used to estimate the Cohen's d (see details).}

\item{unit_type}{the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "value"}

\item{reverse_spearman_r}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab R + Z\cr
\tab \cr
\code{converted effect size measure} \tab D + G + OR\cr
\tab \cr
\code{required input data} \tab See 'Section 4. Pearson's r or Fisher's z'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a Spearman's rank correlation coefficient to several effect size measures
}
\details{
This function first converts a Spearman's rank correlation coefficient to an approximate Pearson's
correlation coefficient using the formula proposed by Rupinski & Dunlap (1996):
\deqn{r_p = 2 \sin(\pi / 6 \times r_s)}

The standard error of the resulting Pearson's r is derived via the delta method.
The sampling variance of the Spearman rho under bivariate normality carries an
extra \eqn{(1 + r_s^2 / 2)} factor relative to the Pearson variance
(Bonett & Wright, 2000); omitting it would understate the SE, increasingly so
for strong correlations:
\deqn{r_p\_se = \sqrt{\left(\frac{\pi}{3} \cos\left(\frac{\pi}{6} r_s\right)\right)^2 \times \left(1 + \frac{r_s^2}{2}\right)\frac{(1 - r_s^2)^2}{n - 1}}}

The converted Pearson's r is then further converted to a Fisher's z, Cohen's d, Hedges' g, and
odds ratio using the same formulas as \code{\link{es_from_pearson_r}()}. This
delta-method SE is propagated to the Fisher's z and (for the
\code{cor_to_smd = "viechtbauer"} and \code{cor_to_smd = "cooper"} paths,
whose SMD standard errors are proportional to the input r SE) to the Cohen's d,
Hedges' g and odds ratio, so all measures share the corrected r variance.
The \code{cor_to_smd = "mathur"} SMD variance contains no r SE term, so its
d/g/OR standard errors are left as computed by \code{\link{es_from_pearson_r}()}.
}
\examples{
es_from_spearman_rho(
  spearman_r = .55, n_sample = 100
)
}
\references{
Rupinski, M. T., & Dunlap, W. P. (1996). Approximating Pearson product-moment correlations from
Kendall's tau and Spearman's rho. Educational and Psychological Measurement, 56(3), 419-429.

Bonett, D. G., & Wright, T. A. (2000). Sample size requirements for estimating Pearson, Kendall
and Spearman correlations. Psychometrika, 65(1), 23-28.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_statistics.R
\name{es_from_ancova_t}
\alias{es_from_ancova_t}
\title{Convert a t-statistic obtained from an ANCOVA model into several effect size measures.}
\usage{
es_from_ancova_t(
  ancova_t,
  cov_outcome_r,
  n_cov_ancova,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_t
)
}
\arguments{
\item{ancova_t}{a t-statistic from an ANCOVA (binary predictor)}

\item{cov_outcome_r}{correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_t}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 18. Adjusted: ANCOVA statistics, eta-squared'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a t-statistic obtained from an ANCOVA model into several effect size measures.
}
\details{
This function first computes an "adjusted" Cohen's d (D), and
Hedges' g (G) from the t-value of an ANCOVA (binary predictor).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate a Cohen's d} the formula used is (table 12.3 in Cooper):
\deqn{cohen\_d =  ancova\_t* \sqrt{\frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp}}\sqrt{1 - cov\_out\_cor^2}}

\strong{To estimate other effect size measures},
Calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
}
\note{
The Cohen's d point estimate is recovered from the reported ANCOVA t (which
already embeds the covariate adjustment), but its sampling variance is rebuilt
from Cooper's eq. 12.26, which assumes a covariate balanced across groups and
treats \code{cov_outcome_r} as known. In observational or otherwise
covariate-imbalanced designs the standard error is a lower bound
(anti-conservative), while the point estimate remains unbiased. See Lai and
Kelley (2012).
}
\examples{
es_from_ancova_t(ancova_t = 2, cov_outcome_r = 0.2, n_cov_ancova = 3, n_exp = 20, n_nexp = 20)
}
\references{
Cooper, H., Hedges, L. V., & Valentine, J. C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Lai, K., & Kelley, K. (2012). Accuracy in parameter estimation for ANCOVA and ANOVA contrasts: Sample size planning via narrow confidence intervals. British Journal of Mathematical and Statistical Psychology, 65(2), 350-370.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_aggregate_df.R
\name{aggregate_df}
\alias{aggregate_df}
\title{Aggregate a dataframe containing dependent effect sizes}
\usage{
aggregate_df(
  x,
  dependence = "outcomes",
  cor_unit = 0.8,
  agg_fact,
  es = "es",
  se = "se",
  col_mean = NA,
  col_weighted_mean = NA,
  weights = NA,
  col_sum = NA,
  col_min = NA,
  col_max = NA,
  col_fact = NA,
  na.rm = TRUE
)
}
\arguments{
\item{x}{a dataframe that should be aggregated (must contain effect size values and standard errors).}

\item{dependence}{The type of dependence in your dataframe (can be "outcomes", "times", or "subgroups"). See details.}

\item{cor_unit}{The correlation between effect sizes coming from the same clustering unit. Used directly when \code{dependence = "outcomes"}. When \code{dependence = "times"} the within-cluster correlation is taken from a per-row \code{cor_unit} column if present (which must be constant within each cluster), and this argument is only used as a fallback when that column is absent; \code{dependence = "times"} additionally requires a \code{time_agg} column giving the time-point of each effect size. Ignored when \code{dependence = "subgroups"}.}

\item{agg_fact}{A character string identifying the column name that contains the clustering units (all rows with the same \code{agg_fact} value will be aggregated together).}

\item{es}{A character string identifying the column name containing the effect size values. Default is "es".}

\item{se}{A character string identifying the column name containing the standard errors of the effect size. Default is "se".}

\item{col_mean}{a vector of character strings identifying the column names for which the dependent values are summarized by taking their mean.}

\item{col_weighted_mean}{a vector of character strings identifying the column names for which the dependent values are summarized by taking their weighted mean.}

\item{weights}{The weights that will be used to estimated the weighted means.}

\item{col_sum}{a vector of character strings identifying the column names for which the dependent values are summarized by taking their sum.}

\item{col_min}{a vector of character strings identifying the column names for which the dependent values are summarized by taking their minimum.}

\item{col_max}{a vector of character strings identifying the column names for which the dependent values are summarized by taking their maximum.}

\item{col_fact}{a vector of character strings identifying the column names that are factors (different values will be separated by a "/" character).}

\item{na.rm}{a logical vector indicating whether missing values should be ignored in the calculations for the \code{col_mean}, \code{col_weighted_mean}, \code{col_sum}, \code{col_min} and \code{col_max} arguments.}
}
\value{
The object returned by the \code{aggregate_df} contains, is a dataframe containing at the very least,
the aggregating factor, and the aggregated effect size values and standard errors. All columns indicated in the \code{col_*} arguments
will also be included in this dataframe.
\tabular{ll}{
\code{row_index} \tab the row number in the original dataset.\cr
\tab \cr
\code{es} \tab the aggregated effect size value.\cr
\tab \cr
\code{se} \tab the standard error of the aggregated effect size.\cr
\tab \cr
\code{...} \tab any columns indicated in the \code{col_*} arguments.\cr
\tab \cr
}
}
\description{
Aggregate a dataframe containing dependent effect sizes
}
\details{
\enumerate{
\item In the \code{dependence} argument, you should indicate "outcomes" if the dependence within the same clustering unit (e.g., study) is due to the presence of multiple effect sizes produced from the same participants at the same time-point (e.g., multiple outcome measures)
\item In the \code{dependence} argument, you should indicate "times" if the dependence within the same clustering unit (e.g., study) is due to the presence of multiple effect sizes produced from the same participants at the different time-points (e.g., an RCT with several follow-up waves). This option requires a \code{time_agg} column (the time-point of each effect size) and a \code{cor_unit} column giving the within-cluster correlation (constant within each cluster; falls back to the \code{cor_unit} argument if the column is absent).
\item In the \code{dependence} argument, you should indicate "subgroups" if the dependence within the same clustering unit (e.g., study) is due to the presence of multiple effect sizes produced by independent subgroups (e.g., one effect size for boys, and one for girls).
}

If you are working with ratio measures, make sure that the information on
the effect size estimates (i.e., the column passed to the es argument of the function)
is presented on the log scale.
}
\examples{
res <- summary(convert_df(df.haza, measure = "d"))
aggregate_df(res, dependence = "outcomes", cor_unit = 0.8,
             agg_fact = "study_id", es = "es_crude", se = "se_crude",
             col_fact = c("outcome", "type_publication"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_2x2.R
\name{es_from_2x2_sum}
\alias{es_from_2x2_sum}
\title{Convert a table with the number of cases and row marginal sums into several effect size measures}
\usage{
es_from_2x2_sum(
  n_cases_exp,
  n_exp,
  n_cases_nexp,
  n_nexp,
  table_2x2_to_cor = "tetrachoric",
  reverse_2x2
)
}
\arguments{
\item{n_cases_exp}{number of cases/events in the exposed group}

\item{n_exp}{total number of participants in the exposed group}

\item{n_cases_nexp}{number of cases/events in the non exposed group}

\item{n_nexp}{total number of participants in the non exposed group}

\item{table_2x2_to_cor}{formula used to obtain a correlation coefficient from the contingency table (see details).}

\item{reverse_2x2}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab OR + RR + NNT + RD\cr
\tab \cr
\code{converted effect size measure} \tab D + G + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 7. Contingency (2x2) table or proportions'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a table with the number of cases and row marginal sums into several effect size measures
}
\details{
This function uses the number of cases in both the exposed
and non-exposed groups and the total number of participants exposed and non-exposed
to recreate a 2x2 table.
Then relies on the calculations of the \code{\link{es_from_2x2}} function.
\deqn{n\_controls\_exp = n\_exp - n\_cases\_exp}
\deqn{n\_controls\_nexp = n\_nexp - n\_cases\_nexp}
}
\examples{
es_from_2x2_sum(n_cases_exp = 10, n_exp = 40, n_cases_nexp = 25, n_nexp = 47)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_RD.R
\name{es_from_rd_se}
\alias{es_from_rd_se}
\title{Convert a risk difference value and its standard error into several effect size measures}
\usage{
es_from_rd_se(
  rd,
  rd_se,
  baseline_risk,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  n_sample,
  reverse_rd
)
}
\arguments{
\item{rd}{risk difference value (control risk minus treatment risk)}

\item{rd_se}{standard error of the risk difference}

\item{baseline_risk}{proportion of cases in the non-exposed/control group.
Required for converting RD to OR and RR.}

\item{n_exp}{number of participants in the exposed/treatment group}

\item{n_nexp}{number of participants in the non-exposed/control group}

\item{n_cases}{number of cases/events across both groups}

\item{n_controls}{number of controls/no-event across both groups}

\item{n_sample}{total number of participants in the sample}

\item{reverse_rd}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab RD\cr
\tab \cr
\code{converted effect size measure} \tab OR + RR + NNT\cr
\tab \cr
\code{required input data} \tab rd + rd_se\cr
\tab \cr
}
}
\description{
Convert a risk difference value and its standard error into several effect size measures
}
\details{
This function converts a risk difference (RD) and its standard error into an odds ratio (OR),
risk ratio (RR), and number needed to treat (NNT).

A risk difference is a ratio-family (binary-outcome) effect size and is \strong{not} converted
to a standardized mean difference (D/G) or a correlation (R/Z). Such a conversion would
require reconstructing an odds ratio from an assumed baseline risk, which is not identified
by the risk difference alone; and because the baseline risk would be treated as a fixed
known constant, the resulting standardized SEs would be anti-conservative. In metaConvert
the standardized families are reached only through the odds ratio or a raw 2x2 table (the
Cox transform \eqn{d = \log(or)\sqrt{3}/\pi}), neither of which needs an assumed baseline
risk. To obtain a D/G/R/Z from a risk difference, first convert it to an OR (supplying the
baseline risk) and then use \code{\link{es_from_or_se}}.

\strong{NNT is always computed from RD:}
\deqn{nnt = \frac{1}{rd}}
\deqn{nnt\_se = \frac{rd\_se}{rd^2}}

\strong{When \code{baseline_risk} is available, the following conversions are performed.}

\strong{To estimate the odds ratio:}
Let \eqn{pt = baseline\_risk - rd} (treatment group risk). Then:
\deqn{or = \frac{pt \times (1 - baseline\_risk)}{baseline\_risk \times (1 - pt)}}
\deqn{logor\_se = \frac{rd\_se}{pt \times (1 - pt)}}
where the SE is derived via the delta method from \eqn{\frac{d(\log OR)}{d(RD)} = \frac{-1}{pt \times (1 - pt)}}.

\strong{To estimate the risk ratio:}
\deqn{rr = 1 - \frac{rd}{baseline\_risk}}
\deqn{logrr\_se = \frac{rd\_se}{|baseline\_risk - rd|}}
where the SE is derived via the delta method from \eqn{\frac{d(\log RR)}{d(RD)} = \frac{-1}{baseline\_risk - rd}}.

Note that the conversions to OR and RR assume the baseline risk is a fixed constant.
}
\examples{
es_from_rd_se(rd = 0.15, rd_se = 0.05,
              baseline_risk = 0.30, n_exp = 100, n_nexp = 100)
}
\references{
Deeks, J.J. (2002). Issues in the selection of a summary statistic for meta-analysis of clinical trials with binary outcomes. Statistics in Medicine, 21(11), 1575-1600.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ALPHA.R
\name{es_from_cronbach_alpha}
\alias{es_from_cronbach_alpha}
\title{Convert Cronbach's alpha into an effect size measure}
\usage{
es_from_cronbach_alpha(
  cronbach_alpha,
  n_sample,
  n_items,
  alpha_to_es = "bonett"
)
}
\arguments{
\item{cronbach_alpha}{Cronbach's alpha reliability coefficient}

\item{n_sample}{the total number of participants that completed the scale}

\item{n_items}{number of items in the scale}

\item{alpha_to_es}{method used to compute the effect size from Cronbach's alpha.
Must be either \code{"bonett"} or \code{"raw"}.}
}
\value{
This function estimates the standard error of the Cronbach's alpha.

\tabular{ll}{
\code{natural effect size measure} \tab alpha\cr
\tab \cr
\code{converted effect size measure} \tab N/A\cr
\tab \cr
\code{required input data} \tab cronbach_alpha + n_sample + n_items\cr
\tab \cr
}
}
\description{
Convert Cronbach's alpha into an effect size measure
}
\details{
This function computes an effect size from a Cronbach's alpha.
\enumerate{
\item When \code{alpha_to_es = "bonett"} (default), the Bonett (2002) transformation
is applied:
\deqn{T(\alpha) = \ln(1 - \alpha)}
\deqn{T\_se = \sqrt{\frac{2k}{(k - 1)(n - 2)}}}
\item When \code{alpha_to_es = "raw"}, the raw alpha is used. Its standard error is
the delta-method back-transform of the Bonett (2002) transformed variance
(equivalently the van Zyl, Neudecker & Nel, 2000, asymptotic variance; this is
the form implemented by \code{metafor}'s \code{measure = "ARAW"}):
\deqn{\alpha\_se = (1 - \alpha) \sqrt{\frac{2k}{(k - 1)(n - 2)}}}
Note that Feldt et al.'s (1987) classical asymptotic variance instead uses an
\eqn{(n - 1)} denominator; the \eqn{(n - 2)} form above follows Bonett (2002)
for consistency with the Bonett transformation used in method 1.
}

The Bonett transformation stabilizes the variance and is recommended for meta-analysis.
}
\examples{
es_from_cronbach_alpha(
  cronbach_alpha = 0.85, n_sample = 200, n_items = 10
)
}
\references{
Bonett, D. G. (2002). Sample size requirements for testing and estimating coefficient alpha.
Journal of Educational and Behavioral Statistics, 27(4), 335-340.

Feldt, L. S., Woodruff, D. J., & Salih, F. A. (1987). Statistical inference for coefficient alpha.
Applied Psychological Measurement, 11(1), 93-103.

van Zyl, J. M., Neudecker, H., & Nel, D. G. (2000). On the distribution of the maximum likelihood
estimator of Cronbach's alpha. Psychometrika, 65(3), 271-280.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PHI_CHISQ.R
\name{es_from_phi}
\alias{es_from_phi}
\title{Convert a phi value to several effect size measures}
\usage{
es_from_phi(phi, n_cases, n_exp, n_sample, reverse_phi)
}
\arguments{
\item{phi}{phi value}

\item{n_cases}{total number of cases/events}

\item{n_exp}{total number of participants in the exposed group}

\item{n_sample}{total number of participants in the sample}

\item{reverse_phi}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab OR + RR + NNT\cr
\tab \cr
\code{converted effect size measure} \tab D + G + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 8. Phi or chi-square'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a phi value to several effect size measures
}
\details{
The functions computes an odds ratio (OR), risk ratio (RR), and number needed to treat (NNT)
from the the phi coefficient, the total number of participants,
the total number of cases and the total number of people exposed.
Cohen's d (D) and Hedges' g (G) are tried to be obtained from the OR, or are converted
using the approach by Lipsey et al. (2001).
The correlation coefficients (R/Z) are converted by assuming that the phi coefficient
is equal to a R, and the variances of R and Z are obtained using the approach proposed
by Lipsey et al. (2001) as well as by our own calculations.

\strong{To estimate the OR, RR, NNT,},
this function reconstructs a 2x2 table (using the approach proposed by Viechtbauer, 2023).

Then, the calculations of the \code{\link{es_from_2x2}()} function are applied.

\strong{To estimate D, G and the correlation coefficients (R/Z) when the 2x2 table cannot be
reconstructed} (e.g., the number of cases or exposed participants is missing), the phi
coefficient -- which is the Pearson correlation of the two binary variables -- is
treated directly as a correlation coefficient and passed to
\code{\link{es_from_pearson_r}()}. This yields R and Z with their standard large-sample
sampling variances,
\deqn{r = phi, \quad r\_se = \frac{1 - r^2}{\sqrt{n\_sample - 1}}}
\deqn{z = atanh(r), \quad z\_se = \frac{1}{\sqrt{n\_sample - 3}}}
and D and G are then obtained from R exactly as in \code{\link{es_from_pearson_r}()}
(the R to D step assumes balanced groups). In this situation the OR, RR and NNT require
the 2x2 margins and are returned as NA. When the 2x2 table can be reconstructed, D, G,
R and Z are instead derived from it (see \code{\link{es_from_2x2}()}).
}
\examples{
es_from_phi(phi = 0.3, n_sample = 120, n_cases = 20, n_exp = 40)
}
\references{
Viechtbauer (2023). Accessed at https://wviechtb.github.io/metafor/reference/conv.2x2.html.
Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_OR.R
\name{es_from_or}
\alias{es_from_or}
\title{Convert an odds ratio value to several effect size measures}
\usage{
es_from_or(
  or,
  logor,
  n_cases,
  n_controls,
  n_sample,
  small_margin_prop,
  baseline_risk,
  n_exp,
  n_nexp,
  or_to_cor = "bonett",
  or_to_rr = "metaumbrella_cases",
  reverse_or
)
}
\arguments{
\item{or}{odds ratio value}

\item{logor}{log odds ratio value}

\item{n_cases}{number of cases/events}

\item{n_controls}{number of controls/no-event}

\item{n_sample}{total number of participants in the sample}

\item{small_margin_prop}{smallest margin proportion of the underlying 2x2 table}

\item{baseline_risk}{proportion of cases in the non-exposed group (n_cases_nexp / n_nexp is used when missing)}

\item{n_exp}{number of participants in the exposed group}

\item{n_nexp}{number of participants in the non-exposed group}

\item{or_to_cor}{formula used to convert the \code{or} value into a correlation coefficient (see details).}

\item{or_to_rr}{formula used to convert the \code{or} value into a risk ratio (see details).}

\item{reverse_or}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab N/A\cr
\tab \cr
\code{converted effect size measure} \tab OR + RR + NNT\cr
\code{} \tab D + G + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 2. Odds Ratio'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an odds ratio value to several effect size measures
}
\details{
This function computes the standard error of the log odds ratio.
Risk ratio (RR), Cohen's d (D), Hedges' g (G) and correlation coefficients (R/Z),
are converted from the odds ratio value.

\strong{Estimation of the standard error of the log OR.}
This function generates the standard error of an odds ratio (OR) based on the OR value and the number of cases and controls.
More precisely, this function simulates all combinations of the possible number of cases and controls in the exposed and non-exposed groups
compatible with the reported OR value and with the overall number of cases and controls. Then, our function assumes that the variance of the OR
is equal to the mean of the standard error of all possible situations. This estimation thus necessarily comes with some imprecision and should not
be used before having requested the value (or raw data) to authors of the original report.

\strong{Conversion of other effect size measures.}
Calculations of \code{\link{es_from_or_se}()} are then applied to
estimate the other effect size measures
}
\examples{
es_or_guess <- es_from_or(or = 0.5, n_cases = 210, n_controls = 220)
es_or <- es_from_or_se(or = 0.5, logor_se = 0.4, n_cases = 210, n_controls = 220)
round(es_or_guess$logor_se, 0.10) == round(es_or$logor_se, 0.10)
}
\references{
Gosling, C. J., Solanes, A., Fusar-Poli, P., & Radua, J. (2023). metaumbrella: the first comprehensive suite to perform data analysis in umbrella reviews with stratification of the evidence. BMJ mental health, 26(1), e300534. https://doi.org/10.1136/bmjment-2022-300534
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_extraction.R
\name{see_input_data}
\alias{see_input_data}
\title{Overview of effect size measures generated from each type of input data}
\usage{
see_input_data(
  measure = c("all", "d", "g", "md", "or", "rr", "nnt", "r", "z", "logvr", "logcvr",
    "irr"),
  type_of_measure = c("natural", "natural+converted"),
  name = "mcv_input_data",
  extension = c("data.frame", ".txt", ".csv", ".xlsx"),
  verbose = TRUE
)
}
\arguments{
\item{measure}{Target effect size measure (one of the 14 available in metaConvert). Default is "all".}

\item{type_of_measure}{One of "natural+converted" or "natural" (see details).}

\item{name}{Name of the file created}

\item{extension}{Extension of the file created. Most common are ".xlsx", ".csv" or ".txt". It is also possible to generate an R dataframe object by using the "data.frame" extension.}

\item{verbose}{logical variable indicating whether some information should be printed (e.g., the location where the sheet is created when using ".xlsx", ".csv" or ".txt" extensions)}
}
\value{
This function returns a table dataset presenting the input data enabling to
compute each effect size measure.
}
\description{
Overview of effect size measures generated from each type of input data
}
\details{
This function generates, on your computer on in the console,
a dataset showing each effect size measure computed from each type of input data.
The exact combination and names of input data required are available in the links.

The \code{measure} argument allows to filter the dataset created.
Only the input data allowing to estimate the selected effect size measure will be shown. Default is "all".
The \code{type_of_measure} argument allows to filter the dataset created.
\itemize{
\item If "natural+converted" is selected, the dataset will contain all input data allowing to naturally estimate
and to convert the selected effect size measure
\item If "natural" is selected, the dataset will contain all input data allowing to naturally estimate
the selected effect size measure
}
\subsection{Extension}{

You can export a file in various formats outside R
(by indicating, for example, ".txt", ".xlsx", or ".csv") in the \code{extension} argument.
You can also visualise this dataset directly in R by setting \code{extension = "R"}.

This table is designed to be used in combination with tables showing the combination of input data
leading to estimate each of the effect size measures (https://metaconvert.org/html/input.html)
}
}
\examples{
see_input_data(measure = "md", extension = "data.frame")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ETASQ.R
\name{es_from_etasq}
\alias{es_from_etasq}
\title{Convert an eta-squared value to various effect size measures}
\usage{
es_from_etasq(
  etasq,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_etasq
)
}
\arguments{
\item{etasq}{an eta-squared value (binary predictor, ANOVA model), defined as
\eqn{SS_{effect} / SS_{total}}. For a one-way, two-group ANOVA this equals the
partial eta-squared \eqn{SS_{effect} / (SS_{effect} + SS_{error})}, so the two
coincide and either label is correct here (unlike the ANCOVA case handled by
\code{\link{es_from_etasq_adj}()}, which requires the partial form specifically).}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation.}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_etasq}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an eta-squared value to various effect size measures
}
\details{
This function first computes a Cohen's d (D) and Hedges' g (G)
from the eta squared of a binary predictor (ANOVA model).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate a Cohen's d} the following formula is used (Cohen, 1988):
\deqn{d = 2 * \sqrt{\frac{etasq}{1 - etasq}}}

Note that this closed form is the large-sample, \strong{equal-groups} (\eqn{n\_exp = n\_nexp})
limit of the exact F-based conversion. For markedly unequal group sizes it is
approximate; supplying the ANOVA F (\code{\link{es_from_anova_f}()}), which
carries the per-group sample sizes, is more accurate.

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_etasq(etasq = 0.28, n_exp = 20, n_nexp = 22)
}
\references{
Cohen, J. (1988). Statistical power analysis for the behavioral sciences. Routledge.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_statistics.R
\name{es_from_ancova_f}
\alias{es_from_ancova_f}
\title{Convert a F-statistic obtained from an ANCOVA model into several effect size measures.}
\usage{
es_from_ancova_f(
  ancova_f,
  cov_outcome_r,
  n_cov_ancova,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_f
)
}
\arguments{
\item{ancova_f}{a F-statistic from an ANCOVA (binary predictor)}

\item{cov_outcome_r}{correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_f}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 18. Adjusted: ANCOVA statistics, eta-squared'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a F-statistic obtained from an ANCOVA model into several effect size measures.
}
\details{
This function first computes an "adjusted" Cohen's d (D), and
Hedges' g (G) from the F-value of an ANCOVA (binary predictor).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate a Cohen's d} the formula used is (table 12.3 in Cooper):
\deqn{cohen\_d =  \sqrt{ancova\_f * \frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp}} * \sqrt{1 - cov\_out\_cor^2}}

\strong{To estimate other effect size measures},
Calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
}
\examples{
es_from_ancova_f(ancova_f = 4, cov_outcome_r = 0.2, n_cov_ancova = 3, n_exp = 20, n_nexp = 20)
}
\references{
Cooper, H., Hedges, L. V., & Valentine, J. C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_COR.R
\name{es_from_fisher_z}
\alias{es_from_fisher_z}
\title{Convert a Fisher's z (r-to-z transformation) to several effect size measures}
\usage{
es_from_fisher_z(
  fisher_z,
  n_sample,
  unit_type = "raw_scale",
  n_exp,
  n_nexp,
  cor_to_smd = "viechtbauer",
  sd_iv,
  unit_increase_iv,
  reverse_fisher_z
)
}
\arguments{
\item{fisher_z}{a Fisher's r-to-z transformed correlation coefficient}

\item{n_sample}{the total number of participants}

\item{unit_type}{the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "value"}

\item{n_exp}{number of the experimental/exposed group}

\item{n_nexp}{number of the non-experimental/non-exposed group}

\item{cor_to_smd}{formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.}

\item{sd_iv}{the standard deviation of the independent variable}

\item{unit_increase_iv}{a value of the independent variable that will be used to estimate the Cohen's d (see details).}

\item{reverse_fisher_z}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab R + Z\cr
\tab \cr
\code{converted effect size measure} \tab D + G + OR\cr
\tab \cr
\code{required input data} \tab See 'Section 4. Pearson's r or Fisher's z'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a Fisher's z (r-to-z transformation) to several effect size measures
}
\details{
This function converts estimates the standard error of the Fisher's z and performs the z-to-r Fisher's transformation.

Last, it converts this r value into a Cohen's d and OR (see details in \code{\link{es_from_pearson_r}()}).
}
\examples{
es_from_fisher_z(
  fisher_z = .21, n_sample = 44,
)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Mathur, M. B., & VanderWeele, T. J. (2020). A Simple, Interpretable Conversion from Pearson's Correlation to Cohen's for d Continuous Exposures. Epidemiology (Cambridge, Mass.), 31(2), e16-e18. https://doi.org/10.1097/EDE.0000000000001105

Viechtbauer W (2010). "Conducting meta-analyses in R with the metafor package." Journal of Statistical Software, 36(3), 1-48. doi:10.18637/jss.v036.i03.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/psychometric_utils.R
\name{compute_sdc}
\alias{compute_sdc}
\title{Compute Smallest Detectable Change (SDC) from SEM}
\usage{
compute_sdc(sem, sem_se)
}
\arguments{
\item{sem}{standard error of measurement}

\item{sem_se}{standard error of the SEM (optional; for propagating uncertainty)}
}
\value{
A data.frame with the SDC, its standard error and 95\% CI.
}
\description{
Compute Smallest Detectable Change (SDC) from SEM
}
\details{
Computes the smallest detectable change (SDC) from the standard error
of measurement:

\deqn{SDC = 1.96 \times \sqrt{2} \times SEM}

The SDC represents the smallest change in scores that can be detected
beyond measurement error with 95\% confidence. If \code{sem_se} is provided,
the uncertainty in SDC is propagated:

\deqn{SDC\_se = 1.96 \times \sqrt{2} \times SEM\_se}
}
\examples{
sem_res <- compute_sem(sd = 10, icc = 0.85, n_sample = 100)
compute_sdc(sem = sem_res$sem, sem_se = sem_res$sem_se)
}
\references{
de Vet, H. C. W., Terwee, C. B., Mokkink, L. B., & Knol, D. L. (2011).
Measurement in Medicine: A Practical Guide. Cambridge University Press.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_REGRESSION.R
\name{es_from_beta_unstd}
\alias{es_from_beta_unstd}
\title{Convert an unstandardized regression coefficient and the standard deviation of the dependent variable
into several effect size measures}
\usage{
es_from_beta_unstd(
  beta_unstd,
  sd_dv,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_beta_unstd
)
}
\arguments{
\item{beta_unstd}{an unstandardized regression coefficient value (binary predictor, no other covariables in the model)}

\item{sd_dv}{standard deviation of the dependent variable}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_beta_unstd}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 13. (Un-)Standardized regression coefficient'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an unstandardized regression coefficient and the standard deviation of the dependent variable
into several effect size measures
}
\details{
This function estimates a Cohen's d (D) and Hedges' g (G) from an unstandardized linear regression coefficient (coming from a model with only one binary predictor),
and the standard deviation of the dependent variable.
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{The formula used to obtain the Cohen's d is}:
\deqn{N = n\_exp + n\_nexp}
\deqn{sd\_pooled = \sqrt{\frac{sd\_dv^2 * (N - 1) - unstd\_beta^2 * \frac{n\_exp * n\_nexp}{N}}{N - 2}}}
\deqn{cohen\_d = \frac{unstd\_beta}{sd\_pooled}}

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_beta_unstd(beta_unstd = 0.7, sd_dv = 0.98, n_exp = 20, n_nexp = 22)
}
\references{
Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_SMD.R
\name{es_from_cohen_d_adj}
\alias{es_from_cohen_d_adj}
\title{Convert an adjusted Cohen's d value to several effect size measures}
\usage{
es_from_cohen_d_adj(
  cohen_d_adj,
  n_cov_ancova,
  cov_outcome_r,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_d
)
}
\arguments{
\item{cohen_d_adj}{Adjusted Cohen's d (i.e., standardized mean difference) value.}

\item{n_cov_ancova}{number of covariates}

\item{cov_outcome_r}{covariate-outcome correlation (in case of multiple covariates, the multiple correlation)}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_d}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 1. Cohen's d or Hedges' g'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an adjusted Cohen's d value to several effect size measures
}
\details{
This function estimates the standard error of an adjusted Cohen's d value and Hedges' g (G), and
converts an odds ratio (OR) and correlation coefficients (R/Z).

\strong{To estimate the standard error of Cohen's d}, the following formula is used (table 12.3 in Cooper):
\deqn{d\_se = \sqrt{\frac{n\_exp+n\_nexp}{n\_exp*n\_nexp} * (1 - cov\_outcome\_r^2) + \frac{cohen\_d\_adj^2}{2*(n\_exp+n\_nexp)}}}

\strong{To estimate other effect size measures}, calculations of the
\code{\link{es_from_cohen_d}()} function are used (with the exception of the degree of freedom
that is estimated as \code{df = n_exp + n_nexp - 2 - n_cov_ancova}).
}
\examples{
es_from_cohen_d_adj(cohen_d_adj = 1, n_cov_ancova = 4, cov_outcome_r = .30, n_exp = 20, n_nexp = 20)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PROP.R
\name{es_from_prop_single_group}
\alias{es_from_prop_single_group}
\title{Convert single-group proportion into an effect size measure}
\usage{
es_from_prop_single_group(prop, n_sample, prop_to_es = "raw", reverse_prop)
}
\arguments{
\item{prop}{proportion of cases in the single group (0-1)}

\item{n_sample}{total sample size}

\item{prop_to_es}{method used to compute the effect size from the proportion.
Must be \code{"raw"} (default), \code{"logit"}, or \code{"freeman_tukey"}.}

\item{reverse_prop}{a logical value indicating whether the direction of the proportion should be flipped.}
}
\value{
This function estimates a single-group proportion.

\tabular{ll}{
\code{natural effect size measure} \tab PROP\cr
\tab \cr
\code{converted effect size measure} \tab N/A\cr
\tab \cr
\code{required input data} \tab prop + n_sample\cr
\tab \cr
}

Note: when \code{prop_to_es = "logit"} or \code{"freeman_tukey"}, the returned
\code{prop}, \code{prop_se}, \code{prop_ci_lo} and \code{prop_ci_up} columns are
on the transformed (log-odds / Freeman-Tukey double-arcsine) scale, \strong{not}
the \verb{[0, 1]} proportion scale, and are not back-transformed. Only
\code{prop_to_es = "raw"} returns values on the proportion scale.
}
\description{
Convert single-group proportion into an effect size measure
}
\details{
This function computes an effect size from a single-group proportion.
\enumerate{
\item \strong{When \code{prop_to_es = "raw"}} (default), the proportion is used as the effect size
and its standard error is:
\deqn{prop\_se = \sqrt{\frac{prop \times (1 - prop)}{n}}}
\item \strong{When \code{prop_to_es = "logit"}}, the proportion is converted to a log odds:
\deqn{logit = \log\left(\frac{prop}{1 - prop}\right)}
\deqn{logit\_se = \sqrt{\frac{1}{n \times prop \times (1 - prop)}}}
\item \strong{When \code{prop_to_es = "freeman_tukey"}}, the Freeman-Tukey double arcsine
transformation is applied (Barendregt et al., 2013):
\deqn{FT = \frac{1}{2}\left[\arcsin\left(\sqrt{\frac{x}{n+1}}\right) + \arcsin\left(\sqrt{\frac{x+1}{n+1}}\right)\right]}
with \eqn{x = n \times prop}, and:
\deqn{FT\_se = \frac{1}{\sqrt{4n + 2}}}
}

When prop is equal to 0 or 1, a 0.5 correction is applied for the raw and logit methods:
\eqn{prop\_corrected = \frac{x + 0.5}{n + 1}}. For the raw method this shifts the
reported point estimate itself (a boundary proportion of exactly 0 or 1 is
returned as \eqn{(x + 0.5)/(n + 1)}, i.e. nudged toward the interior), matching
the convention of \code{metafor}'s \code{measure = "PR"}. The corrected
denominator \eqn{n + 1} also replaces \eqn{n} in the raw and logit standard
errors for these boundary rows (i.e.,
\eqn{\sqrt{p_c (1 - p_c) / (n + 1)}} and \eqn{\sqrt{1 / ((n + 1) \times p_c \times (1 - p_c))}}),
matching \code{metafor}'s \code{"PR"}/\code{"PLO"} convention throughout.

Proportions outside \eqn{[0, 1]} are set to NA with a warning (the affected
rows return NA, but do not abort the run).
}
\examples{
es_from_prop_single_group(prop = 0.30, n_sample = 100)
}
\references{
Barendregt, J. J., Doi, S. A., Lee, Y. Y., Norman, R. E., & Vos, T. (2013).
Meta-analysis of prevalence. Journal of Epidemiology and Community Health, 67(11), 974-978.

Miller, J. J. (1978). The inverse of the Freeman-Tukey double arcsine transformation.
The American Statistician, 32(4), 138-138. (Back-transformation left to the
user; not applied here: Freeman-Tukey results stay on the transformed scale.)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ICC.R
\name{es_from_icc}
\alias{es_from_icc}
\title{Convert an intraclass correlation coefficient (ICC) into an effect size measure}
\usage{
es_from_icc(
  icc,
  n_sample,
  n_measurements,
  icc_type = "agreement",
  icc_to_es = "bonett"
)
}
\arguments{
\item{icc}{intraclass correlation coefficient value}

\item{n_sample}{total sample size (number of subjects)}

\item{n_measurements}{number of measurements or raters}

\item{icc_type}{ICC type: \code{"agreement"} for ICC(2,1) (two-way random, absolute agreement)
or \code{"consistency"} for ICC(3,1) (two-way mixed, consistency). Default is \code{"agreement"}.}

\item{icc_to_es}{method used to compute the effect size from ICC.
Must be either \code{"bonett"} or \code{"raw"}.}
}
\value{
This function estimates the standard error of the ICC.

\tabular{ll}{
\code{natural effect size measure} \tab icc\cr
\tab \cr
\code{converted effect size measure} \tab N/A\cr
\tab \cr
\code{required input data} \tab icc + n_sample + n_measurements\cr
\tab \cr
}
}
\description{
Convert an intraclass correlation coefficient (ICC) into an effect size measure
}
\details{
This function computes an effect size from an ICC.
\enumerate{
\item When \code{icc_to_es = "bonett"} (default), the Bonett (2002) transformation
is applied:
\deqn{T(ICC) = \ln(1 - ICC)}
with the one-way random-model / two-way-consistency leading-order sampling
variance (Bonett, 2002; Donner & Eliasziw, 1987):
\deqn{T\_se = \sqrt{\frac{2 (1 + (k-1) ICC)^2}{k (k - 1)(n - 1)}}}
\item When \code{icc_to_es = "raw"}, the raw ICC is used and its standard error
is obtained by the delta method (\eqn{(1 - ICC)} times the transformed-scale SE).
}

\strong{Scope of the SE formula.} For the two-way consistency ICC(3,1)
(\code{icc_type = "consistency"}) this SE is exact at leading order: deriving
it from \eqn{F_0 = MSR/MSE} (with degrees of freedom \eqn{n-1} and
\eqn{(n-1)(k-1)}) reduces to the same expression, and it still depends on the
ICC value (it is \emph{not} \eqn{\rho}-free). For the two-way
absolute-agreement ICC(2,1) (\code{icc_type = "agreement"}, the default) the
same formula is only a \strong{one-way approximation that assumes negligible
between-rater variance}: when raters differ systematically
(\eqn{\sigma^2_{rater} > 0}), the ICC(2,1) estimator depends on the
between-rater mean square, which has only \eqn{k - 1} degrees of freedom, so
its true sampling variance does not shrink at the \eqn{1/n} rate this formula
assumes and the reported SE/CI can be markedly anti-conservative (simulation:
95\\% CI coverage around 0.74-0.76 with moderate rater variance, degrading as
\eqn{n} grows). The exact ICC(2,1) variance requires the rater-variance
component, which summary data do not report; a per-row informational flag
(V31) marks agreement-type rows for this reason. If the raters are known to
be exchangeable (negligible rater variance), the approximation is accurate.

\strong{Scale note.} Under the default \code{icc_to_es = "bonett"} the returned
\code{icc_se} column is on the \eqn{\ln(1 - ICC)} scale. If you feed it to
\code{\link{compute_sem}} (whose \code{icc_se} argument expects the RAW-scale
SE), convert it first: \code{raw_se = icc_se * (1 - icc)} - or call
\code{es_from_icc} with \code{icc_to_es = "raw"}.
}
\examples{
es_from_icc(
  icc = 0.80, n_sample = 50, n_measurements = 2, icc_type = "agreement"
)
}
\references{
Bonett, D. G. (2002). Sample size requirements for estimating intraclass correlations
with desired precision. Statistics in Medicine, 21(9), 1331-1335.

Shrout, P. E., & Fleiss, J. L. (1979). Intraclass correlations: uses in assessing rater
reliability. Psychological Bulletin, 86(2), 420-428.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_compare_df.R
\name{compare_df}
\alias{compare_df}
\title{Flag the differences between two dataframes.}
\usage{
compare_df(
  df_extractor_1,
  df_extractor_2,
  ordering_columns = NULL,
  tolerance = 0,
  tolerance_type = "ratio",
  output = "html",
  file_name = "comparison.xlsx"
)
}
\arguments{
\item{df_extractor_1}{a first dataset. Differences with the second dataset will be flagged in green.}

\item{df_extractor_2}{a second dataset. Differences with the first dataset will be flagged in red.}

\item{ordering_columns}{column names that should be used to re-order the two datasets before running the comparisons}

\item{tolerance}{the cut-off value used to flag differences between two numeric values}

\item{tolerance_type}{must be either 'difference' or 'ratio'}

\item{output}{type of object returned by the function (see 'Value' section). Must be either 'wide', 'long', 'html', 'html2' or 'xlsx'.}

\item{file_name}{the name of the generated file (only used when \code{output="xlsx"})}
}
\value{
This function returns a dataframe composed of the rows that include a
difference (all identical rows are removed).
Several outputs can be requested :
\enumerate{
\item setting \code{output="xlsx"} returns an excel file. A message indicates the location of the generated file on your computer.
\item setting \code{output="html"} returns an html file
\item setting \code{output="html2"} returns an html file (only useful when the "html" command did not make the html pane appear in R studio).
\item setting \code{output="wide"} a wide dataframe
\item setting \code{output="long"} a long dataframe
}
}
\description{
Flag the differences between two dataframes.
}
\details{
This function aims to facilitate the comparison of two datasets created by blind data extractors during a systematic review.
It is a wrapper of several functions from the 'compareDF' package.
}
\examples{
df.compare1 = df.compare1[order(df.compare1$author), ]
df.compare2 = df.compare2[order(df.compare2$year), ]

compare_df(
  df_extractor_1 = df.compare1,
  df_extractor_2 = df.compare2,
  ordering_columns = c("author", "year")
)
}
\references{
Alex Joseph (2022). compareDF: Do a Git Style Diff of the Rows Between Two Dataframes with Similar Structure. R package version 2.3.3. https://CRAN.R-project.org/package=compareDF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_RR.R
\name{es_from_rr_se}
\alias{es_from_rr_se}
\title{Convert a risk ratio value and standard error to various effect size measures}
\usage{
es_from_rr_se(
  rr,
  logrr,
  logrr_se,
  baseline_risk,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  rr_to_or = "metaumbrella",
  reverse_rr
)
}
\arguments{
\item{rr}{risk ratio value}

\item{logrr}{log risk ratio value}

\item{logrr_se}{standard error of the log risk ratio}

\item{baseline_risk}{proportion of cases in the non-exposed group (n_cases_nexp / n_nexp is used when missing)}

\item{n_exp}{number of participants in the exposed group}

\item{n_nexp}{number of participants in the non-exposed group}

\item{n_cases}{number of cases/events}

\item{n_controls}{number of controls/no-event}

\item{rr_to_or}{formula used to convert the \code{rr} value into an odds ratio (see details).}

\item{reverse_rr}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab RR\cr
\tab \cr
\code{converted effect size measure} \tab OR + NNT + RD\cr
\tab \cr
\code{required input data} \tab See 'Section 3. Risk Ratio'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a risk ratio value and standard error to various effect size measures
}
\details{
This function converts the (log) risk ratio (RR) value and its standard error
to odds ratio (OR) and number needed to treat.

\strong{To estimate the odds ratio and its standard error}, various formulas can be used.

\strong{A.} First, the approach described in Grant (2014) can be used.
However, in the paper, only the formula to convert an RR value to a OR value
is described. To derive the variance, we used this formula to convert the bounds of the 95\% CI, which
were then used to obtain the variance.

This argument requires (rr + baseline_risk + rr_ci_lo + rr_ci_up) to generate a RR.
The following formulas are used (br = baseline_risk):
\deqn{or = \frac{rr * (1 - br)}{1 - rr * br}}
\deqn{or\_ci\_lo = \frac{rr\_ci\_lo * (1 - br)}{1 - rr\_ci\_lo * br}}
\deqn{or\_ci\_up = \frac{rr\_ci\_up * (1 - br)}{1 - rr\_ci\_up * br}}
\deqn{logor\_se = \frac{log(or\_ci\_up) - log(or\_ci\_lo)}{2 * qnorm(.975)}}

\strong{B.} Second, the formulas implemented in the metaumbrella package can be used (\code{or_to_rr = "metaumbrella_exp"}).
This argument requires (rr + logrr_se + n_exp + n_nexp) to generate a OR.
More precisely, we previously developed functions that simulate all combinations of the possible number of cases and controls
in the exposed and non-exposed groups compatible with the actual value of the RR.
Then, the functions select the contingency table whose standard error coincides best with the standard error reported.
The RR value and its standard are obtained from this estimated contingency table.

\strong{C.} Third, it is possible to transpose the RR to a OR (\code{rr_to_or = "transpose"}).
This argument requires (rr + logrr_se) to generate a OR.
It is known that OR and RR are similar when the baseline risk is small.
Therefore, users can request to simply transpose the RR value & standard error into a OR value & standard error.
\deqn{or = rr}
\deqn{logor\_se = logrr\_se}

\strong{D.} Fourth, it is possible to recreate the 2x2 table using the dipietrantonj's formulas (\code{rr_to_or = "dipietrantonj"}).
This argument requires (rr + logrr_ci_lo + logrr_ci_lo) to generate a OR. Information on this approach can be retrieved in
Di Pietrantonj (2006).

\strong{To estimate the NNT}, the formulas used are :
\deqn{nnt = \frac{1}{br * (1 - rr)}}
}
\examples{
es_from_rr_se(rr = 2.12, logrr_se = 0.242, n_exp = 120, n_nexp = 44)
}
\references{
Di Pietrantonj C. (2006). Four-fold table cell frequencies imputation in meta analysis. Statistics in medicine, 25(13), 2299-2322. https://doi.org/10.1002/sim.2287

Gosling, C. J., Solanes, A., Fusar-Poli, P., & Radua, J. (2023). metaumbrella: the first comprehensive suite to perform data analysis in umbrella reviews with stratification of the evidence. BMJ mental health, 26(1), e300534. https://doi.org/10.1136/bmjment-2022-300534

Grant R. L. (2014). Converting an odds ratio to a range of plausible relative risks for better communication of research findings. BMJ (Clinical research ed.), 348, f7450. https://doi.org/10.1136/bmj.f7450

Veroniki, A. A., Pavlides, M., Patsopoulos, N. A., & Salanti, G. (2013). Reconstructing 2x2 contingency tables from odds ratios using the Di Pietrantonj method: difficulties, constraints and impact in meta-analysis results. Research synthesis methods, 4(1), 78-94. https://doi.org/10.1002/jrsm.1061
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions_summary.R
\name{summary.metaConvert}
\alias{summary.metaConvert}
\title{Synthesize information of an object of class \dQuote{metaConvert} into a dataframe}
\usage{
\method{summary}{metaConvert}(
  object,
  digits = 3,
  flags = TRUE,
  flag_options = list(),
  guidance = TRUE,
  include_raw = TRUE,
  ...
)
}
\arguments{
\item{object}{an object of class \dQuote{metaConvert}}

\item{digits}{an integer value specifying the number of decimal places for the rounding of numeric values. Default is 3.}

\item{flags}{a logical value indicating whether quality/plausibility flags should be generated. Default is TRUE. The \code{es_flags} column includes both input validation flags (generated during \code{\link{convert_df}}) and post-computation quality flags.}

\item{flag_options}{a named list of thresholds overriding the defaults (and any options set in \code{\link{convert_df}}).
Available options:
\itemize{
\item \code{smd_max} (default 3): |SMD| above this value is flagged
\item \code{r_max} (default 0.95): same, for |r|
\item \code{log_or_max} (default 5): same, for |logOR| and |logRR|
\item \code{n_min} (default 10): flag sample sizes below this value
\item \code{iqr_mult} (default 3): IQR multiplier for the cross-row outlier detection
\item \code{dispersion_max_smd} / \code{dispersion_max_r} / \code{dispersion_max_logor} (defaults 0.5 / 0.15 / 1.0): maximum SD of the ES across estimation methods (SMD, correlation, logOR/logRR)
\item \code{diff_max_smd} / \code{diff_max_r} / \code{diff_max_logor} (defaults 1.0 / 0.3 / 2.0): maximum min-max ES difference across estimation methods
\item \code{overlap_min} (default 0.80): minimum CI overlap between the min/max estimates (0-1 scale)
\item \code{enable_cross_row} (default TRUE): enable/disable the cross-row checks
\item \code{direction_conflict_min} (default 2): number of significantly-positive and significantly-negative studies needed to raise the direction conflict flag (G1)
}}

\item{guidance}{a logical value indicating whether missing data guidance should be generated for rows where the effect size is NA. Default is TRUE. When enabled, a column \code{es_guidance} (or \code{es_guidance_crude}/\code{es_guidance_adjusted}) is appended, listing the closest estimation methods and which specific columns are missing.}

\item{include_raw}{a logical value indicating whether the raw input columns should be appended after the effect size columns in the returned dataframe. Default is TRUE.}

\item{...}{other arguments that can be passed to the function}
}
\value{
This function returns a dataframe with many columns. We present below the information stored in each column of the returned dataframe

\strong{1. Raw user information.}
The first columns placed at the left of the returned dataset are simply information provided
by the users to facilitate the identification of each row.
If the following columns are missing in the original dataset, these columns will not appear in
the returned dataset.

\tabular{ll}{
\code{row_id} \tab Row number in the original dataset.\cr
\tab \cr
\code{study_id} \tab Identifier of the study.\cr
\tab \cr
\code{author} \tab Name of the author of the study.\cr
\tab \cr
\code{year} \tab Year of publication of the study.\cr
\tab \cr
\code{predictor} \tab Name of the predictor (intervention, risk factor, etc.).\cr
\tab \cr
\code{outcome} \tab Name of the outcome.\cr
\tab \cr
\code{info_expected} \tab Types of input data users expect to be used to estimate their effect size measure.\cr
\tab \cr
}

\strong{2. Information on generated effect sizes.}
Then, the function returns information on calculations. For example, users can retrieve
the effect size measure estimated, the number and type(s) of input data allowing to estimate the
chosen effect size measure, and the method used to obtain a unique effect size if overlapping
input data were available.
These columns could have several suffix.
\itemize{
\item If users requested to separate crude and adjusted estimates,
then the following columns will be presented with both a "_crude" suffix and a "_adjusted" suffix.
\item If users did not request to separate the presentation of crude and adjusted estimates, the following columns
will have no suffix.
}

For example, let's take column "all_info". It can be "all_info_crude" (all input data used to estimate any crude effect size),
"all_info_adjusted" (all input data leading to estimate any adjusted effect size),
or "all_info" (all input data leading to estimate any crude or adjusted effect sizes).

To facilitate the presentation, we thus refer to these columns as \code{name_of_the_column*},
the \code{*} meaning that it could end by _crude, _adjusted or "".

\tabular{ll}{
\code{all_info*} \tab list of input data available in the dataset that was used to estimate any effect size measure.\cr
\tab \cr
\code{measure*} \tab effect size measure requested by the user.\cr
\tab \cr
\code{info_measure*} \tab input data available to estimate the requested effect size measure.\cr
\tab \cr
\code{n_estimations*} \tab number of input data available to estimate the requested effect size measure.\cr
\tab \cr
\code{es_selected*} \tab method chosen by users to estimate the main effect size when overlapping data are present.\cr
\tab \cr
\code{info_used*} \tab type of input data used to estimate the main effect size.\cr
\tab \cr
}

\strong{3. Main effect size.}
The following columns contain the key information, namely, the main effect size + standard error + 95\% CI.

Again, the suffix of these columns can vary depending on the separation of effect sizes
estimated from crude and adjusted input data.

\tabular{ll}{
\code{es*} \tab main effect size value.\cr
\tab \cr
\code{se*} \tab standard error of the effect size.\cr
\tab \cr
\code{es_ci_lo*} \tab lower bound of the 95\% CI around the effect size.\cr
\tab \cr
\code{es_ci_up*} \tab upper bound of the 95\% CI around the effect size.\cr
\tab \cr
}

\strong{4. Overlapping effect sizes}
These columns are useful ONLY if a given comparison (i.e., row) has multiple input data
enabling to compute the requested effect size measure.

These columns identify the smallest/largest effect size per comparison,
and some indicators of consistency.

Again, the suffix of these columns can vary depending on the separation of effect sizes
estimated from crude and adjusted input data.

\tabular{ll}{
\code{min_info*} \tab type of input data leading to the smallest effect size for the comparison.\cr
\tab \cr
\code{min_es_value*} \tab smallest effect size value for the comparison.\cr
\tab \cr
\code{min_es_se*} \tab standard error of the smallest effect size for the comparison.\cr
\tab \cr
\code{min_es_ci_lo*} \tab lower bound of the 95\% CI of the smallest effect size for the comparison.\cr
\tab \cr
\code{min_es_ci_up*} \tab upper bound of the 95\% CI of the smallest effect size for the comparison.\cr
\tab \cr
}

\tabular{ll}{
\code{max_info*} \tab type of input data leading to the largest effect size for the comparison.\cr
\tab \cr
\code{max_es_value*} \tab largest effect size value for the comparison.\cr
\tab \cr
\code{max_es_se*} \tab standard error of the largest effect size for the comparison.\cr
\tab \cr
\code{max_es_ci_lo*} \tab lower bound of the 95\% CI of the largest effect size for the comparison.\cr
\tab \cr
\code{max_es_ci_up*} \tab upper bound of the 95\% CI of the largest effect size for the comparison.\cr
\tab \cr
}

\tabular{ll}{
\code{diff_min_max*} \tab difference between the smallest and largest effect size for the comparison.\cr
\tab \cr
\code{overlap_min_max*} \tab \% of overlap between the 95\% CIs of the largest/smallest effect sizes for the comparison.\cr
\tab \cr
\code{dispersion_es*} \tab standard deviation of all effect sizes for the comparison.\cr
\tab \cr
}

\strong{5. Quality/plausibility flags}
When \code{flags = TRUE} (the default), a flags* column is added, containing
the semicolon-separated list of issues detected for the row (empty when none).

\tabular{ll}{
\code{flags*} \tab quality/plausibility flags for the effect size in this row.\cr
\tab \cr
}

Flag categories:
\itemize{
\item \strong{A (Numeric integrity)}: Inf/NaN values, negative SE, inverted CI, ES outside CI
\item \strong{B (Bounds violations)}: correlation outside \eqn{[-1,1]}, non-positive OR/RR, NNT in (-1,1), proportion outside \eqn{[0,1]}, RD outside \eqn{[-1,1]}
\item \strong{C (Plausibility)}: unusually large SMD, zero SE, high correlation, large logOR/logRR, small sample size
\item \strong{D (Cross-row outliers)}: ES or SE is an outlier relative to other rows (IQR method)
\item \strong{E (Internal consistency)}: high dispersion across estimation methods, low/zero CI overlap, large min-max difference
}

\strong{6. Missing data guidance}
When \code{guidance = TRUE} (the default), an es_guidance* column indicates,
for rows without an effect size, the closest estimation methods and which
input columns are missing (empty otherwise).

\tabular{ll}{
\code{es_guidance*} \tab guidance on which columns to add to obtain an effect size.\cr
\tab \cr
}
}
\description{
Synthesize information of an object of class \dQuote{metaConvert} into a dataframe
}
\details{
Summary method for objects of class \dQuote{metaConvert} produced by the \code{\link{convert_df}}
function. This function automatically:
\enumerate{
\item computes all effect sizes from all available input data
\item selects, if requested, a  main effect size for each association/comparison using the information passed by
the user in the \code{es_selected} argument of the \code{convert_df} function
\item identifies the smallest and largest effect size for each association/comparison
\item estimates the absolute difference between the smallest and largest effect size for each
association/comparison
\item estimates the percentage of overlap between the 95\% confidence intervals of the smallest and
largest effect size for each association/comparison
}
}
\examples{
### generate a summary of the results of an umbrella object
summary(
  convert_df(df.haza, measure = "g"),
  digits = 5)
}
\seealso{
\code{\link{metaConvert-package}} for the formatting of well-formatted datasets\cr
\code{\link{convert_df}} for estimating effect sizes from a dataset\cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANOVA_means.R
\name{es_from_means_sd}
\alias{es_from_means_sd}
\title{Convert means and standard deviations of two independent groups into several effect size measures}
\usage{
es_from_means_sd(
  mean_exp,
  mean_sd_exp,
  mean_nexp,
  mean_sd_nexp,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  smd_denom = "pooled",
  reverse_means
)
}
\arguments{
\item{mean_exp}{mean of participants in the experimental/exposed group.}

\item{mean_sd_exp}{standard deviation of participants in the experimental/exposed group.}

\item{mean_nexp}{mean of participants in the non-experimental/non-exposed group.}

\item{mean_sd_nexp}{standard deviation of participants in the non-experimental/non-exposed group.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the generated \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{smd_denom}{standardizer for the standardized mean difference. "pooled" (default) uses the pooled endpoint SD (Cohen's d / Hedges' g); "glass" (alias "control") uses the control (non-experimental) endpoint SD (Glass's delta); "glass_robust" (alias "control_robust") is Glass's delta with a heteroscedasticity-consistent sampling variance.}

\item{reverse_means}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 9. Means and dispersion (crude)'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert means and standard deviations of two independent groups into several effect size measures
}
\details{
This function first computes a Cohen's d (D), Hedges' g (G) and mean difference (MD)
from the means and standard deviations of two independent groups.
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate a mean difference}  (formulas 12.1-12.6 in Cooper):
\deqn{md = mean\_exp - mean\_nexp}
\deqn{md\_se = \sqrt{\frac{mean\_sd\_exp^2}{n\_exp} + \frac{mean\_sd\_nexp^2}{n\_nexp}}}
The confidence interval uses the unequal-variance (Welch-Satterthwaite) degrees of freedom
that match this standard error (as in \code{stats::t.test(var.equal = FALSE)}):
\deqn{df = \frac{(s_e^2/n\_exp + s_n^2/n\_nexp)^2}{(s_e^2/n\_exp)^2/(n\_exp - 1) + (s_n^2/n\_nexp)^2/(n\_nexp - 1)}}
\deqn{md\_ci\_lo = md - md\_se * qt(.975, df)}
\deqn{md\_ci\_up = md + md\_se * qt(.975, df)}

\strong{To estimate a Cohen's d} the following formulas are used (formulas 12.10-12.18 in Cooper):
\deqn{mean\_sd\_pooled = \sqrt{\frac{(n\_exp - 1) * sd\_exp^2 + (n\_nexp - 1) * sd\_nexp^2}{n\_exp+n\_nexp-2}}}
\deqn{cohen\_d =  \frac{mean\_exp - mean\_nexp}{mean\_sd\_pooled}}
\deqn{cohen\_d\_se = \sqrt{\frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp} + \frac{cohen\_d^2}{2(n\_exp+n\_nexp)}}}
\deqn{cohen\_d\_ci\_lo = cohen\_d - cohen\_d\_se * qt(.975, df = n\_exp + n\_nexp - 2)}
\deqn{cohen\_d\_ci\_up = cohen\_d + cohen\_d\_se * qt(.975, df = n\_exp + n\_nexp - 2)}

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_means_sd(
  n_exp = 55, n_nexp = 55,
  mean_exp = 2.3, mean_sd_exp = 1.2,
  mean_nexp = 1.9, mean_sd_nexp = 0.9
)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_means.R
\name{es_from_ancova_means_se}
\alias{es_from_ancova_means_se}
\title{Convert means and standard errors of two independent groups obtained from an ANCOVA model into several effect size measures}
\usage{
es_from_ancova_means_se(
  n_exp,
  n_nexp,
  ancova_mean_exp,
  ancova_mean_nexp,
  ancova_mean_se_exp,
  ancova_mean_se_nexp,
  cov_outcome_r,
  n_cov_ancova,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_means
)
}
\arguments{
\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{ancova_mean_exp}{adjusted mean of participants in the experimental/exposed group.}

\item{ancova_mean_nexp}{adjusted mean of participants in the non-experimental/non-exposed group.}

\item{ancova_mean_se_exp}{adjusted standard error of participants in the experimental/exposed group.}

\item{ancova_mean_se_nexp}{adjusted standard error of participants in the non-experimental/non-exposed group.}

\item{cov_outcome_r}{correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{smd_to_cor}{formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_means}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert means and standard errors of two independent groups obtained from an ANCOVA model into several effect size measures
}
\details{
This function converts the adjusted means standard errors of two independent groups
into standard deviations,
and then relies on the calculations of the \code{\link{es_from_ancova_means_sd}} function.

\strong{To convert the standard errors into standard deviations}, the following formula is used.
\deqn{ancova\_mean\_sd\_exp = ancova\_mean\_se\_exp * \sqrt{n\_exp}}
\deqn{ancova\_mean\_sd\_nexp = ancova\_mean\_se\_nexp * \sqrt{n\_nexp}}
Calculations of the \code{\link{es_from_ancova_means_sd}()} are then applied.
}
\examples{
es_from_ancova_means_se(
  n_exp = 55, n_nexp = 55,
  ancova_mean_exp = 2.3, ancova_mean_se_exp = 1.2,
  ancova_mean_nexp = 1.9, ancova_mean_se_nexp = 0.9,
  cov_outcome_r = 0.2, n_cov_ancova = 3
)
}
\references{
Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_MD.R
\name{es_from_ancova_md_ci}
\alias{es_from_ancova_md_ci}
\title{Convert an adjusted mean difference and adjusted standard deviation between two independent groups obtained from an ANCOVA model into several effect size measures}
\usage{
es_from_ancova_md_ci(
  ancova_md,
  ancova_md_ci_lo,
  ancova_md_ci_up,
  cov_outcome_r,
  n_cov_ancova,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_md
)
}
\arguments{
\item{ancova_md}{adjusted mean difference between two independent groups}

\item{ancova_md_ci_lo}{lower bound of the covariate-adjusted 95\% CI of the mean difference}

\item{ancova_md_ci_up}{upper bound of the covariate-adjusted 95\% CI of the mean difference}

\item{cov_outcome_r}{correlation between the outcome and covariate (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_md}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 20. Adjusted: Mean difference and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an adjusted mean difference and adjusted standard deviation between two independent groups obtained from an ANCOVA model into several effect size measures
}
\details{
This function converts the mean difference (MD) 95\% CI into a standard error,
and then relies on the calculations of the \code{\link{es_from_ancova_md_se}} function.

\strong{To convert the 95\% CI into a standard error,} the following formula is used (table 12.3 in Cooper):
\deqn{md\_se = \frac{ancova\_md\_ci\_up - ancova\_md\_ci\_lo}{(2 * qt(0.975, n\_exp + n\_nexp - 2 - n\_cov\_ancova))}}
Calculations of the \code{\link{es_from_ancova_md_se}()} are then applied.
}
\examples{
es_from_ancova_md_ci(
  ancova_md = 4, ancova_md_ci_lo = 2,
  ancova_md_ci_up = 6,
  cov_outcome_r = 0.5, n_cov_ancova = 5,
  n_exp = 20, n_nexp = 22
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANOVA_statistics.R
\name{es_from_anova_pval}
\alias{es_from_anova_pval}
\title{Convert a p-value from a one-way independent ANOVA to several effect size measures}
\usage{
es_from_anova_pval(
  anova_f_pval,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_anova_f_pval
)
}
\arguments{
\item{anova_f_pval}{p-value (two-tailed) from an ANOVA (binary predictor). If your p-value is one-tailed, simply multiply it by two.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{anova_f_pval} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_anova_f_pval}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
\tab https://metaconvert.org/html/input.html\cr
\tab \cr
}
}
\description{
Convert a p-value from a one-way independent ANOVA to several effect size measures
}
\details{
This function converts the p-value from the F-value of an ANOVA (one-way, binary predictor) into a t-value,
and then relies on the calculations of the \code{\link{es_from_student_t}()} function.

\strong{To convert the p-value into a t-value,} the following formula is used (table 12.1 in Cooper):
\deqn{student\_t = qt(\frac{anova\_f\_pval}{2}, df = n\_exp + n\_nexp - 2)}
Then, calculations of the \code{\link{es_from_student_t}()} are applied.

As for \code{\link{es_from_anova_f}}, this conversion is valid only for an F-test with a
single numerator degree of freedom (a two-group comparison): the p-value of a
multi-level (numerator df > 1) omnibus F is inverted here as if it were a two-sided
two-group t p-value, which is incorrect. The two-sided p-value also carries no
direction, so the generated effect sizes are always non-negative; use
\code{reverse_anova_f_pval} to encode the correct sign.
}
\examples{
es_from_anova_pval(anova_f_pval = 0.0012, n_exp = 20, n_nexp = 22)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PAIRED_MC.R
\name{es_from_mean_change_ci}
\alias{es_from_mean_change_ci}
\title{Convert mean changes and 95\% CI of two independent groups into standard effect size measures}
\usage{
es_from_mean_change_ci(
  mean_change_exp,
  mean_change_ci_lo_exp,
  mean_change_ci_up_exp,
  mean_change_nexp,
  mean_change_ci_lo_nexp,
  mean_change_ci_up_nexp,
  r_pre_post_exp,
  r_pre_post_nexp,
  n_exp,
  n_nexp,
  max_asymmetry = 10,
  smd_to_cor = "viechtbauer",
  pre_post_to_smd = "cooper",
  pool_sd = FALSE,
  reverse_mean_change
)
}
\arguments{
\item{mean_change_exp}{mean change of participants in the experimental/exposed group.}

\item{mean_change_ci_lo_exp}{lower bound of the 95\% CI around the mean change of the experimental/exposed group.}

\item{mean_change_ci_up_exp}{upper bound of the 95\% CI around the mean change of the experimental/exposed group.}

\item{mean_change_nexp}{mean change of participants in the non-experimental/non-exposed group.}

\item{mean_change_ci_lo_nexp}{lower bound of the 95\% CI around the mean change of the non-experimental/non-exposed group.}

\item{mean_change_ci_up_nexp}{upper bound of the 95\% CI around the mean change of the non-experimental/non-exposed group.}

\item{r_pre_post_exp}{pre-post correlation in the experimental/exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).}

\item{r_pre_post_nexp}{pre-post correlation in the non-experimental/non-exposed group (only used with \code{pre_post_to_smd = "morris_drm"}, see details).}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{max_asymmetry}{A percentage indicating the tolerance before detecting asymmetry in the 95\% CI bounds.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{pre_post_to_smd}{formula used to convert the mean change into a SMD ("morris_drm" or "morris_dz", see details).}

\item{pool_sd}{a logical value indicating whether the standardizing SD should be pooled across the two
groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
which to prefer, so this is a deliberate choice and not a technical detail.
\itemize{
\item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
within-group values are subtracted, their variances adding because the arms are independent. This
is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
broadly applicable of the two.
\item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
assumes the two arms' true standardizing SDs are equal.
}}

\item{reverse_mean_change}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert mean changes and 95\% CI of two independent groups into standard effect size measures
}
\details{
This function converts the mean change and 95\% CI of two independent groups into a Cohen's d.
The Cohen's d is then converted to other effect size measures.

This function simply internally calls the \code{\link{es_from_means_ci_pre_post}} function but setting:
\deqn{mean\_pre\_exp = 0}
\deqn{mean\_pre\_ci\_lo\_exp = 0}
\deqn{mean\_pre\_ci\_up\_exp = 0}

\deqn{mean\_exp = mean\_change\_exp}
\deqn{mean\_ci\_lo\_exp = mean\_change\_ci\_lo\_exp}
\deqn{mean\_ci\_up\_exp = mean\_change\_ci\_up\_exp}

\deqn{mean\_pre\_nexp = 0}
\deqn{mean\_pre\_ci\_lo\_nexp = 0}
\deqn{mean\_pre\_ci\_up\_nexp = 0}

\deqn{mean\_nexp = mean\_change\_nexp}
\deqn{mean\_ci\_lo\_nexp = mean\_change\_ci\_lo\_nexp}
\deqn{mean\_ci\_up\_nexp = mean\_change\_ci\_up\_nexp}

To know more about the calculations, see \code{\link{es_from_means_sd_pre_post}} function.
}
\examples{
es_from_mean_change_ci(
  n_exp = 36, n_nexp = 35,
  mean_change_exp = 8.4,
  mean_change_ci_lo_exp = 6.4, mean_change_ci_up_exp = 10.4,
  mean_change_nexp = 2.43,
  mean_change_ci_lo_nexp = 1.43, mean_change_ci_up_nexp = 3.43,
  r_pre_post_exp = 0.2, r_pre_post_nexp = 0.2
)
}
\references{
Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386. https://doi.org/10.1177/1094428106291059

Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{df.haza}
\alias{df.haza}
\title{Meta-analytic dataset inspired from Haza and colleagues (2024)}
\format{
An object of class \code{data.frame} with 170 rows and 106 columns.
}
\source{
Haza B, Gosling CJ, Conty L & Pinabiaux C (2024). Social Functioning in Children and Adolescents with ADHD: A Meta-analysis. Journal of Child Psychology and Psychiatry and Allied Disciplines.
}
\usage{
df.haza
}
\description{
Dataset of a meta-analysis exploring the specificity of social functioning of children with ADHD (compared to healthy controls) in case-control studies.
This dataset contains:
1. several information coming from the same participants (due to the completion of multiple outcomes).
1. several information coming from the same study (due to the presence of multiple subgroups).
1. overlapping information for the same comparison
1. several information types from which a standardized mean difference can be estimated/converted
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/functions_summary.R
\name{print.metaConvert}
\alias{print.metaConvert}
\title{Print a summary of an object of class \dQuote{metaConvert}}
\usage{
\method{print}{metaConvert}(x, ...)
}
\arguments{
\item{x}{an object of class \dQuote{metaConvert}}

\item{...}{other arguments that can be passed to the function}
}
\value{
Implicitly calls the \code{\link{summary.metaConvert}} function.
}
\description{
Print a summary of an object of class \dQuote{metaConvert}
}
\details{
Summary method for objects of class \dQuote{metaConvert}.
}
\examples{
### print the results of an object of class metaConvert
convert_df(df.haza, measure = "g")
}
\seealso{
\code{\link{summary.metaConvert}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANOVA_med_range_quart.R
\name{es_from_med_min_max}
\alias{es_from_med_min_max}
\title{Convert median, quartiles, and range of two independent groups into several effect size measures}
\usage{
es_from_med_min_max(
  min_exp,
  med_exp,
  max_exp,
  n_exp,
  min_nexp,
  med_nexp,
  max_nexp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_med
)
}
\arguments{
\item{min_exp}{minimum value of the experimental/exposed group.}

\item{med_exp}{median value of the experimental/exposed group.}

\item{max_exp}{maximum value of the experimental/exposed group.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{min_nexp}{minimum value of the non-experimental/non-exposed group.}

\item{med_nexp}{median value of the non-experimental/non-exposed group.}

\item{max_nexp}{maximum value of the non-experimental/non-exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the generated \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_med}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab \cr
\tab \cr
\code{converted effect size measure} \tab MD + D + G\cr
\tab OR + R + Z \cr
\code{required input data} \tab See 'Section 12. Median, range and/or interquartile range'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}

This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab \cr
\tab \cr
\code{converted effect size measure} \tab MD + D + G\cr
\tab OR + R + Z \cr
\code{required input data} \tab See 'Section 12. Median, range and/or interquartile range'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert median, quartiles, and range of two independent groups into several effect size measures
}
\details{
This function first converts a Cohen's d (D), Hedges' g (G)
and mean difference (MD) from the medians and ranges of two independent groups.
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{This function recreates means+SD of the two groups}  (Wan et al., 2014):
\deqn{mean\_exp = \frac{min\_exp + 2*med\_exp + max\_exp}{4} + \frac{min\_exp - 2*med\_exp + max\_exp}{4*n\_exp}}
\deqn{mean\_nexp = \frac{min\_nexp + 2*med\_nexp + max\_nexp}{4} + \frac{min\_nexp - 2*med\_nexp + max\_nexp}{4*n\_nexp}}
\deqn{mean\_sd\_exp = \frac{max\_exp - min\_exp}{2*qnorm((n\_exp-0.375) / (n\_exp+0.25))}}
\deqn{mean\_sd\_nexp = \frac{max\_nexp - min\_nexp}{2*qnorm((n\_nexp-0.375) / (n\_nexp+0.25))}}

Note that if the group sample size is inferior to 50, a correction is applied to estimate the standard deviation.

\strong{From these means+SD, the function computes MD, D and G} using formulas
described in \code{\link{es_from_means_sd}()}.

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.

\strong{Importantly,}, authors of the Cochrane Handbook stated
"As a general rule, we recommend that ranges should not be used
to estimate SDs." (see section 6.5.2.6).
It is thus a good practice to explore the consequences of
the use of this conversion in sensitivity analyses.
}
\examples{
es_from_med_min_max(
  min_exp = 1335, med_exp = 1400,
  max_nexp = 1765, n_exp = 40,
  min_nexp = 1481, med_nexp = 1625,
  max_exp = 1800, n_nexp = 40
)
}
\references{
Wan, X., Wang, W., Liu, J. et al. Estimating the sample mean and standard deviation from the sample size, median, range and/or interquartile range. BMC Med Res Methodol 14, 135 (2014). https://doi.org/10.1186/1471-2288-14-135

Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_means.R
\name{es_from_ancova_means_ci}
\alias{es_from_ancova_means_ci}
\title{Convert means and 95\% CIs of two independent groups obtained from an ANCOVA model into several effect size measures}
\usage{
es_from_ancova_means_ci(
  n_exp,
  n_nexp,
  ancova_mean_exp,
  ancova_mean_ci_lo_exp,
  ancova_mean_ci_up_exp,
  ancova_mean_nexp,
  ancova_mean_ci_lo_nexp,
  ancova_mean_ci_up_nexp,
  cov_outcome_r,
  n_cov_ancova,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_means
)
}
\arguments{
\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{ancova_mean_exp}{adjusted mean of participants in the experimental/exposed group.}

\item{ancova_mean_ci_lo_exp}{lower bound of the adjusted 95\% CI of the mean of the experimental/exposed group}

\item{ancova_mean_ci_up_exp}{upper bound of the adjusted 95\% CI of the mean of the experimental/exposed group}

\item{ancova_mean_nexp}{adjusted mean of participants in the non-experimental/non-exposed group.}

\item{ancova_mean_ci_lo_nexp}{lower bound of the adjusted 95\% CI of the mean of the non-experimental/non-exposed group.}

\item{ancova_mean_ci_up_nexp}{upper bound of the adjusted 95\% CI of the mean of the non-experimental/non-exposed group.}

\item{cov_outcome_r}{correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{smd_to_cor}{formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_means}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert means and 95\% CIs of two independent groups obtained from an ANCOVA model into several effect size measures
}
\details{
This function converts the adjusted means 95\% CI of two independent groups
into a standard error,
and then relies on the calculations of the \code{\link{es_from_ancova_means_se}()} function.

\strong{To convert the 95\% CIs into standard errors,} the following formula is used (table 12.3 in Cooper):
\deqn{ancova\_mean\_se\_exp = \frac{ancova\_mean\_ci\_up\_exp - ancova\_mean\_ci\_lo\_exp}{2 * qt(0.975, df = n\_exp + n\_nexp - 2 - n\_cov\_ancova)}}
\deqn{ancova\_mean\_se\_nexp = \frac{ancova\_mean\_ci\_up\_nexp - ancova\_mean\_ci\_lo\_nexp}{2 * qt(0.975, df = n\_exp + n\_nexp - 2 - n\_cov\_ancova)}}
Calculations of the \code{\link{es_from_ancova_means_se}()} are then applied.
}
\examples{
es_from_ancova_means_ci(
  n_exp = 55, n_nexp = 55, cov_outcome_r = 0.5, n_cov_ancova = 4,
  ancova_mean_exp = 25, ancova_mean_ci_lo_exp = 15, ancova_mean_ci_up_exp = 35,
  ancova_mean_nexp = 18, ancova_mean_ci_lo_nexp = 12, ancova_mean_ci_up_nexp = 24
)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_RR.R
\name{es_from_rr_pval}
\alias{es_from_rr_pval}
\title{Convert a risk ratio value and its p-value to various effect size measures}
\usage{
es_from_rr_pval(
  rr,
  logrr,
  rr_pval,
  baseline_risk,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  rr_to_or = "metaumbrella",
  reverse_rr_pval
)
}
\arguments{
\item{rr}{risk ratio value}

\item{logrr}{log risk ratio value}

\item{rr_pval}{p-value of the risk ratio}

\item{baseline_risk}{proportion of cases in the non-exposed group (only required for the \code{rr_to_or = "grant_CI"} and \code{rr_to_or = "grant_2x2"} arguments).}

\item{n_exp}{number of participants in the exposed group (only required for the \code{rr_to_or = "grant_CI"}, \code{rr_to_or = "grant_2x2"} arguments).}

\item{n_nexp}{number of participants in the non-exposed group (only required for the \code{rr_to_or = "grant_CI"}, \code{rr_to_or = "grant_2x2"} arguments).}

\item{n_cases}{number of cases/events}

\item{n_controls}{number of controls/no-event}

\item{rr_to_or}{formula used to convert the \code{rr} value into an odds ratio (see details).}

\item{reverse_rr_pval}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab RR\cr
\tab \cr
\code{converted effect size measure} \tab OR + NNT\cr
\tab \cr
\code{required input data} \tab See 'Section 3. Risk Ratio'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a risk ratio value and its p-value to various effect size measures
}
\details{
This function uses the p-value of the (log) risk ratio to obtain the standard error (Section 6.3.2 in the Cochrane Handbook).
\deqn{logrr\_z = qnorm(rr_pval/2, lower.tail=FALSE)}
\deqn{logrr\_se = |\frac{\log(rr)}{logrr\_z}|}

Then, calculations of \code{\link{es_from_rr_se}} are applied.
}
\examples{
es_rr <- es_from_rr_pval(
  rr = 3.51, rr_pval = 0.001,
  n_cases = 12, n_controls = 68
)
}
\references{
Higgins, J. P., Thomas, J., Chandler, J., Cumpston, M., Li, T., Page, M. J., & Welch, V. A. (Eds.). (2019). Cochrane handbook for systematic reviews of interventions. John Wiley & Sons.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_MD.R
\name{es_from_ancova_md_sd}
\alias{es_from_ancova_md_sd}
\title{Convert an adjusted mean difference and adjusted standard deviation between two independent groups obtained from an ANCOVA model into several effect size measures}
\usage{
es_from_ancova_md_sd(
  ancova_md,
  ancova_md_sd,
  cov_outcome_r,
  n_cov_ancova,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_md
)
}
\arguments{
\item{ancova_md}{adjusted mean difference between two independent groups}

\item{ancova_md_sd}{covariate-adjusted pooled within-group standard deviation}

\item{cov_outcome_r}{correlation between the outcome and covariate (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_md}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 20. Adjusted: Mean difference and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an adjusted mean difference and adjusted standard deviation between two independent groups obtained from an ANCOVA model into several effect size measures
}
\details{
This function first computes an "adjusted" Cohen's d (D), Hedges' g (G)
from the adjusted mean difference (MD).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate the SE of the adjusted MD}:
\deqn{md\_se = ancova\_md\_sd * \sqrt{\frac{1}{n\_exp} + \frac{1}{n\_nexp}}}
\deqn{md\_lo = md - md\_se * qt(.975, n\_exp + n\_nexp-2-n\_cov\_ancova)}
\deqn{md\_up = md + md\_se * qt(.975, n\_exp + n\_nexp-2-n\_cov\_ancova)}

\strong{To recover the unadjusted pooled SD} (for Cohen's d, table 12.3 in Cooper):
\deqn{md\_sd = \frac{ancova\_md\_sd}{\sqrt{1 - cor\_outcome\_r^2}}}

\strong{To estimate the Cohen's d} (table 12.3 in Cooper):
\deqn{d = \frac{ancova\_md}{md\_sd}}

\strong{To estimate other effect size measures},
Calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
}
\note{
The standardized effect size variance assumes the covariate is balanced across
groups (Cooper eq. 12.26) and treats \code{cov_outcome_r} as known; it omits the
covariate-imbalance ("leverage") term of the exact ANCOVA variance, which cannot
be recovered from summary statistics. In observational or otherwise
covariate-imbalanced designs the standard error is therefore a lower bound
(anti-conservative), while the point estimate remains unbiased. See Lai and
Kelley (2012).
}
\examples{
es_from_ancova_md_sd(
  ancova_md = 4, ancova_md_sd = 2,
  cov_outcome_r = 0.5, n_cov_ancova = 5,
  n_exp = 20, n_nexp = 22
)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Lai, K., & Kelley, K. (2012). Accuracy in parameter estimation for ANCOVA and ANOVA contrasts: Sample size planning via narrow confidence intervals. British Journal of Mathematical and Statistical Psychology, 65(2), 350-370.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PROP.R
\name{es_from_prop_single_group_counts}
\alias{es_from_prop_single_group_counts}
\title{Convert single-group case counts into a proportion effect size measure}
\usage{
es_from_prop_single_group_counts(
  n_cases,
  n_sample,
  prop_to_es = "raw",
  reverse_prop
)
}
\arguments{
\item{n_cases}{number of cases in the single group}

\item{n_sample}{total sample size}

\item{prop_to_es}{method used to compute the effect size from the proportion.
Must be \code{"raw"} (default), \code{"logit"}, or \code{"freeman_tukey"}.}

\item{reverse_prop}{a logical value indicating whether the direction of the proportion should be flipped.}
}
\value{
This function returns the same output as \code{\link{es_from_prop_single_group}()}.
See that function's documentation for details.
}
\description{
Convert single-group case counts into a proportion effect size measure
}
\details{
This is a convenience function that converts case counts to proportions
and then calls \code{\link{es_from_prop_single_group}()}.

The proportion is calculated as:
\deqn{prop = \frac{n\_cases}{n\_sample}}

Then, calculations of the \code{\link{es_from_prop_single_group}()} are applied.

Rows where \code{n_cases > n_sample} are set to NA (both values) with a
warning; the affected rows return NA, but do not abort the run.
}
\examples{
es_from_prop_single_group_counts(n_cases = 25, n_sample = 100)
}
\references{
Barendregt, J. J., Doi, S. A., Lee, Y. Y., Norman, R. E., & Vos, T. (2013).
Meta-analysis of prevalence. Journal of Epidemiology and Community Health, 67(11), 974-978.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PAIRED_SINGLE_GROUP.R
\name{es_from_means_sd_pre_post_single_group}
\alias{es_from_means_sd_pre_post_single_group}
\title{Convert pre-post means of a single group into standard effect size measures}
\usage{
es_from_means_sd_pre_post_single_group(
  mean_pre_exp,
  mean_exp,
  mean_pre_sd_exp,
  mean_sd_exp,
  n_exp,
  r_pre_post_exp = 0.8,
  pre_post_to_smd = "bonett",
  smd_to_cor = "viechtbauer",
  reverse_means_pre_post
)
}
\arguments{
\item{mean_pre_exp}{mean of the group at baseline}

\item{mean_exp}{mean of the group at follow up}

\item{mean_pre_sd_exp}{standard deviation of the group at baseline}

\item{mean_sd_exp}{standard deviation of the group at follow up}

\item{n_exp}{number of participants in the group}

\item{r_pre_post_exp}{pre-post correlation within the group}

\item{pre_post_to_smd}{formula used to convert the pre and post means/SD into a SMD (see details).}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{reverse_means_pre_post}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MDw + Dw + Gw\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert pre-post means of a single group into standard effect size measures
}
\details{
This function converts pre-post means of a single group into within-group Cohen's d (dw), Hedges' g (gw), and mean difference (mdw).
These within-group effect sizes quantify the standardized change from baseline to follow-up within one group.
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from Cohen's d.

Four formulas can be used to convert the pre/post means into a SMD (\code{pre_post_to_smd} argument):
\enumerate{
\item \code{"bonett"}: baseline SD standardizer (Bonett, 2008; equivalent to the SMCRH measure in metafor)
\deqn{d\_w = \frac{mean\_post\_exp - mean\_pre\_exp}{mean\_pre\_sd\_exp}}
\deqn{var(g\_w) = \frac{sd\_change^2}{sd\_pre^2(n-1)} + \frac{g\_w^2}{2(n-1)}}
where \deqn{sd\_change = \sqrt{sd\_pre^2 + sd\_post^2 - 2r\,sd\_pre\,sd\_post}}
\item \code{"cooper"} (alias: "morris_drm"): raw score standardizer (Cooper 2019, Morris & DeShon 2002)
\deqn{d\_rm = \frac{mean\_post\_exp - mean\_pre\_exp}{sd\_change} * \sqrt{2 * (1 - r\_pre\_post\_exp)}}
\deqn{var(d\_rm) = \frac{2 * (1 - r\_pre\_post\_exp)}{n\_exp} + \frac{d\_rm^2}{2 * n\_exp}}
\item \code{"morris_dz"}: change score standardizer (Morris & DeShon, 2002; equivalent to the SMCC measure in metafor)
\deqn{d\_z = \frac{mean\_post\_exp - mean\_pre\_exp}{sd\_change}}
\deqn{var(g\_z) = \frac{1}{n} + \frac{g\_z^2}{2n}}
\item \code{"morris_dav"}: average SD standardizer (Bonett, 2008; equivalent to the SMCRP measure in metafor)
\deqn{d\_av = \frac{mean\_post\_exp - mean\_pre\_exp}{\sqrt{(mean\_pre\_sd\_exp^2 + mean\_post\_sd\_exp^2)/2}}}
\deqn{g\_av = d\_av \times J(mi), \quad mi = \frac{2(n\_exp - 1)}{1 + r\_pre\_post\_exp^2}}
\deqn{var(g\_av) = J^2 \left[\frac{2(1 - r)}{n} + \frac{d^2(1 + r^2)}{4n}\right]}
}

The within-group Hedges' g is obtained by applying a bias correction factor J to d:
\deqn{g\_w = d\_w * J(n\_exp-1)}

The within-group mean difference is simply:
\deqn{md\_w = mean\_post\_exp - mean\_pre\_exp}
\deqn{var(md\_w) = \frac{sd\_change^2}{n\_exp}}

To estimate other effect size measures,
calculations of the \code{\link{es_from_cohen_d}()} are applied, treating the single group as a matched design.
}
\examples{
es_from_means_sd_pre_post_single_group(
  n_exp = 36,
  mean_pre_exp = 98, mean_exp = 102,
  mean_pre_sd_exp = 16, mean_sd_exp = 17,
  r_pre_post_exp = 0.8
)
}
\references{
Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.

Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386. https://doi.org/10.1177/1094428106291059

Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PAIRED_SINGLE_GROUP.R
\name{es_from_means_ci_pre_post_single_group}
\alias{es_from_means_ci_pre_post_single_group}
\title{Convert pre-post means and 95\% CI of a single group into standard effect size measures}
\usage{
es_from_means_ci_pre_post_single_group(
  mean_pre_exp,
  mean_exp,
  mean_pre_ci_lo_exp,
  mean_pre_ci_up_exp,
  mean_ci_lo_exp,
  mean_ci_up_exp,
  n_exp,
  r_pre_post_exp = 0.8,
  pre_post_to_smd = "bonett",
  smd_to_cor = "viechtbauer",
  max_asymmetry = 10,
  reverse_means_pre_post
)
}
\arguments{
\item{mean_pre_exp}{mean of the group at baseline}

\item{mean_exp}{mean of the group at follow up}

\item{mean_pre_ci_lo_exp}{lower bound of the 95\% CI of the mean at baseline}

\item{mean_pre_ci_up_exp}{upper bound of the 95\% CI of the mean at baseline}

\item{mean_ci_lo_exp}{lower bound of the 95\% CI of the mean at follow up}

\item{mean_ci_up_exp}{upper bound of the 95\% CI of the mean at follow up}

\item{n_exp}{number of participants in the group}

\item{r_pre_post_exp}{pre-post correlation within the group}

\item{pre_post_to_smd}{formula used to convert the pre and post means/SD into a SMD (see details).}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{max_asymmetry}{percentage indicating the tolerance before detecting asymmetry in the 95\% CI bounds.}

\item{reverse_means_pre_post}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MDw + Dw + Gw\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert pre-post means and 95\% CI of a single group into standard effect size measures
}
\details{
This function converts the bounds of the 95\% CI of the pre/post means of a single group into standard errors (Section 6.3.1 in the Cochrane Handbook).
\deqn{mean\_pre\_se\_exp = \frac{mean\_pre\_ci\_up\_exp - mean\_pre\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
\deqn{mean\_post\_se\_exp = \frac{mean\_post\_ci\_up\_exp - mean\_post\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}

Then, calculations of the \code{\link{es_from_means_se_pre_post_single_group}()} are applied.
}
\examples{
es_from_means_ci_pre_post_single_group(
  n_exp = 36,
  mean_pre_exp = 98,
  mean_pre_ci_lo_exp = 88, mean_pre_ci_up_exp = 108,
  mean_exp = 102,
  mean_ci_lo_exp = 92, mean_ci_up_exp = 112,
  r_pre_post_exp = 0.8
)
}
\references{
Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022.

Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.

Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386.

Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PAIRED_STATISTICS.R
\name{es_from_paired_t}
\alias{es_from_paired_t}
\title{Convert two paired t-test value of two independent groups into several effect size measures}
\usage{
es_from_paired_t(
  paired_t_exp,
  paired_t_nexp,
  n_exp,
  n_nexp,
  r_pre_post_exp,
  r_pre_post_nexp,
  smd_to_cor = "viechtbauer",
  pre_post_to_smd = "cooper",
  reverse_paired_t
)
}
\arguments{
\item{paired_t_exp}{Paired t-test value of the experimental/exposed group.}

\item{paired_t_nexp}{Paired t-test value of the non-experimental/non-exposed group.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{r_pre_post_exp}{pre-post correlation in the experimental/exposed group}

\item{r_pre_post_nexp}{pre-post correlation in the non-experimental/non-exposed group}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{pre_post_to_smd}{formula used to convert the paired t-test value into a SMD (see details).}

\item{reverse_paired_t}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 16. Paired: Paired F- or t-test'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert two paired t-test value of two independent groups into several effect size measures
}
\details{
This function converts paired t-tests of two independent groups value into a Cohen's d (D) and Hedges' g (G) (table 12.2 in Cooper).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate the Cohen's d,} the following formulas are used (Cooper et al., 2019):
\deqn{cohen\_d\_exp = paired\_t\_exp * \sqrt{\frac{2 * (1 - r\_pre\_post\_exp)}{n\_exp}}}
\deqn{cohen\_d\_nexp = paired\_t\_nexp * \sqrt{\frac{2 * (1 - r\_pre\_post\_nexp)}{n\_nexp}}}
\deqn{cohen\_d\_se\_exp = \sqrt{\frac{2 * (1 - r\_pre\_post\_exp)}{n\_exp} + \frac{d\_exp^2}{2 * n\_exp}}}
\deqn{cohen\_d\_se\_nexp = \sqrt{\frac{2 * (1 - r\_pre\_post\_nexp)}{n\_nexp} + \frac{d\_nexp^2}{2 * n\_nexp}}}
\deqn{cohen\_d = d\_exp - d\_nexp}
\deqn{d\_se = \sqrt{cohen\_d\_se\_exp^2 + cohen\_d\_se\_nexp^2}}

When \code{pre_post_to_smd = "morris_dz"}, the mean difference is standardized by the standard deviation
of the change score and the pre-post correlation is no longer involved (Morris & DeShon, 2002):
\deqn{cohen\_d\_exp = \frac{paired\_t\_exp}{\sqrt{n\_exp}}}
\deqn{cohen\_d\_nexp = \frac{paired\_t\_nexp}{\sqrt{n\_nexp}}}

Note that the Cohen's d obtained from a paired t-test strongly depends on the pre-post correlation.
When \code{r_pre_post_exp} / \code{r_pre_post_nexp} are not indicated, a value of 0.8 is assumed and
users should conduct sensitivity analyses with other plausible values.

\strong{No \code{pool_sd} argument.} Unlike the means and mean-change converters, this function cannot
pool the standardizing SD across arms: a paired t-statistic identifies each arm's
\eqn{mean\_change / sd\_change} ratio but NOT the ratio of the two arms' SDs, so the pooled
standardizer is not recoverable from the reported statistic. Each arm is therefore standardized by
its own SD and the two within-group values are subtracted -- a construction that coincides with the
pooled one only when the arms' SDs are equal. When rows from this route are combined in one pool
with rows that DO use a pooled standardizer, \code{summary(..., flags = TRUE)} raises an
informational flag. If the arm SDs are reported, prefer \code{\link{es_from_means_sd_pre_post}} or
\code{\link{es_from_mean_change_sd}}.

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_paired_t(paired_t_exp = 2.1, paired_t_nexp = 4.2, n_exp = 20, n_nexp = 22)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_COR.R
\name{es_from_pearson_r}
\alias{es_from_pearson_r}
\title{Convert a Pearson's correlation coefficient to several effect size measures}
\usage{
es_from_pearson_r(
  pearson_r,
  sd_iv,
  n_sample,
  n_exp,
  n_nexp,
  cor_to_smd = "viechtbauer",
  unit_increase_iv,
  unit_type = "raw_scale",
  reverse_pearson_r
)
}
\arguments{
\item{pearson_r}{a Pearson's correlation coefficient value}

\item{sd_iv}{the standard deviation of the independent variable}

\item{n_sample}{the total number of participants}

\item{n_exp}{number of the experimental/exposed group}

\item{n_nexp}{number of the non-experimental/non-exposed group}

\item{cor_to_smd}{formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.}

\item{unit_increase_iv}{a value of the independent variable that will be used to estimate the Cohen's d (see details).}

\item{unit_type}{the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "value"}

\item{reverse_pearson_r}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab R + Z\cr
\tab \cr
\code{converted effect size measure} \tab D + G + OR\cr
\tab \cr
\code{required input data} \tab See 'Section 4. Pearson's r or Fisher's z'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a Pearson's correlation coefficient to several effect size measures
}
\details{
This function estimates the variance of a Pearson's correlation coefficient, and computes the Fisher's r-to-z
transformation.
Cohen's d (D), Hedges' g (G) are converted from the Pearson's r, and odds ratio (OR)
are converted from the Cohen's d.
\enumerate{
\item \strong{The formula used to estimate the standard error of the Pearson's correlation coefficient} and 95\% CI
are (Formula 12.27 in Cooper):
\deqn{R\_se = \sqrt{\frac{(1 - pearson\_r^2)^2}{n\_sample - 1}}}
\deqn{R\_lo = pearson\_r - qt(.975, n\_sample - 2) * R\_se}
\deqn{R\_up = pearson\_r + qt(.975, n\_sample - 2) * R\_se}
\item \strong{The formula used to estimate the Fisher's z} are (Formula 12.28 & 12.29 in Cooper):
\deqn{Z = atanh(r)}
\deqn{Z\_se = \sqrt{\frac{1}{n\_sample - 3}}}
\deqn{Z\_ci\_lo = Z - qnorm(.975) * Z\_se}
\deqn{Z\_ci\_up = Z + qnorm(.975) * Z\_se}
\item Several approaches can be used to convert a correlation coefficient to a SMD.
}

\strong{A.} Mathur proposes to use this formula (Formula 1.2 in Mathur, \code{cor_to_smd = "mathur"}):
\deqn{increase = ifelse(unit_type == "sd", unit\_increase\_iv * sd\_iv, unit\_increase\_iv)}
\deqn{d = \frac{r * increase}{sd_iv * \sqrt{1 - r^2}}}
\deqn{d\_se = abs(d) * \sqrt{\frac{1}{r^2 * (n\_sample - 3)} + \frac{1}{2*(n\_sample - 1))}}}
The resulting Cohen's d is the average increase in the dependent variable associated with an increase of x units in the independent variable (with x = \code{unit_increase_iv}).

\strong{B.} Viechtbauer proposes to use the delta method to derive a Cohen's d from a correlation coefficient (Viechtbauer, 2023, \code{cor_to_smd = "viechtbauer"})

\strong{C.} Cooper converts a point-biserial correlation to Cohen's d with the fixed
per-2-SD transformation (Formula 12.38 & 12.39 in Cooper, \code{cor_to_smd = "cooper"}).
Unlike the \code{"mathur"} option, this branch does NOT use \code{unit_increase_iv},
\code{unit_type} or \code{sd_iv}; it always returns the standardized mean difference
between two groups lying one predictor standard deviation apart:
\deqn{d = \frac{2 * r}{\sqrt{1 - r^2}}}
\deqn{d\_se = \sqrt{\frac{4 * R\_se^2}{(1 - r^2)^3}}}
where \eqn{R\_se} is the standard error of the correlation. It therefore coincides
with the \code{cor_to_smd = "mathur"} result only in the special case
\code{unit_type = "sd"} and \code{unit_increase_iv = 2}.

To know how the Cohen's d value is converted to other effect measures (G/OR), see details of the \code{\link{es_from_cohen_d}} function.
}
\examples{
es_from_pearson_r(
  pearson_r = .51, sd_iv = 0.24, n_sample = 214,
  unit_increase_iv = 1, unit_type = "sd"
)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Mathur, M. B., & VanderWeele, T. J. (2020). A Simple, Interpretable Conversion from Pearson's Correlation to Cohen's for d Continuous Exposures. Epidemiology (Cambridge, Mass.), 31(2), e16-e18. https://doi.org/10.1097/EDE.0000000000001105

Viechtbauer W (2010). "Conducting meta-analyses in R with the metafor package." Journal of Statistical Software, 36(3), 1-48. doi:10.18637/jss.v036.i03.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PAIRED_MEANS.R
\name{es_from_means_sd_pre_post}
\alias{es_from_means_sd_pre_post}
\title{Convert pre-post means of two independent groups into various effect size measures}
\usage{
es_from_means_sd_pre_post(
  mean_pre_exp,
  mean_exp,
  mean_pre_sd_exp,
  mean_sd_exp,
  mean_pre_nexp,
  mean_nexp,
  mean_pre_sd_nexp,
  mean_sd_nexp,
  n_exp,
  n_nexp,
  r_pre_post_exp,
  r_pre_post_nexp,
  smd_to_cor = "viechtbauer",
  pre_post_to_smd = "bonett",
  pool_sd = FALSE,
  reverse_means_pre_post
)
}
\arguments{
\item{mean_pre_exp}{mean of the experimental/exposed group at baseline}

\item{mean_exp}{mean of the experimental/exposed group at follow up}

\item{mean_pre_sd_exp}{standard deviation of the experimental/exposed group at baseline}

\item{mean_sd_exp}{standard deviation of the experimental/exposed group at follow up}

\item{mean_pre_nexp}{mean of the non-experimental/non-exposed group at baseline}

\item{mean_nexp}{mean of the non-experimental/non-exposed group at follow up}

\item{mean_pre_sd_nexp}{standard deviation of the non-experimental/non-exposed group at baseline}

\item{mean_sd_nexp}{standard deviation of the non-experimental/non-exposed group at follow up}

\item{n_exp}{number of the experimental/exposed group}

\item{n_nexp}{number of the non-experimental/non-exposed group}

\item{r_pre_post_exp}{pre-post correlation in the experimental/exposed group}

\item{r_pre_post_nexp}{pre-post correlation in the non-experimental/non-exposed group}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{pre_post_to_smd}{formula used to convert the pre and post means/SD into a SMD (see details).}

\item{pool_sd}{a logical value indicating whether the standardizing SD should be pooled across the two
groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
which to prefer, so this is a deliberate choice and not a technical detail.
\itemize{
\item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
within-group values are subtracted, their variances adding because the arms are independent. This
is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
broadly applicable of the two.
\item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
assumes the two arms' true standardizing SDs are equal.
}}

\item{reverse_means_pre_post}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert pre-post means of two independent groups into various effect size measures
}
\details{
This function converts pre-post means of two independent groups into a Cohen's d (D) and Hedges' g (G).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{Four approaches can be used to compute the Cohen's d.}

In these approaches, the standard deviation of the difference within each group first needs to be obtained:
\deqn{adj\_exp = 2*r\_pre\_post\_exp*mean\_pre\_sd\_exp*mean\_sd\_exp}
\deqn{sd\_change\_exp = \sqrt{mean\_pre\_sd\_exp^2 + mean\_sd\_exp^2 - adj\_exp}}
\deqn{adj\_nexp = 2*r\_pre\_post\_nexp*mean\_pre\_sd\_nexp*mean\_sd\_nexp}
\deqn{sd\_change\_nexp = \sqrt{mean\_pre\_sd\_nexp^2 + mean\_sd\_nexp^2 - adj\_nexp}}
\enumerate{
\item In the approach described by Bonett (\code{pre_post_to_smd = "bonett"}), one Cohen's d per group is obtained by
standardizing the pre-post mean difference by the standard deviation at baseline (Bonett, 2008):
\deqn{cohen\_d\_exp = \frac{mean\_exp - mean\_pre\_exp}{mean\_pre\_sd\_exp}}
\deqn{cohen\_d\_nexp = \frac{mean\_nexp - mean\_pre\_nexp}{mean\_pre\_sd\_nexp}}
\item In the approach described by Cooper (\code{pre_post_to_smd = "cooper"} or \code{"morris_drm"}), the following
formulas are used (Cooper et al., 2019; Morris & DeShon, 2002):
\deqn{cohen\_d\_exp = \frac{mean\_exp - mean\_pre\_exp}{sd\_change\_exp} * \sqrt{2 * (1 - r\_pre\_post\_exp)}}
\deqn{cohen\_d\_nexp = \frac{mean\_nexp - mean\_pre\_nexp}{sd\_change\_nexp} * \sqrt{2 * (1 - r\_pre\_post\_nexp)}}
\item In the approach described by Morris & DeShon (\code{pre_post_to_smd = "morris_dz"}), the pre-post mean difference
is standardized by the standard deviation of the change (Morris & DeShon, 2002):
\deqn{cohen\_d\_exp = \frac{mean\_exp - mean\_pre\_exp}{sd\_change\_exp}}
\deqn{cohen\_d\_nexp = \frac{mean\_nexp - mean\_pre\_nexp}{sd\_change\_nexp}}
\item In the approach described by Morris (\code{pre_post_to_smd = "morris_dav"}), the pre-post mean
difference is standardized by the average of the baseline and follow up SDs (Morris, 2008). Note that
Morris (2008, p.384) recommended \emph{against} this standardizer because its sampling variance was
unknown to him; the variance used here is the heteroscedasticity-robust form of Bonett (2008, eq. 10/19):
\deqn{cohen\_d\_exp = \frac{mean\_exp - mean\_pre\_exp}{\sqrt{(mean\_pre\_sd\_exp^2 + mean\_sd\_exp^2)/2}}}
\deqn{cohen\_d\_nexp = \frac{mean\_nexp - mean\_pre\_nexp}{\sqrt{(mean\_pre\_sd\_nexp^2 + mean\_sd\_nexp^2)/2}}}
}

Last, the Cohen's d reflecting the within-group change from baseline to follow-up are combined into one Cohen's d:
\deqn{cohen\_d = d\_exp - d\_nexp}
\deqn{cohen\_d\_se = \sqrt{cohen\_d\_se\_exp^2 + cohen\_d\_se\_nexp^2}}

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_means_sd_pre_post(
  n_exp = 36, n_nexp = 35,
  mean_pre_exp = 98, mean_exp = 102,
  mean_pre_sd_exp = 16, mean_sd_exp = 17,
  mean_pre_nexp = 96, mean_nexp = 102,
  mean_pre_sd_nexp = 14, mean_sd_nexp = 15,
  r_pre_post_exp = 0.8, r_pre_post_nexp = 0.8
)
}
\references{
Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.

Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386. https://doi.org/10.1177/1094428106291059

Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125. https://doi.org/10.1037/1082-989X.7.1.105
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_extraction.R
\name{data_extraction_sheet}
\alias{data_extraction_sheet}
\title{Data extraction sheet generator}
\usage{
data_extraction_sheet(
  measure = c("d", "g", "md", "dw", "gw", "mdw", "or", "rr", "nnt", "rd", "r", "z",
    "logvr", "logcvr", "irr", "prop"),
  type_of_measure = c("natural", "natural+converted"),
  name = "mcv_data_extraction",
  extension = c("data.frame", ".txt", ".csv", ".xlsx"),
  verbose = TRUE
)
}
\arguments{
\item{measure}{Target effect size measure (one of the 14 available in metaConvert). Default is "all".}

\item{type_of_measure}{One of "natural+converted" or "natural" (see details).}

\item{name}{Name of the file created}

\item{extension}{Extension of the file created. Most common are ".xlsx", ".csv" or ".txt". It is also possible to generate an R dataframe object by using the "data.frame" extension.}

\item{verbose}{logical variable indicating whether some information should be printed (e.g., the location where the sheet is created when using ".xlsx", ".csv" or ".txt" extensions)}
}
\value{
This function returns a data extraction sheet that contains all the information
necessary to estimate any effect size using the metaConvert tools.
}
\description{
Data extraction sheet generator
}
\details{
This function generates, on your computer, a data extraction sheet that contains the name of columns
that can be used by our tools to estimate various effect size measures.

If you select a specific measure (e.g., \code{measure = "g"}), you will be presented only with most common
information allowing to estimate this measure (e.g., you will not be provided with columns for contingency
tables if you request a data extraction sheet for \code{measure = "g"}).
\subsection{Measure}{

You can specify a specific effect size measures (among those available in the \code{\link{metaConvert-package}}).
Doing this, the data extraction sheet will contain only the columns of the input data allowing \strong{a natural estimation
of the effect size measure}. For example, if you request \code{measure="d"} the data extraction
sheet will not contain the columns for the contingency table since, although the \code{\link{convert_df}}
function allows you to \strong{convert} a contingency table into a "d", this requires to convert the "OR"
that is naturally estimated from the contingency table into a "d".

This table is designed to be used in combination with tables showing the combination of input data
leading to estimate each of the effect size measures (https://metaconvert.org/html/input.html)
}

\subsection{Extension}{

You can export a file in various formats outside R
(by indicating, for example, ".txt", ".xlsx", or ".csv") in the \code{extension} argument.
You can also visualise this dataset directly in R by setting \code{extension = "data.frame"}.
}
}
\examples{
data_extraction_sheet(measure = "md", extension = "data.frame")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_RD.R
\name{es_from_rd_pval}
\alias{es_from_rd_pval}
\title{Convert a risk difference value and its p-value to several effect size measures}
\usage{
es_from_rd_pval(
  rd,
  rd_pval,
  baseline_risk,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  n_sample,
  reverse_rd_pval
)
}
\arguments{
\item{rd}{risk difference value}

\item{rd_pval}{p-value of the risk difference}

\item{baseline_risk}{proportion of cases in the non-exposed/control group}

\item{n_exp}{number of participants in the exposed/treatment group}

\item{n_nexp}{number of participants in the non-exposed/control group}

\item{n_cases}{number of cases/events across both groups}

\item{n_controls}{number of controls/no-event across both groups}

\item{n_sample}{total number of participants in the sample}

\item{reverse_rd_pval}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab RD\cr
\tab \cr
\code{converted effect size measure} \tab OR + RR + NNT\cr
\tab \cr
\code{required input data} \tab rd + rd_pval\cr
\tab \cr
}
}
\description{
Convert a risk difference value and its p-value to several effect size measures
}
\details{
This function uses the p-value of the risk difference to obtain the standard error
(Section 6.3.2 in the Cochrane Handbook):
\deqn{z = qnorm(rd\_pval / 2, lower.tail = FALSE)}
\deqn{rd\_se = |\frac{rd}{z}|}

Then, calculations of \code{\link{es_from_rd_se}()} are applied.
}
\examples{
es_from_rd_pval(
  rd = 0.15, rd_pval = 0.01,
  baseline_risk = 0.30, n_exp = 100, n_nexp = 100
)
}
\references{
Higgins, J. P., Thomas, J., Chandler, J., Cumpston, M., Li, T., Page, M. J., & Welch, V. A. (Eds.). (2019). Cochrane handbook for systematic reviews of interventions. John Wiley & Sons.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_statistics.R
\name{es_from_ancova_f_pval}
\alias{es_from_ancova_f_pval}
\title{Convert a two-tailed p-value of an ANCOVA t-test into several effect size measures.}
\usage{
es_from_ancova_f_pval(
  ancova_f_pval,
  cov_outcome_r,
  n_cov_ancova,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_f_pval
)
}
\arguments{
\item{ancova_f_pval}{a two-tailed p-value of an F-test in an ANCOVA (binary predictor)}

\item{cov_outcome_r}{correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_f_pval}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 18. Adjusted: ANCOVA statistics, eta-squared'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a two-tailed p-value of an ANCOVA t-test into several effect size measures.
}
\details{
This function converts the p-value of an ANCOVA (binary predictor) into a t value,
and then relies on the calculations of the \code{\link{es_from_ancova_t}()} function.

\strong{To convert the p-value into a t-value,} the following formula is used (table 12.3 in Cooper):
\deqn{df = n\_exp + n\_nexp - 2 - n\_cov\_ancova}
\deqn{t = | qt(ancova\_f\_pval/2, df = df) |}
Then, calculations of the \code{\link{es_from_ancova_t}()} are applied.
}
\examples{
es_from_ancova_f_pval(
  ancova_f_pval = 0.05, cov_outcome_r = 0.2,
  n_cov_ancova = 3, n_exp = 20, n_nexp = 20
)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PHI_CHISQ.R
\name{es_from_chisq}
\alias{es_from_chisq}
\title{Convert a chi-square value to several effect size measures}
\usage{
es_from_chisq(
  chisq,
  n_sample,
  n_cases,
  n_exp,
  yates_chisq = FALSE,
  reverse_chisq
)
}
\arguments{
\item{chisq}{value of the chi-squared}

\item{n_sample}{total number of participants in the sample}

\item{n_cases}{total number of cases/events}

\item{n_exp}{total number of participants in the exposed group}

\item{yates_chisq}{logical value (or vector of length \code{length(chisq)})
indicating whether the chi-square has been performed using Yates' correction
for continuity. When a vector is supplied, each row is back-transformed using
its own setting. Missing values are treated as \code{FALSE}.}

\item{reverse_chisq}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab OR + RR + NNT\cr
\tab \cr
\code{converted effect size measure} \tab D + G + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 8. Phi or chi-square'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a chi-square value to several effect size measures
}
\details{
This function converts a chi-square value (with one degree of freedom)
into a phi coefficient (Lipsey et al. 2001):
\deqn{phi = \sqrt{\frac{chisq}{n\_sample}}}
and then converts it to other effect size measures exactly as in
\code{\link{es_from_phi}()} (including the correlation-based R/Z/D/G fallback with
standard sampling variances when the 2x2 table cannot be reconstructed).

Note that if \code{yates_chisq = TRUE}, the chi-square value is interpreted
as Yates-corrected when back-transforming to a 2x2 contingency table; this is
propagated row by row when a vector is supplied.

Then, the phi coefficient is converted to other effect size measures (see \code{\link{es_from_phi}}).
}
\examples{
es_from_chisq(chisq = 4.21, n_sample = 78, n_cases = 51, n_exp = 50)
}
\references{
Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANOVA_statistics.R
\name{es_from_anova_f}
\alias{es_from_anova_f}
\title{Convert a one-way independent ANOVA F-value to several effect size measures}
\usage{
es_from_anova_f(
  anova_f,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_anova_f
)
}
\arguments{
\item{anova_f}{ANOVA F-value (one-way, binary predictor).}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{anova_f} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_anova_f}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
\tab https://metaconvert.org/html/input.html\cr
\tab \cr
}
}
\description{
Convert a one-way independent ANOVA F-value to several effect size measures
}
\details{
This function converts the F-value (one-way, binary predictor) into a t-value,
and then relies on the calculations of the \code{\link{es_from_student_t}()} function.

\strong{To convert the F-value into a t-value,} the following formula is used (table 12.1 in Cooper):
\deqn{student\_t = \sqrt{anova\_f}}
Then, calculations of the \code{\link{es_from_student_t}()} are applied.

\strong{Important - single numerator degree of freedom only.} The identity
\eqn{\sqrt{F} = |t|} holds only when the F-test has a single numerator degree of freedom,
i.e. a one-way ANOVA comparing exactly two groups (a binary predictor). Supplying an
omnibus F from a factor with three or more levels (numerator df > 1) produces a
meaningless effect size and is not detected by the function. In addition, \eqn{\sqrt{F}}
discards the sign of the effect, so the generated effect sizes are always non-negative;
use \code{reverse_anova_f} to encode direction.
}
\examples{
es_from_anova_f(anova_f = 2.01, n_exp = 20, n_nexp = 22)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_OR.R
\name{es_from_or_se}
\alias{es_from_or_se}
\title{Convert an odds ratio value and its standard error into several effect size measures}
\usage{
es_from_or_se(
  or,
  logor,
  logor_se,
  baseline_risk,
  small_margin_prop,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  n_sample,
  or_to_rr = "metaumbrella_cases",
  or_to_cor = "pearson",
  reverse_or
)
}
\arguments{
\item{or}{odds ratio value}

\item{logor}{log odds ratio value}

\item{logor_se}{the standard error of the log odds ratio}

\item{baseline_risk}{proportion of cases in the non-exposed group}

\item{small_margin_prop}{smallest margin proportion of cases/events in the underlying 2x2 table}

\item{n_exp}{number of participants in the exposed group}

\item{n_nexp}{number of participants in the non-exposed group}

\item{n_cases}{number of cases/events across exposed/non-exposed groups}

\item{n_controls}{number of controls/no-event across exposed/non-exposed groups}

\item{n_sample}{total number of participants in the sample}

\item{or_to_rr}{formula used to convert the \code{or} value into a risk ratio (see details).}

\item{or_to_cor}{formula used to convert the \code{or} value into a correlation coefficient (see details).}

\item{reverse_or}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab OR\cr
\tab \cr
\code{converted effect size measure} \tab RR + NNT + RD\cr
\code{} \tab D + G + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 2. Odds Ratio'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert an odds ratio value and its standard error into several effect size measures
}
\details{
This function converts the log odds ratio into a Risk ratio (RR), Cohen's d (D), Hedges' g (G)
and correlation coefficients (R/Z).

\strong{To estimate the Cohen's d value and its standard error}
The following formulas are used (Cooper et al., 2019):
\deqn{d = \log(or) * \frac{\sqrt{3}}{\pi}}
\deqn{d\_se = \sqrt{\frac{logor\_se^2 * 3}{\pi^2}}}

\strong{To estimate the risk ratio and its standard error, various formulas can be used.}

\strong{A.} First, the approach described in Grant (2014) can be used.
However, in the paper, only the formula to convert an OR value to a RR value
is described.
To derive the variance, we used this formula to convert the bounds of the 95\% CI, which
were then used to obtain the variance.

This argument requires (or + baseline_risk + or_ci_lo + or_ci_up) to generate a RR.
The following formulas are used (br = baseline_risk):
\deqn{rr = \frac{or}{1 - br + br*or}}
\deqn{rr\_ci\_lo = \frac{or\_ci\_lo}{1 - br + br*or\_ci\_lo}}
\deqn{rr\_ci\_up = \frac{or\_ci\_up}{1 - br + br*or\_ci\_up}}
\deqn{logrr\_se = \frac{log(rr\_ci\_up) - log(rr\_ci\_lo)}{2 * qnorm(.975)}}

\strong{B.} Second, the formulas implemented in the metaumbrella package can be used
(\code{or_to_rr = "metaumbrella_cases"} or \code{or_to_rr = "metaumbrella_exp"}).
This argument requires (or + logor_se + n_cases + n_controls) or (or + logor_se + n_exp + n_nexp)
to generate a RR.
More precisely, when the OR value and its standard error, plus either
(i) the number of cases and controls or
(ii) the number of participants in the exposed and non-exposed groups,
are available, we previously developed functions that simulate all combinations of the possible
number of cases and controls
in the exposed and non-exposed groups compatible with the actual value of the OR.
Then, the functions select the contingency table whose standard error coincides best with
the standard error reported.
The RR value and its standard are obtained from this estimated contingency table.

\strong{C.} Third, it is possible to transpose the RR to a OR (\code{or_to_rr = "transpose"}).
This argument requires (or + logor_se) to generate a OR.
It is known that OR and RR are similar when the baseline risk is small.
Therefore, users can request to simply transpose the OR value & standard error into a RR value & standard error.
\deqn{rr = or}
\deqn{logrr\_se = logor\_se}

\strong{D.} Fourth, it is possible to recreate the 2x2 table using the dipietrantonj's formulas (\code{or_to_rr = "dipietrantonj"}).
This argument requires (or + logor_ci_lo + logor_ci_lo) to generate a RR. Information on this approach can be retrieved in
Di Pietrantonj (2006).

\strong{To estimate the NNT, the formulas used are :}
\deqn{treatment\_risk = \frac{or \times br}{1 - br + or \times br}}
\deqn{rd = br - treatment\_risk}
\deqn{nnt = \frac{1}{rd} = \frac{1 - br \times (1 - or)}{br \times (1 - br)}}

\strong{To estimate a correlation coefficient, various formulas can be used.}

\strong{A.} First, the approach described in Pearson (1900) can be used (\code{or_to_cor = "pearson"}).
This argument requires (or + logor_se) to generate a R/Z.
It converts the OR value and its standard error to a tetrachoric correlation.
Note that the formula assumes that each cell of the 2x2 used to estimate the OR has been added 1/2 before estimating the OR value and its standard error.
If it is not the case, formulas can produce slightly less accurate results.

\deqn{c = \frac{1}{2}}
\deqn{r = \cos{\frac{\pi}{1+or^c}}}
\deqn{r\_se = logor\_se * (\pi * c * or^c) * \frac{\sin(\pi / (1+or^c))}{(1+or^c)^2}}
\deqn{or\_ci\_lo = exp(log(or) - qnorm(.975)*logor\_se)}
\deqn{or\_ci\_up = exp(log(or) + qnorm(.975)*logor\_se)}
\deqn{r\_ci\_lo = cos(\frac{\pi}{1 + or\_ci\_lo^c})}
\deqn{r\_ci\_up = cos(\frac{\pi}{1 + or\_ci\_up^c})}
\deqn{z = atanh(r)}
\deqn{z\_se = \sqrt{\frac{r\_se^2}{(1 - r^2)^2}}}
\deqn{z\_ci\_lo = atanh(r\_lo)}
\deqn{z\_ci\_up = atanh(r\_up)}

\strong{B.} Second, the approach described in Digby (1983) can be used (\code{or_to_cor = "digby"}).
This argument requires (or + logor_se) to generate a R/Z.
It converts the OR value and its standard error to a tetrachoric correlation.
Note that the formula assumes that each cell of the 2x2 used to estimate the OR has been added 1/2 before estimating the OR value and its standard error.
If it is not the case, formulas can produce slightly less accurate results.

\deqn{c = \frac{3}{4}}
\deqn{r = \frac{or^c - 1}{or^c + 1}}
\deqn{r\_se = \sqrt{\frac{c^2}{4} * (1 - r^2)^2 * logor\_se^2}}
\deqn{z = atanh(r)}
\deqn{z\_se = \sqrt{\frac{r\_se^2}{(1 - r^2)^2}}}
\deqn{z\_ci\_lo = z - qnorm(.975)*\sqrt{\frac{c^2}{4} * logor\_se}}
\deqn{z\_ci\_up = z + qnorm(.975)*\sqrt{\frac{c^2}{4} * logor\_se}}
\deqn{r\_ci\_lo = tanh(z\_lo)}
\deqn{r\_ci\_up = tanh(z\_up)}

\strong{C.} Third, the approach described in Bonett (2005) can be used (\code{or_to_cor = "bonett"}).
This argument requires (or + logor_se + n_cases + n_exp + small_margin_prop) to generate a R/Z.
Note that the formula assumes that each cell of the 2x2 used to estimate the OR has been added 1/2 before estimating the OR value and its standard error.
If it is not the case, formulas can produce slightly less accurate results.

\deqn{c = \frac{\frac{1 - |n\_exp - n\_cases|}{5} - (0.5 - small\_margin\_prop)^2}{2}}
\deqn{r = \cos{\frac{\pi}{1+or^c}}}
\deqn{r\_se = logor\_se * (\pi * c * or^c) * \frac{\sin(\frac{\pi}{1+or^c})}{(1+or^c)^2}}
\deqn{or\_ci\_lo = exp(log(or) - qnorm(.975)*logor\_se)}
\deqn{or\_ci\_up = exp(log(or) + qnorm(.975)*logor\_se)}
\deqn{r\_ci\_lo = cos(\frac{\pi}{1 + or\_ci\_lo^c})}
\deqn{r\_ci\_up = cos(\frac{\pi}{1 + or\_ci\_up^c})}
\deqn{z = atanh(r)}
\deqn{z\_se = \sqrt{\frac{r\_se^2}{(1 - r^2)^2}} }
\deqn{z\_ci\_lo = atanh(r\_lo)}
\deqn{z\_ci\_up = atanh(r\_up)}

\strong{D.} Last, the approach described in Cooper et al. (2019) can be used (\code{or_to_cor = "lipsey_cooper"}).
This argument requires (or + logor_se + n_exp + n_nexp) to generate a R/Z.
As shown above, the function starts to estimate a SMD from the OR.
Then, as described in \code{\link{es_from_cohen_d}}, it converts this Cohen's d value into a correlation
coefficient using the \code{"lipsey_cooper"} formulas.
}
\examples{
es_from_or_se(or = 2.12, logor_se = 0.242, n_exp = 120, n_nexp = 44)
}
\references{
Bonett, Douglas G. and Robert M. Price. (2005). Inferential Methods for the Tetrachoric Correlation Coefficient. Journal of Educational and Behavioral Statistics 30:213-25.

Bonett, D. G., & Price, R. M. (2007). Statistical inference for generalized Yule coefficients in 2* 2 contingency tables. Sociological methods & research, 35(3), 429-446.

Cooper, H., Hedges, L. V., & Valentine, J. C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Di Pietrantonj C. (2006). Four-fold table cell frequencies imputation in meta analysis. Statistics in medicine, 25(13), 2299-2322. https://doi.org/10.1002/sim.2287

Digby, Peter G. N. (1983). Approximating the Tetrachoric Correlation Coefficient. Biometrics 39:753-7.

Gosling, C. J., Solanes, A., Fusar-Poli, P., & Radua, J. (2023). metaumbrella: the first comprehensive suite to perform data analysis in umbrella reviews with stratification of the evidence. BMJ mental health, 26(1), e300534. https://doi.org/10.1136/bmjment-2022-300534

Grant R. L. (2014). Converting an odds ratio to a range of plausible relative risks for better communication of research findings. BMJ (Clinical research ed.), 348, f7450. https://doi.org/10.1136/bmj.f7450

Pearson, K. (1900). Mathematical Contributions to the Theory of Evolution. VII: On the Correlation of Characters Not Quantitatively Measurable. Philosophical Transactions of the Royal Statistical Society of London, Series A 19:1-47

Veroniki, A. A., Pavlides, M., Patsopoulos, N. A., & Salanti, G. (2013). Reconstructing 2x2 contingency tables from odds ratios using the Di Pietrantonj method: difficulties, constraints and impact in meta-analysis results. Research synthesis methods, 4(1), 78-94. https://doi.org/10.1002/jrsm.1061
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/psychometric_utils.R
\name{compute_sem}
\alias{compute_sem}
\title{Compute Standard Error of Measurement (SEM) from reliability and SD}
\usage{
compute_sem(sd, icc, n_sample, icc_se, n_measurements = 2)
}
\arguments{
\item{sd}{standard deviation of the PROM/test scores}

\item{icc}{intraclass correlation coefficient (test-retest reliability)}

\item{n_sample}{sample size (used for the SEM sampling-variance estimation).
Required in BOTH variance regimes: it feeds \eqn{Var(SD)} in the
external-\code{icc_se} delta method and the degrees of freedom of the
same-sample chi-square variance, so when it is omitted the SEM standard
error and CI are \code{NA}.}

\item{icc_se}{standard error of the ICC (optional), on the RAW ICC scale.
Note that \code{\link{es_from_icc}} under its default
\code{icc_to_es = "bonett"} returns a column named \code{icc_se} on the
\eqn{\ln(1 - ICC)} scale: that value must be converted first
(\code{raw_se = bonett_se * (1 - icc)}) before being passed here (or use
\code{icc_to_es = "raw"}, which returns the raw-scale SE directly).
When supplied, the ICC is treated as an independent external estimate;
when omitted, the ICC and SD are assumed to come from the same sample
(see Details).}

\item{n_measurements}{number of measurement occasions or raters (k) used to
estimate the ICC; default 2 (test-retest); must be >= 2. Only used when
\code{icc_se} is not supplied.}
}
\value{
A data.frame with the SEM, its standard error and 95\% CI.
}
\description{
Compute Standard Error of Measurement (SEM) from reliability and SD
}
\details{
Computes the standard error of measurement (SEM) from a reliability
coefficient and the standard deviation of scores:

\deqn{SEM = SD \times \sqrt{1 - ICC}}

The sampling variance of SEM is computed in one of two ways:
\enumerate{
\item \strong{When \code{icc_se} is supplied}, the ICC is treated as an external
estimate, independent of the within-sample SD, and the bivariate delta method
(with zero covariance) is used:
\deqn{Var(SEM) = \frac{SD^2}{4(1 - ICC)} \times Var(ICC) + (1 - ICC) \times Var(SD)}
with \eqn{Var(ICC) = icc\_se^2} and \eqn{Var(SD) \approx SD^2 / (2(n-1))}.
\item \strong{When \code{icc_se} is not supplied}, the ICC and SD are assumed to come
from the same reliability sample, where they are strongly positively
correlated. Because \eqn{SEM = SD\sqrt{1 - ICC}} equals the within-subject
residual root-mean-square \eqn{\sqrt{MSE}}, and \eqn{MSE / \sigma_e^2} follows
a scaled chi-square with \eqn{(n-1)(k-1)} degrees of freedom, the exact
sampling variance is
\deqn{Var(SEM) = \frac{SEM^2}{2(n-1)(k-1)}}
where \eqn{k} is the number of measurement occasions/raters
(\code{n_measurements}, default 2 for test-retest). Treating the same-sample SD
and ICC as independent (the delta method of case 1) would overestimate this
variance by roughly 2-6x, so the exact form is used instead.
}

The 95\% CI is likewise regime-specific. In the same-sample regime (case 2)
the degrees of freedom \eqn{df = (n - 1)(k - 1)} are known, so the exact
chi-square interval
\deqn{[SEM \sqrt{df / \chi^2_{0.975, df}},\; SEM \sqrt{df / \chi^2_{0.025, df}}]}
is returned; the symmetric Wald interval undercovers at small df (89.7\%
instead of 95\% at n = 10, k = 2). When \code{icc_se} is supplied (case 1)
the df behind the external estimate is unknown, so the Wald interval
\eqn{SEM \pm 1.96 \times SE}, truncated at 0, is kept.

Degenerate inputs: \code{n_measurements} = 1 makes the same-sample df zero,
so the SE and CI are returned as \code{NA} with a warning; an ICC > 1 is
impossible and yields \code{NA} for the SEM, its SE and CI (with a warning).
}
\examples{
compute_sem(sd = 10, icc = 0.85, n_sample = 100)
}
\references{
Weir, J. P. (2005). Quantifying test-retest reliability using the intraclass correlation
coefficient and the SEM. Journal of Strength and Conditioning Research, 19(1), 231-240.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_point_biserial_COR.R
\name{es_from_pt_bis_r}
\alias{es_from_pt_bis_r}
\title{Convert a point-biserial correlation coefficient into several effect size measures}
\usage{
es_from_pt_bis_r(
  pt_bis_r,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_pt_bis_r
)
}
\arguments{
\item{pt_bis_r}{value of a point-biserial correlation coefficient}

\item{n_exp}{total number of participants in the exposed group}

\item{n_nexp}{total number of participants in the non exposed group}

\item{smd_to_cor}{formula used to convert the \code{pt_bis_r} value into a coefficient correlation.}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_pt_bis_r}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a point-biserial correlation coefficient into several effect size measures
}
\details{
This function uses a point-biserial correlation coefficient to estimate a
Cohen's d (D) and Hedges' g (G).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{The formula used to obtain the Cohen's d are (Viechtbauer, 2021)}:
\deqn{m = n\_exp + n\_nexp - 2}
\deqn{h = \frac{m}{n\_exp} + \frac{m}{n\_nexp}}
\deqn{d = \frac{pt\_bis\_r * \sqrt{h}}{\sqrt{1 - pt\_bis\_r^2}}}

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_pt_bis_r(pt_bis_r = 0.2, n_exp = 121, n_nexp = 121)
}
\references{
Viechtbauer (2021). Accessed at: https://stats.stackexchange.com/questions/526789/convert-correlation-r-to-cohens-d-unequal-groups-of-known-size
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_SMD.R
\name{es_from_hedges_g}
\alias{es_from_hedges_g}
\title{Convert a Hedges' g value to other effect size measures (G, OR, COR)}
\usage{
es_from_hedges_g(
  hedges_g,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_g
)
}
\arguments{
\item{hedges_g}{Hedges' g value}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{hedges_g} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_g}{a logical value indicating whether the direction of the \code{hedges_g} value should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 1. Cohen's d or Hedges' g'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a Hedges' g value to other effect size measures (G, OR, COR)
}
\details{
This function estimates the standard error of the Hedges' g and the Cohen's d (D).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate standard error of Hedges'g}, the following formula is used (Hedges, 1981):
\deqn{df = n\_exp + n\_nexp - 2}
\deqn{hedges\_g\_se = \sqrt{cohen\_d\_se^2 * J^2}}
\deqn{hedges\_g\_ci\_lo = hedges\_g - hedges\_g\_se * qt(.975, df = n\_exp+n\_nexp-2)}
\deqn{hedges\_g\_ci\_up = hedges\_g + hedges\_g\_se * qt(.975, df = n\_exp+n\_nexp-2)}

\strong{To estimate the Cohen's d value}, the following formula is used (Hedges, 1981):
\deqn{J = exp(\log_{gamma}(\frac{df}{2}) - 0.5 * \log(\frac{df}{2}) - \log_{gamma}(\frac{df - 1}{2}))}
\deqn{cohen\_d = \frac{hedges\_g}{J}}
\deqn{cohen\_d\_se = \sqrt{(\frac{n\_exp+n\_nexp}{n\_exp*n\_nexp} + \frac{cohen\_d^2}{2*(n\_exp+n\_nexp)})}}

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_hedges_g(hedges_g = 0.243, n_exp = 20, n_nexp = 20)
}
\references{
Hedges LV (1981): Distribution theory for Glass's estimator of effect size and related estimators. Journal of Educational and Behavioral Statistics, 6, 107-28
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_point_biserial_COR.R
\name{es_from_pt_bis_r_pval}
\alias{es_from_pt_bis_r_pval}
\title{Convert a p-value of a point-biserial correlation coefficient into several effect size measures}
\usage{
es_from_pt_bis_r_pval(
  pt_bis_r_pval,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_pt_bis_r_pval
)
}
\arguments{
\item{pt_bis_r_pval}{p-value of a point-biserial correlation coefficient}

\item{n_exp}{total number of participants in the exposed group}

\item{n_nexp}{total number of participants in the non exposed group}

\item{smd_to_cor}{formula used to convert the \code{pt_bis_r_pval} value into a coefficient correlation.}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_pt_bis_r_pval}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a p-value of a point-biserial correlation coefficient into several effect size measures
}
\details{
This function converts the p-value of a point biserial correlation into a Student's t-value.

\strong{The formula used to obtain this Student's t-value is}:
\deqn{t = pt(\frac{pt\_bis\_r\_pval}{2}, df = n\_exp + n\_nexp - 2)}

Calculations of the \code{\link{es_from_student_t}} function are then applied.

Because a two-sided p-value carries no direction, the recovered t (and hence the
generated effect sizes) are always non-negative; use \code{reverse_pt_bis_r_pval} to
encode the correct sign.
}
\examples{
es_from_pt_bis_r_pval(pt_bis_r_pval = 0.2, n_exp = 121, n_nexp = 121)
}
\references{
Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_means.R
\name{es_from_ancova_means_sd}
\alias{es_from_ancova_means_sd}
\title{Convert means and standard deviations of two independent groups obtained from an ANCOVA model into several effect size measures}
\usage{
es_from_ancova_means_sd(
  n_exp,
  n_nexp,
  ancova_mean_exp,
  ancova_mean_nexp,
  ancova_mean_sd_exp,
  ancova_mean_sd_nexp,
  cov_outcome_r,
  n_cov_ancova,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_means
)
}
\arguments{
\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{ancova_mean_exp}{adjusted mean of participants in the experimental/exposed group.}

\item{ancova_mean_nexp}{adjusted mean of participants in the non-experimental/non-exposed group.}

\item{ancova_mean_sd_exp}{adjusted standard deviation of participants in the experimental/exposed group.}

\item{ancova_mean_sd_nexp}{adjusted standard deviation of participants in the non-experimental/non-exposed group.}

\item{cov_outcome_r}{correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{smd_to_cor}{formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_means}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert means and standard deviations of two independent groups obtained from an ANCOVA model into several effect size measures
}
\details{
This function first computes an "adjusted" mean difference (MD),
Cohen's d (D) and Hedges' g (G) from the adjusted means and standard deviations.
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

This function start by estimating the non-adjusted standard deviation of the two groups (formula 12.24 in Cooper);
\deqn{mean\_sd\_exp = \frac{ancova\_mean\_sd\_exp}{\sqrt{1 - cov\_outcome\_r^2}}}
\deqn{mean\_sd\_nexp = \frac{ancova\_mean\_sd\_nexp}{\sqrt{1 - cov\_outcome\_r^2}}}

\strong{To obtain the mean difference}, the following formulas are used (authors calculations):
\deqn{md = ancova\_mean\_exp - ancova\_mean\_nexp}
\deqn{md\_se = \sqrt{\frac{ancova\_mean\_sd\_exp^2}{n\_exp} + \frac{ancova\_mean\_sd\_nexp^2}{n\_nexp}}}
\deqn{md\_ci\_lo = md - md\_se * qt(.975, n\_exp+n\_nexp-2-n\_cov\_ancova)}
\deqn{md\_ci\_up = md + md\_se * qt(.975, n\_exp+n\_nexp-2-n\_cov\_ancova)}

\strong{To obtain the Cohen's d}, the following formulas are used (table 12.3 in Cooper):
\deqn{mean\_sd\_pooled = \sqrt{\frac{(n\_exp - 1) * ancova\_mean\_sd\_exp^2 + (n\_nexp - 1) * ancova\_mean\_sd\_nexp^2}{n\_exp+n\_nexp-2}}}
\deqn{cohen\_d =  \frac{ancova\_mean\_exp - ancova\_mean\_nexp}{mean\_sd\_pooled}}
\deqn{cohen\_d\_se = \sqrt{\frac{(n\_exp+n\_nexp)*(1-cov\_outcome\_r^2)}{n\_exp*n\_nexp} + \frac{cohen\_d^2}{2(n\_exp+n\_nexp)}}}
\deqn{cohen\_d\_ci\_lo = cohen\_d - cohen\_d\_se * qt(.975, n\_exp + n\_nexp - 2 - n\_cov\_ancova)}
\deqn{cohen\_d\_ci\_up = cohen\_d + cohen\_d\_se * qt(.975, n\_exp + n\_nexp - 2 - n\_cov\_ancova)}

\strong{To estimate other effect size measures},
Calculations of the \code{\link{es_from_cohen_d_adj}()} are applied.
}
\note{
The sampling variance follows Cooper's eq. 12.26 and assumes the covariate is
balanced across groups. It omits the covariate-imbalance ("leverage") term of
the exact ANCOVA variance,
\eqn{\sigma^2_{res}\,(1/n\_exp + 1/n\_nexp + (\bar{x}\_exp-\bar{x}\_nexp)^2 / SS_x)},
which is not recoverable from summary statistics, and treats
\code{cov_outcome_r} as known. In balanced/randomised designs the omission is
negligible; in observational or otherwise covariate-imbalanced designs the
standard error is a lower bound (anti-conservative), while the point estimate
remains unbiased. See Lai and Kelley (2012).
}
\examples{
es_from_ancova_means_sd(
  n_exp = 55, n_nexp = 55,
  ancova_mean_exp = 2.3, ancova_mean_sd_exp = 1.2,
  ancova_mean_nexp = 1.9, ancova_mean_sd_nexp = 0.9,
  cov_outcome_r = 0.2, n_cov_ancova = 3
)

}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Lai, K., & Kelley, K. (2012). Accuracy in parameter estimation for ANCOVA and ANOVA contrasts: Sample size planning via narrow confidence intervals. British Journal of Mathematical and Statistical Psychology, 65(2), 350-370.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_variability_from_dispersion.R
\name{es_variab_from_means_sd}
\alias{es_variab_from_means_sd}
\title{Convert means and/or standard deviations of two independent groups into two effect measures (VR/CVR)}
\usage{
es_variab_from_means_sd(
  mean_exp,
  mean_nexp,
  mean_sd_exp,
  mean_sd_nexp,
  n_exp,
  n_nexp,
  reverse_means_variability
)
}
\arguments{
\item{mean_exp}{mean of participants in the experimental/exposed group.}

\item{mean_nexp}{mean of participants in the non-experimental/non-exposed group.}

\item{mean_sd_exp}{standard deviation of participants in the experimental/exposed group.}

\item{mean_sd_nexp}{standard deviation of participants in the non-experimental/non-exposed group.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{reverse_means_variability}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates VR and CVR

\tabular{ll}{
\code{natural effect size measure} \tab VR + CVR\cr
\tab \cr
\code{converted effect size measure} \tab No conversion performed\cr
\tab \cr
\code{required input data} \tab See 'Section 23. User's input (crude)'\cr
\tab https://metaconvert.org/html/input.html\cr
\tab \cr
}
}
\description{
Convert means and/or standard deviations of two independent groups into two effect measures (VR/CVR)
}
\details{
This function converts the means and standard deviations of two independent groups into a log variability ratio (VR) and a log coefficient of variation ratio (CVR).

The formulas used to obtain the log VR are (formulas 5 and 15, Senior et al. 2020):
\deqn{logvr = log(\frac{mean\_sd\_exp}{mean\_sd\_nexp}) + \frac{1}{2 * (n\_exp - 1)} - \frac{1}{2 * (n\_nexp - 1)}}
\deqn{logvr\_se = \sqrt{\frac{1}{2} * (\frac{n\_nexp}{(n\_nexp - 1)^2} + \frac{n\_exp}{(n\_exp - 1)^2})}}
\deqn{logvr\_ci\_lo = logvr - qnorm(.975) * logvr\_se}
\deqn{logvr\_ci\_up = logvr + qnorm(.975) * logvr\_se}

The formulas used to obtain the log CVR are (formulas 6 and 16, Senior et al. 2020):
\deqn{cvt = mean\_sd\_exp / mean\_exp}
\deqn{cvc = mean\_sd\_nexp / mean\_nexp}
\deqn{logcvr = log(\frac{cvt}{cvc}) + \frac{1}{2} * (\frac{1}{n\_exp - 1} - \frac{1}{n\_nexp - 1}) + \frac{1}{2} * (\frac{mean\_sd\_nexp^2}{n\_nexp * mean\_nexp^2} - \frac{mean\_sd\_exp^2}{n\_exp * mean\_exp^2})}
\deqn{vt\_exp = \frac{mean\_sd\_exp^2}{n\_exp * mean\_exp^2} + \frac{mean\_sd\_exp^4}{2 * n\_exp^2 * mean\_exp^4} + \frac{n\_exp}{2 * (n\_exp - 1)^2}}
\deqn{vt\_nexp = \frac{mean\_sd\_nexp^2}{n\_nexp * mean\_nexp^2} + \frac{mean\_sd\_nexp^4}{2 * n\_nexp^2 * mean\_nexp^4} + \frac{n\_nexp}{2 * (n\_nexp - 1)^2}}
\deqn{logcvr\_se = \sqrt{vt\_exp + vt\_nexp}}
\deqn{logcvr\_ci\_lo = logcvr - qnorm(.975) * logcvr\_se}
\deqn{logcvr\_ci\_up = logcvr + qnorm(.975) * logcvr\_se}
}
\examples{
es_variab_from_means_sd(
  n_exp = 55, n_nexp = 55,
  mean_exp = 2.3, mean_sd_exp = 1.2,
  mean_nexp = 1.9, mean_sd_nexp = 0.9
)
}
\references{
Senior, A. M., Viechtbauer, W., & Nakagawa, S. (2020). Revisiting and expanding the meta-analysis of variation: The log coefficient of variation ratio. Research Synthesis Methods, 11(4), 553-567. https://doi.org/10.1002/jrsm.1423
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PLOTS.R
\name{es_from_plot_means}
\alias{es_from_plot_means}
\title{Converts the means and bounds of an error bar (generally extracted from a plot) into four effect measures (SMD, MD, OR, COR)}
\usage{
es_from_plot_means(
  n_exp,
  n_nexp,
  plot_mean_exp,
  plot_mean_nexp,
  plot_mean_sd_lo_exp,
  plot_mean_sd_lo_nexp,
  plot_mean_sd_up_exp,
  plot_mean_sd_up_nexp,
  plot_mean_se_lo_exp,
  plot_mean_se_lo_nexp,
  plot_mean_se_up_exp,
  plot_mean_se_up_nexp,
  plot_mean_ci_lo_exp,
  plot_mean_ci_lo_nexp,
  plot_mean_ci_up_exp,
  plot_mean_ci_up_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_plot_means
)
}
\arguments{
\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{plot_mean_exp}{mean of participants in the experimental/exposed group (extracted from a plot).}

\item{plot_mean_nexp}{mean of participants in the non-experimental/non-exposed group (extracted from a plot).}

\item{plot_mean_sd_lo_exp}{lower bound of an error bar depicting -1 SD from the mean of the experimental/exposed group (extracted from a plot).}

\item{plot_mean_sd_lo_nexp}{lower bound of an error bar depicting -1 SD from the mean of the non-experimental/non-exposed group (extracted from a plot).}

\item{plot_mean_sd_up_exp}{upper bound of an error bar depicting +1 SD from the mean of the experimental/exposed group (extracted from a plot).}

\item{plot_mean_sd_up_nexp}{upper bound of an error bar depicting +1 SD from the mean of the non-experimental/non-exposed group (extracted from a plot).}

\item{plot_mean_se_lo_exp}{lower bound of an error bar depicting -1 SE from the mean of the experimental/exposed group (extracted from a plot).}

\item{plot_mean_se_lo_nexp}{lower bound of an error bar depicting -1 SE from the mean of the non-experimental/non-exposed group (extracted from a plot).}

\item{plot_mean_se_up_exp}{upper bound of an error bar depicting +1 SE from the mean of the experimental/exposed group (extracted from a plot).}

\item{plot_mean_se_up_nexp}{upper bound of an error bar depicting +1 SE from the mean of the non-experimental/non-exposed group (extracted from a plot).}

\item{plot_mean_ci_lo_exp}{lower bound of an error bar depicting the 95\% CI of the mean of the experimental/exposed group (extracted from a plot).}

\item{plot_mean_ci_lo_nexp}{lower bound of an error bar depicting the 95\% CI of the mean of the non-experimental/non-exposed group (extracted from a plot).}

\item{plot_mean_ci_up_exp}{upper bound of an error bar depicting the 95\% CI of the mean of the experimental/exposed group (extracted from a plot).}

\item{plot_mean_ci_up_nexp}{upper bound of an error bar depicting the 95\% CI of the mean of the non-experimental/non-exposed group (extracted from a plot).}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_plot_means}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 21. From plot: means and dispersion (crude)'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Converts the means and bounds of an error bar (generally extracted from a plot) into four effect measures (SMD, MD, OR, COR)
}
\details{
This function uses the bounds of an error bar of a mean obtained from a plot into a standard deviation.
Then, a mean difference (MD), Cohen's d (D), and Hedges' g (G) are estimated.
Odds ratio (OR), risk ratio (RR) and correlation coefficients (R/Z) are converted from the Cohen's d value.

\strong{To convert the bound of an error bar into a standard deviation,}
this function always prioritizes information from the \code{plot_mean_sd_*} arguments,
then those from the \code{plot_mean_se_*} arguments,
then those from the \code{plot_mean_ci_*} arguments.
\enumerate{
\item If the bounds of the standard deviations are provided, the following formulas are used:
\deqn{mean\_sd\_lo\_exp = plot\_mean\_exp - plot\_mean\_sd\_lo\_exp}
\deqn{mean\_sd\_up\_exp = plot\_mean\_sd\_up\_exp - plot\_mean\_exp}
\deqn{mean\_sd\_exp = \frac{mean\_sd\_lo\_exp + mean\_sd\_up\_exp}{2}}
}

\deqn{mean\_sd\_lo\_nexp = plot\_mean\_nexp - plot\_mean\_sd\_lo\_nexp}
\deqn{mean\_sd\_up\_nexp = plot\_mean\_sd\_up\_nexp - plot\_mean\_nexp}
\deqn{mean\_sd\_nexp = \frac{mean\_sd\_lo\_nexp + mean\_sd\_up\_nexp}{2}}

Note that if only one bound (e.g., the upper bound) is provided, it will be the
only information used to estimate the standard deviation value.

Then, calculations of the \code{\link{es_from_means_sd}} are used.
\enumerate{
\item If the bounds of the standard errors are provided, the following formulas are used:
\deqn{mean\_se\_lo\_exp = plot\_mean\_exp - plot\_mean\_se\_lo\_exp}
\deqn{mean\_se\_up\_exp = plot\_mean\_se\_up\_exp - plot\_mean\_exp}
\deqn{mean\_se\_exp = \frac{mean\_se\_lo\_exp + mean\_se\_up\_exp}{2}}
}

\deqn{mean\_se\_lo\_nexp = plot\_mean\_nexp - plot\_mean\_se\_lo\_nexp}
\deqn{mean\_se\_up\_nexp = plot\_mean\_se\_up\_nexp - plot\_mean\_nexp}
\deqn{mean\_se\_nexp = \frac{mean\_se\_lo\_nexp + mean\_se\_up\_nexp}{2}}

Note that if only one bound (e.g., the upper bound) is provided, it will be the
only information used to estimate the standard error value.

Then, calculations of the \code{\link{es_from_means_se}()} are used.
\enumerate{
\item If the bounds of the 95\% confidence intervals are provided, the calculations
of the \code{\link{es_from_means_ci}} are used.
}
}
\examples{
es_from_plot_means(
  n_exp = 35, n_nexp = 35,
  plot_mean_exp = 89, plot_mean_nexp = 104,
  plot_mean_sd_lo_exp = 69, plot_mean_sd_lo_nexp = 83,
  plot_mean_sd_up_exp = 109, plot_mean_sd_up_nexp = 125
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_IRR.R
\name{es_from_cases_time}
\alias{es_from_cases_time}
\title{Convert the number of cases and the person-time of disease-free observation in two independent groups into an incidence rate ratio (IRR)}
\usage{
es_from_cases_time(
  n_cases_exp,
  n_cases_nexp,
  time_exp,
  time_nexp,
  baseline_rate,
  reverse_irr
)
}
\arguments{
\item{n_cases_exp}{number of cases in the exposed group}

\item{n_cases_nexp}{number of cases in the non-exposed group}

\item{time_exp}{person-time of disease-free observation in the exposed group}

\item{time_nexp}{person-time of disease-free observation in the non-exposed group}

\item{baseline_rate}{incidence rate of events (per person-time) in the non-exposed group (n_cases_nexp / time_nexp is used when missing)}

\item{reverse_irr}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates IRR and, when baseline rate information is available, NNT.

\tabular{ll}{
\code{natural effect size measure} \tab IRR + NNT\cr
\tab \cr
\code{converted effect size measure} \tab N/A\cr
\tab \cr
\code{required input data} \tab See 'Section 5. Incidence Ratio Ratio'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert the number of cases and the person-time of disease-free observation in two independent groups into an incidence rate ratio (IRR)
}
\details{
This function estimates the incidence rate ratio from the number of cases and
the person-time of disease-free observation in two independent groups.

\strong{The formula used to obtain the IRR and its standard error} are (Cochrane Handbook (section 6.7.1):
\deqn{logirr = log(\frac{n\_cases\_exp / time\_exp}{n\_cases\_nexp / time\_nexp)}}
\deqn{logirr\_se = \sqrt{\frac{1}{n\_cases\_exp} + \frac{1}{n\_cases\_nexp}}}

\strong{To estimate a person-time NNT} (Mayne et al., 2006), the following formulas are used:
\deqn{ird = baseline\_rate \times (1 - irr)}
\deqn{nnt = \frac{1}{ird}}
where \code{ird} is the incidence rate difference and \code{baseline_rate} is the
incidence rate in the control group.

\strong{To estimate the standard error of the IRD}, two formulas are used.
When \code{baseline_rate} is missing:
\deqn{ird\_se = \sqrt{\frac{n\_cases\_exp}{time\_exp^2} + \frac{n\_cases\_nexp}{time\_nexp^2}}}
When \code{baseline_rate} is entered by users, the delta method is used:
\deqn{ird\_se = baseline\_rate \times IRR \times logirr\_se}
}
\examples{
es_from_cases_time(
  n_cases_exp = 241, n_cases_nexp = 554,
  time_exp = 12.764, time_nexp = 19.743
)
}
\references{
Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.

Mayne, T. J., Whalen, E., & Rost, K. (2006). Annualized was found better than absolute risk reduction in the calculation of number needed to treat in chronic conditions. Journal of clinical epidemiology, 59(3), 217-223.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{df.short}
\alias{df.short}
\title{Short version of the df.haza dataset}
\format{
An object of class \code{grouped_df} (inherits from \code{tbl_df}, \code{tbl}, \code{data.frame}) with 37 rows and 109 columns.
}
\source{
Haza B, Gosling CJ, Conty L & Pinabiaux C (2024). Social Functioning in Children and Adolescents with ADHD: A Meta-analysis. Journal of Child Psychology and Psychiatry and Allied Disciplines.
}
\usage{
df.short
}
\description{
This dataset is a shoter version of the \code{\link{df.haza}} dataset.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PAIRED_MEANS.R
\name{es_from_means_ci_pre_post}
\alias{es_from_means_ci_pre_post}
\title{Convert pre-post means of two independent groups into various effect size measures}
\usage{
es_from_means_ci_pre_post(
  mean_pre_exp,
  mean_exp,
  mean_pre_ci_lo_exp,
  mean_pre_ci_up_exp,
  mean_ci_lo_exp,
  mean_ci_up_exp,
  mean_pre_nexp,
  mean_nexp,
  mean_pre_ci_lo_nexp,
  mean_pre_ci_up_nexp,
  mean_ci_lo_nexp,
  mean_ci_up_nexp,
  n_exp,
  n_nexp,
  r_pre_post_exp,
  r_pre_post_nexp,
  smd_to_cor = "viechtbauer",
  pre_post_to_smd = "bonett",
  max_asymmetry = 10,
  pool_sd = FALSE,
  reverse_means_pre_post
)
}
\arguments{
\item{mean_pre_exp}{mean of the experimental/exposed group at baseline}

\item{mean_exp}{mean of the experimental/exposed group at follow up}

\item{mean_pre_ci_lo_exp}{lower bound of the 95\% CI of the mean of the experimental/exposed group at baseline}

\item{mean_pre_ci_up_exp}{upper bound of the 95\% CI of the mean of the experimental/exposed group at baseline}

\item{mean_ci_lo_exp}{lower bound of the 95\% CI of the mean of the experimental/exposed group at follow up}

\item{mean_ci_up_exp}{upper bound of the 95\% CI of the mean of the experimental/exposed group at follow up}

\item{mean_pre_nexp}{mean of the non-experimental/non-exposed group at baseline}

\item{mean_nexp}{mean of the non-experimental/non-exposed group at follow up}

\item{mean_pre_ci_lo_nexp}{lower bound of the 95\% CI of the mean of the non-experimental/non-exposed group at baseline}

\item{mean_pre_ci_up_nexp}{upper bound of the 95\% CI of the mean of the non-experimental/non-exposed group at baseline}

\item{mean_ci_lo_nexp}{lower bound of the 95\% CI of the mean of the non-experimental/non-exposed group at follow up}

\item{mean_ci_up_nexp}{upper bound of the 95\% CI of the mean of the non-experimental/non-exposed group at follow up}

\item{n_exp}{number of the experimental/exposed group}

\item{n_nexp}{number of the non-experimental/non-exposed group}

\item{r_pre_post_exp}{pre-post correlation in the experimental/exposed group}

\item{r_pre_post_nexp}{pre-post correlation in the non-experimental/non-exposed group}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{pre_post_to_smd}{formula used to convert the pre and post means/SD into a SMD (see details).}

\item{max_asymmetry}{A percentage indicating the tolerance before detecting asymmetry in the 95\% CI bounds.}

\item{pool_sd}{a logical value indicating whether the standardizing SD should be pooled across the two
groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
which to prefer, so this is a deliberate choice and not a technical detail.
\itemize{
\item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
within-group values are subtracted, their variances adding because the arms are independent. This
is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
broadly applicable of the two.
\item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
assumes the two arms' true standardizing SDs are equal.
}}

\item{reverse_means_pre_post}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 15. Paired: pre-post means and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert pre-post means of two independent groups into various effect size measures
}
\details{
This function converts the bounds of the 95\% CI of the pre/post means of two independent groups into standard errors (Section 6.3.1 in the Cochrane Handbook).
\deqn{mean\_pre\_se\_exp = \frac{mean\_pre\_ci\_up\_exp - mean\_pre\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
\deqn{mean\_pre\_se\_nexp = \frac{mean\_pre\_ci\_up\_nexp - mean\_pre\_ci\_lo\_nexp}{2 * qt{(0.975, df = n\_nexp - 1)}}}
\deqn{mean\_se\_exp = \frac{mean\_ci\_up\_exp - mean\_ci\_lo\_exp}{2 * qt{(0.975, df = n\_exp - 1)}}}
\deqn{mean\_se\_nexp = \frac{mean\_ci\_up\_nexp - mean\_ci\_lo\_nexp}{2 * qt{(0.975, df = n\_nexp - 1)}}}

Then, calculations of the \code{\link{es_from_means_se_pre_post}} are applied.
}
\examples{
es_from_means_ci_pre_post(
  n_exp = 36, n_nexp = 35,
  mean_pre_exp = 98,
  mean_pre_ci_lo_exp = 88,
  mean_pre_ci_up_exp = 108,
  mean_exp = 102,
  mean_ci_lo_exp = 92,
  mean_ci_up_exp = 112,
  mean_pre_nexp = 96,
  mean_pre_ci_lo_nexp = 86,
  mean_pre_ci_up_nexp = 106,
  mean_nexp = 102,
  mean_ci_lo_nexp = 92,
  mean_ci_up_nexp = 112,
  r_pre_post_exp = 0.8, r_pre_post_nexp = 0.8
)
}
\references{
Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_SMD.R
\name{es_from_cohen_d}
\alias{es_from_cohen_d}
\title{Convert a Cohen's d value to several effect size measures}
\usage{
es_from_cohen_d(
  cohen_d,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_d
)
}
\arguments{
\item{cohen_d}{Cohen's d (i.e., standardized mean difference) value.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_d}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 1. Cohen's d or Hedges' g'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a Cohen's d value to several effect size measures
}
\details{
This function estimates the standard error of a Cohen's d value and computes a Hedges' g (G).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate the standard error of Cohen's d}, the following formula is used (formula 12.13 in Cooper):
\deqn{cohen\_d\_se =  \sqrt{\frac{n\_exp+n\_nexp}{n\_exp*n\_nexp} + \frac{cohen\_d^2}{2*(n\_exp+n\_nexp)}}}
\deqn{cohen\_d\_ci\_lo = cohen\_d - cohen\_d\_se * qt(.975, df = n\_exp+n\_nexp-2)}
\deqn{cohen\_d\_ci\_up = cohen\_d + cohen\_d\_se * qt(.975, df = n\_exp+n\_nexp-2)}

\strong{To estimate the Hedges' g and its standard error}, the following formulas are used (Hedges, 1981):
\deqn{df = n\_exp + n\_nexp - 2}
\deqn{J = exp(\log_{gamma}(\frac{df}{2}) - 0.5 * \log(\frac{df}{2}) - \log_{gamma}(\frac{df - 1}{2}))}
\deqn{hedges\_g = cohen\_d * J}
\deqn{hedges\_g\_se = \sqrt{cohen\_d\_se^2 * J^2}}
\deqn{hedges\_g\_ci\_lo = hedges\_g - hedges\_g\_se * qt(.975, df = n\_exp+n\_nexp-2)}
\deqn{hedges\_g\_ci\_up = hedges\_g + hedges\_g\_se * qt(.975, df = n\_exp+n\_nexp-2)}

\strong{To estimate the log odds ratio and its standard error}, the following formulas are used (formulas 12.34-12.35 in Cooper):
\deqn{logor = \frac{cohen\_d * \pi}{\sqrt{3}}}
\deqn{logor\_se = \sqrt{\frac{cohen\_d\_se^2 * \pi^2}{3}}}
\deqn{logor\_lo = logor - logor\_se * qnorm(.975)}
\deqn{logor\_up = logor + logor\_se * qnorm(.975)}
Note that this conversion assumes that responses within the two groups follow logistic distributions.

\strong{To estimate the correlation coefficient and its standard error}, various formulas can be used.

\strong{A.} To estimate the 'biserial' correlation (\code{smd_to_cor="viechtbauer"}), the following formulas are used (formulas 5, 8, 13, 17, 18, 19 in Viechtbauer):
\deqn{df = n\_exp + n\_nexp - 2 - n\_cov\_ancova}
\deqn{h = \frac{df}{n\_exp} + \frac{df}{n\_nexp}}
\deqn{r.pb = \frac{cohen\_d}{\sqrt{cohen\_d^2 + h}}}
\deqn{p = \frac{n\_exp}{n\_exp + n\_nexp}}
\deqn{q = 1 - p}
\deqn{f = dnorm(qnorm(1-p))}
\deqn{R = \frac{\sqrt{p*q}}{f} * r.pb}
\deqn{R\_var = \frac{1}{n\_exp + n\_nexp - 1} * (\frac{p*q}{f^2} - (\frac{3}{2} + (1 - \frac{p*qnorm(1-p)}{f})(1 + \frac{q*qnorm(1-p)}{f})) R^2 + R^4)}
\deqn{R\_se = \sqrt{R\_var}}
\deqn{a = \frac{\sqrt{f}}{(p*q)^\frac{1}{4}}}
\deqn{Z = \frac{a}{2} * \log(\frac{1+a*R}{1-a*R})}
\deqn{Z\_var = \frac{1}{n - 1}}
\deqn{Z\_se = \sqrt{Z\_var}}
\deqn{Z\_ci\_lo = Z - qnorm(.975) * Z\_se}
\deqn{Z\_ci\_up = Z + qnorm(.975) * Z\_se}
\deqn{R\_ci\_lo = \frac{1}{a} * tanh(\frac{Z\_lo}{a})}
\deqn{R\_ci\_up = \frac{1}{a} * tanh(\frac{Z\_up}{a})}

Note: this \eqn{Z} is Jacobs & Viechtbauer's (2017) variance-stabilizing transform of the
biserial correlation (with variance \eqn{1/(n-1)}), which is analogous to -- but NOT the
same as -- Fisher's \eqn{atanh(R)} z-transform for a Pearson correlation (the two coincide
only as the effect approaches 0). It is intended for constructing the confidence interval
of a single coefficient. For a meta-analysis that mixes standardized-mean-difference and
genuine-correlation studies, pool on the correlation (\eqn{R}) scale, where biserial and
product-moment correlations are directly comparable (Jacobs & Viechtbauer, 2017), rather
than on this \eqn{Z} scale (which summary() labels "Fisher's z" for all correlation inputs).

\strong{B.} To estimate the correlation coefficient according to Cooper et al. (2019) (formulas 12.40-42)
and Borenstein et al. (2009) (formulas 54-56),
the following formulas are used (\code{smd_to_cor="lipsey_cooper"}):
\deqn{p = \frac{n\_exp}{n\_exp + n\_nexp}}
\deqn{R = \frac{cohen\_d}{\sqrt{cohen\_d^2 + 1 / (p * (1 - p))}}}
\deqn{a = \frac{(n\_exp + n\_nexp)^2}{(n\_exp*n\_nexp)}}
\deqn{var\_R = \frac{a^2 * cohen\_d\_se^2}{(cohen\_d^2 + a)^3}}
\deqn{R\_se = \sqrt{R\_var}}
\deqn{R\_ci\_lo = R - qt(.975, n\_exp+n\_nexp- 2) * R\_se}
\deqn{R\_ci\_up = R + qt(.975, n\_exp+n\_nexp- 2) * R\_se}
\deqn{Z = atanh(R)}
\deqn{Z\_var = \frac{cohen\_d\_se^2}{cohen\_d^2 + (1 / p*(1-p))}}
\deqn{Z\_se = \sqrt{Z\_var}}
\deqn{Z\_ci\_lo = Z - qnorm(.975) * Z\_se}
\deqn{Z\_ci\_up = Z + qnorm(.975) * Z\_se}
}
\examples{
es_from_cohen_d(cohen_d = 1, n_exp = 20, n_nexp = 20)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Borenstein, M., Hedges, L. V., Higgins, J. P., & Rothstein, H. R. (2021). Introduction to meta-analysis. John Wiley & Sons.

Hedges LV (1981): Distribution theory for Glass's estimator of effect size and related estimators. Journal of Educational and Behavioral Statistics, 6, 107-28

Jacobs, P., & Viechtbauer, W. (2017). Estimation of the biserial correlation and its sampling variance for use in meta-analysis. Research synthesis methods, 8(2), 161-180.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_2x2.R
\name{es_from_2x2}
\alias{es_from_2x2}
\title{Convert a 2x2 table into several effect size measures}
\usage{
es_from_2x2(
  n_cases_exp,
  n_cases_nexp,
  n_controls_exp,
  n_controls_nexp,
  table_2x2_to_cor = "tetrachoric",
  reverse_2x2
)
}
\arguments{
\item{n_cases_exp}{number of cases/events in the exposed group}

\item{n_cases_nexp}{number of cases/events in the non exposed group}

\item{n_controls_exp}{number of controls/no-event in the exposed group}

\item{n_controls_nexp}{number of controls/no-event in the non exposed group}

\item{table_2x2_to_cor}{formula used to obtain a correlation coefficient from the contingency table (see details).}

\item{reverse_2x2}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab OR + RR + NNT + RD\cr
\tab \cr
\code{converted effect size measure} \tab D + G + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 7. Contingency (2x2) table or proportions'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a 2x2 table into several effect size measures
}
\details{
This function first computes (log) odds ratio (OR), (log) risk ratio (RR) and number needed to treat (NNT)
from the 2x2 table. Note that if a cell is equal to 0, we applied the typical adjustment (add 0.5) to all cells.
This adjustment is used for the OR/RR only; the RD and NNT are obtained from the raw cell counts.
Cohen's d (D), Hedges' g (G) and correlation coefficients (R/Z) are then estimated from the OR.

\strong{To estimate an OR}, the formulas used (Box 6.4.a in the Cochrane Handbook) are:
\deqn{logor = log(\frac{n\_cases\_exp / n\_cases\_nexp}{n\_controls\_exp / n\_controls\_nexp})}
\deqn{logor\_se = \sqrt{\frac{1}{n\_cases\_exp} + \frac{1}{n\_cases\_nexp} + \frac{1}{n\_controls\_exp} + \frac{1}{n\_controls\_nexp}}}

\strong{To estimate an RR}, the formulas used (Box 6.4.a in the Cochrane Handbook) are:
\deqn{logrr = log(\frac{n\_cases\_exp / n\_exp}{n\_cases\_nexp / n\_nexp})}
\deqn{logrr\_se = \sqrt{\frac{1}{n\_cases\_exp} - \frac{1}{n\_exp} + \frac{1}{n\_cases\_nexp} - \frac{1}{n\_nexp}}}

\strong{To estimate a risk difference (RD) and NNT}, the formulas used are (Wen et al., 2005; Altman, 1998):
\deqn{pt = \frac{n\_cases\_exp}{n\_cases\_exp + n\_controls\_exp}}
\deqn{pc = \frac{n\_cases\_nexp}{n\_cases\_nexp + n\_controls\_nexp}}
\deqn{rd = pc - pt}
\deqn{rd\_se = \sqrt{\frac{pt(1-pt)}{n\_exp} + \frac{pc(1-pc)}{n\_nexp}}}
\deqn{nnt = \frac{1}{rd}}
\deqn{nnt\_se = \frac{rd\_se}{rd^2}}
Note that NNT confidence intervals are set to NA when the RD confidence interval crosses zero
(discontinuous CI; Altman, 1998).

\strong{Direction convention.} The risk difference is defined as \eqn{rd = pc - pt}
(control risk minus exposed risk), so a POSITIVE RD means the control group has the
higher risk. This is the OPPOSITE direction to the OR and RR produced from the same
2x2 table, which are exposed-over-non-exposed (an OR/RR \eqn{> 1} means the exposed
group has the higher risk). Consequently, for the same table, a protective exposure
yields \eqn{OR < 1}, \eqn{RR < 1} but \eqn{RD > 0}; keep this in mind when pooling RD
alongside OR/RR, and use \code{reverse_2x2} if you need to align the directions.

\strong{To convert the 2x2 table into a SMD},
the function estimates an OR value from the 2x2 table (formula above)
that is then converted to a SMD
(see formula in \code{\link{es_from_or_se}()}).

\strong{To convert the 2x2 table into a correlation coefficient},
For now, only the tetrachoric correlation is currently proposed
\itemize{
\item \code{table_2x2_to_cor = "tetrachoric"}.
Given the heavy calculations required for this effect size measure,
we relied on the implementation of the formulas of the 'metafor' package. More
information can be retrieved here
(https://wviechtb.github.io/metafor/reference/escalc.html#-b-measures-for-two-dichotomous-variables).
}
}
\examples{
es_from_2x2(n_cases_exp = 467, n_cases_nexp = 22087, n_controls_exp = 261, n_controls_nexp = 8761)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.

Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Available from www.training.cochrane.org/handbook.

Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.

Sedgwick, P. (2013). What is number needed to treat (NNT)? Bmj, 347.

Altman, D. G. (1998). Confidence intervals for the number needed to treat. BMJ, 317(7168), 1309-1312.

Wen, S., Zhang, L., & Yang, B. (2005). Two approaches to incorporate clinical data uncertainty into number needed to treat. Journal of Clinical Pharmacy and Therapeutics, 30(2), 105-109.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_MD.R
\name{es_from_md_sd}
\alias{es_from_md_sd}
\title{Convert a mean difference between two independent groups and standard deviation into several effect size measures}
\usage{
es_from_md_sd(
  md,
  md_sd,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_md
)
}
\arguments{
\item{md}{mean difference between two independent groups}

\item{md_sd}{standard deviation of the mean difference}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_md}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 10. Mean difference and dispersion (crude)'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a mean difference between two independent groups and standard deviation into several effect size measures
}
\details{
This function converts the mean difference and 95\% CI into a Cohen's d (D) and Hedges' g (G).
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{The formula used to obtain the Cohen's d} is:
\deqn{d = \frac{md}{md\_sd}}
Note that this formula is perfectly accurate only if the \code{md_sd} has been estimated by assuming that the variance of the two groups is equal.

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_md_sd(md = 4, md_sd = 2, n_exp = 20, n_nexp = 22)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaConvert.R
\docType{package}
\name{metaConvert-package}
\alias{metaConvert}
\alias{metaConvert-package}
\title{metaConvert: An R Package Dedicated to Automated Effect Size Calculations}
\description{
The \pkg{metaConvert} package automatically estimates 14 effect size measures from a well-formatted dataframe.
Various other functions can help, for example, removing dependency between several effect sizes,
or identifying differences between two dataframes.
This package is mainly designed to assist in conducting a systematic review with a meta-analysis, but it can be
useful to any researcher interested in estimating an effect size.
}
\section{Overview of the package}{
To visualize all the types of input data that can be used to estimate the 14 effect size measures available
in metaConvert, you can use the \code{\link{see_input_data}()} function.
}

\section{Estimate effect sizes}{
To automatically estimate effect sizes directly from a dataset, you can use the \code{\link{convert_df}()} function.
}

\section{Aggregate dependent effect sizes}{
To automatically aggregate dependent effect sizes using Borenstein's formulas,
you can use the \code{\link{aggregate_df}()} function. This function can handle dependent effect sizes
from multiple subgroups, or dependent effect sizes from the same participants.
}

\section{Flag differences between two datasets}{
If pairs of data extractors have generated similar datasets that should be compared,
you can use the \code{\link{compare_df}()} function.
}

\section{Prepare a dataset extraction sheet}{
If you have not started data extraction yet,
you can use the \code{\link{data_extraction_sheet}()} function to obtain a
perfectly formatted data extraction sheet.
}

\section{Well-formatted dataset}{
One of the specificities of the \pkg{metaConvert} package is that its core function (\code{\link{convert_df}})
does not have arguments to specify the names of the variables contained in the dataset.
While this allow using a convenient automatic process in the calculations, this requires that the datasets
passed to this function respect a very precise formatting (which we will refer to as \code{well-formatted dataset}).

Rather than a long description of all column names, we built several tools
that help you find required information.
\enumerate{
\item You can use the \code{\link{data_extraction_sheet}()} function that
generates an excel/csv/txt file containing all the column names available,
as well as a description of the information it should contain.
\item You can use the \code{\link{see_input_data}()} function that generates
a list of all available types of input data as well as their estimated/converted
effect size measures. This function also points out to the corresponding helper tables
available in https://metaconvert.org
}
}

\section{Effect size measures available}{
Eleven effect size measures are accepted:

\itemize{
\item \bold{"d"}: standardized mean difference (i.e., Cohen's d)
\item \bold{"g"}: Hedges' g
\item \bold{"md"}: mean difference
\item \bold{"r"}: Correlation coefficient
\item \bold{"z"}: Fisher's r-to-z correlation
\item \bold{"or"} or \bold{"logor"}: odds ratio  or its logarithm
\item \bold{"rr"} or \bold{"logrr"}: risk ratio or its logarithm
\item \bold{"irr"} or \bold{"logirr"}: incidence rate ratio or its logarithm
\item \bold{"nnt"}: number needed to treat
\item \bold{"logcvr"}: log coefficient of variation
\item \bold{"logvr"}: log variability ratio
}
}

\section{Output}{
All the functions of the \pkg{metaConvert} package that are dedicated to effect size calculations
(i.e., all the functions named \code{es_from_*}) return a dataframe that contain,
depending on the function - some of the following columns:
\tabular{ll}{
\code{info_used} \tab input data used to generate the effect size.\cr
\tab \cr
\code{md} \tab value of the mean difference.\cr
\tab \cr
\code{md_se} \tab standard error of the mean difference.\cr
\tab \cr
\code{md_ci_lo} \tab lower bound of the 95\% CI of the mean difference.\cr
\tab \cr
\code{md_ci_up} \tab upper bound of the 95\% CI of the mean difference.\cr
\tab \cr
\code{d} \tab value of the Cohen's d.\cr
\tab \cr
\code{d_se} \tab standard error of the Cohen's d.\cr
\tab \cr
\code{d_ci_lo} \tab lower bound of the 95\% CI of the Cohen's d.\cr
\tab \cr
\code{d_ci_up} \tab upper bound of the 95\% CI of the Cohen's d.\cr
\tab \cr
\code{g} \tab value of the Hedges' g.\cr
\tab \cr
\code{g_se} \tab standard error of the Hedges' g.\cr
\tab \cr
\code{g_ci_lo} \tab lower bound of the 95\% CI of the Hedges' g.\cr
\tab \cr
\code{g_ci_up} \tab upper bound of the 95\% CI of the Hedges' g.\cr
\tab \cr
\code{r} \tab value of the correlation coefficient.\cr
\tab \cr
\code{r_se} \tab standard error of the correlation coefficient.\cr
\tab \cr
\code{r_ci_lo} \tab lower bound of the 95\% CI of the correlation coefficient.\cr
\tab \cr
\code{r_ci_up} \tab upper bound of the 95\% CI of the correlation coefficient.\cr
\tab \cr
\code{z} \tab value of the r-to-z transformed correlation coefficient.\cr
\tab \cr
\code{z_se} \tab standard error of the r-to-z transformed correlation coefficient.\cr
\tab \cr
\code{z_ci_lo} \tab lower bound of the 95\% CI of the r-to-z transformed correlation coefficient.\cr
\tab \cr
\code{z_ci_up} \tab upper bound of the 95\% CI of the r-to-z transformed correlation coefficient.\cr
\tab \cr
\code{logor} \tab value of the log odds ratio.\cr
\tab \cr
\code{logor_se} \tab standard error of the log odds ratio.\cr
\tab \cr
\code{logor_ci_lo} \tab lower bound of the 95\% CI of the log odds ratio.\cr
\tab \cr
\code{logor_ci_up} \tab upper bound of the 95\% CI of the log odds ratio.\cr
\tab \cr
\code{logrr} \tab value of the log risk ratio.\cr
\tab \cr
\code{logrr_se} \tab standard error of the log risk ratio.\cr
\tab \cr
\code{logrr_ci_lo} \tab lower bound of the 95\% CI of the log risk ratio.\cr
\tab \cr
\code{logrr_ci_up} \tab upper bound of the 95\% CI of the log risk ratio.\cr
\tab \cr
\code{logirr} \tab value of the log incidence rate ratio.\cr
\tab \cr
\code{logirr_se} \tab standard error of the log incidence rate ratio.\cr
\tab \cr
\code{logirr_ci_lo} \tab lower bound of the 95\% CI of the log incidence rate ratio.\cr
\tab \cr
\code{logirr_ci_up} \tab upper bound of the 95\% CI of the log incidence rate ratio.\cr
\tab \cr
\code{logvr} \tab value of the log variability ratio.\cr
\tab \cr
\code{logvr_se} \tab standard error of the log variability ratio.\cr
\tab \cr
\code{logvr_ci_lo} \tab lower bound of the 95\% CI of the log variability ratio.\cr
\tab \cr
\code{logvr_ci_up} \tab upper bound of the 95\% CI of the log variability ratio.\cr
\tab \cr
\code{logcvr} \tab value of the log coefficient of variation.\cr
\tab \cr
\code{logcvr_se} \tab standard error of the log coefficient of variation.\cr
\tab \cr
\code{logcvr_ci_lo} \tab lower bound of the 95\% CI of the log coefficient of variation.\cr
\tab \cr
\code{logcvr_ci_up} \tab upper bound of the 95\% CI of the log coefficient of variation.\cr
\tab \cr
\code{nnt} \tab number needed to treat.\cr
\tab \cr
}
}

\author{
\strong{Maintainer}: Corentin J. Gosling \email{corentin.gosling@parisnanterre.fr}

Authors:
\itemize{
  \item Corentin J. Gosling \email{corentin.gosling@parisnanterre.fr}
  \item Samuele Cortese \email{samuele.cortese@gmail.com}
  \item Marco Solmi \email{marco.solmi83@gmail.com}
  \item Belen Haza \email{hazabelen@gmail.com}
  \item Eduard Vieta \email{EVIETA@clinic.cat}
  \item Richard Delorme \email{richard.delorme.rdb@gmail.com}
  \item Paolo Fusar-Poli \email{Paolo.fusar-poli@kcl.ac.uk}
  \item Joaquim Radua \email{radua@clinic.cat}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PAIRED_SINGLE_GROUP.R
\name{es_from_mean_change_ci_single_group}
\alias{es_from_mean_change_ci_single_group}
\title{Convert mean change and 95\% CI of a single group into standard effect size measures}
\usage{
es_from_mean_change_ci_single_group(
  mean_change_exp,
  mean_change_ci_lo_exp,
  mean_change_ci_up_exp,
  n_exp,
  r_pre_post_exp = 0.8,
  max_asymmetry = 10,
  smd_to_cor = "viechtbauer",
  pre_post_to_smd = "cooper",
  reverse_mean_change
)
}
\arguments{
\item{mean_change_exp}{mean change of participants in the group.}

\item{mean_change_ci_lo_exp}{lower bound of the 95\% CI around the mean change.}

\item{mean_change_ci_up_exp}{upper bound of the 95\% CI around the mean change.}

\item{n_exp}{number of participants in the group.}

\item{r_pre_post_exp}{pre-post correlation within the group}

\item{max_asymmetry}{percentage indicating the tolerance before detecting asymmetry in the 95\% CI bounds.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{pre_post_to_smd}{formula used to convert the mean change into a SMD (\code{"cooper"} by default; \code{"morris_dz"} also accepted, see details).}

\item{reverse_mean_change}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MDw + Dw + Gw\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 14. Paired: mean change, and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert mean change and 95\% CI of a single group into standard effect size measures
}
\details{
This function converts the mean change and 95\% CI of a single group
into within-group Cohen's d, Hedges' g, and mean difference. The effect sizes are then converted to other measures.

The effect size (d_rm) depends linearly on the pre-post correlation (0.8 assumed when
\code{r_pre_post_exp} is missing); see \code{\link{es_from_mean_change_sd_single_group}}.

This function simply internally calls the \code{\link{es_from_means_ci_pre_post_single_group}} function but setting:
\deqn{mean\_pre\_exp = 0}
\deqn{mean\_pre\_ci\_lo\_exp = 0}
\deqn{mean\_pre\_ci\_up\_exp = 0}
\deqn{mean\_exp = mean\_change\_exp}
\deqn{mean\_ci\_lo\_exp = mean\_change\_ci\_lo\_exp}
\deqn{mean\_ci\_up\_exp = mean\_change\_ci\_up\_exp}

To know more about the calculations, see \code{\link{es_from_means_sd_pre_post_single_group}} function.
}
\examples{
es_from_mean_change_ci_single_group(
  n_exp = 36,
  mean_change_exp = 8.4,
  mean_change_ci_lo_exp = 6.4, mean_change_ci_up_exp = 10.4,
  r_pre_post_exp = 0.2
)
}
\references{
Bonett, D. G. (2008). Confidence intervals for standardized linear contrasts of means. Psychological Methods, 13(2), 99-109.

Morris, S. B. (2008). Estimating effect sizes from pretest-posttest-control group designs. Organizational Research Methods, 11(2), 364-386.

Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANOVA_statistics.R
\name{es_from_student_t}
\alias{es_from_student_t}
\title{Convert a Student's t-test value to several effect size measures}
\usage{
es_from_student_t(
  student_t,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_student_t
)
}
\arguments{
\item{student_t}{Student's t-test value.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{student_t} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_student_t}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z\cr
\tab \cr
\code{required input data} \tab See 'Section 11. ANOVA statistics, Student's t-test, or point-bis correlation'\cr
\tab https://metaconvert.org/html/input.html\cr
\tab \cr
}
}
\description{
Convert a Student's t-test value to several effect size measures
}
\details{
This function converts the Student's t-test value into a Cohen's d (D) and Hedges' g (G),
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate a Cohen's d} the formula used is (table 12.1 in Cooper):
\deqn{cohen\_d = student\_t * \sqrt{\frac{(n\_exp+n\_nexp)}{n\_exp*n\_nexp}}}

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.

\strong{Important - Student's t, not Welch's t.} This formula is the exact inverse of the
pooled-variance (Student) t-test, so \code{student_t} must be the \emph{equal-variance} t.
It should NOT be used with a Welch (unequal-variance) t-test, which is the default of
R's \code{t.test()}. A Welch t uses \eqn{\sqrt{s_1^2/n_1 + s_2^2/n_2}} rather than
\eqn{s_{pooled}\sqrt{1/n_1 + 1/n_2}}, so when group sizes and variances both differ,
plugging a Welch t into this formula yields a biased Cohen's d (the bias can exceed
50\% and may flip sign depending on which arm carries the larger variance; it vanishes
only when \eqn{n_1 = n_2}). A Welch t cannot be converted to Cohen's d from
\code{(t, n_exp, n_nexp)} alone: recovering the pooled SD requires the two arm SDs, in
which case \code{\link{es_from_means_sd}} should be used directly.
}
\examples{
es_from_student_t(student_t = 2.1, n_exp = 20, n_nexp = 22)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_USER.R
\name{es_from_user_adj}
\alias{es_from_user_adj}
\title{Directly input an adjusted effect size value + variance, with optional conversion}
\usage{
es_from_user_adj(
  user_es_original_measure_adj,
  user_es_adj,
  user_se_adj,
  user_ci_lo_adj,
  user_ci_up_adj,
  user_es_target_measure_adj = "g",
  n_exp,
  n_nexp,
  n_sample,
  n_cases,
  n_controls,
  baseline_risk,
  small_margin_prop,
  or_to_rr = "metaumbrella_cases",
  or_to_cor = "pearson",
  smd_to_cor = "viechtbauer",
  cor_to_smd = "viechtbauer",
  rr_to_or = "metaumbrella",
  measure,
  user_es_measure_adj
)
}
\arguments{
\item{user_es_original_measure_adj}{the type of effect size entered (see details).}

\item{user_es_adj}{adjusted effect size value}

\item{user_se_adj}{adjusted standard error of the effect size (on the log scale for the or/rr/irr/hr measures)}

\item{user_ci_lo_adj}{adjusted lower bound of the 95\% CI around the effect size value}

\item{user_ci_up_adj}{adjusted upper bound of the 95\% CI around the effect size value}

\item{user_es_target_measure_adj}{the effect size measure of the output (automatically set to the \code{measure} argument when called by the \code{\link{convert_df}} function)}

\item{n_exp}{number of participants in the exposed group}

\item{n_nexp}{number of participants in the non-exposed group}

\item{n_sample}{total number of participants in the sample}

\item{n_cases}{number of cases/events across exposed/non-exposed groups}

\item{n_controls}{number of controls/no-event across exposed/non-exposed groups}

\item{baseline_risk}{proportion of cases in the non-exposed group}

\item{small_margin_prop}{smallest margin proportion of cases/events in the underlying 2x2 table}

\item{or_to_rr}{formula used to convert an odds ratio value into a risk ratio (see \code{\link{es_from_or_se}}).}

\item{or_to_cor}{formula used to convert an odds ratio value into a correlation coefficient (see \code{\link{es_from_or_se}}).}

\item{smd_to_cor}{formula used to convert a SMD value into a coefficient correlation (see \code{\link{es_from_cohen_d}}).}

\item{cor_to_smd}{formula used to convert a correlation coefficient value into a SMD (see \code{\link{es_from_pearson_r}}).}

\item{rr_to_or}{formula used to convert a risk ratio value into an odds ratio (see \code{\link{es_from_rr_se}}).}

\item{measure}{deprecated alias for \code{user_es_target_measure_adj}, kept for backward compatibility with metaConvert <= 1.0.3.}

\item{user_es_measure_adj}{deprecated alias for \code{user_es_original_measure_adj}, kept for backward compatibility with metaConvert <= 1.0.3.}
}
\value{
This function allows to directly input any of the available effect size measures

\tabular{ll}{
\code{natural effect size measure} \tab Any of the available measures\cr
\tab \cr
\code{converted effect size measure} \tab Depends on the entered measure (see details)\cr
\tab \cr
\code{required input data} \tab See 'Section 24. User's input (adjusted)'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Directly input an adjusted effect size value + variance, with optional conversion
}
\details{
This function is a generic function allowing to include any adjusted effect size measure value + variance.
Importantly, when the \code{user_es_original_measure_adj} is one of the known measures
(d, g, md, mdw, dw, gw, or, logor, rr, logrr, irr, logirr, hr, r, z, rd, nnt),
conversions towards the other effect size measures are performed.
Otherwise, no conversion is performed (the effect size value + variance you enter is
the value + variance exported by this function) and a warning is issued.
The sample sizes and baseline risk are used only to perform the conversions.

For the or/rr/irr/hr measures, the standard error you enter must be on the log scale.
If you indicate the 95\% CI bounds instead of the standard error, the log-transformation
is applied automatically.
}
\examples{
dat = data.frame(user_es_original_measure_adj = "or", user_es_adj = 2.5,
                 user_ci_lo_adj = 1.2, user_ci_up_adj = 5.2,
                 n_exp = 120, n_nexp = 44)
summary(convert_df(dat, measure = "g"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_REGRESSION.R
\name{es_from_linreg_b_ci}
\alias{es_from_linreg_b_ci}
\title{Convert an unstandardized regression coefficient and its confidence interval
into several effect size measures}
\usage{
es_from_linreg_b_ci(
  linreg_b,
  linreg_b_ci_lo,
  linreg_b_ci_up,
  n_sample,
  n_covariates,
  sd_iv,
  unit_increase_iv,
  unit_type = "raw_scale",
  n_exp,
  n_nexp,
  cor_to_smd = "viechtbauer",
  reverse_linreg_b
)
}
\arguments{
\item{linreg_b}{unstandardized regression coefficient from a linear regression model}

\item{linreg_b_ci_lo}{lower bound of the 95\% confidence interval of the regression coefficient}

\item{linreg_b_ci_up}{upper bound of the 95\% confidence interval of the regression coefficient}

\item{n_sample}{the total number of participants}

\item{n_covariates}{the number of covariates in the model (excluding the predictor of interest).}

\item{sd_iv}{the standard deviation of the independent variable (optional, see details)}

\item{unit_increase_iv}{a value of the independent variable that will be used to estimate the Cohen's d (optional, see details).}

\item{unit_type}{the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "raw_scale"}

\item{n_exp}{number of the experimental/exposed group (optional)}

\item{n_nexp}{number of the non-experimental/non-exposed group (optional)}

\item{cor_to_smd}{formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.}

\item{reverse_linreg_b}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.
\tabular{ll}{
\code{natural effect size measure} \tab Rp (partial correlation) + Zp (Fisher's z of partial r)\cr
\tab \cr
\code{converted effect size measure} \tab D + G + OR (approximate, see \code{\link{es_from_linreg_t}})\cr
}
}
\description{
Convert an unstandardized regression coefficient and its confidence interval
into several effect size measures
}
\details{
This function derives the standard error from the 95\% confidence interval using a
t-distribution with \eqn{df = n\_sample - n\_covariates - 2} degrees of freedom:
\deqn{SE(b) = \frac{ci\_up - ci\_lo}{2 \times qt(.975, df)}}

Then, calculations of the \code{\link{es_from_linreg_b_se}} function are applied.
For binary predictors without covariates, use \code{\link{es_from_beta_unstd}} instead.
}
\examples{
es_from_linreg_b_ci(
  linreg_b = 1.5, linreg_b_ci_lo = 0.3, linreg_b_ci_up = 2.7,
  n_sample = 100, n_covariates = 2
)
}
\references{
Aloe, A. M., & Thompson, C. G. (2013). The synthesis of partial effect sizes.
\emph{Journal of the Society for Social Work and Research}, 4(4), 390--405.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_REGRESSION.R
\name{es_from_linreg_t}
\alias{es_from_linreg_t}
\title{Convert a t-statistic from a linear regression model to several effect size measures}
\usage{
es_from_linreg_t(
  linreg_t,
  n_sample,
  n_covariates,
  sd_iv,
  unit_increase_iv,
  unit_type = "raw_scale",
  n_exp,
  n_nexp,
  cor_to_smd = "viechtbauer",
  reverse_linreg_t
)
}
\arguments{
\item{linreg_t}{a t-statistic from a linear regression model}

\item{n_sample}{the total number of participants}

\item{n_covariates}{the number of covariates in the model (excluding the predictor of interest).}

\item{sd_iv}{the standard deviation of the independent variable (optional, see details)}

\item{unit_increase_iv}{a value of the independent variable that will be used to estimate the Cohen's d (optional, see details).}

\item{unit_type}{the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "raw_scale"}

\item{n_exp}{number of the experimental/exposed group (optional)}

\item{n_nexp}{number of the non-experimental/non-exposed group (optional)}

\item{cor_to_smd}{formula used to convert a \code{pearson_r} or \code{fisher_z} value into a SMD.}

\item{reverse_linreg_t}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.
\tabular{ll}{
\code{natural effect size measure} \tab Rp (partial correlation) + Zp (Fisher's z of partial r)\cr
\tab \cr
\code{converted effect size measure} \tab D + G + OR (approximate, see details)\cr
}
}
\description{
Convert a t-statistic from a linear regression model to several effect size measures
}
\details{
This function converts a t-statistic from a linear regression model into a
partial correlation coefficient (Rp) and its Fisher's z transformation (Zp).

The partial correlation is obtained as (Aloe & Thompson, 2013, Eq. 2; Gustafson, 1961):
\deqn{r_p = \frac{t}{\sqrt{t^2 + df}}}
where \eqn{df = n\_sample - n\_covariates - 2} is the residual degrees of freedom
of the regression model (i.e. \eqn{n} minus the intercept, the focal predictor,
and the \code{n_covariates} covariates).

Its sampling variance is estimated as recommended by van Aert & Goos (2023, Eq. 5):
\deqn{var(r_p) = \frac{(1 - r_p^2)^2}{df}}

The Fisher's z transformation and its variance are:
\deqn{z_p = atanh(r_p)}
\deqn{var(z_p) = \frac{1}{n - n\_covariates - 3}}

Cohen's d, Hedges' g and odds ratio are then converted from the partial correlation
using the calculations of the \code{\link{es_from_pearson_r}} function.
A partial correlation controls for covariates and thus targets a different estimand
than a two-group comparison or a bivariate correlation (Aloe & Thompson, 2013);
the converted d/g/OR values should not be pooled with such effect sizes.
For meta-analyses of regression results, use \code{measure = "rp"} or
\code{measure = "zp"} in \code{\link{convert_df}}.
}
\examples{
es_from_linreg_t(linreg_t = 2.5, n_sample = 100, n_covariates = 2)

# Reproduce Aloe & Thompson (2013) Table 1, Cole et al. (2004)
es_from_linreg_t(linreg_t = 6.19, n_sample = 232, n_covariates = 6)
}
\references{
Aloe, A. M., & Thompson, C. G. (2013). The synthesis of partial effect sizes.
\emph{Journal of the Society for Social Work and Research}, 4(4), 390--405.

Gustafson, R. L. (1961). Partial correlations in regression computations.
\emph{Journal of the American Statistical Association}, 56, 363--367.

Mathur, M. B., & VanderWeele, T. J. (2020). A simple, interpretable conversion from
Pearson's correlation to Cohen's d for continuous exposures.
\emph{Epidemiology}, 31(2), e16--e18.

van Aert, R. C. M., & Goos, C. (2023). A critical reflection on computing the sampling
variance of the partial correlation coefficient. \emph{Research Synthesis Methods},
14(3), 520--525.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_disattenuate.R
\name{es_disattenuate}
\alias{es_disattenuate}
\title{Disattenuate (correct for unreliability) a correlation coefficient}
\usage{
es_disattenuate(r, r_se, reliability_x, reliability_y, n_sample)
}
\arguments{
\item{r}{observed correlation coefficient (values outside \eqn{[-1, 1]}
trigger a warning; the arithmetic is still applied so the inputs can be
inspected)}

\item{r_se}{standard error of the observed correlation. Optional: when omitted,
it is derived from \code{n_sample} (see Details).}

\item{reliability_x}{reliability of measure X (e.g., target PROM)}

\item{reliability_y}{reliability of measure Y (e.g., comparator instrument)}

\item{n_sample}{sample size. Used only to derive \code{r_se} when \code{r_se}
is not supplied; otherwise unused.}
}
\value{
A data.frame with the corrected correlation and its Fisher's z
transformation (with their standard errors and 95\% CIs), and the
attenuation factor.
}
\description{
Disattenuate (correct for unreliability) a correlation coefficient
}
\details{
Corrects an observed correlation for attenuation due to measurement error
in both measures, using the classical disattenuation formula
(Hunter & Schmidt, 2004; Spearman, 1904):

\deqn{r_c = \frac{r_{obs}}{\sqrt{rel_x \times rel_y}}}

where \eqn{rel_x} and \eqn{rel_y} are reliability coefficients for the
two measures (e.g., Cronbach's alpha, test-retest ICC).

The standard error of the corrected correlation is approximated as:
\deqn{r_c\_se = \frac{r\_se}{\sqrt{rel_x \times rel_y}}}

This approximation treats the reliabilities as known constants, so the SE
and CI are a lower bound when the reliabilities are themselves estimated
(Hunter & Schmidt, 2004, Ch. 3).

When \code{r_se} is not supplied, it is first obtained from the sample size as
the large-sample Pearson correlation SE \eqn{\sqrt{(1 - r^2)^2 / (n - 1)}}
(Cooper et al., 2019).

The function also provides Fisher's z transformation of the corrected
correlation for use in meta-analysis, with
\eqn{SE(z_c) = SE(r_c) / (1 - r_c^2)} (delta method). The corrected-r
confidence interval is obtained by back-transforming the Fisher-z interval
(\eqn{\tanh}), so it always lies within (-1, 1).

When the corrected correlation is extreme (\eqn{|r_c| > 0.999}, including
the mathematically impossible \eqn{|r_c| \ge 1} that inconsistent inputs
produce), no meaningful Fisher's z or CI exists: the corrected-r CI and all
Fisher's z outputs (\code{z_corrected}, \code{z_corrected_se} and its CI
bounds) are set to \code{NA} and a warning is emitted. The corrected point
estimate and its first-order standard error are always returned as
computed, so the offending inputs can be inspected.

This function is typically applied to the results of
\code{summary(convert_df(..., measure = "r"))}:

\preformatted{
res <- summary(convert_df(my_data, measure = "r"))
corrected <- es_disattenuate(
  r = res$es, r_se = res$se,
  reliability_x = my_data$rel_target,
  reliability_y = my_data$rel_comparator,
  n_sample = my_data$n_sample
)
}
}
\examples{
es_disattenuate(r = 0.50, r_se = 0.05,
                reliability_x = 0.85, reliability_y = 0.80,
                n_sample = 100)

# Only correct for one measure's unreliability (set other to 1)
es_disattenuate(r = 0.50, r_se = 0.05,
                reliability_x = 0.85, reliability_y = 1.0,
                n_sample = 100)
}
\references{
Hunter, J. E., & Schmidt, F. L. (2004). Methods of Meta-Analysis: Correcting
Error and Bias in Research Findings (2nd ed.). Sage Publications.

Spearman, C. (1904). The proof and measurement of association between two things.
The American Journal of Psychology, 15(1), 72-101.

Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of
research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_variability_from_dispersion.R
\name{es_variab_from_means_se}
\alias{es_variab_from_means_se}
\title{Convert means and/or standard errors of two independent groups into two effect measures (VR/CVR)}
\usage{
es_variab_from_means_se(
  mean_exp,
  mean_nexp,
  mean_se_exp,
  mean_se_nexp,
  n_exp,
  n_nexp,
  reverse_means_variability
)
}
\arguments{
\item{mean_exp}{mean of participants in the experimental/exposed group.}

\item{mean_nexp}{mean of participants in the non-experimental/non-exposed group.}

\item{mean_se_exp}{standard error of participants in the experimental/exposed group.}

\item{mean_se_nexp}{standard error of participants in the non-experimental/non-exposed group.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{reverse_means_variability}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates VR and CVR

\tabular{ll}{
\code{natural effect size measure} \tab VR + CVR\cr
\tab \cr
\code{converted effect size measure} \tab No conversion performed\cr
\tab \cr
\code{required input data} \tab See 'Section 23. User's input (crude)'\cr
\tab https://metaconvert.org/html/input.html\cr
\tab \cr
}
}
\description{
Convert means and/or standard errors of two independent groups into two effect measures (VR/CVR)
}
\details{
This function converts the standard errors into standard deviations
(\eqn{SE = SD / \sqrt{n}}, so \eqn{SD = SE \times \sqrt{n}}).
\deqn{mean\_sd\_exp = mean\_se\_exp * \sqrt{n\_exp}}
\deqn{mean\_sd\_nexp = mean\_se\_nexp * \sqrt{n\_nexp}}

Then, calculations of the \code{\link{es_variab_from_means_sd}} are applied.
}
\examples{
es_variab_from_means_se(
  mean_exp = 42, mean_se_exp = 11,
  mean_nexp = 42, mean_se_nexp = 15,
  n_exp = 43, n_nexp = 34
)
}
\references{
Senior, A. M., Viechtbauer, W., & Nakagawa, S. (2020). Revisiting and expanding the meta-analysis of variation: The log coefficient of variation ratio. Research Synthesis Methods, 11(4), 553-567. https://doi.org/10.1002/jrsm.1423
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_PAIRED_SINGLE_GROUP.R
\name{es_from_paired_t_single_group}
\alias{es_from_paired_t_single_group}
\title{Convert paired t-test statistic from a single group into standard effect size measures}
\usage{
es_from_paired_t_single_group(
  paired_t_exp,
  n_exp,
  r_pre_post_exp = 0.8,
  smd_to_cor = "viechtbauer",
  pre_post_to_smd = "cooper",
  reverse_paired_t
)
}
\arguments{
\item{paired_t_exp}{paired t-test statistic (pre-post comparison) for the group.}

\item{n_exp}{number of participants in the group.}

\item{r_pre_post_exp}{pre-post correlation within the group}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{pre_post_to_smd}{formula used to convert the paired t statistic into a SMD (\code{"cooper"} by default; \code{"morris_dz"} also accepted, see details).}

\item{reverse_paired_t}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MDw + Dw + Gw\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 15. Paired: pre-post paired t-test'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert paired t-test statistic from a single group into standard effect size measures
}
\details{
This function converts a paired t-test statistic from a single group
into within-group Cohen's d, Hedges' g, and mean difference using the raw score standardizer (morris_drm/cooper).

The paired t-test statistic is related to the mean difference and standard error of difference:
\deqn{t\_paired = \frac{mean\_diff}{SE\_diff}}

where:
\deqn{SE\_diff = \frac{SD\_diff}{\sqrt{n}}}
\deqn{SD\_diff = \sqrt{SD\_pre^2 + SD\_post^2 - 2 \times r \times SD\_pre \times SD\_post}}

This function calculates the raw score standardized mean difference (d_rm, Morris & DeShon 2002):
\deqn{d\_rm = t\_paired \times \sqrt{\frac{2 \times (1 - r)}{n}}}

The variance is calculated as:
\deqn{var(d\_rm) = \frac{2 \times (1 - r)}{n} + \frac{d\_rm^2}{2 \times n}}

The result depends on the pre-post correlation (0.8 assumed when \code{r_pre_post_exp}
is missing); see \code{\link{es_from_mean_change_sd_single_group}}.

The within-group Hedges' g is obtained by applying a bias correction factor J to d.

To estimate other effect size measures, calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_paired_t_single_group(
  paired_t_exp = 2.5,
  n_exp = 20,
  r_pre_post_exp = 0.7
)
}
\references{
Morris, S. B., & DeShon, R. P. (2002). Combining effect size estimates in meta-analysis with repeated measures and independent-groups designs. Psychological Methods, 7(1), 105-125.

Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_REGRESSION.R
\name{es_from_beta_std}
\alias{es_from_beta_std}
\title{Convert a standardized regression coefficient and the standard deviation of the dependent variable
into several effect size measures}
\usage{
es_from_beta_std(
  beta_std,
  sd_dv,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_beta_std
)
}
\arguments{
\item{beta_std}{a standardized regression coefficient value (binary predictor, no other covariables in the model)}

\item{sd_dv}{standard deviation of the dependent variable}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_beta_std}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 13. (Un-)Standardized regression coefficient'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert a standardized regression coefficient and the standard deviation of the dependent variable
into several effect size measures
}
\details{
This function converts a standardized linear regression coefficient
(coming from a model with only one binary predictor), into an
unstandardized linear regression coefficient.

\deqn{sd\_dummy = \sqrt{\frac{n_exp - (n_exp^2 / (n_exp + n_nexp))}{(n_exp + n_nexp - 1)}}}
\deqn{unstd\_beta = beta\_std * \frac{sd\_dv}{sd\_dummy}}

Calculations of the \code{\link{es_from_beta_unstd}} functions are then used.
}
\examples{
es_from_beta_std(beta_std = 0.35, sd_dv = 0.98, n_exp = 20, n_nexp = 22)
}
\references{
Lipsey, M. W., & Wilson, D. B. (2001). Practical meta-analysis. Sage Publications, Inc.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{df.compare1}
\alias{df.compare1}
\title{Fictitious dataset 1}
\format{
An object of class \code{data.frame} with 5 rows and 7 columns.
}
\usage{
df.compare1
}
\description{
First fictitious dataset aiming to understand how the \code{\link{compare_df}} function works.
Slightly different from \code{\link{df.compare1}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANCOVA_means.R
\name{es_from_ancova_means_sd_pooled_crude}
\alias{es_from_ancova_means_sd_pooled_crude}
\title{Convert adjusted means obtained from an ANCOVA model and crude pooled standard deviation of two independent groups into several effect size measures}
\usage{
es_from_ancova_means_sd_pooled_crude(
  ancova_mean_exp,
  ancova_mean_nexp,
  mean_sd_pooled,
  cov_outcome_r,
  n_cov_ancova,
  n_exp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_ancova_means
)
}
\arguments{
\item{ancova_mean_exp}{adjusted mean of participants in the experimental/exposed group.}

\item{ancova_mean_nexp}{adjusted mean of participants in the non-experimental/non-exposed group.}

\item{mean_sd_pooled}{crude pooled standard deviation.}

\item{cov_outcome_r}{correlation between the outcome and covariate(s) (multiple correlation when multiple covariates are included in the ANCOVA model).}

\item{n_cov_ancova}{number of covariates in the ANCOVA model.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the adjusted \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_ancova_means}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab MD + D + G\cr
\tab \cr
\code{converted effect size measure} \tab OR + R + Z \cr
\tab \cr
\code{required input data} \tab See 'Section 19. Adjusted: Means and dispersion'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert adjusted means obtained from an ANCOVA model and crude pooled standard deviation of two independent groups into several effect size measures
}
\details{
This function first computes an "adjusted" mean difference (MD) and Cohen's d (D)
from the adjusted means and crude pooled standard deviation of two independent groups.
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{To estimate the Cohen's d}:
\deqn{d =  \frac{ancova\_mean\_exp - ancova\_mean\_nexp\_adj}{mean\_sd\_pooled}}

\strong{To estimate the mean difference}:
\deqn{md =  ancova\_mean\_exp - ancova\_mean\_nexp\_adj}
\deqn{md\_se =  \sqrt{\frac{n\_exp + n\_nexp}{n\_exp * n\_nexp} * (1 - cov\_outcome\_r^2) * mean\_sd\_pooled^2}}

Then, calculations of the \code{\link{es_from_ancova_means_sd}()} and \code{\link{es_from_cohen_d_adj}()} are applied.
}
\examples{
es_from_ancova_means_sd_pooled_crude(
  ancova_mean_exp = 29, ancova_mean_nexp = 34,
  mean_sd_pooled = 7, cov_outcome_r = 0.2,
  n_cov_ancova = 3, n_exp = 20, n_nexp = 20
)
}
\references{
Cooper, H., Hedges, L.V., & Valentine, J.C. (Eds.). (2019). The handbook of research synthesis and meta-analysis. Russell Sage Foundation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_ANOVA_med_range_quart.R
\name{es_from_med_min_max_quarts}
\alias{es_from_med_min_max_quarts}
\title{Convert median, range and interquartile range of two independent groups into several effect size measures}
\usage{
es_from_med_min_max_quarts(
  q1_exp,
  med_exp,
  q3_exp,
  min_exp,
  max_exp,
  n_exp,
  q1_nexp,
  med_nexp,
  q3_nexp,
  min_nexp,
  max_nexp,
  n_nexp,
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  reverse_med
)
}
\arguments{
\item{q1_exp}{first quartile of the experimental/exposed group.}

\item{med_exp}{median value of the experimental/exposed group.}

\item{q3_exp}{third quartile of the experimental/exposed group.}

\item{min_exp}{minimum value of the experimental/exposed group.}

\item{max_exp}{maximum value of the experimental/exposed group.}

\item{n_exp}{number of participants in the experimental/exposed group.}

\item{q1_nexp}{first quartile of the non-experimental/non-exposed group.}

\item{med_nexp}{median value of the non-experimental/non-exposed group.}

\item{q3_nexp}{third quartile of the non-experimental/non-exposed group.}

\item{min_nexp}{minimum value of the non-experimental/non-exposed group.}

\item{max_nexp}{maximum value of the non-experimental/non-exposed group.}

\item{n_nexp}{number of participants in the non-experimental/non-exposed group.}

\item{smd_to_cor}{formula used to convert the generated \code{cohen_d} value into a coefficient correlation (see details).}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{reverse_med}{a logical value indicating whether the direction of generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab \cr
\tab \cr
\code{converted effect size measure} \tab MD + D + G\cr
\tab OR + R + Z \cr
\code{required input data} \tab See 'Section 12. Median, range and/or interquartile range'\cr
\tab https://metaconvert.org/input.html\cr
\tab \cr
}
}
\description{
Convert median, range and interquartile range of two independent groups into several effect size measures
}
\details{
This function first converts a Cohen's d (D), Hedges' g (G) and mean difference (MD)
from the medians, ranges, and interquartile ranges of two independent groups.
Odds ratio (OR) and correlation coefficients (R/Z) are then converted from the Cohen's d.

\strong{This function recreates means+SD of the two groups}  (Wan et al., 2014):
\deqn{mean\_exp = \frac{min\_exp + 2*q1\_exp + 2*med\_exp + 2*q3\_exp + max\_exp}{8}}
\deqn{mean\_nexp = \frac{min\_nexp + 2*q1\_nexp + 2*med\_nexp + 2*q3\_nexp + max\_nexp}{8}}
\deqn{mean\_sd\_exp = \frac{max\_exp - min\_exp}{4*qnorm(\frac{n\_exp-0.375}{n\_exp+0.25})} + \frac{q3\_exp-q1\_exp}{4*qnorm(\frac{0.75*n\_exp-0.125}{n\_exp+0.25})}}
\deqn{mean\_sd\_nexp = \frac{max\_nexp - min\_nexp}{4*qnorm(\frac{n\_nexp-0.375}{n\_nexp+0.25})} + \frac{q3\_nexp-q1\_nexp}{4*qnorm(\frac{0.75*n\_nexp-0.125}{n\_nexp+0.25})}}

Note that if the group sample size is inferior to 50, a correction is applied to estimate the standard deviation.

\strong{From these means+SD, the function computes MD, D and G} using formulas
described in \code{\link{es_from_means_sd}()}.

\strong{To estimate other effect size measures},
calculations of the \code{\link{es_from_cohen_d}()} are applied.
}
\examples{
es_from_med_min_max_quarts(
  min_exp = 1102, q1_exp = 1335,
  med_exp = 1400, q3_exp = 1765,
  max_exp = 1899, n_exp = 40,
  min_nexp = 1181, q1_nexp = 1481,
  med_nexp = 1625, q3_nexp = 1800,
  max_nexp = 1910, n_nexp = 40
)
}
\references{
Wan, X., Wang, W., Liu, J. et al. Estimating the sample mean and standard deviation from the sample size, median, range and/or interquartile range. BMC Med Res Methodol 14, 135 (2014). https://doi.org/10.1186/1471-2288-14-135
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_convert_df.R
\name{convert_df}
\alias{convert_df}
\title{Automatically compute effect sizes from a well formatted dataset}
\usage{
convert_df(
  x,
  measure = c("d", "g", "md", "logor", "logrr", "logirr", "nnt", "rd", "r", "z", "rp",
    "zp", "logvr", "logcvr"),
  main_es = TRUE,
  es_selected = c("auto", "hierarchy", "minimum", "maximum"),
  selection_auto = c("crude", "paired", "adjusted"),
  split_adjusted = TRUE,
  format_adjusted = c("wide", "long"),
  verbose = TRUE,
  max_asymmetry = 50,
  hierarchy = "means_sd > means_se > means_ci",
  table_2x2_to_cor = "tetrachoric",
  rr_to_or = "metaumbrella",
  or_to_rr = "metaumbrella_cases",
  or_to_cor = "bonett",
  smd_to_cor = "viechtbauer",
  smd_var = "borenstein",
  smd_denom = "pooled",
  pre_post_to_smd = "bonett",
  r_pre_post = 0.8,
  cor_to_smd = "viechtbauer",
  unit_type = "raw_scale",
  yates_chisq = FALSE,
  pool_sd = FALSE,
  prop_to_es = "raw",
  alpha_to_es = "bonett",
  icc_to_es = "bonett",
  correct_inputs = TRUE,
  flag_options = list()
)
}
\arguments{
\item{x}{a well formatted dataset}

\item{measure}{the effect size measure that will be estimated from the information stored in the dataset. See details.}

\item{main_es}{a logical variable indicating whether a main effect size should be selected when overlapping data are present. See details.}

\item{es_selected}{the method used to select the main effect size when several information allows to estimate an effect size for the same association/comparison. Must be either "minimum" (the smallest effect size will be selected), "maximum" (the largest effect size will be selected) or "hierarchy" (the effect size computed from the information specified highest in the hierarchy will be selected). See details.}

\item{selection_auto}{a character string giving details on the best "auto" hierarchy to use (only useful when \code{hierarchy="auto"} and \code{measure= "d", "g" or "md"}). See details.}

\item{split_adjusted}{a logical value indicating whether crude and adjusted effect sizes should be presented separately. See details.}

\item{format_adjusted}{presentation format of the adjusted effect sizes. See details.}

\item{verbose}{a logical variable indicating whether text outputs and messages should be generated. We recommend turning this option to FALSE only after having carefully read all the generated messages.}

\item{max_asymmetry}{A percentage indicating the tolerance before detecting asymmetry in the 95\% CI bounds. Asymmetric CIs are only flagged, not modified (see \code{\link{summary.metaConvert}}).}

\item{hierarchy}{a character string indicating the hierarchy in the information to be prioritized for the effect size calculations. See details.}

\item{table_2x2_to_cor}{formula used to obtain a correlation coefficient from the contingency table. For now only 'tetrachoric' is available.}

\item{rr_to_or}{formula used to convert the \code{rr} value into an odds ratio.}

\item{or_to_rr}{formula used to convert the \code{or} value into a risk ratio.}

\item{or_to_cor}{formula used to convert the \code{or} value into a correlation coefficient.}

\item{smd_to_cor}{formula used to convert the \code{cohen_d} value into a coefficient correlation.}

\item{smd_var}{name of the sampling-variance formula for the standardized mean difference: "borenstein" (default) or "hedges_olkin" (alias "viechtbauer"). The two differ by a squared small-sample-correction factor (J^2); "hedges_olkin" is a few percent larger at small samples.}

\item{smd_denom}{standardizer for the standardized mean difference. "pooled" (default) uses the pooled endpoint SD (Cohen's d / Hedges' g); "glass" (alias "control") uses the control (non-experimental) endpoint SD (Glass's delta); "glass_robust" (alias "control_robust") is Glass's delta with a heteroscedasticity-consistent sampling variance. Only the endpoint means family (es_from_means_sd/se/ci) honours this argument: rows whose effect size comes from any other method (t/F, cohen_d/hedges_g, eta-squared, point-biserial r, medians/ranges, plots, ANCOVA, raw mean differences) always use the pooled-SD standardizer, and a message lists the scoping when a non-pooled value is requested ("glass_robust" additionally has a single variance form, so smd_var is ignored for it).}

\item{pre_post_to_smd}{formula used to obtain a SMD from pre/post means and SD of two independent groups.}

\item{r_pre_post}{pre-post correlation across the two groups (use this argument only if the precise correlation in each group is unknown). Note that when this correlation is defaulted rather than reported, the pre/post standard errors and confidence intervals are unreliable: at a true correlation of 0.3 with the default 0.8, the reported pre/post variance is about 68\% too small (95\% CI coverage ~0.75). Supply \code{r_pre_post_exp}/\code{r_pre_post_nexp} when available, or run a sensitivity analysis over plausible values.}

\item{cor_to_smd}{formula used to convert a correlation coefficient value into a SMD.}

\item{unit_type}{the type of unit for the \code{unit_increase_iv} argument. Must be either "sd" or "value" (see \code{\link{es_from_pearson_r}}).}

\item{yates_chisq}{a logical value indicating whether the Chi square has been performed using Yates' correction for continuity. Can also be given as a column of the dataset when studies differ (rows left NA use this argument).}

\item{pool_sd}{a logical value indicating whether the standardizing SD should be pooled across the two
groups (default \code{FALSE}). The two options target the same estimand when the arms' true SDs are
equal (as randomization implies at baseline) and differ otherwise; the literature does not agree on
which to prefer, so this is a deliberate choice and not a technical detail.
\itemize{
\item \code{FALSE} (default): each arm's change is standardized by that arm's OWN SD and the two
within-group values are subtracted, their variances adding because the arms are independent. This
is Morris's (2008) \eqn{d_{ppc1}}, from Becker (1988). It makes no assumption that the arms' true
SDs are equal, and Viechtbauer (see the metafor-project Morris 2008 page) describes it as the more
broadly applicable of the two.
\item \code{TRUE}: the difference in mean change is divided by a single SD pooled across arms. This
is Morris's (2008) \eqn{d_{ppc2}} (his eq. 8-9), which he recommends: it is more efficient, but it
assumes the two arms' true standardizing SDs are equal.
}}

\item{prop_to_es}{method used to compute the effect size from the proportion. Must be either "raw", "logit" or "freeman_tukey" (see \code{\link{es_from_prop_single_group}}).}

\item{alpha_to_es}{method used to compute the effect size from Cronbach's alpha. Must be either "bonett" or "raw" (see \code{\link{es_from_cronbach_alpha}}).}

\item{icc_to_es}{method used to compute the effect size from an ICC. Must be either "bonett" or "raw" (see \code{\link{es_from_icc}}).}

\item{correct_inputs}{a logical value indicating whether invalid input values (negative SDs, inverted CI bounds, etc.) should be set to NA before the calculations (default TRUE). If FALSE, invalid values are only flagged in the 'es_flags' column and kept in the data.}

\item{flag_options}{a named list of thresholds used by the quality flags (see \code{\link{summary.metaConvert}} for the list of options and defaults).}
}
\value{
The \code{convert_df()} function returns a list of
more than 70 dataframes
(one for each function automatically applied to the dataset).
These dataframes systematically contain the columns described in
\code{\link{metaConvert-package}}.
The list of dataframes can be easily converted to a single,
calculations-ready dataframe
using the summary function (see \code{\link{summary.metaConvert}}).
}
\description{
Automatically compute effect sizes from a well formatted dataset
}
\details{
This function automatically computes or converts between 14 effect sizes
measures from any relevant type of input data stored in the
dataset you pass to this function.
\subsection{Input validation}{

Before the calculations, the input data are checked (negative sample
sizes/SD/SE/p-values, inverted CI bounds, values outside their CI,
asymmetric CI). By default invalid values are set to NA; issues appear
in the 'es_flags' column of summary().
}

\subsection{Effect size measures}{

Possible effect size measures are:
\enumerate{
\item Cohen's d ("d")
\item Hedges' g ("g")
\item mean difference ("md")
\item (log) odds ratio ("or" and "logor")
\item (log) risk ratio ("rr" and "logrr")
\item (log) incidence rate ratio ("irr" and "logirr")
\item correlation coefficient ("r")
\item transformed r-to-z correlation coefficient ("z")
\item partial correlation coefficient ("rp")
\item Fisher's z of partial correlation ("zp")
\item log variability ratio ("logvr")
\item log coefficient of variation ("logcvr")
\item number needed to treat ("nnt")
\item risk difference ("rd")
}
}

\subsection{Computation of a main effect size}{

If you enter multiple types of input data
(e.g., means/sd of two groups and a student t-test value)
for the same comparison i.e., for the same row of the dataset,
the \code{convert_df()} function can have two behaviours.
If you set:
\itemize{
\item \code{main_es = FALSE} the function will estimate all possible effect sizes from all
types of input data (which implies that if a comparison has \strong{several types of input data},
it will result in \strong{multiple rows} in the dataframe returned by the function)
\item \code{main_es = TRUE} the function will select one effect size per comparison
(which implies that if a comparison has \strong{several types of input data},
it will result in a \strong{unique row} in the dataframe returned by the function)
}
}

\subsection{Selection of input data for the computation of the main effect size}{

If you choose to estimate one main effect size (i.e., by setting \code{main_es = TRUE}),
you have several options to select this main effect size.
If you set:
\itemize{
\item \code{es_selected = "auto"}: the main effect size will be \strong{automatically} selected, by prioritizing
specific types of input data over other (see next section "Hierarchy").
\item \code{es_selected = "hierarchy"}: the main effect size will be selected, by prioritizing
specific types of input data over other (see next section "Hierarchy").
\item \code{es_selected = "minimum"}: the main effect size will be selected, by selecting
the lowest effect size available.
\item \code{es_selected = "maximum"}: the main effect size will be selected, by selecting
the highest effect size available.
}
}

\subsection{Hierarchy}{

More than 70 different combinations of input data can be used to estimate an effect size.
You can retrieve the effect size measures estimated by each combination of input data
in the \code{\link{see_input_data}()} function and online \code{https://metaconvert.org/input.html}.

You have two options to use a hierarchy in the types of input data.
\itemize{
\item an automatic way (\code{es_selected = "auto"})
\item an manual way (\code{es_selected = "hierarchy"})
}
\subsection{Automatic}{

If you select an automatic hierarchy, here are the types of input data that will be prioritized.
\subsection{Crude SMD or MD (\code{measure=c("d", "g", "md")} and \code{selection_auto="crude"})}{
\enumerate{
\item User's input effect size value
\item SMD value
\item Means at post-test
\item ANOVA/Student's t-test/point biserial correlation statistics
\item Linear regression estimates
\item Mean difference values
\item Quartiles/median/maximum values
\item Post-test means extracted from a plot
\item Pre-test+post-test means or mean change
\item Paired ANOVA/t-test statistics
\item Odds ratio value
\item Contingency table
\item Correlation coefficients
\item Phi/chi-square value
}
}

\subsection{Paired SMD or MD (\code{measure=c("d", "g", "md")} and \code{selection_auto="paired"})}{
\enumerate{
\item User's input effect size value
\item Paired SMD value
\item Pre-test+post-test means or mean change
\item Paired ANOVA/t-test statistics
\item Means at post-test
\item ANOVA/Student's t-test/point biserial correlation
\item Linear regression estimates
\item Mean difference values
\item Quartiles/median/maximum values
\item Odds ratio value
\item Contingency table
\item Correlation coefficients
\item Phi/chi-square value
}
}

\subsection{Adjusted SMD or MD (\code{measure=c("d", "g", "md")} and \code{selection_auto="adjusted"})}{
\enumerate{
\item User's input adjusted effect size value
\item Adjusted SMD value
\item Estimated marginal means from ANCOVA
\item F- or t-test value from ANCOVA
\item Adjusted mean difference from ANCOVA
\item Estimated marginal means from ANCOVA extracted from a plot
}
}

\subsection{Odds Ratio (\code{measure=c("or")})}{
\enumerate{
\item User's input effect size value
\item Odds ratio value
\item Contingency table
\item Risk ratio values
\item Phi/chi-square value
\item Correlation coefficients
\item (Then hierarchy as for "d" or "g" option crude)
}
}

\subsection{Risk Ratio (\code{measure=c("rr")})}{
\enumerate{
\item User's input effect size value
\item Risk ratio values
\item Contingency table
\item Odds ratio values
\item Phi/chi-square value
}
}

\subsection{Incidence rate ratio (\code{measure=c("irr")})}{
\enumerate{
\item User's input effect size value
\item Number of cases and time of disease free observation time
}
}

\subsection{Correlation (\code{measure=c("r", "z")})}{
\enumerate{
\item User's input effect size value
\item Correlation coefficients
\item Contingency table
\item Odds ratio value
\item Phi/chi-square value
\item SMD value
\item Means at post-test
\item ANOVA/Student's t-test/point biserial correlation
\item Linear regression estimates
\item Mean difference values
11 Quartiles/median/maximum values
\item Post-test means extracted from a plot
\item Pre-test+post-test means or mean change
\item Paired ANOVA/t-test
}
}

\subsection{Variability ratios (\code{measure=c("vr", "cvr")})}{
\enumerate{
\item User's input effect size value
\item means/variability indices at post-test
\item means/variability indices at post-test extracted from a plot
}
}

\subsection{Number needed to treat (\code{measure=c("nnt")})}{
\enumerate{
\item User's input effect size value
\item Contingency table
\item Odds ratio values
\item Risk ratio values
\item Incidence rate ratio (person-time NNT, requires baseline_rate)
\item Phi/chi-square value
}

NNT values should not be pooled directly (the NNT confidence interval is
disjoint when the risk difference crosses zero). You should pool RD/OR/RR
values and convert the pooled estimate to NNT (Deeks, 2002; Cochrane
Handbook, Chapter 15).
}

\subsection{Risk difference (\code{measure=c("rd")})}{
\enumerate{
\item User's input effect size value
\item Contingency table
\item Odds ratio values
\item Risk ratio values
\item Incidence rate ratio (requires baseline_rate)
\item Phi/chi-square value
}
}

}

\subsection{Manual}{

If you select a manual hierarchy, you can specify the order in which you want to
use each type of input data.
You can prioritize some types of input data by placing them at the begining of the
hierarchy argument, and you must separate all input data with a ">" separator.
For example, if you set:
\itemize{
\item \code{hierarchy = "means_sd > means_se > student_t"}, the convert_df function will prioritize
the means + SD, then the means + SE, then the Student's t-test to estimate the main effect
size.
\item \code{hierarchy = "2x2 > or_se > phi"}, the convert_df function will prioritize
the contingency table, then the odds ratio value + SE, then the phi coefficient to estimate
the main effect size.
}

Importantly, if none of the types of input data indicated in the \code{hierarchy} argument
can be used to estimate the target effect size measure,
the \code{convert_df()} function will automatically try to use other types of input
data to estimate an effect size.
}

}

\subsection{Adjusted effect sizes}{

Some datasets will be composed of crude (i.e., non-adjusted) types of input data
(such as standard means + SD, Student's t-test, etc.) and adjusted types of input data
(such as means + SE from an ANCOVA model, a t-test from an ANCOVA, etc.).

In these situations, you can decide to:
\itemize{
\item treat crude and adjusted input data the same way \code{split_adjusted = FALSE}
\item split calculations for crude and adjusted types of input data \code{split_adjusted = TRUE}
}

If you want to split the calculations, you can decide to present the final dataset:
\itemize{
\item in a long format (i.e., crude and adjusted effect sizes presented in separate rows \code{format_adjusted = "long"})
\item in a wide format (i.e., crude and adjusted effect sizes presented in separate columns \code{format_adjusted = "wide"})
}
}
}
\examples{
res <- convert_df(df.haza,
  measure = "g",
  split_adjusted = TRUE,
  es_selected = "minimum",
  format_adjusted = "long"
)
summary(res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/es_from_stand_RD.R
\name{es_from_rd_ci}
\alias{es_from_rd_ci}
\title{Convert a risk difference value and its 95\% confidence interval to several effect size measures}
\usage{
es_from_rd_ci(
  rd,
  rd_ci_lo,
  rd_ci_up,
  baseline_risk,
  n_exp,
  n_nexp,
  n_cases,
  n_controls,
  n_sample,
  reverse_rd
)
}
\arguments{
\item{rd}{risk difference value}

\item{rd_ci_lo}{lower bound of the 95\% CI around the risk difference}

\item{rd_ci_up}{upper bound of the 95\% CI around the risk difference}

\item{baseline_risk}{proportion of cases in the non-exposed/control group}

\item{n_exp}{number of participants in the exposed/treatment group}

\item{n_nexp}{number of participants in the non-exposed/control group}

\item{n_cases}{number of cases/events across both groups}

\item{n_controls}{number of controls/no-event across both groups}

\item{n_sample}{total number of participants in the sample}

\item{reverse_rd}{a logical value indicating whether the direction of the generated effect sizes should be flipped.}
}
\value{
This function estimates and converts between several effect size measures.

\tabular{ll}{
\code{natural effect size measure} \tab RD\cr
\tab \cr
\code{converted effect size measure} \tab OR + RR + NNT\cr
\tab \cr
\code{required input data} \tab rd + rd_ci_lo + rd_ci_up\cr
\tab \cr
}
}
\description{
Convert a risk difference value and its 95\% confidence interval to several effect size measures
}
\details{
This function computes the standard error of the risk difference from its 95\% CI
(Section 6.5.2.2 in the Cochrane Handbook):
\deqn{rd\_se = \frac{rd\_ci\_up - rd\_ci\_lo}{2 \times qnorm(.975)}}

Then, calculations of \code{\link{es_from_rd_se}()} are applied.
}
\examples{
es_from_rd_ci(
  rd = 0.15, rd_ci_lo = 0.05, rd_ci_up = 0.25,
  baseline_risk = 0.30, n_exp = 100, n_nexp = 100
)
}
\references{
Higgins JPT, Li T, Deeks JJ (editors). Chapter 6: Choosing effect size measures and computing estimates of effect. In: Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch VA (editors). Cochrane Handbook for Systematic Reviews of Interventions version 6.3 (updated February 2022). Cochrane, 2022. Available from www.training.cochrane.org/handbook.
}
